(() => {
    "use strict";

    angular
        .module('easyorder.components.email.float.button', [])
        .component("emailFloatButton", {
            templateUrl: 'js/components/button-float/button-float-basic.template.html',
            controller: EmailFloatButtonController,
            controllerAs: '$ctrl',
            bindings: {
                icon: '@',
                visible: '<',
                disabled: '<',
                animate: '<',
                mails: '<',
                initSendRequest: '<'
            }
        });

    EmailFloatButtonController.$inject = [
                    '$rootScope',
                    '$ionicModal',
                    '$ionicLoading',
                    'MailService',
                    'LogService',
                    'AlertService',
                    'ConfigService'];
    function EmailFloatButtonController(
                    $rootScope,
                    $ionicModal,
                    $ionicLoading,
                    MailService,
                    LogService,
                    AlertService) {
        var ctrl = this;
                        
        ctrl.$onChanges = (_) => { };
        ctrl.$onDestroy = () => { };
        ctrl.$onInit = () => {
            ctrl.icon = ctrl.icon || "envelope";
            
            if (!ctrl.initSendRequest) {
                throw new Error('Função de criação de requisição de e-mail não definida para o componente.')
            }
        };
        
        ctrl.doAction = () => {
            let $scope = $rootScope.$new();
            $scope.mails = ctrl.mails || [];
            $scope.fields = {};
            
            $scope.$on('$destroy', function() {
                $scope.cancelModal();
            });
            $scope.cancelModal = function () {
                return $scope.modal.remove();
            };
            $scope.addMail = function(){
                let mail = $scope.fields.newEmail;
                var validador = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/gi;
                
                if (validador.test(mail)) {
                    if (!$scope.mails.includes(mail)){
                        $scope.mails.push(mail);
                        $scope.fields.newEmail = "";
                    } else {
                        AlertService.error("Endereço de email já inserido!");
                    }
                } else {
                    AlertService.error("Endereço de email inválido!");
                }
            };
            $scope.removeMail = function(mail){
                var index = $scope.mails.indexOf(mail);
                if (index !== -1) $scope.mails.splice(index, 1);
            };
            $scope.sendMail = () => {
                if ($scope.mails && $scope.mails.length) {
                    $ionicLoading.show({
                        template : 'Aguarde... Enviando comprovantes',
                        duration : 0
                    }).then(() => {
                        const mailRequest = ctrl.initSendRequest();
                        mailRequest.destinations = $scope.mails;
    
                        return MailService.send(mailRequest)
                        .then($ionicLoading.hide)
                        .then(() => AlertService.info('Email enviado!'))
                        .then($scope.cancelModal);
                   })
                   .catch(error => {
                        LogService.error(error);
                        $ionicLoading.hide()
                        .then(AlertService.info('Ocorreu um erro ao tentar enviar comprovante por email!'));
                    });
                } else {
                    AlertService.error('Informe ao menos um e-mail válido para o envio');
                }
            };
            
            $ionicModal.fromTemplateUrl('js/components/button-float/email/send-email.template.html', {
                scope: $scope,
                animation: 'slide-in-up'
            }).then(function(modalMail) {
                $scope.modal = modalMail;
                $scope.modal.show()
            });
        };
    }
})();