(function() {
    'use strict';

    angular.module('ln.currency', [])
    
    .directive('lnCampoMoeda', function() {
        return {
            template : '<input id="campo-moeda" type="tel" ng-model="model_presentation" maxlength="{{ tamanhomax }}" class="inputCampoMoeda" ng-keyup="processarMascara()" ng-readonly="somenteleitura">\n' +
                       '<input type="hidden" ng-model="model">',
            transclude: false,
            scope: {
                model: '=model',
                locale: '@',
                moeda: '@',
                digitosfracao: '@',
                tamanhomax: "=",
                somenteleitura: '=',
                valormax: '=',
                valormin: '='
            },
            compile: function(element, attrs) {
                attrs.locale = attrs.locale || 'pt-BR';
                attrs.moeda = attrs.moeda || 'BRL';
                attrs.digitosfracao = attrs.digitosfracao || '2';
                attrs.tamanhomax = attrs.tamanhomax || '12';
                attrs.somenteleitura = attrs.somenteleitura || 'false';
                attrs.valormax = attrs.valormax || '9999999.99';
                attrs.valormin = attrs.valormin || '0.00';
            },
            controller: function CampoMoedaController($scope) {
                var formatter = new Intl.NumberFormat($scope.locale, {
                    style: 'currency',
                    currency: $scope.moeda,
                    minimumFractionDigits: Number.parseInt($scope.digitosfracao)
                });
                
                $scope.$on('clearCurrency', function(event, value) { 
                    $scope.model_presentation = formatter.format(value);
                    $scope.processarMascara();
                });
                $scope.$watch("model", newValue => {
                    $scope.model_presentation = formatter.format(newValue);
                });
                
                $scope.processarMascara = function() {
                    var vr =  $scope.model_presentation.replace(/\D/g, "").replace(/^0*/, ""); //removendo caracteres não-númericos e zeros à esquerda.
                    vr = parseFloat(vr && vr.length > 0 ? vr : "0");
                    vr = vr / (Math.pow(10, Number.parseInt($scope.digitosfracao))); // Aplicando a fração desejada.
                    
                    var valorArredondado = Math.round($scope.valormax * 100) / 100;
                    var valorMinArredondado = Math.round($scope.valormin * 100) / 100;
                    
                    if (vr >= valorMinArredondado && vr <= valorArredondado && $scope.model !== vr) {
                        $scope.model = vr;
                    } else {
                        $scope.model_presentation = formatter.format($scope.model);
                    }
                };
            }
        };
    });
})();
