(function () {
    'use strict';

    angular.module('ln.percent', [])

        .directive('lnCampoPorcentagem', function () {
            return {
                template: '<input id="campo-porcentagem" type="tel" ng-model="model_presentation" maxlength="{{ maxlength }}" class="inputCampoPorcentagem" ng-keyup="loadMask()" ng-readonly="readyonly">\n' +
                    '<input type="hidden" ng-model="model">',
                transclude: false,
                scope: {
                    model: '=model',
                    maxlength: "=",
                    readyonly: '=',
                    maxvalue: '='
                },
                compile: function (element, attrs) {
                    attrs.maxlength = attrs.maxlength || 12;
                    attrs.readyonly = attrs.readyonly || 'false';
                    attrs.maxvalue = attrs.maxvalue || '100';
                },
                controller: function InputPercentController($scope) {
                    /**
                     * Função que adiciona o simbolo de porcentagem no final da string.
                     */
                    let formatterToPercent = function (value) {
                        return value + "%";
                    }

                    $scope.$on('clearPercent', function(event, value) { 
                        $scope.model_presentation = String(value);
                        $scope.loadMask();
                    });

                    $scope.loadMask = function () {
                        let value = $scope.model_presentation.replace(/\D/g, "").replace(/^0*/, ""); //removendo caracteres não-númericos e zeros à esquerda.
                        value = parseFloat(value && value.length > 0 ? value : "0");

                        let maxValue = Math.round($scope.maxvalue * 100) / 100;

                        if (value >= maxValue) {
                            value = maxValue;
                            $scope.model = value;
                            $scope.model_presentation = formatterToPercent(value);
                        }
                        if (value < maxValue) {
                            $scope.model = value;
                            $scope.model_presentation = formatterToPercent(value);
                        }
                        else {
                            $scope.model_presentation = formatterToPercent($scope.model);
                        }
                    };

                    /**
                     * Função para mover o cursor antes do simbolor de porcentagem.
                     */
                    function moveInputCursor() {
                        let input = document.getElementById('campo-porcentagem');
                        let time = null;
                        input.addEventListener('focus', function () {
                            time = setInterval(function () {
                                if (input.value.match(/%/)) {
                                    if (input.selectionStart != input.selectionEnd) {
                                        input.selectionStart = input.value.length - 1;
                                    }
                                    else {
                                        input.selectionEnd = input.value.length - 1;
                                    }
                                }
                            }, 100);
                        })
                        input.addEventListener('blur', function () {
                            clearInterval(time);
                        });
                    }

                    /**
                     * Função de inicialização.
                     */
                    (function () {
                        moveInputCursor();
                        $scope.model_presentation = formatterToPercent($scope.model);
                    })();
                }
            };
        });
})();