(function() {
	'use strict';

	angular.module('elasticLogger', [])

	.factory('$elasticLogger', function(esFactory) {
        var es;
        let logCtx;
       
        function shutdown() {
            if (es != null) {
                es.close();
                es = null;
            }
        }
        function reconfigure(hosts, ctx) {
            logCtx = ctx;
             es = esFactory({
                hosts: hosts
             });
        }

        function debug(message) {
            fire('DEBUG', message);
        }

        function info(message) {
            fire('INFO', message);
        }

        function warn(message) {
            fire('WARN', message);
        }

        function error(message) {
            fire('ERROR', message);
        }

        function fatal(message) {
            fire('FATAL', message);
        }

        function log(level, message, args) {
            if (level !== 'RESPONSE ERROR') {
                // erro de comunicacao http nao enviado ao Elastic para não ficar em loop infinito. 
                fire(level, message, args);
            }
      
        }

        function fire(level, message, args) {
            if (es != null) {
                var logMessage = args ? `${message} ${parseArguments(args)}` : message;
                var date = new Date();

                const body = {
                    '@timestamp': toJson(date),
                    ecs: {
                        version: '8.1.0',
                    },
                    log: {
                        level: level,
                    },
                    message: logMessage,
                };

                if (logCtx) {
                    const labels = {};
                    const service = {};

                    // mapeamento de contexto para ecs
                    const serviceKeys = {
                        appName: 'name',
                        appVersion: 'version'
                    };

                    Object.keys(logCtx).forEach(k => {
                        if (serviceKeys[k]) {
                            const svcKey = serviceKeys[k];
                            service[svcKey] = logCtx[k];
                        } else {
                            labels[k] = logCtx[k];
                        }
                    });

                    if (Object.keys(labels).length > 0) {
                        body.labels = labels;
                    }
                    if (Object.keys(service).length > 0) {
                        body.service = service;
                    }
                }

                es.index({
                    index: createIndexName(date),
                    body: body,
                  }).catch(error => {
                    console.error(`Erro ao enviar pro Elastic: ${error}`);
                  });
            }
           
        }

        function createIndexName(date) {
            return `linx-storex-mobile-${formatDate(date)}`;
        }
    
        function formatDate(date) {
            var y = date.getFullYear();
            var m = date.getMonth() + 1;
            var d = date.getDate();
            return '' + y + '-' + (m < 10 ? '0' : '') + m + '-' + (d < 10 ? '0' : '') + d;
        }

        function stringify(value) {
            try {
                return JSON.stringify(value);
            } catch (e) {
                return value;
            }
        }
        
        function parseArguments(args) {
            if (!angular.isArray(args)) {
                if (angular.isObject(args)) {
                    return stringify(args);
                } else {
                    return args;
                }
                
            } else {
                var ret = [];
                for (var i = 0; i < args.length; i++) {
                    var value = args[i]; 
                    if (angular.isArray(value) || angular.isObject(value)) {
                        ret.push(stringify(value));
                    } else {
                        ret.push(args[i]);
                    }
                }
                return ret.join(' ');
           }
        }

        function toJson(date) {
            return date.toJSON();
        }

        return {
            reconfigure: reconfigure,
            shutdown: shutdown,
            log: log,
            debug: debug,
            info: info,
            warn: warn,
            error: error,
            fatal: fatal
        };
    });
})
();