(function() {
	'use strict';

	angular.module('loggers', [])

	.factory('$loggers', function() {
        var loggers = [];

        function addLogger(logger) {
            var index = loggers.indexOf(logger);
            if (index === -1) {
                loggers.push(logger);
            }
        }

        function removeLogger(logger) {
            var index = loggers.indexOf(logger);
            if (index !== -1) {
                loggers.splice(index, 1);
                if (logger.shutdown) {
                    logger.shutdown();
                }
            }
        }

        function debug(message) {
            fire('DEBUG', message);
        }

        function info(message) {
            fire('INFO', message);
        }

        function warn(message) {
            fire('WARN', message);
        }

        function error(message) {
            fire('ERROR', message);
        }

        function responseError(message) {
            fire('RESPONSE ERROR', message);
        }

        function fatal(message) {
            fire('FATAL', message);
        }

        function log(level, message, args) {
            fire(level, message, args);
        }

        function fire(level, message, args) {
            if (loggers.length > 0) {
                loggers.forEach(logger => {
                    logger.log(level, message, args);
                });
            } else {
                console.log(message, args);
            }

        }

        return {
            addLogger: addLogger,
            removeLogger: removeLogger,
            log: log,
            debug: debug,
            info: info,
            warn: warn,
            error: error,
            responseError: responseError,
            fatal: fatal
        };
    });
})
();