class OrderChecking {

    /**
     * Sumariza os ítens, indicando quantos foram bipados e seus totais e quantos estão nos pedidos.
     * @param {*} cartItems 
     * @param {*} orderItems 
     * @returns 
     */
    static summarizeItems = function(cartItems, orderItems) {
        const hash = {}, result = [];

        // sumariza os ítens bipados
        for (const { productCode, productDescription, displayQuantity } of cartItems) {
            if (hash[productCode]) {
                hash[productCode].scannedQuantity = MathUtils.add(hash[productCode].scannedQuantity, displayQuantity, 3);
            } else {
                result.push(hash[productCode] = { productCode, productDescription, scannedQuantity: displayQuantity, orderQuantity: 0 });
            }
        }

        // sumariza os ítens do pedido
        for (const { productCode, productDescription, quantitySold, productType, minimumSaleUnit } of orderItems) {
            const displayQuantity = OrderChecking.__displayQuantity(quantitySold, productType, minimumSaleUnit);

            if (hash[productCode]) {
                hash[productCode].orderQuantity = MathUtils.add(hash[productCode].orderQuantity, displayQuantity, 3);
            } else {
                result.push(hash[productCode] = { productCode, productDescription, scannedQuantity: 0, orderQuantity: displayQuantity });
            }
        }
        return result;
    }

    static __displayQuantity(quantitySold, productType, minimumSaleUnit) {
        if (productType == 'MINIMUM_SALE_UNIT') {
            return MathUtils.divide(quantitySold, minimumSaleUnit, 3);
        }
        return quantitySold;
    }

    /**
     * Retorna os ítens conflitantes (que possuem mais no pedido que os bipados)
     * @param {*} cartItems 
     * @param {*} orderSoldInStoreItems 
     * @returns 
     */
    static conflitingItems = function(cartItems, orderSoldInStoreItems) {
        return OrderChecking.summarizeItems(cartItems, orderSoldInStoreItems).filter(item => item.scannedQuantity < item.orderQuantity);
    }


    static isSoldInStore = function(fiscalItem) {
        return fiscalItem != null && (fiscalItem.deliveryOption == null || fiscalItem.deliveryOption == 'SOLD_IN_STORE');
    }

    static extractOrderItems = function(orders) {
        return orders.map(order => order.fiscalItems).reduce(function(acc, value){
            return acc.concat(value);
          }, []);
    }

    static extractOrderSoldInStoreItems = function(orders) {
        return OrderChecking.extractOrderItems(orders).filter(fiscalItem => OrderChecking.isSoldInStore(fiscalItem));
    }
    
    static extractConvertItems = function(cartItems, orders) {
        const orderItems = OrderChecking.extractOrderItems(orders);
        const notInOrderItems = OrderChecking.extractNotInOrderItems(cartItems, orderItems);
        
        const totalItems = [...orderItems, ...notInOrderItems];
        // recalcular sequenciais
        let index = 1;
        totalItems.forEach(item => item.sequenceId = index++);
        return totalItems;
    }

    static extractNotInOrderItems = function(cartItems, orderItems) {
        const summarizedItems = OrderChecking.summarizeItems(cartItems, orderItems);
        let notInOrderItems = [];
        if (summarizedItems && summarizedItems.length > 0) {
            // encontra os ítems que precisam ser criados (a mais que no pedido)
            notInOrderItems = summarizedItems
                .filter(item => item.scannedQuantity > item.orderQuantity)
                .map(item => {
                    // cria um FiscalItem a partir dos dados divergentes
                    const newQuantity = MathUtils.subtract(item.scannedQuantity, item.orderQuantity, 3);
                    return OrderChecking.createNewItem(cartItems, item.productCode, newQuantity);
                });
        }

        return notInOrderItems;
    }

    static createNewItem = function(fiscalItems, productCode, displayQuantity) {
        let newItem = null;
        const foundItem = OrderChecking.findItemByProductCode(fiscalItems, productCode);
        if (foundItem) {
            newItem = OrderChecking.cloneObject(foundItem);
            newItem.quantitySold = displayQuantity;
            if (newItem.productType == 'MINIMUM_SALE_UNIT') {
                newItem.quantitySold = MathUtils.multiply(
                    newItem.minimumSaleUnit,
                    displayQuantity,
                    3
                );
            }
        }
        return newItem;
    }

    static findItemByProductCode = function(fiscalItems, productCode) {
        return fiscalItems.find(fiscalItem => fiscalItem.productCode === productCode);
    }

    static cloneObject = function(obj) {
        return obj == null ? null : JSON.parse(JSON.stringify(obj));
    }
}