(() => {
    "use strict";

    angular
        .module('easyorder.components.payment.tef.debit', [])
        .component("debitPayment", {
            templateUrl: 'js/components/payment/payment-method-option.template.html',
            controller: TefDebitPaymentController,
            controllerAs: '$ctrl',
            bindings: {
                onInitProcess: '&',
                onResult: '&',   // Callback com a promessa do resultado.
                plansForOperation: '<',
                paymentValue: '<',
                disabled: '<',
                tefFiscalAuthorization: '<',
                icon: '@',
                description: '@'
            }
        });

    TefDebitPaymentController.$inject = [
                    '$rootScope',
                    '$q',
                    '$ionicModal',
                    'EFTService', 
                    'LogService',
                    'UserService',
                    'AlertService',
                    'PaymentService',
                    'ConfigService'];
    function TefDebitPaymentController(
                    $rootScope,
                    $q,
                    $ionicModal,
                    EFTService,
                    LogService,
                    UserService,
                    AlertService,
                    PaymentService,
                    ConfigService) {
        var ctrl = this;
        
        ctrl.$onChanges = function (_) { };
        ctrl.$onDestroy = function () { };

        ctrl.$onInit = function () {
            ctrl.icon = ctrl.icon || "icon fa fa-credit-card fa-fw";
            ctrl.description =  ctrl.description || "Débito";
        };
        
        ctrl.processPayment = () => {
            let done = false;
            let goingBack = false;
            let $scope = $rootScope.$new();
            let ret = $q.defer();
            
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.options = [];
            $scope.transactionType = EFTService.transactionType;
            
            if (ctrl.onInitProcess) {
                ctrl.onInitProcess({ data: {method: 'DEBIT', value: ctrl.paymentValue} });
            }
            
            $scope.cancelModal = function () {
                return $scope.modal.remove();
            };
            $scope.goBack = () => {
                if (!$scope.isBackButtonDisabled()) {
                    goingBack = true;
                    $scope.message = "Cancelando operação...";
                
                    EFTService.finalizeTransaction()
                    .then($scope.cancelModal)
                    .then(() => ret.reject())
                    .catch(() => {
                        $scope.message = "Não foi possível cancelar operação";
                        goingBack = false
                    });
                }
            };
            $scope.isBackButtonDisabled = () => {
                return done || goingBack;
            };
            $scope.clearDisplay = function() {
                $scope.$apply(function () {
                    $scope.message = "";
                });
                LogService.info("clearDisplay");
            };
            $scope.displayMessage = (message) => {
                /*
                 * plugin de tef não atualiza a mensagem caso seja requisitado
                 * o cancelamento da operação, por isto bloqueamos outras mensagens
                 */
                if (!goingBack) {
                    $scope.$apply(function () {
                        $scope.message = message ? message : $scope.message;
                    });
                    LogService.info("displayMessage: " + $scope.message);
                }
            };
            $scope.selectOption = (title, options, responseHandler) => {
                $scope.$apply(function () {
                    $scope.message = title;
                });
                $scope.processOptions(options, responseHandler);
                LogService.info("Opção tef selecionada: ", title,  " Oções: ",  options);
            };
            $scope.selectOptionExtended = (title, options, responseHandlerExtended) => {
                $scope.$apply(function () {
                    $scope.message = title;
                });
                $scope.processOptions(options, responseHandlerExtended);
                LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
            };
            $scope.numberInput = function(label, responseHandler, fieldSizeRange) {
                $scope.options.length = 0;
                $scope.showWatermark = false;
    
                $scope.options.push({
                    id:                 1, //Dummy
                    label:              label,
                    handler:            responseHandler,
                    type:               'number',
                    maxSize:            fieldSizeRange.maxSize,
                    valuePattern:       '[0-9]*'
                });
                $scope.headerOptions = "Informe";
                $scope.$apply();
            };
            $scope.stringInput = function(label, responseHandler, fieldSizeRange) {
                $scope.options.length = 0;
                $scope.showWatermark = false;
    
                $scope.options.push({
                    id:                 1, //Dummy
                    label:              label,
                    handler:            responseHandler,
                    type:               'string',
                    maxSize:            fieldSizeRange.maxSize,
                    valuePattern:       '[A-Za-z0-9_]*'
                });
                $scope.headerOptions = "Informe";
                $scope.$apply();
            };
            $scope.processOptions = function(options, responseHandler) {    //(1,"Magnetico/Chip")#(2,"Digitado")
                $scope.processLocalOptions(options, responseHandler)
                $scope.$apply();
            };
            $scope.processLocalOptions = function(options, responseHandler, headerText) {    //(1,"Magnetico/Chip")#(2,"Digitado")
                $scope.options.length = 0;
                $scope.showWatermark = false;
    
                var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
                splitedOpt = splitedOpt.split("#"); // Separando as opções.
    
                for (var i = 0; i < splitedOpt.length; i++) {
                    var split = splitedOpt[i].split(",");
    
                    $scope.options.push({
                        id:         split[0].trim(),
                        label:      split[1].trim().replace(/\"/g, ""),
                        value:      split[split.length-1],
                        handler:    responseHandler,
                        type:        "single-option"
                    });
                }
                $scope.headerOptions = headerText || "Opções";
            };
            $scope.tickOptionHandler = function(func, params) {
                $scope.options.length = 0;
                $scope.showWatermark = true;
                func.apply(this, params);
            };
            $scope.tickNumberInputHandler = function(inputOption) {
                var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);
                $scope.tickOptionHandler(inputOption.handler, [value]);
            };
            $scope.tickStringInputHandler = function(inputOption) {
                const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);
                $scope.tickOptionHandler(inputOption.handler, [value]);
            };
            $scope.paymentDone = function(paymentResults) {
                done = true;
                $scope.modal.remove()
                .then(() => ret.resolve({ paymentResults: paymentResults, paymentPlan: ctrl.paymentPlan }))
                .finally($scope.$destroy);
                
            };
            $scope.paymentError = errorMsg => {
                switch (errorMsg) {
                    case window.plugins.lio.PMNT_CANCELLED:
                        AlertService.error('Transação cancelada')
                            .then($scope.cancelModal)
                            .then(() => ret.reject());
                        break;
                    default:
                        AlertService.error(errorMsg)
                            .then($scope.cancelModal)
                            .then(() => ret.reject());
                        break;
                }
            };
            
            /**
             *
             */
            function initPaymentProcess() {
                return $q.when($scope.message = 'Iniciando Pagamento...') 
                .then(() => PaymentService.initPayment('DEBIT', ctrl.paymentValue))
                .then(() => PaymentService.setPaymentOperationType(ctrl.plansForOperation))
                .then(doTefPaymentAuthorization)
                .catch(httpResponse => {
                    var message = (httpResponse && httpResponse.data && httpResponse.data.message) ? httpResponse.data.message : "Não foi possível iniciar pagamento.";
                    
                    if (httpResponse && httpResponse.message){
                        message  = httpResponse.message;
                    }
                    LogService.error(httpResponse);
                    LogService.error(message);
                    return $q.reject(message);
                });
            };
            
            function doTefPaymentAuthorization() {
                if (ctrl.tefFiscalAuthorization) {
                    return PaymentService.tefPaymentAuthorization()
                } else {
                    return $q.resolve();
                }
            }
            
            /**
             * 
             */
            function getPaymentPlanOperation() {
                const defer = $q.defer();
                
                if (ctrl.plansForOperation) {
                    $scope.message = 'Consultando Eventos. Por favor aguarde....';
                    
                    PaymentService.search()
                    .then(result => {
                        $scope.message = 'Escolha o Evento';

                        if (!result || !result.paymentPlanEvents || result.paymentPlanEvents.length === 0) {
                            defer.reject('Não existem eventos de pagamento para a forma escolhida.')
                        } else {
                            result.paymentPlanEvents.sort((e1, e2) => e1.priority > e2.priority ? 1 : (e1.priority === e2.priority ? 0 : -1));
                            let eventOptions = result.paymentPlanEvents.map(r => `(${r.code},"${r.description}")`).join('#');
                            
                            $scope.processLocalOptions(eventOptions, selected => {
                                $scope.message = 'Escolha o Plano';
                                
                                var selectedEvent = result.paymentPlanEvents.find(e => e.code === selected)
                                
                                LogService.info(`Evento de pagamento selecionado ${selectedEvent.code} - ${selectedEvent.description}`);
                                
                                let plans = selectedEvent.paymentPlans;
                                plans.sort(getPlansSortFunction());
                                
                                let plansOptions = plans.map(r => `(${r.code},"${r.description}")`).join('#');
                                
                                $scope.processLocalOptions(plansOptions, selected => {
                                    let payment = PaymentService.currentPayment();
                                    let selectedPlan = plans.find(p =>  p.code === selected);
                                    
                                    LogService.info(`Plano de pagamento selecionado ${selectedPlan.code} - ${selectedPlan.description}`);
                                    
                                    payment.plan = selectedPlan;
                                    payment.plan.paymentCodeEvent = selectedEvent.code;
                                    payment.plan.processorCode = selectedEvent.processorCode;
                                    
                                    defer.resolve(selectedPlan);
                                }, "Planos de Pagamento");
                            },"Eventos de Recebimento");
                        }
                    })
                    .catch(err => {
                        defer.reject('Não foi possível processar Planos de Pagamento!');
                        LogService.error(err);
                    });
                } else {
                    defer.resolve();
                }
                return defer.promise;
            }
            
            /**
             *
             */
            function getPlansSortFunction() {
                if (ConfigService.config.business.payment_plan_display_order === 'REVERSED_INTS_NUMBER') {
                    return (p1, p2) => {
                        return p2.instalments.length - p1.instalments.length;
                    };
                } else if (ConfigService.config.business.payment_plan_display_order === 'NATURAL_INTS_NUMBER') {
                    return (p1, p2) => {
                        return p1.instalments.length - p2.instalments.length;
                    };
                }
            };
            
            /**
             * 
             */
            async function initiateTefProcess(plan) {
                $scope.message = '';
                let transactionType;
                //Para modalidade de pagamento fatura/carnê é enviado o código 16 (DEBIT_INVOICE)
                switch (ctrl.plansForOperation) {
                    case 'PAYMENT_TOTVSCARD_CREDIT_BOOK':
                        transactionType = EFTService.transactionType.DEBIT_INVOICE;
                        break;
                    case 'PAYMENT_TOTVSCARD_CARD_STATEMENT':
                        transactionType = EFTService.transactionType.DEBIT_INVOICE;
                        break;
                    default: 
                        transactionType = EFTService.transactionType.DEBIT;
                        break;
                }
                var params = {
                    transactionType:    transactionType,
                    operatorCode:       UserService.operatorCode(),
                    amount:             ctrl.paymentValue,
                    instalmentsNumber:  plan ? (plan.instalments ? plan.instalments.length : 1) : 1,
                    charges:            plan ? plan.charges : undefined,
                    processorCode:      plan ? plan.processorCode : undefined,
                    interestType:       plan ? plan.interestType : undefined
                };
                EFTService.executeTransaction(params, $scope);
            }
            
            $ionicModal.fromTemplateUrl('js/components/payment/tef/tef-payment.template.html', {
                scope: $scope,
                focusFirstInput: true,
                animation: 'slide-in-up'
            }).then(modal => {
                $scope.transactionInitiated = true;
                $scope.modal = modal;
                
                $scope.modal.show()
                .then(initPaymentProcess)
                .then(getPaymentPlanOperation)
                .then(initiateTefProcess)
                .catch(err => {
                    return $scope.cancelModal()
                    .then(() => PaymentService.clearInitializedPaymentData())
                    .then(() => $q.reject(err));
                })
                .catch(ret.reject);
            });
            
            ctrl.onResult({ promise: ret.promise });
        };
    }
})();