(function () {
    'use strict';

    // Usage:
    // 
    // Creates:
    // 

    angular
        .module('easyorder.components.pdf', [])
        .component('pdf', {
            templateUrl: 'js/components/pdf/pdf.template.html',
            controller: PDFController,
            controllerAs: '$ctrl',
            bindings: {
                pdf: '<',
                fullscreen: '<',
                onRender: '<',
                onError: '<'
            },
        });

    PDFController.$inject = ['$q', 'LogService', '$timeout'];

    function PDFController($q, LogService, $timeout) {
        const $ctrl = this;
        /**
         *
         */
        $ctrl.$onInit = function () {
            $ctrl.loading = true;
            
            window.pdfjsLib.getDocument({ data: $ctrl.pdf.data }).promise
            .then(pdf => pdf.getPage(1))
            .then(page => {
                let viewport = page.getViewport({scale: 1});
                const canvas = document.createElement('canvas');
                const canvasContext = canvas.getContext('2d');

                if ($ctrl.fullscreen) {
                    const scale = window.innerWidth / viewport.width;
                    viewport = page.getViewport({scale: scale});

                    // FIXME: put this after canvas.width, needs testing
                    canvas.style.width = `${viewport.width}px`;
                    canvas.style.height = `${viewport.height}px`;
                }

                canvas.height = viewport.height;
                canvas.width = viewport.width;

                const renderContext = {
                    canvasContext: canvasContext,
                    viewport: viewport
                };
                
                return page.render(renderContext).promise
                .then(() => $timeout(() => $ctrl.imageData = canvas.toDataURL()));
            })
            .then(() => {
                $ctrl.loading = false;
                if ($ctrl.onRender) {
                    $ctrl.onRender();
                }                    
            })
            .catch(error => {
                $ctrl.loading = false;
                LogService.error(error.toString());
                if ($ctrl.onError) {
                    $ctrl.onError();
                } 
            });
        };

        $ctrl.$onChanges = function (changesObj) { $ctrl.imageData = null; $ctrl.$onInit(); };

        $ctrl.$onDestroy = function () { };
    }
})();
