(() => {
    'use strict';

    angular
        .module('easyorder.modals.stylesearch',[])
        .factory('StyleSearchModal', StyleSearchModal);

    StyleSearchModal.$inject = ['$ionicModal', '$rootScope', 'AlertService', 'LogService'];

    function StyleSearchModal($ionicModal, $rootScope, AlertService, LogService) {
        const HEADER = '[StyleSearchModal]'; 
        const $scope = $rootScope.$new(),
            modalOptions = {
                scope: $scope,
                focusFirstInput: true
        };

        $scope.chooseItem = (spec, value) => {
            LogService.info(`${HEADER} Característica escolhida: ${spec} = ${value}.`);
            var filteredProducts = $scope.products.filter(product => product.specs[spec] === value);
            if (filteredProducts) {
                $scope.products = filteredProducts;
                // se restou apenas um, é o escolhido
                if ($scope.products.length === 1) {
                    const productCode = $scope.products[0].productCode;
                    LogService.info(`${HEADER} Produto escolhido: ${productCode}`);
                    $scope.itemSelected(productCode)
                } else {
                    showNextDefinitionValues($scope.definitions, $scope.products);
                }
            }
        }

        function isValidStyleInfo(styleInfo) {
            return styleInfo && styleInfo.name && styleInfo.definitions && styleInfo.definitions.length > 0 && styleInfo.products && styleInfo.products.length > 0;
        }

        function showNextDefinitionValues(definitions, products) {
            const definition = extractAndRemoveDefinition(definitions);
            if (definition) {
                const values = extractDefinitionValues(definition, products);
                if (values) {
                    $scope.definition = definition;
                    $scope.values = values;
                } else {
                    AlertService.error(`Não foram encontrados valores para a definição ${definition}!`);
                }
            } else {
                AlertService.error('Nenhuma outra característica encontrada e existem múltiplos produtos!');
            }
        }

        function extractAndRemoveDefinition(definitions) {
            if (definitions && definitions.length > 0) {
                return definitions.shift();
            } else {
                return null;
            }
        }

        function extractDefinitionValues(definition, products) {
            const rowValues = products.map(product => product.specs[definition]).filter(r => r != null);
            if (rowValues && rowValues.length > 0) {
                // removendo duplicados
                const distinctRowValues =  [...new Set(rowValues)];
                // ordenando
                return [...(distinctRowValues)].sort();
            } else {
                return null;
            }
        }

        function open(styleInfo, itemCallback) {
            if (isValidStyleInfo(styleInfo)) {
                $scope.name =  styleInfo.name;
                $scope.products = styleInfo.products;
                $scope.definitions = styleInfo.definitions;
                showNextDefinitionValues($scope.definitions, $scope.products);
                
                $ionicModal.fromTemplateUrl(
                                'js/components/style/style-search-modal.html',
                                modalOptions
                ).then(function (modalInstance) {
                        $scope.itemSelected = function (productCode) {
                            closeAndRemove(modalInstance)
                            .then(() => {
                                itemCallback(productCode);
                            })
                            
                        };
                        $scope.close = function () {
                            closeAndRemove(modalInstance);
                        };
                        return modalInstance.show();
                });
            } else {
                AlertService.error('Dados dos modelos inválidos!');
            }

        }

        function closeAndRemove(modalInstance) {
            return modalInstance.hide()
                .then(() => {
                    return modalInstance.remove();
                });
        }

        const instance = {
            open: open
        };

        return instance;
    }
})();


