(() => {
    'use strict';

    angular
        .module('easyorder.components.transaction.searchv3', [])
        .component('transactionSearchV3', {
            templateUrl: 'js/components/transaction/search/transaction-searchv3.template.html',
            controller: TransactionSearchController,
            controllerAs: '$ctrl',
            bindings: {
                onResult: '&',   // Callback com a promessa do resultado.
                onSearch: '&',   // Se uma consulta customizada for necessária
                strictInput: '@', //Todos os campos são obrigatórios?
                searchButtonText: '@',
                storeNumber: '<', // Valor default do campo (opcional)
                terminalNumber: '<', // Valor default do campo (opcional)
                provider: '<', // Provider da consulta STOREX ou LOGIC (opcional)
                hiddenFields: '<' // array 'store-number', 'terminal-number' (opcional)
            },
        });

    TransactionSearchController.$inject = ['$q', '$attrs', '$ionicLoading', 'TransactionSearchService', 'ConfigService'];
    function TransactionSearchController($q, $attrs, $ionicLoading, TransactionSearchService, ConfigService) {
        var ctrl = this;

        ctrl.$onChanges = function (_) { };
        ctrl.$onDestroy = function () { };

        ctrl.$onInit = function () {
            ctrl.resetTransactionKey();
            ctrl.date = new Date();
            ctrl.loading = false;
            ctrl.dataInputVisible = false;
            ctrl.strictInput = (ctrl.strictInput == undefined) || !!ctrl.strictInput;
            ctrl.searchButtonText = ctrl.searchButtonText || 'SEGUIR';
            ctrl.provider = ctrl.provider == null ? 'STOREX' : ctrl.provider;
        };

        ctrl.resetTransactionKey = () => {
            ctrl.transactionKey = {};
            if (ctrl.storeNumber != null) {
                ctrl.transactionKey.storeNumber = ctrl.storeNumber;
            }
            if (ctrl.terminalNumber != null) {
                ctrl.transactionKey.terminal = ctrl.terminalNumber;
            }
        }

        ctrl.notHidden = (field) => {
            if (ctrl.hiddenFields == null) {
                return true;
            }
            return !ctrl.hiddenFields.some(hField => hField == field);
        }

        ctrl.onNsuChange = () => {
            if (!ctrl.hasAlreadySaved) {
                ctrl.lastStoreNumber = ctrl.transactionKey.storeNumber;
                ctrl.lastTerminalNumber = ctrl.transactionKey.terminal;
                ctrl.hasAlreadySaved = true;
            } else {
                ctrl.transactionKey.storeNumber = ctrl.lastStoreNumber;
                ctrl.transactionKey.terminal = ctrl.lastTerminalNumber;
                ctrl.hasAlreadySaved = false;
            }
            ctrl.hiddenFields = [];
            ctrl.provider = 'STOREX';
        }

        ctrl.defaultOnSearch = () => {
            $ionicLoading.show({
                template: 'Consultando transação...',
                duration: 0
            }).then(function () {
                ctrl.loading = true;

                if ($attrs.onSearch) {
                    let seachPromiseRet = ctrl.onSearch();
                    seachPromiseRet = $q.when(seachPromiseRet).finally(() => {
                        ctrl.loading = false;
                        ctrl.resetTransactionKey();
                        ctrl.transactionDate = undefined;
                        $ionicLoading.hide();
                    });
                    ctrl.onResult({ promise: seachPromiseRet });
                } else {
                    let seachPromiseRet = TransactionSearchService.search({
                        transactionKey: ctrl.transactionKey,
                        provider: ctrl.provider,
                    });
                    seachPromiseRet = seachPromiseRet.finally(() => {
                        ctrl.loading = false;
                        $ionicLoading.hide();
                        ctrl.resetTransactionKey();
                        ctrl.transactionDate = undefined;
                        ctrl.hiddenFields = [];
                        ctrl.provider = 'STOREX';
                    });
                    ctrl.onResult({ promise: seachPromiseRet });
                }
            });
        };

        ctrl.formatStoreNumber = () => {
            ctrl.transactionKey.storeNumber = ctrl.transactionKey.storeNumber.padStart(4, '0');
        }

        ctrl.onDateChanges = () => {
            if (ctrl.transactionDate) {
                ctrl.transactionDate.setHours(12, 0, 0);
                ctrl.transactionKey.date = ctrl.transactionDate.toISOString();
            }
        };
    };
})();
