(function () {
    'use strict';
    angular.module('easyorder.controllers.ewallet.payment', [
    ]).controller('EWalletPaymentCtrl', function (
        $q,
        $scope,
        $rootScope,
        $stateParams,
        $ionicLoading,
        ConfigService,
        EWalletService,
        CartService,
        PaymentService,
        PaymentProcessor,
        UtilService,
        LogService,
        AlertService
    ) {
        const self = this;

        let aborted = false;
        let block = false;

        const storeNumber = ConfigService.config.storeNumber;
        const terminalNumber = ConfigService.config.terminalNumber;
        const orderId = `${storeNumber}_${terminalNumber}_${randomInt(1, 99999)}_${new Date().toJSON()}`;
        const walletType = $stateParams.paymentMethod.id;

        // utilizado caso operador clique para voltar a tela
        self.undoData = {
            paid: false,
            campaigns: [],
        };

        const paymentValue = initPayment(walletType, $stateParams.paymentValue);

        EWalletService.pay({
            storeNumber: storeNumber,
            terminalNumber: terminalNumber,
            orderId: orderId,
            paymentValue: paymentValue,
            qrOptions: {
                width: 200,
                height: 200,
                theme: 'LINX',
            },
            walletType: walletType,
        }).then((res) => {
            const data = res.data;
            $scope.qrImage = data.qrImage;

            // utilizado para voltar a tela
            self.undoData.paid = true;
            self.undoData.type = 'E_WALLET';
            self.undoData.nsu = CartService.cart.nsu;
            self.undoData.paymentId = data.paymentId;
            self.undoData.remoteTerminalId = data.remoteTerminalId;
            self.undoData.walletType = walletType;

            const salePayment = {
                type: SalePaymentTypes.SALE_E_WALLET_PAYMENT,
                paymentId: data.paymentId,
                paymentValue: paymentValue,
                // FIXME: remover remoteTerminalId futuramente
                remoteTerminalId: data.remoteTerminalId,
                paymentTerminalId: data.remoteTerminalId,
                discountId: data.discountId,
                discountValue: data.discountValue,
                paymentProvider: { id: walletType },
            };

            PaymentProcessor.pushSalePayment({
                ...salePayment,
                walletType: walletType,
            });

            return EWalletService.waitPayment({
                paymentId: data.paymentId,
                remoteTerminalDescription: data.remoteTerminalDescription,
                walletType: walletType,
            }).then((waitRes) => {
                if (aborted) {
                    return;
                }
                block = true;
                const waitData = waitRes.data;

                LogService.info(
                    `Pago ID = ${data.paymentId}, TRNS_ID = ${waitData.transactionId}, WALLET_TYPE = ${walletType}, TERM_DESC = ${data.remoteTerminalDescription}`
                );

                CartService.cart.salePayments.push({
                    ...salePayment,
                    paymentSequence: PaymentService.payments().length,
                    paymentCodeMethod: $stateParams.paymentMethod.id,
                    billedPaymentMethod: waitData.billedPaymentMethod,
                    walletPaymentMethod: waitData.walletPaymentMethod,
                    installments: [],
                    // FIXME: remover futuramente o transactionId
                    transactionId: waitData.transactionId,
                    paymentTransactionId: waitData.transactionId,
                    walletName: waitData.walletName,
                    walletDocument: waitData.walletDocument,
                    paymentState: waitData.paymentState,
                    paymentDate: waitData.paymentDate,
                    billedBankCode: waitData.billedBankCode,
                    billedBankName: waitData.billedBankName,
                    paymentCustomer: waitData.paymentCustomer,
                    paymentAuthorizationCode: waitData.paymentAuthorizationCode,
                    paymentAuthorizerCNPJ: waitData.paymentAuthorizerCNPJ,
                    paymentAuthorizerName: waitData.paymentAuthorizerName,
                    customerPaymentVoucher: waitData.customerPaymentVoucher
                });

                PaymentService.currentPaymentDone();

                if (isSaleFullyPaid()) {
                    delete $rootScope.back;
                    $ionicLoading.show({
                        duration: 0,
                    }).then(() =>
                        UtilService.navigateWithoutHistory('easyorder.paymentresult')
                    );
                } else {
                    block = false;
                    $rootScope.back(true);
                }
            });
        }).catch(err => {
            if (aborted) {
                return;
            }
            AlertService
                .error(err.data != null ? err.data.message : err)
                .then(() => $rootScope.back());
        });

        function initPayment(paymentMethod, paymentValue) {
            // arredondando para duas casas decimais
            paymentValue = Math.round(new Number(paymentValue) * 100) / 100;

            self.undoData.campaigns = CartService
                .processPaymentCampaigns(paymentMethod, paymentValue);

            if (CartService.cart.totalValue < paymentValue) {
                paymentValue = CartService.cart.totalValue;
            }

            PaymentService.initPayment(paymentMethod, paymentValue, false);

            return paymentValue;
        };

        function isSaleFullyPaid() {
            return CartService.cart.totalValue == new Number(PaymentService.totalPayments());
        }

        function randomInt(min, max) {
            min = Math.ceil(min);
            max = Math.floor(max);
            return Math.floor(Math.random() * (max - min)) + min;
        }

        $rootScope.back = (skipUndo) => {
            aborted = true;
            
            $q((resolve) => {
                if (skipUndo || block) {
                    resolve();
                    return;
                }

                // desfazendo pagamento pendente
                PaymentService.clearInitializedPaymentData();
                if (self.undoData.campaigns != null) {
                    self.undoData.campaigns.forEach(CartService.undoCampaignDiscountsPartial);
                }
                if (self.undoData.paid) {
                    PaymentProcessor.undoPayment((payment) =>
                        payment.nsu == self.undoData.nsu &&
                        payment.type == self.undoData.type &&
                        payment.data.undo.paymentId == self.undoData.paymentId &&
                        payment.data.undo.remoteTerminalId == self.undoData.remoteTerminalId &&
                        payment.data.undo.walletType == self.undoData.walletType
                    );
                }

                resolve();
            }).finally(() => {
                if (block) {
                    return;
                }
                if (CartService.hasUnprocessedPredefinedPaymentMethod()) {
                    UtilService.navigateWithoutHistory('easyorder.predefinedpaymentmethods');
                } else {
                    UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                }
                delete $rootScope.back;
            });
        }
    });
})();
