(function () {
    'use strict';

    angular.module('easyorder.controllers.productdiscounts', [
            'easyorder.services.product',
            'easyorder.services.user',
            'easyorder.services.alert',
            'easyorder.services.log'
        ])

        .controller('ProductDiscountsCtrl', function ($scope,
            $ionicLoading,
            $stateParams,
            ProductService,
            CartService,
            UserService,
            ConfigService,
            AlertService,
            UtilService,
            $ionicModal,
            $q,
            LogService) {

            $scope.item = $stateParams.item;
            $scope.prevState = $stateParams.prevState;
            $scope.originalProductValue = $stateParams.item ? $stateParams.item.unitValue : undefined;
            $scope.discountProductValue;
            $scope.confirmBtnDiscount = false;
            $scope.enableHiddenDiscounts = false;
            $scope.automaticDiscount = $stateParams.automatic;
            $scope.data = {};
            $scope.isSelectedDiscount = false;
            $scope.discounts = [];
            $scope.discountsHidden = [];
            $scope.item = $stateParams.item;
            $scope.result = $stateParams.discountResult;
            $scope.cart = $stateParams.cart;
            $scope.cartService = CartService.cart;


            /**
             * 
             */
            $scope.loadNextPage = function () {
                if ($scope.enableHiddenDiscounts) {
                    $scope.enableHiddenDiscounts = false;
                } else {
                    $scope.checkDiscountPermission();
                }
            };


            /**
             * 
             */
            $scope.prepareDiscounts = function () {
                if ($scope.result && $scope.result.discountRuleTypes) {
                    for (let index = 0; index < $scope.result.discountRuleTypes[0].discountRules.length; index++) { 
                        if ($scope.result.discountRuleTypes[0].discountRules[index].hidden == true && Object.keys($scope.result.discountRuleTypes[0].discountRules[index].discount).length !== 0) {
                            $scope.discountsHidden.push($scope.result.discountRuleTypes[0].discountRules[index]);
                        } else {
                            if (Object.keys($scope.result.discountRuleTypes[0].discountRules[index].discount).length !== 0) {
                                $scope.discounts.push($scope.result.discountRuleTypes[0].discountRules[index]);
                            }
                        }
                    }
                } else {
                    $scope.discountType = $stateParams.discountType;
                    if ($scope.discountType) {
                        $scope.discounts = $scope.discountType.discountRules;
                    }
                }
            };

            /**
             * 
             */
            $scope.selectDiscount = function (item, $index, discount) {
                if ($scope.automaticDiscount) {
                    $scope.automaticDiscountPath(item, $index, discount);
                } else {
                    $scope.manualDiscountPath(item, $index, discount);
                }
            };

            /**
             * 
             */
            $scope.automaticDiscountPath = function (item, $index, discount) {
                if ($scope.cart) {
                    let objDiscounts = {
                        value: discount.discount.max,
                        type: discount.discount.type,
                        source: "RULE",
                        externalId: discount.code,
                    };
                    
                    CartService.removeDiscountBySource(objDiscounts.source);
                    CartService.addDiscount(objDiscounts);
                    CartService.updateCartTotalValue();
                    UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});

                    return;
                }

                $scope.selectedDiscount = discount
                $scope.selectedDiscount.sequenceId = $index;
                $scope.activateSelectedCssItem = $index;
                $scope.confirmBtnDiscount = true;
                $scope.isSelectedDiscount = true;
                
                if (Object.keys($scope.selectedDiscount.discount).length === 0) {
                    $scope.discountProductValue = $scope.originalProductValue;
                } else {
                    if ($scope.selectedDiscount.discount.type == "PERCENT") {
                        $scope.discountProductValue = item.unitValue - ((item.unitValue / 100) * $scope.selectedDiscount.discount.max);
                    } else if ($scope.selectedDiscount.discount.type == "VALUE") {
                        $scope.discountProductValue = item.unitValue - $scope.selectedDiscount.discount.max;
                    }
                }

                if ($scope.discountProductValue && $scope.discountProductValue < 0){
                    AlertService.info('Desconto não poderá ser aplicado, pois o valor do desconto deixa produto com valor negativo!');
                    $scope.selectedDiscount = null;
                } else {
                    $scope.addProductWithDiscountToCart();                            
                }
            };

            /**
             * 
             */
            $scope.manualDiscountPath = function (item, $index, discount) {
                UtilService.navigateWithoutHistory('easyorder.includediscounts', {
                    "discountData": {
                        "rule": discount,
                        "cart": $scope.cart,
                        "type": {
                            "description": $scope.discountType.description,
                            "code": $scope.discountType.code,
                        },
                        "item": item
                    }
                });
            };


            /**
             * ADICIONA O DESCONTO
             */
            $scope.addProductWithDiscountToCart = function () {
                if ($scope.selectedDiscount.code &&
                    $scope.selectedDiscount.discount.type &&
                    $scope.selectedDiscount.discount.max) {
                    
                    let discountObj = {
                            value: $scope.selectedDiscount.discount.max,
                            source: "RULE",
                            type: $scope.selectedDiscount.discount.type,
                            factor: $scope.item.quantitySold,
                            sequenceId: $scope.selectedDiscount.sequenceId,
                            externalId: $scope.selectedDiscount.code
                    };
                    if (!$scope.item.discounts) {
                        $scope.item.discounts = [];
                    }

                    if($scope.selectedDiscount.hidden){
                        $scope.item.authorizers = $scope.item.authorizers || {};
                        $scope.item.authorizers[HIDDEN_DISCOUNTS_EXHIBITION] = $scope.authorizer.id;
                    }

                    $scope.item.removeDiscountBySource(discountObj.source);
                    $scope.item.addDiscount(discountObj); //$scope.item foi adicionado ao carrinho antes dessa tela.
                    $scope.item.processTotalValue();
                    CartService.updateCartTotalValue();
                    
                    $scope.isSelectedDiscount = false;
                    $scope.selectedDiscount = null;
                    
                    $scope.addProductWithoutDiscountToCart();
                }
            };

            /**
             * 
             */
            $scope.addProductWithoutDiscountToCart = function () {
                if ($scope.item && $scope.automaticDiscount) {
                    if ($stateParams.prevState === "easyorder.barcode") {
                        UtilService.navigateWithoutHistory('easyorder.barcode');
                    } else {
                        UtilService.navigateWithoutHistory('easyorder.productsearch');
                    }
                } else if ($scope.cart) {
                    UtilService.navigateWithoutHistory('easyorder.purchasesummary');
                } else {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                }
            };

            /**
             * FUNÇÃO QUE ABRE O MODAL
             */
            $scope.checkDiscountPermission = function () {
                $scope.requestauth = {};
                var permission = UserService.checkPermission(HIDDEN_DISCOUNTS_EXHIBITION);
                LogService.info("Permissões: ", permission);
                
                if (!permission) {
                    $scope.openModal();
                } else {
                    $scope.requestauth.user = UserService.currentUser.username;
                    
                    if ($scope.discountsHidden) {
                        $scope.enableHiddenDiscounts = true;
                    }
                    LogService.info('Usuário autorizado por default', $scope.discounts)
                }
            };

            /**
             * 
             */
            $ionicModal.fromTemplateUrl('views/requestauthmodal.html', {
                scope: $scope,
                focusFirstInput: true,
                animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.modal = modal;
            });

            /**
             * 
             */
            $scope.openModal = function () {
                $scope.modal.show();
            };

            /**
             * 
             */
            $scope.closeModal = function () {
                $scope.modal.hide();
            };

            /**
             * 
             */
            $scope.$on('$destroy', function () {
                $scope.modal.remove();
            });

            /**
             * REQUISIÇÃO DE AUTENTICAÇÃO
             */
            $scope.requestAuth = function () {
                $ionicLoading.show({
                    template: 'Por favor, aguarde...',
                    duration: 0
                }).then(function () {
                    if (!$scope.requestauth.user) {
                        AlertService.error("Usuário inválido!");
                    } else if (!$scope.requestauth.password) {
                        AlertService.error("Senha inválida!");
                    } else {
                        UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [HIDDEN_DISCOUNTS_EXHIBITION],
                            (result) => {
                                LogService.info('Resultado da autorização do usuário: ', result);
                                $ionicLoading.hide();

                                if (result.authorized == true) {
                                    LogService.info('Usuário autorizado após inserir senha e login')
                                    $scope.enableHiddenDiscounts = true;
                                    $scope.authorizer = result.user;
                                    $scope.closeModal();
                                } else {
                                    LogService.warn('Usuário negado')
                                    AlertService.warn('Autorização negada!')
                                }
                            },
                            (error) => {
                                $ionicLoading.hide();
                                LogService.error("Erro ao consultar autorização!", error);
                                AlertService.restError(error, "Erro ao consultar autorização!");
                            });
                    }
                });
            };


            (function () {
                $scope.prepareDiscounts();
                
                if (ConfigService.config.business.discount_rule_automatic_application) {
                    if ($stateParams.discountType) {
                        if ($stateParams.discountType.discountRules.length == 1) {
                            $scope.selectDiscount($stateParams.item, 0, $stateParams.discountType.discountRules[0]);
                        }
                    } else if ($stateParams.discountResult) {
                        if ($stateParams.discountResult.discountRuleTypes.length == 1) {
                            if ($stateParams.discountResult.discountRuleTypes[0].discountRules.length == 1) {

                                $scope.selectDiscount($stateParams.item, 0, $stateParams.discountResult.discountRuleTypes[0].discountRules[0]);
                            }
                        }
                    }
                }
            })();
        });
})();
