(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.payment', [])
    .controller('BillingPaymentCtrl', function ($scope,
            $rootScope,
            $q,
            $state,
            $stateParams,
            $ionicLoading,
            ConfigService,
            PaymentService,
            UtilService,
            LogService,
            AlertService,
            PaymentProcessor,
            EFTService,
            UserService,
            BillingPaymentService) {
         
        var processedPayments = [];
        var cardStatementPaymentConfirmtion = {};
                
        /**
         * 
         */
        $scope.goBack = () => {
            let promise = Promise.resolve(true);
            
            if ($scope.isInParcialPayment()) {
                promise = $scope.undoCurrentPayments()
            }

            promise.then((okButton) => {
                if (okButton && $stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();
                    $rootScope.$broadcast('paymentInProgress', false);
                    UtilService.navigateWithoutHistory(backState, params);
                }
            });
        };
        
        /**
         * 
         */
        $scope.goNext = () => {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory:            $stateParams.flowHistory,
                searchData:             $stateParams.searchData,
                bills:                  $stateParams.bills,
                authorizerCode:         $stateParams.authorizerCode,
                paymentOperationType:   $stateParams.paymentOperationType,
                payments:               processedPayments
            };
            transitionParams.searchData.customer =  $scope.customer;

            UtilService.navigateWithoutHistory('easyorder.billingpaymentresult', transitionParams);
        };
        
        /**
         *
         */      
        $scope.isInParcialPayment = function() {
            return PaymentService.isInParcialPayment();
        };
        
        /**
         *
         */
        $scope.undoCurrentPayments = () => {
            return AlertService.confirm("Todos os pagamentos serão cancelados, deseja continuar?")
            .then(resp => {
                if (resp) {
                    return PaymentService.cancel(true)
                    .then(updateTotals)
                    .then(() => processedPayments.length = 0)
                    .then(() => { return true });
                }
            });
        };
        
        /**
         *
         */
        $scope.paymentResult = (promise) => {
            promise.then(result => {
                return $ionicLoading.show()
                .then(() => {
                    let salePayment = processTefPayment(result.paymentResults, PaymentService.currentPayment());

                    return PaymentService.isCardBinEnabled(salePayment.paymentCodeMethod, salePayment.cardNumber)
                    .catch(errorMessage => {
                        throw {data: {message: errorMessage}};
                    })
                   .then(() => {
                        PaymentService.currentPaymentDone();
                        processedPayments.push(salePayment);
                        PaymentProcessor.pushSalePayment(salePayment);
                        LogService.info("Pagamento de Cobrança recebido.");
                    })
                    .catch(error => {
                        PaymentService.currentPaymentDone(); // Faz com que o pagamento fiquei disponível para ser desfeito.
                        paymentErrorUndoPayments(error); // Se o pagamento não é válido, deve ser desfeito.
                    });
                })
                .finally($ionicLoading.hide);
            })
            .catch(err => {
                if (err) {
                    AlertService.error(err);
                }
            })
            .finally(() => {
                updateTotals();
                
                if (!$scope.currency.paymentValue) { // Há resto a pagar?
                    $ionicLoading.show()
                    sitefCardStatementPaymentIntegration()
                    .then(processSitefCardStatementPayment)
                    .then($scope.goNext)
                    .catch(() => {
                        PaymentService.cancel(true)
                        .then(updateTotals); 
                    }).finally($ionicLoading.hide);
                }
            });
        };
        
        /**
         *
         */
        function processTefPayment(tefPaymentResults, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SaleTEFPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.mideReferenceID             = currentPayment.mideReferenceID;
            salePayment.sefazNSU                    = currentPayment.sefazNSU;
            salePayment.fiscalIntegratorAuthModeOff = currentPayment.fiscalIntegratorAuthModeOff;
            salePayment.paymentCodeMethod           = 'DEBIT';
            salePayment.paymentValue                = currentPayment.paymentValue;
            salePayment.installments                = [];
            salePayment.paymentAuthorizerBrandType  = tefPaymentResults.IndiceCodigoBandeira;
            
            if (currentPayment.plan) {
                salePayment.paymentCodePlan       = currentPayment.plan.code;
                salePayment.paymentCodeEvent      = currentPayment.plan.paymentCodeEvent;
                salePayment.numberOfInstallments  = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment = currentPayment.plan.firstInstalment.dueDate;

                currentPayment.plan.instalments.forEach(function(instalment, _, arr) { // Convertendo Instalment em InstallmentsDetails
                    var instalmentDetails = {};
                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = instalment.instalmentValue;
                    this.push(instalmentDetails);
                }, salePayment.installments);

                if (currentPayment.plan.discount) {
                    salePayment.paymentValue -= currentPayment.plan.discount; 
                    // atualiza valor do pagamento
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));

                }

                if (currentPayment.plan.charges) {
                    salePayment.chargeValue = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }
                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            } else {
                salePayment.numberOfInstallments = tefPaymentResults.NumeroParcelas;
            }

            if (tefPaymentResults.NomeBandeiraCartao) {
                salePayment.paymentAuthorizerBrandName  = tefPaymentResults.NomeBandeiraCartao.trim();
            }

            if (ConfigService.device.lio) {
                salePayment.paymentAuthorizerCode = 0;
            } else {
                const tefProvider = ConfigService.config && 
                                    ConfigService.config.tefserver ? ConfigService.config.tefserver.provider : undefined;
                switch(tefProvider) {
                    case 'Auttar':
                        // No caso do CTF, o código do autorizador é obtido no campo 'CodigoAutorizadora'.
                        salePayment.paymentAuthorizerCode = tefPaymentResults.CodigoAutorizadora;
                        salePayment.vanCode               = tefPaymentResults.CodigoVAN;
                        break;
                    case 'Linx':
                    case 'SoftwareExpress':
                        salePayment.paymentAuthorizerCode = tefPaymentResults.IndiceCodigoRede;

                        if (tefPaymentResults.IndiceCodigoRede) {
                            salePayment.vanCode           = '' + parseInt(tefPaymentResults.IndiceCodigoRede, 10);
                        }

                        break;
                    default:
                        throw new Error('TEF provider not implemented!');
                }
            }
            salePayment.paymentAuthorizationCode    = tefPaymentResults.CodigoAutorizacao;
            salePayment.paymentMethodIndicator      = tefPaymentResults.TipoFinanciamento;
            salePayment.paymentAuthorizationNSU     = tefPaymentResults.NSU;
            salePayment.paymentAuthorizerNSU        = tefPaymentResults.NSURede;
            salePayment.cardNumber                  = tefPaymentResults.NumeroCartao;
            salePayment.customerPaymentVoucher      = tefPaymentResults.ComprovanteCliente;
            salePayment.bussinessPaymentVoucher     = tefPaymentResults.ComprovanteEstabelecimento;
            salePayment.acquirerName                = tefPaymentResults.NomeAdquirente;

            return salePayment;
        }
        
        /**
         *
         */
        function paymentErrorUndoPayments(error) {
            PaymentService.cancel(true).then(() => {
                AlertService.restError(error, "Não foi possível processar pagamento TEF")
                .then($scope.cancelModal);
            });
        }
        
        /**
         *
         */
        function sitefCardStatementPaymentIntegration() {
            var ret = $q.defer();
            var CDTBills = BillingPaymentService.filterConductorCardStatements($stateParams.bills);
            
            if (CDTBills.length) {
                var params = {
                    transactionType:    EFTService.transactionType.RENNER_BILLING_PAYMENT,
                    usePinpad:          false,
                    amount:             $scope.totalPayments,
                    typedCardNumber:    CDTBills[0].installmentCode,
                    operatorCode:       UserService.operatorCode()
                };
                
                EFTService.executeTransaction(params, {
                    paymentDone: ret.resolve,
                    paymentError: ret.reject,
                    displayMessage: LogService.info,
                    alertMessage: (message) => {
                        return $ionicLoading.hide().then(() =>
                            AlertService.show('Transação TEF', message)
                        ).finally(() =>
                            $ionicLoading.show()
                        );
                    },
                    clearDisplay: () => {}
                });
            } else {
                ret.resolve();
            }
            return ret.promise;
        }
        
        /**
         *
         */
        async function processSitefCardStatementPayment(tefResult) {
            if (tefResult) {
                var CDTBills = BillingPaymentService.filterConductorCardStatements($stateParams.bills);
                const fiscalInstant = Date.parse(`${tefResult.DataFiscal.replace(/^(\d{4})(\d{2})(\d{2})$/g, '$1-$2-$3')}T${tefResult.HoraFiscal.replace(/^(\d{2})(\d{2})(\d{2})$/g, '$1:$2:$3')}-03:00`);
                
                cardStatementPaymentConfirmtion.type                        = 'CardStatementPayment';
                cardStatementPaymentConfirmtion.paymentAuthorizationCode    = tefResult.CodigoAutorizacao;
                cardStatementPaymentConfirmtion.paymentMethodIndicator      = tefResult.TipoFinanciamento;
                cardStatementPaymentConfirmtion.paymentAuthorizationNSU     = tefResult.NSU;
                cardStatementPaymentConfirmtion.paymentAuthorizerNSU        = tefResult.NSURede;
                cardStatementPaymentConfirmtion.cardNumber                  = tefResult.NumeroCartao;
                cardStatementPaymentConfirmtion.customerPaymentVoucher      = tefResult.ComprovanteCliente;
                cardStatementPaymentConfirmtion.bussinessPaymentVoucher     = tefResult.ComprovanteEstabelecimento;
                cardStatementPaymentConfirmtion.paymentAuthorizerCode       = tefResult.IndiceCodigoRede;
                cardStatementPaymentConfirmtion.paymentAuthorizerBrandType  = tefResult.IndiceCodigoBandeira;
                cardStatementPaymentConfirmtion.paymentAuthorizerBrandName  = tefResult.NomeBandeiraCartao.trim();
                cardStatementPaymentConfirmtion.paidValue                   = $scope.totalPayments;
                cardStatementPaymentConfirmtion.minPaymentValue             = CDTBills[0].minPaymentValue;
                cardStatementPaymentConfirmtion.paymentValue                = CDTBills[0].installmentValue;
                cardStatementPaymentConfirmtion.processorCode               = CDTBills[0].processorCode;
                cardStatementPaymentConfirmtion.fiscalDate                  = new Date(fiscalInstant);
                
                
                PaymentProcessor.pushSalePayment(cardStatementPaymentConfirmtion); //Apenas para provocar a confirmação.
                LogService.info("Pagamento de Prestação enviado para o TEF.");

                let ret = {}
                angular.copy(cardStatementPaymentConfirmtion, ret);
                delete ret.type;
                
                CDTBills[0].cardStatementTEFPaymentData = ret;
                LogService.info(`Dados do Pagamento adicionados à fatura: ${CDTBills[0].toString()}.`);
            }
        }
        
        /**
         *
         */
        function updateTotals() {
            let selectedBillsValue = 0;
            let selectedTotalDiscount = 0;
            let selectedTotalCharges = 0;
            
            $stateParams.bills.forEach(bill => {
                selectedBillsValue += bill.items.reduce((acc, sel) => parseFloat((acc + sel.installmentValue).toFixed(2)), 0);
                selectedTotalDiscount += bill.items.reduce((acc, sel) => parseFloat((acc + sel.discountValue).toFixed(2)), 0);
                selectedTotalCharges += bill.items.reduce((acc, sel) => parseFloat((acc + sel.chargeTotalValue).toFixed(2)), 0);
            });
            
            $scope.currency.paymentValue = parseFloat((selectedBillsValue - selectedTotalDiscount + selectedTotalCharges).toFixed(2));
            $scope.totalPayments = PaymentService.totalPayments();
            $scope.currency.paymentValue = parseFloat($scope.currency.paymentValue - $scope.totalPayments.toFixed(2));
            $scope.currency.currentMaxPaymentValue = $scope.currency.paymentValue;
            
            LogService.info(`Totais atualizados: Total a pagar: ${$scope.currency.paymentValue} Total Pago: ${$scope.totalPayments}`);
            $rootScope.$emit("screen-total-value",  parseFloat((selectedBillsValue - selectedTotalDiscount + selectedTotalCharges).toFixed(2)));
        }
        
        $scope.onInitPaymentProcess = (data) => {
            LogService.info(`Iniciando pagameno para Operação de pagamento '${$scope.paymentOperationType}': ${data.method} - ${data.value}`);
        };
       
        (() => {
            $scope.debitMethodEnabled = !!ConfigService.config.business.available_billing_payment_methods.find(m => m === "DEBIT");
            $scope.currency = {};
            $scope.customer = $stateParams.searchData.customerIdentification;
            $scope.paymentOperationType = $stateParams.paymentOperationType;
            
            updateTotals();
        })();         
    })
})();
