(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.card.statement.list', [])
    .controller('BillingPaymentCardStatementListCtrl', function ($scope,
            $rootScope,
            $state,
            $ionicLoading,
            $stateParams,
            BillingPaymentService,
            UtilService,
            UserService,
            LogService) {
                
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                $rootScope.$emit("screen-total-value",  0);
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        
        /**
         *
         */
        $scope.goNext = () => {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory:    $stateParams.flowHistory,
                searchData:     $stateParams.searchData,
                bills:          [{
                    customer:   $stateParams.searchData.data[0].customer,
                    items:      convertCardStatements($stateParams.searchData.data[0].cardStatements[0])
                }],
                authorizerCode: $scope.authorizerUser ? $scope.authorizerUser : UserService.operatorCode(),
                paymentOperationType: (() => {
                    if ($stateParams.searchData.typeFilter === BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT) {
                        return 'PAYMENT_TOTVSCARD_CARD_STATEMENT';
                    }
                })()
            };
            UtilService.navigateWithoutHistory('easyorder.billingpaymentcustomeridentification', transitionParams);
        };
        
        /**
         *
         */
        function convertCardStatements(cardStatement) {
            var ret = [];
            
            if (cardStatement) {
                let bill = {
                    type:                           'CARD_STATEMENT',
                    installmentCode:                cardStatement.debtIdentifier,
                    installmentValue:               $scope.currency.paymentValue,
                    discountValue:                  0,
                    chargeTotalValue:               0,
                    installmentDate:                $scope.dueDate,
                    processorCode:                  cardStatement.processorCode,
                    minPaymentValue:                cardStatement.minPaymentValue,
                    purchaseCardNumber:             cardStatement.purchaseCardNumber
                };
                bill.toString = () => `[${bill.installmentCode} - ${bill.installmentValue}] Data: ${bill.installmentDate}`;
                ret.push(bill);
            }
            return ret;   
        }
        
        /**
         *
         */
        $scope.$watch('currency.paymentValue', newValue => {
            $rootScope.$emit("screen-total-value",  newValue);
        });
        
        /**
         *
         */
        (() => {
            $ionicLoading.show()
            .then($ionicLoading.hide)
            .then(() => {
                const params = $stateParams.searchData;
                const cardStatement = params.data[0].cardStatements[0];
                
                $scope.customerName = params.data[0].customer.name;
                $scope.totalValue = cardStatement.totalValue;
                $scope.minPaymentValue = cardStatement.minPaymentValue;
                
                $scope.currency = {
                    paymentValue: cardStatement.totalValue,
                    minPaymentValue: cardStatement.minPaymentValue
                };
                
                if (cardStatement.dueDate) {
                    $scope.dueDate = new Date(new Date(params.data[0].cardStatements[0].dueDate).setHours(0,0,0,0));
                }
                LogService.info(`Fatura selecionada: ${$scope.currency.paymentValue}-${$scope.currencyminPaymentValue}-${$scope.dueDat}`);
            });
        })();
    })
})();