(function () {
    'use strict';
    
    angular.module('easyorder.billing.payment.v1.customer', [])
    .controller('BillingPaymentCustomerIdentificationCtrl', function ($scope,
            $state,
            $stateParams,
            $ionicLoading,
            UtilService,
            AlertService,
            LogService,
            BillingPaymentService) {
        
        $scope.startedWithDocument = !!$stateParams.searchData.customerIdentification && !!$stateParams.searchData.customerIdentification.document;
        var schData = {};
        angular.copy($stateParams.searchData, schData);
                                        
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        
        /**
         * 
         */
        $scope.goNext = () => {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory:            $stateParams.flowHistory,
                searchData:             schData,
                bills :                 $stateParams.bills,
                authorizerCode:         $stateParams.authorizerCode,
                paymentOperationType:   $stateParams.paymentOperationType
            };
            transitionParams.searchData.customerIdentification =  $scope.customer;

            UtilService.navigateWithoutHistory('easyorder.billingpayment', transitionParams);
        };
        
        /*
         *
         */
        /**
         *
         */
        $scope.validateDocument = () => {
            var doc = $scope.customer.document;

            if (doc && !$scope.startedWithDocument) {
                if (doc.length == 11) {
                    if ($scope.validDocument = validaCPF(doc)) {
                        searchCustomer();
                    } else {
                        AlertService.warn("CPF inválido")
                        .then(() => $scope.customer.document = "");
                    }
                }
            }
        };
        
        /**
         *
         */
        function searchCustomer() {
            if ($scope.validDocument) {
                LogService.info(`Buscando informações para CPF: ${$scope.customer.document}`);
                
                $ionicLoading.show()
                .then(() => BillingPaymentService.getCustomer($scope.customer.document))
                .finally($ionicLoading.hide)
                .then(result => $scope.customer = result)
                .catch(err => {
                    AlertService.restError(err)
                    .then(() => $scope.customer.email = "");
                 });
            }
        };
        
        /**
         * 
         */
        function getCustomer(customer) {
            var promise;
            
            if (customer && customer.email) {
                promise = Promise.resolve(customer);
            } else {
                promise = BillingPaymentService.getCustomer(customer.document)
                .catch(() => customer);
            }
            return promise;
        }

        (() => {
            LogService.info(`Cliente foi informado previamente: ${$scope.startedWithDocument}`);
            
            $ionicLoading.show()
            .then(() => schData.customerIdentification)
            .then(getCustomer)
            .then(result => $scope.customer = result)
            .then(customer => $scope.validDocument = validaCPF(customer.document))
            .finally($ionicLoading.hide);
        })();
    });
})();