(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.list', [])
    .controller('BillingPaymentListCtrl', function ($scope,
            $rootScope,
            $state,
            $ionicLoading,
            $stateParams,
            $ionicPopup,
            $filter,
            BillingPaymentService,
            UtilService,
            UserService,
            LogService,
            AlertService,
            ConfigService) {
        
        $scope.selectedBills = [];
        $scope.bills = [];
        $scope.allItemsTabClass = "tab-item active";
        $scope.primaryItemsTabClass = "tab-item";
        $scope.secondaryItemsTab = "tab-item";
        
        $scope.selectedCount = 0;
        $scope.selectedTotal = 0;
        $scope.selectedBillsValue = 0;
        $scope.selectedTotalDiscount = 0;
        $scope.selectedTotalCharges = 0;
        $scope.reportShown = false;
        $scope.selectedForChargesEdit = undefined;
        $scope.authorizerUser = undefined;
        $scope.selectedTab = 0
        
        let holderCardNumber = null;
        const TAB_FILTER_ALL = 0;
        const TAB_FILTER_HOLDER = 1;
        const TAB_FILTER_DEPENDENTS = 2;
        
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                $rootScope.$emit("screen-total-value",  0);
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        
        /**
         *
         */
        $scope.goNext = () => {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory:        $stateParams.flowHistory,
                searchData:         $stateParams.searchData,
                bills:              [{         
                    customer:   $stateParams.searchData.data[0].customer,
                    items:      $scope.selectedBills,
                }],
                authorizerCode:     $scope.authorizerUser ? $scope.authorizerUser : UserService.operatorCode(),
                paymentOperationType: (() => {
                    if ($stateParams.searchData.typeFilter === BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK) {
                        return 'PAYMENT_TOTVSCARD_CREDIT_BOOK';
                    }
                })()
            };
            UtilService.navigateWithoutHistory('easyorder.billingpaymentcustomeridentification', transitionParams);
              
        };
        
        /**
         *
         */
        $scope.clickOnBill = (bill) => {
            LogService.info(`Validando mudança de seleção para: ${bill.toString()}`);
            
            validateClickOnBill(bill)
            .then(() => $scope. $apply(bill.selected = !bill.selected))
            .then(() => $scope.$apply(updateSelectedData))
            .catch(AlertService.warn)
            .catch(LogService.warn);
        };
        
        /**
         *
         */
        $scope.selectTab = (tabIndex) => {
            var promise = Promise.resolve(true);
            
            if ($scope.selectedTab !== tabIndex) {
                if ($scope.selectedBills.length) {
                    promise = AlertService.confirm("Ao mudar de filtro, a seleção de parcelas será desfeita");
                }
            }
                
            promise.then(res => {
                if (res) {
                    $scope.selectedBills = [];
                    
                    if (tabIndex === 0) {
                        $scope.allItemsTabClass = "tab-item active";
                        $scope.primaryItemsTabClass = "tab-item";
                        $scope.secondaryItemsTab = "tab-item";
                        $scope.selectedTab = tabIndex;
                        return applyTabFilter(TAB_FILTER_ALL);
                    } else if (tabIndex === 1) {
                        $scope.allItemsTabClass = "tab-item";
                        $scope.primaryItemsTabClass = "tab-item active";
                        $scope.secondaryItemsTab = "tab-item";
                        $scope.selectedTab = tabIndex;
                        return applyTabFilter(TAB_FILTER_HOLDER);
                    } else if (tabIndex === 2) {
                        $scope.allItemsTabClass = "tab-item";
                        $scope.primaryItemsTabClass = "tab-item";
                        $scope.secondaryItemsTab = "tab-item active";
                        $scope.selectedTab = tabIndex;
                        return applyTabFilter(TAB_FILTER_DEPENDENTS);
                    }
                }   
            });
        };
        
         /**
          * FUNÇÃO QUE ABRE O MODAL
          */
        $scope.checkChargesChangePermission = function (bill, event) {
            event.stopPropagation();
            $scope.shutdownReport(); //Porque a propagação do evento foi cancelada, fazendo manualmente.
            
            if (bill.chargeInitialTotalValue) {
                if (!$scope.authorizerUser) {
                    return UserService.requestAuthorization(BILLING_PAYMENT)
                    .then(username =>  $scope.authorizerUser = username)
                    .then(() => LogService.info(`Usuário autorizador "${$scope.authorizerUser}" foi atribuído no controlador.`))
                    .then(() => editCharges(bill));
                } else {
                    return editCharges(bill);
                }
            }
        };
        
        /**
         *
         */
        $scope.changeReportVisibility = () => {
            $scope.reportShown = !$scope.reportShown;
        };
        
        /**
         *
         */
        $scope.shutdownReport = () => {
            $scope.reportShown = false;
        };
        
        /**
         *
         */
        function editCharges(bill) {
            $scope.popup = {};
            $scope.popup.editedCharges = bill.chargeTotalValue;
            
            var title = 'Alteração Juros de Parcela';
            var subTitle = 'Valor máximo: ' + $filter('currency')(bill.chargeInitialTotalValue) 
                + '<br/>Valor mínimo (IOF): ' + $filter('currency')(bill.iofValue); 
            var body = `<style type="text/css">
                            input.inputCampoMoeda {
                                border: none;
                                height: 1.5em;
                                width: 100%;
                                font-size: 2.8em;
                                text-align: center;
                                background-color: transparent !important;
                            }
                            
                            input.inputCampoMoeda:focus {
                                outline: none;
                            }
                        </style>
                        <span><ln-campo-moeda model="popup.editedCharges" valormax="${bill.chargeInitialTotalValue}" valormin="${bill.iofValue}"/></span>`;
            
            return $ionicPopup.confirm({
                title: title,
                subTitle: subTitle,
                template: body,
                scope: $scope,
                buttons: [
                    { text: '<b>Cancelar</b>', type: 'button-stable' },
                    { 
                        text: '<b>Confirmar</b>', type: 'button-theme',
                        onTap: () => {
                            bill.chargeTotalValue = $scope.popup.editedCharges;
                            updateSelectedData();
                        } 
                    }
                ]
            });
        }
        
        /**
         *
         */
        function validateClickOnBill(bill) {
            var ret = Promise.resolve(bill);
            var clickIntendedToSelect = !bill.selected;
            
            if (clickIntendedToSelect) {
                ret = ret.then(hasUserPermissionSelect180DaysBill);
            }
            ret = ret.then(checkBillPrecedenceConstraint);
            
            return ret;
        }
        
        /**
         * Regra extraída do PDV
         */
        function hasUserPermissionSelect180DaysBill(bill) {
            let ret = Promise.resolve(bill);
            
            var differenceInTime = new Date().getTime() - bill.installmentDate.getTime();
            var differenceInDays = Math.floor(differenceInTime / (1000 * 3600 * 24));
            LogService.info(`Validando regra 180 para ${bill.toString()}: ${differenceInDays} dias de diferença.`);
            
            if (differenceInTime > 0) {
                if (differenceInDays > 180 && !$scope.authorizerUser) {
                    LogService.info("Prestação como mais de 180 dias de vencimento! Solicitando autorização...");
                    
                    ret = AlertService.warn("Prestação como mais de 180 dias de vencimento!")
                    .then(() => {
                        return UserService.requestAuthorization(BILLING_PAYMENT)
                        .then(username => $scope.authorizerUser = username)
                        .then(() => LogService.info("Seleção autorizada."))
                        .then(() => Promise.resolve(bill))
                        .catch(() => Promise.reject("Autorização não obtida."))
                     });
                }
            }
            return ret;
        }
        
        /**
         * Checa a regra de que não se pode selecionar uma conta sem que a contas mais 
         * antigas do mesmo contrato também não tenham sido.
         *
         */
        function checkBillPrecedenceConstraint(bill) {
            let ret = Promise.resolve(bill);
            
            var constraint = $scope.bills.find(b => 
                b.contractCode === bill.contractCode 
                && b.installmentNumber !== bill.installmentNumber
                && ((!b.selected && b.installmentDate < bill.installmentDate)
                    || (b.selected && b.installmentDate > bill.installmentDate)));
                
            if (constraint) {
                if (!bill.selected) {
                    ret = Promise.reject(`Existem prestações em aberto anteriores à prestação selecionada. (${constraint.installmentNumber}/${constraint.contractInstallmentsNumber})`);
                } else {
                    ret = Promise.reject(`Há seleção de prestações posteriores à prestação selecionada. (${constraint.installmentNumber}/${constraint.contractInstallmentsNumber})`);
                }
            }
            return ret;
        }
        
        /**
         *
         */
        async function updateSelectedData() {
            $scope.selectedBills = $scope.bills.filter(bill => bill.selected);
            $scope.selectedCount = $scope.selectedBills.length;
            $scope.selectedBillsValue = $scope.selectedBills.reduce((acc, sel) => parseFloat((acc + sel.installmentValue).toFixed(2)), 0);
            $scope.selectedTotalDiscount = $scope.selectedBills.reduce((acc, sel) => parseFloat((acc + sel.discountValue).toFixed(2)), 0);
            $scope.selectedTotalCharges = $scope.selectedBills.reduce((acc, sel) => parseFloat((acc + sel.chargeTotalValue).toFixed(2)), 0);
            $scope.selectedTotal = $scope.selectedBillsValue - $scope.selectedTotalDiscount + $scope.selectedTotalCharges;
            $rootScope.$emit("screen-total-value",  $scope.selectedTotal);
        }
        
        /**
         *
         */
        function applyTabFilter(filterType) {
            return $ionicLoading.show()
                .then(() => filterBills(filterType))
                .then(convertBills)
                .then(updateSelectedData)
                .finally($ionicLoading.hide);
        }
        
        /**
         *
         */
        async function filterBills(filterType) {
            const params = $stateParams.searchData;
            let ret = params.data[0].contracts || [];
            
            if (params.processorFilter) {
                ret = ret.filter(d => d.processorCode === params.processorFilter);
            }
                
            if ($scope.holderCardNumber) {
                if (TAB_FILTER_HOLDER === filterType) {
                    ret = ret.filter(d => d.purchaseCardNumber === holderCardNumber);
                } else if (TAB_FILTER_DEPENDENTS === filterType) {
                    ret = ret.filter(d => d.purchaseCardNumber !== holderCardNumber);    
                }
            }
            return ret;
        }
        
        /**
         *
         */
        async function convertBills(bills) {
            $scope.bills = convertContracts(bills);
        }
        
        /**
         *
         */
        function convertContracts(contracts) {
            var ret = [];
            if (contracts) {
                contracts.forEach(c => {
                    let bill = {
                        type:                           billType(c.productType),
                        installmentCode:                c.debtIdentifier,
                        installmentValue:               c.installmentValue,
                        installmentNumber:              c.installmentNumber,
                        installmentDate:                new Date(new Date(c.installmentDate).setHours(0,0,0,0)),
                        contractInstallmentsNumber:     c.contractInstallmentsNumber, 
                        contractCode:                   c.contractCode,
                        productType:                    c.productType,
                        arrearsInterestValue:           c.arrearsInterestValue,
                        insuranceValue:                 c.insuranceValue,
                        insuranceArrearsInterestValue:  c.insuranceArrearsInterestValue,
                        arrearsPenaltyValue:            c.arrearsPenaltyValue,
                        remunerationInterestValue:      c.remunerationInterestValue,
                        chargeTotalValue:               parseFloat(((c.insuranceArrearsInterestValue || 0) + (c.arrearsInterestValue || 0) 
                                                            + (c.arrearsPenaltyValue || 0) + (c.iofValue || 0) + (c.remunerationInterestValue || 0)).toFixed(2)),
                        iofValue:                       c.iofValue || 0,    
                        discountValue:                  c.discountValue,
                        processorCode:                  c.processorCode,
                        purchaseCardNumber:             c.purchaseCardNumber,
                        selected:                       false,
                        arreared:                       new Date(c.installmentDate).setHours(0,0,0,0) < new Date().setHours(0,0,0,0),
                    };
                    bill.chargeInitialTotalValue = bill.chargeTotalValue;
                    bill.selected = bill.arreared;
                    bill.toString = () => `[${bill.contractCode} - ${c.installmentNumber} / ${c.contractInstallmentsNumber}] Data: ${bill.installmentDate} Selecionada: ${bill.selected}`;
                    
                    ret.push(bill);
                });
            }
            return ret;
        }

        function billType(productType) {
            switch (productType) {
            case "E":
            case "EP":
            case "EP/SR":
            case "SR":
                return "LOAN";
            case "T":
            case "SPR":
            case "SVB":
                return "INSURANCE";
            case "AC":
                return "ARRANGEMENT";
            case "V":
            default:
                return "CONTRACT";
            }
        }
        
        /**
         * Consulta número do cartão do cliente através de seu CPF.
         */
        function getCustomerCardNumber() {
            const params = $stateParams.searchData;
            const customerIdentification = params.customerIdentification;

            return Promise.resolve(
                hasCustomerIdentification(params) && !isUnencryptedCard(params) ?
                    decryptCardNumber(customerIdentification.cardNumber) : checkCardNumber(params)
            );
        }

        function checkCardNumber (params) {
            return hasCustomerIdentification(params) ? params.customerIdentification.cardNumber : "--";
        }

        /**
         * Apenas a consulta por cpf trás o número do cartão criptografado
         */
        function isUnencryptedCard (params) {
            return params.readInputType != 'TYPED';
        }

        function hasCustomerIdentification (params) {
            return params.customerIdentification && params.customerIdentification.cardNumber;
        }

        function decryptCardNumber(cardNumber) {
            const key = ConfigService.config.business.triple_des_key;

            if (key == null) {
                LogService.warn(
                    'Chave Triple-DES não configurada, retornando cartão, criptografado...'
                );
                return cardNumber;
            }

            return CryptoJS.TripleDES.decrypt({
                ciphertext: CryptoJS.enc.Hex.parse(cardNumber),
            }, CryptoJS.enc.Hex.parse(`${key}${key}`), {
                mode: CryptoJS.mode.ECB,
                padding: CryptoJS.pad.NoPadding,
            }).toString(CryptoJS.enc.Utf8);
        }
        
        /**
         *
         */
        function cardNumberMask (cardNumber){
            if (cardNumber && cardNumber.length == 2) {
                return cardNumber;
            }

            let mask; 
            let prefix = cardNumber.substr(0, 4)
            let suffix = cardNumber.substr(12, 16)
            mask = prefix + " **** **** " + suffix
            return mask;   
        }

        (() => {
            $ionicLoading.show()
            .then(getCustomerCardNumber)
            .then(cardNumber => {
                holderCardNumber = cardNumber;
                $scope.holderCardNumber = cardNumberMask(cardNumber);
            })
            .then($ionicLoading.hide)
            .then(() => $scope.selectTab(0))
            .then(() => {
                const params = $stateParams.searchData;
               
                $scope.customerName = params.data[0].customer.name;
                $scope.totalCreditValue = params.data[0].customer.totalCreditValue;
                $scope.availableCreditValue = params.data[0].customer.availableCreditValue;
                $scope.customerType = params.data[0].cardStatements && params.data[0].cardStatements.length > 0 && params.data[0].cardStatements[0].processorCode == 'CONDUCTOR' ? 'CBR' : 'CCR';
            });
        })();
    });
})();