(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.result', [])
    .controller('BillingPaymentResultCtrl', function (
                    $rootScope,
                    $scope,
                    $q,
                    $stateParams,
                    $ionicLoading,
                    UtilService,
                    BillingPaymentService,
                    ConfigService,
                    CartService,
                    UserService,
                    AlertService,
                    PaymentService,
                    LogService) {
                
        $scope.newOrderActionText = ($rootScope.externallyLoadedApp ? "Sair" : $scope.NEW_ORDER);
        $scope.printingEnabled = ConfigService.config.business.printing_enabled;
        $scope.sendMailEnabled = ConfigService.config.business.send_mail_enabled;
        
        /**
         * Em caso de erro na integração, a NSU será alterada e então precisamos
         * armazenar aqui para confirmação/cancelamento dos pagamentos.
         */
        var paymentServiceNSU = CartService.cart.nsu;
        
        /**
         *
         */
        async function createIntegrationRequest() {
            return {
                appInfo: {
                    name: $scope.APP_NAME,
                    version: $scope.appVersion
                },
                data: {
                    transactionKey: {
                        companyNumber:  ConfigService.config.companyNumber,
                        storeNumber:    ConfigService.config.storeNumber,
                        nsu:            CartService.cart.nsu,
                        terminalNumber: ConfigService.config.terminalNumber,
                        date:           new Date().toISOString()
                    },
                    operatorCode:               UserService.operatorCode(),
                    authorizerCode:             $stateParams.authorizerCode,
                    customerIdentification:     $stateParams.searchData.customer,
                    readInputType:              $stateParams.searchData.readInputType,
                    payments:                   $stateParams.payments,
                    billing:                    $stateParams.bills.map(data => {
                        return {
                            customer: (dataCustomer => {
                                delete dataCustomer.availableCreditValue;
                                delete dataCustomer.totalCreditValue;

                                return dataCustomer;
                            })(data.customer),
                            billingItems:   data.items.map(b => {
                                delete b.selected;
                                delete b.arreared;
                                delete b.chargeInitialTotalValue;
                                delete b.contractInstallmentsNumber;
                                delete b.minPaymentValue;
                                
                                b.installmentTotalValue = parseFloat((b.installmentValue + b.chargeTotalValue - b.discountValue).toFixed(2));
                                b.installmentNSU = ConfigService.nsuControll.incrementNSU();
                                    
                                return b;
                            })
                        };
                    })
                }
            };
        }
        
        /**
         *
         */
        $scope.initSendEmailRequest = () => {
            const mailRequest = new MailRequest();
            mailRequest.type = MailRequest.TYPES.REPORT;
            mailRequest.subject = "Pagamento de Cobranças";
            mailRequest.attachmentContent = $scope.documents.map(doc => {
                if (doc instanceof PDF || doc instanceof IncompletePDF) {
                    return new Attachment(doc.base64, doc.printContents[0].title + ".pdf");
                }
            });
            return mailRequest;
        };
        /**
         *
         */
        $scope.onConfimRestart = (result) => {
            result.then(() => {
                CartService.initCart();
                $rootScope.$emit("screen-total-value",  0);
                
                UtilService.navigateToMainScreen();
            });
        };
        /**
         *
         */
        $scope.onDocumentClick = doc => {
            $scope.printDocument = doc;
            if (doc instanceof PDF) {
                $scope.pdf = doc;
            }
        }
        /**
         *
         */
        function processIntegration(request) {
            return BillingPaymentService.integrate(request)
            .catch(error => {
                 LogService.error(error);
                 
                return $ionicLoading.hide()
                .then(() => AlertService.confirm('Tentar efetivar o pagamento novamente?'))
                .then(result => {
                    if (result) {
                        CartService.reloadNSU();
                        
                        return createIntegrationRequest()
                        .then(req => processIntegration(req));
                    } else {
                        return $q.reject();
                    }
                })
            });
        }
        /**
         *
         */
        function confirmTransaction(integrationResp) {
            var request = {
                appInfo: {
                    name: $scope.APP_NAME,
                    version: $scope.appVersion
                },
                transactionKey: {
                    companyNumber:  ConfigService.config.companyNumber,
                    storeNumber:    ConfigService.config.storeNumber,
                    nsu:            CartService.cart.nsu,
                    terminalNumber: ConfigService.config.terminalNumber,
                    date:           new Date().toISOString()
                }
            };
            
            return BillingPaymentService.confirmTransaction(request)
            .then(() => integrationResp)
            .catch(error => {
                LogService.error(error);
                
                return $ionicLoading.hide()
                .then(() => AlertService.confirm(`Falha ao confirmar transação, deseja tentar novamente?`))
                .then(tryAgain => {
                    if (tryAgain) {
                        return confirmTransaction(integrationResp);
                    } else {
                        return $q.reject();
                    }
                });
            });
        }
        /**
         *
         */
        function processPaymentConfirmation(integrationResp) {
            return PaymentService.confirmPaymentsWith(paymentServiceNSU)
            .then(() => integrationResp)
            .catch(() => {
                return $ionicLoading.hide()
                .then(() => AlertService.confirm(`Falha ao confirmar pagamentos, deseja tentar novamente?`))
                .then(tryAgain => {
                    if (tryAgain) {
                        return processPaymentConfirmation(integrationResp);
                    } else {
                        return $q.reject();
                    }
                });
            });
        }
        /**
         *
         */
        function cancelPayments(error) {
            LogService.error(error);
            
            PaymentService.cancelWith(paymentServiceNSU)
            .catch(err => LogService.error(err))
            .then(() => CartService.reloadNSU())
            .then(() => AlertService.alert("Aviso!", "Pagamentos existentes foram desfeitos."))
            .then(() => {
                if ($stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();
                    UtilService.navigateWithoutHistory(backState, params);
                }
            });
        }
        /**
         *
         */
        async function presentVouchers(integrationResp) {
            return parsePrintContents(integrationResp.printData)
            .then(printContentsMap => {
                var documents = [];
                
                for (var content of printContentsMap) {
                    var pdf = content[1][0].base64PDF;
                    documents.push(getPDF(pdf  !== undefined ? pdf : danfeBase64, content[1]));
                }
                return documents;
            })
            .then(showFirstPDF); 
        }
        /**
         *
         */
        async function parsePrintContents(printData) {
            var printContentsMap = new Map();
            
            printData.forEach(data => {                
                if (printContentsMap.has(data.documentTitle)) {
                    printContentsMap
                        .get(data.documentTitle)
                        .push(new PrintContent(data.documentTitle, data.paperType, data.base64PrintData, data.base64PDF));
                } else {
                    printContentsMap
                        .set(data.documentTitle, [new PrintContent(data.documentTitle, data.paperType, data.base64PrintData, data.base64PDF)]);
                }
            });
            return printContentsMap;
        }
        /**
         *
         */
        function getPDF(base64PDF, printContents) {
            let ret;
            if (base64PDF) {
               try {
                    ret = new PaymentResultPDF(base64PDF, printContents);
                } catch(err) {
                    LogService.error(err instanceof Error ? err.stack : err);
                    LogService.error(`failed to parse pdf base64:\n${base64PDF}`);
                    pdf = new IncompletePDF(base64PDF, printContents);
                }
            } else {
                ret = new DummyDocument(printContents);
            }
            return ret;
        }
        /**
         *
         */
        async function showFirstPDF(documents) {
            $scope.documents = documents;
            const doc = documents.find(doc => doc instanceof PDF);
            
            if (doc) {
                $scope.pdf = doc;
                $scope.printDocument = doc;
            } else {
                $scope.printDocument = documents[0];
            }
        }
        
        /**
         *
         */
        async function showAutoEmailSentMessage(integrationResp) {
            if (integrationResp && integrationResp.isAutoEmailEnable) {
                return AlertService.info("Comprovantes serão enviados automaticamente ao e-mail cadastrado.")
                .then(() => integrationResp);
            }
            return integrationResp; 
        }
        
        (() => {
            $scope.mails = [$stateParams.searchData.customer.email];
            
            let integrationPromise = $ionicLoading.show()
            .then(createIntegrationRequest)
            .then(processIntegration)
            .then(confirmTransaction)
            .then(processPaymentConfirmation);
            
            integrationPromise.then(integrationResp => {
                showAutoEmailSentMessage(integrationResp)
                .then(presentVouchers)
                .then(() => LogService.info("Operação de pagamento do cobranças efetuada com sucesso."));
            })
            .finally($ionicLoading.hide)
            .catch(cancelPayments);
        })(); 
    });
    
    class PrintContent {
        constructor(title, paperType, data, base64PDF) {
            this.title = title;
            this.paperType = paperType;
            this.data = data;
            this.base64PDF = base64PDF;
        }
    }
    class PDF {
        constructor(base64) {
            const byteCharacters = atob(base64);
            const byteNumbers = new Array(byteCharacters.length);
    
            for (let i = 0; i < byteCharacters.length; i++) {
                byteNumbers[i] = byteCharacters.charCodeAt(i);
            }
    
            this.base64 = base64;
            this.data = new Uint8Array(byteNumbers);
            this.type = 'PDF';
    
        }
    
        toURL() {
            const blob = new Blob([this.data], {type: 'application/pdf'});
    
            return URL.createObjectURL(blob);
        }
    }
    class PaymentResultPDF extends PDF {
        constructor(base64, printContents) {
            super(base64);
            this.printContents = printContents;
            this.printed = false;
        }
    }
    class DummyDocument {
        constructor(printContents) {
            this.printContents = printContents;
            this.type = 'DUMMY';
            this.printed = false;
        }
    }
    class IncompletePDF extends DummyDocument {
        constructor(base64, printContents) {
            super(printContents);
            this.base64 = base64;
        }
    }
})();

