(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.chip.search', [])
    .controller('BillingPaymentChipSearchCtrl', function ($scope,
            $q,
            $timeout,
            $state,
            $stateParams,
            $ionicLoading,
            UtilService,
            AlertService,
            EFTService,
            BillingPaymentService,
            CartService,
            LogService) {
                
        var searchCustomer = CartService.cart.customer || {};
        
        /**
         * 
         */
        $scope.goBack = () => {
             if ($scope.goingBack) {
                return;
            }
            $scope.goingBack = true;
            
            $scope.message = "Cancelando operação...";
            
            $q((resolve, reject) => {
                EFTService.finalizeTransaction().then(resolve, reject);
            })
            .then(() => {
                if ($stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();
                    
                    UtilService.navigateWithoutHistory(backState, params);
                }
            })
            .catch(() => {
                $scope.message = "Não foi possível cancelar operação";
                $scope.goingBack = false;
            });
        };
        
        /**
         * Inicializa a consulta do cliente através da leitura do cartãoo do cliente.
         */
        $scope.doChipSearch = function() {
            EFTService.executeTransaction(
                {
                    transactionType:    EFTService.transactionType.TRAIL_READ,
                    amount:            0
                }, {
                    paymentDone: function(paymentResults) {
                        if (paymentResults && paymentResults.Trilha2) {
                            const inputType  = paymentResults.Trilha2.indexOf('=') > 0 ? 'CHIP': 'READ_CARD';
                            var searchValue = paymentResults.Trilha2.split('=')[0];
                            this.clearDisplay();
                            
                            var req = {
                                cardNumber: searchValue,
                                customerIdentificationType: inputType === 'CHIP' ? 'CHIP_CARTAO': 'TARJA_MAGNETICA_CARTAO',
                                operationCode : "VENDA_VAREJO"
                            };
                
                            $ionicLoading.show()
                            .then(() => BillingPaymentService.search(req))
                            .finally($ionicLoading.hide)
                            .then(data => {
                                searchCustomer.name = data.customer.name;
                                searchCustomer.document = data.customer.document;
                                searchCustomer.documentType = 'CPF';
                                searchCustomer.cardNumber = req.cardNumber;
                                
                                goOn(data, inputType);
                            })
                            .catch((error) => {
                                AlertService.restError(error, "Erro ao obter dados do cartão.")
                                .then($scope.goBack);
                            });
                        } else {
                            AlertService.error("Erro ao obter dados do cartão. Retorno do pinpad inválido.")
                            .then($scope.goBack);
                        }
                    },
                    paymentError: function(paymentResults) {
                        var errMsg = paymentResults && paymentResults.MensagemErro ? paymentResults.MensagemErro : "Não foi possível ler cartão.";
                        
                        AlertService.error(errMsg).then(function() {
                            $scope.goBack();
                        });
                    },
                    clearDisplay: function() {
                        $timeout(function () {
                            $scope.message = "";
                        });
                        LogService.info("clearDisplay");
                    },
                    displayMessage: function(message) {
                        if (message) {
                            $timeout(function () {
                                $scope.message = message;
                            });
                        }
                        LogService.info("displayMessage: " + $scope.message);
                    },
                    selectOptionExtended: function (title, options, responseHandlerExtended) {
                        $scope.$apply(function () {
                            $scope.message = title;
                        });
                        $scope.processOptions(options, responseHandlerExtended);
                        LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
                    }
                }
            );
        };

        /**
         *
         */
        function goOn(data, inputType) {
            let contracts = data.contracts || [];
            let cardStatements = data.cardStatements || [];
            let singleProcessor = hasSingleProcessorCode([...contracts, ...cardStatements]);
            let singleType = !!(!!contracts.length ^ !!cardStatements.length);
            
            if (singleProcessor && singleType) {
                const type = contracts.length ? BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK : BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT;
                
                LogService.info(`Navegando para Listagem de cobranças com filtro: ${type} - ${singleProcessor}`);
                
                goToListing(data, singleProcessor, type, inputType);
            } else if (!singleProcessor) {
                AlertService.warn("Cliente com Processadoras distintas!")
                .finally (() => goToProcessorFilter(data, inputType));
            } else {
               goToTypeFilter(data, inputType);
            }
        }

        $scope.processOptions = function (options, responseHandler) {
           $scope.options.length = 0;
           $scope.showWatermark = false;

           var splitedOpt = options.replace(/\(|\)/g, "");
           splitedOpt = splitedOpt.split("#");

           for (var i = 0; i < splitedOpt.length; i++) {
               var split = splitedOpt[i].split(",");

               $scope.options.push({
                   id: split[0].trim(),
                   label: split[1].trim().replace(/\"/g, ""),
                   value: split[split.length - 1],
                   handler: responseHandler,
                   type: "single-option"
               });
               $q(resolve => resolve()).then(() => {});
           }
           $scope.headerOptions = "Opções";
       };
       
       /**
        * Função executa a função com os parâmetros passados a esta.
        */
        $scope.tickOptionHandler = function(func, params) {
            $scope.options.length = 0;
            $scope.showWatermark = true;
            $scope.message = "Iniciando. Favor aguarde...";
            
            func.apply(this, params);
        };
        
       /**
        *
        */
       $scope.tickNumberInputHandler = function(inputOption) {
           var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);
           $scope.message = "Iniciando. Favor aguarde...";
           $scope.tickOptionHandler(inputOption.handler, [value]);
       };
 
       /**
        *
        */
       $scope.tickStringInputHandler = function(inputOption) {
           const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);
           $scope.message = "Iniciando. Favor aguarde...";
           $scope.tickOptionHandler(inputOption.handler, [value]);
       };
       
        /**
         *
         */
        async function goToListing(searchedData, processorCode, type, inputType) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: processorCode,
                    typeFilter: type,
                    readInputType: inputType
                }
            };
            UtilService.navigateWithoutHistory('easyorder.billingpaymentlist', transitionParams);  
        }
        
        /**
         *
         */
        async function goToProcessorFilter(searchedData, inputType) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: inputType
                }
            };
            UtilService.navigateWithoutHistory('easyorder.billingpaymentprocessorfilter', transitionParams);
        }
        
        /**
         *
         */
        async function goToTypeFilter(searchedData, inputType) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: inputType
                }
            };
            UtilService.navigateWithoutHistory('easyorder.billingpaymenttypefilter', transitionParams);
        }
        /**
         *
         */
        function hasSingleProcessorCode(data) {
            let reference = data[0].processorCode;
            
            for (var i = 0; i < data.length; i++) {
                 if (data[i].processorCode !== reference) {
                    return undefined;
                }           
            }
            return reference;
        }
            
        (() => {
            $scope.showWatermark = true;
            $scope.options = [];
            $scope.message = "";
            $scope.goingBack = false;
            
            $scope.doChipSearch();
        })(); 
    });
})();