(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.document.search', [])
    .controller('BillingPaymentDocumentSearchCtrl', function ($scope,
            $timeout,
            $state,
            $stateParams,
            $ionicLoading,
            UtilService,
            AlertService,
            CartService,
            BillingPaymentService,
            LogService) {
                
        $scope.searchCustomer = {};
        
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        
        /**
         *
         */
        $scope.validateDocument = () => {
            var doc = $scope.searchCustomer.document;
            $scope.documentInvalid = false;
        
            if (doc) {
                if (doc.length == 11 && !validaCPF(doc)) {
                    $scope.documentInvalid = true;
                    AlertService.warn("CPF inválido").finally(putFocusOnInput);
                }
            }
        };
        
        /**
         * 
         */
        $scope.doSearch = () => {
            $ionicLoading.show({
                template : 'Consultando. Por favor aguarde...',
                duration : 0
            }).then(() => { 
                $scope.searchInProgress = true;
                var req = {
                    customerDocument: $scope.searchCustomer.document,
                    customerIdentificationType: "CPF",
                    operationCode : "VENDA_VAREJO"
                };
                
                BillingPaymentService.search(req)
                .finally($ionicLoading.hide)
                .then(data => {
                    return $ionicLoading.show()
                    .then(() => $scope.searchCustomer)
                    .then(getCustomer)
                    .then(result => $scope.searchCustomer = result)
                    .then(_ => $scope.searchCustomer.name = data.customer.name)
                    .then($ionicLoading.hide)
                    .then(() => goOn(data));
                })
                .catch(error => {
                    AlertService.restError(error).then(putFocusOnInput);
                })
                .finally(() => {
                    $scope.searchInProgress = false;
                });
            });
        };
        
        async function putFocusOnInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_registrationCPF');
                var angularEl = angular.element(myEl);
                angularEl[0].focus();
            });
        }
        
        /**
         * 
         */
        function getCustomer() {
            var promise;
            
            if ($scope.searchCustomer && $scope.searchCustomer.cardNumber) {
                promise = Promise.resolve($scope.searchCustomer);
            } else {
                promise = BillingPaymentService.getCustomer($scope.searchCustomer.document)
                .catch(_ => { return {document: $scope.searchCustomer.document}; });
            }
            return promise;
        }
        
        /**
         *
         */
        function goOn(data) {
            let contracts = data.contracts || [];
            let cardStatements = data.cardStatements || [];
            let singleProcessor = hasSingleProcessorCode([...contracts, ...cardStatements]);
            let singleType = !!(!!contracts.length ^ !!cardStatements.length);
            
            if (singleProcessor && singleType) {
                const type = contracts.length ? BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK : BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT;
                
                LogService.info(`Navegando para Listagem de cobranças com filtro: ${type} - ${singleProcessor}`);
                
                goToListing(data, singleProcessor, type);
            } else if (!singleProcessor) {
                AlertService.warn("Cliente com Processadoras distintas!")
                .finally (() => goToProcessorFilter(data));
            } else {
               goToTypeFilter(data);
            }
        }
        
        /**
         *
         */
        async function goToListing(searchedData, processorCode, type) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: $scope.searchCustomer,
                    processorFilter: processorCode,
                    typeFilter: type,
                    readInputType: 'TYPED'
                }
            };
            
            if (type === BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK) {
                return UtilService.navigateWithoutHistory('easyorder.billingpaymentlist', transitionParams);
            } else {
                return UtilService.navigateWithoutHistory('easyorder.billingpaymentcardstatementlist', transitionParams);
            }
        }
        
        /**
         *
         */
        async function goToProcessorFilter(searchedData) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: $scope.searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: 'TYPED'
                }
            };
            return UtilService.navigateWithoutHistory('easyorder.billingpaymentprocessorfilter', transitionParams);
        }
        
        /**
         *
         */
        async function goToTypeFilter(searchedData) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: $scope.searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: 'TYPED'
                }
            };
            return UtilService.navigateWithoutHistory('easyorder.billingpaymenttypefilter', transitionParams);
        }
        
        /**
         *
         */
        function hasSingleProcessorCode(data) {
            let reference = data[0].processorCode;
            
            for (var i = 0; i < data.length; i++) {
                 if (data[i].processorCode !== reference) {
                    return undefined;
                }           
            }
            return reference;
        }
        
        (() => {
            $scope.validateDocument();
            
            if (CartService.cart.customer) {
                $scope.searchCustomer = CartService.cart.customer;
            }
        })(); 
    });
})();