(function () {
    'use strict';
    angular.module('easyorder.billing.payment.v1.scanned.search', [])
    .controller('BillingPaymentScannedSearchCtrl', function ($scope,
            $rootScope,
            $q,
            $state,
            $stateParams,
            $ionicLoading,
            UtilService,
            AlertService,
            BarcodeService,
            ConfigService,
            CartService,
            BillingPaymentService,
            LogService,
            TransactionSearchService,
            BluebirdCaptureService) {
                
        const CCR_SCAN_MODE = 'CCR_SCAN_MODE';
        const CREDIT_BOOK_SCAN_MODE_TC = 'CREDIT_BOOK_SCAN_MODE_TC';
        const CREDIT_BOOK_SCAN_MODE_CD = 'CREDIT_BOOK_SCAN_MODE_CD';
        const ARRANGEMENT_SCAN_MODE_TC = 'ARRANGEMENT_SCAN_MODE_TC';
        const ARRANGEMENT_SCAN_MODE_CD = 'ARRANGEMENT_SCAN_MODE_CD';
        const CARD_STATEMENT_SCAN_MODE_TC = 'CARD_STATEMENT_SCAN_MODE_TC';
        const CARD_STATEMENT_SCAN_MODE_CD = 'CARD_STATEMENT_SCAN_MODE_CD';
        
        var scanning_method = ConfigService.config.business.barcode_scanning_method;
        var scanMode;
        var dataFromSelectedContracts = [];
        
        $scope.scanningInProgress = false;
        $scope.selectedContracts = [];
        
        /**
         *
         */
        $scope.$watch('selectedContracts.length', newValue => {
            $scope.scanningInProgress = !!newValue;
        });
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                BarcodeService.cancel();
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.finishScanning = () => {
            var srchData = {
                data: dataFromSelectedContracts,
                customerIdentification: CartService.cart.customer || {},
                processorFilter: scanMode == CREDIT_BOOK_SCAN_MODE_TC 
                    || scanMode == ARRANGEMENT_SCAN_MODE_TC ? BillingPaymentService.SEARCH_TYPE_PROCESSOR_TOTVS : BillingPaymentService.SEARCH_TYPE_PROCESSOR_CONDUCTOR,
                typeFilter: scanMode == CREDIT_BOOK_SCAN_MODE_TC 
                    || scanMode == CREDIT_BOOK_SCAN_MODE_CD
                    || scanMode == ARRANGEMENT_SCAN_MODE_TC
                    || scanMode == ARRANGEMENT_SCAN_MODE_CD ? BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK : BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT,
                readInputType: 'BARCODE'
            };
            
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: srchData
            };
            
            UtilService.navigateWithoutHistory('easyorder.billingpaymentdetachedlist', transitionParams);
        };
        /**
         * 
         */
        function initBarcodeScanner() {
                if (ConfigService.config.business.enables_rfid_reading){
                    rfidConnect() 
                } else {
                    if (scanning_method !== "CAMERA") {
                        if (window.bluebirdBarcodeScanner) {
                            bluebirdBarcodeScanner.register((barcode) => {
                                LogService.info(`Código obtido via leitura de scanner: ${barcode}`);
                                
                                handleDataObtainedFromScanner(barcode)
                                .catch(error => {
                                    if (!$scope.scanningInProgress) {
                                        scanMode = undefined;
                                    }
                                    AlertService.error(error, 'Não foi possível consultar.');
                                });
                            }, (argument) => {
                                LogService.info("Não foi possível inicializar Scanner: " + argument);
                            });
                        }
                    } else {
                        BarcodeService.init((session) => {
                            BarcodeService.cancel();
                            
                            handleDataObtainedFromScanner(session[0])
                            .catch(error => {
                                if (!$scope.scanningInProgress) {
                                    scanMode = undefined;
                                }
                                AlertService.error(error, 'Não foi possível consultar.')
                                .finally(initBarcodeScanner);
                            });
                        }, { inSizeCam: ['1/136/1/130', '2/116/2/110'], outSizeCam: ['275/136/0/130', '275/116/0/110'] });
                    }
                }
        }

        async function callbackResult(data) {
            const dataInput = data.raw;
            LogService.info(`Iniciando callback sucess - SLED-Barcode: ${dataInput}`)
            
            if (!$scope.loading) {
                return await $q.resolve()
                .then(() => LogService.info(`Leitura SLED-Barcode: ${dataInput}`))
                .then(() => $scope.loading = true)
                .then(() => {
                    handleDataObtainedFromScanner(dataInput)
                    .catch(error => {
                        if (!$scope.scanningInProgress) {
                            scanMode = undefined;
                        }
                        AlertService.error(error, 'Não foi possível consultar.');
                    });
                })
                .finally(() => $scope.loading = false)
                .catch(error => {
                    if (error) {
                        return AlertService.restError(error);
                    }   
                });
            }
        }

        function callbackError(data) {
            LogService.info(`Erro na leitura SLED-Scanner: ${data.message}`);
        }
        /**
         *
         */

        function rfidConnect() {
            $scope.loading = true;
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.data = {
                lastPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultPower: ConfigService.config.business.rfid_reading_power
            };
            return $ionicLoading.show({
                template: 'Habilitando Sled. Aguarde...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.barcodeConnect())
            .then(() => BluebirdCaptureService.startReading(callbackResult, callbackError))
            .then(() => $scope.loading = false)
            .finally(() => {
                $ionicLoading.hide;
                return $q.resolve();
            })
            .catch(error => {
                if (ConfigService.config.business.enables_exchange_by_rfid_only) {
                    AlertService.alert('Sled não conectado', 'Retorne após conectar o Sled para prosseguir com a operação')
                    .then($scope.goBack);
                } else {
                    let promise;
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                    promise = AlertService.warn('Leitor não conectado.');
                    } else {
                    promise = AlertService.warn(`Não foi possível conectar ao Leitor: [${data.errorCode}]`);
                    }
                    promise.finally(() => UtilService.navigateWithoutHistory('easyorder.operations'));
                }
            });

        }

        /**
         * Carne e acordo avulsos podem ser pagos juntos desde que tenham a mesma processadora.
         */
        function handleDataObtainedFromScanner(dataRead) {
            var mode = getScanModeFor(dataRead)
            
            LogService.info(`Modo de Leitura obtido para cobrança lida: '${mode}'`)
            
            if (!mode) {
                return $q.reject('Código de Barras não suportado por esta funcionalidade.');
            } else if (scanMode && scanMode != mode && scanMode.split("_").slice(-1)[0] !== mode.split("_").slice(-1)[0]) { //Comparando a processadora.
                return $q.reject('Tipo não compatível com a leitura anterior.');
            } else if (mode == CARD_STATEMENT_SCAN_MODE_TC) {
                scanMode = mode;
                var searchRequest = getSearchRequestFrom(dataRead, mode);
                return processTCCardStatementeData(searchRequest);                                
            } else if (mode == CARD_STATEMENT_SCAN_MODE_CD) {
                scanMode = mode;
                return processCDTCardStatementeData(dataRead);
            } else {
                scanMode = mode;
                var searchRequest = getSearchRequestFrom(dataRead, mode);
                
                return $ionicLoading.show()
                .then(() => BillingPaymentService.search(searchRequest))
                .finally($ionicLoading.hide)
                .catch(error => $q.reject(error.data.message))
                .then(data => processSearchReturnData(searchRequest, data, dataRead, mode))
                .then(initBarcodeScanner);
            }
        }
        /**
         *
         */
        function getScanModeFor(dataRead) {
            var ret = undefined;
            
            if (dataRead.match(/^\d{18}$/)) {
                ret = CCR_SCAN_MODE;
            } else if (dataRead.match(/^\d{17}\$\d{20}\$\d{1,2}\$\d{8}\$\d{6}\$\d\$\d{6}\$TC$/) || dataRead.indexOf('9636513') == 0) {
                ret = CREDIT_BOOK_SCAN_MODE_TC;
            } else if (dataRead.match(/^\d{17}\$\d{20}\$\d{1,2}\$\d{8}\$\d{6}\$\d\$\d{6}\$CD$/)) {
                ret = CREDIT_BOOK_SCAN_MODE_CD;
            } else if (dataRead.match(/^11\$\d{8}\$\d{8}\$\d{8}\$\d{9}\$\d{11}\$TC$/)) {
                ret = ARRANGEMENT_SCAN_MODE_TC;
            } else if (dataRead.match(/^11\$\d{8}\$\d{8}\$\d{8}\$\d{9}\$\d{11}\$CD$/)) {
                ret = ARRANGEMENT_SCAN_MODE_CD;
            } else if (dataRead.match(/^636513\d{34}$/)) { //5636135376142840000000009734000000009734
                ret = CARD_STATEMENT_SCAN_MODE_TC;
            } else if (dataRead.match(/^999090\d{34}$/)) { //9990900000578138000000002289000000013981
                ret = CARD_STATEMENT_SCAN_MODE_CD;
            }
            return ret;
        }
        /**
         *
         */
        
        function getSearchRequestFrom(dataRead, mode) {
            var req = undefined;
            
            if (mode == CCR_SCAN_MODE) {
                req = {
                    cardNumber:                 dataRead.slice(0,16),
                    customerIdentificationType: 'CODIGO_BARRAS',
                    operationCode :             "VENDA_VAREJO"
                };
            } else if (mode == CREDIT_BOOK_SCAN_MODE_TC || mode == CREDIT_BOOK_SCAN_MODE_CD) {
                const data = dataRead.split('$');
                req = {
                    cardNumber:                 data[0].substring(1),
                    contractCode:               data[1],
                    customerIdentificationType: 'CODIGO_BARRAS',
                    operationCode :             "VENDA_VAREJO",
                };
            } else if (mode == ARRANGEMENT_SCAN_MODE_TC || mode == ARRANGEMENT_SCAN_MODE_CD) {
                const data = dataRead.split('$');
                req = {
                    contractCode:               data[1],
                    channel:                    '1',
                    customerIdentificationType: 'CODIGO_BARRAS',
                    operationCode :             'ACORDO',
                    installmentValue:           parseFloat(data[4]) / 100,
                    installmentDate:            moment(data[2], "DDMMYYYY").toDate()
                };
            } else if (mode == CARD_STATEMENT_SCAN_MODE_TC) {
                req = {
                    cardNumber:                  dataRead.substring(0, 16),
                    customerIdentificationType: 'CODIGO_BARRAS',
                    operationCode :             "VENDA_VAREJO"
                };
            }
            return req;
        }
        /**
         *
         */
        function processSearchReturnData(searchRequest, data, dataRead, mode) {
            if (mode == CCR_SCAN_MODE) {
                return processCCRScanResult(searchRequest, data);
            } else if (mode == CREDIT_BOOK_SCAN_MODE_TC || mode == CREDIT_BOOK_SCAN_MODE_CD) {
                return processCreditBookContractData(data, dataRead);
            } else if (mode == ARRANGEMENT_SCAN_MODE_TC || mode == ARRANGEMENT_SCAN_MODE_CD) {
                return processArrangementContractData(data, dataRead);                
            }
        }
        /**
         *
         */
        function processCCRScanResult(searchRequest, data) {
            LogService.info(`Processando cobranças lidas do cliente '${data.customerDocument}'...`);
            
            let searchCustomer = {};
            searchCustomer.name = data.customer.name;
            searchCustomer.document = data.customer.document;
            searchCustomer.documentType = 'CPF';
            searchCustomer.cardNumber = searchRequest.cardNumber;
            
            let contracts = data.contracts || [];
            let cardStatements = data.cardStatements || [];
            let singleProcessor = hasSingleProcessorCode([...contracts, ...cardStatements]);
            let singleType = !!(!!contracts.length ^ !!cardStatements.length);
            
            if (singleProcessor && singleType) {
                const type = contracts.length ? BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK : BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT;
                
                LogService.info(`Navegando para Listagem de cobranças com filtro: ${type} - ${singleProcessor}`);
                
                return goToListing(data, singleProcessor, type, searchCustomer);
            } else if (!singleProcessor) {
                BarcodeService.cancel();
                return AlertService.warn("Cliente com Processadoras distintas!")
                .finally (() => goToProcessorFilter(data, searchCustomer));
            } else {
               return goToTypeFilter(data, searchCustomer);
            }
        }
        /**
         *
         */
        function processCreditBookContractData(data, dataRead) {
            LogService.info(`Processando dados lidos do carnê...`);
            
            const installmentNumber = parseInt(dataRead.split('$')[2]);
            let selected = undefined;
            let dueBefore = undefined;
            let contracts = data.contracts || [];
            
            if (!(selected = contracts.find(c => c.installmentNumber === installmentNumber))) {
                return $q.reject('Pagamento já realizado!');
            } else if ((dueBefore = contracts.find(c => c.installmentNumber < installmentNumber)) && // Existe anterior ?
                    !$scope.selectedContracts.find(selected => selected.debtIdentifier == dueBefore.debtIdentifier)) { //A anterior já foi selecionada previamente?
                return $q.reject('Existem prestações em aberto anteriores à prestação bipada.');
            } else if ($scope.selectedContracts.find(c => c.debtIdentifier === selected.debtIdentifier)) {
                return $q.reject('Parcela já bipada neste cupom!');
            } else {
                var previousData = dataFromSelectedContracts.find(d => d.customer.document === data.customer.document);
                
                if (previousData) {
                    previousData.contracts.push(selected);    
                } else {
                    data.contracts = data.contracts.filter(c => c === selected);
                    dataFromSelectedContracts.push(data);
                }
                $scope.selectedContracts.push(selected);
                $rootScope.$emit("screen-total-value", getContractsTotalValue());
                return $q.resolve();  
            }
        }
        /**
         *
         */
        function processArrangementContractData(data, dataRead) {
            LogService.info(`Processando dados lidos do acordo...`);
            
            const installmentNumber = 1;
            let selected = undefined;
            let contracts = data.contracts || [];
            
            data.customer.document = dataRead.split('$')[5];
            
            if (!(selected = contracts.find(c => c.installmentNumber === installmentNumber))) {
                return $q.reject('Pagamento já realizado!');
            } else if ($scope.selectedContracts.find(c => c.debtIdentifier === selected.debtIdentifier)) {
                return $q.reject('Parcela já bipada neste cupom!');
            } else {
                var previousData = dataFromSelectedContracts.find(d => d.customer.document === data.customer.document);
                
                if (previousData) {
                    previousData.contracts.push(selected);    
                } else {
                    data.contracts = data.contracts.filter(c => c === selected);
                    dataFromSelectedContracts.push(data);
                }
                $scope.selectedContracts.push(selected);
                $rootScope.$emit("screen-total-value", getContractsTotalValue());
                return $q.resolve();  
            }
        }
        /**
         *
         */
        function processTCCardStatementeData(searchRequest) {
            return $ionicLoading.show()
            .then(() => BillingPaymentService.search(searchRequest))
            .finally($ionicLoading.hide)
            .catch(error => $q.reject(error.data.message))
            .then(data => {
                 var srchData = {
                    data: [data],
                    customerIdentification: {
                        name:           data.customer.name,
                        document:       data.customer.document,
                        documentType:   'CPF',
                        cardNumber:     searchRequest.cardNumber
                    },
                    processorFilter: BillingPaymentService.SEARCH_TYPE_PROCESSOR_TOTVS,
                    typeFilter: BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT,
                    readInputType: 'BARCODE'
                };
                $stateParams.flowHistory.flow.push($state.current.name);
                $stateParams.flowHistory.params.push($stateParams);
                
                const transitionParams = {
                    flowHistory: $stateParams.flowHistory,
                    searchData: srchData
                };
                return UtilService.navigateWithoutHistory('easyorder.billingpaymentcardstatementlist', transitionParams);
            });
        }
        /**
         *
         */
        function processCDTCardStatementeData(dataRead) {
            let cardNumber = dataRead.substring(0, 16);
            let minPaymentValue = parseFloat(dataRead.substring(16, 28)) / 100;
            let totalValue = parseFloat(dataRead.substring(28)) / 100;
            
            var srchData = {
                data: [{
                    customer: {},
                    cardStatements: [{
                        processorCode: BillingPaymentService.SEARCH_TYPE_PROCESSOR_CONDUCTOR,
                        dueDate: undefined,
                        debtIdentifier: cardNumber,
                        purchaseCardNumber: cardNumber,
                        minPaymentValue: minPaymentValue,
                        totalValue: totalValue
                    }]
                }],
                customerIdentification: {},
                processorFilter: scanMode == BillingPaymentService.SEARCH_TYPE_PROCESSOR_CONDUCTOR,
                typeFilter: BillingPaymentService.SEARCH_TYPE_CARD_STATEMENT,
                readInputType: 'BARCODE'
            };
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: srchData
            };
            return UtilService.navigateWithoutHistory('easyorder.billingpaymentcardstatementlist', transitionParams);
        }
        /**
         *
         */
        function getContractsTotalValue() {
            let selectedBillsValue = $scope.selectedContracts.reduce((acc, sel) => parseFloat((acc + sel.installmentValue).toFixed(2)), 0);
            let selectedTotalDiscount = $scope.selectedContracts.reduce((acc, sel) => parseFloat((acc + sel.discountValue).toFixed(2)), 0);
            let selectedTotalCharges = $scope.selectedContracts.reduce((acc, sel) => 
                    parseFloat((acc + (parseFloat(((sel.insuranceArrearsInterestValue || 0) + (sel.arrearsInterestValue || 0) 
                                                            + (sel.arrearsPenaltyValue || 0) + (sel.iofValue || 0) + (sel.remunerationInterestValue || 0)).toFixed(2)))).toFixed(2)), 0);
            return selectedBillsValue - selectedTotalDiscount + selectedTotalCharges;
        }
        /**
         *
         */
        async function goToListing(searchedData, processorCode, type, searchCustomer) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: processorCode,
                    typeFilter: type,
                    readInputType: 'BARCODE'
                }
            };
            if (type === BillingPaymentService.SEARCH_TYPE_CREDIT_BOOK) {
                return UtilService.navigateWithoutHistory('easyorder.billingpaymentlist', transitionParams);
            } else {
                return UtilService.navigateWithoutHistory('easyorder.billingpaymentcardstatementlist', transitionParams);
            }
        }
        
        /**
         *
         */
        async function goToProcessorFilter(searchedData, searchCustomer) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: 'BARCODE'
                }
            };
            return UtilService.navigateWithoutHistory('easyorder.billingpaymentprocessorfilter', transitionParams);
        }
        
        /**
         *
         */
        async function goToTypeFilter(searchedData, searchCustomer) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);
            
            const transitionParams = {
                flowHistory: $stateParams.flowHistory,
                searchData: {
                    data: [searchedData],
                    customerIdentification: searchCustomer,
                    processorFilter: undefined,
                    typeFilter: undefined,
                    readInputType: 'BARCODE'
                }
            };
            return UtilService.navigateWithoutHistory('easyorder.billingpaymenttypefilter', transitionParams);
        }
        /**
         *
         */
        function hasSingleProcessorCode(data) {
            let reference = data[0].processorCode;
            
            for (var i = 0; i < data.length; i++) {
                 if (data[i].processorCode !== reference) {
                    return undefined;
                }           
            }
            return reference;
        }
            
        (() => {
            $rootScope.$emit("screen-total-value", 0);
            initBarcodeScanner();
        })(); 
    });
})();