(function () {
    'use strict';
    angular.module('easyorder.controllers.cardnumbersearch', [])
        .controller('cardNumberSearchCtrl', function (
            $scope,
            $ionicLoading,
            CustomerService,
            UtilService,
            AlertService,
            LogService,
            PaymentService,
            CartService,
            $q,
            $rootScope,
            BiometricsService) {


            $scope.formData = {};
            $scope.goBack = () => {
                UtilService.navigateWithoutHistory('easyorder.paymentmethods');
            };

            $scope.validateCPF = (cpf) => {
                return validaCPF(cpf);
            }

            function navigateToPaymentPlans() {
                const params = {
                    flowHistory: {
                        flow: ['easyorder.paymentmethods'],
                        params: []
                    },
                    paymentState: 'easyorder.tefpayment',
                    disableSorting: true
                };

                UtilService.navigateWithoutHistory('easyorder.paymentplans', params);
            };

            $scope.search = () => {
                const document = $scope.formData.document;
                const currentPayment = PaymentService.currentPayment();
                const provider = currentPayment.provider;
                LogService.info(`Iniciando busca de cartões do '${provider}' pelo CPF '${document}'.`);
                $ionicLoading.show({
                    template: "Por favor, aguarde...",
                    duration: 0
                }).then(() => {
                    if (isEmpty(document)) {
                        return $q.reject('CPF vazio ou inválido!');
                    }
                    return CustomerService.searchByType(document, 'CPF', provider);
                }).then((result) => {
                    if (result.length === 0) {
                        $ionicLoading.hide();
                        return $q.reject('Usuário não encontrado!');
                    } else {
                        const customer = result[0];
                        return $q.resolve(customer);
                    }
                }).then((customer) => {
                    const paymentMethod = PaymentService.currentPayment().paymentMethod;
                    if (BiometricsService.isBiometricPaymentMethod(paymentMethod)) {
                        if (!BiometricsService.hasCustomerBiometricsInfo(customer)) {
                            // se não veio informações acerca da biometria mas ela está habilitada por parâmetro, exibe um erro e não segue o fluxo!
                            LogService.info(`Biometria para a forma '${paymentMethod}' habilitada mas não foram retornados dados da biometria para o cliente!`);
                            return $q.reject(`Erro de comunicação com a validação da biometria, não será possível utilizar esta forma de pagamento!`);
                        } else {
                            return $q.resolve(customer);
                        }
                    } else {
                        return $q.resolve(customer);
                    }
                }).then((customer) => {
                    const cardNumber = customer.cardNumber;
                    const cardExpirationDate = customer.cardExpirationDate;
                    const firstPurchase = customer.firstPurchase === true;
                    if (cardNumber) {
                        // hoje os atributos do pagamento estão sendo injetados no pagamento para serem enviados no REST planos e eventos
                        currentPayment.cardNumber = cardNumber;
                        currentPayment.cardExpirationDate = cardExpirationDate;
                        currentPayment.customer = customer;

                        // pela regra da C&A, primeira compra só é usada se não tiver pagamentos já efetuados na venda
                        const payments = PaymentService.payments();
                        const paymentAlreadyDone = payments ? payments.length > 0 : false;
                        currentPayment.firstPurchase = paymentAlreadyDone ? false : firstPurchase;

                        if (!$rootScope.promotionEngine || !firstPurchase || paymentAlreadyDone) {
                            const message = !$rootScope.promotionEngine ? `Motor de promoções desabilitado, promoções não serão aplicadas para a primeira compra.` : 
                                !firstPurchase ? `Não é a primeira compra com o cartão para o cliente de CPF '${document}'.` : `Já existem pagamentos, não serão aplicados descontos de primeira compra C&A Pay!`;
                                
                            LogService.warn(message);
                            $ionicLoading.hide();
                            navigateToPaymentPlans();
                        } else {
                            LogService.info(`Primeira compra com o cartão para o cliente de CPF '${document}'.`);
                            const currentTotalValue = new Number(CartService.cart.totalValue);
                            CartService.updateDiscountCart({ id: currentPayment.paymentMethod }, currentPayment.paymentValue, PaymentService.payments(), firstPurchase)
                                .then(() => {
                                    const newTotalValue = new Number(CartService.cart.totalValue);
                                    LogService.info(`Busca pelas promoções de cliente C&A Pay com primeira compra efetuada com sucesso. Valor da compra antes: ${currentTotalValue}. Depois: ${newTotalValue}.`);
                                    if (currentPayment.paymentValue > newTotalValue) {
                                        LogService.info(`Pagamento solicitado é maior que o valor atual da compra, alterando o valor para ${newTotalValue}.`) ;
                                        currentPayment.paymentValue = newTotalValue;
                                    }
                            }).catch((err) => {
                                LogService.error('Busca pelas promoções de cliente C&A Pay com primeira compra com erro');
                                LogService.error(err);
                            }).finally(() => {
                                $ionicLoading.hide();
                                navigateToPaymentPlans();
                            });
                        }
                    } else {
                        $ionicLoading.hide();
                        return $q.reject('Cartão não retornado!');
                    }
                }).catch((error) => {
                    $ionicLoading.hide();
                    AlertService.restError(error, error);
                });
            };

            function isEmpty(str) {
                return (str == undefined || str == '');
            }

        });
})();
