(function() {
	'use strict';

	angular.module('easyorder.controllers.cart', [
	    'ln.currency',
		'easyorder.services.cart',
		'easyorder.services.order',
		'easyorder.services.util',
		'easyorder.services.config',
		'easyorder.services.user',
		'easyorder.services.loyalty',
		'easyorder.services.alert',
		'easyorder.services.customer',
		'easyorder.services.sms',
		'easyorder.services.product',
		'easyorder.services.log',
        'easyorder.controllers.home'
    ])

	.controller('CartCtrl', function($rootScope,
									 $scope,
                                     $ionicModal,
                                     $ionicPopup,
                                     $ionicLoading,
                                     $ionicNavBarDelegate,
									 $ionicListDelegate,
									 $state,
									 $q,
									 $timeout,
									 CartService,
									 CustomerService,
									 OrderService,
                                     UtilService,
									 ConfigService,
									 UserService,
									 AlertService,
									 NFCeService,
									 SmsService,
									 FreightService,
									 ProductService,
									 DiscountService,
									 ServiceSaleService,
									 LogService,
									 CovenantService,
									 MessagingService,
									 BarcodeService,
									 OrderCheckingModal,
									 FiscalCustomerModal,
									 PaymentService) {

		$scope.itemsOpts = createItemsOpts(CartService.cart.fiscalItems);
		$scope.items = CartService.cart.fiscalItems;
		$scope.$watch('items', function() {
			updateItemsOpts($scope.items, $scope.itemsOpts);
		}, true);

	    $scope.cart = CartService.cart;
		$scope.customerValidationWarnMsg = undefined;
		$scope.newOrderActionText = ($rootScope.externallyLoadedApp ? "Sair" : $scope.NEW_ORDER);
		$scope.manualDiscount = ConfigService.config.business.manual_discount;
		$scope.itemSeller = ConfigService.config.business.seller_identification_mode === 'item';
		$scope.currentItem = null;
		$scope.mandatoryParameterFromServer = ConfigService.config.business.customer_mandatory_fields;
		$scope.checkPermissionManualDiscount = ConfigService.config.business.check_permission_manual_discount;
		$scope.productQueryShortcutOnTheCartScreen = ConfigService.config.business.product_query_shortcut_on_the_cart_screen_enabled
														&& (CartService.cart.orderStatus !== 'READY_TO_PACK');
		$scope.productRemoveShortcutOnTheCartScreen = ConfigService.config.business.product_remove_shortcut_on_the_cart_screen_enabled;
        $scope.isVouchersBalanceEnabled = ConfigService.config.business.promo_voucher_balance_enabled;
		var searchTimeoutPromise;
		$scope.alertCoupons;
		$scope.tabSelected = "CART";
        $scope.useRFID = ConfigService.config.business.enables_rfid_reading;
        const [singularLabel, _] = ConfigService.config.business.promo_voucher_label;
        $scope.voucherLabel = (ConfigService.config.business.giftcard_service_provider === 'PROMO' ? singularLabel : 'Voucher');

        $scope.showroomingLabel = ConfigService.config.business.showrooming_label;

		const noFiscalCustomerCheckActions = [
			"ORDER_FULFILMENT",
			"ORDER_PACKING",
			"ORDER_PACKING_V2",
		];

		$scope.hasShowroomingItems = function (){
			return CartService.hasShowrooming();
		};

		$scope.isPickupOrder = function (){
			return CartService.isPickupOrder();
		};

	    if ($scope.cart.customer && $scope.cart.customer.document) {
			if ($scope.cart.customer.documentType == "CPF") {
				$scope.documentoFormatado = applyCpfMask($scope.cart.customer.document);
			} else if ($scope.cart.customer.documentType == "CNPJ") {
				$scope.documentoFormatado = applyCnpjMask($scope.cart.customer.document);
			}else {
				$scope.documentoFormatado = $scope.cart.customer.document;
			}
		};

		$scope.orderService = OrderService;

		var undoServiceAuthorization = function (index) {
		    var error = function(error) {
		        LogService.error('Erro no desfazimento do item: ', JSON.stringify(error));
		    };

		    var success = function(resp) {
				LogService.info('Desfazimento do item efetuado com sucesso!');
		    };
		    var companyNumber = ConfigService.config.companyNumber;
		    var storeNumber = ConfigService.config.storeNumber;
		    var terminalNumber = ConfigService.config.terminalNumber;
		    var nsu = CartService.cart.nsu;
		    var serviceItem = CartService.cart.serviceItems[index];

		    LogService.info('Desfazendo autorização do item de serviço de índice ' + index);
            NFCeService.undoServiceAuthorization(companyNumber,storeNumber,terminalNumber, nsu, serviceItem, success, error);
		}

		/**
		 *
		 */
		$scope.showAlert = function(erro) {
   			var alertPopup = $ionicPopup.alert({
			title: 'Erro!',
			template: erro,
			buttons:[ { text: 'Ok', type: 'button button-theme' }]
		  });
		};

		/**
		 *
		 */
		$scope.insertCoupon = function(){
		    var removeButtons = [
                {text: 'Fechar', type: 'button button-stable', onTap: function(e) { return false; }},
                {text: 'Remover', type: 'button button-theme',
                    onTap: function(e) {
                        $scope.couponValid = false;
                        delete $scope.data.coupon;
                        let discounts = CartService.cart.discounts;

                        for (var i = 0; i < discounts.length; i++) {
                            if (discounts[i].source === 'COUPON') {
                                discounts.splice(i, 1);
                            }
                        }
                        CartService.updateCartTotalValue();
                    }
                }
            ];
            var searchButtons = [
                {text: 'Cancelar', type: 'button-stable', onTap: function(e) { return false; }},
                {text: 'Pesquisar', type: 'button-theme',
                    onTap: function(e) {
                        $scope.searchCoupon().then(function(discountObj) {
                            if (discountObj) {
                                $scope.couponValid = true;
                                CartService.removeDiscountBySource(discountObj.source);
								CartService.addDiscount(discountObj);
								CartService.cart.customer.coupon = {};
								CartService.cart.customer.coupon.couponCode = discountObj.indentifierCode;
								CartService.cart.customer.coupon.value = discountObj.value;
                            }
                            $scope.myPopup.close();
                        }, function (err) {
                            $scope.couponValid = false;
                            delete $scope.data.coupon;

							$scope.myPopup.close();
							if (err.data){
								$scope.showAlert(err.data.message);
							}else {
								$scope.showAlert(err);
							}

                        }).finally (function() {
                            CartService.updateCartTotalValue();
                        })
                    }
                }
            ];

            let buttonsArr = $scope.couponValid ? removeButtons : searchButtons;

			$scope.myPopup = $ionicPopup.show({
				title: 'Cupom',
				templateUrl: './partials/insertCoupon.html',
				subTitle: '',
				scope: $scope,
				buttons: buttonsArr
			});
		};

		/**
		 *
		 */
		$scope.searchCoupon = function(){
		    if (!!$scope.data.coupon && $scope.data.coupon.length) {
    			$scope.data.loading = true;

    			var params = {};
    			params.cpf = CartService.cart.customer.document;
				params.couponCode = $scope.data.coupon;
				params.sessionData = CartService.getSessionData();
    			params.couponCode = $scope.data.coupon;

    			return CustomerService.searchCoupon(params).then(function(response) {
    			    $scope.data.loading = false;

    				if (response.result && response.result.length > 0) {
						let timeDiff = new Date(response.result[0].end).getTime() - new Date().getTime();
						let days =  Math.ceil(timeDiff / (1000 * 3600 * 24));
						if (days > 0) {
							LogService.info('service.cart.discounts',CartService.cart.discounts)
							LogService.info('response.result',response.result[0].value);

							let discountObj = {
								value:            response.result[0].value,
								indentifierCode:  response.result[0].couponCode,
								type:             response.result[0].valueType,
								source:           "COUPON",
								factor:           undefined
							};
							return discountObj;
						} else {
							return $q.reject("Cupom Expirado");
						}
    				} else  if (response.coupons && response.coupons.length > 0) {
						response.coupons.forEach( (coupon) => {
							CartService.addCoupon(coupon);
						});
						$scope.data.coupon = "";
						CartService.undoCampaignDiscounts();
						CartService.applyLoyaltyDiscount()
							.finally(() =>	CartService.processCampaigns())
							.finally(() => CartService.applyEmployeeDiscount());
						return null;
					}else{
    				    return $q.reject("Cupom não encontrado");
    				}
    			}, function(err) {
    			    $scope.data.loading = false;
    			    return $q.reject(err);
    			});
		    }
		};

		$scope.removeItem = function(item, index) {
            $ionicPopup.show({
                template: 'Deseja excluir o item selecionado do carrinho?',
                title: 'Confirmação',
                scope: $scope,
                buttons: [
                    {   text: 'Não',
                        onTap: function(e) {
                            $ionicListDelegate.closeOptionButtons();
                        }
                    }, {
                        text: '<b>Sim</b>',
                        type: 'button-theme',
                        onTap: function(event) {
							if (ConfigService.config.business.enable_reserve_products === true && item.itemUuid && item.itemUuid.value) {
								const cancelParams = {
									storeNumber : ConfigService.config.storeNumber,
									orderNumber: CartService.sessionId,
									terminalNumber: ConfigService.config.terminalNumber,
									operatorName: UserService.currentUser.name,
									products: [
											{
												productCode: item.productCode,
												uuid: item.itemUuid.value,
												quantity: 1
											}
										]
								};
								return $ionicLoading.show({
					                template : 'Cancelando reserva no estoque...',
					                duration : 0
					            }).then(function() {
    								ProductService.cancelSale(cancelParams).then(result => {
    									// TODO: EXTERNAR ESSA FUNÇÃO DESSE IF E ELSE...iguais...
    									LogService.info('CANCELAMENTO DA RESERVA: ', result);
    									if (CartService.getCurrentOperation() === 'SERVICE') {
    										// Caso sejam serviços, é necessário disparar a mensagem para cancelamento no LINX-WS.
    										// Mensagem é assíncrona e erros, ignorados.
    										undoServiceAuthorization(index);
    									}
    									CartService.undoCampaignDiscounts();
    									CartService.removeCustomerRelatedDiscounts();
										CartService.removeItemsByItemGroup(index);
										CartService.applyLoyaltyDiscount()
											.finally(() =>	CartService.processCampaigns())
											.finally(() => CartService.applyEmployeeDiscount());

    									if (!CartService.hasItems()) {
											UtilService.navigateToMainScreen();
    									}
    								}, (error) => {
    								    $ionicLoading.hide();
    									AlertService.restError(error, 'Erro ao cancelar a reserva!');
    								}).finally (() => {
    								    $ionicLoading.hide();
    								});
					            });
							} else {
								if (CartService.getCurrentOperation() === 'SERVICE') {
									// Caso sejam serviços, é necessário disparar a mensagem para cancelamento no LINX-WS.
									// Mensagem é assíncrona e erros, ignorados.
									undoServiceAuthorization(index);
								}
								CartService.undoCampaignDiscounts();
								CartService.removeCustomerRelatedDiscounts();
								CartService.removeItemsByItemGroup(index);
								CartService.applyLoyaltyDiscount()
									.finally(() =>	CartService.processCampaigns())
									.finally(() => CartService.applyEmployeeDiscount());
							}

							if (!CartService.hasItems()) {
							    UtilService.navigateToMainScreen();
							}
                        }
                    }
                ]
            });
		};

		$scope.additionalServices = function(item) {
			$scope.additionalServiceItems = item.nonFiscalItems;
            $ionicPopup.show({
				title: 'Serviços',
                templateUrl: './partials/productServicesDetails.html',
                scope: $scope,
                buttons: [
					{   text: 'Ok',
						type: 'button-theme',
                        onTap: function(e) {
                            $ionicListDelegate.closeOptionButtons();
                        }
                    }
                ]
            });
		};

		$scope.isNumber = function isNumber(n) {
		    return !isNaN(parseFloat(n)) && isFinite(n);
		};

		$scope.showPriceChangeDialog = function(item) {
			$ionicPopup.show({
				templateUrl: 'views/pricechange.html',
				title: 'Alterar Preço',
				scope: $scope,
				buttons: [
					{   text: 'Cancelar',
					    onTap: function(e) {
					        $ionicListDelegate.closeOptionButtons();
					    }
					}, {
						text: '<b>Salvar</b>',
						type: 'button-theme',
						onTap: function(e) {
							var error = false;
							var errorMessage = "";

							if ($scope.reasons && $scope.reasons.length == 1) {
								$scope.data.reason = $scope.reasons[0];
							}
							if ($scope.data.newPrice == null) {
								error = true;
								errorMessage = "Informe o novo valor do item!"
							} else if (!$scope.isNumber($scope.data.newPrice)) {
								error = true;
								errorMessage = "Valor não numérico!"
							} else if ($scope.data.newPrice <= 0) {
								error = true;
								errorMessage = "Valor inválido!"
							} else if ($scope.data.reason == null || $scope.data.reason.trim() == "") {
								error = true;
								errorMessage = "Selecione um motivo para a alteração do preço!";
							} else if ($scope.data.newPrice > $scope.data.maxPrice) {
							    error = true;
                                errorMessage = "Novo preço não pode ser maior do que o valor original do item.";
							}

							if (error) {
								AlertService.warn(errorMessage);
								e.preventDefault();
								return;
							}
							$ionicListDelegate.closeOptionButtons();

							var permissions = [];
							permissions[PRICE_CHANGE_PERMISSION] = $scope.requestauth.user;

							var priceOverride = ConfigService.config.business.price_override === true;
							if (!priceOverride) {
								// se não usar sobreposição como novo preço, não precisa buscar impostos
								CartService.updateItemPrice(item, $scope.data.newPrice, $scope.data.reason, permissions);
							} else {
								// buscar as taxas de acordo com novo valor do produto, se sobrescrever preço original
								$ionicLoading.show({
									template: 'Por favor, aguarde...',
									duration: 0
								}).then(function () {
									ProductService.searchTaxes({ code: item.productCode, storeNumber: ConfigService.config.storeNumber, codeType: 'INTERNAL', baseValue: $scope.data.newPrice }).then((result) => {
										$ionicLoading.hide();
										if (result && result.length > 1) {
											CartService.updateItemPrice(item, $scope.data.newPrice, $scope.data.reason, permissions, result);
										} else {
											AlertService.restError(error, 'Lista de impostos para o novo valor não retornada!');
										}
									}, (error) => {
										$ionicLoading.hide();
										AlertService.restError(error, 'Não foi possível obter impostos para o novo valor!');
									});
								});
							}


						}
					}
				]
			});
		};

		/**
		 *
		 */
		$scope.changePrice = function(item) {
			$scope.data = {
				currentPrice: item.unitValue,
				newPrice: null,
				maxPrice: (ConfigService.config.business.price_change_behavior === 'CONSTRAINED_PRICE' ? item.unitValue : Number.MAX_VALUE)
			};

			$scope.requestauth = {};

			// Verifica permissões
			var permission = UserService.checkPermission(PRICE_CHANGE_PERMISSION);
			LogService.info("Permissão: ", permission);

			if (!permission) {
				var authPopup = $ionicPopup.show({
					templateUrl: 'views/requestauth.html',
					scope: $scope,
					buttons: [
						{ text: 'Cancelar' },
						{
							text: '<b>Autorizar</b>',
							type: 'button-theme',
							onTap: function(event) {
								var sucesso = function(result) {
									authPopup.close();

									if (result.authorized) {
										$scope.showPriceChangeDialog(item);
									} else {
										AlertService.error('Autorização negada!')
									}
								};

								var erro = function(error) {
									LogService.error(error);
									AlertService.restError(error, "Erro ao consultar autorização!");
								};

								if (!$scope.requestauth.user) {
									AlertService.error("Usuário inválido!");
								} else if (!$scope.requestauth.password) {
									AlertService.error("Senha inválida!");
								} else {
									UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [PRICE_CHANGE_PERMISSION], sucesso, erro);
								}
								event.preventDefault();
							}
						}
					]
				});
			} else {
			    $scope.requestauth.user = UserService.currentUser.username;
				$scope.showPriceChangeDialog(item);
			}
		};

		$scope.createOrder = function() {
			var customer = $scope.cart.customer;
			const isCustomerNotMandatory = saleIdentificationType() === 'NOT_MANDATORY';

            if ((customer && customer.document && customer.document.length > 0 && customer.name && customer.name.length > 0) ||
                isCustomerNotMandatory) {

                validateMandatoryIdentification()
                .then(checkFiscalCustomer)
                .then(() => {$ionicLoading.show({template: 'Gerando pedido...', duration: 0})})
                .then(function() {
					var successFn = function(order) {
						$ionicLoading.hide();

						AlertService.info("Pedido: " + order.orderNumber + " gerado com sucesso!").then(function() {
							CartService.initCart();
							UtilService.navigateToMainScreen();
						});
					};

					var errorFn = function(error) {
						$ionicLoading.hide();
						AlertService.restError(error, 'Erro gerando pedido!');
					};

					var inventoryInfo;

                    if (ConfigService.config.business.enable_reserve_products && CartService.cart.fiscalItems.filter(p => (p.itemUuid)).length > 0) {
                        inventoryInfo = {
                            storeNumber : CartService.cart.storeNumber,
                            terminalNumber: CartService.cart.terminalNumber,
                            orderNumber: CartService.sessionId
						};
					}

					// Clona e converte o carrinho no order
					var order = OrderService.cartToOrderConverter(CartService.cart);

					if (order) {
						order.status = 'CHECKOUT';
						var i;

						for (i = 0; i < order.fiscalItems.length; i++) {
							order.fiscalItems[i].gift = true;
							var fiscalItem = order.fiscalItems[i];

							if (!fiscalItem.sequenceId) {
								fiscalItem.sequenceId = i + 1;
							}
							fiscalItem.deliveryOption = $rootScope.createDANFE && !ConfigService.config.business.ignore_mid_settings ? 'SOLD_IN_STORE' : 'PICK_UP_FROM_WAREHOUSE';
							fiscalItem.totalValue = Math.round(fiscalItem.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
						}
						order.totalValue =  Math.round(order.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.

						// cria a cotação de frete escolhida
						if (inventoryInfo) {
							OrderService.confirmInventoryAndCreate(order, inventoryInfo, successFn, errorFn);
						} else{
							OrderService.create(order, successFn, errorFn);
						}
					} else {
						$ionicLoading.hide();
						AlertService.error('Não foi possível converter objeto de requisição do pedido!');
					}
				}).catch(error => {
                    if (error instanceof CustomerNotIdentifiedError) {
                        AlertService.warn(error.message).then(() => {
                            UtilService.navigateWithoutHistory('easyorder.customermenu');
                        });
                    } else {
                        AlertService.restError(error, "Falha ao criar pedido!");
                    }
                });
            } else {
                AlertService.error("Para gerar um pedido é necessário identificar o cliente!");
            }

		};

		/**
		 * Valida as informações do Cliente. Retorna mensagens de aviso para o usuário ou
		 *  <code>undefined</code> se as informações estão válidas.
		 */
		$scope.customerValidationWarn = function() {
		    $scope.customerValidationWarnMsg = CartService.customerValidationWarn();

		    return $scope.customerValidationWarnMsg;
        }

        $scope.isIdentified = () => {
            return CustomerService.isIdentified($scope.cart.customer);
        }

        $scope.isFiscalIdentified = () => {
            return CustomerService.isFiscalIdentified($scope.cart.customer);
		}

		$scope.showDiscounts = (item) => {
			$scope.requestauth = {};
			if ($scope.checkPermissionManualDiscount){

				var permission = UserService.checkPermission(ITEM_DISCOUNT);
				LogService.info("Permissão: ", permission);

				if (!permission) {
					var authPopup = $ionicPopup.show({
						templateUrl: 'views/requestauth.html',
						scope: $scope,
						buttons: [
							{ text: 'Cancelar' },
							{
								text: '<b>Autorizar</b>',
								type: 'button-theme',
								onTap: function(event) {
									var sucesso = function(result) {
										authPopup.close();

										if (result.authorized) {
											$scope.loadDiscounts(item);
										} else {
											AlertService.error('Autorização negada!')
										}
									};

									var erro = function(error) {
										LogService.error(error);
										AlertService.restError(error, "Erro ao consultar autorização!");
									};

									if (!$scope.requestauth.user) {
										AlertService.error("Usuário inválido!");
									} else if (!$scope.requestauth.password) {
										AlertService.error("Senha inválida!");
									} else {
										UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [ITEM_DISCOUNT], sucesso, erro);
									}
									event.preventDefault();
								}
							}
						]
					});

				} else {
					$scope.requestauth.user = UserService.currentUser.username;
					$scope.loadDiscounts(item);
				}
			}else{
				$scope.requestauth.user = UserService.currentUser.username;
				$scope.loadDiscounts(item);
			}
		};

		$scope.loadDiscounts = (item) => {
			$ionicLoading.show({
				template: 'Buscando Descontos...',
				duration: 0
			}).then(function() {
				if (item){
					$scope.discountInItem(item);
				}else {
					$scope.discountInItem(item, null);
				}
			})
		}
		/**
		 *
		 */
		$scope.discountInItem = (item) => {

			var discountParams = {
				productCode: item.productCode,
				employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false,
				ownBrand: item.ownBrand,
				companyNumber: ConfigService.config.companyNumber,
				marketingCode: item.marketingCode,
				promotionalPrice: item.promotionalPrice,
				automatic: false,
				unitValue: item.unitValue,
				storeNumber : ConfigService.config.storeNumber,
				customer: CartService.cart.customer ? CartService.cart.customer : null
			};

			DiscountService.searchDiscountItem(discountParams).then((result) => {
				$ionicLoading.hide();
				if (result.discountRuleTypes){
					if (result.discountRuleTypes.length == 0){
						$scope.showAlert('O produto não possui descontos disponíveis');
					}else {
						UtilService.navigateWithoutHistory('easyorder.discountstypes', {"item": item, "result": result});
					}
				}
			}).catch(err => {
				$ionicLoading.hide();
				LogService.error(err);
			});
		};

		/**
		 *
		 */
		$scope.discountInFiscalCoupon = () => {

			var discountParams = {
				companyNumber: ConfigService.config.companyNumber,
				storeNumber : ConfigService.config.storeNumber,
				automatic: true,
				employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false, //TODO
				fiscalItems: DiscountService.prepareFiscalItems(CartService.cart.fiscalItems)
			};

			if (CartService.cart.discounts.length > 0){
				UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});
			}else {
				DiscountService.searchDiscountFiscalCoupon(discountParams).then((result) => {
					$ionicLoading.hide();
					if (!result){
						UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});
					}else {
						UtilService.navigateWithoutHistory('easyorder.discountstypes', {"cart": true, "automatic":true, "result": result});
					}
				}).catch(err => {
					$ionicLoading.hide();
					UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});
					LogService.info(err);
				});
			}

		};

		/**
		 *
		 */
		$scope.isCartValid = function() {
			return CartService.isCartValid();
		};

		$ionicModal.fromTemplateUrl('views/orderconfirmation.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.modal = modal;
		});

		$scope.openModal = function(order, infoText, warningText) {
			$scope.confirmationInfo = {
				orderNumber: order.orderNumber,
				storeNumber: order.storeNumber,
				status: order.status,
				infoText: infoText,
				warningText: warningText
			};

			$ionicNavBarDelegate.showBackButton(false);
			$scope.modal.show();
		};

		$scope.closeModal = function() {
			$scope.modal.hide();
		};

		$scope.$on('$destroy', function() {
			$scope.modal.remove();
		});

		$scope.storeMapAddrress = function(currentStore) {
		    var ret = 'https://www.google.com/maps/embed/v1/place?key=AIzaSyD241usOdsnkPzXCoIMvyc53ieJ7S90AiI&';
		    ret += 'language=pt&';
		    ret += 'region=BR&';

            if (currentStore && currentStore.address) {
                var address = currentStore.address;

                var addSep = address.streetAddress.indexOf(',');
                addSep =  addSep > 0 ? addSep : address.streetAddress.length;

                ret += 'zoom=15&';
                ret += 'q=' + (address.streetAddress ? address.streetAddress.substring(0, addSep).replace(/\s+/g, '+') : '')
                    		+ (address.streetNumber ? ',' + address.streetNumber.replace(/\s+/g, '+') : '')
                        + (address.neighborhood ? ',' + address.neighborhood.replace(/\s+/g, '+') : '')
                        + (address.postalCode ? ',' + address.postalCode.replace(/\D/g, '').replace(/\s+/g, '+') : '');
                        + (address.city ? ',' + address.city.replace(/\s+/g, '+') : '')
                        + (address.state ? ',' + address.state.replace(/\s+/g, '+') : '');
            } else {
                ret += 'zoom=3&';
                ret += 'q=Brasil';
            }
            return $sce.trustAsResourceUrl(ret);
		};

		/**
         *
         */
		var updateOrderAndSendWarnMessage = function(order, newStatus) {
			$ionicLoading.show({
				template: 'Atualizando status do pedido...',
				duration: 0
			}).then(function() {
				var successFn = function() {
					$ionicLoading.hide();
					var order = CartService.cart;
					var numberPromise;

					if (order && order.customer && order.customer.address && order.customer.address.phone) {
						numberPromise = $q.resolve(order.customer.address.phone);
					} else {
						numberPromise = AlertService.warn("Cliente não possui telefone no cadastro.").then(() => {
							return askMessageNumberConfirmation();
						});
					}

					numberPromise.then(msgNumber => {
						processCustomerWarnMessage(order, msgNumber).then(() => {
							var text = getOrderConfirmationText(order) + " Mensagem enviada para o cliente!";
							$scope.openModal(order, text);
						}).catch(() => {
							var text = getOrderConfirmationText(order);
							$scope.openModal(order, text, "Mensagem não foi enviada para o cliente!");
						});
					}).catch(() => {
						var text = getOrderConfirmationText(order);
						$scope.openModal(order, text, "Mensagem não foi enviada para o cliente!");
					});
				};
				var errorFn = function(error) {
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro atualizando pedido!');
				};
				OrderService.statusUpdate(order.companyNumber, order.storeNumber, order.orderNumber, newStatus, successFn, errorFn);
			});
		};

		/**
         *
         */
		var processCustomerWarnMessage = (order, msgNumber) => {
			var doPromise;

			if ($scope.orderReadySmsInfo) {
				doPromise = sendSms(order, msgNumber);
			} else {
				doPromise = sendWhatsapp(order, msgNumber);
			}
			return doPromise.catch(() => {
				return askMessageNumberConfirmation(msgNumber).then(newNumber => {
					return processCustomerWarnMessage(order, newNumber);
				});
			});
		}

		/**
         *
         */
		var askMessageNumberConfirmation = function(smsNumber) {
			return AlertService.prompt("Mensagem para o cliente", "(00) 00000-0000", "tel", "Telefone do cliente", smsNumber).then(resultSmsNumber => {
				if (resultSmsNumber != null) {
					return $q.resolve(resultSmsNumber);
				} else {
					return $q.reject();
				}
			});
		};

		/**
         *
         */
		var sendSms = (order, smsNumber) => {
			var defer = $q.defer();

			if (SmsService.isSmsNumberValid(smsNumber)) {
				$ionicLoading.show({
					template: "Enviando SMS para '" + smsNumber + "'...",
					duration: 0
				}).then(() => {
					var message = $scope.orderReadySmsInfo.message;
					var id = order.companyNumber + '-' + order.terminalNumber + '-' + new Date().getTime();
					var finalSmsNumber = SmsService.getFinalSmsNumber(smsNumber);

					LogService.info("Enviando SMS para o número: '" + finalSmsNumber + "'...");

					SmsService.sendSms($scope.orderReadySmsInfo.sender_id, finalSmsNumber, message, id).then(smsSendResponse => {
						if (SmsService.isSmsSent(smsSendResponse)) {
							defer.resolve();
						} else {
							var errorMessage = SmsService.getSmsResponseMessage(smsSendResponse);
							AlertService.error("Não foi possível enviar mensagem para o número '" + smsNumber + "'. Resposta: (" + errorMessage + ").")
								.then(defer.reject);
						}
					}).catch(error => {
						AlertService.restError(error, "Não foi possível enviar mensagem para o número '" + smsNumber + "'!")
							.then(defer.reject);
					}).finally($ionicLoading.hide);
				});
			} else {
				AlertService.warn("Telefone inválido: " + smsNumber).then(defer.reject);
			}
			return defer.promise;
		};

		/**
		 *
		 */
		var sendWhatsapp = async (order, number) => {
			if (MessagingService.isPhoneNumberValid(number)) {
				return $ionicLoading.show({
					template: "Enviando mensagem para '" + number + "'...",
					duration: 0
				}).then(() => {
					LogService.info("Enviando whatsapp para o número: '" + number + "'...");

					return MessagingService.sendWhatsApp(undefined, number,`Olá, ${order.customer.name}. Seu Pedido '${order.orderNumber}' no valor de R$${order.totalValue} foi processado pela loja!`);
				}).catch(error => {
					return AlertService.restError(error, "Não foi possível enviar mensagem para o número '" + number + "'!").then($q.reject);
				}).finally($ionicLoading.hide);
			} else {
				return AlertService.warn("Telefone inválido: " + number).then($q.reject);
			}
		};

		/**
		 *
		 */
		$scope.placeOrder = function(status) {
			$ionicLoading.show({
				template: 'Finalizando pedido...',
				duration: 0
			}).then(function() {
				var successFn = function(order) {
					$ionicLoading.hide();
					order.status = status;
					var text = getOrderConfirmationText(order);
					$scope.openModal(order, text);
				};

				var errorFn = function(error) {
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro finalizando pedido!');
				};
				var i;

				CartService.cart.customer.document = removeMask(CartService.cart.customer.document);

				for (i = 0; i < CartService.cart.fiscalItems.length; i++) {
					var fiscalItem = CartService.cart.fiscalItems[i];

					if (!fiscalItem.sequenceId) {
						fiscalItem.sequenceId = i + 1;
					}
					fiscalItem.totalValue = Math.round(fiscalItem.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
				}
				CartService.cart.totalValue =  Math.round(CartService.cart.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.

				// O objeto de venda / pedido é representado pelo carrinho hoje. É preciso corrigir essa distorção.
				var order = OrderService.cartToOrderConverter(CartService.cart);
				order.status = status;

				if (order) {
					if (!CartService.cart.orderNumber) {
						OrderService.create(order, successFn, errorFn);
					} else {
						OrderService.update(order, successFn, errorFn);
					}
				} else {
					$ionicLoading.hide();
					AlertService.error('Não foi possível converter objeto de requisição do pedido!');
				}
			});
		};

		$scope.validateDocument = function() {
			if ($scope.document.type == 'CNPJ') {
				$scope.documentForm.invalid = !validaCNPJ($scope.document.value);
			} else if ($scope.document.type == 'CPF') {
				$scope.documentForm.invalid = !validaCPF($scope.document.value);
			}
		}

		$scope.reuseFiscalCustomerPopup = function() {
			if (ConfigService.config.business.use_customer_as_fiscal_customer
					|| !CartService.cart.fiscalCustomer
					|| !isFiscalCustomerIncomplete(CartService.cart._orderFiscalCustomer)) {
				return $q.resolve(true);
			}

			const fiscalCustomer = CartService.cart.fiscalCustomer;

		    return $ionicPopup.show({
                title: `Cliente Fiscal ${fiscalCustomer.documentType} - ${fiscalCustomer.document} informado, deseja reutiliza-lo?`,
                scope: $scope,
				buttons: [
					{   text: 'Não',
					    onTap: function(e) {
							return false;
					    }
					}, {
						text: '<b>Sim</b>',
						type: 'button-theme',
						onTap: function(e) {
							return true;
						}
					}
				]
			});
		};

		$scope.showDocumentDialog = function() {
			if (shouldSkipDocumentDialog()) {
				return $q.resolve(true);
			}

		    return $ionicPopup.show({
                title: ConfigService.config.business.add_fiscal_document_msg,
                scope: $scope,
				buttons: [
					{   text: 'Não',
					    onTap: function(e) {
							return false;
					    }
					}, {
						text: '<b>Sim</b>',
						type: 'button-theme',
						onTap: function(e) {
							return true;
						}
					}
				]
			});
		};

		function validateMandatoryIdentification() {
            var defer = $q.defer();
            const customerRequired = ConfigService.config.business.customer_identification_required;
            const customerAsFiscal = ConfigService.config.business.use_customer_as_fiscal_customer;
            const isNFCE = ConfigService.config.business.enable_type_nfce;
            const saleType = saleIdentificationType();
            const overlimit = isOverLimit();
            const mandatoryIdentification = saleType === 'MANDATORY';
            const notMandatoryIdentification = saleType === 'NOT_MANDATORY';

            if (customerRequired && !CartService.cart.customer && customerAsFiscal
                && (mandatoryIdentification || (notMandatoryIdentification && overlimit) || !isNFCE)) {

                if (overlimit) {
                    defer.reject(new CustomerNotIdentifiedError(`Compras a partir de`
                                        +`R$ ${$rootScope.minimumValueFiscalIdentification.toFixed(2)}`
                                        + ` devem possuir o cliente identificado`));
                } else {
                    defer.reject(new CustomerNotIdentifiedError(`Por favor identificar o cliente!`));
                }


            } else {
                defer.resolve();
            }
            return defer.promise;
        };

		function shouldSkipDocumentDialog() {
			if (ConfigService.config.business.skip_fiscal_dialog_when_overlimit && isOverLimit()) {
                return true;
            }

            if (saleIdentificationType() === 'MANDATORY') {
                return true;
            }

            if (CartService.cart.fiscalCustomer) {
                return true;
            }

            return mustForceCustomerAsFiscal();
		}

		function mustForceCustomerAsFiscal() {
			return CartService.cart.customer
					&& !CartService.cart.fiscalCustomer
					&& ConfigService.config.business.use_customer_as_fiscal_customer;
		}

		/**
		 * PICK_UP_FROM_WAREHOUSE(1) e DELIVERY(2) não emitem NF no storex mobile
		 * não necessita de identificação fiscal, por isso não entram no somatório
		 */
		function totalFiscalValue() {
			const cart = CartService.cart;
			const fiscalItems = cart.fiscalItems || [];
			let totalValue = 0;
			for (let i = 0; i < fiscalItems.length; i++) {
				var fiscalItem = fiscalItems[i];

				if (CartService.isSoldInStore(fiscalItem)) {
					totalValue += fiscalItem.totalValue;
				}
			}

			return totalValue;
		}


        /**
         *
         */
		$scope.doAction = function() {
			const checkExchangeReceiptQuantity = ConfigService.config.business.enable_editing_of_exchange_coupon_quantity;

            $q.resolve($rootScope.batteryAtMinimum)
            .then(batteryAtMinimum => {
                if (batteryAtMinimum) {
                    return AlertService.warn("O nível da bateria está baixo.<p>Conecte o aparelho ao carregador para prosseguir.</p>")
                    .then($q.reject);
                }
            })
			.then(validateMaximumSaleValue)
			.then(CartService.validateIfCartHasDofOrSaleRestrictionProduct)
            .then(CartService.validateIfCartHasPartnerProduct)
            .then(validateMandatoryIdentification)
			.then(() => CartService.addExchangeCoupon(checkExchangeReceiptQuantity, $scope))
            .then(checkFiscalCustomer)
            .then(() => {
                if ($rootScope.prescriptionEnable && !validProductionControlled()) { //Verificando se existe remédios controlados sem receita.
                    return false;
                }

                var currentCartAction = CartService.getCurrentCartAction();

                if ($rootScope.promotionEngine && $scope.manualDiscount && currentCartAction != "SHOWROOMING_CREATION") {
                    if (CartService.cart.discounts.length > 0) {
                        UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});
                    } else {
                        $scope.discountInFiscalCoupon();
                    }
                    return false;
                }

                switch (currentCartAction) {
                    case "ORDER_FULFILMENT": // pedido resgatado para separação
                        updateOrderAndSendWarnMessage(CartService.cart, OrderService.orderStatus.CHECKOUT);
                        break;
                    case "ORDER_PACKING":
                        updateOrderAndSendWarnMessage(CartService.cart, OrderService.orderStatus.DELIVERED);
                        break;
					case "ORDER_PACKING_V2":
						deliverOrder(CartService.cart.currentOrder)
						break;
                    case "ORDER_PAYMENT": // pedido resgatado para pagamento
						if (validateFiscalObligations()) {
							orderChecking()
								.then(() => $scope.payOrder());
						}
                        break;
                    case "ORDER_CREATION":
                        // não deve ser chamado pois o botão está oculto. Existe nesse caso os botões de enviar para av/pdv.
                        $scope.placeOrder(OrderService.orderStatus.CHECKOUT);
                        break;
                    case "ORDER_REMOTE_CREATION": // pedido a ser criado para outra loja
                        $scope.placeOrder(OrderService.orderStatus.ON_HOLD);
                        break;
                    case "SALE_PAYMENT": // venda
						if (validateFiscalObligations()) {
							$scope.payOrder();
						}
                        break;
                    case "SHOWROOMING_CREATION": // criando pedido showrooming
						$scope.chooseFulfillments();
                        break;
                    default:
                        break;
                }
            }).catch(err => {
				if (err instanceof SaleRestrictionError) {
					AlertService.warn(err.message)
				} else {
					AlertService.restError(err, err)
				}
			});
		};

		function checkFiscalCustomer() {
            if (noFiscalCustomerCheckActions.includes(CartService.getCurrentCartAction())) {
                return $q.resolve();
            }

            const defer = $q.defer();

            const overLimit = isOverLimit();
            const forceCustomerAsFiscal = mustForceCustomerAsFiscal();
            let currentFiscalCustomer = CartService.cart.fiscalCustomer;
            const lockFiscalCustomer = forceCustomerAsFiscal || CartService.cart._orderFiscalCustomer != null;
            const mandatoryIdentification = saleIdentificationType() === 'MANDATORY';

            // usa o parâmetro. Mas se tiver acima do limite, exibe obrigatoriamente.
            if ($rootScope.cpfInNoteEnabled || overLimit || lockFiscalCustomer) {

            $scope.reuseFiscalCustomerPopup()
            .then(reuse => {
                if (reuse) {
                    return;
                }
                //Em vendas de auto serviço o valor será sempre undefined, em resgate de pedido irá sobrescrever
                CartService.cart.fiscalCustomer = CartService.cart._orderFiscalCustomer;
                currentFiscalCustomer = CartService.cart._orderFiscalCustomer;
            }).then(() => {
                    $scope.showDocumentDialog()
                    .then(confirmation => {
                        const onError = () => {
                            if (overLimit) {
                                defer.reject(
                                    `Compras a partir de R$ ${$rootScope.minimumValueFiscalIdentification.toFixed(2)}`
                                    + ` devem possuir identificação fiscal completa!`
                                );
                            } else if (lockFiscalCustomer || mandatoryIdentification) {
                                defer.reject(
                                    `As informações do Cliente Fiscal devem ser preenchidas por completo`
                                );
                            } else {
                                defer.resolve();
                            }
                        };

                        if (confirmation) {
                            const store = ConfigService.config.store;
                            const currentStoreState = store && store.address && store.address.state ? store.address.state.toUpperCase() : null;
                            const customer = forceCustomerAsFiscal ? CartService.cart.customer : currentFiscalCustomer;

                            let customerPromise = $q.resolve(customer);

                            if (isFiscalCustomerIncomplete(customer)) {
                                customerPromise = FiscalCustomerModal.open(
                                    customer || CartService.cart.customer,
                                    $rootScope.mandatoryFiscalFields,
                                    overLimit || mandatoryIdentification,
                                    currentStoreState,
                                    lockFiscalCustomer,
                                );
                            }

                            customerPromise.then((fiscalCustomer) => {
                                CartService.cart.fiscalCustomer = fiscalCustomer;
                                defer.resolve();
                            }).catch(() =>
                                onError()
                            );
                        } else {
                            onError();
                        }
                    });
                });
            } else {
                defer.resolve();
            }

            return defer.promise;
        }

		function isFiscalCustomerIncomplete(customer) {
			if (customer == null || customer.document == null || customer.documentType == null) {
				return true;
			}

			if (!isOverLimit() && saleIdentificationType() === 'NOT_MANDATORY') {
                return false;
            }

			if ($rootScope.mandatoryFiscalFields == null) {
				return false;
			}

			const mandatoryName = $rootScope.mandatoryFiscalFields.includes('NAME');
			const mandatoryAddress = $rootScope.mandatoryFiscalFields.includes('ADDRESS');

			if (!mandatoryName && !mandatoryAddress) {
				return false;
			}

			if (mandatoryName) {
				if (!customer.name) {
					return true;
				}
			}

			if (mandatoryAddress) {
				const address = customer.address;

				if (!address) {
					return true;
				}

				const mandatoryAddressFields = [
					'zipCode', 'addressName', 'addressNumber',
					'addressComplement', 'neighborhood',
					'city', 'cityCode', 'state'
				];

				for (const field of mandatoryAddressFields) {
					if (isEmpty(address[field])) {
						return true;
					}
				}
			}

			return false;
		}

		function isEmpty(arg) {
			if (arg == null) {
				return true;
			}

			if (typeof arg == 'string') {
				return arg.length == 0;
			}

			return false;
		}

		function deliverOrder(order) {
			order = JSON.parse(JSON.stringify(order));
			order.status = 'DELIVERED';

			$ionicLoading.show({
				template: 'Atualizando status do pedido...',
				duration: 0
			}).then(() =>
				OrderService.update(order)
			).finally(() =>
				$ionicLoading.hide()
			).then(() => {
				if (order.additionalInformations) {
					const printDatas = order.additionalInformations.filter(coupon =>
						(coupon.paperType == "THERMAL_56COLS" || coupon.paperType == "THERMAL_42COLS")
						&& coupon.textLines[0]
						&& coupon.textLines[1])
					if (printDatas) {
						UtilService.navigateWithoutHistory('easyorder.couponview',
							{
								pdf: printDatas[0].textLines[1],
								printContents: [
									{
										paperType: printDatas[0].paperType,
										text: printDatas[0].textLines[0]
									}
								],
								documentTitle: 'order-coupon'
							}
						);
					} else {
						$scope.openModal(order, 'Pedido entregue com sucesso, obrigado pela preferência');
					}
				} else {
					$scope.openModal(order, 'Pedido entregue com sucesso, obrigado pela preferência');
				}
			}).catch((err) =>
				AlertService.restError(err, err)
			)
		}

		function validateMaximumSaleValue() {
			var defer = $q.defer();
			const totalValue = new Number(CartService.cart.totalValue);

			if (totalValue > $rootScope.maximumSaleValue) {
				AlertService.error(`Compra atual excede o limite fiscal de R$ ${$rootScope.maximumSaleValue.toFixed(2)} e não pode ser efetuada!`)
					.then(() => defer.reject());
			} else {
				defer.resolve();
			}
			return defer.promise;
        };

		function isOverLimit() {
			const totalValue = new Number(totalFiscalValue());
			return totalValue >= $rootScope.minimumValueFiscalIdentification;

		}

		function validateFiscalObligations() {
			var ret = true;

			if (isOverLimit()) {
				var fiscalCustomer = CartService.cart.fiscalCustomer;

                if (!fiscalCustomer || !fiscalCustomer.document) {
					ret = false;
					AlertService.warn(`Compras a partir de R$ ${$rootScope.minimumValueFiscalIdentification.toFixed(2)} devem possuir identificação fiscal!`);
				}
            }

            return ret;
        };

		function errorCallback(error) {
			AlertService.error(error.message);
		}

		function orderChecking() {
			let defer = $q.defer();

			// verifica se tem ítens conflitantes, caso esteja habilitado o parâmetro
			const cartItems = CartService.cart.fiscalItems;
			if (CartService.cart.orderChecking != null && UtilService.isNotEmptyArray(cartItems)) {
				// verifica se tem ítens vendidos em loja no pedido
				const orders = CartService.cart.orderChecking.orders ? CartService.cart.orderChecking.orders : [];
                const orderSoldInStoreItems = OrderChecking.extractOrderSoldInStoreItems(orders);

				if (UtilService.isNotEmptyArray(orderSoldInStoreItems)) {
					const orderItems = OrderChecking.extractOrderItems(orders);
					const conflitingItems = OrderChecking.conflitingItems(cartItems, orderItems);
					if (UtilService.isNotEmptyArray(conflitingItems)) {
						LogService.info(`Total de ítens conflitantes: ${conflitingItems.length}.`);
						OrderCheckingModal.open(conflitingItems)
							.finally(() => defer.reject());
					} else {
						// tem ítens de pedido mas não são conflitantes, então modifica os dados do carrinho para os do pedido
						// e efetua backup dos ítens do carrinho (para desfazimento ao voltar sem finalizar a venda)
						CartService.cart.orderChecking.cartItems = cartItems;
						// insere os ítens que não são do pedido, como ítens de auto-serviço + todos os pedidos (sem mudar descontos nem atributos)
						const items = OrderChecking.extractConvertItems(cartItems, orders);
						LogService.info(`Atualizando carrinho com os pedidos originais e ítens de auto-serviço.`)
						// não pode substituir diretamente pois o ítem é uma classe. Não precisa tratar o erro pois são ítens já adicionados previamente
						const fiscalItems = UtilService.cloneObject(items);
						CartService.cart.fiscalItems = [];
						let rejected = false;
						for (let index = 0; index < fiscalItems.length && !rejected; index++) {
							const fiscalItem = fiscalItems[index];
							rejected = !CartService.addItem(fiscalItem, null, errorCallback);
						}

						if (rejected) {
							// se ocoreram erros, retorna o carrinho pro status anterior e rejeita
							undoOrderCartItems();
							defer.reject();
						} else {
							// atualiza valores totais
							CartService.updateCartTotalValue();
							defer.resolve();

						}
					}
				} else {
					defer.resolve();
				}
			} else {
				defer.resolve();
			}
            return defer.promise;
        };

		$scope.getActionText = function() {
			var currentCartAction = CartService.getCurrentCartAction();
			var actionText;

			switch (currentCartAction) {
				case "ORDER_FULFILMENT":
				case "ORDER_PACKING":
				case "ORDER_PACKING_V2":
					actionText = "Finalizar separação";
					break;
				case "SALE_PAYMENT":
				case "ORDER_PAYMENT":
					actionText = ($rootScope.promotionEngine) ? "FINALIZAR" : "Pagar";
					break;
				case "ORDER_REMOTE_CREATION":
					actionText = "Gerar pedido loja " + CartService.cart.destinationStoreNumber;
					break;
				case "ORDER_CREATION":
					actionText = "Gerar pedido";
					break;
				case "SALE_PAYMENT":
					actionText = "Pagamento";
					break;
				case "SHOWROOMING_CREATION":
					actionText = "Selecionar opções de entrega";
					break;
				default:
					actionText = "(?)";
					break;
			}
			return actionText;
		};

		/**
		 * método chamado apenas no modal de confirmação de envio de pedidos.
		 */
		var getOrderConfirmationText = function(order) {
			var confirmationText = "";

			if (order) {
				var currentCartAction = CartService.getCurrentCartAction();

				switch (currentCartAction) {
    				case "ORDER_FULFILMENT":
					case "ORDER_PACKING":
    					confirmationText = "Pedido separado na loja " + CartService.cart.destinationStoreNumber + ".";
    					break;
    				case "ORDER_REMOTE_CREATION":
    					confirmationText = "Dirija o cliente para a loja " + CartService.cart.destinationStoreNumber + ".";
    					break;
    				case "ORDER_CREATION":
    					if (order.status == OrderService.orderStatus.NEW) {
    						confirmationText = "Dirija o cliente para atendimento.";
    					} else {
    						confirmationText = "Dirija o cliente para pagamento.";
    					}
    					break;
    				default:
    					confirmationText = "(?)";
    					break;
				}
			}
			return confirmationText;
		};

        function saleIdentificationType() {
            const codes = ConfigService.config.business.no_fiscal_identification_marketing_codes;

            if (codes == null || codes.length == 0) {
                return "NO_CONFIGURATION";
            }

            if (hasNoMandatoryIdentificationItems(codes)) {
                return "NOT_MANDATORY";
            }

            return "MANDATORY";
        };

        function hasNoMandatoryIdentificationItems(codes) {
           return CartService.cart.fiscalItems.every(item => {
                  return codes.some(code => {
                        if (code.includes('*')) {
                            const markWithoutAsterisc = code.replace('.*', '');
                            return item.marketingCode && item.marketingCode.indexOf(markWithoutAsterisc) == 0;
                        } else {
                            return item.marketingCode == code;
                        }
                  });
          });
        };
		/**
		 *
		 */
		$rootScope.$on("CallPayOrderMethod", function(){
			$scope.payOrder();
		 });

		$scope.payOrder = function() {
			if ($scope.isMandatoryParameterValid()) {
			    var customerRequired = ConfigService.config.business.customer_identification_required;
				var documentEmpty = (!$scope.cart.customer || !$scope.cart.customer.document || $scope.cart.customer.document.length == 0);

                var isMobileRecharge = CartService.getCurrentServiceOperationType() === 'MOBILE_RECHARGE';

                const isCustomerNotMandatory = saleIdentificationType() === 'NOT_MANDATORY';

				if (ConfigService.config.tefserver && ConfigService.config.tefserver.host != '' && ConfigService.config.tefserver.port != 0) {
                    if ((customerRequired && !documentEmpty && $scope.validateFields()) || !customerRequired || isCustomerNotMandatory || isMobileRecharge || (CartService.hasNonFiscalItems && !documentEmpty)) {

						if (CartService.preConfirmedPayments != null) {
							CartService.preConfirmedPayments.forEach(p => {
								PaymentService.initPayment(
									p.paymentCodeMethod,
									p.paymentValue,
									p._privateLabel
								);
								p.paymentSequence = PaymentService.payments().length;
								CartService.cart.salePayments.push(p);
								PaymentService.currentPaymentDone();
							});
						}

						if (CartService.predefinedPaymentMethodsDetails && CartService.predefinedPaymentMethodsDetails.length) {
							CartService.processedPredefinedPayments.length = 0; //limpando.
							UtilService.navigateWithoutHistory('easyorder.predefinedpaymentmethods');
						} else if (PaymentService.totalPayments() >= CartService.cart.totalValue) {
							$ionicLoading.show({
								template : 'Por favor, aguarde...',
								duration : 0
							})
							.then(() =>
								UtilService.navigateWithoutHistory('easyorder.paymentresult')
							);
						} else {
							UtilService.navigateWithoutHistory('easyorder.paymentmethods');
						}
                    }  else {
                        AlertService.warn('Favor identificar cliente!');
					}
				} else {
					AlertService.warn('TEF não configurado!');
				}
			}
		};

		$scope.isMandatoryParameterValid = function() {
			var isValid = true;

			if ($scope.mandatoryParameterFromServer == "SET2"){
				if (!$scope.cart.customer || !$scope.cart.customer.document || !$scope.validateFields() || !$scope.cart.customer.email){
					AlertService.warn('Para realizar o pagamento é necessário informar o CPF e o E-mail do cliente!');
					isValid = false;
				}
			}

			return isValid;
		};

		const searchItemListFreight = (searchOnly) => {
			var cart = CartService.cart;
			var showroomingItems = CartService.getShowroomingItems();
			var isPickupInStore =  cart.showroomingInfo.searchOption === 'pickup_in_store';
			var loadingMessage;
			var zipcode;
			if (isPickupInStore) {
				loadingMessage = "Consultando lojas disponíveis para retirada dos produtos...";
			} else {
				zipcode = $scope.cart.deliveryAddress.zipCode;
				loadingMessage = "Consultando opções de entrega para o CEP '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'...";
			}

			$ionicLoading.show({
				template :  loadingMessage,
				duration : 0
			}).then(function() {
				var error = function(error) {
					$scope.loadingState = false;
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro ao consultar opções de entrega!');
				};

				var success = function(freights) {
					$scope.loadingState = false;
					$ionicLoading.hide();

					var hasShowrooming = false;
					if (freights && freights.length > 0)  {
                        for (const f of freights) {
                            if (f && f.groups && f.groups.length > 0) {
                                var group = f.groups[0];
                                if  (group.pickups && group.pickups.length > 0) {
                                    hasShowrooming = true;
                                }
                                // se for apenas pickup in store, não considera os shipments. E remove-os para não serem expostos e escolhidos.
                                if (isPickupInStore) {
                                    group.shipments = [];
                                } else {
                                    if (group.shipments && group.shipments.length > 0) {
                                        hasShowrooming = true;
                                    }
                                }
                            }
                        }
                    }
					if (hasShowrooming) {
						if (!searchOnly){
							UtilService.navigateWithoutHistory('easyorder.showroomingfulfillments', { freightQuote: freights });
						}
					} else {
						if (isPickupInStore) {
							AlertService.error("Nenhuma loja disponível para entrega de todos os produtos do carrinho!")
						} else {
							AlertService.error("Não existem mais opções de entrega para os itens atuais do carrinho no CEP: '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'.")
						}

						if (searchOnly){
							$scope.decrementInCart($scope.currentItem);
						}
					}
				}

                const orderValue = cart.totalValue - cart.totalDiscount;

				if (isPickupInStore) {
					FreightService.searchItemListByStore(cart.companyNumber, cart.storeNumber, orderValue, showroomingItems, success, error);
				} else {
					FreightService.searchItemListByZipcode(cart.companyNumber, cart.storeNumber, zipcode, orderValue, showroomingItems, success, error);
				}
			});
		};

		function validateCustomer() {
			var customer = $scope.cart.customer;
			if (customer && customer.document && customer.document.length > 0 && customer.name && customer.name.length > 0) {
				return true;
			}
			return false;
		}

		$scope.chooseFulfillments = function() {
			if (validateCustomer()) {
				var isPickupInStore =  $scope.cart.showroomingInfo.searchOption === 'pickup_in_store';
				if (isPickupInStore) {
					searchItemListFreight(false);
				} else {
					if ($scope.cart.deliveryAddress && $scope.cart.deliveryAddress.zipCode) {
						searchItemListFreight(false);
					} else {
						AlertService.error("CEP não foi informado!");
					}
				}
			} else {
			    AlertService.error("Para realizar Showrooming é necessário identificar o cliente, ou informar ao menos o seu Nome e CPF!");
			}
		};

		/**
		 * Nome preenchido e Documento preenchido
		 */
		$scope.validateFields = function() {
			var isValid = false;

			switch ($scope.cart.customer.documentType) {
			    case "CPF":
			        isValid = validaCPF($scope.cart.customer.document);
			        break;
			    case "CNPJ":
			        isValid = validaCNPJ($scope.cart.customer.document);
					break;
				case "RNE":
			        isValid = validaRNE($scope.cart.customer.document);
			        break;
			    case "PASSPORT":
			        isValid = validaPassport($scope.cart.customer.document);
			        break;
			}
			return isValid;
		};

		$scope.doNewOrderAction = function() {
			if ($rootScope.externallyLoadedApp) {
				$rootScope.exitApp();
			} else {
				var confirmPopup = AlertService.confirm('Deseja iniciar uma nova venda?');

				confirmPopup.then(function(result) {
					if (result) {
						$scope.clearCart();
					}
				});

			}
		}

		/**
		 *
		 */
		$scope.clearCart = function() {
			CartService.initCart();

			UtilService.navigateToMainScreen();
		};

		$scope.identifyCustomer = function() {
            if (ConfigService.config.business.unified_customer_identification_enabled) {
               return UtilService.navigateWithoutHistory('easyorder.customer');
            } else {
                return UtilService.navigateWithoutHistory('easyorder.customermenu');
            }
		};

		$scope.searchProduct = function() {
			UtilService.navigateToMainSearchScreen();
		};

		function createItemsOpts(cartItems) {
			const opts = {};

			if (cartItems == null) {
				return opts;
			}

			updateItemsOpts(cartItems, opts);

			return opts;
		}

		function updateItemsOpts(cartItems, opts) {
			for (const cartItem of cartItems) {
				updateItemOpts(cartItem, opts);
			}
		}

		function updateItemOpts(cartItem, opts) {
			const idx = cartItem.sequenceId;
			const opt = opts[idx] || {};
			opt.disableQttyInput = mustDisableQuantityInput(cartItem);
			opt.disableIncrement = mustDisableItemIncrement(cartItem);
			opt.disableDecrement = mustDisableItemDecrement(cartItem);
			opts[idx] = opt;
		}

		function mustDisableItemIncrement(item) {
			return mustDisableQuantityInput(item);
		};

		function mustDisableItemDecrement(item) {
			return 1 >= item.displayQuantity
				|| mustDisableQuantityInput(item);
		}

		function mustDisableQuantityInput(item) {
			return item.orderNumber
				|| item.productType == 'PARTNER'
				|| item.groupCode
				|| CartService.isRemoteOrder();
		}

		/**
         *
         */
		$scope.incrementInCart = function(item) {
			$scope.currentItem = item;
			item.quantitySold = Number(item.quantitySold);
		    CartService.undoCampaignDiscounts();
		    CartService.removeCustomerRelatedDiscounts();
		    CartService.incrementInCart(item);
			CartService.applyLoyaltyDiscount()
				.finally(() =>	CartService.processCampaigns())
				.finally(() => CartService.applyEmployeeDiscount());
			if (item.showrooming){
				searchItemListFreight(true);
			}
        };

        /**
         *
         */
        $scope.decrementInCart = function(item) {
            CartService.undoCampaignDiscounts();
            CartService.removeCustomerRelatedDiscounts();
            CartService.decrementInCart(item);
			CartService.applyLoyaltyDiscount()
				.finally(() =>	CartService.processCampaigns())
				.finally(() => CartService.applyEmployeeDiscount());
		};

		$scope.updateQuantityInCart = function(item){
			$timeout.cancel(searchTimeoutPromise);

            searchTimeoutPromise = $timeout(function() {
				const displayQuantity = item.displayQuantity;

				if (displayQuantity < 1 || displayQuantity > 9999) {
					AlertService.info("A quantidade do produto deve estar entre 1 e 9999.");
					item.displayQuantity = 1;
				}

				item.processTotalValue();

				CartService.undoCampaignDiscounts();
				CartService.removeCustomerRelatedDiscounts();


				CartService.applyLoyaltyDiscount()
					.finally(() =>	CartService.processCampaigns())
					.finally(() => CartService.applyEmployeeDiscount());
                //chama reload bag cart
				$rootScope.$emit("cart-item-quantity");

			}, 800);
		};

		$scope.setCurrentQuantitySoldValue = function(item){
			$scope.currentQuantitySold = item.quantitySold;
			item.quantitySold = Number(item.quantitySold);
		};

        $scope.$on('$ionicView.loaded', function() {
    		CartService.undoCampaignDiscounts();
        	CartService.removeCustomerRelatedDiscounts();
			CartService.applyLoyaltyDiscount()
				.finally(() =>	CartService.processCampaigns())
				.finally(() => CartService.applyEmployeeDiscount());
			$scope.checkMideSettings();
		});

		$scope.removeCustomerRelatedDiscounts = function() {
			CartService.removeCustomerRelatedDiscounts();
			CartService.updateDiscountCart();
		};

		$scope.checkMideSettings = () => {
			if ($state.is('easyorder.cart') && ConfigService.config.business.ignore_mid_settings && !CartService.hasShowrooming() && !CartService.isPickupOrder()) {
				AlertService.info("O Caixa Rápido não está configurado para este tipo de operação, apenas showrooming é permitido.").then(() => UtilService.navigateToMainScreen());
			}
		}

		$scope.showNumberOfGifts = function() {
			var giftQuantity  = $scope.cart.giftsQuantity ? $scope.cart.giftsQuantity : 0;
			var msgComplement = giftQuantity && giftQuantity > 1 ? " brindes!" : " brinde!";
			AlertService.info("Nesta compra você ganhou " + giftQuantity + msgComplement);
		};

		$scope.doNewOperation = function() {
			var currentOperation = CartService.getCurrentServiceOperationType();
			if (currentOperation === 'GIFT_CARD_RELOAD') {
				UtilService.navigateWithoutHistory('easyorder.giftcardsearch');
			} else {
				AlertService.error("Tipo de serviço inválido: " + currentOperation);
			}
		};

		var undoServiceOperation = function() {
		    var error = function(error) {
		        LogService.error('Erro no desfazimento do serviço: ', JSON.stringify(error));
		    };

		    var success = function(resp) {
		        LogService.info("Desfazimento do serviço efetuado com sucesso!");
		    };

		    var serviceOperation = {
		        companyNumber: ConfigService.config.companyNumber,
		        storeNumber: ConfigService.config.storeNumber,
		        terminalNumber: ConfigService.config.terminalNumber,
		        operatorCode: UserService.operatorCode(),
		        nsu: CartService.cart.nsu,
		        serviceItems: CartService.cart.serviceItems,
		        payments: CartService.cart.salePayments,
		        totalValue: CartService.cart.totalValue,
				date: new Date().toJSON()
		    };

		    LogService.info('Efetuando cancelamento implícito e assíncrono do serviço...');
		    NFCeService.undoServiceIntegration(serviceOperation, success, error);
		};

		$scope.cancelOperation = function() {
			// método só chamado em operações de serviço.
			AlertService.confirm("Deseja cancelar a operação?").then(function(result) {
               if (result) {
            	   undoServiceOperation();
            	   $scope.clearCart();
               }
           });
		};

		$scope.isRemoteOrder = function() {
			return CartService.isRemoteOrder();
		};

		$scope.showDiscountDetails = function() {
			var discounts = $scope.cart.discounts;

			if (discounts) {
				var discountsDetailsMessage;
				for (var i = 0; i < discounts.length; i++) {
					var discount = discounts[i];
					discountsDetailsMessage = "";
					switch (discount.source) {
						case "CAMPAING":
							discountsDetailsMessage = "Campanha: R$ ";
							break;
						case "EMPLOYEE":
							discountsDetailsMessage = "Funcionário: R$ ";
							break;
						case "FIRST_PURCHASE":
							discountsDetailsMessage = "Primeira compra: R$ ";
							break;
						case "PLAN":
							discountsDetailsMessage = "Plano: R$ ";
							break;
						case "EXTERNAL":
							discountsDetailsMessage = "Outros: R$ ";
							break;
						case "CASHBACK":
							discountsDetailsMessage = "Cashback: R$ ";
							break;
						default:
							break;
					}
					discountsDetailsMessage += discount.value.toString().replace(".", ",");
					discountsDetailsMessage += "\n";
				}
				AlertService.alert("Descontos", discountsDetailsMessage);
			}
		};

        CustomerService.registerObserverCallback("cartCtrl", function() {
            $scope.removeCustomerRelatedDiscounts();
		});

		var validProductionControlled = function() {
			var permission = UserService.checkPermission(PRESCRIPTION_AVOIDANCE_PERMISSION);
			var qtdProducts = CartService.countProductsControlled();
			var qtdProductsPrescription = CartService.countProductsInPrescriptionControlled();

			if (qtdProducts > 0 || qtdProductsPrescription > 0) {
				LogService.info('Items Cart', CartService.cart.fiscalItems);
				LogService.info('qtdPrescriptions',CartService.cart.prescriptions);
				LogService.info('qtdProductsControl',qtdProducts);

				if (qtdProducts > 0 && !qtdProductsPrescription && qtdProductsPrescription == 0 ) {
					AlertService.confirm("Você adicionou produtos controlados e não adicionou nenhuma receita!<br>"
					        + "Deseja adicionar agora?").then(function(result){
						if (result) {
							UtilService.navigateWithoutHistory('easyorder.prescriptions');
						} else {
							AlertService.confirm("Para efetuar o pagamento é necessário autorização do gerente.").then(function(result) {
								if (result) {
									UtilService.navigateWithoutHistory('easyorder.authmanager');
								}
							});
						}
					});
					return false
				} else {
					var listCompareProduct = CartService.getCompareListProductPrescription();
					var listComparePrescription = CartService.getCompareListPrescriptionProduct();
					//Produtos que estão faltando na receitas.
					var productMissingPrescription = listCompareProduct.some(function(item){return item.comparePrescription < 0 && item.productType == "PRESCRIPTION_CONTROLLED"});
					var productOffPrescription = listCompareProduct.some(function(item){return (item.quantitySold + item.comparePrescription) == 0 && item.productType == "PRESCRIPTION_CONTROLLED"});
					var productOverPrescription = listCompareProduct.some(function(item){return item.comparePrescription > 0 && item.productType == "PRESCRIPTION_CONTROLLED"});

					LogService.info('listCompareProduct', listComparePrescription);

					var prescriptionMissingProducts = listComparePrescription.some(function(prescription){
						var teste = prescription.items.some(function(item){
							LogService.info(item.compareProduct);
							return item.compareProduct < 0});
						LogService.info(teste);
						return teste;
					});

					if (prescriptionMissingProducts) {
						LogService.info("Existem mais receitas que produtos");
						AlertService.confirm("Existem menos produtos controlados do que as receitas."
						        + "	Deseja adicionar agora?").then(function(result){
							if (result) {
								UtilService.navigateWithoutHistory('easyorder.productsearch');
							} else {
								AlertService.confirm("Para efetuar o pagamento é necessário autorização do gerente.").then(function(result) {
									if (result) {
										UtilService.navigateWithoutHistory('easyorder.authmanager');
									}
								});
							}
						});
						return false;
					} else if (productOverPrescription) {
						LogService.info("Existe produtos sobrando na receita");
						AlertService.confirm("Existem menos produtos controlados do que as receitas."
						+ "	Deseja adicionar agora?").then(function(result){
							if (result){
								UtilService.navigateWithoutHistory('easyorder.productsearch');
							}else{
								AlertService.confirm("Para efetuar o pagamento é necessário autorização do gerente.").then(function(result) {
									if (result) {
										UtilService.navigateWithoutHistory('easyorder.authmanager');
									}
								});
							}
						});
						return false;
					} else if (productOffPrescription) {
						LogService.info("Existe produtos sem receita");
						AlertService.confirm("Existe produto(s) sem receita(s)."
						        + "	Deseja adicionar agora?").then(function(result){
							if (result){
								UtilService.navigateWithoutHistory('easyorder.prescriptions');
							}else{
								AlertService.confirm("Para efetuar o pagamento é necessário autorização do gerente.").then(function(result) {
									if (result){
										UtilService.navigateWithoutHistory('easyorder.authmanager');
									}
								});
							}
						});
						return false;
					} else if (productMissingPrescription && !productOffPrescription) {
						LogService.info("Existe produtos faltando na receita");
						AlertService.confirm("Existem mais produtos controlados do que as receitas permitem."
						        + "	Deseja adicionar uma nova receita?").then(function(result) {
							if (result) {
								UtilService.navigateWithoutHistory('easyorder.authmanager');
							} else {
								AlertService.confirm("Para efetuar o pagamento é necessário autorização do gerente.").then(function(result) {
									if (result) {
										UtilService.navigateWithoutHistory('easyorder.authmanager');
									}
								});
							}
						});
						return false;
					}
				}
			}
			return true;
		};

		$scope.deleteService = function(item, nonFiscalItem){
			let index = item.nonFiscalItems.indexOf(nonFiscalItem);
			item.nonFiscalItems.splice(index, 1);
			item.processTotalValue();
			CartService.updateCartTotalValue();
			CartService.reindexCartItems();

			if (item.nonFiscalItems.length == 0){
				delete item.nonFiscalItems;
				$scope.isActivedSwipe = true;
			}
		}

		$scope.updateAdditionalServices = function(item){
			return $ionicLoading.show({
				template : 'Aguarde...',
				duration : 0
			}).then(function() {
				ServiceSaleService.fetchServices(item,
				resp => {
					$ionicLoading.hide();
					if (resp.services.length > 0){
						UtilService.navigateWithoutHistory('easyorder.servicesale', { "item": item, "services": resp.services, "updateAdditionalServices": true });
					}
				}, error => {
					AlertService.warn(error.data.message)
					$ionicLoading.hide();
				});
			});
		}

		$scope.addService = function(item){
			UtilService.navigateWithoutHistory('easyorder.servicesale', item);
		}

		$scope.isActivedSwipe = true;
		$scope.activeSwipe = function(index){
			var input = document.getElementById("openMenu"+index);


			if (!input.checked) {
				input.checked = true;
			} else {
				input.checked = false;
			}

			let total = 0;
			CartService.cart.fiscalItems.forEach((element, index) => {
				if (element.serviceItemsAvailable && element.nonFiscalItems && element.nonFiscalItems.length > 0){
					var input = document.getElementById("openMenu"+index);
					if (input.checked){
						total++;
					}
				}
			});
			if (total > 0){
				$scope.isActivedSwipe = false;
			}else {
				$scope.isActivedSwipe = !input.checked;
			}
		}

		$scope.init = function() {
			CartService.undoPlanDiscount();
			CartService.updateCartTotalValue();
		    let couponDiscount = CartService.cart.discounts && CartService.cart.discounts.filter(d => d.source === "COUPON")[0];
			$scope.selectedCovenant = CovenantService.currentCovenant();
		    $scope.data = {};
	        $scope.data.coupon = couponDiscount ? couponDiscount.indentifierCode : undefined;
            $scope.couponenable = ConfigService.config.business
                && ConfigService.config.business.discount_coupon_mode
                && ConfigService.config.business.discount_coupon_mode != "NENHUM";
			$scope.couponValid = !!$scope.data.coupon;
			$scope.additional_services = ConfigService.config.business.additional_services;

			$scope.reasons = ConfigService.config.business.price_change_reasons_list;
			$scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
			$scope.itemExclusionByScanning = $rootScope.isCordova && ConfigService.config.business.item_exclusion_by_barcode_scanning === true;
			$scope.itemExclusionBySliding = ConfigService.config.business.item_exclusion_by_sliding !== false;
			$scope.productCodeType = ConfigService.config.business.product_code_character_type;

			if (ConfigService.config.sms_info && ConfigService.config.sms_info.order_ready
					&& ConfigService.config.sms_info.order_ready.sender_id
					&& ConfigService.config.sms_info.order_ready.message) {
				$scope.orderReadySmsInfo = ConfigService.config.sms_info.order_ready;
			}

			if (!CartService.cart.orderRetrieved) {
				CartService.removeFreightValue();
			}

			undoOrderCartItems();


		};

		function undoOrderCartItems() {
			if (CartService.cart.orderChecking != null && UtilService.isNotEmptyArray(CartService.cart.orderChecking.cartItems)) {
				LogService.info(`Retornando o carrinho para os ítens escaneados originalmente antes da conferência de ítens no pedido.`)
				CartService.cart.fiscalItems = CartService.cart.orderChecking.cartItems;
				CartService.cart.orderChecking.cartItems = null;
				CartService.updateCartTotalValue();
			}
		}

		$scope.showCoupons = () => {
			$scope.alertCoupons = AlertService.showAlertList($scope, "Cupons", CartService.cart.coupons,
				 [{"field":"barcode"}], $scope.removeCoupon);
		}

		$scope.showSuggestions = () => {
			AlertService.showAlertList($scope, "Sugestões", CartService.cart.suggestions, [{"field":"descriptor"},]);
		}

		$scope.showGiftBenefts = () => {
			AlertService.showAlertList($scope, "Presentes", CartService.cart.giftBenefits, [{"field":"giftType"},]);
		}

		$scope.getCountSuggestions = () => {
			return  CartService.countSuggestions();
		}


		$scope.getCountGiftBenefits = () => {
			return  CartService.countGiftBenefits();
		}


		$scope.getCountCoupons = () => {
			return CartService.countCoupons();
		}


		$scope.removeCoupon = (coupon) => {
            AlertService.confirm(`Deseja remover o Cupom: <b>${coupon.barcode}</b>?`).then( (result) => {
                if (result){
                    CartService.removeCoupon(coupon.barcode);
                    if ($scope.alertCoupons){
						$scope.alertCoupons.close();
						CartService.undoCampaignDiscounts();
						CartService.applyLoyaltyDiscount()
						.finally(() =>	CartService.processCampaigns())
						.finally(() => {
							CartService.applyEmployeeDiscount();
							if ($scope.alertCoupons && CartService.countCoupons() > 0) $scope.showCoupons();
						});
                    }
                }
            });
		}

        $scope.onTabSelected = (tab) => {
            $scope.tabSelected = tab;
        }

        $scope.showDiscountCoupon = () => {
            return (($scope.cart.totalDiscount && $scope.cart.totalDiscount > 0) || ($scope.cart.giftsQuantity && $scope.cart.giftsQuantity > 0))
        }

        /**
         *
         */
        $scope.confirmClearCart = () => {
            AlertService.confirm('Deseja limpar os itens atuais e reiniciar a leitura?')
            .then(result => {
                if (result) {
                    var cartItens = CartService.cart.fiscalItems;

					const proceed = () => {
						Promise.resolve()
                        .then(() => CartService.undoCampaignDiscounts())
                        .then(() => CartService.cart.fiscalItems.length = 0)
						.then(() => CartService.updateCartTotalValue())
                        .then(() => $rootScope.$emit("cart-item-quantity"))
                        .then(() => $rootScope.$emit("screen-total-value", 0))
						.then(() => { UtilService.navigateToMainSearchScreen() } )
						.catch((e) => {
							console.log('Erro ao remover ítens do carrinho.');
							console.log(e);
						})
                    };
					if (ConfigService.config.business.enable_reserve_products === true && cartItens.length > 0
                            &&  (cartItens = cartItens.filter(item => item.itemUuid && item.itemUuid.value)).length) {
                        const cancelParams = {
                            storeNumber : ConfigService.config.storeNumber,
                            orderNumber: CartService.sessionId,
                            terminalNumber: ConfigService.config.terminalNumber,
                            operatorName: UserService.currentUser.name,
                            products: cartItens.map(item => {
                                return {
                                    productCode: item.productCode,
                                    uuid: item.itemUuid.value,
                                    quantity: item.quantitySold
                                };
                            })
                        };
                        ProductService.cancelSale(cancelParams).finally(() => {
                            proceed();
                        });
                    } else {
                        proceed();
                    }
                }
            });
        };

		$scope.showManualItemPopup = () => {
			LogService.info("Exibindo popup de exclusão manual do ítem.");
			$scope.itemPopup = {};
			//TODO
			$scope.popupRemoveItemByBarcode = $ionicPopup.show({
				templateUrl: 'views/remove-item.html',
				title: 'Exclusão de ítem',
				scope: $scope,
				buttons: [
					{
						text: 'Cancelar',
						onTap: function () {

						}
					},
					{
						text: 'OK',
						onTap: function (e) {
							e.preventDefault();
							if ($scope.itemPopup.value) {
								$scope.removeItemByCode($scope.itemPopup.value)
								.then(() => {
									$scope.popupRemoveItemByBarcode.close();
									if (!CartService.hasItems()) {
										UtilService.navigateToMainSearchScreen();
								   }
								});
							} else {
								AlertService.warn(`Informe um código de próduto!`);
							}

						}
					},
				]
			});
		}


		$scope.showBarcodeItemPopup = () => {
			LogService.info("Iniciando scanner para exclusão de ítens.");

			$scope.popupRemoveItemByBarcode = $ionicPopup.show({
				template: `<div style="display: flex;" ng-style="{ height: '26em' }"></div>`,
				title: 'Exclusão de ítem',
				scope: $scope,
				buttons: [
					{
						text: 'Cancelar',
						onTap: function () {
							BarcodeService.cancel();
						}
					},
					{
						text: 'Digitar',
						onTap: function () {
							BarcodeService.cancel();
							$scope.showManualItemPopup();
						}
					}
				]
			});

			$scope.removeItemByCode = (itemCode) => {
				let defered  = $q.defer();
				// A C&A manipula etiquetas compostas onde o código do produto é uma parte dela
				// Todos os outros clientes recebem o mesmo código passado por parâmetro
				const productCode = $rootScope.customBarcodeHandler.extractProductCode(itemCode);
				const item = CartService.cart.fiscalItems.find(item => item.productCode === productCode);
				if (item) {
					const label = item.quantitySold > 1 ? 'diminuir a quantidade do' : 'excluir o';

					AlertService.confirm(`Deseja ${label} ítem "${item.productDescription}" (código ${productCode}) do carrinho?`)
								.then(res => {
									if (res) {
										LogService.info(`Produto de código ${productCode} será decrementado ou excluído.`);
										// [processamento descentralizado, trecho copiado]
										if (CartService.getCurrentOperation() === 'SERVICE') {
											// Caso sejam serviços, é necessário disparar a mensagem para cancelamento no LINX-WS.
											// Mensagem é assíncrona e erros, ignorados.
											undoServiceAuthorization(index);
										}
										CartService.undoCampaignDiscounts();
										CartService.removeCustomerRelatedDiscounts();
										CartService.decrementInCart(item);
										CartService.applyLoyaltyDiscount()
											.finally(() =>	CartService.processCampaigns())
											.finally(() => CartService.applyEmployeeDiscount());
									}
									defered.resolve();
								});
				} else {
					AlertService.warn(`Produto de código ${productCode} não encontrado no carrinho!`)
						.then(() => defered.reject());
				}
				return defered.promise;
			}

			$scope.popupRemoveItemByBarcode.then(function(result) {
				if (result && result.length > 1) {
					$scope.removeItemByCode(result[0])
						.then(() => {
							if (!CartService.hasItems()) {
								UtilService.navigateToMainSearchScreen();
						   }
						});
				}
			});

			$scope.initBarcodeScanner();
		}

		$scope.initBarcodeScanner = () => {
			const rect = {
				inSizeCam: ['18/110/18/120', '55/420/55/390'],
				outSizeCam: ['275/110/20/100', '275/80/20/80']
			};

			BarcodeService.init(barcode => {
				BarcodeService.cancel();
				$scope.popupRemoveItemByBarcode.close(barcode);
			}, rect);
		};

		$scope.openItemSellerListDialog = function(fiscalItem) {
			$scope.currentSellerItem = fiscalItem;

			const user = UserService.currentUser;
			let sellers = CartService.cart.fiscalItems.filter(i => (i.seller !=  null)).map(i => i.seller);
			// adiciona o usuário local como vendedor
			sellers.push({ login: user.username, name: user.name });
			// clona
			sellers = JSON.parse(JSON.stringify(sellers));
			// pega os vendedores (distintos)
			$scope.distinctSellers = [...new Map(sellers.map(item =>[item['login'], item])).values()];
			// ordena-os
			$scope.distinctSellers = $scope.distinctSellers.sort((a, b) => a.name.localeCompare(b.name))

			const buttons = [{
                text: '<b>Cancelar</b>',
                type: 'button-theme',
                onTap: function(e) {
					$ionicListDelegate.closeOptionButtons();
                    return true;
                }
            }];

			$scope.currentPopup = $ionicPopup.confirm({
                title : 'Vendedor',
                templateUrl : './partials/seller/seller-list.html',
                subTitle : 'Selecione Vendedor',
                scope: $scope,
                buttons : buttons
            })
		}

		$scope.openItemSellerSearchDialog = function(fiscalItem) {
			$scope.sellerSearch = {};

			const buttons = [{
                text: '<b>Cancelar</b>',
                type: 'button-stable',
                onTap: function(e) {
					$ionicListDelegate.closeOptionButtons();
                    return true;
                }
            },
			{
                text: '<b>Listar</b>',
                type: 'button-stable',
                onTap: function(e) {
					$scope.openItemSellerListDialog(fiscalItem);
                    return true;
                }
            },
			{
				text: '<b>Pesquisar</b>',
				type: 'button-theme',
				onTap: function(e) {
					const login = $scope.sellerSearch.login;

					if (UtilService.isNullOrEmpty(login)) {
						AlertService.warn("Por favor, entre com um vendedor!");
						e.preventDefault();
						return;
					} else {
						$ionicLoading.show({
							template : 'Por favor, aguarde...',
							duration : 0
						}).then(function() {
							UserService.searchSeller({ login: login }, function(result) {
								fiscalItem.salesAssociateNumber = login;
								fiscalItem.seller = result.sellers[0];
								$ionicLoading.hide();
								$scope.showItemSellerInfo(fiscalItem);
							}, function(error) {
								$ionicLoading.hide();
								AlertService.restError(error, 'Não foi possível obter dados do vendedor.')
									.then(() => $scope.openItemSellerSearchDialog(fiscalItem));
							});
						});
						return true;
					}

				}
			}];

			$ionicPopup.confirm({
                title : 'Vendedor',
                templateUrl : './partials/seller/seller-search.html',
                subTitle : 'Dados do Vendedor',
                scope: $scope,
                buttons : buttons
            })
		}

		$scope.showItemSellerInfo = function(fiscalItem) {
			$scope.sellerFiscalItem = fiscalItem;

			const buttons = [
			{
				text: '<b>Remover</b>',
				type: 'button-stable',
				onTap: function(e) {
					fiscalItem.salesAssociateNumber = null;
					fiscalItem.seller = null;
					$ionicListDelegate.closeOptionButtons();
					return true;
				}
			},
			{
				text: '<b>Alterar</b>',
				type: 'button-stable',
				onTap: function(e) {
					$scope.openItemSellerSearchDialog(fiscalItem);
					return true;
				}
			},
			{
                text: '<b>OK</b>',
                type: 'button-theme',
                onTap: function(e) {
					$ionicListDelegate.closeOptionButtons();
                    return true;
                }
            }];

			$ionicPopup.confirm({
                title : 'Vendedor',
                templateUrl : './partials/seller/seller-info.html',
                subTitle : 'Vendedor do Ítem',
                scope: $scope,
                buttons : buttons
            })
		}

		$scope.selectSeller = function(seller, fiscalItem) {
			fiscalItem.salesAssociateNumber = seller.login;
			fiscalItem.seller = seller;
			if ($scope.currentPopup) {
				$scope.currentPopup.close();
				$scope.currentPopup = null;
			}
			$scope.showItemSellerInfo(fiscalItem);
		}

		$scope.openItemSellerDialog = function(fiscalItem) {
			if (fiscalItem.salesAssociateNumber == null) {
				$scope.openItemSellerSearchDialog(fiscalItem);
			} else {
				$scope.showItemSellerInfo(fiscalItem);
			}
		}

        /**
         *
         */
		$scope.init();
	});
})();
