(function () {
    'use strict';
    angular.module('easyorder.controllers.coupon.view', [
        'ionic',
        'ionic.closePopup'
    ]).controller('CouponViewCtrl', function (
                        $scope,
                        $ionicLoading,
                        UtilService,
                        AlertService,
                        $stateParams,
                        ConfigService,
                        CartService,
                        $ionicModal,
                        LogService,
                        PrinterService,
                        MailService) {
        
        $scope.printingEnabled = ConfigService.config.business.printing_enabled;
        $scope.sendMailEnabled = ConfigService.config.business.send_mail_enabled;
        $scope.redirectsWhenPrinting = ConfigService.config.business.redirects_new_sale_when_printing !== false;
        $scope.pdf = new PDF($stateParams.pdf);
        $scope.documentTitle = $stateParams.documentTitle;
        $scope.showPdf = $stateParams.showPdf;
        const printContents = $stateParams.printContents;

        if($stateParams.pdf == " " && $stateParams.printContents.length == 0){
            AlertService.info('Resgate do pedido realizado com sucesso. Nenhum cupom gerado para esta operação.').then(() => {
                $scope.nextPage();
            });
        }

        $scope.percentButtonMail = function(){
            if ($scope.sendMailEnabled){
                if ($scope.printingEnabled){
                    return "27%";
                }else{
                    return "9%";
                }
            }
        };
        $ionicModal.fromTemplateUrl('views/sendmailorder.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modalMail) {
            $scope.modalMail = modalMail;
        });
        $ionicModal.fromTemplateUrl('views/printerslist.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });
        
        function showPrintersModal() {
            return $scope.modal.show();
        }

        function hidePrintersModal() {
            return $scope.modal.hide();
        }
        
        $scope.openModalMail = function() {
            $scope.modalMail.show();
        };
        
        $scope.closeModalMail = function() {
            return $scope.modalMail.hide();
        };

        $scope.$on('$destroy', () => {
            $scope.modal.remove();
            $scope.modalMail.remove();
        });

        $scope.closeModal = () => {
            hidePrintersModal();
        };

        $scope.listPrinters = () => {
            PrinterService.search({
                connection_status: 'ONLINE'
            }).then(printers => {
                $ionicLoading.hide();
                $scope.loadingState = false;

                $scope.pluginPrinters = [];
                $scope.wsPrinters = [];

                const supportedPrinters = printers.filter(printer => printerFilter(printer, printContents));

                const pluginPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_PLUGIN);
                const wsPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_WS);

                if (supportedPrinters.length == 1) {
                    $scope.printingDoc = printContents;
                    $scope.printersSearchResult = supportedPrinters;
                    $scope.pluginPrinters = pluginPrinters;
                    $scope.wsPrinters = wsPrinters;
                    $scope.print(supportedPrinters[0], $scope.voucher);
                } else if (supportedPrinters.length > 0) {
                    $scope.printingDoc = printContents;
                    $scope.printersSearchResult = supportedPrinters;
                    $scope.pluginPrinters = pluginPrinters;
                    $scope.wsPrinters = wsPrinters;
                    showPrintersModal();
                } else {
                    AlertService.error('Nenhuma impressora disponível para o documento escolhido!');
                }
            }).catch(error => {
                $ionicLoading.hide();
                AlertService.error('Nenhuma impressora disponível');
            });
        };

        function printerFilter(printer, printContents) {
            const length = printContents.length;

            for (let i = 0; i < length; ++i) {
                if (printer.paperType === printContents[i].paperType) {
                    return true;
                }
            }
            return false;
        }

        $scope.print = function(printer) {
            $ionicLoading.show({
                template : 'Imprimindo...',
                duration : 0
            }).then(() => {
                const paperType = printer.paperType;
                const printContent = printContents.find(content => {
                    return paperType === content.paperType;
                });

                const params = {
                    'printerid':    printer.id,
                    'contentType':  printer.paperType === 'A_SERIES' ? 'application/pdf' 
                                                            : 'application/vnd.linx.printer-text',
                    'content':      printContent.text,
                    'title':        'Doc',
                    'terminal':     ConfigService.config.terminalNumber,
                    'origin':       printer.origin,
                    'paperType':    printer.paperType
                };
                return doPrint(params, printer);
            });
        };

        function doPrint(params, printer) {
            return PrinterService.submit(params)
                .then(() => {
                    return $ionicLoading.hide().then(() => {
                        $scope.loadingState = false;
                    });
                }).then(() => {
                    return AlertService.info(`Documento impresso na impressora ${printer.name}`, 'OK');
                }).then(() => {
                    return hidePrintersModal();
                }).then(() => {
                    afterPrint();
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
                        $scope.loadingState = false;
                    }).then(() => {
                        AlertService.restError(error, 'Erro no envio da impressão');
                    });
                });
        }

        function afterPrint() {
            if ($scope.redirectsWhenPrinting) {
                $scope.nextPage();
            }
        }
        
        $scope.doSendMailAction = function() {
            if ($scope.sendMailEnabled) {
                $scope.attachmentContent = [];
                $scope.mails = [];
                $scope.textMail = {
                    value: ""
                };
                var customer = CartService.cart.customer;
                
                if (customer && customer.email){
                    $scope.mails.push(customer.email);
                }
                var doc = $scope.pdf;
                
                if (doc instanceof PDF || doc instanceof IncompletePDF) {
                    $scope.attachmentContent.push(new Attachment(doc.base64, $scope.documentTitle + ".pdf"));
                }
                $scope.openModalMail();
            }
        };
        
        $scope.addMail = function(mail){
            var validador = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/gi;
            
            if (validador.test(mail)) {
                if (!$scope.mails.includes(mail)) {
                    $scope.mails.push(mail);
                     $scope.textMail.value = "";
                } else {
                    AlertService.error("Endereço de email já inserido!");
                }
            } else {
                AlertService.error("Endereço de email inválido!");
            }
        };

        $scope.removeMail = function(mail){
            var index = $scope.mails.indexOf(mail);
            if (index !== -1) $scope.mails.splice(index, 1);
        };

        $scope.sendMail = function() {
            $scope.useDestinationInputValue();
            if ($scope.mails && $scope.mails.length > 0) {
                $ionicLoading.show({
                    template : 'Aguarde... Enviando comprovantes',
                    duration : 0
                }).then(function() {
                    const mailRequest = new MailRequest();

                    if ($scope.nfceAuthResponse) {
                        mailRequest.type = MailRequest.TYPES.NFCE;

                        let document = $scope.nfceAuthResponse.nfceResponse;
                        mailRequest.protocolNumber = document.nProt;
                        mailRequest.docURL = document.xUrlDanfe;
                        mailRequest.number = document.numeroNotaAutorizada;
                        mailRequest.docDate = document.dhRetorno;

                        mailRequest.terminalNumber = ConfigService.config.terminalNumber;
                        mailRequest.nsu = CartService.cart.nsu;
                        mailRequest.storeNumber = ConfigService.config.storeNumber;
                    } else {
                        mailRequest.type = MailRequest.TYPES.SERVICE;
                    }
                    mailRequest.attachmentContent = $scope.attachmentContent;
                    mailRequest.destinations = $scope.mails;

                    MailService.send(mailRequest, () => {
                        $ionicLoading.hide();
                        AlertService.info('Email enviado!').then(() => {
                            $scope.closeModalMail().then($scope.nextPage);
                        });
                    }, error => {
                        LogService.error(error);
                        $ionicLoading.hide();
                        AlertService.info('Ocorreu um erro ao tentar enviar comprovante por email!');
                    });
                }, error => {
                    LogService.error(error);
                    $ionicLoading.hide();
                    AlertService.info('Ocorreu um erro ao tentar enviar comprovante por email!');
                });
            } else {
                AlertService.error('Informe ao menos um e-mail válido para o envio');
            }
        };
        
        $scope.useDestinationInputValue = function() {
            if ($scope.textMail.value && $scope.textMail.value !== "") {
                LogService.info("Adicionando e-mail informado no input à lista de destino");
                $scope.addMail($scope.textMail.value);
            }
        }

        $scope.nextPage = () => {
            ConfigService.nsuControll.incrementNSU();
            CartService.initCart();
            CartService.reloadNSU();
            UtilService.navigateToMainScreen();
        };
    })
})();