(function () {
    'use strict';
    angular.module('easyorder.controllers.covenant', [
            'easyorder.components.accordion',
            'easyorder.services.covenant',
            'easyorder.services.cart'
    ])
    .controller('covenantCtrl', function ($scope, 
                                            $rootScope,
                                            $ionicPopup,
                                            CovenantService,
                                            $ionicLoading,
                                            $stateParams,
                                            $timeout,
                                            ConfigService,
                                            UtilService,
                                            CartService) {
        var searchTimeoutPromise;
        var searchTimeout = undefined;
       
        /**
         * 
         */
        $scope.error = function (err) {
            $scope.listAllCovenant.length = 0;
            $ionicLoading.hide();
            
            if (err.status !== 404) {
                $ionicPopup.alert({
                    title: 'Erro!',
                    template: err.data.message,
                    buttons:[ { text: 'Ok', type: 'button button-theme' }]
                });
            }
            $scope.$broadcast('scroll.infiniteScrollComplete');
        };
        
        /**
         * 
         */
        $scope.onInfinityError = function (err) {
            if (err.status !== 404) {
                $scope.error(err);
            } else {
                $scope.$broadcast('scroll.infiniteScrollReachedOut');
            }
            $scope.$broadcast('scroll.infiniteScrollComplete');
        };
        
        /**
         * 
         */
        $scope.success = function (response) {
            $scope.$broadcast('scroll.infiniteScrollFeeding');
            
            updateAllCovenantList(response.result);
            
            if (response.result.length < CovenantService.pageSize) {
                $scope.$broadcast('scroll.infiniteScrollReachedOut');
            }
            $scope.$broadcast('scroll.infiniteScrollComplete');
            $ionicLoading.hide();
        };
        
        /**
         * 
         */
        $scope.onClickCovenant = function (covenant) {
            UtilService.navigateWithoutHistory("easyorder.covenantidentification", {"covenant": covenant});
        };
        
        /**
         * 
         */
        $scope.removeCovenant = function () {
            CovenantService.clearCovenant();
            $scope.selectedCovenant = CovenantService.currentCovenant();
            CartService.cart.fiscalItems.forEach(removeDiscounts);
        }

        /**
         * 
         * Remove os descontos para cada item e atualiza valor deste sem desconto
         */
        function removeDiscounts(item) {
            item.discounts = [];
            item.processTotalValue();
        }
        
        /**
         * 
         */
        var updateAllCovenantList = function (inputData) {
            angular.forEach(inputData, function (covenant) {
                $scope.listAllCovenant.push(covenant);
            });
        };
        
        /**
         * 
         */
        $scope.searchCovenants = function (page, sort, filter, success, error) {
            page = page || 1; 
            let successFn = success || $scope.success;
            let errorFn = error || $scope.error;
            
            return CovenantService.searchCovenants(page, sort, filter, successFn, errorFn);
        };
        
        /**
         * 
         */
        $scope.nextDataPage = function() {
            $scope.dataPageIndex++;
            $scope.searchCovenants($scope.dataPageIndex, undefined, $scope.filterCovenant, undefined, $scope.onInfinityError);
        };
        
        /**
         * Observer para mudança do valor dos campos de filtro de convênios.
         */
        $scope.$watchCollection('[filterCovenant.description, filterCovenant.code]', function (newValues, oldValues, events) {
            $timeout.cancel(searchTimeoutPromise);
            
            searchTimeoutPromise = $timeout(function() { 
                document.activeElement.blur(); //Removendo o foco do campo ativo no momento da consulta. faz o teclado ser recolhido.
                $scope.listAllCovenant.length = 0;
                $scope.dataPageIndex = 1;
                $scope.searchCovenants($scope.dataPageIndex, null, $scope.filterCovenant);
            }, searchTimeout || 0);
            searchTimeout = 800; //Tempo até consulta ser feita após a pausa na digitação.
        });

        (function () {
            $scope.selectedCovenant = CovenantService.currentCovenant();
            $scope.productCodeType = ConfigService.config.business.product_code_character_type;
            $scope.listMyCovenant = [];
            $scope.listAllCovenant = [];
            $scope.dataPageIndex = 1;
            $scope.expandedMyCovenants = false;
            $scope.expandedAllCovenants = true;
            $scope.filterCovenant = {};

            if (CartService.cart.customer) {
                $scope.expandedMyCovenants = true;
                $scope.expandedAllCovenants = false;

                if (CartService.cart.customer.covenants) {
                    $scope.listMyCovenant = CartService.cart.customer.covenants;
                }
            }
            $ionicLoading.show({
                template: 'Por favor, aguarde...',
                duration: 0
            });
        })();
    });
})();
