(function() {
    'use strict';

    angular.module('easyorder.controllers.customer', [
        'easyorder.services.util',
        'easyorder.services.customer',
        'easyorder.services.config',
        'easyorder.services.alert',
        'easyorder.services.user',
        'easyorder.components.item.list',
        'easyorder.services.log',
        'easyorder.services.voucherservice',
        'easyorder.services.loyalty',
        'otpInputDirective'
    ])

    .controller('CustomerCtrl', function($scope,
                                        $rootScope,
                                        $q,
                                        $ionicLoading,
                                        $state,
                                        $timeout,
                                        $stateParams,
                                        $ionicPopup,
                                        CartService,
                                        LoyaltyService,
                                        UtilService,
                                        CustomerService,
                                        BarcodeService,
                                        ConfigService,
                                        AlertService,
                                        UserService,
                                        EFTService,
                                        LogService,
                                        CashbackService,
                                        VoucherService) {

        /*
         * caso o callback $scope.paymentdone for chamado não podemos permitir voltar
         */
        let done = false;
        let goingBack = false;
        $scope.isLoyaltyParticipant = false;
        $scope.marketingFields = {
            email: {
                enabled: false,
                checkboxes: null,
                emptyFn: () => isEmailEmpty()
            },
            phone: {
                enabled: false,
                checkboxes: null,
                emptyFn: () => isPhoneEmpty()
            }
        }

        /**
         *  formato:
         *  {
         *      email: ['email'],
         *      phone: ['phoneCall|sms,whatsapp']
         *  };
         *
         **/
        function parseMarketingFieldsConfiguration(configuration) {
            const ret = {};
            if (isEmpty(configuration)) {
                return ret;
            }
            const fields = configuration.split(';');
            for (const field of fields) {
                const index = field.indexOf(':');
                // configuração inválida, não está no formato
                // correto chave:campo<, ou |>campo
                if (index < 0) {
                    continue;
                }
                const fieldKey = field.substring(0, index);
                // chave não suportada, deve ser email ou phone
                if (!isMarketingFieldKeyValid(fieldKey)) {
                    continue;
                }
                let subFields = field.substring(index + 1, field.length);
                // campo não pode ser vazio
                if (isEmpty(subFields)) {
                    continue;
                }
                const parsedSubFields = [];
                subFields = subFields.split(',');
                for (const subField of subFields) {
                    const f = subField.split('|');
                    const parsedSubField = f.reduce((acc, val, i) => {
                        if (!isMarketingSubFieldValid(val)) {
                            return acc;
                        }
                        if (i > 0) {
                            acc += '|';
                        }
                        acc += parseSubFieldValue(val);
                        return acc;
                    }, '');
                    if (isEmpty(parsedSubField)) {
                        continue;
                    }
                    parsedSubFields.push(parsedSubField);
                }
                if (isEmpty(parsedSubFields)) {
                    continue;
                }
                ret[fieldKey] = parsedSubFields;
            }
            return ret;
        }

        function isMarketingFieldKeyValid(fieldKey) {
            return !isEmpty(fieldKey) && (
                fieldKey == 'email' ||
                fieldKey == 'phone'
            );
        }

        function isMarketingSubFieldValid(subField) {
            return !isEmpty(subField) && (
                subField == 'phonecall' ||
                subField == 'sms' ||
                subField == 'whatsapp' ||
                subField == 'email'
            );
        }

        function parseSubFieldValue(subField) {
            switch (subField) {
                case 'phonecall':
                    return 'phoneCall';
                case 'whatsapp':
                case 'sms':
                case 'email':
                    return subField;
                default:
                    // não é pra chegar até aqui nunca
                    throw new Error('invalid subField value');
            }
        }

        function configureMarketingFields() {
            const configuration = parseMarketingFieldsConfiguration(
                ConfigService.config.business.customer_data_marketing_fields
            );
            for (const key in configuration) {
                switch (key) {
                    case 'email':
                        configureMarketingField('email', 'customer.email', configuration.email);
                        break;
                    case 'phone':
                        configureMarketingField('phone', 'customer.address.phone', configuration.phone);
                        break;
                    default:
                        break;
                }
            }
        }

        function configureMarketingField(field, property, marketingFields) {
            $scope.$watch(property, (newValue, oldValue) => {
                // habilita ou desabilita os checkboxes de marketing
                $scope.marketingFields[field].enabled =
                    !$scope.marketingFields[field].emptyFn() || !isEmpty(newValue);
                // seta o valor das preferências para falso se os checkboxes estiverem desabilitados
                if (!$scope.marketingFields[field].enabled) {
                    fillMarketingPreferences($scope.customer);
                    marketingFields.forEach(marketingField => {
                        const subFields = marketingField.split('|');
                        subFields.forEach(subField => {
                            $scope.customer.marketingPreferences[subField] = false;
                        });
                    });
                }
            });
            // checando checkboxes que são concatenados, para alterar o valor da preferência do cliente,
            // de acordo com o primeiro
            marketingFields.forEach(marketingField => {
                const subFields = marketingField.split('|');
                if (subFields.length > 1) {
                    $scope.$watch('customer.marketingPreferences.' + subFields[0], (newValue, oldValue) => {
                        for (let i = 1; i < subFields.length; i++) {
                            $scope.customer.marketingPreferences[subFields[i]] = newValue;
                        }
                    });
                }
            });
            $scope.marketingFields[field].checkboxes = marketingFields.map(marketingField => {
                return {
                    label: parseMarketingLabel(marketingField),
                    model: marketingField.split('|')[0]
                };
            });
        }

        function parseMarketingLabel(marketingField) {
            return marketingField.split('|').reduce((acc, val, i) => {
                if (i != 0) {
                    acc += ' / ';
                }
                switch (val) {
                    case 'email':
                        acc += 'E-mails';
                        break;
                    case 'phoneCall':
                        acc += 'ligações';
                        break;
                    case 'sms':
                        acc += 'SMS';
                        break;
                    case 'whatsapp':
                        acc += 'WhatsApp';
                        break;
                    default:
                        break;
                }
                return acc;
            }, 'Deseja receber ') + '?';
        };

        if ($rootScope.showMarketingFields) {
            configureMarketingFields();
        }

        $scope.documentInvalid = true;
        $scope.cpfBarcode  = ConfigService.config.business.customer_identification_types;

        $scope.initCPFSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.customercpfsearch');
        };

        $scope.initChipSearch = function() {
            goToChipSearch();
        };

        $scope.initBarcodeSearch = function() {
            goToBarcodeScanner();
        };

        $scope.initCardNumberSearch = function() {
            goToTypedDocument();
        };

        $scope.initRegistrationNumberSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.customerregistrationnumbersearch');
        };

        $scope.initEmailSearch = function() {
        	UtilService.navigateWithoutHistory('easyorder.customeremailsearch');
        };

        $scope.initDirectIdentification = function() {
            UtilService.navigateWithoutHistory('easyorder.customer');
        };

        $scope.initDirectProductSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.productsearch');
        };

        function isEmpty(str) {
            return str == null || str.length == 0;
        }

        function isPhoneEmpty() {
            return $scope.customer.address != null ? isEmpty($scope.customer.address.phone) : true;
        }

        function isEmailEmpty() {
            return isEmpty($scope.customer.email);
        }

        function isAddressStateEmpty() {
            return $scope.customer.address != null ? isEmpty($scope.customer.address.state) : true;
        }

        function fillCustomerProperties(customer) {
            customer.documentType = customer.documentType || "CPF";
            fillMarketingPreferences(customer);
            return customer;
        }

        function fillMarketingPreferences(customer) {
            if (!$rootScope.showMarketingFields) {
                return customer;
            }
            customer.marketingPreferences = customer.marketingPreferences || {
                phoneCall: false,
                sms: false,
                email: false,
                whatsapp: false
            }
            return customer;
        }

        $scope.$on('$destroy', () => {
            if (window.bluebirdBarcodeScanner) {
                bluebirdBarcodeScanner.unregister(() => console.log('BlueBird => unregister'), (argument) => {
                    LogService.info("Erro ao remover callback do pluign do scanner: " + argument);
                });
            }
        });

        function initBarcodeScanner() {
            if ($state.is('easyorder.customerbarcodesearch')) {
                if($scope.scanning_method !== "CAMERA"){
                    if (window.bluebirdBarcodeScanner) {
                        bluebirdBarcodeScanner.register((barcode) => {
                            $scope.doDefaultCustomerSearch({'searchType' : 'CPF', 'searchValue' : barcode});
                        }, (argument) => {
                            LogService.info("Erro ao registrar Scanner: " + argument);
                        });
                    }
                } else {
                    BarcodeService.init(function success(session) {
                        if($scope.cpfBarcode.includes("SCANNED_CPF")){
                            BarcodeService.cancel();
                            $scope.doDefaultCustomerSearch({'searchType' : 'CPF', 'searchValue' : session[0]});
                        }else{
                            Promise.resolve(BarcodeService.cancel())
                            .then(() => new BarcodeLoadingHelper(() => CustomerService.searchByType(session[0], 'SCANNED_CARD'), $scope))
                            .then(checkSelfService)
                            .then(resp => showCustomers(resp, resp.document, 'CPF'))
                            .catch(error => {
                                onCustomerSearchError(error)
                                .then(() => initBarcodeScanner());
                            });
                        }
                    }, { inSizeCam: ['1/136/1/130', '2/116/2/110'], outSizeCam: ['275/136/0/130', '275/116/0/110'] });
                }
            }

            listenCT60Scanner();
        };

        /**
         *
         */
        $scope.doDefaultCustomerSearch = function(paramsCpfBarcode) {
            $ionicLoading.show({
               template : 'Por favor, aguarde...',
               duration : 0
           }).then(function() {
                const provider = ConfigService.config.business.customer_search_provider;

                const error = function(error) {
                    $scope.loadingState = false;
                    $scope.customerRetrieved = false;
                    $ionicLoading.hide();

                    if (ConfigService.config.business.enable_customer_crud_by_rest_api) {
                        if (provider == 'LREN' && error.data && error.data.message && error.data.message.contains('Inexistente')) {
                            return AlertService.info(
                                'Cliente não encontrado, realize o cadastro do novo cliente.',
                                'Confirmar'
                            ).then(() => {
                                UtilService.navigateWithoutHistory('easyorder.customer', {
                                    autoSearchByCustomerDocument : true,
                                    customerToEdit : {
                                        documentType : (!$scope.search.searchType) ? 'CPF' : $scope.search.searchType,
                                        document: $scope.search.value
                                    }
                                });
                            });
                        }
                        if (provider == 'LEBES') {
                            return AlertService.restError(error, 'Serviço indisponível');
                        }
                    }

                    return AlertService.restError(
                        error, 'Erro consultando cliente!'
                    ).then(() => {
                        goToHomeCustomerMenu();
                    });
               }

               var params = (!paramsCpfBarcode) ? {
                   'searchType' : (!$scope.search.searchType) ? 'CPF' : $scope.search.searchType,
                   'searchValue' : $scope.search.value
               } : paramsCpfBarcode;

               CustomerService.search(params, function(result) {
                   var customer = result && result.items && result.items.length > 0 ? result.items[0] : null;

                   customer.documentType = params.searchType;

                   if (customer) {
                        $ionicLoading.hide();

                        if (isUserDoingSelfService(customer)) {
                            error({
                                data : {
                                    message : 'Proibido auto serviço!'
                                }
                            });
                       } else {
                           showCustomerDetails(result.items[0]);
                       }
                   } else {
                        error({
                            data : {
                                message : 'Nenhum cliente foi encontrado com o documento informado'
                            }
                        });
                   }
               }, error);
           });
        };

        $scope.doSearchByType = (value, type) => {
            searchByType(value, type);
        };

        function searchByType(value, type) {
            return new IonicLoadingHelper(() => CustomerService.searchByType(value, type), $ionicLoading)
            .then(checkSelfService)
            .then(resp => showCustomers(resp, value, type))
            .then(onCustomerConfirmed)
            .catch(error => {
                onCustomerSearchError(error)
                .then(goToHomeCustomerMenu);
            })
            .finally(listenCT60Scanner());
        }

        function showCustomers(customers, value, type) {
            const length = customers.length;

            if (length === 1) {
                return showCustomerDetails(customers[0]);
            } else if (length > 1) {
                return showMultipleCustomers(customers);
            } else {
                if(ConfigService.config.business.enable_customer_crud_by_rest_api) {
                    AlertService.info(type !== undefined ? (type, ' não encontrado, continue o cadastro do novo cliente.') : 'Cliente não encontrado, continue o cadastro do novo cliente.');
                    $scope.customer = fillCustomerProperties({ type : value});
                } else {
                    return Promise.reject(new CustomerNotFoundError('No customer found'));
                }
            }
        }

        function showMultipleCustomers(customers) {
            if (!customers) {
                throw new Error('customers can\'t be undefined');
            }
            customers.forEach(fillCustomerProperties);
            $scope.customers = customers;
            $scope.lastClickedCustomer = null;
            $scope.onCustomerClick = customer => {
                if ($scope.lastClickedCustomer) {
                    $scope.lastClickedCustomer.selected = false; // unselects the last clicked customer
                }
                customer.selected = true; // selects the customer
                $scope.lastClickedCustomer = customer;
            };
            return $ionicPopup.confirm({
                title: 'Selecione o cliente',
                template: '<item-list item-template="\'CUSTOMER\'" items=customers on-item-click="onCustomerClick(item)"></item-list>',
                scope: $scope,
                buttons: [
                    { text: '<b>Cancelar</b>', type: 'button-stable' },
                    {
                        text: '<b>Selecionar</b>', type: 'button-theme',
                        onTap: function(event) {
                            const customer = this.scope.$parent.lastClickedCustomer;

                            if (customer) {
                                delete customer.selected; // we don't want this attr out of this scope
                                return customer;
                            }
                            AlertService.error('Nenhum cliente selecionado');
                            event.preventDefault();
                        }
                    }
                ]
            });
        }

        function checkSelfService(customer) {
            if (isUserDoingSelfService(customer)) {
                return Promise.reject(new SelfServiceNotAllowedError('Proibido auto serviço!'));
            }
            return customer;
        }

        function isUserDoingSelfService(customer) {
            var ret = false;
            const user = UserService.currentUser;

            if (Array.isArray(customer)) {
                if (customer.length == 1) {
                    customer = customer[0];
                } else {
                    // se tiver mais de um cliente não tem como validar aqui
                    return false;
                }
            }

            if (user && user.document && customer && customer.document &&
                user.document === customer.document) {
                ret = true;
            }

            return ret;
        }

        function onCustomerConfirmed(customer) {
            if (customer) {
                CartService.cart.customer = customer;
                if(ConfigService.config.business.loyalty_enabled) {
                    CartService.cart.customer.loyaltyParticipant = $scope.isLoyaltyParticipant;
                }

                if (ConfigService.config.business.enable_customer_crud_by_rest_api && !CartService.hasItems()) {
                    UtilService.navigateToMainSearchScreen();
                } else if (CartService.hasItems()) {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                } else {
                    goHome();
                }
            }
        }

        function saveCustomer(customer) {
            customer = JSON.parse(JSON.stringify(customer));
            customer.storeCode = ConfigService.config.storeNumber;
            if (ConfigService.config.business.customer_search_provider) {
                customer.provider = ConfigService.config.business.customer_search_provider;
            }
            new IonicLoadingHelper(() => CustomerService.save(customer), $ionicLoading)
                .then(() => AlertService.info('Cliente salvo com sucesso'))
                .catch(onCustomerSaveError)
                .then(()=> cashbackSearch($scope.customer))
                .then(() => {
                    var customerDiscounts = $scope.customerDiscounts;
                    if (customerDiscounts > 0) {
                        if ($scope.sourceDiscount == 'EMPLOYEE') {
                            $scope.customer.employeeDiscountPercentage = customerDiscounts;
                        }
                    }
                })
                .finally(() => onCustomerConfirmed($scope.customer));
        }

        function updateCustomer(customer) {
            customer = JSON.parse(JSON.stringify(customer));
            customer.storeCode = ConfigService.config.storeNumber;
            if (ConfigService.config.business.customer_search_provider) {
                customer.provider = ConfigService.config.business.customer_search_provider;
            }
            new IonicLoadingHelper(() => CustomerService.update(customer), $ionicLoading)
                .then(() => AlertService.info('Cliente salvo com sucesso'))
                .catch(onCustomerSaveError)
                .then(() => {
                    var customerDiscounts = $scope.customerDiscounts;
                    if (customerDiscounts > 0) {
                        if ($scope.sourceDiscount == 'EMPLOYEE') {
                            $scope.customer.employeeDiscountPercentage = customerDiscounts;
                        }
                    }
                })
                .finally(() => onCustomerConfirmed($scope.customer));
        }

        function onCustomerSaveError(error) {
            if (error instanceof SelfServiceNotAllowedError) {
                $scope.customer = fillCustomerProperties({});
                CartService.cart.customer = null;
                return AlertService.error('Proibido auto serviço!');
            } else {
                CartService.cart.customer = $scope.customer;
                return AlertService.error('Erro ao cadastrar/atualizar cliente.');
            }
        }

        function onCustomerSearchError(error) {
            $scope.customerRetrieved = false;
            if (error instanceof CustomerNotFoundError) {
                return AlertService.error('Nenhum cliente encontrado com o documento informado');
            } else if (error instanceof SelfServiceNotAllowedError) {
                $scope.customer = fillCustomerProperties({});
                return AlertService.error('Proibido auto serviço!');
            } else {
                return AlertService.restError(error, 'Falha ao consultar cliente');
            }
        }

        /**
         * Inicializa a consulta do cliente através da leitura do cartãoo do cliente.
         */
        $scope.doChipSearch = function() {
            EFTService.executeTransaction(
                {
                    transactionType:    EFTService.transactionType.TRAIL_READ,
                    amount:            0
                }, {
                    paymentDone: function(paymentResults) {
                        done = true;

                        if (paymentResults && paymentResults.Trilha2) {
                            const type  = paymentResults.Trilha2.indexOf('=') > 0? 'CHIP': 'READ_CARD';
                            $scope.search.value = paymentResults.Trilha2.split('=')[0];
                            this.clearDisplay();

                            new IonicLoadingHelper(() => CustomerService.searchByType($scope.search.value, type), $ionicLoading)
                            .then(checkSelfService)
                            .then(resp => showCustomers(resp, $scope.search.value, type))
                            .then(onCustomerConfirmed)
                            .catch(error => {
                                onCustomerSearchError(error)
                                .then(goToHomeCustomerMenu);
                            });
                        } else {
                            AlertService.error("Erro ao obter dados do cartão. Retorno do pinpad inválido.").then(function() {
                                goToHomeCustomerMenu();
                            });
                        }
                    },
                    paymentError: function(paymentResults) {
                        var errMsg = paymentResults && paymentResults.MensagemErro ? paymentResults.MensagemErro : "Não foi possível ler cartão.";

                        AlertService.error(errMsg).then(function() {
                            goToHomeCustomerMenu();
                        });
                    },
                    clearDisplay: function() {
                        $timeout(function () {
                            $scope.message = "";
                        });
                        LogService.info("clearDisplay");
                    },
                    displayMessage: function(message) {
                        if (message) {
                            $timeout(function () {
                                $scope.message = message;
                            });
                        }
                        LogService.info("displayMessage: " + $scope.message);
                    },
                    selectOptionExtended: function (title, options, responseHandlerExtended) {
                        $scope.$apply(function () {
                            $scope.message = title;
                        });
                        $scope.processOptions(options, responseHandlerExtended);
                        LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
                    }
                }
            );
        };

        /**
         *
         */
        $scope.goBackChip = () => {
            if ($scope.isBackButtonDisabled()) {
                return Promise.resolve();
            }
            goingBack = true;
            $scope.message = "Cancelando operação...";

            return EFTService.finalizeTransaction()
            .catch(() => {
                $scope.message = "Não foi possível cancelar operação";
                goingBack = false
            });
        };

        /**
         *
         */
        $scope.isBackButtonDisabled = () => {
            return done || goingBack;
        };

        $scope.isEFTBackButtonVisible = () => {
            return ConfigService.config.tefserver.provider === "SoftwareExpress";
        };

        /**
        *
        */
       $scope.processOptions = function (options, responseHandler) { //(1,"Magnetico/Chip")#(2,"Digitado")
           $scope.options.length = 0;
           $scope.showWatermark = false;

           var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
           splitedOpt = splitedOpt.split("#"); // Separando as opções.

           for (var i = 0; i < splitedOpt.length; i++) {
               var split = splitedOpt[i].split(",");

               $scope.options.push({
                   id: split[0].trim(),
                   label: split[1].trim().replace(/\"/g, ""),
                   value: split[split.length - 1],
                   handler: responseHandler,
                   type: "single-option"
               });
               $scope.$apply();
           }
           $scope.headerOptions = "Opções";
       };

       /**
        * Função executa a função com os parâmetros passados a esta.
        */
       $scope.tickOptionHandler = function(func, params) {
           $scope.options.length = 0;
           $scope.showWatermark = true;
           $scope.message = "Iniciando. Favor aguarde...";

           func.apply(this, params);
       };

      /**
       *
       */
      $scope.tickNumberInputHandler = function(inputOption) {
          var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);
          $scope.message = "Iniciando. Favor aguarde...";
          $scope.tickOptionHandler(inputOption.handler, [value]);
      };

      /**
       *
       */
      $scope.tickStringInputHandler = function(inputOption) {
          const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);
          $scope.message = "Iniciando. Favor aguarde...";
          $scope.tickOptionHandler(inputOption.handler, [value]);
      };

        /**
         *
         */
        var showCustomerDetails = function(customer) {
            CustomerService.showCustomerDetails(customer, 'SELECTION', true).then(async function(tapResult) {
                if (tapResult) {
                    $scope.customer = customer;
                    CartService.cart.customer = customer;
                    await cashbackSearch(customer);
                    await checkVoucherPromoBalance(customer);

                    checkLoyalt()
                        .finally(() => {
                            const authMethods = ConfigService.config.business.payment_auth_methods;
                            let faceDetectionEnabled = authMethods && Object.entries(authMethods).some(item => item[1] && item[1] == "FACE_DETECTION");
                            if (ConfigService.config.business.enable_customer_crud_by_rest_api && !CartService.hasItems()) {
                                if (customer.faceBiometryStatus && customer.faceBiometryStatus != "OFFLINE") {
                                    if (customer.faceBiometryStatus == "INACTIVE" || customer.faceBiometryStatus == "NO_DATA") {
                                     AlertService.info('Cliente não possui biometria facial. Para pagamento com CCR ou Meu Cartão, direcione-o para cadastro!');
                                      }
                                    }
                                if (faceDetectionEnabled && customer.faceBiometryStatus && customer.faceBiometryStatus == "OFFLINE"){
                                     AlertService.info('Serviço de reconhecimento facial indisponível. Pagamentos com CCR ou Meu cartão serão realizados através da senha do cliente.');
                                }
                                return UtilService.navigateToMainSearchScreen();
                            } else if (!CartService.hasItems()) {
                                return UtilService.navigateWithoutHistory('easyorder.cart');
                            } else {
                                if (customer.faceBiometryStatus && customer.faceBiometryStatus != "OFFLINE") {
                                    if (customer.faceBiometryStatus == "INACTIVE" || customer.faceBiometryStatus == "NO_DATA") {
                                     AlertService.info('Cliente não possui biometria facial. Para pagamento com CCR ou Meu Cartão, direcione-o para cadastro!');
                                       }
                                 }
                                if (faceDetectionEnabled && customer.faceBiometryStatus && customer.faceBiometryStatus == "OFFLINE"){
                                     AlertService.info('Serviço de reconhecimento facial indisponível. Pagamentos com CCR ou Meu cartão serão realizados através da senha do cliente.');
                                }
                                goHome()
                            }
                        });
                } else {
                    goToHomeCustomerMenu();
                }
                $scope.search = {};
            });
        };

        /**
         *
         * @param {*} customer
         */
        const cashbackSearch = (customer) => {
            const cashbackPaymentMethods = ConfigService.config.business.payment_methods_with_cashback;
            if(!cashbackPaymentMethods || cashbackPaymentMethods.length === 0 || !customer) {
                return;
            }

            $ionicLoading.show({
                template : 'Buscando saldo Cashback...',
                duration : 0
            })
            const request = new CashbackInfoRequest(customer.documentType, customer.document, 0);

            return CashbackService.getCashbackInfo(request).then((resp) => {
                $ionicLoading.hide()
                return AlertService.info(`Valor cashback: R$ ${parseFloat(resp.availableValue).toFixed(2)}`);
            }).catch((error) => {
                $ionicLoading.hide()
                if (error.status === 500) {
                    return AlertService.error('Não foi possível consultar o saldo cashback.');
                }
            })
        }

        const checkVoucherPromoBalance = (customer) => {
            if (!customer.document || !ConfigService.config.business.promo_voucher_balance_enabled) {
                return;
            }

            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            })
            LogService.info('Buscando saldo dos vouchers promo ...')

            return VoucherService.balance(
                ConfigService.nsuControll.currentNSU(),
                customer.document,
                ConfigService.config.storeNumber,
                ConfigService.config.business.giftcard_service_provider,
                ConfigService.config.companyNumber,
                ConfigService.config.storeNumber
            ).then((resp) => {
                $ionicLoading.hide();
                const totalBalance = resp.vouchers.map(voucher => voucher.value).reduce((sum, v) => sum + v, 0);
                CartService.cart.customer.vouchersBalance = totalBalance;
                LogService.info(`Valor vouchers: R$ ${totalBalance}`);
            }).catch(error => {
                $ionicLoading.hide()
                LogService.error(error);
            });
        }

        /**
         *
         */
        var error = function(error) {
            $scope.customerRetrieved = false;
            $scope.customer = fillCustomerProperties({
                document: $scope.customer.document,
                documentType: $scope.customer.documentType,
                loyaltyParticipant: $scope.isLoyaltyParticipant
            });
            $scope.validateDocument($scope.customer.documentType, true);
            $ionicLoading.hide();
            return AlertService.restError(error, 'Erro consultando clientes no servidor!');
        };

        $scope.searchAndComplete = function() {
            checkLoyalt()
            .finally(() => {
                if(ConfigService.config.business.enable_customer_crud_by_rest_api) {
                    var documentValue = $scope.customer.document;
                    var documentType = $scope.customer.documentType;


                    if($scope.documentInvalid) {
                        $scope.customer = fillCustomerProperties({});
                    } else {
                        //Removendo o foco do campo ativo no momento da consulta, para o teclado ser recolhido.
                        if ("activeElement" in document)
                            document.activeElement.blur();
                        if(ConfigService.config.business.discount_employee_search_enabled && documentType && documentType == "CPF") {
                            $ionicLoading.show({
                                template : 'Buscando desconto de colaborador...',
                                duration : 0
                            });

                            CustomerService.searchDiscount(documentValue, documentType)
                                .then((ret) => {
                                    $ionicLoading.hide();
                                    customerDiscountHandler(ret);
                                })
                                .catch(error)
                                .then(() => searchCustomerByAPIs(documentValue, documentType));
                        } else {
                            searchCustomerByAPIs(documentValue, documentType);
                        }
                    }
                }
            })
        }

        /**
         * Verifica se o cliente faz parte do programa de fidelidade
         */
        function checkLoyalt() {
            if (ConfigService.config.business.loyalty_enabled && $scope.customer.documentType != 'PASSPORT') {
                $ionicLoading.show({
                    template : 'Consultando o programa de fidelidade...',
                    duration : 0
                });

                const participantRequest = new ParticipantRequest();
                participantRequest.documentNumber = $scope.customer.document;
                participantRequest.startDate = new Date();

                return LoyaltyService.getParticipant(participantRequest).finally(() => {
                    $ionicLoading.hide();
                }).then((ret) => {
                    $scope.isLoyaltyParticipant = ret.enrollment || false;
                    if(ret.message) {
                        return AlertService.info(ret.message);
                    } else {
                        return AlertService.error('Serviço indisponível');
                    }
                }).catch(error => {
                    LogService.error(error);
                    return AlertService.error('Serviço indisponível');
                });
            } else {
                $ionicLoading.hide();
                return $q.resolve();
            }
        }

        /**
         * Realiza a busca síncrona dos dados do cliente (Separação criada para atender o fluxo alternativo da RENNER - ObterClient e API REST)
         *
         * @param {Valor do documento} documentValue
         * @param {Tipo do documento} documentType
         */
        function searchCustomerByAPIs(documentValue, documentType) {
            searchCustomerByCommonAPI(documentValue, documentType);
        }

        /**
         * Busca os dados de um cliente baseados na API padrão do LWS
         *
         * @param {Valor do documento} documentValue
         * @param {Tipo do documento} documentType
         */
       function searchCustomerByCommonAPI(documentValue, documentType) {
            $ionicLoading.show({
                template : 'Buscando informações de cliente...',
                duration : 0
            });

            LogService.info("Iniciando busca de dados do cliente...");
            CustomerService.searchByType(documentValue, documentType)
                    .then(resp => {
                        $ionicLoading.hide().then(() => {
                            let customer = (resp.length > 0) ? resp[0] : null;
                            if (customer) {
                                let promise = $q.resolve();
                                const authMethods = ConfigService.config.business.payment_auth_methods;
                                let faceDetectionEnabled = authMethods && Object.entries(authMethods).some(item => item[1] && item[1] == "FACE_DETECTION");
                                if (faceDetectionEnabled && customer.faceBiometryStatus && (customer.faceBiometryStatus == "INACTIVE" || customer.faceBiometryStatus == "NO_DATA")) {
                                    promise.then(() => AlertService.warn('Cliente não possui biometria facial. Para pagamento com CCR ou Meu Cartão, direcione-o para cadastro!'));
                                }
                                if (faceDetectionEnabled && customer.faceBiometryStatus && customer.faceBiometryStatus == "OFFLINE"){
                                    AlertService.info('Serviço de reconhecimento facial indisponível. Pagamentos com CCR ou Meu cartão serão realizados através da senha do cliente.');
                                }
                                if (customer.customerWarnings && customer.customerWarnings.length > 0) {
                                    const warnList = customer.customerWarnings;
                                    for (let i = 0; i < warnList.length; i++) {
                                        promise = promise.then(() => AlertService.warn(warnList[i]));
                                    }
                                }
                            }
                        });
                        formAutoComplete(resp);
                    })
                    .catch(err => {
                        error(err)
                        .then(searchCustomerByRestAPI);
                    });
        }


        /**
        * Devido à mudança estrutural da tela de cliente solicitada pela Renner, foi necessário separar a consulta padrão (ObterCliente) da
        * API Rest que eles utilizam para o Marketing.
        */
        function searchCustomerByRestAPI() {
            if (ConfigService.config.business.enable_customer_crud_by_rest_api && $scope.customer &&  $scope.customer.document && ConfigService.config.business.customer_search_provider === 'LREN') {
                LogService.info("Cliente não encontrado na ObterCliente! Iniciando uma nova busca na API REST...");
                $ionicLoading.show({
                    template : 'Buscando informações de cliente...',
                    duration : 0
                });
                LogService.info("Parâmetro de busca por API REST ativo, iniciando busca...");
                CustomerService.searchByType($scope.customer.document, 'CPF_REST')
                        .then(resp => {
                            $ionicLoading.hide()
                            formAutoComplete(resp);
                        })
                        .catch(err => {
                            error(err)
                            .then(() => {
                                $scope.customer = fillCustomerProperties({
                                    document: $scope.customer.document,
                                    documentType: $scope.customer.documentType,
                                    loyaltyParticipant: $scope.isLoyaltyParticipant
                                });
                            });
                        });
            }
        }

        /**
         * Atualmente é utilizado para buscar o desconto de funcionário Renner
         * @param {Resposta da consulta de descontos de cliente} customerDiscountReponse
         */
        async function customerDiscountHandler(customerDiscountReponse) {
            if(customerDiscountReponse && customerDiscountReponse.discounts.length && customerDiscountReponse.discounts[0].value  > 0){
                $scope.customerDiscounts = customerDiscountReponse.discounts[0].value;
                $scope.sourceDiscount = customerDiscountReponse.discounts[0].source;

                $scope.myPopup = $ionicPopup.show({
                    title: 'Desconto colaborador ' + $scope.customerDiscounts + '%: solicite documento',
                    subTitle: '',
                    scope: $scope,
                    buttons: [ { text: 'Ok', type: 'button button-theme' }]
                });
             }
        }

        function formAutoComplete(customers) {
            if(customers.length > 0 ) {
                $scope.customer = fillCustomerProperties(angular.copy(customers[0]));
                $scope.customerRetrieved = true;
                $scope.formatPhoneNumber();
                return Promise.resolve();
            } else {
                $scope.customerRetrieved = false;
                return Promise.reject(new CustomerNotFoundError('Cliente não encontrado, continue o cadastro do novo cliente.'));
            }
        }

        /**
         *
         */
        $scope.findAddress = function() {
            if($scope.customer.address.zipCode == null) {
                $scope.customerAddressNameLoaded = false;
                $scope.customerNeighborhoodLoaded = false;
                $scope.customerStateLoaded = false;
                $scope.customerCityLoaded = false;
                $scope.customerCountryLoaded = false;

                $scope.zipCodeLenght = 'lessThan-8';// TODO ?
            }
            if ($scope.customer.address.zipCode && $scope.customer.address.zipCode.length == 8 && $scope.customer.address.zipCode !== null) {
                $scope.zipCodeLenght = 'equal-8'; // TODO ?

                $ionicLoading.show({
                    template : 'Buscando endereço por cep...',
                    duration : 0
                }).then(function() {
                    CustomerService.getAddress($scope.customer.address.zipCode, function(result) {
                        $ionicLoading.hide();
                        LogService.info(result);

                        $scope.customer.address.addressName = result.address;
                        $scope.customer.address.neighborhood = result.district;
                        $scope.customer.address.state = result.state;
                        $scope.customer.address.city = result.city;
                        $scope.customer.address.cityCode = result.cityCode;
                        $scope.customer.address.country = result.country;

                        $scope.customer.address.addressName && $scope.customer.address.addressName.length > 0 ? $scope.customerAddressNameLoaded = true : $scope.customerAddressNameLoaded = false;
                        $scope.customer.address.neighborhood && $scope.customer.address.neighborhood.length > 0 ? $scope.customerNeighborhoodLoaded = true : $scope.customerNeighborhoodLoaded = false;
                        $scope.customer.address.state && $scope.customer.address.state.length > 0 ? $scope.customerStateLoaded = true : $scope.customerStateLoaded = false;
                        $scope.customer.address.city && $scope.customer.address.city.length > 0 ? $scope.customerCityLoaded = true: $scope.customerCityLoaded = false;
                        $scope.customerCountryLoaded = (
                            $scope.customer.address.country
                            && $scope.customer.address.country.length > 0
                        );

                    },  function(error) {
                        $ionicLoading.hide();
                        var errMsg = error.data.message ? error.data.message : "Falha ao realizar consulta de CEP";
                        AlertService.warn(errMsg);
                        LogService.warn(error);
                    });
                });
            } else {
                $scope.customerAddressNameLoaded = false;
                $scope.customerNeighborhoodLoaded = false;
                $scope.customerStateLoaded = false;
                $scope.customerCityLoaded = false;

                $scope.zipCodeLenght = 'lessThan-8';// TODO ?
            }
        }

        $scope.formatPhoneNumber = function() {
            if($scope.customer.address && $scope.customer.address.phone) {
                var phoneNumber = $scope.customer.address ? ($scope.customer.address.phone ? $scope.customer.address.phone : '') : '';
                phoneNumber = phoneNumber.replace(/[^0-9]/g, '');
                phoneNumber = phoneNumber.replace(/^(\d{2})*(\d{1})*(\d{4})(\d{4})$/g, "($1)$2$3-$4");
                $scope.customer.address.phone = phoneNumber;
            }
        }

        function phoneCheck() {
            if ($scope.mandatoryPhone) {
                return isValidPhoneNumber();
            }

            return isPhoneNotEmpty() ? isValidPhoneNumber() : true;
        }

        function isValidPhoneNumber() {
            return isPhoneNotEmpty() && $scope.customer.address.phone.length >= 13;
        }


        function isPhoneNotEmpty() {
            return $scope.customer.address !== undefined && $scope.customer.address.phone !== undefined && $scope.customer.address.phone !== '';
        }

        /**
         *
         */
        $scope.validDocumentLength = function (document, documentType) {
            if (documentType && document) {
                if ((documentType === "CPF" && document.length == 11) ||
                    (documentType === "CNPJ" && document.length == 14) ||
                    (documentType === "PASSPORT" && document.length == 9)) {
                    return true;
                }
            }
            return false;
        };

        /**
         *
         */
        $scope.validateDocument = function(documentType, silent) {
            $scope.customer.document = $scope.customer.document || $scope.search.value;
            var document = $scope.customer.document;
            $scope.customerDocumentType = documentType;

            if (document && $scope.validDocumentLength(document, documentType)) {
                if ((documentType === "CPF" && !validaCPF(document))
                        || (documentType === "CNPJ" && !validaCNPJ(document))
                        || (documentType === "PASSPORT" && !validaPassport(document))) {
                    if (!silent) {
                        AlertService.warn($scope.customer.documentType + " inválido");
                    }
                    $scope.customer = fillCustomerProperties({});
                    $scope.search = {};
                    $scope.documentInvalid = true;
                } else {
                    $scope.documentInvalid = isUserDoingSelfService($scope.customer);

                    if ($scope.documentInvalid) {
                        $scope.customer = fillCustomerProperties({});
                        $scope.search = {};
                        AlertService.error('Proibido auto serviço!');
                    }
                }
            } else {
                $scope.documentInvalid = true;
            }
        };

        $scope.resetCustomerFields = function(fields) {
            fields.forEach(k => $scope.customer[k] = '');
        }

        $scope.searchCustomer = function(documentType, silent) {
            $scope.validateDocument(documentType, silent);
            if (!$scope.documentInvalid) {
                $scope.searchAndComplete();
            }
        };


        $scope.checkCustomerIdentified = function() {
            if (!CartService.cart.customer) {
                var confirmPopup = AlertService.confirm('Cliente ainda não identificado, deseja identificar?');

                confirmPopup.then(function(result) {
         			if (result) {
         				CartService.initCart();
         				goToHomeCustomerMenu();
         			}
         		});
            }
        };

        $scope.cancelCustomer = function(){
            $scope.customer = fillCustomerProperties({});
            if(CartService.hasItems()) {
                $rootScope.$broadcast('editCustomerInProgress', false);
                return UtilService.navigateWithoutHistory('easyorder.cart')
            }
            return UtilService.navigateToMainScreen();
        }

        $scope.validateCustomer = function(isValid){
            $scope.isValid = isValid;
            if (ConfigService.config.business.enable_end_user_license_agreement) {
                checkEula();
            } else {
                $scope.identifyCustomer(isValid);
            }
            $rootScope.$broadcast('editCustomerInProgress', false);
        }

        $scope.identifyCustomer = function(isValid){
            const provider = ConfigService.config.business.customer_search_provider;

            if (isValid) {
                var isCustomerSaleAllowed = CustomerService.isCustomerSaleAllowed($scope.customer, ConfigService.config.store);

                if (isCustomerSaleAllowed) {
                    validateStateRegistration()
                        .then(function() {
                            if (ConfigService.config.business.enable_customer_crud_by_rest_api) {
                                let confirmPopup = $q.resolve(true);
                                if (provider == 'LREN' && !$scope.customerRetrieved) {
                                    // Exibe popup de cadastro só no caso dele não ter sido consultado
                                    confirmPopup = AlertService.confirm('Deseja cadastrar o cliente?');
                                }
                                if (provider == 'LEBES') {
                                    confirmPopup = $q.resolve($scope.customer.customerId != null);
                                }
                                confirmPopup.then(async function(result) {
                                    if (result) {
                                        if($scope.customer.address && $scope.customer.address.phone) {
                                            $scope.customer.address.phone = $scope.customer.address.phone.replace(/[^0-9]/g, '')
                                        }
                                        if (provider == 'LREN') {
                                            saveCustomer($scope.customer);
                                        } else if (provider == 'LEBES') {
                                            updateCustomer($scope.customer);
                                        }
                                    } else {
                                        await cashbackSearch($scope.customer);
                                        onCustomerConfirmed($scope.customer);
                                    }
                                });
                            } else {
                                CartService.cart.customer = $scope.customer;
                                cashbackSearch($scope.customer);

                                if (ConfigService.config.business.enable_customer_crud_by_rest_api && !CartService.hasItems()) {
                                    return UtilService.navigateToMainSearchScreen();
                                } else if(CartService.hasItems()){
                                    return UtilService.navigateWithoutHistory('easyorder.cart');
                                } else {
                                    goToHomeCustomerMenu();
                                }
                            }
                        }).catch(function(error) {
                            if (error instanceof CustomerFieldError) {
                                AlertService.warn(error.message);
                            } else {
                                AlertService.error(error.message);
                            }
                        });
                } else {
                    AlertService.warn('Não é permitida a venda para clientes de outros estados!</b>');
                }
            }
        }

        function validateStateRegistration() {
            var defer = $q.defer();
            if ($scope.customer.documentType !== 'CNPJ') {
                delete $scope.customer.stateRegistration;
                defer.resolve(true);
            } else if ($scope.customer.stateRegistration.toUpperCase() === 'ISENTO' || isEmpty($scope.customer.stateRegistration)) {
                $scope.customer.stateRegistration = $scope.customer.stateRegistration.toUpperCase();
                defer.resolve(true);
            } else if (!isAddressStateEmpty() && ValidateStateRegistration.isValid($scope.customer.stateRegistration, $scope.customer.address.state)) {
                defer.resolve(true);
            } else {
                const message = ConfigService.config.business.state_registration_error_msg;
                defer.reject(new CustomerFieldError(message));
            }

            return defer.promise;
        }

        function getFormmatedPhone() {
            let responsePhone = null;
            if (isValidPhoneNumber()) {
                responsePhone = '55' + $scope.customer.address.phone.replace(/\D/g, "");
            }
            return responsePhone;
        }


        const searchEula = () => {
            return new IonicLoadingHelper(() => CustomerService.searchEula({
                documentValue: $scope.customer.document,
                email: $scope.customer.email,
                phone: getFormmatedPhone()
            }), $ionicLoading);
        }

        const confirmEula = (pin) => {
            return new IonicLoadingHelper(() => CustomerService.confirmEula({
                name: $scope.customer.name,
                documentValue: $scope.customer.document,
                email: $scope.customer.email,
                pin: pin
            }), $ionicLoading);
        }

        function checkEula() {
            searchEula()
            .then((result) => {
                const searchEulaResponse = result;
                if (searchEulaResponse) {
                    if (searchEulaResponse.alreadyAccepted) {
                        $scope.identifyCustomer($scope.isValid);
                    } else if (searchEulaResponse.key.type != 'AES' || searchEulaResponse.key.length != 128) {
                        LogService.error("Formato de criptografia não suportado");
                        AlertService.error("Formato de criptografia não suportado");
                    } else {
                        $scope.otpInput = {
                            size: 4,
                            type: "tel",
                            onlyNumber: true
                        };
                        var pinPopup = $ionicPopup.show({
                            templateUrl: 'views/requestPin.html',
                            scope: $scope,
                            buttons: [
                                { text: 'Cancelar' },
                                {
                                    text: '<b>Confirmar</b>',
                                    type: 'button-theme',
                                    onTap: function (event) {
                                        if (checkPin($scope.otpInput.value)) {
                                            const covertedKey = CryptoJS.enc.Base64.parse(searchEulaResponse.key.value);
                                            const encrypted = CryptoJS.AES.encrypt($scope.otpInput.value, covertedKey, {
                                                mode: CryptoJS.mode.ECB,
                                                padding: CryptoJS.pad.Pkcs7,
                                            });

                                            const pin = encrypted.ciphertext.toString(CryptoJS.enc.Base64);

                                            confirmEula(pin).then((response) => {
                                                $scope.customer.pinCode = $scope.otpInput.value;
                                                $scope.customer.pinType = response.pinType;
                                                pinPopup.close();
                                                LogService.info('Confirmação de termo realizada com sucesso.');
                                                $scope.identifyCustomer($scope.isValid);
                                            })
                                            .catch((error) => {
                                                $scope.otpInput.value = "";
                                                LogService.error(error);
                                                AlertService.restError(error, "Erro na confirmação do Termo");
                                            });
                                        }
                                        event.preventDefault();
                                    }
                                }
                            ]
                        });
                    }
                }
            })
            .catch((error) => {
                AlertService.restError(error, 'Erro consultando termo de consentimento do cliente');
            });
        }

        function checkPin(pin) {
            return pin && pin != "" && !isNaN(pin);
        }

        function listenCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            if ($rootScope.ct60ScannerStatus === 'DISABLED') {
                reEnableCT60Scanner();
                return;
            }

            window.plugins.honeywell.nativeListen(ct60Search, LogService.error);
            $rootScope.ct60ScannerStatus = 'LISTENING';
        }


        function ct60Search(data) {
            if (!$state.is('easyorder.customerbarcodesearch')) {
                return;
            }

            disableCT60Scanner();
            $scope.scannedCode = data;
            $scope.doSearchByType($scope.scannedCode, 'SCANNED_CARD');
        }

        function reEnableCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            window.plugins.honeywell.nativeClaim();
            $rootScope.ct60ScannerStatus = 'CLAIMING';
            listenCT60Scanner();
        }

        function disableCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            window.plugins.honeywell.nativeRelease();
            $rootScope.ct60ScannerStatus = 'DISABLED';
        }

        function isCT60() {
            return $scope.isCordova && device.model === 'CT60';
        }

        function goToChipSearch() {
            var history = $stateParams.flowHistory || [];
            history.flow = history.flow || [];
            history.flow.push($state.current.name);

            UtilService.navigateWithoutHistory('easyorder.customerchipsearch', {flowHistory: history});
        };

        function goToTypedDocument() {
            var history = $stateParams.flowHistory || [];
            history.flow = history.flow || [];
            history.flow.push($state.current.name);

            UtilService.navigateWithoutHistory('easyorder.customercardnumbersearch', {flowHistory: history});
        };

        function goToBarcodeScanner() {
            var history = $stateParams.flowHistory || [];
            history.flow = history.flow || [];
            history.flow.push($state.current.name);

            UtilService.navigateWithoutHistory('easyorder.customerbarcodesearch', {flowHistory: history});
        };

        /**
         * Tipos de identificação disponíveis.
         */
        var allTypes = [{
            id: "CPF",
            handler: $scope.initCPFSearch,
            css: "fa fa-id-card",
            description: ConfigService.config.business.enable_more_documents ? "Documentos" : "CPF"
        }, {
            id: "CHIP",
            handler: $scope.initChipSearch,
            css: "fa fa-credit-card",
            description: getIdentificationMethodName("CHIP") || "Leitura de Cartão"
        }, {
            id: "SCANNED_CARD",
            handler: $scope.initBarcodeSearch,
            css: "fa fa-barcode",
            description: getIdentificationMethodName("SCANNED_CARD") || "Código de Barras"
        }, {
            id: "SCANNED_CPF",
            handler: $scope.initBarcodeSearch,
            css: "fa fa-barcode",
            description: getIdentificationMethodName("SCANNED_CPF") || "Código de Barras"
        },{
            id: "TYPED_CARD",
            handler: $scope.initCardNumberSearch,
            css: "far fa-keyboard",
            description: getIdentificationMethodName("TYPED_CARD") || "Número de Cartão"
        }, {
            id: "DIRECT",
            handler: $scope.initDirectIdentification,
            css: "fa fa-user",
            description: getIdentificationMethodName("DIRECT") || "Dados do cliente"
        }, {
            id: "REGISTRATION",
            handler: $scope.initRegistrationNumberSearch,
            css: "far fa-id-card",
            description: getIdentificationMethodName("REGISTRATION") || "Número de Matrícula"
        }, {
            id: "EMAIL",
            handler: $scope.initEmailSearch,
            css: "fa fa-envelope",
            description: getIdentificationMethodName("EMAIL") || "E-mail"
        }];

        function getIdentificationMethodName(id) {
            var names = ConfigService.config.business.customer_identification_names;

            const method = names ? names.find(name => name.id == id) : undefined;
            return method ? method.name : null;
        }

        function goToHomeCustomerMenu(){
            if ($scope.unifiedIdentification) {
               return UtilService.navigateWithoutHistory('easyorder.customer');
            } else {
                return UtilService.navigateWithoutHistory('easyorder.customermenu');
            }
        }

        /**
        *
        */
        var goHome = () => {
            UtilService.navigateToMainScreen();
        };

        /**
         *
         */
        $scope.goBack = () => {
            var promise = Promise.resolve();

            if ($state.is('easyorder.customerchipsearch')) {
                promise = $scope.goBackChip();
            }
            promise.finally(() => UtilService.navigateWithoutHistory($stateParams.flowHistory.flow.pop(), $stateParams));
        };

        /**
         * Função de inicialização.
         */
        (function() {
            $scope.goBackState = $stateParams.flowHistory && $stateParams.flowHistory.flow;
            $scope.showWatermark = true;
            $scope.options = [];
            $scope.message = "";
            $scope.unifiedIdentification = ConfigService.config.business.unified_customer_identification_enabled;
            $scope.customerRetrieved = false;
            if($state.is('easyorder.customermenu') && $scope.unifiedIdentification) {
                return UtilService.navigateWithoutHistory('easyorder.customer');
            }

            $scope.search = {};
            $scope.customer = fillCustomerProperties({});

            $scope.scannedCode = "";
            $scope.acceptsMoreDocuments = ConfigService.config.business.enable_more_documents;
            $scope.scanning_method = ConfigService.config.business.barcode_scanning_method;

            $scope.mandatoryParameterFromServer = ConfigService.config.business.customer_mandatory_fields;
            if ($scope.mandatoryParameterFromServer == "DEFAULT") {
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
            } else if ($scope.mandatoryParameterFromServer == "SET1") {
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryzipCode = true;
                $scope.mandatoryaddressName = true;
                $scope.mandatoryaddressNumber = true;
                $scope.mandatoryneighborhood = true;
                $scope.mandatorycity = true;
                $scope.mandatoryState = true;
                $scope.mandatoryStateRegistration = true;
            } else if ($scope.mandatoryParameterFromServer == "SET2"){
                $scope.mandatoryDocument = true;
                $scope.mandatoryMail = true;
            } else if ($scope.mandatoryParameterFromServer == "SET3"){
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryMail = true;
            } else if ($scope.mandatoryParameterFromServer == "SET4") {
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryMail = true;
                $scope.mandatoryPhone = true;
            } else if ($scope.mandatoryParameterFromServer == "DOCUMENT") {
                $scope.mandatoryDocument = true;
            }

            $scope.disableConfirmButton = () => {
                return $scope.documentInvalid || !phoneCheck();
            };

            initBarcodeScanner(); // Só inicializar no estado correto.

            var identification_types = ConfigService.config.business.customer_identification_types;

            identification_types = identification_types ? identification_types.split(";") : ["DIRECT"];

            $scope.allowedIdentificationTypes = (function(data) {
                var ret = [];
                while (data.length) {
                    var row;
                    ret.push(row = allTypes.filter(function(type) {
                        return type.id === this[0] || type.id === this[1];
                    }, data.splice(0, 2)));
                    row.sort((a, b) => identification_types.indexOf(a.id) - identification_types.indexOf(b.id)); //Garantindo a ordem que está no parâmetro.
                }
                return ret;
            })(angular.copy(identification_types));

            if ($state.is('easyorder.customer')) {
                if(ConfigService.config.business.enable_customer_crud_by_rest_api && $stateParams.customerToEdit) {
                    $scope.customerRetrieved = true;
                    $scope.customer = fillCustomerProperties(angular.copy($stateParams.customerToEdit));
                    $scope.documentInvalid = false;
                    if($scope.customer.document != null && $stateParams.autoSearchByCustomerDocument){
                        $scope.searchAndComplete();
                    }
                } else if (CartService.cart.customer != null &&
                        ConfigService.config.business.allow_current_customer_editing) {
                    $scope.customerRetrieved = true;
                    $scope.customer = fillCustomerProperties(angular.copy(CartService.cart.customer));
                    $scope.documentInvalid = isEmpty($scope.customer.document) ? true :
                        $scope.validateDocument($scope.customer.documentType, true);
                }
                $scope.formatPhoneNumber();
            } else if ($state.is('easyorder.customerchipsearch')) {
                $scope.doChipSearch();
            }
        })();
    })
})();

class SelfServiceNotAllowedError extends Error {
    constructor(message) {
        super(message);
        this.name = 'SelfServiceNotAllowedError';
    }
}

class LoadingHelper {
    /**
     *
     * @param {*} action - action function
     * @param {*} show - function that returns a promise
     * @param {*} hide - function that returns a promise
     */
    constructor(action, show, hide) {
        this.promise = show()
        .then(() => action())
        .then(ret => {
            hide();
            return ret;
        })
        .catch(error => {
            hide();
            return Promise.reject(error);
        });
    }

    then(successCallback, errorCallback) {
        let ret = this.promise;

        if (successCallback) {
            ret = ret.then(successCallback);
        }
        if (errorCallback) {
            ret = ret.catch(errorCallback);
        }

        return ret;
    }

    catch(errorCallback) {
        let ret = this.promise;

        if (errorCallback) {
            ret = ret.catch(errorCallback);
        }

        return ret;
    }

    finally(finallyCallback) {
        this.promise.finally(finallyCallback);
    }
}

class IonicLoadingHelper extends LoadingHelper {
    constructor(action, $ionicLoading) {
        super(action, $ionicLoading.show, $ionicLoading.hide);
    }
}

class BarcodeLoadingHelper extends LoadingHelper {
    constructor(action, $scope) {
        super(action,
        () => { // show
            $scope.loadingState = true;
            return Promise.resolve();
        },
        () => { // hide
            $scope.loadingState = false;
            return Promise.resolve();
        });
    }
}
