
(function () {
    'use strict';
    angular.module('easyorder.controllers.donation', [
        'ionic',
        'ionic.closePopup'
    ]).controller('DonationCtrl', function (
        $rootScope,
        $scope,
        $ionicLoading,
        UtilService,
        AlertService,
        ProductService,
        $stateParams,
        ConfigService,
        UserService,
        CartService,
        $q
    ) {

        $scope.op = $stateParams.op;
        $scope.items = [{
            removable: false,
            value: '',
        }];

        $scope.addItem = () => {
            $scope.items.push({
                removable: true,
                value: '',
            });
        };

        $scope.removeItem = item => {
            const index = $scope.items.indexOf(item);
            if (index) {
                $scope.items.splice(index, 1);
            }
        };

        $scope.goBack = () => {
            UtilService.navigateWithoutHistory('easyorder.operations');
        };

        $scope.goNext = () => {
            $ionicLoading.show({
                template: 'Aguarde...',
                duration: 0,
            }).then(() => {
                return checkUserData();
            }).then(() => {
                return searchItems();
            }).then(items => {
                $ionicLoading.hide().then(() => {
                    UtilService.navigateWithoutHistory('easyorder.transactionjustification', {
                        op: $scope.op,
                        authorizer: $stateParams.authorizer,
                        data: {
                            fiscalItems: items,
                        },
                    });
                });
            }).catch(error => {
                $ionicLoading.hide().then(() => {
                    AlertService.restError(error, error);
                });
            });
        };

        function checkUserData() {
            return $q.all($scope.items.map(item => {
                return $q((resolve, reject) => {
                    if (item.value == '') {
                        return reject('Todos os campos devem ser preenchidos.');
                    } else {
                        return resolve();
                    }
                });
            }));
        }

        function searchItems() {
            return $q.all($scope.items.map(item => item.value).map(searchItemByUuid));
        }

        function searchItemByUuid(uuidValue) {
            return ProductService.lookupByUUID({
                storeNumber: ConfigService.config.storeNumber,
                productUuid: uuidValue,
            }).then(result => {
                const inventory = result.productInventories[0];
                if (inventory.availableQuantity <= 0) {
                    return $q.reject(`Não é permitida a doação para este produto`);
                }
                return searchItem(inventory.productCode, uuidValue);
            }).catch(error =>
                error.status && error.status == 404 ? $q.reject('ICCID para o produto inexistente, insira outro ICCID') : $q.reject(error)
            );
        }

        function searchItem(code, uuidValue) {
            return $q((resolve, reject) => {
                ProductService.search({code : code}, resolve, reject);
            }).then(result => {
                if (result.items.length === 0) {
                    return $q.reject(`ICCID para o produto inexistente, insira outro ICCID`);
                }
                const item = result.items[0];
                if (!item.uuidInfo) {
                    return $q.reject(`Não é permitida a doação para este produto`);
                }
                if (item.uuidInfo.type != 'ICCID') {
                    return $q.reject(`Não é permitida a doação para este produto`);
                }

                return {
                    description: item.productDescription,
                    compactDescription: item.productDescription,
                    code: item.productCode,
                    automationCode: item.automationCode,
                    unitValue: item.unitValue,
                    totalValue: item.unitValue,
                    quantity: 1,
                    uuid: parseUuid(item.uuidInfo, uuidValue),
                    marketingCode: item.marketingCode
                };
            });
        }

        function parseUuid(uuidInfo, uuidValue) {
            return {
                type: uuidInfo.type,
                value: uuidValue,
            };
        }
    })
})();
