(function () {
    'use strict';
    angular.module('easyorder.controllers.exchange.uuid.inputs', [
        'ionic',
        'ionic.closePopup'
    ]).controller('ExchangeUuidInputsCtrl', function (
        $rootScope,
        $scope,
        $ionicLoading,
        UtilService,
        AlertService,
        TransactionReturnService,
        $stateParams,
        ConfigService,
        UserService,
        CartService,
        $q
    ) {

        $scope.op = $stateParams.op;
        $scope.sourceTransaction = $stateParams.exchangeData.sourceTransaction;
        $scope.exchangeData = $stateParams.exchangeData;
        $scope.returnReason = $stateParams.returnReason;

        // view object
        $scope.displayTuples = $scope.exchangeData.fiscalItems.filter(item => item.uuid)
        .map(item => {
            const tuple = {
                srcItem: {
                    sequenceId: item.sequenceId,
                    code: item.code,
                    automationCode: item.automationCode,
                    marketingCode: item.marketingCode,
                    unitValue: item.unitValue,
                    totalValue: item.totalValue,
                    quantity: item.quantity,
                    description: item.description,
                    compactDescription: item.compactDescription,
                    uuid: item.uuid
                },
                dstItem: {
                    sequenceId: item.sequenceId,
                    code: item.code,
                    automationCode: item.automationCode,
                    marketingCode: item.marketingCode,
                    unitValue: item.unitValue,
                    totalValue: item.totalValue,
                    quantity: item.quantity,
                    description: item.description,
                    compactDescription: item.compactDescription,
                    uuid: {
                        type: item.uuid.type,
                        value: '',
                    }
                }
            };

            return tuple;
        });

		$scope.parseTupleInfo = tuple => {
			const uuid = tuple.srcItem.uuid;
			if (uuid) {
				return `${uuid.type} - ${uuid.value}`;
			} else {
				return `COD. - ${tuple.srcItem.code}`;
			}
		};

        $scope.goBack = () => {
            UtilService.navigateWithoutHistory('easyorder.transactionjustification', {
                op: $scope.op,
                authorizer: $stateParams.authorizer,
                data: $scope.exchangeData
            });
        };

        $scope.goNext = () => {
            $ionicLoading.show({
                template: 'Aguarde...',
                duration: 0
            }).then(() => {
                return checkEmptyFields($scope.displayTuples);
            }).then(() => {
                return checkDuplicatedInfo($scope.displayTuples);
            }).then(() => {
                return exchange($scope.displayTuples);
            }).then(exchangeResult => {
                return goToNextPage(exchangeResult);
            }).catch(error => {
                onError(error);
            });
        };

        function checkEmptyFields(tuples) {
            return $q.all(tuples.map(emptyFieldPromise));
        }

        function emptyFieldPromise(tuple) {
            return $q((resolve, reject) => {
                if (tuple.srcItem.uuid && tuple.dstItem.uuid.value === '') {
                    reject(`Você deve preencher o UUID do item ${tuple.srcItem.uuid.value}`);
                } else {
                    resolve();
                }
            });
        }

        function checkDuplicatedInfo(tuples) {
            return $q.all(tuples.map(tuple => checkDuplicatedInfoPromise(tuple, tuples)));
        }

        function checkDuplicatedInfoPromise(tuple, tuples) {
            return $q((resolve, reject) => {
                const srcUuidCount = tuples.filter(tpl => tpl.srcItem.uuid.type === tuple.dstItem.uuid.type)
                    .filter(tpl => tpl.srcItem.uuid.value == tuple.dstItem.uuid.value)
                    .length;
                if (srcUuidCount >= 1) {
                    reject(`
                        IMEI / ICCID inválido! Código do item é o mesmo do equipamento de entrada
                    `);
                    return;
                }
                const dstUuidCount = tuples.filter(tpl => tpl.dstItem.uuid.type === tuple.dstItem.uuid.type)
                    .filter(tpl => tpl.dstItem.uuid.value == tuple.dstItem.uuid.value)
                    .length;
                if (dstUuidCount > 1) {
                    // NÀO EXISTE ESTE TRATAMENTO NA EF!!!!!!
                    reject(`
                        ${tuple.dstItem.uuid.type} ${tuple.dstItem.uuid.value} inválido! código do item 
                        informado ${dstUuidCount} vezes
                    `);
                    return;
                }
                resolve();
            });
        }

        function exchange(tuples) {
            return TransactionReturnService.return({
                appInfo: {
                    name: $rootScope.APP_NAME,
                    version: $rootScope.appVersion
                },
                sourceTransactionKey: $scope.sourceTransaction.key,
                transactionKey: {
                    storeInfo: {
                        companyNumber: ConfigService.config.companyNumber,
                        storeNumber: ConfigService.config.storeNumber,
                    },
                    date: new Date(),
                    nsu: ConfigService.nsuControll.incrementNSU(),
                    terminal: ConfigService.config.terminalNumber,
                },
                operationType: $scope.op,
                returnReason: $scope.returnReason,
                operator: {
                    code: UserService.operatorCode(),
                    name: UserService.operatorName()
                },
                authorizer: {
                    code: $stateParams.authorizer.id,
                    name: $stateParams.authorizer.name
                },
                customer: CartService.cart.customer,
                tuples: tuples,
                sourceTransactionTotalValue: $scope.sourceTransaction.totalValue,
                totalValue: tuples.reduce((acc, tuple) => new Number((acc + tuple.srcItem.totalValue).toFixed(2)), 0),
            });
        }

        function goToNextPage(exchangeResult) {
            $ionicLoading.hide().then(() => {
                UtilService.navigateWithoutHistory('easyorder.couponview', {
                    pdf: exchangeResult.pdf,
                    printContents: exchangeResult.printContents,
                    documentTitle: 'exchange'
                });
            });
        }

        function onError(error) {
            $ionicLoading.hide().then(() => {
                AlertService.restError(error, error);
            });
        }
    })
})();

