(function() {
    'use strict';

	angular.module('easyorder.controllers.externalloading', [
    'easyorder.services.config',
    'easyorder.services.cart',
    'easyorder.services.util',
		'easyorder.services.externalloading',
		'easyorder.services.log',
	])

	.controller('ExternalLoadingCtrl', function($scope, $rootScope, $ionicBody,$ionicPopup, $ionicBackdrop, $timeout , $ionicLoading, $stateParams,
                                       UserService, ConfigService, CartService, UtilService, AlertService, OrderService, LogService) {

     const closeAllPopups = () => {
            var noop = angular.noop;
            var elevated = false;
            var popupStack = $ionicPopup._popupStack;
            if (popupStack.length > 0) {
              popupStack.forEach(function(popup, index) {
                if (popup.isShown === true) {
                  popup.remove();
                  popupStack.pop();
                }
              });
            }

            $ionicBackdrop.release();
            $timeout(function() {
              $ionicBody.removeClass('popup-open');
              $ionicBody.removeClass('modal-open');
            }, 500, false);
            ($ionicPopup._backButtonActionDone || noop)();
          }
        

        const showLoadingMessage = (message) => {
        	return $ionicLoading.show({
				template : message,
				duration : 0
			});
        }
        
        const handleError = (message) => {
        	$ionicLoading.hide();
        	AlertService.show('Erro ao resgatar pedido externo', message, "Sair").then(
				function() {
					LogService.info('Saindo da aplicação..');
					$scope.exitApp();
				}
			);
        }
    
        const doOrderRetrieval = (uri) => {
        	var uriParsed = URI.parse(uri.toString());
        	var uriPath = uri;
        	
            var showErrorMessage = function(httpResponse, defaultMessage) {
            	var message = (httpResponse && httpResponse.data && httpResponse.data.message) ? httpResponse.data.message : defaultMessage;
            	handleError(message);
            }
            
            closeAllPopups();
            
        	return showLoadingMessage('Efetuando login do usuário...').then(function() {
        		UserService.login(uriParsed.username, uriParsed.password).then(
    					function(success) {
    						LogService.info("Usuário '"+ uriParsed.username +"' logado!");
    						showLoadingMessage('Efetuando busca do pedido...').then(function(uri) {
    							var queryParams = uriPath.search(true);
								var params = {
										companyNumber: ConfigService.config.companyNumber,
										storeNumber: queryParams['storeNumber'],
										externalOrderNumber: queryParams['externalOrderNumber']
									};
    							 OrderService.getExternal(params,function(result) {
    								 $ionicLoading.hide();
    								 var successCallback = function() {
										 CartService.loadCart(result);
										 $rootScope.externallyLoadedApp = true;
    									 UtilService.navigateWithoutHistory('easyorder.cart');
    								 };
    								 
									 var errorCallback = function(error) {
										handleError(error.message);
									 };
    									
    								 OrderService.validateOrder(result, ConfigService.config.storeNumber, successCallback, errorCallback);
 									 
    							 }, function(error) {
    								 showErrorMessage(error, 'Não foi possível resgatar pedido no servidor!');
    							 });
    							 
    						}, function(error) {
    							showErrorMessage(error, 'Não foi possível resgatar pedido no servidor!');
    						});
    					},
    					function(error) {
    						showErrorMessage(error, 'Não foi possível se autenticar no servidor!');
    					}
    			);
        	});
        	
        }


        const handleUri = (uri) => {
				LogService.info("URI externa a ser manipulada: " + uri);
			if (uri) {
				// no momento existe apenas a operação de resgate de pedidos.
				doOrderRetrieval(new URI(uri)).catch(error => {
					handleError(error.message);
				});
			}

    		
    	};
        /**
         * Função de inicialização.
         */
        (function() {
			var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
            var iw = (iOS) ? screen.width : window.innerWidth,
                ih = (iOS) ? screen.height : window.innerHeight;

            $scope.screenHeight = ih - 300;
            $scope.screenWidth = iw;
            handleUri($stateParams.uri);
        })();
    })
})();
