(function () {
    'use strict';

    angular.module('easyorder.controllers.giftcard', [
            'easyorder.services.giftcard',
            'easyorder.services.config',
            'easyorder.services.cart',
            'easyorder.services.util',
            'easyorder.services.alert',
            'easyorder.services.log',
        ])

        .controller('GiftCardCtrl', function ($scope,
                                            $ionicLoading,
                                            $timeout,
                                            $state,
                                            GiftCardService,
                                            CartService,
                                            UtilService,
                                            AlertService,
                                            EFTService,
                                            LogService,
                                            ConfigService) {

            $scope.giftcard = {};
                                 
            /**
             *
             */
            $scope.processOptions = function (options, responseHandler) { //(1,"Magnetico/Chip")#(2,"Digitado")
                $scope.options.length = 0;
                $scope.showWatermark = false;

                var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
                splitedOpt = splitedOpt.split("#"); // Separando as opções.

                for (var i = 0; i < splitedOpt.length; i++) {
                    var split = splitedOpt[i].split(",");

                    $scope.options.push({
                        id: split[0].trim(),
                        label: split[1].trim().replace(/\"/g, ""),
                        value: split[split.length - 1],
                        handler: responseHandler,
                        type: "single-option"
                    });
                    $scope.$apply();
                }
                $scope.headerOptions = "Opções";
            };
            
            /**
             * Função executa a função com os parâmetros passados a esta.
             */
            $scope.tickOptionHandler = function(func, params) {
                $scope.options.length = 0;
                $scope.showWatermark = true;
                $scope.message = "Aguarde...";
                
                func.apply(this, params);
            };
            
           /**
            *
            */
           $scope.tickNumberInputHandler = function(inputOption) {
               var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);
               $scope.message = "Aguarde...";
               $scope.tickOptionHandler(inputOption.handler, [value]);
           };

           /**
            *
            */
           $scope.tickStringInputHandler = function(inputOption) {
               const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);
               $scope.message = "Aguarde...";
               $scope.tickOptionHandler(inputOption.handler, [value]);
           };

            /**
             * Inicializa a consulta do cliente através da leitura do cartão do cliente.
             */
            $scope.doChipSearch = function () {
                EFTService.executeTransaction({
                    transactionType: EFTService.transactionType.TRAIL_READ,
                    instalmentsNumber: undefined,
                    amount: 0
                }, {
                    paymentDone: function (paymentResults) {
                        if (paymentResults && paymentResults.Trilha2) {
                            var value = paymentResults.Trilha2.split('=')[0];

                            LogService.info('Cartão presente a ser consultado: ' + value);

                            var canSearch = true;
                            var serviceItems = CartService.cart.serviceItems;

                            if (serviceItems && serviceItems.length > 0) {
                                serviceItems.forEach(function (serviceItem) {
                                    if (serviceItem.documentNumber === value) {
                                        canSearch = false;
                                        AlertService.error("Cartão presente de número '" + serviceItem.documentNumber + "' já adicionado!").then(function () {
                                            UtilService.navigateWithoutHistory('easyorder.cart');
                                        });
                                    }
                                });
                            }
                            if (canSearch) {
                                doSearch(value);
                            }
                        } else {
                            AlertService.error("Erro ao obter dados do cartão. Retorno do pinpad inválido.")
                            .then(UtilService.navigateToMainSearchScreen);
                        }
                    },
                    paymentError: function (paymentResults) {
                        var errMsg = paymentResults && paymentResults.MensagemErro ? paymentResults.MensagemErro : "Operação TEF não realizada.";

                        if ($state.is('easyorder.giftcardsearch')) { //Ainda está nesse estado?
                            AlertService.error(errMsg)
                            .then(UtilService.navigateToMainSearchScreen);
                        }
                    },
                    clearDisplay: function () {
                        $timeout(function () {
                            $scope.message = "";
                        });
                        LogService.info("clearDisplay");
                    },
                    displayMessage: function (message) {
                        if (message) {
                            $timeout(function () {
                                $scope.message = message;
                            });
                        }
                        LogService.info("displayMessage: " + $scope.message);
                    },
                    selectOptionExtended: function (title, options, responseHandlerExtended) {
                        $scope.$apply(function () {
                            $scope.message = title;
                        });
                        $scope.processOptions(options, responseHandlerExtended);
                        LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
                    }
                });
            };

            /**
             * 
             */
            var doSearch = function (cardNumber) {
                $ionicLoading.show({
                    template: 'Consultando cartão presente...',
                    duration: 0
                }).then(function () {
                    var success = function (response) {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                        var gift = response.giftCard;

                        if (gift) {
                            UtilService.navigateWithoutHistory('easyorder.giftcardinfo', {
                                giftCard: gift
                            });
                        } else {
                            AlertService.error('Cartão presente não retornado!').then(function () {
                                UtilService.navigateWithoutHistory('easyorder.barcode')
                            });
                        }
                    };
                    var error = function (response) {
                        $scope.loadingState = false;
                        $ionicLoading.hide();

                        AlertService.restError(response, 'Erro ao consultar cartão presente.').then(function () {
                            UtilService.navigateWithoutHistory('easyorder.barcode')
                        });
                    };
                    GiftCardService.search(cardNumber, success, error);
                });
            };

            $scope.typedSearch = function () {
                doSearch($scope.giftcard.cardNumber);
            }

            $scope.goBack = function () {
                UtilService.navigateWithoutHistory('easyorder.giftcardmenu');
            }

            function read()  {
                if (ConfigService.config.business.giftcard_service_provider == "PROMO") {
                    UtilService.navigateWithoutHistory('easyorder.giftcardtypedrecharge');
                } else {
                    $scope.doChipSearch();
                }
            }

            /**
             * Função de inicialização.
             */
            (function () {
                $scope.showWatermark = true;
                $scope.options = [];
                $scope.message = "";

                read();
            })();
        })
})();