(function() {
    'use strict';

	angular.module('easyorder.controllers.giftcardinfo', [
    'easyorder.services.giftcard',
    'easyorder.services.config',
    'easyorder.services.printer',
    'easyorder.services.cart',
    'easyorder.services.util',
    'easyorder.services.alert'])

	.controller('GiftCardInfoCtrl', function($scope,
                                       $ionicLoading,
                                       $stateParams,
                                       GiftCardService,
                                       ConfigService,
                                       CartService,
                                       UtilService,
                                       AlertService) {

        $scope.giftCard = $stateParams.giftCard;
        $scope.reloadData = {
            amount: 0
        }	

        var doReload = function() {
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(function() {
                var success = function(response) {
                    $scope.loadingState = false;
                    $ionicLoading.hide();

                    var item = {};
                    item.operationType = "GIFT_CARD_RELOAD";
                    item.documentNumber = response.cardNumber;
                    item.receiptText = response.receiptText;
                    item.value = response.amount;
                    item.nsuNumber = response.nsuNumber;
                    CartService.addServiceItem(item);

                    UtilService.navigateWithoutHistory('easyorder.cart');
                };

                var error = function(response) {
                    $scope.loadingState = false;
                    $ionicLoading.hide();
                    AlertService.restError(response, 'Erro ao recarregar cartão presente.');
                };
                GiftCardService.reload($scope.giftCard.cardNumber, $scope.reloadData.amount, success, error);
            });
        };

        $scope.reload = function() {
            if (CartService.getCurrentOperation() === 'FISCAL') {
                AlertService.warn("Produtos fiscais já adicionados ao carrinho. Não é possível realizar recarga de cartão presente.");
            } else {
                var canReload = true;
                var serviceItems = CartService.cart.serviceItems;
                
                if (serviceItems && serviceItems.length > 0) {
                    serviceItems.forEach(function(serviceItem) {
                        if (serviceItem.documentNumber === $scope.giftCard.cardNumber) {
                            canReload = false;
                            AlertService.error("Cartão presente de número '" + serviceItem.documentNumber + "' já adicionado!");
                        }
                    });
                }
                if (canReload) {
                    if (ConfigService.config.business.giftcard_service_provider == "PROMO") {
                        reloadInternally()
                    } else {
                        doReload();
                    }
                }
            }
        };

        
        $scope.mayReload = function () {
            if ($scope.reloadData.amount <= 0
                || $scope.reloadData.amount > $scope.giftCard.maxReloadValue
                || $scope.reloadData.amount < $scope.giftCard.minReloadValue) {
                    return false;
            }
            if (ConfigService.config.business.giftcard_service_provider == 'PROMO'){
                return $scope.giftCard.hasOwnProperty('activated') ? $scope.giftCard.activated : false;
            } else {
                return $scope.giftCard.hasOwnProperty('activated') ? !$scope.giftCard.activated : true
            }
        }

        function reloadInternally () {
            var item = {};
            item.operationType = "GIFT_CARD_RELOAD";
            item.documentNumber = $scope.giftCard.cardNumber;
            item.value = $scope.reloadData.amount;
            CartService.addServiceItem(item);

            UtilService.navigateWithoutHistory('easyorder.cart');
        }
    })
})();
