(function() {
    'use strict';

    angular.module('easyorder.controllers.giftcardpayment', [
        'easyorder.services.cart',
        'easyorder.services.nfce',
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.services.payment',
        'easyorder.services.giftcard',
        'easyorder.services.log'
    ])

    .controller('GiftCardPaymentCtrl', function($scope,
                                        $ionicLoading,
                                        $stateParams,
                                        $timeout,
                                        $q,
                                        CartService,
                                        UtilService,
                                        ConfigService,
                                        PaymentService,
                                        AlertService,
                                        GiftCardService,
                                        UserService,
                                        EFTService,
                                        LogService,
                                        PaymentProcessor,
                                        uuid) {
        
        $scope.giftCardProvider = ConfigService.config.business.giftcard_service_provider;
        const contextUid = uuid.v4();
                                            
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        
        $scope.isTransactionInitiated = function() {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $timeout(function() {
                $scope.message = "";
            });
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            $timeout(function() {
                $scope.message = message;
            });
            LogService.info("displayMessage: " + $scope.message);
        };

        $scope.selectOptionExtended = function(title, options, responseHandlerExtended) {
            $scope.$apply(function () {
                $scope.message = title;
            });
            $scope.processOptions(options, responseHandlerExtended);
            LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
        };

           /**
         *
         */
        $scope.processOptions = function(options, responseHandler) {    //(1,"Magnetico/Chip")#(2,"Digitado")
            $scope.options.length = 0;
            $scope.showWatermark = false;

            var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
            splitedOpt = splitedOpt.split("#"); // Separando as opções.

            for (var i = 0; i < splitedOpt.length; i++) {
                var split = splitedOpt[i].split(",");

                $scope.options.push({
                    id:         split[0].trim(),
                    label:      split[1].trim().replace(/\"/g, ""),
                    value:      split[split.length-1],
                    handler:    responseHandler,
                    type:        "single-option"
                });
                $scope.$apply();
            }
            $scope.headerOptions = "Opções";
        };
        /**
         *
         */
        $scope.inputText = function(label, type, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               type,
                maxSize:            fieldSizeRange.maxSize,
                minSize:            fieldSizeRange.minSize
            });
            $scope.headerOptions = "Informe";
            $scope.$apply();
        };


    
        /**
         * Função executa a função com os parâmetros passados a esta.
         */
        $scope.tickOptionHandler = function(func, params) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func.apply(this, params);
        };

        /**
         *
         */
        $scope.tickTelInputHandler = function(inputOption) {
            $scope.tickOptionHandler(inputOption.handler, [inputOption.value]);
        };

        /**
         *
         */
        $scope.tickStringInputHandler = function(inputOption) {
            const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);

            $scope.tickOptionHandler(inputOption.handler, [value]);
        };
        /**
         * Função para tratamento após leitura de trilha do cartão presente.
         */
        $scope.paymentDone = function(paymentResults) {
            $scope.clearDisplay();
            $scope.showWatermark = false;
            var trail = undefined;
            
            if (paymentResults && paymentResults.Trilha2) {
                trail = paymentResults.Trilha2.split('=')[0];
                
                if (!trail.startsWith('8800')) { //TODO: Verificaçã temporária para cartões não Givex da Renner.
                    $timeout(function() {
                        $scope.displayMessage("Digite a senha para autorização da venda");
                    }, 800); //Firula.
                    
                    $scope.inputText("Senha do Cartão", 'password', (data) => {
                        $scope.loadingState = true;
                        
                        $scope.cardVerification(trail).then(() =>{
                            $ionicLoading.show({
                                template : 'Autorizando pagamento. Favor aguarde...',
                                duration : 0
                            }).then(function() {
                                authGiftcardPayment(trail, data);
                            });
                        }, () => {
                            UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                        }).finally (() => $scope.loadingState = false);
                    }, { minSize: 4, maxSize: 15 });
                } else {
                    $scope.cardVerification(trail).then(() =>{
                        $ionicLoading.show({
                            template : 'Autorizando pagamento. Favor aguarde...',
                            duration : 0
                        }).then(function() {
                            authGiftcardPayment(trail, undefined);
                        });
                    }, () => {
                        UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                    }).finally (() => $scope.loadingState = false);
                }
            } else {
                AlertService.error("Erro ao obter dados do cartão. Retorno do pinpad inválido.").then(function() {
                    UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                });
            }
        };
        
        /**
         * 
         */
        $scope.processTypedCard = function() {
            $scope.cardVerification($scope.auth.cardNumber).then(() =>{
                $ionicLoading.show({
                    template : 'Autorizando pagamento. Favor aguarde...',
                    duration : 0
                }).then(function() {
                    authGiftcardPayment($scope.auth.cardNumber, $scope.auth.cardPassword);
                });
            }, () => {
                UtilService.navigateWithoutHistory('easyorder.paymentmethods');
            }).finally (() => $scope.loadingState = false);
        };
        
        /**
         * 
         */
        $scope.cardVerification = function(trail) {
            var defer = $q.defer();
            
            $ionicLoading.show({
                template : 'Consultando cartão presente...',
                duration : 0
            }).then(function() {
                GiftCardService.search(trail).then((response) => {
                    $ionicLoading.hide();
                    const giftCard = response.giftCard;
                    
                    if (giftCard) {
                        const checkPromise = checkBalance(giftCard);
                        
                        checkPromise.then((res) => {
                            switch(res) {
                                case -1:
                                    PaymentService.currentPayment().paymentValue = giftCard.balanceAvailable;
                                    defer.resolve();
                                    break;
                                case 0:
                                case 1:
                                    defer.resolve();
                                    break;
                            }
                        }, () => defer.reject());
                    } else {
                        AlertService.error('Cartão Presente não retornado!').then(function() {
                            defer.reject();
                        });
                    }
                }, (error) => {
                    $ionicLoading.hide();
                    
                    AlertService.restError(error, 'Não foi possível consultar Cartão Presente.').then(function() {
                        defer.reject();
                    });
                });
            });
            return defer.promise;
        };
        
        /**
         * Valida o saldo do CPR. A função retorna "-1" se o saldo for menor do que o valor do pagamento atual, "0" se igual e 
         * "1" se for maior.
         * 
         * A função apresente uma mensagem de confirmação para o operador no caso quando o retorno retorno for "-1" ou "1". O retorno é
         * uma promessa resolvida com os valores citados ou rejeitada caso o usuário responda à confirmação de forma negativa.
         */
        var checkBalance = function(giftCard) {
            const defer = $q.defer();
            
            if (giftCard.balanceAvailable) {
                if (giftCard.balanceAvailable < $scope.currentPayment.paymentValue) {
                    const msg = "Saldo do cartão é " + currencyFormat(giftCard.balanceAvailable) + ", inferior ao valor do pagamento." + 
                        "Deseja prosseguir com o saldo como parte do pagamento?";
                    
                    AlertService.confirm(msg).then((res) => {
                        if (res) {
                            defer.resolve(-1);
                        } else {
                            defer.reject();
                        }
                    });
                } else if (giftCard.balanceAvailable > $scope.currentPayment.paymentValue) {
                    const msg = "Você utilizará " + currencyFormat($scope.currentPayment.paymentValue) + ", restando " 
                        + currencyFormat(giftCard.balanceAvailable - $scope.currentPayment.paymentValue) + " no saldo do cartão presente. Deseja prosseguir?";
                    
                    AlertService.confirm(msg).then((res) => {
                        if (res) {
                            defer.resolve(1);
                        } else {
                            defer.reject();
                        }
                    });
                } else {
                    defer.resolve(0);
                }
            } else {
                if (!giftCard.activated) {
                    AlertService.warn("Cartão presente não está ativo").then(defer.reject());
                } else {
                    AlertService.warn("Cartão presente já utilizado").then(defer.reject());
                }
            }
            return defer.promise;
        };
        
        /**
         * Envia a requisição para autorização do Cartão Presente.
         */
        var authGiftcardPayment = function (trail, password) {
            $timeout(function() {
                $scope.displayMessage("Autorizando pagamento. Favor aguarde...");
            }, 800); //Firula.
            
            var currentPayment      = PaymentService.currentPayment();

            var reqParams = {};
            reqParams.amount            = new Number(PaymentService.currentPayment().paymentValue).toFixed(2);
            reqParams.cardNumber        = trail;
            
            if (password) {
                reqParams.securityCode  = password;
            }
            reqParams.companyNumber     = currentPayment.companyNumber;
            reqParams.storeNumber       = currentPayment.storeNumber;
            reqParams.terminalNumber    = ConfigService.config.terminalNumber;
            reqParams.nsu               = ConfigService.nsuControll.currentCartNSU();
            reqParams.cardTypeId        = ConfigService.config.business.giftcard_type;
            reqParams.contextUid        = contextUid;

            if ($scope.giftCardProvider == 'PROMO') {
                PaymentProcessor.pushSalePayment({
                    type: 'SaleGiftCardPayment',
                    paymentCodeMethod: 'GIFTCARD',
                    companyNumber : ConfigService.config.companyNumber,
                    storeNumber : ConfigService.config.storeNumber,
                    terminalNumber : ConfigService.config.terminalNumber,
                    cardTypeId : ConfigService.config.business.giftcard_type,
                    cardNumber: trail,
                    amount: new Number(PaymentService.currentPayment().paymentValue).toFixed(2),
                    contextUid: contextUid,
                });
            }

            GiftCardService.authGiftCardPayment(reqParams, 
                function(result) {
                    $scope.clearDisplay();

                    const wasAuthorized = (res) => {
                        const provider = ConfigService.config.business.giftcard_service_provider;
                        if (provider == 'PROMO') {
                            return res.paymentAuthorizationNSU != null
                                && res.paymentAuthorizationNSU.length > 0;
                        } else {
                            return res.responseCode && res.responseCode === "00";
                        }
                    };

                    if (wasAuthorized(result)) {
                        $scope.displayMessage("Pagamento autorizado!");
                       
                        $scope.processPayment(result, CartService.cart, PaymentService.currentPayment());
                        PaymentService.currentPaymentDone();
                        
                        $timeout(function() {
                            $ionicLoading.show({
                                template : 'Por favor, aguarde...',
                                duration : 0
                            }).then(function() {
                                 UtilService.navigateAfterReceivedPayment();
                            });
                        }, 1000);
                    } else if (result.message) {
                        if ($scope.giftCardProvider == 'PROMO') {
                            undoPendingPayment();
                        }
                        AlertService.restError(error, result.message).then(function() {
                            UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                        });
                    } else {
                        if ($scope.giftCardProvider == 'PROMO') {
                            undoPendingPayment();
                        }
                        UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                    }
                }, function(error) {
                    $ionicLoading.hide();
                    $scope.clearDisplay();

                    AlertService.restError(error, 'Erro ao autorizar pagamento!')
                        .then(function(result) {
                            $scope.paymentError();
                    });
                }
            );
        };
        
        /**
         * Processa o pagamento atual e o resultado da transação de autorização CP.
         */
        $scope.processPayment = function(paymentResults, cart, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SaleGiftCardPayment';
            salePayment.installments                = [];
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = new Number(currentPayment.paymentValue).toFixed(2);
            salePayment.paymentAuthorizationNSU     = paymentResults.paymentAuthorizationNSU;
            salePayment.cardNumber                  = paymentResults.cardNumber;
            salePayment.customerPaymentVoucher      = paymentResults.customerPaymentVoucher;
            salePayment.paymentCodePlan             = 2; //À Vista.
            salePayment.providerName                = paymentResults.providerName;
            if($scope.giftCardProvider == 'PROMO') {
                PaymentProcessor.updateSalePayment(
                    (payment) => payment.data.confirm.contextUid == contextUid,
                    { ...salePayment, contextUid: contextUid },
                );
            } else {
                PaymentProcessor.pushSalePayment({
                    ...salePayment, contextUid: contextUid
                });
            }
            cart.salePayments.push(salePayment);
        };
        
        /**
         * Função chamada por callback do TEF. Desfaz os pagamentos.
         */
        $scope.paymentError = function(paymentResults) {
            var errMsg = paymentResults && paymentResults.MensagemErro ? paymentResults.MensagemErro : "Operação TEF não realizada.";
            
            AlertService.error(errMsg).then(function() {
                PaymentService.cancel(true).then(function() {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                });
            });
        };
        
        /**
         * O algorítimo é mod 10 (parecido com o do cartão de crédito) com peso alternado.
         * É calculado em todo o número, exceto o último dígito.
         * O peso (variável mult no exemplo) alterna entre 1 e 2, começando com 1 de trás para a frente, a partir do penúltimo dígito
         * 
         */
        $scope.validTypedCard = function() {
            if ($scope.giftCardProvider == 'PROMO') {
                return $scope.auth.cardNumber.length == 11;
            }

            let c = $scope.auth.cardNumber;
            let parity = false;
            let checksum = 0;
            
            for (let i = c.length - 2; i >= 0; i--) { // "c" já inclui o dígito, evitando-o: -2.
                let numberAt = parseInt(c.charAt(i));
                checksum += numberAt;
                
                if (parity) {
                    checksum += numberAt;
                }
                parity = !parity;
            }
            return (checksum % 10) == c.charAt(c.length - 1);
        };
        
        /**
         * 
         */
        var currencyFormat = function (value) {
            let formatter = new Intl.NumberFormat('pt-BR', {
                style: 'currency',
                currency: 'BRL',
                minimumFractionDigits: 2
            });
            return formatter.format(value);
        };

        function undoPendingPayment() {
            PaymentProcessor.undoPayment(
                (payment) => payment.data.undo.contextUid == contextUid
            );
            PaymentService.clearInitializedPaymentData();
        }
        

         /**
          * Função de inicialização.
          */
        (function(scopeObject) {
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.options = [];
            $scope.transactionInitiated = true;
            $scope.currentPayment = PaymentService.currentPayment();
            
            $scope.auth = {};
            $scope.auth.cardPassword = "";
            
            if ($stateParams.thruTrail) {
                EFTService.executeTransaction({
                    transactionType:   EFTService.transactionType.TRAIL_READ,
                    operatorCode:       UserService.operatorCode(),
                    amount:             0
                }, scopeObject);
            }
        })($scope);
    });
})();
 
