(function () {
    'use strict';
    angular.module('easyorder.controllers.giftcardvirtual', [
        'easyorder.services.util',
        'easyorder.services.config'
    ])
    .controller('GiftCardVirtualCtrl', function ($scope, $stateParams, $ionicLoading, ConfigService,
                                                 UtilService, UserService, AlertService, GiftCardService,
                                                 CartService) {
        
        /**
         * 
         */
        $scope.goNext = function () {
            if (isUserDoingSelfService()) {
                AlertService.error('Proibido auto serviço!');
            } else if (CartService.getCurrentOperation() === 'FISCAL') {
                    AlertService.warn("Produtos fiscais já adicionados ao carrinho. Não é possível realizar recarga de cartão presente.");
            } else {
                $ionicLoading.show({
                    template : 'Criando Cartão...',
                    duration : 0
                }).then(function() {
                   const success = function(response) {
                       $ionicLoading.hide();
                       
                       for (var i = 0; i < response.certInfos.length; i++) {
                           const item = {};
                           item.operationType   = "VIRTUAL_GIFT_CARD_RELOAD";
                           item.documentNumber  = response.certInfos[i].givexNumber;
                           item.nsuNumber       = response.certInfos[i].orderAddressID;
                           item.authorizerCode  = response.checkoutOrderID;
                           item.value           = $scope.giftCard.amount;
                           item.recipientEmail  = $scope.giftCard.recipient.email;
                           item.senderName      = $scope.data.sender.name;
                           item.senderDocument  = $scope.data.sender.document;
                           item.recipientName   = $scope.data.recipient.name
                           

                           CartService.addServiceItem(item);
                       }
                       UtilService.navigateWithoutHistory('easyorder.cart');
                   };
                   const error = function (response) {
                       $ionicLoading.hide();
                       
                       AlertService.restError(response, 'Não foi possível ao criar Cartão Presente virtual.')
                           .then(() => UtilService.navigateWithoutHistory('easyorder.giftcardmenu'));
                   };
                   $scope.giftCard.sender = $scope.data.sender;
                   $scope.giftCard.recipient = $scope.data.recipient;
                   $scope.giftCard.deliveryDate = $scope.data.deliveryDate;
                   $scope.giftCard.deliveryMessage = $scope.data.deliveryMessage;
                   
                   GiftCardService.createVirtualCard($scope.giftCard).then(success, error);
                });
            }
        };
        
        /**
         * 
         */
        var isUserDoingSelfService = () => {
            const user = UserService.currentUser;
                       
            if (user && user.document && $scope.data.document && user.document === $scope.data.document) {
                return true;
            }
            return false;
        };

        /**
         * Função de inicialização.
         */
        (function () {
            $scope.data = {};
            $scope.data.sender = {
                name: CartService.cart.customer ? CartService.cart.customer.name : "",
                email: CartService.cart.customer ? CartService.cart.customer.email : "",
                document: CartService.cart.customer ? CartService.cart.customer.document : ""
            };
            $scope.data.recipient = {
                name: "",
                email: "",
                phone: ""
            };
            $scope.disableCustomerEdition  = ConfigService.config.business.gift_card_mandatory_customer_identification;
            $scope.data.deliveryDate = new Date();
            $scope.data.deliveryMessage = "";
            
            $scope.startDate = new Date();
            $scope.giftCard = $stateParams.giftCard;
        })();
    })
})();
