(function() {
	'use strict';

	angular.module('easyorder.controllers.home', [
		'easyorder.services.barcode',
		'easyorder.services.cart',
		'easyorder.services.user',
		'easyorder.services.util',
		'easyorder.services.customer',
		'easyorder.services.config',
        'easyorder.services.alert',
        'easyorder.services.covenant',
        'easyorder.components.badge',
        'easyorder.services.log'
	])

    .controller('HomeCtrl', function($scope,
	                                 $parse,
                                     $rootScope,
                                     $state,
                                     $timeout,
                                     $ionicBody,
                                     $ionicPopup,
                                     $ionicSideMenuDelegate,
									 $ionicBackdrop,
									 $ionicLoading,
									 $http,
		                             $ionicHistory,
									 Idle,
                                     BarcodeService,
                                     CartService,
                                     UserService,
                                     UtilService,
                                     PaymentService,
                                     ConfigService,
                                     CustomerService,
                                     CovenantService,
                                     ProductService,
                                     AlertService,
                                     AuditService,
                                     EFTService,
                                     LogService,
                                     BluebirdCaptureService,
                                     FiscalMenuModal) {

        var screenTotalValue = 0;

		$scope.cart = CartService.cart;
        $scope.userService = UserService;
        $scope.config = ConfigService.config;
        $scope.totalCartItems = 0;//Variável que será mostada na tela como badge na imagem do carrinho.

		// Inicia um novo timer ao logar
		Idle.watch();

		$scope.events = [];
		$scope.canDragContent = true;
		$scope.sellerSaleModeEnabled = ConfigService.config.business.seller_identification_mode === 'venda';
        if ($scope.sellerSaleModeEnabled
            && ConfigService.config.business.automatic_seller_identification_enabled === true) {
            const loggedOperator = {
                login: UserService.operatorCode(),
                name: UserService.operatorName(),
            }   
            CartService.seller = loggedOperator;
        }
        $scope.hideFooterInfo = false;

		if (CartService.cart && CartService.cart.orderRetrieved) {
			$scope.canDragContent = false;
		}

		/**
         * Se houver mudança de estado
         */
        var unregisterStateChange = $rootScope.$on("$stateChangeStart", function (event, toState, toParams, fromState, fromParams) {
            $scope.customerInSale = undefined;

            $rootScope.previousState = fromState.name;
            $rootScope.currentState = toState.name;

            const fromData = fromState.views && fromState.views.menuItems;
            const toData = toState.views && toState.views.menuItems;

            LogService.debug("State changed: [" + fromState.name +
                                    ", " + (fromData ? fromState.views.menuItems.templateUrl : undefined) +
                                    ", " + (fromData ? fromState.views.menuItems.controller : undefined) + "] " +
                            "-> [" + toState.name +
                                        ", " +  (toData ? toState.views.menuItems.templateUrl : undefined) +
                                        ", " + (toData ? toState.views.menuItems.controller : undefined) + "]" );

            $scope.customer = CartService.cart.customer;
            if (CustomerService.isIdentified($scope.customer)) {
                $scope.customerInSale = $scope.customer; // Atualizando valor da variável.
            }

            LogService.debug("Customer: [" + ($scope.customer && $scope.customer.document ? $scope.customer.document : undefined) +"], "
                    + "NSU: [" + ConfigService.nsuControll.currentNSU() + "], " + "Operator: [" + CartService.cart.operatorCode + "]");

            $http.pendingRequests.forEach(function(pendingRequest) { // "pendingRequest" é o objeto "config" do response
                if (pendingRequest.canceler) {
                    pendingRequest.canceler.resolve(); // Cancela a requisição. Ver 'httpIntercept' em app.js para mais info.
                }
                pendingRequest.responseCanceled = true;
            });

            if (toState.name !== 'easyorder.barcode') {
                BarcodeService.cancel();
            }
            if (toState.name === 'easyorder.paymentmethods' || toState.name === 'easyorder.predefinedpaymentmethods') {
                PaymentService.clearInitializedPaymentData();
            }

            if (fromState.disconnetBluebirdOnExit) {
                BluebirdCaptureService.disconnect();
            }

            if (toState.name == 'easyorder.paymentresult'
                    || $scope.paymentInProgress
                    || (CartService.cart && CartService.cart.orderRetrieved)
                    || (CartService.cart && CartService.getCurrentOperation() === 'SERVICE')) {
                $scope.canDragContent = false;
            } else {
                if (toState.hideSideMenu) {
                    $scope.canDragContent = false;
                } else {
                    $scope.canDragContent = true;
                }
            }
            $scope.showFooterButtons = !toState.hideFooterButtons;
            $scope.showFooterTotal = !toState.hideFooterTotal;
            $scope.hideFooterInfo = toState.hideFooterInfo ? true : false;

            $ionicSideMenuDelegate.canDragContent($scope.canDragContent);
        });

        $scope.$on('$destroy', () => {
            unregisterStateChange();
        });

        /**
         *
         */
        $rootScope.$on('paymentInProgress', (_, value) => {
            $scope.paymentInProgress = value;

        });

        $rootScope.$on('editCustomerInProgress', (_, value) => {
            $scope.editCustomerInProgress = value;
        });

        /**
         *
         */
        $scope.cartDisabled = function() {
            return $scope.paymentInProgress
                || $state.is('easyorder.cart')
                || $state.is('easyorder.tefpayment')
                || $state.is('easyorder.privatelabelpayment')
                || $state.is('easyorder.giftcardpayment')
                || $state.is('easyorder.paymentresult')
                || $state.is('easyorder.customerchipsearch')
                || $state.is('easyorder.paymentlink')
                || $state.is('easyorder.billingpaymentmenu')
                || $state.is('easyorder.billingpaymentdocumentsearch')
                || $state.is('easyorder.billingpaymentscannedsearch')
                || $state.is('easyorder.billingpaymentchipsearch')
                || $state.is('easyorder.billingpaymentprocessorfilter')
                || $state.is('easyorder.billingpaymenttypefilter')
                || $state.is('easyorder.billingpaymentlist')
                || $state.is('easyorder.billingpaymentdetachedlist')
                || $state.is('easyorder.billingpaymentcustomeridentification')
                || $state.is('easyorder.billingpayment');
        };

        $scope.closeAllPopups = function () {
            var noop = angular.noop;
            var popupStack = $ionicPopup._popupStack;

            if (popupStack.length > 0) {
                popupStack.forEach(function(popup) {
                    if (popup.isShown === true) {
                        popup.remove();
                        popupStack.pop();
                    }
                });
            }

            $ionicBackdrop.release();

            $timeout(function() {
                $ionicBody.removeClass('popup-open');
                $ionicBody.removeClass('modal-open');
            }, 400, false);

            ($ionicPopup._backButtonActionDone || noop)();
        }

        $scope.onIdlePromise = false;
        $scope.$on('IdleTimeout', function() {
            if ($scope.onIdlePromise) {
                return;
            }
            $scope.onIdlePromise = true;
			if (UserService.isAuthenticated()) {
				$scope.closeAllPopups();
                BarcodeService.cancel();

                EFTService.finalizeTransaction().then(() => {
                    PaymentService.cancel(true).finally(() => {
                        if (ConfigService.config.business.sox_enabled) {
                            $ionicLoading.show({
                                template: 'Sessão expirada, aguarde...',
                                duration : 0
                            }).then(() => {
                                var doLogoutRestRequest = true;

                                AuditService.expiredSession()
                                    .then(() => {doLogoutRestRequest = false})
                                    .finally(() => {
                                        UserService.logout(doLogoutRestRequest).finally(() => {
                                            $ionicLoading.hide().then(() => { AlertService.warn('Sessão expirada!'); });
                                        });
                                        $scope.onIdlePromise = false;
                                    });
                            });

                        } else {
                            UserService.logout(true).finally(() => {
                                AlertService.warn('Sessão expirada!');
                            });
                            $scope.onIdlePromise = false;
                        }
                    });
                });
            } else {
                LogService.info('Usuário já está deslogado!');
                $scope.onIdlePromise = false;
            }
        });

        $scope.NEW_ORDER = "Nova Venda";

		$scope.showCart = function() {
		    if (!$scope.cartDisabled()) {
		        UtilService.navigateWithoutHistory('easyorder.cart');
		    }
		};

		/**
		 *
		 */
		$scope.$watch(
			function() {
                if($scope.currentScreenName !== $state.current.name){
                    $scope.currentScreenName = $state.current.name;
                }
                return $ionicSideMenuDelegate.isOpen();
            },
			function(newValue, oldValue) {
				if ($ionicSideMenuDelegate.isOpen()) {
					BarcodeService.resizeOut();
					processMenuItemEntrace(true);
				} else {
					BarcodeService.resizeIn();
					processMenuItemEntrace(false);
				}
			}
		);

		/**
		 * Controla a entrada/saída dos itens do menu. Esse controle individualiza a execução da animação.
		 * Os itens são iniseridos/removidos a cada de 30 milisegundos.
		 */
		var processMenuItemEntrace = function(inV) {
		    var i= 1;
            while (i <= 14) {
                $timeout(function(v) {
                    v.assign($scope, inV);
                }, 30 * i, true, $parse("menuOpener_" + i++));
            }
		};

		/**
		 *
		 */
		$scope.logout = function() {
		    BarcodeService.cancel();

			var confirmPopup = AlertService.confirm('Deseja efetuar o logout do usuário?<br><b>Informações não salvas serão descartadas.</b>');

			confirmPopup.then((result) => {
			    if (result) {
			        UserService.logout(true);
			    } else {
			        reInitScan();
			    }
			});
		};

		/**
		 *
		 */
        $scope.searchMethod = function() {
            var scanActivated = BarcodeService.scanActivated();

            if (scanActivated) {
                BarcodeService.cancel();
            }

            if (ConfigService.config.business.enables_rfid_reading) {
                UtilService.navigateToMainSearchScreen();
            } else {
                if (scanActivated) {
                    BarcodeService.reInitScan();
                } else {
                    UtilService.navigateWithoutHistory('easyorder.barcode');
                }
            }

        }
		/**
		 *
		 */
        $scope.newOrder = function() {
            var scanActivated = BarcodeService.scanActivated();
           

            if (scanActivated) {
                BarcodeService.cancel();
            }
		    var confirmPopup = AlertService.confirm('Deseja cancelar a venda atual e iniciar nova?');

		    confirmPopup.then(function(result) {
				if (result) {
                    var cartItens = CartService.cart.fiscalItems;
                    $scope.customerInSale = false;
                    const proceed = () => {
                        CartService.initCart();

                        if (window.cordova) {
                            if (ConfigService.config.business.enables_rfid_reading) {
                                UtilService.navigateWithoutHistory('easyorder.rfid-product-search');
                            } else {
                                if (scanActivated) {
                                    BarcodeService.reInitScan();
                               } else {
                                     UtilService.navigateWithoutHistory('easyorder.barcode');
                               }
                            }
                        } else {
                             UtilService.navigateWithoutHistory('easyorder.productsearch');
                        }
                    };

                    if (ConfigService.config.business.enable_reserve_products === true && cartItens.length > 0
                            &&  (cartItens = cartItens.filter(item => item.itemUuid && item.itemUuid.value)).length) {
                        const cancelParams = {
                            storeNumber : ConfigService.config.storeNumber,
                            orderNumber: CartService.sessionId,
                            terminalNumber: ConfigService.config.terminalNumber,
                            operatorName: UserService.currentUser.name,
                            products: cartItens.map(item => {
                                return {
                                    productCode: item.productCode,
                                    uuid: item.itemUuid.value,
                                    quantity: item.quantitySold
                                };
                            })
                        };
                        ProductService.cancelSale(cancelParams).finally(() => {
                            proceed();
                        });
                    } else {
                        proceed();
                    }
				} else {
				    if (scanActivated) {
                        BarcodeService.reInitScan();
                    }
                }
			});
		};

		/**
         * Controla a exibição dos detalhes do cliente identificado.
         */
		$scope.showCustomerDetails = function() {
            const customerEditDisabledOnState = $state.current && $state.current.customerEditDisabled === true;

		    const userEditAllowed = !$scope.paymentInProgress && !customerEditDisabledOnState;

            if (!userEditAllowed) {
                AlertService.warn("Não é permitido alterações na identificação do cliente durante este processo.");
            } else {
                if ($scope.customerInSale) {
                    var scanActivated = BarcodeService.scanActivated();

                    if (scanActivated) {
                        BarcodeService.cancel();
                    }

                    const canConfirm = userEditAllowed && !CartService.cart.orderRetrieved;
                    CustomerService.showCustomerDetails(CartService.cart.customer, 'SALE', canConfirm)
                    .then(function(remover) {
                        if (remover) {
                            CartService.cart.customer = undefined;
                            CartService.cart.fiscalCustomer = undefined;
                            CartService.cart.discounts = [];
                            CartService.cart.fiscalItems.forEach(removeDiscounts);
                            CustomerService.notifyObservers();
                            $scope.customerInSale = false;
                        }
                    }).finally(function() {
                        if (scanActivated) {
                            BarcodeService.reInitScan();
                        }
                    });
                } else {
                    $scope.checkAvoidCustomerMenu();
                }
            }

        };

        $scope.customerButtonDisabled = () => {
            return $state.is('easyorder.paymentresult')
                || $state.is('easyorder.paymentlink');
        }

        /**
         *
         * Remove os descontos para cada item e atualiza valor deste sem desconto
         */
        function removeDiscounts(item) {
            item.discounts = [];
            item.totalExhib = item.unitValue * item.quantitySold;
        }

		/**
         * Controla o modo de exibição para Busca ou remoção de Vendedor
         */
        $scope.showSellerSearchDialog = function(confirmMode) {
            BarcodeService.cancel();

            var removeButtons = [
                {
                    text: '<b>Fechar</b>',
                    type: 'button-stable',
                    onTap: function(e) {
                        reInitScan();
                        return false;
                    }
                },
                {
                    text: '<b>Remover</b>',
                    type: 'button-theme',
                    onTap: function(e) {
                        CartService.registerSellerToSaleItems(undefined);
                        reInitScan();

                        return true;
                    }
                }
            ];
            var searchButtons = [
                {
                    text: '<b>Cancelar</b>',
                    type: 'button-stable',
                    onTap: function(e) {
                        reInitScan();
                        return false;
                    }
                },
                {
                    text: '<b>Pesquisar</b>',
                    type: 'button-theme',
                    onTap: function(e) {
                        var loginSrc = $scope.sellerDetails.login;

                        if (!loginSrc) {
                            e.preventDefault();
                            return;
                        }
                        $ionicLoading.show({
                            template : 'Por favor, aguarde...',
                            duration : 0
                        }).then(function() {
                            UserService.searchSeller({ login: loginSrc }, function(result) {
                                $ionicLoading.hide();
                                CartService.registerSellerToSaleItems(result.sellers[0]);

                                $scope.showSellerSearchDialog(true);
                            }, function(error) {
                                $ionicLoading.hide();
                                AlertService.restError(error, 'Não foi possível obter dados do vendedor.', $scope.showSellerSearchDialog);
                            });
                        });
                        return true;
                    }
                }
            ];
            var closeButton = [{
                text: '<b>Fechar</b>',
                type: 'button-theme',
                onTap: function(e) {
                    reInitScan();
                    return true;
                }
            }];

            if (!$state.is('easyorder.paymentresult')) {
                if (confirmMode) {
                    CartService.showSellerDialog(closeButton, $scope);
                } else if (CartService.seller) {
                    CartService.showSellerDialog(removeButtons, $scope);
                } else {
                    CartService.showSellerDialog(searchButtons, $scope);
                }
            }
        };


        $scope.hasSaleSeller = function() {
            return CartService.seller != null;
        }
        /**
         *
         */
        $scope.checkAvoidCustomerMenu = function() {
            CustomerService.checkAvoidCustomerMenu();
        };

        /**
         *
         */
        $scope.checkAvoidCovenantMenu = function() {
            CovenantService.checkAvoidCovenantMenu();
        };

        /**
         *
         */
        $scope.chooseOrderSearchRoute = function() {
            var nextState = 'easyorder.ordersearchmenu';
            var search_types = ConfigService.config.business.order_search_types;

            search_types = search_types ? search_types.split(";") : ["DETAILED"];

            if (search_types.length == 1) {
                var search_type = search_types[0];
                // apenas o detalhado é que muda o estado sem exibir a tela com ícones
                if (search_type === "DETAILED") {
                	nextState = 'easyorder.ordersearch';
                }
            }
            UtilService.navigateWithoutHistory(nextState);
        };

        /**
         *
         */
        var reInitScan = function() {
            if ($state.is($state.get('easyorder.barcode').redirectTo(null, ConfigService.config)) || $state.is('easyorder.customerbarcodesearch')) {
                BarcodeService.reInitScan();
            }
        };

        /**
         * tratador para modficação do contador de itens do carrinho.
         */
        $rootScope.$on("cart-item-quantity", () => {
            if (CartService.cart && CartService.cart.fiscalItems) {
                $scope.totalCartItems = CartService.cart.fiscalItems.reduce(function(total, current){ return MathUtils.add(total, current.displayQuantity, 3); }, 0);
            }
        });

        $rootScope.defaultBack = () => {
            $ionicHistory.goBack();
        };

        $scope.virtualSupport = function(){
            var sApp = startApp.set({
                    "application":"com.logmein.rescuemobile"
                });

                sApp.start(function() {
                    LogService.info("Aplicação Rescue Mobile iniciada.");

                }, function(err) { /* fail */
                    var appStoreApp = startApp.set({
                        "action" : "ACTION_VIEW",
                        "application":"com.android.vending",
                        "uri": "market://details?id=com.logmein.rescuemobile"
                    });

                    appStoreApp.start(function() { /* success */
                        LogService.info("Aplicação Rescue Mobile iniciada para instalação.");
                    }, function(error) {
                        LogService.error(error);
                    });
                });
        };

        $scope.$watch('cart.totalValue', newValue => screenTotalValue = newValue);
        $rootScope.$on("screen-total-value", (_, value) => {
            if (!value || isNaN(value)) {
                value = 0;
            }
            screenTotalValue = value;
        });

        $scope.totalValue = () => {
            return ConfigService.config.business.apply_charges_on_payment_value ?
                parseFloat(screenTotalValue) + parseFloat($scope.cart.additionalCharge) :
                parseFloat(screenTotalValue);
        };

        $scope.showFiscalMenuModal = () => {
            FiscalMenuModal.open();
        };

        (function() {
            var chosenMethods = ConfigService.config.business.available_payment_methods.concat(CartService.cart.customer ? CartService.cart.customer.avaliablePaymentMethods : []);

            $scope.enableChashPayment = false;
            $scope.showFooterButtons = true;
            $scope.showFooterTotal = true;

            if (chosenMethods) {
                while (chosenMethods.length) {
                    var param = chosenMethods.shift(); //Remove o primeiro e retorna.
                    if (param == "CASH") {
                        $scope.enableChashPayment = true;
                        break;
                    }
                }
            }
        })();
	});
})();
