(function () {
    'use strict';
    angular.module('easyorder.controllers.includediscounts', [
            'ln.currency',
            'ln.percent',
            'easyorder.services.includediscounts',
            'easyorder.services.log'
        ])
        .controller('IncludeDiscountsCtrl', function ($scope,
                                                        DiscountService,
                                                        CartService,
                                                        UtilService,
                                                        $ionicLoading,
                                                        AlertService,
                                                        $ionicPopup,
                                                        $ionicModal,
                                                        $stateParams,
                                                        LogService){

            $scope.changeValueType = function (valueType) {
                $scope.$broadcast('clearCurrency', 0);
                $scope.$broadcast('clearPercent', 0);
                $scope.data.valueType = valueType;
            };

            /**
             * 
             */
            $scope.applyDiscounts = function () {
                if ($scope.data){
                    if ($scope.data.dataPercent && $scope.data.dataPercent >= 0 && $scope.rule && $scope.rule.discount){
                        let value = Number($scope.data.dataPercent);
                        if (value >= 0){
                            if (value > $scope.rule.discount.max){
                                AlertService.error("Valor do desconto supera o permitido!");
                                return;
                            }else if (value < $scope.rule.discount.min){
                                AlertService.error("Valor do desconto abaixo do permitido!");
                                return;
                            }    
                        }
                    }

                    if ($scope.data.dataValue && $scope.data.dataValue >= 0 && $scope.rule && $scope.rule.discount){
                        let productValue = $scope.item.totalValue;
                        var maxPercentValue = new Number(parseFloat(productValue * ($scope.rule.discount.max/100)).toFixed(2));
                        var minPercentValue = new Number(parseFloat(productValue * ($scope.rule.discount.min/100)).toFixed(2));
                        
                        let value = Number($scope.data.dataValue);
                        if (value >= 0){
                            if ($scope.rule.discount.type === "PERCENT"){
                                if (value > maxPercentValue){
                                    AlertService.error("Valor do desconto supera o permitido!");
                                    return;    
                                } else if (value < minPercentValue){
                                    AlertService.error("Valor do desconto abaixo do permitido!");
                                    return;
                                }
                            }else{
                                if (value > $scope.rule.discount.max){
                                    AlertService.error("Valor do desconto supera o permitido!");
                                    return;
                                }else if (value < $scope.rule.discount.min){
                                    AlertService.error("Valor do desconto abaixo do permitido!");
                                    return;
                                }
                            }
                        }
                    }
                }
                $ionicLoading.show({
                    template: 'Aguarde ...',
                    duration: 0
                }).then(() => {
                    try {
                        if (!$scope.cart) {
                            DiscountService.applyInItem(DiscountService.discountParams(null, null, {
                                rule: $scope.rule,
                                type: $scope.typeCode,
                                data: $scope.data,
                                item: $scope.item
                            }), $scope.certifySuccess, $scope.certifyError);
                        } else {
                            DiscountService.applyInCouponFiscal(DiscountService.discountParams(null, null, {
                                rule: $scope.rule,
                                type: $scope.typeCode,
                                data: $scope.data
                            }), $scope.certifySuccess, $scope.certifyError);
                        }
                    } catch (error) {
                        $ionicLoading.hide();
                        LogService.error(error);
                    }
                });
            };

            /**
             * 
             */
            $scope.certifySuccess = function (resp) {
                $ionicLoading.hide();
                if (resp.authorizationInfo === "AUTHORIZED") {
                    if (!$scope.cart) {
                        $scope.applyInItem($scope.item, resp.discountValue);
                    } else {
                        $scope.applyInCart(resp.discountValue);
                    }
                } else {
                    $ionicPopup.alert({
                        title: 'Erro!',
                        template: DiscountService.errMessage(resp.authorizationInfo),
                        buttons: [{
                            text: 'Ok',
                            type: 'button button-theme',
                            onTap: function (e) {
                                $scope.openModal();
                            }
                        }]
                    });
                }
            };

            /**
             * 
             */
            $scope.certifyError = function (err) {
                $ionicLoading.hide();
                AlertService.show('Error', err.data.message, 'OK');
            };

            /**
             * 
             */
            $scope.requestAuth = function () {
                try {
                    $ionicLoading.show({
                        template: 'Aguarde ...',
                        duration: 0
                    }).then(() => {
                        if (!$scope.cart) {
                            DiscountService.authorizeDiscountInItem(DiscountService.discountParams($scope.requestauth.user, $scope.requestauth.password, {
                                    rule: $scope.rule,
                                    type: $scope.typeCode,
                                    data: $scope.data,
                                    item: $scope.item
                                }),
                                $scope.authSuccess, $scope.authError);
                        } else {
                            DiscountService.authorizeDiscountInCouponFiscal(DiscountService.discountParams($scope.requestauth.user, $scope.requestauth.password, {
                                    rule: $scope.rule,
                                    type: $scope.typeCode,
                                    data: $scope.data
                                }),
                                $scope.authSuccess, $scope.authError);
                        }
                        $scope.requestauth.user = null;
                        $scope.requestauth.password = null;
                    });
                } catch (error) {
                    LogService.error(error);
                }
            };

            /**
             * 
             */
            $scope.authSuccess = function (resp) {
                $ionicLoading.hide();
                if (resp.authorizationInfo === "AUTHORIZED") {
                    if (!$scope.cart) {
                        $scope.applyInItem($scope.item, resp.discountValue);
                    } else {
                        $scope.applyInCart(resp.discountValue);
                    }
                } else {
                    $ionicPopup.alert({
                        title: 'Erro!',
                        template: DiscountService.errMessage(resp.authorizationInfo),
                        buttons: [{
                            text: 'Ok',
                            type: 'button button-theme'
                        }]
                    });
                }
            };

            /**
             * 
             */
            $scope.authError = function (err) {
                $ionicLoading.hide();
                AlertService.show('Error', err.data.message, 'OK');
            };

            /**
             * 
             */
            $scope.applyInItem = function (item, resp) {
                CartService.cart.fiscalItems.forEach(fiscalItem => {
                    if (fiscalItem.sequenceId == item.sequenceId && fiscalItem.automationCode == item.automationCode) {
                        let discount = {
                            value: resp,
                            type: "VALUE",
                            source: "MANUAL_RULE",
                            externalId: $scope.rule.code,
                            factor: item.quantitySold
                        }
                        LogService.info("Desconto a ser aplicado: ", discount);
                        
                        fiscalItem.removeDiscountBySource(discount.source);
                        fiscalItem.addDiscount(discount);
                        CartService.updateInCart(fiscalItem);
                        UtilService.navigateWithoutHistory('easyorder.cart', {});
                    }
                });
            };

            /**
             * 
             */
            $scope.applyInCart = function (resp) {
                let discount = {
                    value: resp,
                    type: "VALUE",
                    source: "RULE",
                    externalId: $scope.rule.code
                };
                LogService.info("Aplicando desconto ao carrrinho: ", discount);
                
                CartService.removeDiscountBySource(discount.source);
                CartService.addDiscount(discount);
                CartService.updateCartTotalValue();
                UtilService.navigateWithoutHistory('easyorder.purchasesummary', {});
            };

            /**
             * 
             */
            $scope.openModal = function () {
                $scope.modal.show();
            };

            /**
             * 
             */
            $scope.closeModal = function () {
                $scope.modal.hide();
            };

            /**
             * 
             */
            $scope.$on('$destroy', function () {
                $scope.modal.remove();
            });

            /**
             * 
             */
            $ionicModal.fromTemplateUrl('views/requestauthmodal.html', {
                scope: $scope,
                focusFirstInput: true,
                animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.modal = modal;
            });

            (function () {
                $scope.data = {};
                $scope.data.dataValue = 0.00;
                $scope.data.dataPercent = 0;
                $scope.data.valueType = "VALUE";
                $scope.requestauth = {};
                $scope.requestauth.user = null;
                $scope.requestauth.password = null;
                $scope.cart = $stateParams.discountData.cart;
                $scope.item = $stateParams.discountData.item;
                $scope.rule = $stateParams.discountData.rule;
                $scope.typeCode = $stateParams.discountData.type.code;
                $scope.discountType = $stateParams.discountData.type.description;
            })();
        });
})();
