(function() {
    'use strict';

    angular.module('easyorder.controllers.login',[
        'ngStorage',
        'easyorder.services.order',
        'easyorder.services.product',
        'easyorder.services.config',
        'easyorder.services.user',
        'easyorder.services.eft',
        'easyorder.services.config',
        'easyorder.services.cart',
        'easyorder.services.alert',
        'easyorder.services.log',
        'easyorder.services.telemetry'
    ])

    /**
     * Apenas para evitar que valores gravados como String sejam enviados para um campo Number.
     */
    .directive('stringToNumber', function() {
        return {
            require: 'ngModel',
            link: function(scope, element, attrs, ngModel) {
                ngModel.$formatters.push(function(value) {
                    return parseInt(value);
                });
            }
        };
    })

    .controller('LoginCtrl', function($scope,
                                      $rootScope,
                                      $timeout,
                                      $ionicModal,
                                      $ionicPopup,
                                      $ionicLoading,
                                      $q,
                                      ConfigService,
                                      ConfigUpdateService,
                                      UserService,
                                      toaster,
                                      CartService,
                                      AlertService,
                                      UtilService,
                                      LogService,
                                      TelemetryService,
                                      FiscalMenuModal,
                                      EFTService,
                                      BarcodeService) {

        const INCOMPLETE_CONFIG_MSG = "A configuração deste dispositivo está incompleta. Favor realizar uma nova configuração.";
        const INCOMPLETE_NSU_MSG = "NSU do terminal inválida. Favor realizar uma nova configuração.";
        const INCOMPLETE_AUTH_MSG = "Informe usuário / senha.";

        $rootScope.selectedTheme = ConfigService.config.theme;
        /* FIXME: verificar se essas dua variáveis abaixo são realmente utilizadas */
        $scope.APP_NAME = APP_NAME;
        $scope.APP_NAME_DESC = APP_NAME_DESC;
        $scope.confirmPopup;
        
        var spinning_class        = 'fa fa-spinner fa-pulse fa-2x fa-fw dark';
        var fail_class            = 'fa fa-times fa-2x assertive';
        var disabled_fail_class   = 'fa fa-exclamation-triangle fa-2x energized';
        var success_class         = 'fa fa-check fa-2x balanced';

        const self = this;
        
        var loadSettingsFile = function() {
            $scope.config = ConfigService.config;
        };
        
        $scope.toastSox = function(){
            if (ConfigService.config.business.sox_enabled) {
                $ionicLoading.show();

                UserService.soxToast(function(result) {
                    $ionicLoading.hide();
                    toaster.pop('info', "Pós-login", result.posLoginMessage);
                }, function(error) {
                    $ionicLoading.hide();
                    LogService.error(error.data.message);
                });
            }
        };

        $scope.login = (user) => {
            if (user == null) {
                return;
            }
            $scope.loadingState = true;
            $scope.disableLoginInput = true;
            
            this.checkConfigLoaded()
                .then(this.checkExchangedLWS)
                .then(this.checkNsuStorage)
                .then(() => this.checkEmptyAuthInfo(user))
                .then(() => {
                    return UserService.login(user.username, user.password).then(
                        (success) => {
                            ConfigService.updateAppSettings();
                            CartService.initCart();
                            $scope.toastSox();
                            
                            const request = {
                                storeId : ConfigService.config.storeNumber,
                                componentId : ConfigService.config.terminalNumber,
                                componentType : 'CAIXA_RAPIDO',
                                metricName : 'login_usuario'
                            };
                            
                            //Caso tenha conseguido realizar o login, registrar a metrica
                            TelemetryService.trackMetric(request)
                                .then((result) => {
                                    LogService.info(result, 'RESULTADO');
                                })
                                .catch((error) => {
                                        LogService.error(error.data.message);
                                })
                                .finally(() => {
                                    // Telas com redirect sempre causam erro
                                    // de navegação lançado pelo ionic.
                                    // Logo não devemos retornar a promessa neste caso
                                    // pode ser exibido um erro em tela porque interrompemos
                                    // a troca da tela anterior.
                                    UtilService.navigateToMainScreen();
                                });                            
                        },
                        (error) => {
                            if (error && error.data && error.data.code == 307) {
                                $scope.checkLoginPermission(user.username, user.password);
                            } else if (typeof error == "string") {
                                AlertService.error(error);
                            } else {
                                AlertService.restError(error, 'Não foi possível se autenticar no servidor!');
                            }
                        }
                    );
                })
                .catch(error => {
                    if(error.data && error.data.message) {
                        AlertService.restError(error)
                    } else {
                        AlertService.warn(error);
                    }
                })
                .finally(() => {
                     $scope.disableLoginInput = false;
                     $scope.loadingState = false;
                });
        };

        this.checkConfigLoaded = () => {
            return this.loginValidation(
                !ConfigService.config.configLoaded,
                INCOMPLETE_CONFIG_MSG
            );
        };

        this.checkExchangedLWS = () => {
            const current = ConfigService.config.linxwebservices.host;
            const before = ConfigService.systemData.currentLinxWebservicesHost;
            const errMsg = `O host do Linx WebServices atual '${current}' difere da última configuração '${before}'. Favor realizar uma reconfiguração`;

            return this.loginValidation(
                ConfigService.config.linxwebservices.host != ConfigService.systemData.currentLinxWebservicesHost,
                errMsg
            );
        };

        this.checkNsuStorage = () => {
            return $rootScope
                .dataSourceManager
                .checkNsuStorage()
                .catch(() => {
                    return $q.reject(INCOMPLETE_NSU_MSG);
                });
        };

        this.checkEmptyAuthInfo = (user) => {
            this.loginValidation(
                user.username == null || user.username.length == 0 || user.password == null || user.password.length == 0,
                INCOMPLETE_AUTH_MSG
            );
        };

        this.loginValidation = (errCondition, errMsg, noConcat) => {
            return $q((resolve, reject) => {
                if (errCondition) {
                    reject(noConcat ? errMsg : this.concatError(errMsg));
                    return;
                }
                resolve();
            });
        };

        this.concatError = (msg) => {
            const toConcat = ConfigService.config.errorMessage ?
                `<br><br>${ConfigService.config.errorMessage}` : '';
            return `${msg}${toConcat}`
        };

        $ionicModal.fromTemplateUrl('views/config.html', {
            scope: $scope,
            focusFirstInput: false,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });

        $scope.openModal = function() {
            loadSettingsFile();
            $scope.modal.show();
        };

        $scope.closeModal = function() {
            stopScanning();
            $scope.modal.hide();
        };

        $ionicModal.fromTemplateUrl('views/pwd-change.html', {
            scope: $scope,
            focusFirstInput: false,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.changePwdModal = modal;
        });

        $scope.openPwdChangeModal = function() {
            $scope.changePwdData = {};
            $scope.changePwdModal.show();
        };

        $scope.closePwdChangeModal = function() {
            $scope.changePwdModal.hide();
        };

        $scope.openFiscalMenuModal = function() {
            FiscalMenuModal.open();
        }

        $scope.$on('$destroy', function() {
            $scope.modal.remove();
        });

        $scope.newStoreNumber = { value: "" };
        
        $scope.editStoreNumber = function() {
            AlertService.confirm("Configuração necessária para LWS coorporativo!\n\nInforme o número da loja com 4 dígitos, ex: 0001").then(function(result) {
                if (result) {
                    $scope.confirmPopup = $ionicPopup.show({
                        scope: $scope,
                        title: 'Edição de número de loja',
                        templateUrl: 'views/editstorenumber.html',
                        buttons: [
                        {
                            text: 'Cancelar',
                        },
                        {
                            text: 'Salvar',
                            type: 'button-theme',
                            onTap: function(e) {
                                if ($scope.newStoreNumber.value && $scope.newStoreNumber.value.length == 4) {
                                    ConfigService.config.storeNumber = $scope.newStoreNumber.value;
                                } else {
                                    e.preventDefault();
                                }
                            }
                        }
                    ]});
                    
                }                             
            });
        }

        /**
         *
         */
        $scope.validateLinxWebServicesCommunication = function() {
            loadBaseConfiguration()
                .then(loadAppConfigurations)
                .then(loadNsuConfiguration)
                .then(loadEFTConfiguration)
                .then(endDeviceConfiguration)
                .catch(error => {
                    $scope.loadingConfig = false;

                    if (typeof error == "string") {
                        return AlertService.error(error);
                    }

                    return AlertService.restError(error, 'Erro ao carregar configurações!')
                });
        };

        /**
         *
         */
        var loadBaseConfiguration = function() {
            var defered = $q.defer();

            ConfigUpdateService.loadBaseConfigurations()
            .then((success) => {
                $scope.deviceRegisterClass = success_class;
                defered.resolve();
            }, (error) => {
                $scope.deviceRegisterClass = fail_class;
                $scope.appConfigClass = disabled_fail_class;
                $scope.nsuConfigClass = disabled_fail_class;
                $scope.eftConfigClass = disabled_fail_class;
                defered.reject(error);
            });
            return defered.promise;
        };

        /**
         *
         */
        var loadAppConfigurations = function() {
            var defered = $q.defer();

            ConfigUpdateService.loadAppConfigurations()
            .then((success) => {
                $scope.appConfigClass = success_class;
                ConfigService.updateAppSettings();
                defered.resolve();
            }, (error) => {
                $scope.appConfigClass = fail_class;
                $scope.nsuConfigClass = disabled_fail_class;
                $scope.eftConfigClass = disabled_fail_class;
                defered.reject(error);
            });
            return defered.promise;
        };

        /**
         *
         */
        const loadNsuConfiguration = () => {
            return $rootScope
                .dataSourceManager
                .checkNsuStorage()
                .catch(() => {
                    // devemos buscar se existe nsu no dia
                    return ConfigUpdateService.nextNsu();
                }).then(() => {
                    $scope.nsuConfigClass = success_class;
                }).catch(error => {
                    $scope.nsuConfigClass = fail_class;
                    $scope.eftConfigClass = disabled_fail_class;
                    return $q.reject(error);
                });
        };

        const loadEFTConfiguration = () => {
            return EFTService.activate({ force: true }).then(() => {
                $scope.eftConfigClass = success_class;
            }).catch(err => {
                $scope.eftConfigClass = fail_class;
                return $q.reject(err);
            });
        };

        /**
         *
         */
        var endDeviceConfiguration = () => {
            ConfigService.updateAppSettings();

            LogService.info(`Removendo arquivos de configuração.`);
            $rootScope.dataSourceManager.remove().finally(() => {
                LogService.info(`Persistindo arquivos de configuração.`);
                $rootScope.dataSourceManager.persist()
                .catch(LogService.error)
                .finally(() => {
                    LogService.logStatus(ConfigService.config.terminalNumber,
                            ConfigService.config.storeNumber, ConfigService.config.companyNumber,
                            ConfigService.nsuControll.currentNSU(), ConfigService.config.linxwebservices.host,
                            ConfigService.config.linxwebservices.port);

                    $scope.loadingConfig = false
                });
            });
       
            return $q.resolve();
        };

        $scope.showParameters = function() {
            const wasScanning = stopScanning();

            $ionicPopup.show({
                scope: $scope,
                title: 'Configuração Atual',
                templateUrl: 'views/currentconfig.html',
                cssClass: 'currentConfigClass',
                buttons: [{
                    text: 'OK',
                    type: 'button-theme'
                }]
            }).finally(() => {
                if (wasScanning) {
                    $scope.toggleScanner();
                }
            });
        };

        $scope.checkLoginPermission = function(usernameFromScreen,passwordFromScreen) {
            $scope.requestauth = {};
            var authPopup = $ionicPopup.show({
                templateUrl: 'views/requestauthlogin.html',
                scope: $scope,
                buttons: [
                    { text: 'Cancelar' },
                    {
                        text: '<b>Autorizar</b>',
                        type: 'button-theme',
                        onTap: function(event) {
                            var sucesso = function(result) {
                                LogService.info(result, 'RESULTADO');
                                authPopup.close();
                                if (result.authorized == true) {
                                    LogService.info('Usuário autorizado pos inserir senha e login')
                                } else {
                                    LogService.warn('Usuário negado')
                                    AlertService.error('Autorização negada!')
                                }
                            };

                            var error = function(error) {
                                LogService.error(error);
                                AlertService.restError(error, "Erro ao consultar autorização!");
                            };

                            if (!usernameFromScreen || !$scope.requestauth.superUser) {
                                AlertService.error("Usuário inválido!");
                            } else if (!passwordFromScreen || !$scope.requestauth.superPassword) {
                                AlertService.error("Senha inválida!");
                            } else {
                                UserService.authorizedLogin(usernameFromScreen,
                                    passwordFromScreen,
                                    $scope.requestauth.superUser,
                                    $scope.requestauth.superPassword).then(function(success) {
                                        authPopup.close();
                                        $scope.toastSox();

                                        UtilService.navigateToMainScreen();
                                    }, error);
                            }
                            event.preventDefault();
                        }
                    }
                ]
            });
        };

        $scope.showLocalInfo = function() {
            const wasScanning = stopScanning();

            $ionicPopup.show({
                scope: $scope,
                title: 'Informações do dispositivo',
                templateUrl: 'views/currentdeviceinfo.html',
                cssClass: 'currentConfigClass',
                buttons: [{
                    text: 'OK',
                    type: 'button-theme'
                }]
            }).finally(() => {
                if (wasScanning) {
                    $scope.toggleScanner();
                }
            });
        };

        $scope.changePassword = function() {
            if ($scope.changePwdData.currentPwd === $scope.changePwdData.password) {
                AlertService.error('Nova senha é igual à anterior!');
            } else if ($scope.changePwdData.password !== $scope.changePwdData.newPwdConfirmation) {
                AlertService.error('Nova senha difere da confirmação da senha!');
            } else if ($scope.changePwdData.password.length < 8 ) {
                // TODO parametrizar o CXR, validar as regras do BD no LWS e mostrar alertas em tela ao invés de popup
                AlertService.error('Senha deve ter no mínimo 8 caracteres!');
            } else {
                $scope.doChangePassword();
            }
            
        };

        $scope.doChangePassword = function() {
            $scope.loadingState = true;
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(() => {
                UserService.changePassword($scope.changePwdData.username, $scope.changePwdData.currentPwd, $scope.changePwdData.password).then(
                    (success) => {
                        $ionicLoading.hide().then(() => {
                            $scope.loadingState = false;
                            AlertService.info("Senha atualizada!").then(() => {
                                $scope.closePwdChangeModal();
                            });
                        });
                    },
                    (error) => {
                        return $ionicLoading.hide().then(() => {
                            $scope.loadingState = false;
                            return AlertService.restError(error, 'Erro ao atualizar senha!');  
                        });
                    }
                );
            });
        };

        $scope.testConfig = function() {
            stopScanning();

            $scope.deviceRegisterClass = spinning_class;
            $scope.appConfigClass = spinning_class;
            $scope.nsuConfigClass = spinning_class;
            $scope.eftConfigClass = spinning_class;
            $scope.loadingConfig = true;

            $scope.confirmPopup  = $ionicPopup.show({
                scope: $scope,
                title: 'Carregando configurações...',
                templateUrl: 'views/configchecklist.html',
            });
               
            $timeout(function() {
                $scope.validateLinxWebServicesCommunication();
            }, 1500);
        };

        $scope.closePopupTestConfig = function(){
            if (!$scope.loadingConfig) {
                $scope.confirmPopup.close();
            }
        };

        $scope.toggleScanner = ($event) => {
            if ($rootScope.linxWebservicesExternallyConfigured) {
                return AlertService.error(
                    'Aplicação foi configurada remotamente, não será possível ler QRCode');
            }

            if ($scope.scanning) {
                stopScanning();
            } else {
                const boxEl = document.getElementById('stx-cloud-box');
                const boxHdrEl = document.getElementById('stx-cloud-box-hdr');

                const bPadding = window.innerHeight - boxEl.clientHeight - boxHdrEl.clientHeight - 1;

                BarcodeService.init(function(session) {
                    const barcode = session[0];
                    const cfg = JSON.parse(barcode);

                    if (isValidCfgData(cfg)) {
                        const cloudUrl = new URL(cfg.cloudUrl);

                        $scope.config.linxwebservices.host = cloudUrl.hostname;
                        $scope.config.linxwebservices.port = urlPort(cloudUrl);
                        $scope.config.storeNumber = cfg.storeNumber;

                        $scope.testConfig();
                    }
                }, {
                    inSizeCam: [`10/120/10/${bPadding}`, '2/64/2/148'],
                    outSizeCam: ['275/44/0/128', '275/64/0/148'],
                    extraOpts: {
                        qr: true,
                        code128: false,
                        dataMatrix: false,
                        pdf417: false,
                        code39: false,
                        upce: false,
                        ean8: false,
                        ean13AndUpc12: false,
                        scanningHotSpotHeight: undefined
                    }
                });
                $scope.scanning = true;
            }

            $event.preventDefault();
        };

        function stopScanning() {
            if ($scope.scanning) {
                BarcodeService.cancel();
                $scope.scanning = false;
                return true;
            } else {
                return false;
            }
        }

        function urlPort(url) {
            if (url.port == null || url.port == '') {
                if (url.protocol == 'https:') {
                    return '443';
                }
                throw new Error(`Protocolo '${url.protocol}' não suportado`);
            }
            return url.port;
        }

        function isValidCfgData(cfg) {
            return cfg != null
                && cfg.appType == APP_TYPE
                && isValidStoreNumber(cfg.storeNumber)
                && isValidUrl(cfg.cloudUrl)
        }

        function isValidStoreNumber(storeNumber) {
            return storeNumber != null && storeNumber.length == 4 && !isNaN(storeNumber);
        }

        function isValidUrl(url) {
            if (url == null) {
                return false;
            }
            try {
                new URL(url);
                return true;
            } catch(err) {
                LogService.error(err);
                return false;
            }
        }

        (function() {
            UserService.logoutUser();
            $scope.loadingConfig = false;
            $scope.config = ConfigService.config;
            $scope.nsuControll = ConfigService.nsuControll;

            if (!$scope.config.linxwebservices) {
                $scope.config.linxwebservices = { host: '', port: 0, secure: false};
            }

            if (ConfigService.config.business.sox_enabled) {
                $ionicLoading.show();

                UserService.soxToast(function(result) {
                    $ionicLoading.hide();
                    toaster.pop('info', "Pré-login", result.preLoginMessage);
                }, function(error) {
                    $ionicLoading.hide();
                    LogService.error(error.data.message);
                });
            }
            
            $scope.loadingState = false;

            var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;

            $scope.isiOS = !iOS;

            var iw = (iOS) ? screen.width : window.innerWidth,
                ih = (iOS) ? screen.height : window.innerHeight;

            $scope.screenHeight = ih - 44;
            $scope.screenWidth = iw;
            ConfigService.updateAppSettings();
        })();
    });
})();
