(function() {
    'use strict';

	angular.module('easyorder.controllers.mobilerechargephonenumber', [
    'easyorder.services.eft',
	'easyorder.services.cart',
    'easyorder.services.util',
    'easyorder.services.alert',
    'easyorder.services.log'
    ])

	.controller('MobileRechargePhoneNumberCtrl', function($scope,
                                       $ionicLoading,
                                       $stateParams,
                                       CartService,
                                       UtilService,
                                       MobileRechargeService,
                                       AlertService,
                                       LogService,
                                       EFTService,
                                       ConfigService,
                                       UserService) {

        $scope.mobileRechargeOption = $stateParams.mobileRechargeOption;
        
        $scope.recharge = {
                
        };

        function phoneNumberValidated() {
            var item = {};
            item.operationType = "MOBILE_RECHARGE";
            item.value = $scope.mobileRechargeOption.value;
            item.id = $scope.mobileRechargeOption.code;
            item.documentNumber = $scope.recharge.phone;
            CartService.addServiceItem(item);
            $scope.loadingState = false;
            $ionicLoading.hide();
            UtilService.navigateWithoutHistory('easyorder.cart');               
        }
        
        function validationError(error) {
            $scope.loadingState = false;
            $ionicLoading.hide();
            LogService.error(error);
            AlertService.restError(error, 'Erro ao consultar número de telefone!'); 
        } 

        const checkPhoneNumberLWS = function() {
            LogService.info('Consultando número de telefone ' + $scope.recharge.phone + ' via LWS.');
            $ionicLoading.show({
                template : 'Consultando número de telefone, aguarde...',
                duration : 0
            }).then(function() {
                const nsuLastDigit = (ConfigService.nsuControll.currentCartNSU() + "").slice(-1);
                const storeNumber = ("0000" + ConfigService.config.storeNumber).slice(-4);
                const terminalNumber = ("000" + ConfigService.config.terminalNumber).slice(-3);
                // id de transação [pode se repetir]: loja [4], terminal [3], último dígito da nsu [1]
                const transactionId = storeNumber + terminalNumber + nsuLastDigit;
                const validateMobileRechargeRequest = {
                    transactionId: transactionId,
                    phoneNumber: $scope.recharge.phone, 
                    amount: $scope.mobileRechargeOption.value
                };
                MobileRechargeService.validate(validateMobileRechargeRequest).then(phoneNumberValidated).catch(validationError);
                

            });
        };

        $scope.doMobileRecharge = function() {
            if (CartService.getCurrentOperation() === 'FISCAL') {
                AlertService.warn("Produtos fiscais já adicionados ao carrinho. Não é possível realizar recarga de crédito digital.");
            } else {
                var serviceItems = CartService.cart.serviceItems;
                
                if (serviceItems && serviceItems.length > 0) {
                    AlertService.error("Não é possível efetuar recarga. Já existe um ítem no carrinho!");
                } else {
                    if (MobileRechargeService.isLWSMode()) {
                        checkPhoneNumberLWS();
                    } else {
                        checkPhoneNumberTEF();
                    }
                    
                }
            }
        };

        $scope.clearDisplay = function() {
            $scope.$apply(function () {
                $scope.message = "";
            });
            LogService.info("clearDisplay");
        };

        $scope.displayMessage = function(message) {
            if (message) {
                $scope.$apply(function () {
                    $scope.message = message;
                });
            }
            LogService.info("displayMessage: " + $scope.message);
        };
        
        $scope.paymentDone = function() {
            var item = {};
            item.operationType = "MOBILE_RECHARGE";
            item.value = $scope.mobileRechargeOption.value;
            item.id = $scope.mobileRechargeOption.code;
            item.documentNumber = $scope.recharge.phone;
            CartService.addServiceItem(item);
            $scope.loadingState = false;
            $ionicLoading.hide();
            UtilService.navigateWithoutHistory('easyorder.cart');               
        };
        
        $scope.paymentError = function() {
            $scope.loadingState = false;
            $ionicLoading.hide();
            AlertService.error('Erro ao verificar número de telefone!');    
        };
        
        const checkPhoneNumberTEF = function() {
            LogService.info('Consultando número de telefone ' + $scope.recharge.phone + ' via TEF.');
            $ionicLoading.show({
                template : 'Verificando número de telefone, aguarde...',
                duration : 0
            }).then(function() {
                var params = {
                    transactionType: EFTService.transactionType.CHECK_PHONE_NUMBER,
                    operatorCode:       UserService.operatorCode(),
                    amount: $scope.mobileRechargeOption.value,
                    mobileOperatorCode: 'LD',
                    DDD: $scope.recharge.phone.substring(0,2), 
                    phoneNumber: $scope.recharge.phone.substring(2,11)
                };

                EFTService.executeTransaction(params, $scope);
            });
        };

    });
})();
