(function () {
    'use strict';
    angular.module('easyorder.controllers.operations', [])
        .controller('OperationsCtrl', function (
            $scope,
            $rootScope,
            $stateParams,
            $state,
            UserService,
            UtilService,
            CartService,
            CustomerService,
            ConfigService) {

            $scope.requestauth = {};
            $scope.customer = $scope.cart.customer;
            $scope.canExchange = $rootScope.operations && ($rootScope.operations.includes("EXCHANGE") || $rootScope.operations.includes("EXCHANGE_V1"));
            $scope.canExchangeV2 = $rootScope.operations && $rootScope.operations.includes("EXCHANGE_V2");
            $scope.canExchangeV3 = $rootScope.operations && $rootScope.operations.includes("EXCHANGE_V3");

            $scope.canExchangeLogic = $rootScope.operations && $rootScope.operations.includes("EXCHANGE_LOGIC");
            $scope.canReturn = $rootScope.operations && $rootScope.operations.includes("RETURN");
            $scope.canDonate = $rootScope.operations && $rootScope.operations.includes("DONATION");
            $scope.canReprint = $rootScope.operations && $rootScope.operations.includes("REPRINT");
            $scope.canBillingPayment = $rootScope.operations && $rootScope.operations.includes("BILLING_PAYMENT");
            $scope.canDetachedChangeRFIDTagHeader = $rootScope.operations && $rootScope.operations.includes("DETACHED_CHANGE_RFID_TAG")
                && ConfigService.config.business.enables_rfid_reading;
            $scope.canRFIDDeactivation = $rootScope.operations && $rootScope.operations.includes("RFID_DEACTIVATION")
                && ConfigService.config.business.enables_rfid_reading;

            $scope.rfidEnabledTagHeader = ConfigService.config.business.epc_enable_prefix;
            $scope.rfidDisabledTagHeader = ConfigService.config.business.epc_disable_prefix;
            $scope.canAnnulment = $rootScope.operations && $rootScope.operations.includes("ANNULMENT");

            $scope.disabled = () => {
                if(!$scope.customer) {
                    return {
                        "opacity": "0.35",
                        "cursor": "auto",
                        "pointer-events": "none"
                    };
                }
            };

            $scope.donation = () => {
                navigateToManagedScreen('DONATION', DONATION, 'easyorder.donation');
            };

            $scope.voucherReturn = () => {
                navigateToTransactionSearch('RETURN');
            };

            $scope.annulment = () => {
                navigateToManagedScreen('ANNULMENT', ANNULMENT, 'easyorder.transactionsearch', null, 'Anulação da Venda');
            };

            $scope.exchange = () => {
                navigateToTransactionSearch('EXCHANGE');
            };

            $scope.exchangeV2 = () => {
                clearCartItems()
                .then(() => navigateToExchangeMenu('EXCHANGE'));
            };

            $scope.exchangeV3 = () => {
                clearCartItems()
                .then(() => {
                    const transitionParams = {
                        flowHistory: {
                            flow: [$state.current.name],
                            params: [$stateParams]
                        }
                    };
                    return UtilService.navigateWithoutHistory('easyorder.exchangetransactioncustomerIdentificationv3', transitionParams);
                });
            };

            function isValidOption(option) {
                return option === 'DOCUMENT' || option === 'NSU' || option === 'LAST_STORE_TRANSACTIONS' || option === 'LAST_TERMINAL_TRANSACTIONS';
            }

            $scope.reprint = () => {
                const reprintOptions = ConfigService.config.business.reprint_options ? ConfigService.config.business.reprint_options.toUpperCase().split(';') : [];
                const reprintOptionsFiltered = reprintOptions.filter(option => isValidOption(option));

                const transitionParams = {
                    flowHistory: {
                        flow: [$state.current.name],
                        params: [],
                    },
                    options: reprintOptionsFiltered,
                    option: reprintOptionsFiltered.length > 1 ? null: reprintOptionsFiltered[0]

                };

                // se tiver mais de uma opção ou se incluir alguma das buscas de transação (não tem tela de filtro), exibe o menu para escolher
                if (reprintOptionsFiltered.length > 1 || reprintOptionsFiltered.includes('LAST_STORE_TRANSACTIONS') || reprintOptionsFiltered.includes('LAST_TERMINAL_TRANSACTIONS')) {
                    UtilService.navigateWithoutHistory('easyorder.reprintoptions', transitionParams);
                } else {
                    // por padrão (até pelo legado, mantém a tela de busca)
                    UtilService.navigateWithoutHistory('easyorder.reprintsearch', transitionParams);
                }
            };

            $scope.billingPayment = () => {
                clearCartItems()
                .then(() => navigateToOperationScreen('BILLING_PAYMENT', UserService.currentUser, 'easyorder.billingpaymentmenu'));
            };

            $scope.detachedChangeRFIDTagheader = (changeHeadersTo) => {
                clearCartItems()
                .then(() => navigateToManagedScreen('DETACHED_CHANGE_RFID_TAG',
                        DETACHED_DISABLE_RFID_TAGS,
                        'easyorder.rfid-detached-change-tag-header',
                        { changeHeadersTo:  changeHeadersTo},
                        'Alteração Avulsa de Tags RFID'));
            };

            $scope.rfidDeactivationMenu = () => {
                clearCartItems()
                .then(() => navigateToOperationScreen('RFID_DEACTIVATION', UserService.currentUser, 'easyorder.rfid-tag-deactivation-menu'));
            };

            async function clearCartItems() {
                CartService.undoCampaignDiscounts();
                CartService.cart.fiscalItems.length = 0;
                $rootScope.$emit("cart-item-quantity");
                $rootScope.$emit("screen-total-value", 0);
            }

            function navigateToOperationScreen(op, authorizer, state, stateParams) {
                return UtilService.navigateWithoutHistory(state, angular.merge({
                    op: op,
                    authorizer: authorizer
                }, stateParams || {}));
            }

            function navigateToManagedScreen(op, permission, state, stateParams, permissionDesc) {
                return UserService.checkRequestAuthorization(permission, permissionDesc)
                .then(authorizer => navigateToOperationScreen(op, authorizer, state, stateParams));
            }

            function navigateToTransactionSearch(op) {
                navigateToOperationScreen(op, UserService.currentUser, 'easyorder.transactionsearch');
            }

            function navigateToExchangeMenu(op) {
                return navigateToManagedScreen(op,
                    EXCHANGE_OPERATION,
                    'easyorder.exchangetransactioncustomerIdentification',
                    undefined,
                    'Operação de Troca');
            }

            $scope.checkAvoidCustomerMenu = () => {
                CustomerService.checkAvoidCustomerMenu();
            };
        });
})();
