(function() {
    'use strict';

    angular.module('easyorder.controllers.order.search.list', [])

    .controller('OrderSearchListCtrl', function($scope,
                                                $stateParams,
                                                $ionicLoading,
                                                $ionicModal,
                                                $ionicPopup,
                                                $q,
                                                $timeout,
                                                OrderService,
                                                ConfigService,
                                                UtilService,
                                                AlertService,
                                                CartService,
                                                CustomerService,
                                                PaymentLinkService,
                                                MessagingService,
                                                UserService,
                                                LogService,
                                                DonationService,
                                                $rootScope) {
        $scope.selectedStatus = "";
        const operations = ConfigService.config.business.order_search_screen_operations;
        $scope.selectedOrdersToMerge = [];

        $ionicModal.fromTemplateUrl('views/order/order-modal.html', {
            scope: $scope,
            focusFirstInput: false,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });

        $scope.openModal = function() {
            $scope.modal.show();
        };

        $scope.closeModal = function() {
            delete $scope.orderDetail;
            $scope.modal.hide();
        };

        function hideModals() {
			return hideModal($scope.modal);
		}

		function hideModal(modal) {
			return $q((resolve) => {
				if (modal != null && modal.isShown()) {
					modal.hide()
						.finally(() => resolve());
				} else {
					resolve();
				}
			});
		}

        $scope.$on('$destroy', function() {
            $scope.modal.remove();
        });

        /**
         *
         */
        $scope.showFurtherInfo = (order) => {
              $scope.orderDetail = order;
              $scope.openModal();
        };

        /**
         *
         */
        $scope.requestCancel = (order) => {
            AlertService.confirm(`Deseja solicitar o cancelamento do Pedido '${order.orderNumber}'?`)
            .then(result => {
                if (result) {
                    $ionicLoading.show().then(() => {
                        const orderRequest = JSON.parse(JSON.stringify(order));
                        orderRequest.status = 'CANCEL_REQUESTED';

                        return OrderService.updateAsync(orderRequest)
                        .finally($ionicLoading.hide)
                        .then(() => {
                            order.status = 'CANCEL_REQUESTED';
                            return AlertService.info(`Cancelamento solicitado para Pedido '${order.orderNumber}'`);
                         })
                        .catch(AlertService.restError);
                    });
                }
            })
        };

        /**
         *
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();

                UtilService.navigateWithoutHistory(backState, params);
            }
        };

        /**
         *
         */
        $scope.searchOrder = (status) => {
            return $ionicLoading.show({
                template : 'Consultando pedidos...',
                duration : 0
            }).then(() => {
                let params = {};
                params.companyNumber = ConfigService.config.companyNumber;
                params.destinationStoreNumber = ConfigService.config.storeNumber;
                params.filterFromDate = new Date();
                params.filterFromDate.setDate(params.filterFromDate.getDate() - ConfigService.config.business.search_orders_return_date_minus);
                params.document = $stateParams.orderCustomerDocument === 'NONE' ? undefined : $stateParams.orderCustomerDocument;
                params.status = status && status.length ? status : undefined;

                return OrderService.list(params)
                    .then(result =>
                        $scope.orderList = (result.orders || []).filter(order => mayShowOrder(order, params.status))
                    ).finally($ionicLoading.hide);
            })
            .catch(AlertService.restError)
        };

        // FIXME: remover esta trava quando o PDV
        // compatibilizar os links de Pagamento com
        // o CXR, é apenas uma forma rápida de validar
        // se é um pedido do PDV
        function mayShowOrder(order, status) {
            const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

            if (!$scope.hasPaymentLink(order)) {
                return $scope.filterOrderByStatus(order, status);
            }

            if (paymentLinkProvider == 'PAGAR_ME') {
                // pedidos do PDV no momento sempre terão este campo
                // nulo
                return $scope.hasPaymentLinkState(order) && $scope.filterOrderByStatus(order, status);
            }

            return $scope.filterOrderByStatus(order);
        }

        /**
         * Filtra os pedidos que estão com status existentes na lista de filtro de pedidos por status
         * @param {Pedido} order 
         * @returns Pedido válido dentro da lista de status permitidos
         */
        $scope.filterOrderByStatus = (order, status) => {
            if (ConfigService.config.business.order_status_filter.length > 0 && !status) {
                return ConfigService.config.business.order_status_filter.filter(filterStatus => order.status == filterStatus).length > 0;
            } 
            return true;
        }

        /**
         *
         * @param {Pedido} order Pedido a ser exibido
         * @returns Se o pedido foi pago com link de pagamento
         */
        $scope.hasPaymentLink = (order) => {
            return getFirstPaymentLink(order) != null;
        }

        $scope.hasTerminalNumber = (order) => {
            return order
                && order.terminalNumber
                && order.terminalNumber != "0";
        }

        $scope.hasNsuNumber = (order) => {
            return order
                && order.nsu
                && order.nsu != "0";
        }

        $scope.hasPaymentLinkState = (order) => {
            const paymentLink = getFirstPaymentLink(order);
            return paymentLink
                && paymentLink.paymentState != null;
        };

        $scope.hasPendingPaymentLink = (order) => {
            const paymentLink = getFirstPaymentLink(order);
            return paymentLink
                && paymentLink.paymentState == 'IN_PROGRESS';
        };

        /**
         *
         * @param {Pedido} order Pedido
         * @returns Retorna a descrição do estado do primeiro pagamento realizado com Link de Pagamento no Pedido
         */
        $scope.getFirstPaymentLinkDesc = (order) => {
            const paymentLink = getFirstPaymentLink(order);
            return getPaymentStateDescription(paymentLink);
        }

        /**
         * Retorna descrição do estado de pagamento (Utilizado atualmente em link de pagamento)
         * @param {*} orderPayment Pagamento do pedido
         */
        function getPaymentStateDescription(orderPayment) {
            switch(orderPayment.paymentState) {
                case "NONE":
                    return "Nenhum";
                case "AUTHORIZED":
                    return "Autorizado";
                case "UNDONE":
                    return "Desfeito";
                case "CONFIRMED":
                    return "Pago";
                case "CANCELED":
                    return "Cancelado";
                case "IN_PROGRESS":
                    return "Processando";
                default:
                    return "Sem informação";
            }
        }

        function getFirstPaymentLink(order) {
            return order
                && order.orderPayments
                && order.orderPayments.find(paymentLinkFilter);
        }

        function paymentLinkFilter(orderPayment) {
            return orderPayment && orderPayment.paymentCodeMethod == 'PAYMENT_LINK';
        }

        $scope.mayDetailOrder = (order) => {
            return !order || operations.includes('DETAIL_ORDER');
        }

        $scope.mayCancelOrder = (order) => {
            if (!order || !operations.includes('CANCEL_ORDER')) {
                return false;
            }

            return order.status === 'PAYMENT';
        };

        $scope.mayLoadOrder = (order) => {
            if (!order || !operations.includes('LOAD_ORDER')) {
                return false;
            }

            const paymentLink = getFirstPaymentLink(order);
            const allowedStatuses = ['READY_TO_PACK', 'ON_HOLD'];

			if (paymentLink == null) {
				allowedStatuses.push('CHECKOUT');
			} else {
				const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

				if (paymentLinkProvider == 'PAGAR_ME') {
					switch (paymentLink.paymentState) {
					case 'CANCELED':
					case 'ERROR':
					case 'NONE':
						allowedStatuses.push('CHECKOUT');
						break;
					case 'IN_PROGRESS':
						allowedStatuses.push('PAYMENT');
						break;
					default:
						break;
					}
				}
			}

            // READY_TO_PACK, ON_HOLD - Sempre exibiremos o botão
            // CHECKOUT - Apenas exibiremos em duas condições (1 - Não tem link de Pagamentos, 2 - Link de Pagamentos é da Pagar.me)
            // PAYMENT - Apenas exibiremos se tiver Link de Pagamento e se o Link de Pagamento for da Pagar.me
            return allowedStatuses.includes(order.status);
        }

        $scope.loadOrderLabel = (order) => {
            if (!order) {
                return null;
            }
            switch (order.status) {
            case 'ON_HOLD':
            case 'READY_TO_PACK':
                return 'Separar';
            case 'CHECKOUT':
            default:
                return 'Resgatar';
            }
        };

        $scope.mayResendPaymentLink = (order) => {
            const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;
            return order
                && order.status == 'PAYMENT'
                && $scope.hasPendingPaymentLink(order)
                && paymentLinkProvider == 'PAGAR_ME';
        };

        $scope.resendPaymentLink = (order) => {
            const paymentLink = getFirstPaymentLink(order);

            $scope.phone = {};

            $scope.phone.value =
                paymentLink.customer ?
                    paymentLink.customer.phoneNumber : undefined;

            if (!$scope.phone.value) {
                $scope.phone.value =
                    order.customer &&
                    order.customer.address &&
                    order.customer.address.phone ?
                        order.customer.address.phone : undefined;
            }

            if ($scope.phone.value) {
                $scope.phone.value = $scope.phone.value.replace(/[^0-9]/g, '');
                if ($scope.phone.value.length >= 13) {
                    $scope.phone.value = $scope.phone.value.substring(2, 13);
                }
            }

            $scope.phone.types =
                ConfigService
                    .config
                    .business.payment_link_send_methods.filter(
                        item => item == 'WHATSAPP' || item == 'SMS'
                    );
            $scope.phone.types = $scope.phone.types || ['WHATSAPP'];
            $scope.phone.type = $scope.phone.types[0];

            $ionicPopup.confirm({
                title : 'Telefone',
                templateUrl : './views/order/order-link-phone.html',
                subTitle : 'Informe o Telefone para envio do Link de Pagamento',
                scope: $scope,
                buttons : [{
                    text: 'Cancelar',
                    type: 'button-theme',
                }, {
                    text: 'Confirmar',
                    type: 'button-theme',
                    onTap: function(e) {
                        const phone = $scope.phone.value;
                        if (!MessagingService.isPhoneNumberValid(phone)) {
                            AlertService.error('Telefone inválido');
                            return e.preventDefault();
                        }
                        return $scope.phone.value;
                    }
                }],
            }).then((phone) =>
                phone == null ? $q.reject() : $q.resolve(phone)
            ).then(() =>
                $ionicLoading.show({
                    template : 'Consultando Link de Pagamento...',
                    duration : 0
                })
            ).then(() =>
                PaymentLinkService.verifyPaymentLink(paymentLink.paymentAuthorizationNSU)
            ).then((remotePayment) => {
                $ionicLoading.show({
                    template : 'Recuperando mensagem...',
                    duration : 0
                });
                return remotePayment;
            }).then((remotePayment) =>
                PaymentLinkService.buildMessage({
                    paymentUrl: remotePayment.data.checkoutUrl,
                    totalValue: paymentLink.paymentValue,
                    storeNumber: order.storeNumber,
                    orderDate: order.orderDate,
                    customer: order.customer,
                    fiscalItems: order.fiscalItems,
                    nonFiscalItems: order.nonFiscalItems,
                    messageType: 'CHECKOUT',
                    messageProtocol: $scope.phone.type,
                })
            ).then((msg) => {
                $ionicLoading.show({
                    template : 'Enviando mensagem...',
                    duration : 0
                });
                return msg;
            }).then((msg) => {
                let msgFn = $scope.phone.type == 'SMS' ?
                    MessagingService.sendSms :
                    MessagingService.sendWhatsApp;
                return msgFn(
                    undefined,
                    $scope.phone.value,
                    msg.data.message,
                    msg.data.messageContentType,
                    msg.data.messageContentParameters
                );
            }).then(() =>
                $ionicLoading.show({
                    template : 'Mensagem enviada com sucesso',
                    duration : 0
                }).then(() =>
                    $timeout(2000)
                )
            ).finally(() =>
                $ionicLoading.hide()
            ).catch((err) =>
                err && AlertService.restError(err, err)
            );
        };

        $scope.loadOrder = (order) => {
			const onError = (error) => {
				AlertService.error(error.message);
			};
			const onSuccess = (order) => {
				AlertService.confirm(ConfigService.config.business.order_with_delivery_of_merged_items_allowed ?
                    'Deseja carregar o pedido?' : 'Deseja carregar o pedido? (As informações atuais serão substituídas.)'
                ).then(res => {
                    if (!res) {
                        return $q.reject();
                    }
                }).then(() =>
                    checkPaymentLink(order)
                    .then((newOrder) =>  order = newOrder)
                ).then(() => {
                    const mustReloadCustomer =
                        order.customer
                        && order.customer.document
                        && ConfigService.config.business.must_reload_customer;

                    if (!mustReloadCustomer) {
                        return $q.resolve(order.customer);
                    }

                    const reloadBehavior =
                        ConfigService
                            .config
                            .business
                            .order_search_reload_customer_behavior || 'V1';

                    const searchCustomer = () => $ionicLoading.show({
                        template : 'Consultando Cliente do Pedido...',
                        duration : 0
                    }).then(() =>
                        CustomerService.search({
                            searchType: order.customer.documentType || 'CPF',
                            searchValue: order.customer.document,
                        }).then((result) =>
                            result && result.items && result.items.length > 0 ?
                                result.items[0] : null
                        )
                    ).finally(() =>
                        $ionicLoading.hide()
                    );

                    // após o reload, é retornado o cliente da consulta
                    // e o customer do pedido também é atualizado
                    // com os dados de acordo com a versão do comportamento...
                    if (reloadBehavior == 'V1') {
                        if (order.status == OrderService.orderStatus.ON_HOLD) {
                            return order.customer;
                        }
                        return reloadCustomerV1(searchCustomer(), order);
                    } else if (reloadBehavior == 'V2') {
                        return reloadCustomerV2(searchCustomer(), order);
                    } else {
                        // algum dev fez besteira...
                        return $q.reject(
                            `Comportamento de Reload de Cliente não suportado '${reloadBehavior}'`
                            + `, Por favor contate o suporte`
                        );
                    }
                }).then((customer) => {
                    const user = UserService.currentUser;
                    if (user && user.document
                         && customer && customer.document
                         && user.document === customer.document) {
                        return $q.reject('Proibido auto serviço');
                    }
                    return customer;
                }).then((customer) =>
                    showDonationItems(order)
					.then(() => customer)
                ).then((customer) => {
                    const donationOnly = order.donationItems && order.donationItems.length
                            && (!order.fiscalItems || !order.fiscalItems.length)
                            && (!order.nonFiscalItems || !order.nonFiscalItems.length);
                    if (donationOnly) {
                        return donate(order, customer);
                    }
                    if (!validateOrderProducts(order)) {
                        return;
                    }

                    const orders = [order];
                    const hasSoldInStoreItemsToScan = CartService.mustScanOrderItems() && UtilService.isNotEmptyArray(OrderChecking.extractOrderSoldInStoreItems(orders));
                    // se tiver habilitado para escanear os ítens e tiver ítens SOLD_IN_STORE, não usa a mesclagem de pedidos atual do CXR
                    if (hasSoldInStoreItemsToScan) {
                        // foi acertado que inicialmente nenhum desconto será aplicado ao resgatar pedidos desse tipo.
                        CartService.undoDiscounts();
                        // considera o parâmetro para saber se deve descartar os ítens
                        const mustInitCart = ConfigService.config.business.order_with_delivery_of_merged_items_allowed === true;
                        CartService.loadOrders(orders, mustInitCart);
                        UtilService.navigateWithoutHistory('easyorder.cart');
                        return;
                    }

                    if (ConfigService.config.business.order_with_delivery_of_merged_items_allowed) {
						// Se houver itens no carrinho, transforma em pedido para mesclagem
						const autoserviceOrder = OrderService.cartToOrderConverter(CartService.cart);
                        
						if (!autoserviceOrder || order.status === 'READY_TO_PACK') {
							CartService.loadCart(order, null, customer);
							return UtilService.navigateWithoutHistory('easyorder.cart');
						}
						
						if (!validateOrderProducts(autoserviceOrder)) {
                            return;
						}

                        $scope.selectedOrdersToMerge.push(autoserviceOrder);
						$scope.selectedOrdersToMerge.push(order);
						
						return $ionicLoading.show({
							template : 'Processando pedido...',
							duration : 0
						}).then(() =>
							OrderService.merge($scope.selectedOrdersToMerge, CartService.cart.customer || customer)
                            .finally(() => $ionicLoading.hide())
                            .then(resp => {
                                CartService.loadCart(resp.order, resp.mergedOrders);
                                return UtilService.navigateWithoutHistory('easyorder.cart');
							}).catch((error) => {
                                $scope.selectedOrdersToMerge = [];
                                return $q.reject(error);
							})
						);
					}

                    CartService.loadCart(order, null, customer);
                    return UtilService.navigateWithoutHistory('easyorder.cart');
                }).catch(err =>
                    err && AlertService.restError(err, err)
                );
			};
			OrderService.validateOrder(
                order,
                ConfigService.config.storeNumber,
                onSuccess,
                onError,
                ['CHECKOUT', 'ON_HOLD', 'READY_TO_PACK', 'PAYMENT']
            );
		};

        function checkPaymentLink(order) {
            const paymentLink = getFirstPaymentLink(order);

            if (paymentLink == null) {
				return $q.resolve(order);
			}

            const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

            // link de pagamento da pagar.me do cxr sempre terá esse campo preenchido
            // caso não tenham é um bug
			if (paymentLinkProvider == 'PAGAR_ME' && paymentLink.paymentState == null) {
                return $q.reject('Link de Pagamento não possui Status definido');
			}

            if (order.status == 'CHECKOUT') {
                if (paymentLinkProvider != 'PAGAR_ME') {
                    return $q.reject(
                        'Apenas Pedidos com Link de Pagamento da Pagar.me suportam resgate com o'
                        + ` Status do Pedido '${order.status}'`
                    );
                }

                const allowedStates = ['CANCELED', 'ERROR', 'NONE'];
                if (!allowedStates.includes(paymentLink.paymentState)) {
                    return $q.reject(
                        `O pedido não pode ser resgatado, Status do Link de Pagamento`
                        + ` '${getPaymentStateDescription(paymentLink)}' não permite resgate`
                        + ` para pedidos com Status '${order.status}'`
                    );
                }
            } else if (order.status == 'PAYMENT') {
                // apenas pagar_me hoje suporta requests para desabilitar links...
                if (paymentLinkProvider != 'PAGAR_ME') {
                    return $q.reject(
                        'Apenas Pedidos com Link de Pagamento da Pagar.me suportam resgate com o'
                        + ` Status do Pedidos '${order.status}'`
                    );
                }

                const allowedStates = ['IN_PROGRESS'];
                if (!allowedStates.includes(paymentLink.paymentState)) {
                    return $q.reject(
                        `O pedido não pode ser resgatado, Status do Link de Pagamento`
                        + ` '${getPaymentStateDescription(paymentLink)}' não permite resgate`
                        + ` para pedidos com Status '${order.status}'`
                    );

                }

                return AlertService.confirm(
                    'O link de pagamento será cancelado, deseja continuar?'
                ).then((confirmed) => {
                    if (!confirmed) {
                        return $q.reject();
                    }
                }).then(() =>
                    $ionicLoading.show({
                        template : 'Cancelando Link de Pagamento...',
                        duration : 0
                    })
                ).then(() =>
                    PaymentLinkService
                        .disablePaymentLink(paymentLink.paymentAuthorizationNSU)
                ).then(() =>
                    $ionicLoading.show({
                        template : 'Atualizando Pedido...',
                        duration : 0
                    })
                ).then(() => {
                    // devemos atualizar o pedido para checkout
                    // e marcar o link de pagamento com status cancelado
                    let orderClone = JSON.parse(JSON.stringify(order));
                    let paymentLinkClone = getFirstPaymentLink(orderClone);
                    paymentLinkClone.paymentState = 'CANCELED';
                    orderClone.status = 'CHECKOUT';
                    return OrderService.update(orderClone);
                }).finally(() =>
                    $ionicLoading.hide()
                );
            }

            return $q.resolve(order);
        }

        function reloadCustomerV1(searchPromise, order) {
            return searchPromise.then((customer) => {
                if (!customer) {
                    // vai cair no fluxo do catch, alertando
                    // que não foi retornado o cliente
                    return $q.reject({
                        data: {
                            message: 'Nenhum cliente foi encontrado com o documento informado'
                        }
                    });
                }

                LogService.info('Substituindo dados do cliente pelo conteúdo retornado da nova consulta');
                customer.email = (order.customer && order.customer.email && 0 !== order.customer.email.length ? order.customer.email : customer.email);

                order.customer = OrderService.toOrderCustomer(
                    JSON.parse(JSON.stringify(customer))
                );

                return customer;
            }).catch(error => {
                let cause =  "Erro consultando cliente!";

                if (error && error.data && error.data.message) {
                    cause =  error.data.message + ".";
                }

                return AlertService.error(
                    `${cause} Recebimento por cartão próprio não poderá ser efetuado.`
                ).then(() => order.customer)
            });
        };

        function reloadCustomerV2(searchPromise, order) {
            return searchPromise.then((customer) => {
                if (!customer) {
                    // vai cair no fluxo do catch, alertando
                    // que não foi retornado o cliente
                    return $q.reject({
                        data: {
                            message: 'Nenhum cliente foi encontrado com o documento informado'
                        }
                    });
                }
                LogService.info('Atualizando os dados do cliente pelo conteúdo retornado da nova consulta');
                // fazendo o merge, atualizando apenas os dados não existentes
                customer = { ...customer, ...order.customer };
                order.customer = OrderService.toOrderCustomer(
                    JSON.parse(JSON.stringify(customer))
                );
                return customer;
            }).catch((err) => {
                // o ideal era alertarmos o erro e prosseguirmos
                // mas foi solicitado a remoção do alerta de erro
                LogService.error(err);
                return order.customer;
            });
        }

        function showDonationItems(order) {
		    let ret  = $q.defer();

            const layout = (donationItems) => {
                let ret = '<ion-list class="list list-inset dosis" style="margin: unset;width:100%;">';

                donationItems.forEach(item => {
                    ret += `<ion-item class="item item-remove-animate item-inset item-text-wrap">
                        <div class="row" style="display:flex;flex-flow:row wrap; justify-content:space-between;align-items:flex-start;padding: .5em 0;">
                            <div class="col col-100" style="flex:0 50%;">
                                <h2 color="primary">${item.productDescription}</h2>
                                <div class="col col-100 no-margin dosis" style="word-break:break-all;white-space: nowrap;display:flex;flex-flow:column;align-items:flex-start;padding:0;margin-top:5px;">
                                    <b> Cód.:</b> <span>${ $rootScope.getItemDisplayCode(item) }</span>
                                </div>
                            </div>
                        </div>
                      </ion-item>`;
                });

                return ret + '</ion-list>';
            };

		    if (order.donationItems && order.donationItems.length) {
		        $ionicPopup.show({
		            template: layout(order.donationItems),
	                title: 'Itens de doação nessa operação:',
	                scope: $scope,
	                buttons: [
                        {   text: 'Cancelar',
                            onTap: function() { ret.reject(); }
                        }, {
                            text: '<b>Ok</b>',
                            type: 'button-theme',
                            onTap: function() { ret.resolve(); }
                        }]
		        });
		    } else {
		        ret.resolve();
		    }

		    return ret.promise;
		}

        function donate(order, customer) {
		    $ionicLoading.show({
                template : 'Processando Pedido de Doação...',
                duration : 0
            }).then(() => {
                const validateDonationItems = (order) => {
                    const constainsNoImeiItem = order.donationItems.find(item => !item.imei);
                    if (constainsNoImeiItem) {
                        const msg = `Não é permitida a doação para este produto: ${invalidItem.productDescription}`;
                        AlertService.error(msg);
                        throw new Error(msg);
                    }
                }
                const parseUuid = (uuidValue) => {
                    return {
                        type: "ICCID",
                        value: uuidValue,
                    };
                }

    		    validateDonationItems(order);

    		    const fiscalItems = order.donationItems.map(item => {
    		        return {
                        description: item.productDescription,
                        compactDescription: item.productDescription,
                        code: item.productCode,
                        automationCode: item.automationCode,
                        unitValue: item.unitValue,
                        totalValue: item.totalValue,
                        quantity: item.quantitySold,
                        uuid: parseUuid(item.imei),
                        marketingCode: item.marketingCode
                    };
    		    });

                return DonationService.donate({
                    appInfo: {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    },
                    transactionKey: {
                        storeInfo: {
                            companyNumber: ConfigService.config.companyNumber,
                            storeNumber: ConfigService.config.storeNumber,
                        },
                        date: new Date(),
                        nsu: ConfigService.nsuControll.incrementNSU(),
                        terminal: ConfigService.config.terminalNumber,
                    },
                    reason: {
                        code: 5,
                        description: 'Outros'
                    },
                    operator: {
                        code: UserService.operatorCode(),
                        name: UserService.operatorName()
                    },
                    authorizer: {
                        code: order.operatorCode
                    },
                    customer: customer,
                    fiscalItems: fiscalItems,
                    totalValue: fiscalItems.reduce((acc, item) => new Number((acc + item.totalValue).toFixed(2)), 0),
                    orderNumber: order.orderNumber,
                    externalOrderCode: order.externalOrderCode,
                    originSystem: order.originSystem
                }).then(result => {
                    return hideModals().then(() => {
                        return UtilService.navigateWithoutHistory('easyorder.couponview', {
                            pdf: result.data.pdf || " ",
                            printContents: result.data.printContents || [],
                            documentTitle: 'donation',
                            showPdf: result.data.pdf != null && result.data.pdf.length > 0
                        });
                    });
                }, (error) => {
                    AlertService.restError(error);
                }).finally(() => $ionicLoading.hide());
            });
        }

        function validateOrderProducts(order) {
            let dofExists = false;
            let restrictionExists = false;
            let index;

	        for (index in order.fiscalItems) {
        	    var fiscalItem = order.fiscalItems[index];
				if (fiscalItem.restrictions && fiscalItem.restrictions.find(e => e.type != null && e.type == 'DOF')) {
                    dofExists = true;
                }
				if (fiscalItem.restrictions && fiscalItem.restrictions.find(e => e.type != null && e.type == 'SALE_RESTRICTION')) {
					restrictionExists = true;
				}
	        }

            if (dofExists){
				if (!order.customer) {
					AlertService.warn("Produto de Origem Florestal. Identificação do cliente obrigatória.");
                    return false;
                }
            }

            if (restrictionExists) {
				if (!order.customer) {
					AlertService.warn("VENDA PROIBIDA PARA MENORES DE 18 ANOS. Identificação do cliente OBRIGATÓRIA");
                    return false;
                }
            }


            return true;
        }

        (() => {
            $scope.searchOrder();
        })();
    });
})();
