(function() {
	'use strict';

	angular.module('easyorder.controllers.order', [
		'easyorder.services.order',
		'easyorder.services.config',
		'easyorder.services.cart',
		'easyorder.services.util',
		'easyorder.services.alert',
		'easyorder.services.log'
	])

	.controller('OrderCtrl', function($scope,
                                      $ionicModal,
                                      $ionicLoading,
                                      $q,
                                      $ionicPopup,
                                      $rootScope,
                                      $stateParams,
                                      $state,
                                      ProductService,
                                      OrderService,
                                      ConfigService,
                                      CartService,
									  UtilService,
									  AlertService,
									  UserService,
									  CustomerService,
									  LogService,
									  DonationService,
									  PaymentLinkService,
									  MessagingService) {

	    $scope.maxDate = new Date();

	    /**
	     *
	     */
	    $scope.init = function(result, selectedOrdersToMerge) {
    		$scope.orderSearchParams = {
    		    documentType: "CPF"
    		};
			$scope.selectedOrdersToMerge = selectedOrdersToMerge;
			$scope.searchResult = (result || []).filter(mayShowOrder);
	    }
	    $scope.init([],[]);

		// FIXME: remover esta trava quando o PDV
        // compatibilizar os links de Pagamento com
        // o CXR, é apenas uma forma rápida de validar
        // se é um pedido do PDV
        function mayShowOrder(order) {
            const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

            if (!$scope.hasPaymentLink(order)) {
                return true;
            }

            if (paymentLinkProvider == 'PAGAR_ME') {
                // pedidos do PDV no momento sempre terão este campo
                // nulo
                return $scope.hasPaymentLinkState(order);
            }

            return true;
        }

		/**
         *
         * @param {Pedido} order Pedido a ser exibido
         * @returns Se o pedido foi pago com link de pagamento
         */
		$scope.hasPaymentLink = (order) => {
			return getFirstPaymentLink(order) != null;
		}

		$scope.hasPaymentLinkState = (order) => {
			const paymentLink = getFirstPaymentLink(order);
			return paymentLink
				&& paymentLink.paymentState != null;
		};

		/**
         *
         * @param {Pedido} order Pedido
         * @returns Retorna a descrição do estado do primeiro pagamento realizado com Link de Pagamento no Pedido
         */
		$scope.getFirstPaymentLinkDesc = (order) => {
			const paymentLink = getFirstPaymentLink(order);
			return getPaymentStateDescription(paymentLink);
		}

		/**
         * Retorna descrição do estado de pagamento (Utilizado atualmente em link de pagamento)
         * @param {*} orderPayment Pagamento do pedido
         */
		function getPaymentStateDescription(orderPayment) {
			switch(orderPayment.paymentState) {
				case "NONE":
					return "Nenhum";
				case "AUTHORIZED":
					return "Autorizado";
				case "UNDONE":
					return "Desfeito";
				case "CONFIRMED":
					return "Pago";
				case "CANCELED":
					return "Cancelado";
				case "IN_PROGRESS":
					return "Processando";
				default:
					return "Sem informação";
			}
		}

		function getFirstPaymentLink(order) {
            return order
                && order.orderPayments
                && order.orderPayments.find(paymentLinkFilter);
        }

        function paymentLinkFilter(orderPayment) {
            return orderPayment && orderPayment.paymentCodeMethod == 'PAYMENT_LINK';
        }

		$ionicModal.fromTemplateUrl('views/ordersearchresult.html', {
			scope : $scope,
			animation : 'slide-in-up',
			hardwareBackButtonClose : false
		}).then(function(modal) {
			$scope.modal = modal;
		});
		$scope.openModal = function() {
			if ($scope.searchResult && $scope.searchResult.length) {
				$scope.modal.show();
			} else {
				AlertService.error('Nenhum pedido foi encontrado!');
			}
		};

		/**
	     *
	     */
		 $ionicModal.fromTemplateUrl('views/ordermergelist.html', {
			scope : $scope,
			animation : 'slide-in-up',
			hardwareBackButtonClose : false
		}).then(function(modal) {
			$scope.mergeModal = modal;
		});
		$scope.openMergeModal = function() {
			if ($scope.selectedOrdersToMerge && $scope.selectedOrdersToMerge.length) {
				$scope.mergeModal.show();
			} else {
				AlertService.error('Nenhum pedido foi encontrado!');
			}
		};

		/**
	     *
	     */
		$ionicModal.fromTemplateUrl('views/ordersearchupdateresult.html', {
			scope : $scope,
			animation : 'slide-in-up',
			hardwareBackButtonClose : false
		}).then(function(modal) {
			$scope.updateModal = modal;
		});
		$scope.openUpdateModal = function() {
			if ($scope.searchResult && $scope.searchResult.length) {
				$scope.updateModal.show();
			} else {
				AlertService.error('Nenhum pedido foi encontrado!');
			}
		};

		function hideModals() {
			return hideModal($scope.modal)
				.then(() => hideModal($scope.updateModal))
				.then(() => hideModal($scope.mergeModal));
		}

		function hideModal(modal) {
			return $q((resolve) => {
				if (modal != null && modal.isShown()) {
					modal.hide()
						.finally(() => resolve());
				} else {
					resolve();
				}
			});
		}

		$scope.$on('$destroy', function() {
			$scope.modal.remove();
			$scope.updateModal.remove();
			$scope.mergeModal.remove();
			$scope.searchResult = null;
		});

		function isNormalInteger(str) {
		    return /^\+?\d+$/.test(str);
		}

		/**
		 *
		 */
		$scope.isSearchDisabled = function() {
			var orderSearchParams = $scope.orderSearchParams;
			var searchDisabled = true;

			if (orderSearchParams.document) {
				searchDisabled = false;
			} else if (orderSearchParams.customerName) {
				searchDisabled = false
			} else if (orderSearchParams.orderDate) {
				searchDisabled = false
			} else if (orderSearchParams.orderNumber || orderSearchParams.externalOrderNumber) {
				if (orderSearchParams.storeNumber) {
					if (orderSearchParams.storeNumber.length == 4 && isNormalInteger(orderSearchParams.storeNumber)) {
						var number = Number(orderSearchParams.storeNumber);
						searchDisabled = number <= 0 || number > 9999;
					}
				} else {
					searchDisabled = false;
				}
			}
			return searchDisabled;
		};


		/**
		 *
		 */
		$scope.doSearch = function() {
			$ionicLoading.show({
				template : 'Consultando pedido...',
				duration : 0
			}).then(function() {
				var error = function(error) {
					$ionicLoading.hide();

					AlertService.restError(error, 'Erro consultando pedido no servidor!');
				};
				// se tiver o número do pedido
				if ($scope.orderSearchParams.orderNumber || $scope.orderSearchParams.externalOrderNumber) {
					var params = {
						companyNumber: ConfigService.config.companyNumber,
						orderNumber: $scope.orderSearchParams.orderNumber,
						externalOrderNumber: $scope.orderSearchParams.externalOrderNumber
					};

					// se não for inserido a loja, usa a loja corrente.
					if ($scope.orderSearchParams.storeNumber) {
						params.storeNumber = $scope.orderSearchParams.storeNumber;
					} else {
						params.storeNumber = ConfigService.config.storeNumber;
					}

					if ($scope.orderSearchParams.orderNumber) {
					    OrderService.get(params, function(result) {
					        $ionicLoading.hide();
					        $scope.init([result], $scope.selectedOrdersToMerge);
					        $scope.modal.show();
					    }, error);
					} else {
					    OrderService.listExternal(params, function(result) {
					    	$ionicLoading.hide();
					    	if (result.orders && result.orders.length > 0) {
								$scope.init(result.orders, $scope.selectedOrdersToMerge);
								$scope.modal.show();
							} else {
								AlertService.error('Nenhum pedido foi encontrado com os filtros informados!');
							}
                        }, error);
					}
				} else {
					var cloneParams = JSON.parse(JSON.stringify($scope.orderSearchParams));
					// remove os parâmetros vazios
					var params = UtilService.removeEmptyAttributes(cloneParams);

					params.companyNumber = ConfigService.config.companyNumber;
					params.destinationStoreNumber = ConfigService.config.storeNumber;

					// se não tiver o número do pedido, chama do 'list'
					OrderService.list(params, function(result) {
						$ionicLoading.hide();

						if (result.orders && result.orders.length > 0) {
							$scope.init(result.orders, $scope.selectedOrdersToMerge);
							$scope.modal.show();
						} else {
							AlertService.error('Nenhum pedido foi encontrado com os filtros informados!');
						}
					}, error);
				}
			});
		};

		 /**
		  * Remove pedido da lista de mesclagem
		  * @param {Pedido à ser removido da lista de mesclagem} order
		  */
		 $scope.removeOrder = function(order) {
			AlertService.confirm('Deseja realmente remover este pedido da lista de resgate de pedidos?')
			.then(res => {
				if (res) {
					if ($scope.selectedOrdersToMerge) {
						for (var i = 0; i < $scope.selectedOrdersToMerge.length; i++) {
							if ($scope.selectedOrdersToMerge[i] === order) {
								$scope.selectedOrdersToMerge.splice(i--, 1);
							}
						}
					}
				}
			});
		 }

		 /**
		  * Verifica se o parâmetro para resgate de múltiplos pedidos está habilitado
		  * @returns Se true - Parâmetro de resgate de múltiplos pedidos habilitado, false para o contrário.
		  */
		 $scope.multipleOrdersEnabled = function() {
			return ConfigService.config.business.multiple_orders_enabled;
		 }

		 /**
		  * Habilita e desabilita tela de mesclagem de pedidos
		  *
		  * @returns Se true - Tela e botão de listagem de pedidos para mesclagem habilitado | Se false Tela e botão de listagem de pedidos para mesclagem desabilitado
		  */
		 $scope.isSelectedOrdersToMergeViewDisabled = function() {
			return $scope.selectedOrdersToMerge.length == 0;
		 }

		 $scope.hasSelectedOrderToMerge = function(order) {
			 if ($scope.selectedOrdersToMerge.filter(orderToMerge => orderToMerge.orderNumber == order.orderNumber && orderToMerge.storeNumber == order.storeNumber).length > 0) {
				return true;
			} else {
				return false;
			}
		 }
		 /**
		  * Abre a listagem de pedidos selecionados para a mesclagem
		  */
		 $scope.openSelectedOrdersToMerge = function() {
			$scope.init([], $scope.selectedOrdersToMerge);
			$scope.openMergeModal();
		 }

		 /**
		  * Mescla pedidos selecionados
		  */
		 $scope.mergeOrders = function() {
			AlertService.confirm('Deseja carregar os pedidos selecionados?')
				.then(res => {
					if (res) {
						$ionicLoading.show({
							template : 'Processando pedidos selecionados...',
							duration : 0
						}).then(() => {
							const hasSoldInStoreItemsToScan = CartService.mustScanOrderItems() && UtilService.isNotEmptyArray(OrderChecking.extractOrderSoldInStoreItems($scope.selectedOrdersToMerge));
							// se tiver habilitado para escanear os ítens e tiver ítens SOLD_IN_STORE, não usa a mesclagem de pedidos atual do CXR
							if (hasSoldInStoreItemsToScan) {
								// foi acertado que inicialmente nenhum desconto será aplicado ao resgatar pedidos desse tipo.
								CartService.undoDiscounts();
								// considera o parâmetro para saber se deve descartar os ítens
								const mustInitCart = ConfigService.config.business.order_with_delivery_of_merged_items_allowed === true;
								CartService.loadOrders($scope.selectedOrdersToMerge, mustInitCart);
								$ionicLoading.hide();
								UtilService.navigateWithoutHistory('easyorder.cart');
							} else {
								// Se houver itens no carrinho, transforma em pedido para mesclagem
								var autoserviceOrder = OrderService.cartToOrderConverter(CartService.cart);
								if (validateOrderProducts(autoserviceOrder)){
									$scope.selectedOrdersToMerge.push(autoserviceOrder);
								}

								const clonedCustomer = CartService.cart.customer ? OrderService.toOrderCustomer(UtilService.cloneObject(CartService.cart.customer)) : null;
								OrderService.merge($scope.selectedOrdersToMerge, clonedCustomer)
									.then(resp => {
											CartService.loadCart(resp.order, resp.mergedOrders);
											$ionicLoading.hide();
											UtilService.navigateWithoutHistory('easyorder.cart');
									}).catch((error) => {
										$ionicLoading.hide();
										AlertService.restError(error);
									})
							}
						});
					} else {
						return $q.reject();
					}
				});
		 }

		$scope.selectOrder = (order) => {
			const onError = (error) => {
				AlertService.error(error.message);
			};
			const onSuccess = (order) => {
				AlertService.confirm(
					$scope.multipleOrdersEnabled() || ConfigService.config.business.order_with_delivery_of_merged_items_allowed ?
						'Deseja carregar o pedido?'
                    	: 'Deseja carregar o pedido? (As informações atuais serão substituídas.)'
                ).then(res => {
                    if (!res) {
                        return $q.reject();
                    }
				}).then(() =>
                    validatePrices(order)
				).then(() =>
                    validateCustomer(order)
				).then(() =>
                    checkPaymentLink(order)
					.then((newOrder) =>  order = newOrder)
                ).then(() => {
					// consultando o cliente do pedido...
                    const mustReloadCustomer =
                        order.customer
                        && order.customer.document
                        && ConfigService.config.business.must_reload_customer;

                    if (!mustReloadCustomer) {
                        return $q.resolve(order.customer);
                    }

                    const reloadBehavior =
                        ConfigService
                            .config
                            .business
                            .order_search_reload_customer_behavior || 'V1';

                    const searchCustomer = () => $ionicLoading.show({
                        template : 'Consultando Cliente do Pedido...',
                        duration : 0
                    }).then(() =>
                        CustomerService.search({
                            searchType: order.customer.documentType || 'CPF',
                            searchValue: order.customer.document,
                        }).then((result) =>
                            result && result.items && result.items.length > 0 ?
                                result.items[0] : null
                        )
                    ).finally(() =>
                        $ionicLoading.hide()
                    );

                    // após o reload, é retornado o cliente da consulta
                    // o customer do pedido é atualizado
                    // com os dados de acordo com a versão do comportamento...
                    if (reloadBehavior == 'V1') {
                        if (order.status == OrderService.orderStatus.ON_HOLD) {
                            return order.customer;
                        }
                        return reloadCustomerV1(searchCustomer(), order);
                    } else if (reloadBehavior == 'V2') {
                        return reloadCustomerV2(searchCustomer(), order);
                    } else {
                        // algum dev fez besteira...
                        return $q.reject(
                            `Comportamento de Reload de Cliente não suportado '${reloadBehavior}'`
                            + `, Por favor contate o suporte`
                        );
                    }
                }).then((customer) => {
                    const user = UserService.currentUser;
                    if (user && user.document
                         && customer && customer.document
                         && user.document === customer.document) {
                        return $q.reject('Proibido auto serviço');
                    }
                    return customer;
                }).then((customer) =>
                    showDonationItems(order)
					.then(() => customer)
                ).then((customer) => {
                    const donationOnly = order.donationItems && order.donationItems.length
                            && (!order.fiscalItems || !order.fiscalItems.length)
                            && (!order.nonFiscalItems || !order.nonFiscalItems.length);
                    if (donationOnly) {
                        return donate(order, customer);
                    }
					if (!validateOrderProducts(order)) {
						return;
					}
					if ($scope.multipleOrdersEnabled() && $state.is("easyorder.ordersearch")) {
						if (order.status === 'READY_TO_PACK') {
							return $q.reject(
								'Não é possível adicionar um pedido pronto para embalagem à lista de mesclagem'
							);
						}
						$scope.selectedOrdersToMerge.push(order);
						$scope.modal.hide();
						$scope.searchResult = null;
						return;
					} else if(ConfigService.config.business.order_with_delivery_of_merged_items_allowed) {
						// Se houver itens no carrinho, transforma em pedido para mesclagem
						const autoserviceOrder = OrderService.cartToOrderConverter(CartService.cart);
                        
						if (!autoserviceOrder || order.status === 'READY_TO_PACK') {
							CartService.loadCart(order, null, customer);
							return UtilService.navigateWithoutHistory('easyorder.cart');
						}
						
						if (!validateOrderProducts(autoserviceOrder)) {
                            return;
						}

                        $scope.selectedOrdersToMerge.push(autoserviceOrder);
						$scope.selectedOrdersToMerge.push(order);
						
						return $ionicLoading.show({
							template : 'Processando pedido...',
							duration : 0
						}).then(() =>
							OrderService.merge($scope.selectedOrdersToMerge, CartService.cart.customer || customer)
                            .finally(() => $ionicLoading.hide())
                            .then(resp => {
                                CartService.loadCart(resp.order, resp.mergedOrders);
                                return UtilService.navigateWithoutHistory('easyorder.cart');
							}).catch((error) => {
                                $scope.selectedOrdersToMerge = [];
                                return $q.reject(error);
							})
						);
					}

					CartService.loadCart(order, null, customer);
					return UtilService.navigateWithoutHistory('easyorder.cart');
                }).catch(err =>
                    err && AlertService.restError(err, err)
                );
			};

			const paymentLink = getFirstPaymentLink(order);
			const allowedStatuses = ['READY_TO_PACK', 'ON_HOLD'];

			if (paymentLink == null) {
				allowedStatuses.push('CHECKOUT');
			} else {
				const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

				if (paymentLinkProvider == 'PAGAR_ME') {
					switch (paymentLink.paymentState) {
					case 'CANCELED':
					case 'ERROR':
					case 'NONE':
						allowedStatuses.push('CHECKOUT');
						break;
					case 'IN_PROGRESS':
						allowedStatuses.push('PAYMENT');
						break;
					default:
						break;
					}
				}
			}

			OrderService.validateOrder(
                order,
                ConfigService.config.storeNumber,
                onSuccess,
                onError,
                allowedStatuses
            );
		};

		function extractCustomerDocument(customer) {
			return customer && !UtilService.isNullOrEmpty(customer.document) ? customer.document : null;
		}

		function compareCustomer(cartCustomer, orderCustomer) {
			// se não tiver cliente atual do carrinho, aceita independentemente de como estiver no pedido
			// caso contrário, compara com o cliente do pedido (que deve ser necessariamente igual)
			return cartCustomer == null ? true : cartCustomer === orderCustomer;
		}

		function validateCustomer(order) {
			// verifica se o resgate é de pedidos com clientes diferentes
			// se não permitir clientes diferentes (do pedido ou identificado)
			if ($scope.allowDifferentCustomerInOrders) {
				return $q.resolve();
			} else {
				// verifica se o CPF difere do cliente identificado
				const cartCustomer = extractCustomerDocument(CartService.cart.customer);
				const orderCustomer = extractCustomerDocument(order.customer);
				
				if (!compareCustomer(cartCustomer, orderCustomer)) {
					return $q.reject(`CPF identificado '${cartCustomer}' difere do pedido '${orderCustomer}'!`);
				} else {
					if (!$scope.multipleOrdersEnabled()) {
						// se não tiver habilitado múltiplos pedidos, a única validação é a efetuada anteriormente
						return $q.resolve();
					} else {
						const orders = $scope.selectedOrdersToMerge && $scope.selectedOrdersToMerge.length ? $scope.selectedOrdersToMerge : [];

						const orderFound = orders.find(order => {
							const currentCustomer = extractCustomerDocument(order.customer);
							return currentCustomer !== orderCustomer;
						});

						if (orderFound) {
							const currentCustomer = extractCustomerDocument(orderFound.customer);
							return $q.reject(`CPF do pedido a ser resgatado '${orderCustomer}' difere de um dos pedidos adicionados. Pedido '${orderFound.orderNumber}', CPF: '${currentCustomer}'!`);
						} else {
							return $q.resolve();
						}
					}
				}
			}
		}

		function validatePrices(order) {
			const itemDifPriceFound = order.fiscalItems.find(fiscalItem => {
				const itemFound = CartService.findItemByProductCode(CartService.cart.fiscalItems, fiscalItem.productCode); 
				return itemFound ? fiscalItem.unitValue.toFixed(2) !== itemFound.unitValue.toFixed(2) : false;
			});

			if (itemDifPriceFound) {
				return $q.reject(`Pedido não pode ser resgatado. Existe um produto com preço diferente do atual no carrinho: Código: '${itemDifPriceFound.productCode}', valor no carrinho: ${itemDifPriceFound.unitValue.toFixed(2)}!`);
			} else {
				// verificar nos pedidos a serem resgatados
				const orders = $scope.selectedOrdersToMerge && $scope.selectedOrdersToMerge.length ? $scope.selectedOrdersToMerge : [];
				
				const itemDifPriceOrderFound = OrderChecking.extractOrderItems(orders).find(fiscalItem => {
					const itemFound = CartService.findItemByProductCode(CartService.cart.fiscalItems, fiscalItem.productCode); 
					return itemFound ? fiscalItem.unitValue.toFixed(2) !== itemFound.unitValue.toFixed(2) : false;
				});

				if (itemDifPriceOrderFound) {
					return $q.reject(`Pedido não pode ser resgatado. Existe um produto com preço diferente de um dos pedidos já resgatados no carrinho: Código: '${itemDifPriceOrderFound.productCode}', valor no outro pedido: ${itemDifPriceOrderFound.unitValue.toFixed(2)}!`);
				} else {
					return $q.resolve();
				}
			}
		}

        $scope.getExternalOrderNumberKeyboardType = () => {
            return ConfigService.config.business.external_order_number_keyboard_type == 'Numeric' ? $rootScope.platformNumberInputType : 'text';
        };

        function checkPaymentLink(order) {
            const paymentLink = getFirstPaymentLink(order);

            if (paymentLink == null) {
				return $q.resolve(order);
			}

            const paymentLinkProvider = ConfigService.config.business.paymentlink_service_provider;

            // link de pagamento da pagar.me do cxr sempre terá esse campo preenchido
            // caso não tenham é um bug
			if (paymentLinkProvider == 'PAGAR_ME' && paymentLink.paymentState == null) {
                return $q.reject('Link de Pagamento não possui Status definido');
			}

            if (order.status == 'CHECKOUT') {
                if (paymentLinkProvider != 'PAGAR_ME') {
                    return $q.reject(
                        'Apenas Pedidos com Link de Pagamento da Pagar.me suportam resgate com o'
                        + ` Status do Pedido '${order.status}'`
                    );
                }

                const allowedStates = ['CANCELED', 'ERROR', 'NONE'];
                if (!allowedStates.includes(paymentLink.paymentState)) {
                    return $q.reject(
                        `O pedido não pode ser resgatado, Status do Link de Pagamento`
                        + ` '${getPaymentStateDescription(paymentLink)}' não permite resgate`
                        + ` para pedidos com Status '${order.status}'`
                    );
                }
            } else if (order.status == 'PAYMENT') {
                // apenas pagar_me hoje suporta requests para desabilitar links...
                if (paymentLinkProvider != 'PAGAR_ME') {
                    return $q.reject(
                        'Apenas Pedidos com Link de Pagamento da Pagar.me suportam resgate com o'
                        + ` Status do Pedidos '${order.status}'`
                    );
                }

                const allowedStates = ['IN_PROGRESS'];
                if (!allowedStates.includes(paymentLink.paymentState)) {
                    return $q.reject(
                        `O pedido não pode ser resgatado, Status do Link de Pagamento`
                        + ` '${getPaymentStateDescription(paymentLink)}' não permite resgate`
                        + ` para pedidos com Status '${order.status}'`
                    );

                }

                return AlertService.confirm(
                    'O link de pagamento será cancelado, deseja continuar?'
                ).then((confirmed) => {
                    if (!confirmed) {
                        return $q.reject();
                    }
                }).then(() =>
                    $ionicLoading.show({
                        template : 'Cancelando Link de Pagamento...',
                        duration : 0
                    })
                ).then(() =>
                    PaymentLinkService
                        .disablePaymentLink(paymentLink.paymentAuthorizationNSU)
                ).then(() =>
                    $ionicLoading.show({
                        template : 'Atualizando Pedido...',
                        duration : 0
                    })
                ).then(() => {
                    // devemos atualizar o pedido para checkout
                    // e marcar o link de pagamento com status cancelado
                    let orderClone = JSON.parse(JSON.stringify(order));
                    let paymentLinkClone = getFirstPaymentLink(orderClone);
                    paymentLinkClone.paymentState = 'CANCELED';
                    orderClone.status = 'CHECKOUT';
                    return OrderService.update(orderClone);
                }).finally(() =>
                    $ionicLoading.hide()
                );
            }

            return $q.resolve(order);
        }

        function reloadCustomerV1(searchPromise, order) {
            return searchPromise.then((customer) => {
                if (!customer) {
                    // vai cair no fluxo do catch, alertando
                    // que não foi retornado o cliente
                    return $q.reject({
                        data: {
                            message: 'Nenhum cliente foi encontrado com o documento informado'
                        }
                    });
                }

                LogService.info('Substituindo dados do cliente pelo conteúdo retornado da nova consulta');
                customer.email = (order.customer && order.customer.email && 0 !== order.customer.email.length ? order.customer.email : customer.email);

                order.customer = OrderService.toOrderCustomer(
                    JSON.parse(JSON.stringify(customer))
                );

                return customer;
            }).catch(error => {
                let cause =  "Erro consultando cliente!";

                if (error && error.data && error.data.message) {
                    cause =  error.data.message + ".";
                }

                return AlertService.error(
                    `${cause} Recebimento por cartão próprio não poderá ser efetuado.`
                ).then(() => order.customer)
            });
        };

		function reloadCustomerV2(searchPromise, order) {
			return searchPromise.then((customer) => {
				if (!customer) {
					// vai cair no fluxo do catch, alertando
					// que não foi retornado o cliente
					return $q.reject({
						data: {
							message: 'Nenhum cliente foi encontrado com o documento informado'
						}
					});
				}
				LogService.info('Atualizando os dados do cliente pelo conteúdo retornado da nova consulta');
				// fazendo o merge, atualizando apenas os dados não existentes
				customer = { ...customer, ...order.customer };
				order.customer = OrderService.toOrderCustomer(
					JSON.parse(JSON.stringify(customer))
				);
				return customer;
			}).catch((err) => {
				// o ideal era alertarmos o erro e prosseguirmos
				// mas foi solicitado a remoção do alerta de erro
				LogService.error(err);
				return order.customer;
			});
		}

        function showDonationItems(order) {
		    let ret  = $q.defer();

            const layout = (donationItems) => {
                let ret = '<ion-list class="list list-inset dosis" style="margin: unset;width:100%;">';

                donationItems.forEach(item => {
                    ret += `<ion-item class="item item-remove-animate item-inset item-text-wrap">
                        <div class="row" style="display:flex;flex-flow:row wrap; justify-content:space-between;align-items:flex-start;padding: .5em 0;">
                            <div class="col col-100" style="flex:0 50%;">
                                <h2 color="primary">${item.productDescription}</h2>
                                <div class="col col-100 no-margin dosis" style="word-break:break-all;white-space: nowrap;display:flex;flex-flow:column;align-items:flex-start;padding:0;margin-top:5px;">
                                    <b> Cód.:</b> <span>${ $rootScope.getItemDisplayCode(item) }</span>
                                </div>
                            </div>
                        </div>
                      </ion-item>`;
                });

                return ret + '</ion-list>';
            };

		    if (order.donationItems && order.donationItems.length) {
		        $ionicPopup.show({
		            template: layout(order.donationItems),
	                title: 'Itens de doação nessa operação:',
	                scope: $scope,
	                buttons: [
                        {   text: 'Cancelar',
                            onTap: function() { ret.reject(); }
                        }, {
                            text: '<b>Ok</b>',
                            type: 'button-theme',
                            onTap: function() { ret.resolve(); }
                        }]
		        });
		    } else {
		        ret.resolve();
		    }

		    return ret.promise;
		}

		/**
		 * Carrega o pedido passado no aplicativo.
		 */
		$scope.selectOrderForPaymentLinkRenew = (order) => {
			AlertService.confirm(`Deseja reeviar o link de pagamento para ${order.customer.name}, CPF ''${order.customer.document}'' referente ao Pedido '${order.orderNumber}'?`)
			.then(res => {
		        if (res) {
		            return $q.resolve();
		        } else {
		            return $q.reject();
		        }
		    }).then(() => {
				return $ionicLoading.show({
	               	template : 'Processando novo Link de Pagamento...',
	              	duration : 0
	       		}).then(() => {
					const paymentLink = order.orderPayments.find(p => p.paymentCodeMethod === 'PAYMENT_LINK');
					const oldLinkNSU = paymentLink.paymentAuthorizationNSU;
                    const checkoutRequest = {
                        companyNumber:        '00001',
                        storeNumber:          ConfigService.config.storeNumber,
                        operatorName:         UserService.operatorName() ? UserService.operatorCode() : 'Não informado',
                        operatorCode:         UserService.operatorCode() ? UserService.operatorCode() : 'Não informado',
                        terminalNumber:       ConfigService.config.terminalNumber,
                        nsu:                  ConfigService.nsuControll.currentNSU(),
                        totalValue:           paymentLink.paymentValue,
                        customer:             order.customer,
                        fiscalItems:          order.fiscalItems,
                        orderNumber:          oldLinkNSU,
                        paymentPlans:         PaymentLinkService.getPlansLinx(paymentLink.paymentValue),
                        orderServerOrderNumber: order.orderNumber
                    };

					return PaymentLinkService.checkout(checkoutRequest)
					.then(data => {
						paymentLink.paymentAuthorizationNSU = data.data.orderNumber;
						$scope.paymentLink = {
							url: data.data.accessUrl,
							message: data.data.message
						};
					})
					.then(CartService.reloadNSU) // O número do Link usa a NSU para ser criado. Incrementar aqui possibilita refazer a operação.
					.then(() => {
						return PaymentLinkService.verifyPaymentLink(paymentLink.paymentAuthorizationNSU).then(resp => {
							if (resp.data.urlPaymentStatus != 'ACTIVE') {
				                return $q.reject({
									data: {	message: "Não foi possível recriar link de pagamento!" }
								});
				            }
						});
					})
					.then(() => PaymentLinkService.disablePaymentLink(oldLinkNSU))
					.then(() => sendMessageToClient(order.customer, $scope.paymentLink.message, 'Link de Pagamento'))
					.then(() => OrderService.updateAsync(order))
					.finally(() => $ionicLoading.hide())
                    .then(() => $scope.updateModal.hide())
					.then(() => AlertService.info(`Novo Link de Pagmamento criado no pedido <b>'${order.orderNumber}'</b>!`))
					.catch(AlertService.restError);
				});
			});
		};

        function sendMessageToClient(customer, message, subject) {
            return MessagingService.sendMessageToClient(customer, message, subject)
            .catch(error => {
                LogService.error(error);
                AlertService.warn('Não foi possível enviar novo link para o cliente.');
            });
        }

		/**
		 * Carrega o pedido passado no aplicativo.
		 */
		$scope.selectOrderRecreation = (order) => {
			AlertService.confirm(`Deseja copiar o Pedido na criação de um novo?`)
				.then(res => {
			        if (res) {
			            return $q.resolve();
			        } else {
			            return $q.reject();
			        }
			    }).then(() => {
					$ionicLoading.show({
		               	template : 'Carregando dados do Pedido...',
		              	duration : 0
		       		}).then(() => {
                        let promises = [];
                        CartService.initCart();

                        LogService.info ("Consultando itens do Pedido para adicionar ao carrinho...");

                        order.fiscalItems.forEach(orderItem => {
                            promises.push(ProductService.search({code: orderItem.productCode})
                            .then(result => {
                                result.items[0].quantitySold = orderItem.quantitySold;
                                CartService.addItem(result.items[0]);
                            }));
                        });

                        LogService.info (`Consultando cliente com CPF '${order.customer.document}'...`);

                        promises.push(CustomerService.searchByType(order.customer.document, 'CPF')
                        .then(result => {
                            if (result) {
                                CartService.cart.customer = result[0];
                            }
                        })
                        .catch(() => {
                            return CustomerService.searchByType(order.customer.document, 'CPF_REST')
                            .then(result => {
                                if (result) {
                                    CartService.cart.customer = result[0];
                                }
                            });
                        }));

                        $q.all(promises)
						.then(() => $scope.updateModal.hide())
						.then(() => UtilService.navigateWithoutHistory('easyorder.cart'))
						.then($ionicLoading.hide)
						.catch(error => {
							$ionicLoading.hide();
							AlertService.restError(error);
						});
                    });
				});
		};

		function donate(order) {
		    $ionicLoading.show({
                template : 'Processando Pedido de Doação...',
                duration : 0
            }).then(() => {
    		    validateDonationItems(order);

    		    const fiscalItems = order.donationItems.map(item => {
    		        return {
                        description: item.productDescription,
                        compactDescription: item.productDescription,
                        code: item.productCode,
                        automationCode: item.automationCode,
                        unitValue: item.unitValue,
                        totalValue: item.totalValue,
                        quantity: item.quantitySold,
                        uuid: parseUuid(item.imei),
                        marketingCode: item.marketingCode
                    };
    		    });

                return DonationService.donate({
                    appInfo: {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    },
                    transactionKey: {
                        storeInfo: {
                            companyNumber: ConfigService.config.companyNumber,
                            storeNumber: ConfigService.config.storeNumber,
                        },
                        date: new Date(),
                        nsu: ConfigService.nsuControll.incrementNSU(),
                        terminal: ConfigService.config.terminalNumber,
                    },
                    reason: {
                        code: 5,
                        description: 'Outros'
                    },
                    operator: {
                        code: UserService.operatorCode(),
                        name: UserService.operatorName()
                    },
                    authorizer: {
                        code: order.operatorCode
                    },
                    customer: order.customer,
                    fiscalItems: fiscalItems,
                    totalValue: fiscalItems.reduce((acc, item) => new Number((acc + item.totalValue).toFixed(2)), 0),
                    orderNumber: order.orderNumber,
                    externalOrderCode: order.externalOrderCode,
					originSystem: order.originSystem
                }).then(result => {
					return hideModals().then(() => {
						return UtilService.navigateWithoutHistory('easyorder.couponview', {
                            pdf: result.data.pdf || " ",
                            printContents: result.data.printContents || [],
                            documentTitle: 'donation',
                            showPdf: result.data.pdf != null && result.data.pdf.length > 0
                        });
					});
                }, (error) => {
                    AlertService.restError(error);
                }).finally(() => $ionicLoading.hide());
            });
        }

		function validateDonationItems(order) {
		    const invalidItem = order.donationItems.find(item => !item.imei);

            if (invalidItem) {
                const msg = `Não é permitida a doação para este produto: ${invalidItem.productDescription}`;
                AlertService.error(msg);
                throw new Error(msg);
            }
		}

		function parseUuid(uuidValue) {
            return {
                type: "ICCID",
                value: uuidValue,
            };
        }

		function validateOrderProducts(order) {
            let dofExists = false;
            let restrictionExists = false;
            let index;

	        for (index in order.fiscalItems) {
        	    var fiscalItem = order.fiscalItems[index];
				if (fiscalItem.restrictions && fiscalItem.restrictions.find(e => e.type != null && e.type == 'DOF')) {
                    dofExists = true;
                }
				if (fiscalItem.restrictions && fiscalItem.restrictions.find(e => e.type != null && e.type == 'SALE_RESTRICTION')) {
					restrictionExists = true;
				}
	        }

            if (dofExists){
				if (!order.customer) {
					AlertService.warn("Produto de Origem Florestal. Identificação do cliente obrigatória.");
                    return false;
                }
            }

            if (restrictionExists) {
				if (!order.customer) {
					AlertService.warn("VENDA PROIBIDA PARA MENORES DE 18 ANOS. Identificação do cliente OBRIGATÓRIA");
                    return false;
                }
            }


            return true;
        }

		var searchByStatus = function(params) {
			return $ionicLoading.show({
				template : 'Consultando pedido...',
				duration : 0
			}).then(function() {
				var innerParams = {
                    companyNumber: ConfigService.config.companyNumber,
                    destinationStoreNumber: ConfigService.config.storeNumber,
                    limit: ConfigService.config.business.search_orders_return_limited_to
                };
			    params = Object.assign(innerParams, params);

				return OrderService.list(params)
					.catch(AlertService.restError)
					.finally($ionicLoading.hide);
			});
		};

        var initDetailsSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.ordersearch');
        };

        var initDetailsSearchV2 = function() {
            UtilService.navigateWithoutHistory('easyorder.ordersearch-v2');
        };

		var initDetailsSearchV3 = function() {
            $scope.goSearchByStatus();
        };

        var initOnHoldSearch = function() {
        	searchByStatus({ status: "ON_HOLD" }).then(result => {
				$scope.init(result.orders, $scope.selectedOrdersToMerge);
				$scope.openModal();
			});
        };

        var initCheckoutSearch = function() {
            searchByStatus({ status: "CHECKOUT", orderDate: new Date() }).then(result => {
				$scope.init(result.orders, $scope.selectedOrdersToMerge);
				$scope.openModal();
			});
        };

		var initReadyToCheckSearch = () => {
            let params = {};
            params.status = "READY_TO_PACK";
            params.filterFromDate = new Date();
            params.filterFromDate.setDate(params.filterFromDate.getDate() - ConfigService.config.business.search_orders_return_date_minus);
			searchByStatus(params).then(result => {
				$scope.init(result.orders, $scope.selectedOrdersToMerge);
				$scope.openModal();
			});
		};

		var initRenewPaymentLinkSearch = () => {
			AlertService.prompt("Recriação de Link de Pagamento", "", "tel", "CPF do cliente")
			.then(document => {
				if (document) {
					document = document.replace(/\D/g, "");
					var promises = [];
					var innerParams = {
	                    companyNumber: ConfigService.config.companyNumber,
	                    destinationStoreNumber: ConfigService.config.storeNumber,
	                    limit: ConfigService.config.business.search_orders_return_limited_to
	                };

					return $ionicLoading.show({
						template : 'Consultando pedido...',
						duration : 0
					}).then(() => {
						promises.push(OrderService.list(Object.assign(innerParams, { status: OrderService.orderStatus.PAYMENT, orderDate: new Date(), document: document })));

						$q.all(promises)
							.then(results => {
								var orders = [];

								results.forEach(result => {
									orders.push(... result.orders);
								});
								orders = orders.filter(o => o.orderPayments
									&& o.orderPayments.length
									&& o.orderPayments.find(p => p.paymentCodeMethod === 'PAYMENT_LINK'));
								$scope.init(orders, $scope.selectedOrdersToMerge);
							})
							.then($scope.openUpdateModal)
							.catch(AlertService.restError);
					}).finally($ionicLoading.hide);
				}
			});
		};

		var initOrderCopySearch = () => {
			AlertService.prompt("Cópia de Pedidos", "", "tel", "CPF do cliente")
			.then(document => {
				if (document) {
					document = document.replace(/\D/g, "");
					var promises = [];
					var innerParams = {
	                    companyNumber: ConfigService.config.companyNumber,
	                    destinationStoreNumber: ConfigService.config.storeNumber,
	                    limit: ConfigService.config.business.search_orders_return_limited_to
	                };

					return $ionicLoading.show({
						template : 'Consultando pedido...',
						duration : 0
					}).then(() => {
						promises.push(OrderService.list(Object.assign(innerParams, { status: OrderService.orderStatus.CANCELED, orderDate: new Date(), document: document })));

						$q.all(promises)
							.then(results => {
								var orders = [];

								results.forEach(result => {
									orders.push(... result.orders);
								});
								$scope.init(orders, $scope.selectedOrdersToMerge);
							})
							.then($scope.openUpdateModal)
							.catch(AlertService.restError);
					}).finally($ionicLoading.hide);
				}
			});
		};

        $scope.goSearchByDocument = () => {
           const transitionParams = {
                flowHistory: {
                    flow: [$state.current.name],
                    params: [$stateParams]
                }
            };
            UtilService.navigateWithoutHistory('easyorder.order-doc-search', transitionParams);
        };

        $scope.goSearchByStatus = () => {
           const transitionParams = {
                flowHistory: {
                    flow: [$state.current.name],
                    params: [$stateParams]
                },
                orderCustomerDocument: 'NONE'
            }
            UtilService.navigateWithoutHistory('easyorder.order-search-list', transitionParams);
        };

        /**
         * Tipos de busca disponíveis.
         */
        var allTypes = [{
            id: "DETAILED",
            handler: initDetailsSearch,
            css: "fa fa-search",
            description: "Detalhes do Pedido"
        }, {
            id: "ON_HOLD",
            handler: initOnHoldSearch,
            css: "fa fa-check",
            description: "Separação de Estoque"
        }, {
            id: "CHECKOUT",
            handler: initCheckoutSearch,
            css: "fa fa-shopping-cart",
            description: "Liberado para Pagamento"
        },
		{
            id: "DETAILED_V3",
            handler: initDetailsSearchV3,
            css: "fa fa-shopping-cart",
            description: "Busca de Pedidos"
        },
		{
            id: "READY_TO_PACK",
            handler: initReadyToCheckSearch,
            css: "fas fa-box-open",
            description: "Pronto para Embalagem"
        }, {
            id: "RENEW_PAYMENT_LINK",
            handler: initRenewPaymentLinkSearch,
            css: "fas fa-redo-alt",
            description: "Recriar link Pagamento"
        }, {
            id: "ORDER_COPY",
            handler: initOrderCopySearch,
            css: "fas fa-file-import",
            description: "Copiar para novo Pedido"
        }, {
            id: "DETAILED_V2",
            handler: initDetailsSearchV2,
            css: "fas fa-search",
            description: "Busca de Pedidos"
        }];

        /**
         * Função de inicialização.
         */
        (function() {
			$scope.allowDifferentCustomerInOrders = ConfigService.config.business.allow_different_customers_on_orders !== false;

            var search_types = ConfigService.config.business.order_search_types;

            search_types = search_types ? search_types.split(";") : ["DETAILED"];

            $scope.allowedSearchTypes = (function(data) {
                var ret = [];
                while (data.length) {
                    ret.push(allTypes.filter(function(type) {
                        return type.id === this[0] || type.id === this[1];
                    }, data.splice(0, 2)));
                }
                return ret;
            })(search_types);
        })();
	});
})();
