(function() {
    'use strict';

    angular.module('easyorder.controllers.passwordgiftcardpayment', ["angular-uuid"])
    .controller('PasswordGiftCardPaymentCtrl', function(
                                                    $stateParams,
                                                    $ionicLoading,
                                                    $scope,
                                                    $q,
                                                    CartService,
                                                    UtilService,
                                                    ConfigService,
                                                    PaymentProcessor,
                                                    PaymentService,
                                                    AlertService,
                                                    GiftCardService,
                                                    uuid) {

        $scope.formData = {};
        $scope.cardNumber = $stateParams.giftCard.cardNumber;
        const contextUid = uuid.v4();
        const provider = ConfigService.config.business.giftcard_service_provider;

        AlertService.restError = new Proxy(AlertService.restError, {
            apply: applyProxy
        });

        function applyProxy(target, __, argumentsList, load) {
            return $ionicLoading.hide()
                .then(() => target(...argumentsList))
                .then((res) => {
                    if (load) {
                        return $ionicLoading.show()
                            .then(() => res);
                    }
                    return res;
                });
        }

        let lock = false;
        $scope.authorize = (password) => {
            if (lock) {
                return;
            }
            lock = true;

            $ionicLoading
                .show()
                .then(() =>
                    authorizeGiftCardPayment(
                        $stateParams.giftCard.cardNumber,
                        password
                    )
                ).then(() =>
                    navigate()
                ).catch((err) => {
                    if (err == null) { // apertou em cancelar no popup
                        return $ionicLoading.hide();
                    }
                    if (provider == 'PROMO') {
                        undoPendingPayment();
                    }
                    return AlertService.restError(err, "Não foi possível autorizar o Cartão Presente");
                }).finally(() => {
                    lock = false;
                });
        }

        function authorizeGiftCardPayment(cardNumber, password) {
            const currentPayment = PaymentService.currentPayment();

            if (provider == 'PROMO') {
                PaymentProcessor.pushSalePayment({
                    type: 'SaleGiftCardPayment',
                    paymentCodeMethod: 'GIFTCARD',
                    companyNumber : ConfigService.config.companyNumber,
                    storeNumber : ConfigService.config.storeNumber,
                    terminalNumber : ConfigService.config.terminalNumber,
                    cardTypeId : ConfigService.config.business.giftcard_type,
                    cardNumber: trail,
                    amount: new Number(PaymentService.currentPayment().paymentValue).toFixed(2),
                    contextUid: contextUid,
                });
            }

            return GiftCardService.authGiftCardPayment({
                amount: new Number(currentPayment.paymentValue).toFixed(2),
                cardNumber: cardNumber,
                companyNumber: currentPayment.companyNumber,
                storeNumber: currentPayment.storeNumber,
                terminalNumber: ConfigService.config.terminalNumber,
                nsu: ConfigService.nsuControll.currentNSU(),
                securityCode: password,
                cardTypeId: ConfigService.config.business.giftcard_type,
                contextUid: contextUid
            }).then((res) => {
                
                const wasAuthorized = (res) => {
                    if (provider == 'PROMO') {
                        return res.paymentAuthorizationNSU != null 
                            && res.paymentAuthorizationNSU.length > 0;
                    } else {
                        return res.responseCode && res.responseCode === "00";
                    }
                }
                
                if (!wasAuthorized) {
                    return $q.reject(res.message);
                }
                $scope.processPayment(res, CartService.cart, PaymentService.currentPayment());
                PaymentService.currentPaymentDone();
            });
        }

        $scope.processPayment = function(paymentResults, cart, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SaleGiftCardPayment';
            salePayment.installments                = [];
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = new Number(currentPayment.paymentValue).toFixed(2);
            salePayment.paymentAuthorizationNSU     = paymentResults.paymentAuthorizationNSU;
            salePayment.cardNumber                  = paymentResults.cardNumber;
            salePayment.customerPaymentVoucher      = paymentResults.customerPaymentVoucher;
            salePayment.paymentCodePlan             = 2; //À Vista.
            salePayment.providerName                = paymentResults.providerName;
            if (provider == 'PROMO') {
                PaymentProcessor.updateSalePayment(
                    (payment) => payment.data.confirm.contextUid == contextUid,
                    { ...salePayment, contextUid: contextUid },
                );
            } else {
                PaymentProcessor.pushSalePayment({
                    ...salePayment, contextUid: contextUid
                });
            }
            cart.salePayments.push(salePayment);

        }

        function undoPendingPayment() {
            PaymentProcessor.undoPayment(
                (payment) => payment.data.undo.contextUid == contextUid
            );
            PaymentService.clearInitializedPaymentData();
        }

        function navigate() {
            return UtilService.navigateAfterReceivedPayment();
        }
    });
})();

