(function() {
    'use strict';

    angular.module('easyorder.controllers.covenantpayment', [
        'easyorder.services.cart',
        'easyorder.services.nfce',
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.services.log'
    ])

    .controller('CovenantPaymentCtrl', function($scope,
                                        $ionicLoading,
                                        $timeout,
                                        CartService,
                                        UserService,
                                        UtilService,
                                        PaymentService,
                                        CovenantService,
                                        AlertService,
                                        LogService) {

        $scope.customer = CartService.cart.customer;
        
        /**
         * 
         */
        $scope.isTransactionInitiated = function() {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $scope.message = "";
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            if (message) {
                $scope.message = message;
            }
            LogService.info("displayMessage: " + $scope.message);
        };

        /**
         *
         */
        $scope.inputText = function(label, type, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               type,
                maxSize:            fieldSizeRange.maxSize,
                minSize:            fieldSizeRange.minSize
            });
            $scope.headerOptions = "Informe";
        };


        /**
         * Função executa a função com os parâmetros passados a esta.
         */
        $scope.tickHandler = function(func, data) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func(data);
        };

        /**
         * Solicita autorização para realização de pagamento.
         */
        $scope.covenantSaleAuth = function(data) {
            $scope.clearDisplay();

            $timeout(function() {
                $scope.displayMessage("Autorizando pagamento. Favor aguarde...");
            }, 800); //Firula.
            
            let userAuthRequest             = {};
            userAuthRequest.covenantCode    = CovenantService.currentCovenant().code;
            userAuthRequest.operatorCode    = UserService.operatorCode();
            userAuthRequest.userLogin       = CovenantService.currentCovenant().covenantAuth.cardNumber;
            userAuthRequest.userPassword    = data.value;
            
            CovenantService.covenantAuthUser(userAuthRequest, function(userAuthResponse) {
                let reqParams           = {};
                reqParams.paymentValue  = new Number(PaymentService.currentPayment().paymentValue).toFixed(2);

                CovenantService.covenantSaleAuth(reqParams, function(result) {
                    $scope.displayMessage("Pagamento autorizado!");

                    $timeout(function() {
                        $scope.paymentDone(result);
                    }, 1500); //Firula.
                }, function(error) {
                    $scope.clearDisplay();

                    AlertService.restError(error, 'Não foi possível autorizar pagamento!').then(function(result) {
                        $scope.paymentError();
                    });
                });
            }, function(error) {
                $scope.clearDisplay();

                AlertService.restError(error, 'Não foi possível autorizar pagamento!').then(function(result) {
                    $scope.paymentError();
                });
            });
        };

        /**
         * Função para tratamento após finalização do pagamento com cartão próprio
         */
        $scope.paymentDone = function(paymentResults) {
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(function() {
                $scope.processPayment(paymentResults, CartService.cart, PaymentService.currentPayment());
                
                PaymentService.currentPaymentDone();
                UtilService.navigateAfterReceivedPayment();
            });
        };

        /**
         * Processa o pagamento atual e o resultado da transação TEF no objeto enviado para integração e geração da NFC-e.
         */
        $scope.processPayment = function(paymentResults, cart, currentPayment) {
            var salePayment = {};
            
            salePayment.type                        = 'SaleCovenantPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = new Number(currentPayment.paymentValue).toFixed(2);

            if (currentPayment.plan) {
                salePayment.paymentCodePlan             = currentPayment.plan.code;
                salePayment.paymentCodeEvent            = currentPayment.plan.paymentCodeEvent;
                salePayment.paymentPlanExternalCode     = currentPayment.plan.externalCode;
                salePayment.paymentPlanProvider         = currentPayment.plan.provider;
                salePayment.numberOfInstallments        = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment       = currentPayment.plan.firstInstalment.dueDate;
                salePayment.firstInstalmentValue        = new Number(currentPayment.plan.firstInstalment.instalmentValue).toFixed(2);
                salePayment.installments                = [];
                salePayment.discountValue               = 0;   

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { // Convertendo Instalment em InstallmentsDetails
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = new Number(instalment.instalmentValue).toFixed(2);
                    instalmentDetails.instalmentOriginalValue   = new Number(instalment.instalmentOriginalValue ? instalment.instalmentOriginalValue : 0).toFixed(2);

                    this.push(instalmentDetails);
                }, salePayment.installments)

                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            }
            salePayment.paymentAuthorizationNSU     = paymentResults.nsu;
            salePayment.paymentAuthorizerBrandName  = paymentResults.paymentAuthorizerBrandName;
            salePayment.cardNumber                  = CovenantService.currentCovenant().covenantAuth.cardNumber;
            salePayment.customerPaymentVoucher      = paymentResults.paymentVouchers[0];
            salePayment.covenantCode                = paymentResults.covenantCode;
            
            salePayment.installments.sort(function(a,b) {
                return a.paymentSequence - b.paymentSequence;
            });
            cart.salePayments.push(salePayment);
        };

       /**
        * Função desfaz os pagamentos.
        */
       $scope.paymentError = function() {
           $q.resolve()
               .then(undoPaymentCampaigns)
               .then(() => UtilService.navigateWithoutHistory('easyorder.paymentmethods'));
       };
       
       async function undoPaymentCampaigns() {
           const campaigns = PaymentService.currentPayment.campaigns;

           if (campaigns && campaigns.length) {
               campaigns.forEach(campaign => {
                   LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                   CartService.undoCampaignDiscountsPartial(campaign);
               });
           }
       };

        /**
         * Função de inicialização.
         */
        (function() {
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.options = [];
            $scope.nfceAuthResponse = {};
            $scope.transactionInitiated = true;

            $scope.clearDisplay();

            $timeout(function() {
                $scope.displayMessage("Digite a senha para autorização da venda");
            }, 800); //Firula.
            $scope.inputText("Senha do Cartão", "tel", $scope.covenantSaleAuth, { minSize: 4, maxSize: 15 });
        })();
    });
})();
