(() => {
    'use strict';

    angular
    .module('easyorder.controllers.genericpayment', [
        'easyorder.services.log'
    ])

    .controller('GenericPaymentCtrl', function($scope,
                                                $q,
                                                $timeout,
                                                $ionicLoading,
                                                FinancialService,
                                                PaymentService,
                                                CartService,
                                                UtilService,
                                                LogService,
                                                ConfigService) {

        /**
         *
         */
        $scope.isTransactionInitiated = function() {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $scope.message = "";
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            if (message) {
                $scope.message = message;
            }
            LogService.info("displayMessage: " + $scope.message);
        };

        /**
         * Função para tratamento após finalização do pagamento.
         */
        $scope.paymentDone = function() {
            $scope.processPayment(CartService.cart, PaymentService.currentPayment()).then(() => {
                $scope.clearDisplay();

                $timeout(() => {
                    $ionicLoading.show({
                        template : 'Por favor, aguarde...',
                        duration : 0
                    }).then(function() {
                        PaymentService.currentPaymentDone();
                        UtilService.navigateAfterReceivedPayment();
                    })
                }, 200);
            });
        };

        /**
         * Processa o pagamento atual e o resultado da transação TEF no objeto enviado para integração e geração da NFC-e.
         */
        $scope.processPayment = function(cart, currentPayment) {
            var salePayment = {};
            var defer = $q.defer();

            salePayment.type                        = 'SaleGenericPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;

            salePayment.paymentValue = currentPayment.paymentValue;
            salePayment.installments = [];

            if (currentPayment.plan) {
                salePayment.paymentCodePlan       = currentPayment.plan.code;
                salePayment.paymentCodeEvent      = currentPayment.plan.paymentCodeEvent;
                salePayment.numberOfInstallments  = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment = currentPayment.plan.firstInstalment.dueDate;

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { // Convertendo Instalment em InstallmentsDetails
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = instalment.instalmentValue;

                    this.push(instalmentDetails);
                }, salePayment.installments)

                if (currentPayment.plan.discount) {
                    var discount = {}

                    discount.sequenceId     = cart.discounts.length;
                    discount.value          = currentPayment.plan.discount;
                    discount.source         = 'PLAN';

                    salePayment.paymentValue -= currentPayment.plan.discount; /*O desconto deve ser contabilizado no valor do pagamento enviado para o WS.*/
                    // atualiza valor do pagamento
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));

                    cart.discounts.push(discount);
                }
                if (currentPayment.plan.charges) {
                    cart.additionalCharge += currentPayment.plan.charges;
                    salePayment.chargeValue     = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }
                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            }

            if (currentPayment.predefinedDetails) {
                let predefinedDetails =  currentPayment.predefinedDetails;
                salePayment.paymentAuthorizerBrandType = predefinedDetails.paymentAuthorizerBrandName;
            }
            cart.salePayments.push(salePayment);

            if (CartService.cart.orderRetrieved) {
                var cart = CartService.cart;
                delete cart.orderStatus;
                const appInfo = {
                    name: $scope.APP_NAME,
                    version: $scope.appVersion
                };
                const orderInfo = {
                    orderNumber: cart.orderNumber,
                    externalOrderCode: cart.externalOrderCode,
                    originSystem: cart.originSystem,
                    orderDate: cart.orderDate,
                    deliveryOption: cart.deliveryOption,
                    footerTextLines: (cart.cupomFooterNote && cart.cupomFooterNote.textLines?cart.cupomFooterNote.textLines:null),
                    sellerName: cart.operatorCode // Não existe o nome do vendedor no 'Order'.
                };
                FinancialService.processFinancialGenericOrderPayments(cart, orderInfo, appInfo)
                    .then(ret => {
                        cart.genericOrderPaymentsFinancialDocuments = ret.data;
                    }).finally(() => {
                        defer.resolve();
                    });
            } else {
                defer.resolve();
            }
            return defer.promise;
        };

        /**
         * Função de inicialização.
         */
        (function() {
            $scope.transactionInitiated = true;
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.displayMessage("Criando pagamento Genérico...");
            $timeout(() => $scope.paymentDone(), 200);
        })();
    });
})();
