(function () {
    'use strict';

    angular.module('easyorder.controllers.paymentlink', [
    ]).controller('PaymentLinkCtrl', function (
		$q,
		$scope,
		$rootScope,
		$state,
		$ionicLoading,
		$window,
		$interval,
		$timeout,
		PaymentLinkService,
		CartService,
		PaymentService,
		MessagingService,
		UtilService,
		LogService,
		AlertService,
		OrderService,
		UserService,
		PaymentProcessor,
		ConfigService
    ) {
		const self = this;

		$scope.$on('$destroy', () => {
			delete $rootScope.back;
			if ($scope.paymentPoll != null) {
				$interval.cancel($scope.paymentPoll)
			}
		});

		const SCREEN_MODES = {
			MANUAL_ORDER_CREATION: {
				showCopyButton: true,
				disableCopyButton: !ConfigService.config.business.payment_link_enable_copy_url_button,
				showActionButton: true,
				goBackOnError: false,
				orderCreationMode: 'MANUAL',
				orderNotificationType: 'ALERT',
				completionStages: ['ORDER-CREATION', 'AFTER-PAYMENT'],
			},
			AUTOMATIC_ORDER_CREATION: {
				showCopyButton: false, // não faz sentido exibirmos
				disableCopyButton: true, // já que não exibimos, não tem problemas desabilitar
				showActionButton: false, // não faz sentido exibirmos
				goBackOnError: true,
				orderCreationMode: 'AUTOMATIC',
				orderNotificationType: 'ON-SCREEN',
				completionStages: ['SEND-LINK'],
			}
		};

		$scope.screenMode =
			SCREEN_MODES[ConfigService.config.business.payment_link_screen_mode]
			|| SCREEN_MODES.MANUAL_ORDER_CREATION;

		$scope.actionText = CartService.cart.currentOrder ?
			"Atualizar Pedido" : "Gerar Pedido";

		$scope.paymentLinkSettings = CartService.cart.paymentLinkSettings;

		$scope.meanOfSendingMessages =
			ConfigService
				.config
				.business.payment_link_send_methods.filter(
					item => item == 'WHATSAPP' || item == 'SMS'
				);
		if ($scope.meanOfSendingMessages.length == 1) {
			$scope.paymentLinkSettings.meanOfSendingMessage = $scope.meanOfSendingMessages[0];
		}

        // utilizado caso operador clique para voltar a tela
        self.undoData = {
            paid: false,
            campaigns: [],
        };

		$scope.loadingMessage = 'Gerando o Link de Pagamento...';
        const paymentValue = PaymentService.currentPayment().paymentValue;

		$scope.paymentLink = {
			number: PaymentLinkService.getOrderNumber()
		};

		let proc = createLink()
			.then(onLinkCreated)
            .then(checkBackLock)
            .then($timeout(4000))
            .then(checkBackLock)
			.then(validateLink)
			.then(checkBackLock);

		switch ($scope.screenMode.orderCreationMode) {
		case 'AUTOMATIC':
			proc = proc
				.then(() => $scope.createOrder())
				.then(checkBackLock)
				.then(sendLink);
			break;
		case 'MANUAL':
		default:
			proc = proc
				.then(sendLink)
				.then(checkBackLock)
				.then(waitPayment)
				.finally(() => $interval.cancel($scope.paymentPoll))
				.then(finalizePaymentProcess);
			break;
		}

		proc.catch(err =>
			AlertService
				.restError(err, err)
				.then(goBack)
		);

        function createLink() {
            const checkoutRequest = {
                companyNumber:        '00001',
                storeNumber:          ConfigService.config.storeNumber,
                operatorName:         UserService.operatorName() ? UserService.operatorCode() : 'Não informado',
                operatorCode:         UserService.operatorCode() ? UserService.operatorCode() : 'Não informado',
                terminalNumber:       ConfigService.config.terminalNumber,
                orderNumber:          $scope.paymentLink.number,
                nsu:                  ConfigService.nsuControll.currentNSU(),
                totalValue:           paymentValue,
                customer:             PaymentLinkService.getCustomer(),
                fiscalItems:          CartService.cart.fiscalItems,
                paymentPlans:         PaymentLinkService.getPlansLinx(paymentValue),
				messageProtocol:	  $scope.paymentLinkSettings.meanOfSendingMessage,
            };
            return PaymentLinkService.checkout(checkoutRequest);
        }

		async function onLinkCreated(data) {
	        $scope.paymentLink.url = data.data.accessUrl;
	        $scope.paymentLink.message = data.data.message;
			$scope.paymentLink.messageContentType = data.data.messageContentType;
			$scope.paymentLink.messageContentParameters = data.data.messageContentParameters;
		}

		async function validateLink() {
			$scope.loadingMessage = 'Verificando link gerado...';
			return PaymentLinkService.verifyPaymentLink($scope.paymentLink.number).then(checkLinkState);
		}

		/**
         * Tratador para consulta de Link de Pagamento
         */
        async function checkLinkState(searchResponse) {
            if (searchResponse.data.urlPaymentStatus != 'ACTIVE') {
				return $q.reject({
					data: {	message: "Link de pagamento não foi ativado, realize o pagamento novamente." }
				});
            }
        }

		async function sendLink() {
			$scope.loadingMessage = 'Enviando link de pagamento, aguarde...';

			let msgType = $scope.paymentLinkSettings.meanOfSendingMessage;
			let msgTypeLabel;
			let msgFn;

			switch (msgType) {
			case 'SMS':
				msgFn = MessagingService.sendSms;
				msgTypeLabel = 'SMS';
				break;
			case 'WHATSAPP':
			default:
				msgFn = MessagingService.sendWhatsApp;
				msgTypeLabel = 'WhatsApp';
				break;
			}

			return msgFn(
				undefined,
				$scope.paymentLinkSettings.messageReceiverCustomer.phoneNumber,
				$scope.paymentLink.message,
				$scope.paymentLink.messageContentType,
				$scope.paymentLink.messageContentParameters
			).then(() => {
				if ($scope.screenMode.completionStages.includes('SEND-LINK')) {
					$scope.completed = true;
				}
				$scope.loadingMessage = 'Link enviado com sucesso';
				return $timeout(1500);
			}).then(() => {
				let ret = $q.resolve();
				if ($scope.screenMode.completionStages.includes('SEND-LINK')) {
					ret = ret
						.then(() => PaymentService.confirmPayments())
						.then(startNewSale);
				}
				return ret;
			}).catch(() => {
				let msg = `Não foi possível enviar dados do link via ${msgTypeLabel}.`;
				if ($scope.screenMode.showCopyButton) {
					msg += ' O link deve ser copiado para envio ao cliente.'
				}
				return $scope.screenMode.goBackOnError ?
					$q.reject(msg) : AlertService.warn(msg);
			});
		}

		function waitPayment() {
			$scope.loadingMessage = 'Aguardando pagamento...';
			let defer = $q.defer();
			let polling = false;

			$scope.paymentPoll = $interval(async () => {
				if (polling) {
					return;
				}

				polling = true;

				try {
					await checkBackLock();
				} catch(err) {
					defer.reject(err);
					polling = false;
					return;
				}

				LogService.info('Consultando status do uso do link de pagamento...');

				PaymentLinkService.verifyPaymentLink($scope.paymentLink.number).then(resp => {
					if (resp.data.urlPaymentStatus == 'USED') {
						return $q.resolve();
					} else if (resp.data.urlPaymentStatus == 'INACTIVE' || resp.urlPaymentStatus == 'EXPIRED') {
						defer.reject({
							data: {	message: "Link de pagamento inativo ou expirado." }
						});
						return $q.reject();
					} else {
						return $q.reject();
					}
				}).then(() => {
					LogService.info('Consultando status do pagamento do link...');

					return PaymentLinkService.verifyTransaction($scope.paymentLink.number).then(resp => {
						const status = resp.data.transactionStatus;

						if (status == 'CAPTURED' || status == 'AUTHORIZED') {
							LogService.info('Pagamento do link realizado.');
							defer.resolve(resp.data);
						} else if (status == 'NOT_AUTHORIZED' || status == 'REJECT' || status == 'CANCELED') {

							LogService.info('Consultando status do pagamento do link...');
							defer.reject({
								data: {	message: "O pagamento não foi aceito, realize-o novamente." }
							});
							return $q.reject();
						} else {
							return checkBackLock().catch(err => {
								defer.reject(err);
								return $q.reject();
							});
						}
					});
				}).finally(() => polling = false);
			}, 10000);
			return defer.promise
		}

		/**
         * Finaliza a compra após confirmação
         * @param {*} resp
         */
		 async function finalizePaymentProcess(resp) {
            // protegendo de entrarmos aqui várias vezes
            if ($scope.completed) {
                return;
            }
			$scope.completed = true;
			$ionicLoading.show({
                duration: 0,
            }).then(() => {
	            LogService.info(`Número do Pedido = ${resp.orderNumber}, REQUEST_ID = ${resp.requestId}`);

				PaymentService.clearInitializedPaymentData();

				for (const transaction of resp.transactions) {
					if (transaction.status != "AUTHORIZED" && transaction.status != "CAPTURED") {
						continue;
					}

					PaymentService.initPayment(transaction.type, transaction.amount, false);

					const salePayment = {
						paymentSequence: PaymentService.payments().length,
						paymentCodeMethod: transaction.type, // o LWS já retorna no formato correto
						paymentTransactionId: transaction.id,
						paymentAuthorizationNSU: transaction.nsu,
						paymentGroupId: $scope.paymentLink.number,
						paymentValue: transaction.amount,
						numberOfInstallments: transaction.installments,
						installments: [], // a api não nos retorna data de vencimento e valores por parcelas
						paymentMethodIndicator: transaction.installments > 1 ? 1 : 0,
						paymentCustomer: preparePaymentCustomer($scope.paymentLinkSettings.messageReceiverCustomer),
						paymentProvider: transaction.paymentProvider,
						paymentChannel: transaction.paymentChannel,
						customerPaymentVoucher: transaction.customerPaymentVoucher,
						paymentOrigin: {
							type: 'SALE',
							id: parseOriginId('SALE'),
							application: {
								id: 'STOREX_MOBILE',
								vendor: {
									id: 'LINX'
								},
							}
						}
					};

					switch(transaction.type) {
						case "CREDIT":
						case "DEBIT":
							salePayment.type= 'SaleTEFPayment',
							salePayment.paymentAuthorizerNSU = transaction.authorizerNsu;
							// Apenas fizemos este artifício pois no ambiente simulado da pagar.me não é retornado o código da adquirente
							salePayment.paymentAuthorizerCode = transaction.authorizerCode || '0000';
							salePayment.paymentAuthorizationCode = transaction.authorizationCode;
							salePayment.acquirerName = transaction.authorizerName;
							salePayment.paymentAuthorizerCNPJ = transaction.authorizerCNPJ;

							const card = transaction.card;
							if (card) {
								salePayment.paymentAuthorizerBrandName = card.brandCard;
								salePayment.cardNumber = card.bin;
							}
							break;
						case 'PIX':
							salePayment.type = 'SalePixPayment';
							break;
						default:
							// se entrar aqui é erro de implementação...
							throw new Error(
								`Forma de pagamento retornada do link ainda não suportada: '${transaction.type}'.`);
					}

					CartService.cart.salePayments.push(salePayment);
					PaymentService.currentPaymentDone();
				}

				LogService.info("Pagamento realizado com sucesso! Adicionando pagamento de link ao processador de pagamentos...");
				PaymentProcessor.pushSalePayment({
					type: SalePaymentTypes.SALE_PAYMENT_LINK,
					paymentAuthorizationNSU: resp.orderNumber,
					paymentValue: paymentValue,
					paymentSequence: PaymentService.payments().length,
					paymentCodeMethod: "PAYMENT_LINK"
				});

				// FIXME: não estamos validando se o link foi pago por completo...
				UtilService.navigateWithoutHistory('easyorder.paymentresult')
            });
        }

		$scope.confirmMeansOfSendingMessages = () => {
			const phone = $scope.paymentLinkSettings.messageReceiverCustomer.phoneNumber;
			if (!isPhoneNumberValid(phone)) {
				return AlertService.warn('Telefone inválido')
			}
			CartService.cart.paymentLinkSettings = $scope.paymentLinkSettings;
			return UtilService.navigate('easyorder.paymentlink');
		}

		function isPhoneNumberValid(phone) {
			return MessagingService.isPhoneNumberValid(phone);
		}

		// FIXME: esta função deve futuramente ficar exposta para todos os javascripts
        function parseOriginId(type) {
            let ret = null;

            const cfg = ConfigService.config;

            if (type == 'SALE') {
                const companyNumber = `${cfg.companyNumber}`.padStart(5, '0');
                const storeNumber = `${cfg.storeNumber}`.padStart(5, '0');
                const terminalNumber = `${cfg.terminalNumber}`.padStart(5, '0');
                const nsu = `${CartService.cart.nsu}`.padStart(5, '0');
                return `${companyNumber}_${storeNumber}_${terminalNumber}_${parseDate(new Date())}_${nsu}`;
            } else if (type == 'ORDER') {
                return CartService.cart.orderNumber;
            }

            return ret;
        }

        function parseDate(dt) {
            const day = `${dt.getDate()}`.padStart(2, '0');
            const month = `${dt.getMonth() + 1}`.padStart(2, '0');
            return `${dt.getFullYear()}${month}${day}`;
        }

		$scope.copyToClipboard = function () {
        	var copyElement = angular.element('<textarea/>');

			copyElement.css({
                position: 'fixed',
                opacity: '0'
            });
			copyElement.val($scope.paymentLink.url);

            var body = angular.element($window.document.body);
            body.append(copyElement);

            copyElement[0].select();

			if (document.execCommand('copy')) {
            	$scope.copySuccess = true;
			} else {
				AlertService.warn("Não foi possível copiar link!");
			}
            copyElement.remove();
        }

		$scope.createOrder = () => {
			var currentOrder = CartService.cart.currentOrder;

			$scope.orderData = {
				old: currentOrder
			}

			let confirmPromise = $q.resolve(true);

			if ($scope.screenMode.orderCreationMode == 'MANUAL') {
				const message = currentOrder ?
					'Deseja atualizar o Pedido para esta compra?' : 'Deseja criar um Pedido para esta compra?';
				confirmPromise = confirmPromise.then(checkBackLock)
                .then(AlertService.confirm(message))
                .then(checkBackLock);
			}

			return confirmPromise.then(function(result) {
               if (result) {
					if (currentOrder) {
						$scope.loadingMessage = `Atualizando o Pedido '${currentOrder.orderNumber}' para pagamento com Link...`;
					} else {
						$scope.loadingMessage = `Criando Pedido para pagamento com Link...`;
					}

					createOrderPayment();

					if (currentOrder) {
						currentOrder = JSON.parse(JSON.stringify(currentOrder));
					}

					var order = OrderService.cartToOrderConverter(CartService.cart);
					order.status = OrderService.orderStatus.PAYMENT;

					for (var i = 0; i < order.fiscalItems.length; i++) {
						var fiscalItem = order.fiscalItems[i];

						if (!fiscalItem.sequenceId) {
							fiscalItem.sequenceId = i + 1;
						}

						fiscalItem.deliveryOption = $rootScope.createDANFE && !ConfigService.config.business.ignore_mid_settings ? 'SOLD_IN_STORE' : 'PICK_UP_FROM_WAREHOUSE';
						fiscalItem.totalValue = Math.round(fiscalItem.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
					}

					order.totalValue =  Math.round(order.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.

					let actionPromise;

					if (currentOrder) {
						currentOrder.fiscalCustomer = order.fiscalCustomer;
						currentOrder.nsu = order.nsu;
						currentOrder.orderPayments = order.orderPayments;
						currentOrder.fiscalItems = order.fiscalItems;
						currentOrder.nonFiscalItems = order.nonFiscalItems;
						currentOrder.totalValue = order.totalValue;
						currentOrder.status = order.status;
						currentOrder.terminalNumber = order.terminalNumber;
						currentOrder.storeNumber = order.storeNumber;
						currentOrder.companyNumber =  order.companyNumber;
						currentOrder.operatorCode = order.operatorCode;
						currentOrder.orderDate =
							currentOrder.orderDate || new Date();

						actionPromise = OrderService.update(currentOrder)
						.then(updatedOrder => {
							$scope.orderData.dirty = true;
							$scope.orderData.new = updatedOrder;
							$scope.loadingMessage = `Pedido '${updatedOrder.orderNumber}' atualizado`;
							return $scope.loadingMessage;
						});
					} else {
						actionPromise = OrderService.create(order)
						.then(newOrder => {
							$scope.orderData.dirty = true;
							$scope.orderData.new = newOrder;
							$scope.loadingMessage = `Pedido '${newOrder.orderNumber}' criado`;
							return $scope.loadingMessage;
						});
					}

					let proc = actionPromise;

					if ($scope.screenMode.completionStages.includes('ORDER-CREATION')) {
						proc = proc.then(checkBackLock)
                        .then(actionPromise).then((msg) => {
							$scope.completed = true;
							return msg;
						});
					}

					if ($scope.screenMode.orderNotificationType == 'ON-SCREEN') {
						// delay de 2s, para o operador ler na tela o
						// número do pedido
						proc = proc.then(() => $timeout(2000));
					} else {
						proc = proc.then(AlertService.info);
					}

					if ($scope.screenMode.completionStages.includes('ORDER-CREATION')) {
						proc = proc
							.then(() => PaymentService.confirmPayments())
							.then(startNewSale);
					}

					if ($scope.screenMode.goBackOnError) {
						return proc;
					} else {
						return proc.catch(AlertService.restError);
					}
				}
			});
		}

		function createOrderPayment() {
			const nsuLink = $scope.paymentLink.number;
            LogService.info(`Criando pagamento em link: ${nsuLink}`);

            CartService.cart.salePayments.push({
                type: 'SaleGenericPayment',
                paymentValue: paymentValue,
                paymentSequence: PaymentService.payments().length,
                paymentCodeMethod: 'PAYMENT_LINK',
				paymentAuthorizationNSU: nsuLink,
				paymentCustomer: preparePaymentCustomer($scope.paymentLinkSettings.messageReceiverCustomer),
				paymentState: 'IN_PROGRESS',
                installments: [],
				mandatoryMethod: true,
            });

            PaymentService.currentPaymentDone();
		}

		function preparePaymentCustomer(customer) {
			if (!customer || !customer.phoneNumber) {
				return customer;
			}

			customer = clone(customer);
			customer.phoneNumber = customer.phoneNumber.replace(/[^0-9]/g, '');
			// provavelmente é um telefone brasileiro, logo por enquanto deixaremos
			// hardcoded o prefixo 55
			if (!customer.phoneNumber.startsWith('55')) {
				customer.phoneNumber = `55${customer.phoneNumber}`;
			}

			return customer;
		}

		function clone(arg) {
			return arg != null ? JSON.parse(JSON.stringify(arg)) : arg;
		}

       	async function startNewSale() {
        	CartService.initCart();
			UtilService.navigateToMainScreen();
		}

		function goBack() {
			if ($scope.paymentLink.url) {
				PaymentService.cancel(false)
				.then(CartService.reloadNSU) //Possibilita que o Link possa ser usado novamente como forma de pagamento.
				.then(() =>
					PaymentLinkService.disablePaymentLink($scope.paymentLink.number)
						.catch(err => LogService.warn(err, 'Não foi possível desativar o link de pagamento.'))
				).then(() => {
					let ret = $q.resolve();

					if ($scope.orderData && $scope.orderData.dirty) {
						// se tivermos o campo old nesse objeto
						// significa que nós estávamos atualizando
						// o objeto old contém o estado do order antes
						// de manipularmos
						if ($scope.orderData.old) {
							ret = OrderService.update($scope.orderData.old);
						} else {
							$scope.orderData.new.state = 'CANCELED';
							ret = OrderService.update($scope.orderData.new);
						}
					}

					return ret.catch(err => LogService.error(err, 'Não foi possível reverter ou cancelar o pedido.'));
				}).finally(() =>
					UtilService.navigateWithoutHistory('easyorder.paymentmethods')
				);
			} else {
				UtilService.navigateWithoutHistory('easyorder.paymentmethods');
			}
		}

        $rootScope.back = () => {
            if ($scope.completed) {
                return;
            }
            if ($scope.backLock) {
                return;
            }
            const defer = $q.defer();
            $scope.backLock = defer.promise;
            AlertService.confirm(
                "O link de pagamento será cancelado deseja continuar?"
            ).then(accepted => {
                if (accepted) {
                    defer.reject({ data: { message: 'Operação Cancelada' }});
                } else {
                    defer.resolve(() => $scope.backLock = null);
                }
            });
        }

		async function checkBackLock(arg) {
			return $scope.backLock != null ? $scope.backLock.then(() => arg)
										   : $q.resolve(arg);
		}
    });
})();
