(function () {
    'use strict';

    angular.module('easyorder.controllers.sitef.paymentlink', [
    ]).controller('SitefPaymentLinkCtrl', function (
        $ionicLoading,
        $scope,
        $rootScope,
        $window,
        $interval,
        PaymentLinkService,
        CartService,
        PaymentService,
        MessagingService,
        UtilService,
        LogService,
        AlertService,
        OrderService,
        ConfigService,
        UserService
    ) {
        $scope.loadingMessage = 'Gerando o Link de Pagamento...';
        $scope.paymentLink = {};
        $scope.completed = false;
        
        var serverOrder = undefined;
        const paymentValue = PaymentService.currentPayment().paymentValue;
        
        createOrder()
        .then(checkout)
        .then(handleLinkCreation)
        .then(updateOrderPayment)
        .then(PaymentService.confirmPayments)
        .then(() => $scope.completed = true)
        .then(sendMessageToClient)
        .then(processDone)
        .catch(err => {
            AlertService.restError(err)
            .then($ionicLoading.show)
            .then(goBack)
            .finally($ionicLoading.hide);
        });
        
         /**
         *
         */
       function createOrder() {
            $scope.loadingMessage = 'Criando Pedido para pagamento com Link...';
            createOrderPayment();
            
            const voucherPaymentArray = CartService.cart.salePayments.filter(p => p.type === 'SaleVoucherGivexPayment');
            if (voucherPaymentArray.length > 0) {
                let customer = PaymentLinkService.getCustomer();
                voucherPaymentArray.forEach(voucherPayment => {
                    voucherPayment.customer = customer;
                });
            }
            var order = OrderService.cartToOrderConverter(CartService.cart);
            order.status = OrderService.orderStatus.PAYMENT;
            
            for (var i = 0; i < order.fiscalItems.length; i++) {
                var fiscalItem = order.fiscalItems[i];

                if (!fiscalItem.sequenceId) {
                    fiscalItem.sequenceId = i + 1;
                }
                fiscalItem.deliveryOption = $rootScope.createDANFE && !ConfigService.config.business.ignore_mid_settings ? 'SOLD_IN_STORE' : 'PICK_UP_FROM_WAREHOUSE';
                fiscalItem.totalValue = Math.round(fiscalItem.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
            }
            order.totalValue =  Math.round(order.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
            
            // precisamos confirmar os pagamentos pendentes antes de criarmos o pedido
            return PaymentService.confirmPayments()
            .then(() => OrderService.create(order))
            .then(result => serverOrder = result)
            .then(result => {
                $scope.loadingMessage = `Pedido '${result.orderNumber}' criado! Aguarde...`;
                return result;
            });
        }
        
        /**
         *
         */
        function checkout() {
            const checkoutRequest = {
                companyNumber:        '00001',
                storeNumber:          ConfigService.config.storeNumber,
                operatorName:         UserService.operatorName() ? UserService.operatorCode() : 'Não informado',
                operatorCode:         UserService.operatorCode() ? UserService.operatorCode() : 'Não informado',
                terminalNumber:       ConfigService.config.terminalNumber,
                nsu:                  ConfigService.nsuControll.currentNSU(),
                totalValue:           paymentValue,
                customer:             PaymentLinkService.getCustomer(),
                fiscalItems:          CartService.cart.fiscalItems,
                orderServerOrderNumber: serverOrder.orderNumber
            };
            return PaymentLinkService.checkout(checkoutRequest);
        }
        
        /**
         *
         */
        async function handleLinkCreation(data) {
            $scope.paymentLink.nit = data.data.orderNumber;
            $scope.paymentLink.url = data.data.accessUrl;
            $scope.paymentLink.message = data.data.message;
        }
        
        /**
         *
         */
        function updateOrderPayment() {
            let payment = serverOrder.orderPayments.find(sp => sp.paymentCodeMethod === 'PAYMENT_LINK');
            payment.paymentAuthorizationNSU = $scope.paymentLink.nit; 
            
            LogService.info(`Atualizando pagamento de pedido com NIT do link criado: ${$scope.paymentLink.nit}`);
            
            return OrderService.update(serverOrder)
                .then(order => LogService.info(`Pedido "${order.orderNumber}" atualizado com NIT: ${$scope.paymentLink.nit}.`));
        }
        
        /**
         *
         */
        async function cancelOrder() {
            if (serverOrder) {
                LogService.info(`Solicitando cancelamento do Pedido '${serverOrder.orderNumber}'...`);
                serverOrder.status = OrderService.orderStatus.CANCEL_REQUESTED;
                
                return OrderService.updateAsync(serverOrder)
                    .then(order => serverOrder = order)
                    .then(order => LogService.info(`Pedido com link de Pagamento de número "${order.orderNumber}" foi Cancelado.`))
                    .catch(LogService.error);
            }
        }
        
        /**
         *
         */
        function sendMessageToClient() {
            $scope.loadingMessage = 'Enviando link de pagamento para o cliente, aguarde...';
            
            return MessagingService.sendMessageToClient(PaymentLinkService.getCustomer(), $scope.paymentLink.message, 'Link de Pagamento')
            .catch(() => {
                AlertService.warn('Não foi possível enviar dados para o cliente. Você ainda pode copiá-lo manualmente ao clicar no botão.');
            });
        }
        
        /**
         *
         */
        function goBack() {
            LogService.info(`Voltando da criação de Link...`);
            
            return cancelOrder()
            .finally(() => PaymentService.cancel(false))
            .finally(() => CartService.reloadNSU()) //Possibilita que o Link possa ser usado novamente como forma de pagamento.
            .finally(() => UtilService.navigateWithoutHistory('easyorder.paymentmethods'));
        }
        
        /**
         *
         */
        async function processDone() {
            LogService.info(`Pedido ${serverOrder.orderNumber} criado com link: ${$scope.paymentLink.url}`);
            $scope.loadingMessage = `Tudo pronto! Pedido n° ${serverOrder.orderNumber}.`;
            
            $scope.countDown = ConfigService.config.business.gui_button_choice_timeout;
            
            $scope.countDonwPromise = $interval(() => { 
                $scope.countDown--; 
            }, 1000, $scope.countDown);
            
            $scope.countDonwPromise.then($scope.startNewSale);
        }
        
        /**
         *
         */
        $scope.copyToClipboard = function () {
            var copyElement = angular.element('<textarea/>');
            
            copyElement.css({
                position: 'fixed',
                opacity: '0'
            });
            copyElement.val($scope.paymentLink.url);

            var body = angular.element($window.document.body);
            body.append(copyElement);

            copyElement[0].select();
            
            if (document.execCommand('copy')) {
                $scope.copySuccess = true;
            } else {
                AlertService.warn("Não foi possível copiar link!");
            }
            copyElement.remove();
        }
        
        /**
         *
         */
        function createOrderPayment() {
            CartService.cart.salePayments.push({
                type: 'SaleGenericPayment',
                paymentValue: paymentValue,
                paymentSequence: PaymentService.payments().length,
                paymentCodeMethod: 'PAYMENT_LINK',
                installments: [],
                mandatoryMethod: true
            });
            PaymentService.currentPaymentDone();
        }
        
       /**
        *
        */
       $scope.startNewSale = () => {
            $interval.cancel($scope.countDonwPromise);
            CartService.initCart();
            UtilService.navigateToMainScreen();
        }
        
        $scope.$on('$destroy', () => {
             delete $rootScope.back;
        });
        
        /**
         *
         */
        $rootScope.back = () => {
            if (!$scope.completed) {
                AlertService.confirm("O link de pagamento será cancelado e os pagamentos desfeitos. Deseja continuar?")
                .then(accepted => {
                    if (accepted) {
                        goBack();
                    }
                });
            } else {
                AlertService.info("A operação já foi finalizada.");
            }
        }
    });
})();