(function() {
    'use strict';

    angular.module('easyorder.controllers.predefinedpaymentmethods', [
        'ln.currency',
        'easyorder.services.payment',
        'easyorder.services.alert'
    ])

    .controller('PreDefinedPaymentMethodsCtrl', function($scope, 
                                                       $ionicLoading,
                                                       $q,
                                                       $timeout,
                                                       CartService,
                                                       PaymentService,
                                                       AlertService,
                                                       UtilService,
													   LogService) {
        
        $scope.predefinedPaymentMethods = [];
        
        /**
         * 
         */
        $scope.undoCurrentPayments = function() {
            AlertService.confirm("Todos os pagamentos serão cancelados, deseja continuar?").then (
                function(success) {
                    if (success) {
                        PaymentService.cancel(true).then(function() {
                            UtilService.navigateWithoutHistory('easyorder.cart');
                        });
                    }
                });
        };
        
        /**
         * 
         */
        $scope.isInParcialPayment = function() {
            return PaymentService.isInParcialPayment();
        };
        
        /**
         * 
         */
        $scope.genericPayment = function(entry) {
            var paymentMethod = entry.method;
            var predefinedDetails = entry.predefinedDetails;
            
            PaymentService.initPayment(paymentMethod.id, predefinedDetails.paymentValue, paymentMethod.privateLabel);
            PaymentService.currentPayment().plan = predefinedDetails.plan;
            PaymentService.currentPayment().predefinedDetails = predefinedDetails; //Propriedade removida na confirmação do pagamento (PaymentService.currentPaymentDone()).
            
            return UtilService.navigateWithoutHistory('easyorder.genericpayment');
        };

        /**
         * Inicia processamento de pagamento TEF.
         */
        $scope.tefPayment = function(entry) {
            var paymentMethod = entry.method;
            var predefinedDetails = entry.predefinedDetails;
            
            PaymentService.initPayment(paymentMethod.id, predefinedDetails.paymentValue, paymentMethod.privateLabel);
            PaymentService.currentPayment().plan = predefinedDetails.plan;
            PaymentService.currentPayment().predefinedDetails = predefinedDetails; //Propriedade removida na confirmação do pagamento (PaymentService.currentPaymentDone()).
            
            return UtilService.navigateWithoutHistory('easyorder.tefpayment');
        };

		/**
		 *
		 */
		$scope.paymentLink = (entry) => {
            var predefinedDetails = entry.predefinedDetails;
			LogService.info(`Executando pagamento predefinido de Link de Pagamento: ${predefinedDetails.paymentValue}`);
            
            PaymentService.initPayment(entry.method.id, predefinedDetails.paymentValue, false);
			
			CartService.cart.salePayments.push({
                type: 'SaleGenericPayment',
                paymentValue: predefinedDetails.paymentValue,
                paymentSequence: PaymentService.payments().length,
                paymentCodeMethod: 'PAYMENT_LINK',
                installments: [],
				paymentAuthorizationCode: predefinedDetails.paymentAuthorizationCode,
				paymentAuthorizationNSU: predefinedDetails.paymentAuthorizationNSU,
				paymentAuthorizerNSU: predefinedDetails.paymentAuthorizerNSU,
				paymentAuthorizerBrandName: predefinedDetails.paymentAuthorizerBrandName,
				customerPaymentVoucher: predefinedDetails.customerPaymentVoucher,
				bussinessPaymentVoucher: predefinedDetails.bussinessPaymentVoucher,
				cardNumber: predefinedDetails.cardNumber,
				acquirerName: predefinedDetails.acquirerName
            });
            PaymentService.currentPaymentDone();

            $ionicLoading.show({
                duration: 0,
            }).then(() => {
				 UtilService.navigateAfterReceivedPayment();
            });
		};
        
       /**
        *
        */
        $scope.processPaymentMethods = function() {
            var deferred = $q.defer();
            
            $timeout(function() {
                var chosenMethods = CartService.predefinedPaymentMethodsDetails.concat([]);
                var ret = [];
                
                if (chosenMethods) {
                    while (chosenMethods.length) {
                        var entry = { predefinedDetails: chosenMethods.shift() }; //Remove o primeiro e retorna.

						const method = allMethods.find(method => method.id === entry.predefinedDetails.paymentMethod);
						
						if (method) {
							entry.method = method;
							let description = entry.method.description;
                                
                            if (entry.predefinedDetails.paymentAuthorizerBrandName && entry.predefinedDetails.paymentAuthorizerBrandName.length > 0) {
                                description = description + " (" + entry.predefinedDetails.paymentAuthorizerBrandName + ")"; 
                            }
                            entry.method.description = description;
                            
                            if (CartService.processedPredefinedPayments.includes(entry.predefinedDetails)) {
                                entry.processed = true;
                            }
							ret.push(entry);
                        } else {
                            deferred.reject("Forma de pagamento não suportada. [" + entry.predefinedDetails.paymentMethod + "]");
                            break;
                        }
                    }
                }
                deferred.resolve(ret);
            });
            return deferred.promise;
        };
        
        /**
         * Tipos de identificação disponíveis.
         */
        var allMethods = [{
            id: "CREDIT",
            handler: $scope.tefPayment,
            icon: "icon fa fa-credit-card fa-fw",
            description: "Crédito",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }, {
            id: "CREDIT_WITH_INSTALMENTS",
            handler: $scope.tefPayment,
            icon: "icon fa fa-credit-card fa-fw",
            description: "Crédito Parcelado Loja",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }, {
            id: "DEBIT",
            handler: $scope.tefPayment,
            icon: "icon fa fa-credit-card fa-fw",
            description: "Débito",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }, {
            id: "VIRTUAL_CARD",
            handler: $scope.tefPayment,
            icon: "icon fas fa-clone fa-fw",
            description: "Cartão Virtual",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }, {
            id: "CDC_DIGITAL_PAYMENT_BOOK",
            handler: $scope.genericPayment,
            icon: "icon fas fa-list-ol fa-fw",
            description: "CDC Digital",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }, {
            id: "PAYMENT_LINK",
            handler: $scope.paymentLink,
            icon: "icon fa fas fa-link fa-fw",
            description: "Link de Pagamento",
            privateLabel: false,
            disabled: (function() {
                return false;
            })()
        }];
        
        (function() {
            $ionicLoading.hide();
            
            $scope.totalPayments = PaymentService.totalPayments();
            
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(function() {
                $scope.processPaymentMethods().then(function(predefinedPaymentEntries) {
                    $ionicLoading.hide();
                    $scope.predefinedPaymentEntries = predefinedPaymentEntries;
                }, function(msg) {
                    $ionicLoading.hide();
                    AlertService.error(msg);
                    UtilService.navigateWithoutHistory('easyorder.cart');
                });
            });
        })();
    });
})();
