(function() {
    'use strict';

    angular.module('easyorder.controllers.privatelabelbiopayment', [
        'easyorder.services.cart',
        'easyorder.services.nfce',
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.services.insurance',
        'easyorder.services.log',
        'easyorder.services.privatelabel'
    ])

    .controller('PrivateLabelPaymentV1Ctrl', function($q,
                                        $rootScope,
                                        $scope,
                                        $state,
                                        $timeout,
                                        $ionicLoading,
                                        $ionicModal,
                                        CartService,
                                        ConfigService,
                                        UtilService,
                                        PaymentService,
                                        AlertService,
                                        InsuranceService,
                                        LogService,
                                        PrivateLabelService) {

        $scope.customer = CartService.cart.customer;
        
        $scope.goBack = () => {
            return $q.resolve()
            .then(checkUndoInsuranceAuth)
            .then(undoPaymentCampaigns)
            .then(CartService.undoFirtsPurchaseDiscount)
            .finally(() => UtilService.navigateWithoutHistory('easyorder.paymentmethods'));

        };
        $scope.goToPasswordScreen = () => {
            if (CartService.cart.customer.avaliablePaymentMethods[0] == 'CONDUCTOR_CREDIT_BOOK' || CartService.cart.customer.avaliablePaymentMethods[0] == 'CONDUCTOR_CARD_STATEMENT') {
                return UtilService.navigateWithoutHistory('easyorder.tefpayment');
            }else {
                return UtilService.navigateWithoutHistory('easyorder.privatelabelpaymentv1');
            }
        }
        async function undoPaymentCampaigns() {
            const campaigns = PaymentService.currentPayment.campaigns;
 
            if (campaigns && campaigns.length) {
                campaigns.forEach(campaign => {
                    LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                    CartService.undoCampaignDiscountsPartial(campaign);
                });
            }
        };
 
        function checkUndoInsuranceAuth() {
            if (InsuranceService.currentSimulation && CartService.cart.insuranceData.authorizationNumber) {
                LogService.info(`Desfazendo autorização de Seguro: ${CartService.cart.insuranceData.authorizationNumber}`);
                
                InsuranceService.undoInsuranceAuth(CartService.cart).finally(() => {
                    InsuranceService.clearSimutaionData();
                    CartService.cart.insuranceData = {};
                });
            }
        };

        /**
         * Biometria Facial
         */
        $scope.closeModal = function () {
            return $scope.modal.hide();
        };

        $scope.closeAndCall = function (action) {
            $scope.closeModal().then(action);
        };

        $scope.$on('$destroy', function () {
            $scope.modal.remove();
        });

        $scope.loadModel = function (name) {
            var filePath = "file:///android_asset/www/unico/models/" + name;
            return $q((resolve) => {
                window.resolveLocalFileSystemURL(
                    filePath,
                    function (fileEntry) {
                        fileEntry.file(
                            function (file) {
                                var reader = new FileReader();

                                reader.onloadend = function () {
                                    resolve(this.result);
                                };

                                reader.readAsArrayBuffer(file);
                            },
                            function () {
                                resolve(null);
                            }
                        );
                    },
                    function () {
                        resolve(null);
                    }
                );
            })
        }

        function onCapture(result) {
            $q.resolve()
            .then(() => $scope.encryptPhoto(result))
            .then($scope.authPrivateLabelPayment)
            .catch((err) => {
                LogService.error(err);
                $ionicLoading.hide()
                .then(() => {
                    feedbackHandler(null, 'Falha no processo de autorização', false);
                });
            });
        }

        function onCaptureError(error) {
            let message = (error && error.message) ? error.message : "Erro na captura da face";
            message = message + ", deseja tentar novamente?";
            return AlertService.confirm(message)
            .then((result) => {
                if (result) {
                    $scope.initCamera();
                } else {
                    $scope.goBack();
                }
            });
        }

        $scope.initCamera = () => {
            var configurations = {
                TYPE: 2,
                optional: {
                    FACE_MODE: 1,
                    LABEL_DOCUMENT_TYPE_OTHERS: "doc",
                },
                cameraType: {
                    code: 2
                }
            }

            var layout = {
                silhouette: {
                    primaryColor: "#0bbd26",
                    secondaryColor: "#bd0b0b",
                    neutralColor: "#fff",
                },
                buttonCapture: {
                    backgroundColor: "#2980ff",
                    iconColor: "#fff",
                },
                popupLoadingHtml: `<div style="position: absolute; top: 45%; right: 50%; transform: translate(50%, -50%); z-index: 10; text-align: center;">Loading...</div>`,
                boxMessage: {
                    backgroundColor: "#2980ff",
                    fontColor: "#fff"
                },
                boxDocument: {
                    backgroundColor: "#2980ff",
                    fontColor: "#fff"
                }
            }

            var callback = {
                on: {
                    success: function (result) {
                        onCapture(result);
                    },
                    error: function (error) {
                        onCaptureError(error);
                    },
                    support: function (e) {
                        console.log(e)
                    },
                    cameraLoaded: function (e) {
                        $ionicLoading.hide();
                    }
                }
            }

            return $ionicLoading.show({
                template: 'Iniciando camera...',
                duration: 0
            })
            .then(() => $scope.camera.initCamera(configurations, callback, layout));
        }

        $scope.prepareCamera = async () => {
            $scope.camera = new unico.UnicoCamera();

            let saveModelPromise = null;
            if ($rootScope.isCordova) {
                if (ionic.Platform.isAndroid()) {
                    let modelBuffers = {};
                    let promises = [];
                    let models = ['tiny_face_detector_model-shard1.json', 'face_landmark_68_model-shard1.json','tiny_face_detector_model-weights_manifest.json' ,'face_landmark_68_model-weights_manifest.json' ]
                    models.forEach(modelName => {
                        promises.push($scope.loadModel(modelName).then((buffer) => { modelBuffers[modelName] = buffer } ))
                    });

                    saveModelPromise = await $q.all(promises)
                    .then(() => $scope.camera.savePreLoadedModels(modelBuffers))
                } else {
                    saveModelPromise = AlertService.error('Sistema operacional não comporta carregamento dos arquivos da camera')
                    .then($q.reject);
                }
            } else {
                saveModelPromise = $q.resolve();
            }
            
            return $ionicLoading.show({
                template: 'Configurando camera...',
                duration: 0
            })
            .then(() => saveModelPromise)
            .then(() => $scope.camera.loadModels("unico/models"))
            .then(() => $scope.initCamera())
            .catch(() => {
                $ionicLoading.hide();
            });

        }

        $scope.encryptPhoto = (captureData) => {
            let keyInfo = JSON.parse(sessionStorage.getItem('sessionCryptoKey'));
                
            const base64Image = captureData.image.replace("data:image/jpeg;base64,", "");
            const covertedKey = CryptoJS.enc.Base64.parse(keyInfo.value);
            const encrypted = CryptoJS.AES.encrypt(base64Image, covertedKey, {
                mode: CryptoJS.mode.ECB,
                padding: CryptoJS.pad.Pkcs7,
            });

            return { value: encrypted.ciphertext.toString(CryptoJS.enc.Base64) }
        }

        $scope.authPrivateLabelPayment = function(data) {

            let reqParams                               = {};
            reqParams.cardNumber                        = CartService.cart.customer.cardNumber;
            reqParams.documentValue                     = CartService.cart.customer.document;
            let authProperty                            = ($scope.isFaceBioPayment) ? 'photo' : 'password';
            reqParams[authProperty]                     = data.value;

            let currentPayment                          = PaymentService.currentPayment();
            if (currentPayment.plan) {
                reqParams.installmentsNumber            = currentPayment.plan.instalments.length;
                reqParams.firstInstallmentDate          = currentPayment.plan.firstInstalment.dueDate
                reqParams.paymentMethod                 = currentPayment.paymentMethod;
                reqParams.paymentValue                  = MathUtils.round(currentPayment.paymentValue);
                if (currentPayment.plan.discount) {
                    LogService.info('Valor do desconto: [' + currentPayment.plan.discount + "]");
                    reqParams.paymentValue              = MathUtils.subtract(reqParams.paymentValue, currentPayment.plan.discount);
                    LogService.info('Valor ajustado: [' + reqParams.paymentValue + "]");
                }
                LogService.info('Juros: [' + currentPayment.plan.charges + "]");
                reqParams.taxIncluded                   = (currentPayment.plan.charges ? true : false);
                const containsInsurance                 = currentPayment.plan.instalments.some(i => i.insuranceValue > 0);
                LogService.info('Seguro: [' + containsInsurance ? ' ' : 'NÃO ' + "Contém Seguro]");
                reqParams.insuranceIncluded             = containsInsurance;
            }

            $scope.doAuth(reqParams, false);
        };

        function onAuthSucess(result) {
            switch (result.authStatus) {
                case 'AUTHORIZED':
                    $scope.paymentDone(result);
                    break;
                case 'UNSUCCESSFUL':
                    let message     = result.message; 
                    let canTryAgain = !(message && message.toLowerCase().indexOf('bloqueado') >= 0);
                    feedbackHandler(message, "Não foi possível autorizar pagamento.", canTryAgain, $scope.initCamera);
                    break;
                case 'PENDING_AUTHORIZATION':
                    $scope.paymentAuthId    = ($scope.paymentAuthId) ? $scope.paymentAuthId : result.paymentAuthorizationCode;
                    $scope.attemptNumber    = ($scope.attemptNumber) ? $scope.attemptNumber : 0;
                    $scope.maxTrials        = ($scope.maxTrials) ? $scope.maxTrials : 3;
                    reAttempAuth(result.message);
                    break;
                case 'ERROR':
                    feedbackHandler(result.message, "Serviço indisponível.", false);
                    break;
                default:
                    break;
            }
        }

        function onAuthError(error) {
            let errorMessage = (error && error.data && error.data.message) ? error.data.message : null;
            let errorCode = (error && error.data && error.data.code) ? error.data.code : null;
            if ($state.is('easyorder.privatelabelbiopayment') && errorCode == 500) {
                feedbackHandler(errorMessage, "Erro ao autorizar pagamento.", false, $scope.goToPasswordPayment);
            } else {
                feedbackHandler(errorMessage, "Erro ao autorizar pagamento.", false);
            }
        }

        $scope.doAuth = (data, isPending) => {
            $ionicLoading.show({
                template: 'Autorizando pagamento',
                duration: 0
            })
            .then(() => isPending ? PrivateLabelService.paymentStatus(data) : PrivateLabelService.authorizePayment(data))
            .then((result) => {
                $ionicLoading.hide().then(() => {
                    onAuthSucess(result);
                });
            })
            .catch((error) => {
                $ionicLoading.hide().then(() => {
                    onAuthError(error);
                });
            }); 
        }

        $scope.authorizePendingPayment = () => {
            $scope.doAuth($scope.paymentAuthId, true);
        }

        function feedbackHandler(message, defaultMessage, canTryAgain, successFunc, subtitle) {
            let popupMessage = $scope.errorMessage(message, defaultMessage, canTryAgain, subtitle);

            if (!canTryAgain) {
                AlertService.error(popupMessage)
                .then(() => {
                    if ($state.is('easyorder.privatelabelbiopayment') && successFunc) {
                        successFunc();
                    } else {
                        $scope.goBack();
                    }
                });
                return;
            }

            AlertService.confirm(popupMessage)
            .then((result) => {
                if (!result) {
                    $scope.goBack();
                    return;
                }
                successFunc();
            });
        }

        function reAttempAuth(message) {
            if ($scope.attemptNumber >= $scope.maxTrials) {
                feedbackHandler(null, 'Número máximo de tentativas esgotadas. O pagamento será desfeito', false);
                return;
            }

            $scope.attemptNumber += 1;
            let subtitle = "<br><br><b>Tentativa " + $scope.attemptNumber + "/" + $scope.maxTrials + "</b>";
            feedbackHandler(message, "Autorização pendente.", true, $scope.authorizePendingPayment, subtitle);
        }

        $scope.paymentDone = function(paymentResults) {
            $scope.description  = paymentResults.message || 'Pagamento autorizado';
            $scope.cssClass     = 'animation-popup';
            $scope.type         = 'success';

            AlertService.expirablePopup('views/animatedfeedback.html', $scope, 2500)
            .then(() => $ionicLoading.show())
            .then(() => {
                $scope.processPayment(paymentResults, CartService.cart, PaymentService.currentPayment());
                PaymentService.currentPaymentDone();
                UtilService.navigateAfterReceivedPayment();
            });
        };

        $scope.processPayment = function(paymentResults, cart, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SalePrivateLabelPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = parseFloat(new Number(currentPayment.paymentValue).toFixed(2));

            if (currentPayment.plan) {
                salePayment.paymentCodePlan             = currentPayment.plan.code;
                salePayment.paymentDescriptionPlan      = currentPayment.plan.description;
                salePayment.paymentCodeEvent            = currentPayment.plan.paymentCodeEvent;
                salePayment.paymentPlanExternalCode     = currentPayment.plan.externalCode;
                salePayment.paymentPlanProvider         = currentPayment.plan.provider;
                salePayment.numberOfInstallments        = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment       = currentPayment.plan.firstInstalment.dueDate;
                salePayment.firstInstalmentValue        = new Number(currentPayment.plan.firstInstalment.instalmentValue).toFixed(2);
                salePayment.installments                = [];
                salePayment.discountValue               = 0;   

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { 
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = new Number(instalment.instalmentValue).toFixed(2);
                    instalmentDetails.instalmentOriginalValue   = new Number(instalment.instalmentOriginalValue ? instalment.instalmentOriginalValue : 0).toFixed(2);
                    instalmentDetails.insuranceValue            = new Number(instalment.insuranceValue ? instalment.insuranceValue : 0).toFixed(2);

                    this.push(instalmentDetails);
                }, salePayment.installments)

                if (currentPayment.plan.discount) {
                    var discount = {}

                    discount.sequenceId         = cart.discounts.length;
                    discount.value              = currentPayment.plan.discount;
                    discount.source             = 'PLAN';

                    salePayment.paymentValue    -= currentPayment.plan.discount; 
                    salePayment.discountValue   += currentPayment.plan.discount;

                    salePayment.discountValue = new Number(salePayment.discountValue).toFixed(2);
                    
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));
                    cart.discounts.push(discount);
                }
                if (currentPayment.plan.charges) {
                    cart.additionalCharge       += currentPayment.plan.charges;
                    salePayment.chargeValue     = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }
                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            }

            salePayment.paymentValue                = Math.round(new Number(salePayment.paymentValue) * 100) / 100;
            salePayment.paymentAuthorizationNSU     = paymentResults.paymentAuthorizationNSU;
            salePayment.contractNumber              = paymentResults.paymentAuthorizationNSU;
            salePayment.cardNumber                  = cart.customer.customerId;
            salePayment.paymentAuthorizationCode    = paymentResults.paymentAuthorizationCode;
            
            salePayment.installments.sort(function(a,b) {
                return a.paymentSequence - b.paymentSequence;
            });

            const authProviders = ConfigService.config.business.payment_auth_providers;
            if (authProviders && authProviders[currentPayment.paymentMethod]) {
                salePayment.paymentProvider         = {id: authProviders[currentPayment.paymentMethod]};
            }

            if ($scope.isFaceBioPayment) {
                salePayment.authMethod              = "FACIAL_BIOMETRICS"; 
            }

            cart.salePayments.push(salePayment);
        };

        $scope.calculateDaysToPayment = (firstPaymentDate) => {
            return moment(firstPaymentDate).startOf('day').diff(moment().startOf('day'), 'days');
        }

        $scope.errorMessage = (message, defaultMessage, canTryAgain, subtitle) => {
            let errorMessage = (message) ? message : defaultMessage;
            if (canTryAgain) {
                errorMessage += (errorMessage.slice(-1) == ".") ? " Deseja tentar novamente?" : ", deseja tentar novamente?";
            }
            if (subtitle) {
                errorMessage += subtitle;
            } 
            return errorMessage;
        }

        $scope.initializeFaceBioPayment = () => {
            $ionicModal.fromTemplateUrl('views/privatelabelbiotipsmodal.html', {
                scope: $scope,
                focusFirstInput: true,
                animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.modal = modal;
                $scope.modal.show();
            });
        }

        /**
         * Senha
         */ 
        $scope.clearDisplay = function() {
            $scope.message = "";
            LogService.info("clearDisplay");
        };

        $scope.displayMessage = function(message) {
            if (message) {
                $scope.message = message;
            }
            LogService.info("displayMessage: " + $scope.message);
        };

        $scope.inputText = function(label, type, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1,
                label:              label,
                handler:            responseHandler,
                type:               type,
                maxSize:            fieldSizeRange.maxSize,
                minSize:            fieldSizeRange.minSize
            });
            $scope.headerOptions = "Informe";
        };

        $scope.tickHandler = function(func, data) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func(data);
        };

        $scope.paymentError = function () {
            return UtilService.navigateWithoutHistory('easyorder.paymentmethods')
        };

        $scope.initializePasswordPayment = function () {
            $scope.options = [];
            $scope.clearDisplay();
            $timeout(function() {
                $scope.displayMessage("Digite a senha para autorização da venda");
            }, 800);
            $scope.inputText("Senha do Cartão", $scope.platformNumberInputType, $scope.authPrivateLabelPayment, { minSize: 4, maxSize: 15 });
        };

        $scope.goToPasswordPayment = () => {
            CartService.cart.customer.faceBiometryStatus = "OFFLINE";
            $scope.isFaceBioPayment = false;
            $state = 'easyorder.privatelabelpaymentv1';
            return $q.resolve()
            .finally(() => UtilService.navigateWithoutHistory('easyorder.privatelabelpaymentv1'));
        };

        /**
         * Função de inicialização.
         */
        (function() {
            if (!(CartService.cart.customer.faceBiometryStatus == "INACTIVE" || CartService.cart.customer.faceBiometryStatus == "NO_DATA")){
                if ($state.is('easyorder.privatelabelpaymentv1')) {
                    $scope.initializePasswordPayment();
                } else {
                    $scope.isFaceBioPayment = true;
                    $scope.initializeFaceBioPayment();
                }
            } else {
                $scope.goToPasswordScreen();
            }
        })();
    });
})();
