(function() {
    'use strict';

    angular.module('easyorder.controllers.sitefQRPayment', [])

    .controller('SitefQRPaymentCtrl', function (
        $q,
        $timeout,
        $scope,
        $ionicLoading,
        CartService,
        PaymentService,
        UtilService,
        LogService,
        AlertService,
        EFTService,
        UserService,
        ConfigService,
        PaymentProcessor
    ) {
		/* 
         * caso o callback $scope.paymentdone for chamado não podemos permitir voltar
         */
        let done = false;
        let goingBack = false;

		/**
		 *
		 */
		$scope.goBack = () => {
            if ($scope.isBackButtonDisabled()) {
                return;
            }
            goingBack = true;
            
            $scope.message = "Cancelando operação...";
            
            EFTService.finalizeTransaction()
            .then(rewind)
            .catch(() => {
                $scope.message = "Não foi possível cancelar operação";
                goingBack = false
            });
        };
	
		/**
		 *	
		 */
		$scope.isBackButtonDisabled = () => {
            return done || goingBack;
        };

        /**
         *
         */
        $scope.isTransactionInitiated = () => {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $scope.$apply(function () {
                $scope.message = "";
                $scope.twoLineMessage = "";
            });
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            /*
             * plugin de tef não atualiza a mensagem caso seja requisitado
             * o cancelamento da operação, por isto bloqueamos outras mensagens
             */
            if (goingBack) {
                return;
            }
            $scope.$apply(function () {
                if (message) {
                    if (message.length <= 50) {
                        $scope.twoLineMessage = "";
                        $scope.message = message;
                    } else {
                        $scope.message = "";
                        $scope.twoLineMessage = message;
                    }
                }
            });
            LogService.info("displayMessage: " + $scope.message);
        };

        /**
         *
         */
        $scope.selectOption = function(title, options, responseHandler) {
            $scope.$apply(function () {
                $scope.message = title;
            });
            $scope.processOptions(options, responseHandler);
            LogService.info("Opção tef selecionada: ", title,  " Oções: ",  options);
        };

        /**
         *
         */
        $scope.selectOptionExtended = function(title, options, responseHandlerExtended) {
            $scope.$apply(function () {
                $scope.message = title;
            });
            $scope.processOptions(options, responseHandlerExtended);
            LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
        };

        /**
         *
         */
        $scope.numberInput = function(label, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               'number',
                maxSize:            fieldSizeRange.maxSize,
                valuePattern:       '[0-9]*'
            });
            $scope.headerOptions = "Informe";

            $scope.$apply();
        };

        /**
         *
         */
        $scope.stringInput = function(label, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               'string',
                maxSize:            fieldSizeRange.maxSize,
                valuePattern:       '[A-Za-z0-9_]*'
            });
            $scope.headerOptions = "Informe";

            $scope.$apply();
        };

        /**
         *
         */
        $scope.processOptions = function(options, responseHandler) {    //(1,"Magnetico/Chip")#(2,"Digitado")
            $scope.options.length = 0;
            $scope.showWatermark = false;

            var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
            splitedOpt = splitedOpt.split("#"); // Separando as opções.

            for (var i = 0; i < splitedOpt.length; i++) {
                var split = splitedOpt[i].split(",");

                $scope.options.push({
                    id:         split[0].trim(),
                    label:      split[1].trim().replace(/\"/g, ""),
                    value:      split[split.length-1],
                    handler:    responseHandler,
                    type:        "single-option"
                });
                $scope.$apply();
            }
            $scope.headerOptions = "Opções";
        };

        /**
         * Função executa a função com os parâmetros passados a esta.
         */
        $scope.tickOptionHandler = function(func, params) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func.apply(this, params);
        };

        /**
         *
         */
        $scope.tickNumberInputHandler = function(inputOption) {
            var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);

            $scope.tickOptionHandler(inputOption.handler, [value]);
        };

        /**
         *
         */
        $scope.tickStringInputHandler = function(inputOption) {
            const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);

            $scope.tickOptionHandler(inputOption.handler, [value]);
        };

        /**
         * Função para tratamento após finalização do pagamento TEF Chama autorização de NFC-e
         */
        $scope.paymentDone = function(paymentResults) {
            done = true;

            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(() => {
                $scope.processSitefQRPayment(paymentResults).then(salePayment => {
                    var currentPayment = PaymentService.currentPayment();
                    currentPayment.paymentValue = salePayment.paymentValue;
                    
                    let processorPayment = JSON.parse(JSON.stringify(salePayment));
                    processorPayment.type = SalePaymentTypes.SALE_TEF_PAYMENT;
                    
                    PaymentProcessor.pushSalePayment(processorPayment);
                    PaymentService.currentPaymentDone();
                    UtilService.navigateAfterReceivedPayment();
                }, $scope.paymentErrorUndoPayments);
            });
        };
        
        /**
         * Tratador de erro que desfaz pagamentos.
         */
        $scope.paymentErrorUndoPayments = function(error) {
            PaymentService.cancel(true).then(() => {
                AlertService.restError(error, "Não foi possível processar pagamento TEF").then(() => {
                    UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                });
            });
        };
        
       /**
        * Função chamada por callback do TEF.
        */
        $scope.paymentError = errorMsg => {
            switch (errorMsg) {
                case window.plugins.lio.PMNT_CANCELLED:
                    AlertService.error('Transação cancelada')
                        .then(rewind);
                    break;
                default:
                    AlertService.error(errorMsg)
                        .then(rewind);
                    break;
            }
        };
		
	   /**
        * Processa o valor a ser pago pelo cliente.
        */
        $scope.processPaymentValue = function() {
            var currentPayment = PaymentService.currentPayment();
           	var ret = new Number(currentPayment.paymentValue);
           	var paymentPlan = currentPayment.plan;
	
	        LogService.info("Processando valor do Pagamento: [" + ret + "]");

    	    if (paymentPlan) {
                ret = MathUtils.subtract(
                    paymentPlan.totalValue,
                    paymentPlan.downPaymentValue
                );
            }
            LogService.info("Valor do pagamento: [" + ret + "]");
            return ret;
        };
        
        /**
         *
         */
        $scope.exibirQRCode = (qrCodeText) => {
            $scope.showWatermark = false;
            
            $timeout(() => {
                LogService.info(`QR Code retornado: ${qrCodeText}`);
                $scope.qrCode = qrCodeText;
            });
        };
        
        /**
         *
         */
        $scope.removerQRCode = () => {
           $scope.qrCode = undefined;
           $timeout(() => $scope.showWatermark = true);
        };
        
		function rewind() {
            return $q.resolve()
                .then(CartService.undoCashbackDiscountPartial)
                .then(CartService.undoFirtsPurchaseDiscount)
                .then(undoPaymentCampaigns)
                .finally(() => UtilService.navigateWithoutHistory('easyorder.paymentmethods'));
        }

        /**
         *
         */
		async function undoPaymentCampaigns() {
            const campaigns = PaymentService.currentPayment().campaigns;

            if (campaigns && campaigns.length) {
                campaigns.forEach(campaign => {
                    LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                    CartService.undoCampaignDiscountsPartial(campaign);
                });
            }
        };
        
        /**
         *
         */
        $scope.processSitefQRPayment = async (tefPaymentResults) => {
            var salePayment = {};
            var currentPayment = PaymentService.currentPayment();

            salePayment.type                        = SalePaymentTypes.SALE_E_WALLET_PAYMENT;
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = currentPayment.paymentValue;
            salePayment.installments                = [];

            if (currentPayment.plan) {
                salePayment.paymentCodePlan         = currentPayment.plan.code;
                salePayment.paymentCodeEvent        = currentPayment.plan.paymentCodeEvent;
                salePayment.paymentPlanExternalCode = currentPayment.plan.externalCode;
                salePayment.paymentPlanProvider     = currentPayment.plan.provider;
                salePayment.numberOfInstallments    = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment   = currentPayment.plan.firstInstalment.dueDate;

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { // Convertendo Installment em InstallmentsDetails
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = instalment.instalmentValue;

                    this.push(instalmentDetails);
                }, salePayment.installments);

                if (currentPayment.plan.discount) {
                    var discount = {}

                    discount.sequenceId     = cart.discounts.length;
                    discount.value          = currentPayment.plan.discount;
                    discount.source         = 'PLAN';

                    /* O desconto deve ser contabilizado no valor do pagamento enviado para o WS. */
                    salePayment.paymentValue -= currentPayment.plan.discount; 
                    // atualiza valor do pagamento
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));

                    cart.discounts.push(discount);
                }

                if (currentPayment.plan.charges) {
                    cart.additionalCharge += currentPayment.plan.charges;
                    salePayment.chargeValue = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }
                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            } else {
                salePayment.numberOfInstallments = tefPaymentResults.NumeroParcelas;
            }
            salePayment.paymentAuthorizerBrandType  = tefPaymentResults.IndiceCodigoBandeira;

            if (tefPaymentResults.NomeBandeiraCartao) {
                salePayment.paymentAuthorizerBrandName  = tefPaymentResults.NomeBandeiraCartao.trim();
            }

            if (ConfigService.device.lio) {
                salePayment.paymentAuthorizerCode = 0;
            } else {
                const tefProvider = ConfigService.config && 
                                    ConfigService.config.tefserver ? ConfigService.config.tefserver.provider : undefined;
                switch(tefProvider) {
                    case 'Auttar':
                        salePayment.paymentAuthorizerCode = tefPaymentResults.CodigoAutorizadora;
                        salePayment.vanCode               = tefPaymentResults.CodigoVAN;
                        break;
                    case 'Linx':
                    case 'SoftwareExpress':
                        salePayment.paymentAuthorizerCode = tefPaymentResults.IndiceCodigoRede;

                        if (tefPaymentResults.IndiceCodigoRede) {
                            salePayment.vanCode           = '' + parseInt(tefPaymentResults.IndiceCodigoRede, 10);
                        }
                        break;
                    default:
                        throw new Error('TEF provider not implemented!');
                }
            }
            salePayment.paymentAuthorizationCode    = tefPaymentResults.CodigoAutorizacao;
            salePayment.paymentMethodIndicator      = tefPaymentResults.TipoFinanciamento;
            salePayment.paymentAuthorizationNSU     = tefPaymentResults.NSU;
            salePayment.paymentAuthorizerNSU        = tefPaymentResults.NSURede;
            salePayment.customerPaymentVoucher      = tefPaymentResults.ComprovanteCliente;
            salePayment.bussinessPaymentVoucher     = tefPaymentResults.ComprovanteEstabelecimento;
            salePayment.acquirerName                = tefPaymentResults.NomeAdquirente;
            
            salePayment.walletID                    = tefPaymentResults.IdCarteiraDigital;
            salePayment.walletName                  = tefPaymentResults.NomeCarteiraDigital;
            
            if (CartService.cart.customer != null) {
                salePayment.walletPaymentDocument          = CartService.cart.customer.document;
            }
            if (tefPaymentResults.TipoProduto === '01') {
                salePayment.walletProductType       = 'CREDIT';
            } else if (tefPaymentResults.TipoProduto === '02') {
                salePayment.walletProductType       = 'DEBIT';
            }

            CartService.cart.salePayments.push(salePayment);
            
            return salePayment;
        }
		
        /**
         * Função de inicialização.
         */
        (scopeObject => {
            $scope.options = [];
            $scope.nfceAuthResponse = {};
            $scope.transactionInitiated = true;
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.twoLineMessage = "";
            
            var plan = PaymentService.currentPayment().plan;
            var params = {
                transactionType:            EFTService.transactionType.SITEF_QR,
                operatorCode:               UserService.operatorCode(),
                instalmentsNumber:          plan ? (plan.instalments ? plan.instalments.length : 1) : 1,
                amount:                     $scope.processPaymentValue(),
                hasInsurance:               false,
                charges:                    plan ? plan.charges : undefined,
                processorCode:              plan ? plan.processorCode : undefined,
                interestType:               plan ? plan.interestType : undefined,
                returnQRCodeString:         true,
                customerDocument:           CartService.cart.customer ? CartService.cart.customer.document : undefined,
                listNotAcceptedVouchers:    ConfigService.config.business.sitef_qr_not_accepted_vouchers || ["01", "02", "03", "04", "05", "06", "07", "99"]
            };
			EFTService.executeTransaction(params, scopeObject);
			
        })($scope);
    });
})();
