(function() {
    'use strict';

    angular.module('easyorder.controllers.paymentmethods', [
        'ln.currency',
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.controllers.covenantpassword',
        'easyorder.services.cashback',
        'easyorder.services.log'
    ])

    .controller('PaymentMethodsCtrl', function($rootScope,
        $state,
        $scope,
        $q,
        $ionicLoading,
        $ionicModal,
        $ionicPopup,
        CartService,
        UtilService,
        PaymentService,
        ConfigService,
        CustomerService,
        AlertService,
        CovenantService,
        CashbackService,
        LogService,
        VoucherService,
        UserService,
        InstallmentPlanService) {

        $scope.isInParcialPayment = function() {
            return PaymentService.isInParcialPayment();
        };

        $scope.undoCurrentPayments = function() {
            var hasMobileRecharge = CartService.getCurrentServiceOperationType() === 'MOBILE_RECHARGE';
            var message = hasMobileRecharge ? "Todos os pagamentos, inclusive a recarga, serão cancelados, deseja continuar?" : "Todos os pagamentos serão cancelados, deseja continuar?";
            var cashPaymentCurrent = PaymentService.totalCashPayments();
            message = cashPaymentCurrent > 0 ? ("Você deseja finalizar esta operação e estornar todos os pagamentos? Total em dinheiro para devolução: R$ " + cashPaymentCurrent) : message;

            undoCurrentPayments(message);
        };

        function undoCurrentPayments(message) {
            AlertService.confirm(message).then(
                function(success) {
                    if (success) {
                        PaymentService.cancel(true).then(function() {
                            $state.go('easyorder.cart');
                        });
                    }
                }
            );
        }

        /**
         *
         */
        $scope.tefPayment = async function(paymentMethod, paymentValue, cashbackAddedInThisProcess) {
            var defer = $q.defer();
            let paymentState = checkPrivateLabelState(paymentMethod, 'easyorder.tefpayment');
            if (!paymentState) {
                return;
            }

            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            if ($rootScope.promotionEngine) {
                CartService.updateDiscountCart(paymentMethod, $scope.content.paymentValue, PaymentService.payments())
                    .then(() => {
                        // TODO --- VERIFICAR
                        if ($scope.content.paymentValue !== paymentValue) {
                            _initPayment(paymentMethod.id, $scope.content.paymentValue, paymentMethod.privateLabel);
                        } else {
                            _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
                        }
                    }, (err) => {
                        LogService.restError(err, err);
                        return $q.reject(err);
                    }).finally(() => defer.resolve());
            } else {
                if (paymentValue == CartService.cart.totalValue) {
                    var customer = CartService.cart.customer;
                    if (customer && customer.firstPurchaseDiscountPercentage > 0 &&
                        (paymentMethod.id == "CONDUCTOR_CREDIT_BOOK" || paymentMethod.id == "CONDUCTOR_CARD_STATEMENT") &&
                        !CartService.hasNonFiscalItems()) {
                        addFirstPurchaseDiscount(paymentValue, customer);
                    }
                    CartService.updateCartTotalValue();

                    _initPayment(paymentMethod.id, CartService.cart.totalValue, paymentMethod.privateLabel);
                    defer.resolve();
                } else {
                    _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
                    defer.resolve();
                }
            }

            defer.promise.then(() => {
                $ionicLoading.show({
                    template: 'Autorizando Pagamento...',
                    duration: 0
                }).then(function() {
                    PaymentService.tefPaymentAuthorization().then(() => {
                        if (paymentMethod.id === "CEA_PAY") {
                            PaymentService.currentPayment().provider = paymentMethod.id;
                            UtilService.navigateWithoutHistory('easyorder.cardnumbersearch');
                        } else {
                            if (ConfigService.config.business.payment_plan_enabled) {
                                navigateToState('easyorder.paymentplans', { paymentState: paymentState, cashbackAddedInThisProcess });
                            } else {
                                UtilService.navigateWithoutHistory('easyorder.tefpayment');
                            }
                        }
                        $ionicLoading.hide();
                    }, (error) => {
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Não foi possível autorizar pagamento TEF');
                        LogService.error('Não foi possível autorizar pagamento TEF: ', error);
                    });
                });
            });
        };

        /**
         * Método responsável pela inicialização de pagamento em dinheiro
         */
        $scope.cashPayment = function(paymentMethod, paymentValue) {

            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            $scope.cashPaymentCurrent = parseFloat(paymentValue) + parseFloat(PaymentService.totalCashPayments())
            _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
            $scope.totalPayments = (parseFloat($scope.totalPayments) + parseFloat(paymentValue)).toFixed(2);

            if ($scope.totalPayments >= CartService.cart.totalValue) {
                $scope.cashChange = (parseFloat($scope.cashPaymentCurrent) - parseFloat(CartService.cart.totalValue)); // TROCO
                CartService.cart.cashChange = (parseFloat($scope.cashChange.toFixed(2)));
                LogService.info('Total Pagamento em Dinheiro: [R$' + $scope.cashPaymentCurrent + ']');
                LogService.info('Troco em Dinheiro: [R$' + $scope.cashChange.toFixed(2) + ']');
                LogService.info('Finalizando operação de pagamento em Dinheiro');
            } else {
                $scope.paymentValue = (CartService.cart.totalValue - $scope.totalPayments).toFixed(2);
                var campo_moeda = document.getElementById('campo-moeda');
                campo_moeda.value = 'R$ ' + $scope.paymentValue.replace('.', ',');
            }
            UtilService.navigateWithoutHistory('easyorder.genericpayment');
        }

        /**
         * Pagamento com Cartão Próprio.
         */
        $scope.privateLabelPayment = function(paymentMethod, paymentValue, cashbackAddedInThisProcess) {
            var customer = CartService.cart.customer;
            let paymentState = checkPrivateLabelState(paymentMethod, 'easyorder.privatelabelpayment');
            if (!paymentState) {
                return;
            }

            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            if (CartService.cart.insuranceData) {
                CartService.cart.insuranceData.serverOffline = false;
            }

            if (paymentValue == CartService.cart.totalValue) {
                if (customer && customer.firstPurchaseDiscountPercentage > 0 && !CartService.hasNonFiscalItems()) {
                    addFirstPurchaseDiscount(paymentValue, customer);
                }
                CartService.updateCartTotalValue();
                _initPayment(paymentMethod.id, CartService.cart.totalValue, paymentMethod.privateLabel);
            } else {
                _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
            }

            if (ConfigService.config.business.payment_plan_enabled) {
                navigateToState('easyorder.paymentplans', { paymentState: paymentState, cashbackAddedInThisProcess });
            } else {
                UtilService.navigateWithoutHistory('easyorder.privatelabelpayment');
            }
        };

        /**
         * Pagamento do cartão presente.
         */
        $scope.giftCardPayment = function(paymentMethod, paymentValue, cashbackAddedInThisProcess) {

            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            var giftCardMandatoryCustomerIdentification = ConfigService.config.business.gift_card_mandatory_customer_identification;

            if(giftCardMandatoryCustomerIdentification && CartService.cart.customer == null) {
                AlertService.warn('Deve ser identificado o cliente')
                .then(() => {
                    if (!PaymentService.paymentInProgress()) {
                        if (ConfigService.config.business.unified_customer_identification_enabled) {
                           return UtilService.navigateWithoutHistory('easyorder.customer');
                        } else {
                            return UtilService.navigateWithoutHistory('easyorder.customermenu');
                        }
                    }
                });
                return;
            }

            _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
            navigateToState('easyorder.giftcardpaymentmenu', { cashbackAddedInThisProcess });

        };

        /**
         *
         */
        $scope.employeePayment = function(paymentMethod, paymentValue) {
            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            _initPayment(paymentMethod.id, paymentValue, false);

            autoSelectUniquePaymentEventTo(paymentMethod, 'easyorder.privatelabelpayment');
        };

        /**
         *
         */
        $scope.covenantPayment = function(paymentMethod, paymentValue) {
            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            let availableCreditValue = undefined;
            if (CovenantService.currentCovenant() && CovenantService.currentCovenant().covenantAuth) {
                availableCreditValue = new Number(CovenantService.currentCovenant().covenantAuth.availableCredit);
            }

            if (paymentValue > availableCreditValue) {
                paymentValue = availableCreditValue;
                AlertService.warn('O valor do pagamento será ajustado para o valor do crédito disponível. [R$ ' + availableCreditValue + ']')
            }
            _initPayment(paymentMethod.id, paymentValue, false);

            PaymentService.currentPayment().cardBin = CovenantService.currentCovenant().code;
            PaymentService.currentPayment().authorizerCode = 'AG';

            autoSelectUniquePaymentEventTo(paymentMethod, 'easyorder.covenantpayment');
        };

        $scope.showAssurants = (paymentMethod, paymentValue) => {
            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearch', {
                customer: CartService.cart.customer,
                vouchers: $scope.assurants,
                containsUsed: $scope.containsAssurantUsed,
                voucherType: 'VOUCHER_ASSURANT',
                paymentValue: paymentValue
            });
        };

        $scope.showStorexVouchers = (paymentMethod, paymentValue) => {
            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearch', {
                customer: CartService.cart.customer,
                vouchers: $scope.storexVouchers,
                containsUsed: $scope.containsUsedStorexVoucher,
                voucherType: 'VOUCHER_STOREX',
                paymentValue: paymentValue
            });
        }

        $scope.showGivexVouchers = (paymentMethod, paymentValue) => {
            const stateParams = {
                customer: CartService.cart.customer,
                paymentValue: paymentValue,
                voucherType: paymentMethod.id
            };
            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchoptions', stateParams);
        };

        $scope.showOmniVoucher = function(paymentMethod, paymentValue) {

            $scope.requestauth = {};

			// Verifica permissões
			var permission = UserService.checkPermission(EXCHANGE_OPERATION);
			LogService.info("Permissão: ", permission);

			if (!permission) {
				var authPopup = $ionicPopup.show({
					templateUrl: 'views/requestauth.html',
					scope: $scope,
					buttons: [
						{ text: 'Cancelar' },
						{
							text: '<b>Autorizar</b>',
							type: 'button-theme',
							onTap: function(event) {
								var sucesso = function(result) {
									authPopup.close();

									if (result.authorized) {
										const stateParams = {
                                            customer: CartService.cart.customer,
                                            paymentValue: paymentValue,
                                            voucherType: paymentMethod.id,
                                            isPasswordRequired: true,
                                        };
                                        UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchoptions', stateParams);
									} else {
										AlertService.error('Autorização negada!')
									}
								};

								var erro = function(error) {
									LogService.error(error);
									AlertService.restError(error, "Erro ao consultar autorização!");
								};

								if (!$scope.requestauth.user) {
									AlertService.error("Usuário inválido!");
								} else if (!$scope.requestauth.password) {
									AlertService.error("Senha inválida!");
								} else {
									UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [PRICE_CHANGE_PERMISSION], sucesso, erro);
								}
								event.preventDefault();
							}
						}
					]
				});
			} else {
			    $scope.requestauth.user = UserService.currentUser.username;
				const stateParams = {
                    customer: CartService.cart.customer,
                    paymentValue: paymentValue,
                    voucherType: paymentMethod.id,
                    isPasswordRequired: true,
                };
                UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchoptions', stateParams);
			}
        };

        $scope.showSafeVouchers = (paymentMethod, paymentValue) => {
            const stateParams = {
                customer: CartService.cart.customer,
                paymentValue: paymentValue,
                voucherType: paymentMethod.id
            };
            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchoptionsv3', stateParams);
        };

        function checkPrivateLabelState(paymentMethod, standardState) {
            let paymentState = standardState;

            const authProviders = ConfigService.config.business.payment_auth_providers;
            let isRealizePayment = authProviders && authProviders[paymentMethod.id] == "REALIZE_V1";
            if (isRealizePayment) {
                paymentState = 'easyorder.privatelabelpaymentv1';
            }

            const authMethods = ConfigService.config.business.payment_auth_methods;
            if (authMethods && authMethods[paymentMethod.id] == "FACE_DETECTION") {
                let customer = CartService.cart.customer;
                const authMethods = ConfigService.config.business.payment_auth_methods;
                let faceDetectionEnabled = authMethods && Object.entries(authMethods).some(item => item[1] && item[1] == "FACE_DETECTION");
                if (faceDetectionEnabled && customer.faceBiometryStatus && customer.faceBiometryStatus != "OFFLINE") {
                    if (customer.faceBiometryStatus == "INACTIVE" || customer.faceBiometryStatus == "NO_DATA") {
                        AlertService.warn('Cliente não possui biometria facial. Para pagamento com CCR ou Meu Cartão, direcione-o para cadastro!');
                    }
                    if (!isRealizePayment) {
                        AlertService.warn('Provedor de pagamento configurado não suporta autenticação por biometria facial');
                        return;
                    }
                    paymentState = 'easyorder.privatelabelbiopayment';
                }
                if (faceDetectionEnabled && customer.faceBiometryStatus && customer.faceBiometryStatus == "OFFLINE") {
                    AlertService.info('Serviço de reconhecimento facial indisponível. Operação será realizada através da senha do cliente.');
                }
            }

            return paymentState;
        }

        function checkVouchers() {
            if (CartService.cart.customer) {
                return $ionicLoading.show({
                        template: "Por favor, aguarde...",
                        duration: 0
                    }).then(() => searchVouchers('VOUCHER_STOREX'))
                    .catch(LogService.error)
                    .then(() => searchVouchers('VOUCHER_ASSURANT'))
                    .catch(LogService.error)
                    .then(() => searchGivexVouchers())
                    .catch(LogService.error)
                    .finally($ionicLoading.hide);
            } else {
                return $q.resolve();
            }
        }

        function searchGivexVouchers() {
            if (UtilService.previousStateData.name === 'easyorder.cart' &&
                ConfigService.config.business.enable_givex_voucher_redirection_request) {
                return searchVouchers('VOUCHER_GIVEX');
            } else {
                return $q.resolve();
            }
        }

        function searchVouchers(voucherType) {
            if (!ConfigService.config.business.available_payment_methods.includes(voucherType)) {
                return $q.resolve();
            }
            const voucherMethod = selectPaymentMethodFromDefList(voucherType);

            if (!voucherMethod || voucherMethod.disabled) {
                LogService.warn(`Método de pagamento não encontrado ou desabilitaddo: ${voucherType}`);
                return $q.resolve();
            }
            return VoucherService.searchByCustomer($scope.customer, ConfigService.config.storeNumber, ConfigService.config.companyNumber,
                ConfigService.config.terminalNumber, ConfigService.nsuControll.currentNSU(), voucherType).then(res => {

                CartService.cart.salePayments.filter(pmnt => pmnt.paymentCodeMethod == voucherType).forEach(pmnt => {
                    const usedVoucher = res.vouchers.find(voucher => voucher.number == pmnt.voucherNumber);

                    if (usedVoucher) {
                        usedVoucher.used = usedVoucher;
                    }
                });
                //verifica se ja foi utilizado algum voucher e marca como utilizado
                res.vouchers
                    .filter(voucher => voucher.status == 'USED' || voucher.status == 'PROCESSING')
                    .forEach(voucher => {
                        voucher.used = true;
                    });

                const containsUsedVoucher = res.vouchers.find(voucher => voucher.status == 'USED' || voucher.status == 'PROCESSING') != null ||
                    CartService.cart.salePayments.filter(pmnt => pmnt.paymentCodeMethod == voucherType).length > 0;
                const containsAvailableVoucher = res.vouchers.find(voucher => voucher.status == 'AVAILABLE') != null;

                switch (voucherType) {
                    case 'VOUCHER_ASSURANT':
                        $scope.assurants = res.vouchers;
                        $scope.containsAssurantUsed = containsUsedVoucher;
                        $scope.containsAssurantAvailable = containsAvailableVoucher;
                        break;
                    case 'VOUCHER_STOREX':
                        $scope.storexVouchers = res.vouchers;
                        $scope.containsUsedStorexVoucher = containsUsedVoucher;
                        $scope.containsAvailableStorexVoucher = containsAvailableVoucher;
                        break;
                    case 'VOUCHER_GIVEX':
                        $scope.givexVouchers = res.vouchers;
                        $scope.containsUsedStorexVoucher = containsUsedVoucher;
                        $scope.containsAvailableStorexVoucher = containsAvailableVoucher;
                        break;
                    default:
                        throw new Error(`VOUCHER_TYPE ${voucherType} NOT SUPPORTED`);
                }
            });
        }

        function doVouchersAutoRedirectionOption() {
            if ($scope.givexVouchers && $scope.givexVouchers.length) {
                AlertService.confirm("Deseja pagar com Vale Troca?").then(accepted => {
                    if (accepted) {
                        $scope.showGivexVouchers({ id: 'VOUCHER_GIVEX' }, $scope.content.paymentValue);
                    }
                });
            }
            return $q.resolve();
        }

        function voucherAssurantAvailable() {
            return CartService.cart.customer && $scope.containsAssurantAvailable;
        }

        function storexVoucherAvailable() {
            return CartService.cart.customer && $scope.containsAvailableStorexVoucher;
        }

        function paymentLink(paymentMethod, paymentValue) {
            const label = paymentMethod.description;
            const customer = CartService.cart.customer;

            // ainda não suportamos múltiplos pedidos na tela de link de pagamentos
            if (CartService.cart.mergedOrders) {
                return AlertService.warn(
                    `Não é possível pagamentos com a forma ${label} se múltiplos pedidos estiverem habilitados`
                );
            }

            // loops de processamento de link do lws ainda não suportam serviços
            if (CartService.hasServiceItems()) {
                return AlertService.warn(
                    `Não é possível pagamentos com a forma ${label} com itens de serviços`
                );
            }

            if (!CartService.cart.fiscalItems || CartService.cart.fiscalItems.length == 0) {
                return AlertService.warn(
                    `Não é possível pagamentos com a forma ${label} sem itens fiscais`
                );
            }

            if (parseFloat(PaymentService.totalPayments()) > 0) {
                return AlertService.warn(
                    `Não é possível pagamentos parciais com a ${label}`
                );
            }

            if (!customer) {
                return AlertService.warn(
                    `Cliente deve ser identificado para pagamentos com a forma ${label}`
                );
            }

            const requiredCustomerFields = [{
                value: () => customer.name,
                desc: 'o Nome'
            }, {
                value: () => customer.document,
                desc: 'o Documento'
            }, {
                value: () => customer.email,
                desc: 'o Email'
            }, {
                value: () => customer.address,
                desc: 'o Endereço'
            }, {
                value: () => customer.address.zipCode,
                desc: 'o CEP'
            }, {
                value: () => customer.address.phone,
                desc: 'o Telefone'
            }, {
                value: () => customer.address.addressName,
                desc: 'o Logradouro'
            }, {
                value: () => customer.address.addressNumber,
                desc: 'o Número do Endereço'
            }, {
                value: () => customer.address.neighborhood,
                desc: 'o Bairro'
            }, {
                value: () => customer.address.city,
                desc: 'a Cidade'
            }, {
                value: () => customer.address.state,
                desc: 'o Estado'
            }];

            for (const field of requiredCustomerFields) {
                if (!field.value()) {
                    return AlertService.warn(
                        `Atualizar ${field.desc} do Cliente para pagamentos com a forma ${label}`
                    );
                }
            }

            if(!CartService.cart.paymentLinkSettings.messageReceiverCustomer || !CartService.cart.paymentLinkSettings.messageReceiverCustomer.phoneNumber) {
                CartService.cart.paymentLinkSettings.messageReceiverCustomer = JSON.parse(JSON.stringify(CartService.cart.customer));
                CartService.cart.paymentLinkSettings.messageReceiverCustomer.phoneNumber =
                    CartService.cart.customer.address ? CartService.cart.customer.address.phone : undefined;
            }
            if (paymentValue < CartService.cart.totalValue) {
                AlertService.confirm("O Link de Pagamento utiliza o valor total da compra. Deseja continuar?").then(accepted => {
                    if (accepted) {
						_initPayment(paymentMethod.id, CartService.cart.totalValue, false);
                        if(ConfigService.config.business.paymentlink_contact_confirmation_enabled) {
                            return UtilService.navigate('easyorder.paymentlinkRecipientCustomerConfirmation');
                        } else {
                            return UtilService.navigate('easyorder.paymentlink');
                        }
                    }
                });
            } else {
                _initPayment(paymentMethod.id, paymentValue, false);
                if(ConfigService.config.business.paymentlink_contact_confirmation_enabled) {
                    return UtilService.navigate('easyorder.paymentlinkRecipientCustomerConfirmation');
                } else {
				    return UtilService.navigate('easyorder.paymentlink');
                }
            }
        }

        function qrLinxPayment(paymentMethod, paymentValue) {
            return UtilService.navigate('easyorder.e-wallet-payment', {
                paymentValue: paymentValue,
                paymentMethod: paymentMethod
            });
        }

        /**
         *
         */
        function sitefPaymentLink(paymentMethod, paymentValue) {
            if (MathUtils.subtract(CartService.cart.totalValue, $scope.totalPayments) !== MathUtils.round(paymentValue)) {
                AlertService.warn("Selecione outra forma de pagamento.\nOpção válida apenas p/ pagto total ou finalização da venda.")
                    .finally(() => content.paymentValue = $scope.content.paymentValue);
            } else if ($scope.hasNonVoucherOrGiftCardPaymentMethos()) {
                AlertService.warn("Selecione outra forma de pagamento.\nOpção válida apenas p/ pagto total ou como última forma de pagto selecionada.")
                .finally(() => content.paymentValue = $scope.content.paymentValue);
            } else if (!CartService.cart.customer) {
                AlertService.warn("Cliente deve ser identificado para esta forma de pagamento");
            } else {
                _initPayment("PAYMENT_LINK", paymentValue, false);
                return UtilService.navigate('easyorder.sitef-paymentlink');
            }
        }

        async function initInstallmentPlanPayment(paymentMethod, paymentValue, checkCashback) {
            const customer = CartService.cart.customer;

            if (!customer) {
                await AlertService.warn("Cliente deve ser identificado para esta forma de pagamento");
                return;
            }

            let availableCredit = customer.avaliableCredit;

            if (availableCredit == null) {
                await $ionicLoading.show({
                    template: 'Consultando limite...',
                    duration: 0
                });

                try {
                    const res = await InstallmentPlanService.searchAvailableCredit({
                        identificationType: customer.documentType,
                        identification: customer.document
                    });
                    availableCredit = res.data.availableCredit;
                    await $ionicLoading.hide();
                } catch(err) {
                    LogService.error(err);
                    await $ionicLoading.hide();
                    await AlertService.restError(err, "Falha ao consultar limite");
                    return;
                }
            }

            if (MathUtils.round(paymentValue) > MathUtils.round(availableCredit)) {
                await AlertService.warn("Cliente sem limite suficiente para esta operação, deve ser gerado limite no PDV");
                return;
            }

            _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
            navigateToState('easyorder.paymentplans', { paymentState: 'easyorder.installment-plan-payment', checkCashback });
        }

        /**
         *
         */
        var autoSelectUniquePaymentEventTo = function(paymentMethod, toState) {
            $ionicLoading.show({
                template: 'Por favor, aguarde...',
                duration: 0
            }).then(function() {
                PaymentService.search().then((result) => {
                    $ionicLoading.hide();

                    if (!result || !result.paymentPlanEvents || result.paymentPlanEvents.length === 0) {
                        AlertService.warn('Não existem eventos de pagamento para a forma escolhida.').finally(function() {
                            $scope.loadingState = false;
                            UtilService.navigateWithoutHistory('easyorder.cart');
                        });
                    } else if (result.paymentPlanEvents.length === 1 &&
                        result.paymentPlanEvents[0].paymentPlans.length === 1 &&
                        paymentMethod.autoSelectUniquePaymentEvent) {
                        var payment = PaymentService.currentPayment();
                        payment.plan = result.paymentPlanEvents[0].paymentPlans[0];
                        payment.plan.paymentCodeEvent = result.paymentPlanEvents[0].code;
                        payment.plan.processorCode = result.paymentPlanEvents[0].processorCode;

                        UtilService.navigateWithoutHistory(toState);
                    } else {
                        navigateToState('easyorder.paymentplans', { paymentState: toState });
                    }
                }, (err) => {
                    AlertService.restError(err, 'Não foi possível processar os planos!').finally(function() {
                        UtilService.navigateWithoutHistory('easyorder.cart');
                    });
                    LogService.error(err);
                    $ionicLoading.hide();
                });
            });
        };

		/**
         *
         */
        $scope.sitefqrPayment = async function(paymentMethod, paymentValue, cashbackAddedInThisProcess) {
            var defer = $q.defer();

            LogService.info(`Iniciando pagamento ${paymentMethod.id}: ${paymentValue}`);

            if ($rootScope.promotionEngine) {
                CartService.updateDiscountCart(paymentMethod, $scope.content.paymentValue, PaymentService.payments())
                    .then(() => {
                        if ($scope.content.paymentValue !== paymentValue) {
                            _initPayment(paymentMethod.id, $scope.content.paymentValue, paymentMethod.privateLabel);
                        } else {
                            _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
                        }
                    }).finally(defer.resolve);
            } else {
                _initPayment(paymentMethod.id, paymentValue, paymentMethod.privateLabel);
                defer.resolve();
            }

            defer.promise.then(() => {
                $ionicLoading.show({
                    template: 'Autorizando Pagamento...',
                    duration: 0
                }).then(() => {
                    PaymentService.tefPaymentAuthorization().then(() => {
                        if (ConfigService.config.business.payment_plan_enabled) {
                            navigateToState('easyorder.paymentplans', { paymentState: 'easyorder.sitefqrpayment', cashbackAddedInThisProcess });
                        } else {
                            UtilService.navigateWithoutHistory('easyorder.sitefqrpayment');
                        }
                    }, error => {
                        AlertService.restError(error, 'Não foi possível autorizar pagamento TEF');
                        LogService.error('Não foi possível autorizar pagamento TEF: ', error);
                    }).finally($ionicLoading.hide);
                });
            });
        };

        /**
         * Inicializa o pagamento. O valor do pagamento pode ser alterado pelo resultado do Motor de promoções
         * ou por um desconto atrelado ao usuário identificado no carrinho.
         */
        var _initPayment = function(paymentMethod, paymentValue, privateLabel) {
            let ret = paymentValue;
            let campaigns = [];

            if (!$scope.pendingDownPayment) {
                campaigns = CartService.processPaymentCampaigns(paymentMethod, paymentValue);
                CartService.updateCartTotalValue();
            }

            if (new Number(CartService.cart.totalValue) < new Number(paymentValue)) {
                if ($scope.pendingDownPayment) {
                    // isso não deve acontecer nunca, de toda forma
                    // estamos nos protegendo
                    throw new Error(`Valor da entrada ${paymentValue} maior que o total do carrinho ${CartService.cart.totalValue}`);
                }
                ret = CartService.cart.totalValue;
            }

            PaymentService.initPayment(paymentMethod, ret, privateLabel);
            PaymentService.currentPayment.campaigns = campaigns;

            return ret;
        };

        /**
         * Carrega campanhas de pagamento. Caso seja um processamento de pedido, as campanhas não são carregadas para os itens, mas
         * devem ser aplicadas nos pagamentos.
         */
        var loadOldPaymentCampaigns = () => {
            // não devemos processar descontos durante o pagamento de uma entrada
            if ($scope.pendingDownPayment) {
                return $q.resolve();
            }

            return $ionicLoading.show({
                template: "Consultando Campanhas. Por favor, aguarde...",
                duration: 0
            }).then(() => {
                if ($rootScope.campaignEnabled &&
                    (!CartService.cart.campaignsPayment || !CartService.cart.campaignsPayment.length)) {

                    var fiscalItems = CartService.cart.fiscalItems;

                    if (fiscalItems) {
                        return CartService.executeCampaigns()
                        .then(response => {
                            var campaigns = response.promotions;

                            if (campaigns && campaigns.length) {
                                var paymentCampaigns = campaigns.filter(c => c.campaignPayment);

                                if (paymentCampaigns.length) {
                                    if (!CartService.cart.campaignsPayment) {
                                        CartService.cart.campaignsPayment = [];
                                    }
                                    CartService.cart.campaignsPayment = [];
                                    CartService.cart.campaignsPayment.push(...paymentCampaigns);
                                } else {
                                    LogService.info('Nenhuma Campanha de Pagamento foi retornada pela consulta.')
                                }
                            } else {
                                LogService.info('Nenhuma Campanha de foi retornada pela consulta.')
                            }
                        })
                        .catch(error => LogService.warn('Não foi possível consultar Campanhas', error));
                    } else {
                        return Promise.resolve();
                    }
                } else if ($rootScope.promotionEngine) {
                    return CartService.processCampaigns(null, null, PaymentService.payments()).finally(function() {
                        CartService.applyEmployeeDiscount();
                    });
                } else {
                    return Promise.resolve();
                }
            }).finally(() => {
                updatePaymentValue();
                $ionicLoading.hide();
            });
        };

        /**
         * Tipos de identificação disponíveis.
         */
        var allMethods = [{
                id: "CREDIT",
                handler: $scope.tefPayment,
                icon: "icon fa fa-credit-card fa-fw",
                description: "Crédito",
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CREDIT") != null,
                disabled: (function() {
                    return false;
                })()
            }, {
                id: "CREDIT_WITH_INSTALMENTS",
                handler: $scope.tefPayment,
                icon: "icon fa fa-credit-card fa-fw",
                description: "Crédito Parcelado Loja",
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CREDIT_WITH_INSTALMENTS") != null,
                disabled: (function() {
                    return false;
                })()
            }, {
                id: "DEBIT",
                handler: $scope.tefPayment,
                icon: "icon fa fa-credit-card fa-fw",
                description: "Débito",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "DEBIT") != null,
                disabled: (function() {
                    return false;
                })()
            }, {
                id: "CASH",
                handler: $scope.cashPayment,
                icon: "icon fa fa-money-bill-alt fa-fw",
                description: "Dinheiro",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CASH") != null,
                disabled: (function() {
                    return false;
                })()
            }, {
                id: "TOTVSCARD_CREDIT_BOOK",
                handler: $scope.privateLabelPayment,
                icon: "icon fas fa-list-ol fa-fw",
                description: "Carnê Cliente",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "TOTVSCARD_CREDIT_BOOK") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "TOTVSCARD_CARD_STATEMENT",
                handler: $scope.privateLabelPayment,
                icon: "icon fas fa-file-invoice fa-fw",
                description: "Fatura Cliente",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "TOTVSCARD_CARD_STATEMENT") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "CONDUCTOR_CREDIT_BOOK",
                handler: $scope.tefPayment,
                icon: "icon fas fa-list-ol fa-fw",
                description: "Carnê Cliente (Meu Cartão)",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CONDUCTOR_CREDIT_BOOK") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "CONDUCTOR_CARD_STATEMENT",
                handler: $scope.tefPayment,
                icon: "icon fas fa-file-invoice fa-fw",
                description: "Fatura Cliente (Meu Cartão)",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CONDUCTOR_CARD_STATEMENT") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "LEGACY_CLIENT",
                handler: $scope.privateLabelPayment,
                icon: "icon fa fa-id-card-o fa-fw",
                description: "Cartão Cliente",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "LEGACY_CLIENT") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "GIFT_CARD",
                handler: $scope.giftCardPayment,
                icon: "icon fa fa-gift fa-fw",
                description: "Cartão Presente",
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "GIFT_CARD") != null,
                disabled: (function() {
                    return CartService.hasServiceItems();
                })()
            }, {
                id: "EMPLOYEEE_CARD",
                handler: $scope.employeePayment,
                icon: "icon fa fa-id-card fa-fw",
                description: "Associado",
                autoSelectUniquePaymentEvent: true,
                privateLabel: true,
                exclusive: true,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "EMPLOYEEE_CARD") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment() || !ConfigService.config.business.payment_plan_enabled);
                })()
            }, {
                id: "COVENANT",
                handler: $scope.covenantPayment,
                icon: "icon fa fa-handshake fa-fw",
                description: "Convênio",
                autoSelectUniquePaymentEvent: true,
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CONVENANT") != null,
                disabled: (function() {
                    return (!CovenantService.currentCovenant() || !CovenantService.currentCovenant().paymentCovenant || !CovenantService.currentCovenant().covenantAuth);
                })()
            }, {
                id: "VOUCHER_ASSURANT",
                handler: $scope.showAssurants,
                icon: "icon fa fas fa-ticket-alt fa-fw",
                description: "Recompra",
                autoSelectUniquePaymentEvent: true,
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "VOUCHER_ASSURANT") != null,
                disabled: (function() {
                    return !CartService.cart.customer;
                })()
            }, {
                id: "VOUCHER_STOREX",
                handler: $scope.showStorexVouchers,
                icon: "icon fa fas fa-ticket-alt fa-fw",
                description: "Voucher",
                autoSelectUniquePaymentEvent: true,
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "VOUCHER_STOREX") != null,
                disabled: (function() {
                    return false;
                })()
            },{
                id: "VOUCHER_GIVEX",
                handler: $scope.showGivexVouchers,
                icon: "icon fa fas fa-ticket-alt fa-fw",
                description: "Vale Troca",
                autoSelectUniquePaymentEvent: true,
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "VOUCHER_GIVEX") != null,
                disabled: (function() {
                    return CartService.hasServiceItems();
                })()
            },{
                id: "VOUCHER_OMNI",
                handler: $scope.showOmniVoucher,
                icon: "icon fa fa-voucher-omni fa-fw",
                description: "Vale OMNI",
                autoSelectUniquePaymentEvent: true,
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "VOUCHER_OMNI") != null,
                balance: (function() {
                    return (CartService.cart.customer && ConfigService.config.business.promo_voucher_balance_enabled) ?
                            CartService.cart.customer.vouchersBalance : null;
                })(),
                disabled: (function() {
                    return CartService.hasServiceItems();
                })()
            },{
                id: "VOUCHER_SAFE",
                handler: $scope.showSafeVouchers,
                icon: "icon fa fas fa-ticket-alt fa-fw",
                description: "Vale Troca SAFE",
                autoSelectUniquePaymentEvent: true,
                privateLabel: false,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "VOUCHER_SAFE") != null,
                disabled: (function() {
                    return CartService.hasServiceItems();
                })()
            }, {
                id: "SITEF_QR",
                handler: $scope.sitefqrPayment,
                icon: "icon fa fas fa-qrcode fa-fw",
                description: "QR Sitef",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "SITEF_QR") != null,
              	disabled: false
            },
			{
                id: "QR_LINX",
                handler: qrLinxPayment,
                icon: "icon fa fas fa-qrcode fa-fw",
                description: "QR Linx",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "QR_LINX") != null,
                disabled: false
            },
            {
                id: "PAYMENT_LINK",
                handler: paymentLink,
                icon: "icon fa fas fa-link fa-fw",
                description: "Link de Pagamento",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "PAYMENT_LINK") != null,
                disabled: disablePaymentLink()
            },
            {
                id: "SITEF_PAYMENT_LINK",
                handler: sitefPaymentLink,
                icon: "icon fa fas fa-link fa-fw",
                description: "Link de Pagamento SiTef",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "SITEF_PAYMENT_LINK") != null,
                disabled: disableSitefPaymentLink()
            },
            {
                id: "CEA_PAY",
                handler: $scope.tefPayment,
                icon: "icon fa fa-credit-card fa-fw",
                description: "C&A Pay",
                privateLabel: true,
                exclusive: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.find(method => method == "CEA_PAY") != null,
                disabled: (function() {
                    return (PaymentService.containsPrivateLabelPayment());
                })()
	        },
	        {
                id: "INSTALLMENT_PLAN",
                handler: initInstallmentPlanPayment,
                icon: "icon fa fas fa-money-check fa-fw",
                description: "Crediário",
                privateLabel: false,
                cashbackEnabled: ConfigService.config.business.payment_methods_with_cashback.some(method => method == "INSTALLMENT_PLAN"),
                disabled: false,
            },
        ];

        /**
         *
         */
        CustomerService.registerObserverCallback("paymentMethodsCtrl", function() {
            if ($state.is('easyorder.paymentmethods')) {
                AlertService.info("O carrinho deverá ser reprocessado.").then(function() {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                });
            }
        });

        function disablePaymentLink() {
            const customer = CartService.cart.customer;

            // ainda não suportamos múltiplos pedidos na tela de link de pagamentos
            if (CartService.cart.mergedOrders) {
                return true;
            }

            // loops ainda não suporta recargas..
            if (CartService.hasServiceItems()) {
                return true;
            }

            if (parseFloat(PaymentService.totalPayments()) > 0) {
                return true;
            }

            if (!CartService.cart.fiscalItems || CartService.cart.fiscalItems.length == 0) {
                return true;
            }

            return false;
        }

        /**
         *
         */
        function disableSitefPaymentLink() {
            return !!CartService.cart.salePayments.find(p => p.type === 'SaleTEFPayment') || !CartService.cart.fiscalItems || CartService.cart.fiscalItems.length == 0;
        }

        /**
         *
         */
        function selectPaymentMethodFromDefList(methodId) {
            const method = allMethods.find(method => method.id === methodId);
            const customName = ConfigService.config.business.available_payment_methods_names.find(pmnt => pmnt.id === methodId);

            if (method && customName) {
                method.description = customName.name;
            }
            return method;
        }
        /**
         *
         */
        $scope.processPaymentMethods = function() {
            const configPmntMethods = ConfigService.config.business.available_payment_methods;
            const customerPmntMethods = CartService.cart.customer ? CartService.cart.customer.avaliablePaymentMethods : null;

            // ps, poderia utilizar spread operator, porém
            // alguns dispositivos ainda tem webview antigo
            const paymentMethods = [].concat(configPmntMethods);

            if (paymentMethods && customerPmntMethods) {
                customerPmntMethods.forEach((method) => {
                    if (paymentMethods.includes(method)) {
                        return;
                    }
                    paymentMethods.push(method);
                });
            }

            $scope.activeMethods = (function(data) {
                let ret = [];

                if (data) {
                    while (data.length) {
                        const m = data.shift(); //Remove o primeiro e retorna.

                        if (m) {
                            const selectedMethod = selectPaymentMethodFromDefList(m);

                            if (selectedMethod) {
                                if (selectedMethod.id == 'VOUCHER_ASSURANT') {
                                    if (voucherAssurantAvailable()) {
                                        ret.push(selectedMethod);
                                    }
                                } else if (selectedMethod.id == 'VOUCHER_STOREX') {
                                    if (storexVoucherAvailable()) {
                                        ret.push(selectedMethod);
                                    }
                                } else {
                                    ret.push(selectedMethod);
                                }
                            } else {
                                LogService.warn(`Forma de pagamento parametrizada não é suportada: ${m}`);
                            }
                        }
                    }
                }

                const exclusive = ret.filter(p => p.exclusive); //Filtrando formas exlcusivas.

                if (exclusive.length) {
                    ret = exclusive;
                }

                if ($scope.pendingDownPayment) {
                    const downPaymentMethods = PaymentService.downPaymentMethods();
                    ret = ret.filter(method => downPaymentMethods.includes(method.id));
                }

                return ret;
            })(paymentMethods);
        };

        $scope.$watch('content.paymentValue', function() {
            if ($scope.activeMethods) {
                var hasCashMethod = $scope.activeMethods.find(m => m.id == "CASH");

                if (hasCashMethod) {
                    $scope.paymentValue = $scope.content.paymentValue;

                    if ((parseFloat($scope.paymentValue) + parseFloat($scope.totalPayments)) > parseFloat(CartService.cart.totalValue)) { // Vai ter troco?
                        $scope.activeMethods.forEach(method => {
                            method.disabled = method.id != 'CASH';
                        });
                    } else if ((parseFloat($scope.paymentValue) + parseFloat($scope.totalPayments)) <= parseFloat(CartService.cart.totalValue)) {
                        $scope.activeMethods.forEach(method => {
                            method.disabled = false;
                        });
                        $scope.processPaymentMethods();
                    }
                }
            }
        });
        /**
         *
         */
        function navigateToState(state, params) {
            const transitionParams = {
                flowHistory: {
                    flow: [$state.current.name],
                    params: []
                }
            };
            angular.merge(transitionParams, params || {});

            UtilService.navigateWithoutHistory(state, transitionParams);
        };


        $scope.hasNonCashbackPaymentMethods = () => {
            let found = false;

            if(CartService.cart.salePayments) {
                CartService.cart.salePayments.forEach(payment => {
                    var method = allMethods.find(method => method.id === payment.paymentCodeMethod);
                    if (method && !method.cashbackEnabled) {
                        found = true;
                    }
                })
            }
            return found;
        };

        $scope.hasNonVoucherOrGiftCardPaymentMethos = () => {
            let found = false;
            if(CartService.cart.salePayments) {
                CartService.cart.salePayments.forEach(payment => {
                    if (payment.paymentCodeMethod !== "GIFT_CARD" && payment.paymentCodeMethod !== "VOUCHER_GIVEX") {
                        found = true; 
                    }
                })
            }
            return found;
        };

        /**
         * Adiciona cashback e processa pagamento
         */
        $scope.addCashbackValueAndProcessPayment = function(paymentMethod, paymentValue) {
            var cashbackAddedInThisProcess = false;


            // se cashback habilitado e já existem formas usadas que não permitem o cashback, exibe mensagem.
            if (paymentMethod.cashbackEnabled
                && $scope.hasNonCashbackPaymentMethods()
                && !getCashbackDiscountOnCart()
                && $scope.cashback.availableValueToPurchase > 0) {

                var message = "Já existem pagamentos com formas que não resgatam Cashback, deseja continuar e não usar o Cashback?";
                LogService.warn(message);
                AlertService.confirm(message).then(
                    function(success) {
                        if (success) {
                            // desalibita o uso do cashback na forma atual
                            paymentMethod.cashbackEnabled = false;
                            initPaymentHandler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
                        }
                    }
                );
            } else if (!paymentMethod.cashbackEnabled && ($scope.cashback.availableValueToPurchase > 0 || getCashbackDiscountOnCart())) {
                var message = getCashbackDiscountOnCart() ?
                                "Forma não aceita para resgatar cashback, todos os pagamentos serão desfeitos"
                                    : "Forma não aceita para resgatar cashback";
                LogService.warn(message);
                AlertService.confirm(message).then(
                    function(success) {
                        if (success) {
                            if(CartService.cart.salePayments.length) {
                                PaymentService.cancel(true).then(function() {
                                    $state.go('easyorder.cart');
                                });
                            } else {
                                initPaymentHandler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
                            }
                        }
                    }
                );
            } else {
                initPaymentHandler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
            }

        }

        /**
         *
         * @param {Método de pagamento} paymentMethod
         * @param {Valor de pagamento} paymentValue
         * @param {Define se o cashback foi adicionado neste processo} cashbackAddedInThisProcess
         */
        function initPaymentHandler(paymentMethod, paymentValue, cashbackAddedInThisProcess) {
            if (paymentMethod.cashbackEnabled && CartService.cart.customer  && $scope.cashback.minimumPurchaseAmount <= CartService.cart.totalValue) {
                if(!getCashbackDiscountOnCart() && $scope.cashback.availableValueToPurchase) {
                    addCashbackDiscount($scope.cashback.availableValueToPurchase);
                    cashbackAddedInThisProcess = true;
                }

                var isConfirmCashbackUses = new Number(CartService.cart.totalValue) <= (new Number(PaymentService.totalPayments()) + new Number(paymentValue));
                if (isConfirmCashbackUses && getCashbackDiscountOnCart()) {
                    var request = buildCashbackRequest();
                    return new IonicLoadingHelper(() => CashbackService.createCashbackAuthorizationRequest(request), $ionicLoading)
                        .then((resp) => {
                            CartService.cart.usedCashback = resp;
                            paymentMethod.handler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
                        })
                        .catch(error => onUsesCashbackRequestError(error, paymentMethod, paymentValue));
                } else {
                    paymentMethod.handler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
                }
            } else {
                paymentMethod.handler(paymentMethod, paymentValue, cashbackAddedInThisProcess);
            }
        }

        function buildCashbackRequest() {
            var request = new CashbackCreatenRequest();
            request.cashbackAuthorization.saleId = getCashbackSaleId();
            request.cashbackAuthorization.baseValue = new Number(CartService.cart.totalValue) + getCashbackDiscountOnCart().value;
            request.cashbackAuthorization.cashbackAmount = getCashbackDiscountOnCart().value;
            request.customer.document.type = CartService.cart.customer.documentType;
            request.customer.document.value = CartService.cart.customer.document;
            request.customer.name = CartService.cart.customer.name;
            request.customer.email = CartService.cart.customer.email;
            return request;
        }

        function getCashbackSaleId() {
            let monthNumber = new Date().getMonth() + 1;
            let year = new Date().getFullYear() + "";
            let month = monthNumber.toString().padStart(2, "0");
            let day = new Date().getDate().toString().padStart(2, "0") + "";
            let nsu = ConfigService.nsuControll.currentCartNSU() + "";
            let storeNumber = ConfigService.config.storeNumber + "";
            let terminalNumber = ConfigService.config.terminalNumber + "";
            return year + month + day + nsu + terminalNumber + storeNumber; //ANO + MES + DIA + NSU + Terminal + Loja
        }

        function addCashbackDiscount(cashbackValue) {
            var discount = {};
            discount.sequenceId = CartService.cart.discounts.length;
            discount.source = "CASHBACK";
            discount.value = cashbackValue;

            CartService.removeDiscountBySource(discount.source);
            CartService.addDiscount(discount);
            CartService.updateCartTotalValue();
        }

        function addFirstPurchaseDiscount(paymentValue, customer){
            var discount = {};
            discount.factor = customer.firstPurchaseDiscountPercentage;
            discount.source = "FIRST_PURCHASE";
            discount.value = parseFloat((paymentValue * customer.firstPurchaseDiscountPercentage / 100).toFixed(2));

            CartService.removeDiscountBySource(discount.source);
            CartService.updateCartTotalValue();

            CartService.addDiscount(discount);
            CartService.cart.firstPurchaseDiscountValue = discount.value;
        }

        function onUsesCashbackRequestError(error, paymentMethod, paymentValue) {
            var message = 'Não foi possível aplicar o saldo cashback, deseja finalizar a venda sem o desconto cashback?';
            AlertService.confirm(message).then(
                function(success) {
                    if (success) {
                        CartService.undoCashbackDiscountPartial();
                        paymentMethod.handler(paymentMethod, paymentValue);
                    }
                }
            );


            LogService.error(error);
        }

        $scope.hasPaymentMethodWithCashbackEnabled = function() {
            return allMethods.find(method => method.cashbackEnabled);
        }

        function hasNoCashbackPaymentOnCart() {
            let hasNoCashbackPaymentOnCart = false;

            if(CartService.cart.salePayments) {
                CartService.cart.salePayments.forEach(payment => {
                    var method = allMethods.find(method => method.id === payment.id);
                    if(method && method.cashbackEnabled) {
                        hasNoCashbackPaymentOnCart = true;
                    }
                })
            }
            return hasNoCashbackPaymentOnCart;
        }

        function getCashbackDiscountOnCart() {
            let cashbackDiscountOnCart = 0;

            if(CartService.cart.discounts){
                CartService.cart.discounts.forEach(discount => {
                    if (discount.source === 'CASHBACK') {
                        cashbackDiscountOnCart = discount;
                    }
                });
            }
            return cashbackDiscountOnCart;
        }

        /**
         * Busca informações de cashback
         */
        function loadCashbackInfo() {
            var customer = CartService.cart.customer;

            // Se não houver desconto de cashback no carrinho, nem pagamento que não resgate cashback, realiza a busca do saldo no LWS

            if (!hasNoCashbackPaymentOnCart() && !getCashbackDiscountOnCart() && $scope.hasPaymentMethodWithCashbackEnabled() && customer && customer.document && customer.documentType) {
                $scope.loadingState = true;

                var cashbackInfoRequest = new CashbackInfoRequest(customer.documentType, customer.document, CartService.cart.totalValue);

                return new IonicLoadingHelper(() => CashbackService.getCashbackInfo(cashbackInfoRequest), $ionicLoading)
                    .then((result) => {
                        $scope.cashback.availableValueToPurchase = result.availableValueToPurchase;
                        $scope.cashback.minimumPurchaseAmount = result.minimumPurchaseAmount ? result.minimumPurchaseAmount : 0.0;
                        $scope.cashback.customer = result.customer;
                    })
                    .catch((error) => {
                        LogService.error(error);
                        if (error.status === 500) {
                            return AlertService.error('Não foi possível consultar o saldo cashback.');
                        }
                    });
            }

            // Se houver desconto de cashback no carrinho, retirar da visão do usuário o saldo disponível
            if(getCashbackDiscountOnCart() && !hasNoCashbackPaymentOnCart()) {
                $scope.cashback.availableValueToPurchase = 0.0;
                $scope.cashback.minimumPurchaseAmount = 0.0;
                $scope.cashback.customer = {
                    documentType: customer.documentType,
                    document: customer.document
                }

            }
            return $q.resolve();
        }

        function updatePaymentValue() {
            $scope.paymentValue = $scope.pendingDownPayment ? PaymentService.downPaymentValue()
                                                            : MathUtils.subtract(CartService.cart.totalValue, $scope.totalPayments);
            $scope.content.paymentValue = $scope.paymentValue;
        }

        /**
         * Função de inicialização.
         */
        (function() {
            $scope.cashback = {
                availableValueToPurchase: 0.0,
                minimumPurchaseAmount: 0.0,
                customer: {}
            }

            $scope.content = {}
            $scope.pendingDownPayment = PaymentService.isPendingDownPayment();
            $scope.cashPaymentCurrent = PaymentService.totalCashPayments();
            $scope.totalPayments = PaymentService.totalPayments();
            $scope.paymentValue = $scope.pendingDownPayment ? PaymentService.downPaymentValue()
                                                            : MathUtils.subtract(CartService.cart.totalValue, $scope.totalPayments);
            $scope.content.paymentValue = $scope.paymentValue;
            $scope.currentMaxPaymentValue = ConfigService.config.business.available_payment_methods.includes('CASH') ? 10000 : $scope.paymentValue;
            $scope.allowParcialPayment = !$scope.pendingDownPayment && ConfigService.config.business.parcials_payment_enabled;
            $scope.cart = CartService.cart;
            $scope.isVouchersBalanceEnabled = ConfigService.config.business.promo_voucher_balance_enabled;
            $ionicLoading.hide();

            Promise.resolve()
                .then(checkVouchers)
                .then($scope.processPaymentMethods)
                .then(loadCashbackInfo)
                .then(loadOldPaymentCampaigns)
                .then(doVouchersAutoRedirectionOption);
        })();

        $ionicModal.fromTemplateUrl('views/paymentmethodsdetails.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modalMethodDetails) {
            $scope.modalMethodDetails = modalMethodDetails;
        });

        $scope.$on('$destroy', function() {
            if ($scope.modal) {
                $scope.modal.remove();
            }

            if ($scope.modalMethodDetails) {
                $scope.modalMethodDetails.remove();
            }
        });

        $scope.openMethodDetails = function() {
            $scope.modalMethodDetails.show();
        };

        $scope.closeMethodDetails = function() {
            $scope.modalMethodDetails.hide();
        };

    });
})();
