(function() {
    'use strict';

    angular.module('easyorder.controllers.paymentplans', [
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.services.insurance',
        'easyorder.services.config',
        'easyorder.services.user',
        'easyorder.services.log'
    ])

    .controller('PaymentPlansCtrl', function($scope,
        $stateParams,
        $ionicPopup,
        CartService,
        UtilService,
        PaymentService,
        AlertService,
        InsuranceService,
        ConfigService,
        UserService,
        LogService,
        $ionicLoading,
        BiometricsService,
        FaceBiometricsModal) {

        $scope.campaignPaymentTotalDiscount = CartService.cart.campaignPaymentTotalDiscount;
        $scope.firstPurchaseDiscountValue = CartService.cart.firstPurchaseDiscountValue;
        $scope.customer = CartService.cart.customer;

        /**
         * TODO: Nao misturar funções!! Aqui deveria ser uma função para Eventos de Pagamento e OUTRA para Planos!
         */
        $scope.displaysInterestInstallmentAmountsByAythorizer = (param) => {
            let ret = true;
            let showInterests = ConfigService.config.business.displays_interest_installment_amounts;
                        
            if (!showInterests) {
                if (param.paymentPlans && param.paymentPlans.length) { // É Evento?
                    ret = param.paymentPlans[0].interestType === "STORE";
                } else {
                    ret = param.interestType === "STORE";
                }
            }
            return ret;
        }

        $scope.goBack = () => {
           if ($stateParams.flowHistory) {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                })
                .then(CartService.undoCashbackDiscountPartial)
                .then(CartService.undoFirtsPurchaseDiscount)
                .then(undoPaymentCampaigns)
                .then(() => {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();
                    
                    UtilService.navigateWithoutHistory(backState, params);
                }).finally($ionicLoading.hide);
            }
        };

        async function undoPaymentCampaigns() {
            const campaigns = PaymentService.currentPayment.campaigns;

            if (campaigns && campaigns.length) {
                campaigns.forEach(campaign => {
                    LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                    CartService.undoCampaignDiscountsPartial(campaign);
                });
            }
        };

        /**
         *
         */
        $scope.selectPaymentEvent = function(paymentPlanEvent) {
            LogService.info(`Evento de pagamento selecionado: ${paymentPlanEvent.description}`);

            simulatePaymentPlansInsurance(paymentPlanEvent);

            $scope.paymentPlanEvent = paymentPlanEvent;
        };

        /**
         *
         */
        $scope.selectPaymentPlan = async function(paymentPlan) {
            LogService.info(`Plano de pagamento selecionado: ${paymentPlan.description}`);

            const payment = PaymentService.currentPayment();
            payment.plan = paymentPlan;
            payment.event = $scope.paymentPlanEvent;
            payment.plan.paymentCodeEvent = $scope.paymentPlanEvent.code;
            payment.plan.processorCode = $scope.paymentPlanEvent.processorCode;

            const insuranceProvider =
                ConfigService.config.business.payment_insurance_provider;

            try {
                if (insuranceProvider == 'LEBES') {
                    paymentPlan = await simulatePaymentPlanInsurance(paymentPlan);
                } else if (insuranceProvider == 'LREN') {
                    await processInsurancePlanAuthorization(payment).$promise;
                }
            } catch(err) {
                LogService.error(err);
            }

            // usuário fechou o popup de planos sem selecionar o plano desejado
            if (paymentPlan == null) {
                LogService.info('Operador desistiu do plano selecionado');
                return;
            }

            paymentPlan.paymentCodeEvent = $scope.paymentPlanEvent.code;
            paymentPlan.processorCode = $scope.paymentPlanEvent.processorCode;
            payment.plan = paymentPlan;

            LogService.info('Valor do pagamento solicitado: [' + payment.paymentValue + "]");
            payment.paymentValue = MathUtils.subtract(
                $scope.paymentPlanEvent.maxValue,
                paymentPlan.downPaymentValue || 0
            );
            LogService.info('Valor do pagamento aplicado: [' + payment.paymentValue + "]");

            UtilService.navigateWithoutHistory($stateParams.paymentState);
        };

        async function simulatePaymentPlanInsurance(paymentPlan) {
            const paymentMethod = PaymentService.currentPayment().paymentMethod;

            if (paymentMethod != 'INSTALLMENT_PLAN') {
                return await showPlanPopup(paymentPlan);
            }

            await $ionicLoading.show({
                template : 'Simulando seguro...',
                duration : 0
            });

            try {
                const pp = { ...paymentPlan };
                delete pp.paymentCodeEvent;

                const res = await InsuranceService.simulatePaymentPlan({
                    transactionKey: {
                        companyNumber: ConfigService.config.companyNumber,
                        storeNumber: ConfigService.config.storeNumber,
                        terminal: ConfigService.config.terminalNumber,
                        nsu: CartService.cart.nsu,
                        date: new Date(),
                    },
                    customer: CartService.cart.customer,
                    operatorCode: UserService.operatorCode(),
                    fiscalItems: CartService.cart.fiscalItems,
                    paymentPlan: pp,
                });

                await $ionicLoading.hide();

                paymentPlan = await showPlanPopup(paymentPlan, res.data.insuredPaymentPlan);
            } catch(err) {
                LogService.error(err);
                await $ionicLoading.hide();
                paymentPlan = await showPlanPopup(paymentPlan);
            }

            delete $scope.popup;

            return paymentPlan;
        }

        async function showPlanPopup(paymentPlan, insuredPlan) {
            $scope.popup = {
                plan: paymentPlan,
                insuredPlan: insuredPlan,
                insured: insuredPlan != null,
                current: insuredPlan || paymentPlan,
                switchPlan: () => {
                    $scope.popup.current = $scope.popup.insured ?
                        $scope.popup.insuredPlan : $scope.popup.plan;
                },
                formatDate: (date) => moment(date).format('DD/MM/yyyy'),
                formatValue: (value) => MathUtils.round(value).toFixed(2),
            };

            paymentPlan = await $ionicPopup.show({
                title: 'Parcelas',
                subTitle: 'Conferência de parcelas',
                scope: $scope,
                template: planTemplate(),
                buttons: [{
                    text: 'Voltar',
                    onTap: function() { return null; },
                }, {
                    text: 'Continuar',
                    type: 'button-theme',
                    onTap: function() { return $scope.popup.current; }
                }]
            });

            delete $scope.popup;

            return paymentPlan;
        }

        function planTemplate() {
            return `
                <ion-scroll direction="y" style="height: 50vh">
                    <ion-list class="list list-inset dosis" style="margin: unset;width:100%;">
                        <ion-item ng-if="popup.current.downPaymentValue" class="item item-inset item-text-wrap">
                            <div class="row" style="display:flex;flex-flow:row wrap; justify-content:space-between;align-items:flex-start;padding: .5em 0;">
                                <div class="col col-100" style="flex:0 50%;">
                                    <h2 color="primary">ENTRADA</h2>
                                    <div class="col col-100 no-margin dosis" style="word-break:break-all;white-space: nowrap;display:flex;flex-flow:column;align-items:flex-start;padding:0;margin-top:5px;">
                                        <span>R$ {{ popup.formatValue(popup.current.downPaymentValue) }}</span>
                                    </div>
                                </div>
                            </div>
                        </ion-item>

                        <ion-item ng-if="popup.current.instalments" ng-repeat="i in popup.current.instalments" class="item item-inset item-text-wrap">
                            <div class="row" style="display:flex;flex-flow:row wrap; justify-content:space-between;align-items:flex-start;padding: .5em 0;">
                                <div class="col col-100" style="flex:0 50%;">
                                    <h2 color="primary">{{ i.number + 1 }} - {{ popup.formatDate(i.dueDate) }}</h2>
                                    <div class="col col-100 no-margin dosis" style="word-break:break-all;white-space: nowrap;display:flex;flex-flow:column;align-items:flex-start;padding:0;margin-top:5px;">
                                        <span>R$ {{ popup.formatValue(i.instalmentValue) }}</span>
                                    </div>
                                </div>
                            </div>
                        </ion-item>
                    </ion-list>
                </ion-scroll>

                <ion-checkbox
                    ng-if="popup.insuredPlan"
                    ng-model="popup.insured"
                    ng-change="popup.switchPlan()"
                    style="font-size: 1em">Seguro Prestamista
                </ion-checkbox>
                <div style="margin-top: 4px">Total: {{ popup.formatValue(popup.current.totalValue) }}</div>
            `;
        };
        
        /**
         *
         */
        var simulatePaymentPlansInsurance = function(paymentPlanEvent) {
            LogService.info("Oferecer seguro para cliente: ", $scope.customer && $scope.customer.offerCustomerInsurance);

            if (CartService.cart.insuranceData &&
                PaymentService.currentPayment().privateLabel &&
                $scope.customer &&
                $scope.customer.offerCustomerInsurance &&
                (PaymentService.currentPayment().paymentMethod === "TOTVSCARD_CREDIT_BOOK" ||
                    PaymentService.currentPayment().paymentMethod === "CONDUCTOR_CREDIT_BOOK")) {

                var events = [];

                paymentPlanEvent.paymentPlans.forEach(function(plan) {
                    events.push({ saleValue: plan.totalValue, numberOfInstallments: plan.instalments.length, plans: [plan.code] });
                });

                var sucesso = function(result) {
                    LogService.info(result);

                    InsuranceService.currentSimulation = result.insuranceSimulations;

                    if (InsuranceService.currentSimulation) {
                        paymentPlanEvent.paymentPlans.forEach(function(paymentPlan) {
                            revertInsurancePlanSimulation(paymentPlan); //Removendo simulação anterior.
                            processInsurancePlanSimulation(paymentPlan);
                        });
                    }
                    $scope.holder = {};
                    $scope.holder.birthday = result.holderBirthday;
                    $scope.holder.gender = result.holderGender;
                    $scope.holder.occupation = result.holderOccupation;
                    $scope.insuranceCode = result.insuranceCode;
                    $scope.affinityClient = result.affinityClient;

                    $scope.loadingState = false;
                    LogService.info(`Simulação de seguro para Evento de pagamento: ${$scope.paymentPlanEvent}`);
                    LogService.info(`Simulação de seguro suporte: ${$scope.holder}`);
                    LogService.info(`Simulação de seguro Código do Seguro: ${$scope.insuranceCode}`);
                };
                var erro = function(response) {
                    LogService.error(response);
                    $scope.loadingState = false;
                };

                CartService.cart.insuranceData.nsu = ConfigService.nsuControll.currentNSU();
                $scope.cupomNumber = CartService.cart.insuranceData.nsu;
                $scope.cardNumber = $scope.customer.customerId;
                $scope.loadingState = true;

                InsuranceService.simulate(UserService.currentUser.username, ConfigService.config.terminalNumber, $scope.cupomNumber, $scope.customer.processor,
                    null, $scope.cardNumber, events, sucesso, erro);
            } else {
                InsuranceService.clearSimutaionData();
            }
        };

        /**
         * Processa autorizacão do seguro para o pagamento passado, se houver um seguro definido.
         */
        var processInsurancePlanAuthorization = function(payment) {
            var sucesso = function(response) {
                if (!response.insuranceQuote || !response.insuranceQuote.insuranceCoverageQuoteItems) {
                    $scope.loadingState = false;
                    LogService.info("Serviço de seguros indisponível.");
                }
                if (payment.insurance === undefined) {
                    payment.insurance = {};
                }
                if (response.insuranceQuote && response.insuranceQuote.insuranceCoverageQuoteItems) {
                    payment.insurance.totalValue = response.insuranceQuote.insuranceCoverageQuoteItems.insuranceTotalValue;
                    payment.insurance.iofTotalValue = response.insuranceQuote.insuranceCoverageQuoteItems.iofTotalValue;

                    CartService.cart.insuranceData.totalValue = response.insuranceQuote.insuranceCoverageQuoteItems.insuranceTotalValue;
                    CartService.cart.insuranceData.iofTotalValue = response.insuranceQuote.insuranceCoverageQuoteItems.iofTotalValue;
                }
                payment.insurance.nsuCode = response.nsuCode;
                payment.insurance.code = response.insuranceData.code;
                CartService.cart.insuranceData.authorizationNumber = response.nsuCode;
                CartService.cart.insuranceData.code = response.insuranceData.code;
                CartService.cart.insuranceData.effectiveNumber = response.insuranceEffectiveNumber;
                CartService.cart.insuranceData.contractInfo = response.contractInfo;

                CartService.cart.insuranceData.policyNumber = response.policyNumber;
                CartService.cart.insuranceData.serialNumber = response.serialNumber;
                CartService.cart.insuranceData.luckyNumber = response.luckyNumber;

                $scope.loadingState = false;
                LogService.info(`Autorização de Seguro: ${CartService.cart.insuranceData.authorizationNumber} Plano: ${payment.plan.code}`);
            };
            var erro = function(response) {
                LogService.info(response);

                revertInsurancePlanSimulation(payment.plan);
                InsuranceService.clearSimutaionData();

                $scope.loadingState = false;
            };

            $scope.loadingState = true;

            return InsuranceService.authorize(UserService.operatorCode(), // operatorCode
                new Date(), // channelDate
                ConfigService.config.terminalNumber, // componentNumber
                1, // sequenceNumber
                $scope.cupomNumber, // cupomNumber
                $scope.customer.processor, // processorCode
                new Number(PaymentService.currentPayment().plan.totalValue).toFixed(2),
                $scope.insuranceCode, // 1, //insuranceCode
                $scope.holder && $scope.holder.birthday ? $scope.holder.birthday : '', // '1982-06-13', //birthDate - vem da
                // simulação
                $scope.holder && $scope.holder.gender ? $scope.holder.gender : '', // 'FEMININO', //genre - vem da simulação
                payment.plan.instalments.length, // numberOfInstallments
                $scope.affinityClient, // 1, //clientAffinity - vem da simulação
                null, // occupationCode
                $scope.customer.document, // cpfCnpj
                null, // insurancePlan
                'S', // confirmationFlag
                new Date(), // operationDate
                sucesso, erro);
        };

        /**
         * Atualiza o valor das parcelas do plano com os valores de seguro, se houver simulação para o mesmo.
         */
        var processInsurancePlanSimulation = function(paymentPlan) {
            var insuranceSimulation = InsuranceService.isPlanInsured(paymentPlan);

            if (insuranceSimulation) {
                paymentPlan.instalments.forEach(function(instalment) {
                    for (var i = 0; i < insuranceSimulation.insuranceInstallments.length; i++) {
                        var insuranceInstalment = insuranceSimulation.insuranceInstallments[i];

                        if (instalment.number == insuranceInstalment.installmentNumber - 1) {
                            instalment.insuranceValue = insuranceInstalment.installmentValue;
                            break;
                        }
                    }
                    instalment.instalmentOriginalValue = instalment.instalmentValue;
                    instalment.instalmentValue += (instalment.insuranceValue ? instalment.insuranceValue : 0);

                    if (instalment.number == 0) {
                        paymentPlan.firstInstalment.instalmentValue = instalment.instalmentValue;
                    }
                });
            }
        };

        /**
         * Desfaz as alterações no valor do plano causadas pelo processamento de uma simulação se seguro,
         * se houver simulação para o mesmo.
         */
        var revertInsurancePlanSimulation = function(paymentPlan) {
            var insuranceSimulation = InsuranceService.isPlanInsured(paymentPlan);

            if (insuranceSimulation) {
                paymentPlan.instalments.forEach(function(instalment) { //Atualiza o valor das parcelas do plano.
                    for (var i = 0; i < insuranceSimulation.insuranceInstallments.length; i++) {
                        var insuranceInstalment = insuranceSimulation.insuranceInstallments[i];

                        if (instalment.number == insuranceInstalment.installmentNumber - 1) {
                            delete instalment.insuranceValue;
                            break;
                        }
                    }
                    instalment.instalmentValue = instalment.instalmentOriginalValue ? instalment.instalmentOriginalValue : instalment.instalmentValue;
                    delete instalment.instalmentOriginalValue;

                    if (instalment.number == 0) {
                        paymentPlan.firstInstalment.instalmentValue = instalment.instalmentValue;
                    }
                });
            }
        };

        /**
         *
         */
        var getPlansSortFunction = () => {
            if (ConfigService.config.business.payment_plan_display_order === 'REVERSED_INTS_NUMBER') {
                return (p1, p2) => {
                    if (p1.code < p2.code) return 1;
                    if (p1.code > p2.code) return -1;
                    return 0;
                };
            } else if (ConfigService.config.business.payment_plan_display_order === 'NATURAL_INTS_NUMBER') {
                return (p1, p2) => {
                    if (p1.code < p2.code) return -1;
                    if (p1.code > p2.code) return 1;
                    return 0;
                };
            }
        };

        function confirmCustomerPhoto(customerFaceImage) {
            $scope.customerFaceImage = customerFaceImage;
            LogService.info(`Confirmação da foto pelo operador solicitada para segur fluxo C&A PAY.`);
            $ionicPopup.confirm({
                templateUrl: 'js/components/biometrics/customer-photo-confirmation.html',
                title: 'O cliente é o mesmo da foto?',
                scope: $scope,
                buttons: [
                    { text: 'Não',
                        onTap: function(event) {
                            LogService.info(`Foto não confirmada pelo operador, fluxo será abortado!`);
                            const message = createPhotoConfirmationMessageLog(false);
                            LogService.info(message);
                            AlertService.error("Pagamento com C&A Pay não autorizado!")
                                    .then(() => $scope.goBack());
                        }
                    },
                    {
                        text: '<b>Sim</b>',
                        type: 'button-theme',
                        onTap: function(event) {
                            LogService.info(`Foto confirmada pelo operador, fluxo C&A Pay continuará!`);
                            const message = createPhotoConfirmationMessageLog(true);
                            LogService.info(message);
                            UtilService.navigateWithoutHistory($stateParams.paymentState);
                        }
                    }
                ]
            });
        }


        function createPhotoConfirmationMessageLog(photoConfirmed) {
            // foi solicitada uma mensagem de log específica, contendo a mensagem abaixo.
            const storeNumber = ConfigService.config.storeNumber;
            const terminalNumber = ConfigService.config.terminalNumber;
            const operatorCode = UserService.operatorCode();
            const nsu = CartService.cart.nsu;

            return `Store: [${storeNumber}], WorkstationId: [${terminalNumber}], OperatorPartyId: [${operatorCode}], TransactionSequence: [${nsu}] - [biometrics validation] - User reported that the photo displayed is ${photoConfirmed ? 'match' : 'not match'} with customer`;
        }

        function goFoward() {
            const paymentMethod = PaymentService.currentPayment().paymentMethod;
            const customer = PaymentService.currentPayment().customer;
            if (BiometricsService.isBiometricPaymentMethod(paymentMethod)) {
                if (!BiometricsService.hasCustomerBiometricsInfo(customer)) {
                    // se não veio informações acerca da biometria mas ela está habilitada por parâmetro, exibe um erro e não segue o fluxo!
                    LogService.info(`Biometria para a forma '${paymentMethod}' habilitada mas não foram retornados dados da biometria para o cliente!`);
                    AlertService.error(`Erro de comunicação com a validação da biometria, não será possível utilizar esta forma de pagamento!`)
                        .then(() => $scope.goBack());
                } else if (BiometricsService.isCustomerBiometricsEnabled(customer)) {
                    LogService.info(`Biometria para a forma '${paymentMethod}' habilitada e requisitada para o cliente.`);
                    handleBiometry(customer);
                } else {
                    LogService.info(`Biometria para a forma '${paymentMethod}' habilitada mas não foi requisitada para o cliente.`);
                    const customerFaceImage = BiometricsService.getCustomerFaceImage(customer);
                    confirmCustomerPhoto(customerFaceImage);
                }
            } else {
                if (paymentMethod === 'CEA_PAY') {
                    // No caso de C&A Pay sempre deve ser exibido a confirmação da foto do cliente (independentemente da utilização da biometria), mesmo que ela não seja retornada
                    LogService.info(`Biometria para a forma '${paymentMethod}' não habilitada no servidor de configuração. Será solicitada a confirmação do operador.`);
                    const customerFaceImage = BiometricsService.getCustomerFaceImage(customer);
                    confirmCustomerPhoto(customerFaceImage);
                } else {
                    UtilService.navigateWithoutHistory($stateParams.paymentState);
                }
            }
        }

        function handleBiometry(customer) {
            FaceBiometricsModal.open(customer, 'SALE')
                .then(() => UtilService.navigateWithoutHistory($stateParams.paymentState))
                .catch(() => {
                    AlertService.error("Pagamento com C&A Pay não autorizado!")
                        .then(() => $scope.goBack());
                });
        }

        /**
         *
         */
        (function() {
            $scope.loadingState = true;

            PaymentService.search().then((result) => {
                if (!result || !result.paymentPlanEvents || result.paymentPlanEvents.length === 0) {
                    AlertService.warn('Não existem eventos de pagamento para a forma escolhida.')
                        .finally(function() {
                            $scope.loadingState = false;
                            $scope.goBack();
                        });
                } else {
                    if ($stateParams.disableSorting !== true) {
                        result.paymentPlanEvents.forEach(evt => evt.paymentPlans.sort(getPlansSortFunction()));
                    }
                    $scope.paymentPlanEvents = result.paymentPlanEvents;
                    $scope.hideUniqueEvent = ConfigService.config.business.auto_select_unique_event === true && result.paymentPlanEvents.length === 1;
    
                    if ($scope.hideUniqueEvent) {
                        $scope.paymentPlanEvent = result.paymentPlanEvents[0];
                    }
                    $scope.loadingState = false;
                }
            }, (err) => {
                AlertService.restError(err, 'Não foi possível processar os planos! ')
                    .finally(function() {
                        $scope.goBack();
                    });
                LogService.error(err);
                $scope.loadingState = false;
            });
        })();
    });
})();
