(function () {
    'use strict';
    angular.module('easyorder.controllers.prescriptions', [
        'easyorder.services.cart', 
        'easyorder.services.prescriptions',
        'easyorder.services.log'
    ])
        .controller('prescriptionsCtrl', function ($scope, $ionicPopup, CartService, PrescriptionsService, $ionicLoading, LogService) {

            $scope.data = {
                code: ''
            };
            $scope.prescriptions = [];

            /**
             *  Limpa dados de convênio
             */
            var clearPrescriptions = function() {
                CartService.cart.prescriptions = [];
            };
            /*
             *
             */
            $scope.searchPrescription = function () {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(function() {
                
                $scope.loadingPrescription = true;

                if (!findValue()) {
                    PrescriptionsService.searchPrescription({
                        code: $scope.data.code
                    }, function (response) {
                        $ionicLoading.hide();
                        LogService.info('Receitas: ', response)
                        $scope.data.code = '';
                        if(!findValue()){
                            $scope.prescriptions.push(response.result[0]);
                            CartService.cart.prescriptions = $scope.prescriptions;
                            $scope.verifyPrescriptionValid();
                        }
                        $scope.loadingPrescription = false;
                    }, function (err) {
                        $ionicLoading.hide();
                        $ionicPopup.show({
                            template: err.data.message,
                            title: 'Erro',
                            scope: $scope,
                            buttons: [{
                                text: 'Ok',
                                type: 'button-theme'
                            }]
                        });
                        $scope.loadingPrescription = false;
                    });
                } else {
                    $ionicLoading.hide();
                    $ionicPopup.show({
                        template: 'A receita já foi inserida.',
                        title: 'Erro',
                        scope: $scope,
                        buttons: [{
                            text: 'Ok',
                            type: 'button-theme',
                            onTap: function(e) {
                                $scope.data.code = '';
                            }
                        }]
                    });
                    $scope.loadingPrescription = false;
                }
            });
        };


            var findValue = function(){
                let result = $scope.prescriptions.find(function (pcpt) {
                    return pcpt.code == $scope.data.code;
                });

                if(result){
                    return true;
                }else {
                    return false;
                }
            }
            /*
             *
             */
            $scope.removePrescription = function (prescription) {
                LogService.info("Prescrições: ", CartService.cart.prescriptions);
                $scope.prescriptions = $scope.prescriptions.filter(function (pcpt) {
                    return pcpt.code != prescription.code;
                });
                CartService.cart.prescriptions = $scope.prescriptions;
                $scope.verifyPrescriptionValid();
                LogService.info("Prescrições: ", CartService.cart.prescriptions);
            };

            $scope.verifyPrescriptionValid = function(){
                $scope.prescriptions = CartService.getCompareListPrescriptionProduct();
                angular.forEach($scope.prescriptions, function(prescription){
                    prescription.isValid = !prescription.items.some(function(item){return item.compareProduct != 0});
                });
            };

            
            (function () {
                CartService.initCartObservers.add(clearPrescriptions);
                if (!CartService.cart.prescriptions) {
                    CartService.cart.prescriptions = [];
                } else {
                    $scope.verifyPrescriptionValid();
                }
            })();
        });
})();
