(function() {
    'use strict';

    angular.module('easyorder.controllers.privatelabelpayment', [
        'easyorder.services.cart',
        'easyorder.services.nfce',
        'easyorder.services.payment',
        'easyorder.services.alert',
        'easyorder.services.insurance',
        'easyorder.services.log'
    ])

    .controller('PrivateLabelPaymentCtrl', function($scope,
                                        $ionicLoading,
                                        $timeout,
                                        CartService,
                                        UtilService,
                                        ConfigService,
                                        PaymentService,
                                        CustomerService,
                                        AlertService,
                                        InsuranceService,
                                        LogService) {

        $scope.customer = CartService.cart.customer;
        $scope.mayGoBack = false;

        $scope.isTransactionInitiated = function() {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $scope.message = "";
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            if (message) {
                $scope.message = message;
            }
            LogService.info("displayMessage: " + $scope.message);
        };

        /**
         *
         */
        $scope.inputText = function(label, type, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               type,
                maxSize:            fieldSizeRange.maxSize,
                minSize:            fieldSizeRange.minSize
            });
            $scope.headerOptions = "Informe";
        };


        /**
         * Função executa a função com os parâmetros passados a esta.
         */
        $scope.tickHandler = function(func, data) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func(data);
        };

        /**
         * Solicita autorização para realização de pagamento.
         */
        $scope.authPrivateLabelSale = function(data) {
            $scope.clearDisplay();
            $scope.mayGoBack = true;

            $timeout(function() {
                $scope.displayMessage("Autorizando pagamento. Favor aguarde...");
            }, 800); //Firula.

            var reqParams = {};
            reqParams.totalValue    = new Number(PaymentService.currentPayment().paymentValue).toFixed(2);
            reqParams.cardNumber    = CartService.cart.customer.customerId;
            reqParams.processorCode = CartService.cart.customer.processor;
            reqParams.password      = data.value;
            reqParams.coupomNumber  = ConfigService.nsuControll.currentCartNSU();

            var currentPayment              = PaymentService.currentPayment();

            if (currentPayment.plan) {
                reqParams.paymentPlan           = currentPayment.plan.code;
                reqParams.installmentsNumber    = currentPayment.plan.instalments.length;
                reqParams.chargeValue           = currentPayment.plan.charges || 0;
                reqParams.discountValue         = currentPayment.plan.discount || 0;
                
                reqParams.salePayment           = {};
                reqParams.salePayment.type                        = 'SalePrivateLabelPayment';
                reqParams.salePayment.paymentSequence             = 1;
                reqParams.salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
                reqParams.salePayment.paymentValue                = new Number(currentPayment.paymentValue).toFixed(2);
                reqParams.salePayment.paymentCodePlan             = currentPayment.plan.code;
                reqParams.salePayment.paymentCodeEvent            = currentPayment.plan.paymentCodeEvent;
                reqParams.salePayment.numberOfInstallments        = currentPayment.plan.instalments.length;
                reqParams.salePayment.dateTofirstInstalment       = currentPayment.plan.firstInstalment.dueDate;
                reqParams.salePayment.firstInstalmentValue        = new Number(currentPayment.plan.firstInstalment.instalmentValue).toFixed(2);
                reqParams.salePayment.installments                = [];
                reqParams.salePayment.discountValue               = currentPayment.plan.discount || 0;   
                
                var insuranceSimulation = InsuranceService.isPlanInsured(currentPayment.plan);
                
                if (insuranceSimulation) {
                    reqParams.insurance = {
                        totalValue: currentPayment.insurance.totalValue,
                        insuranceAthorizationNumber: currentPayment.insurance.nsuCode, // @see paymentplansCtrl.js
                        code: currentPayment.insurance.code, 
                        //charged:
                        installments: []
                    };
                    var insuranceTotalValue = 0;
                    
                    insuranceSimulation.insuranceInstallments.forEach(function(installment) {
                        reqParams.insurance.installments.push({ totalValue: installment.installmentValue, number: installment.installmentNumber });
                        insuranceTotalValue += installment.installmentValue;
                    });
                    reqParams.insurance.totalValue = new Number(insuranceTotalValue).toFixed(2);
                }
            }
 
            CustomerService.authPrivateLabelPayment(reqParams, function(result) {
                $scope.clearDisplay();

                if (result.status === "AUTHORIZED" || result.status === "AUTHORIZED_OVER_LIMIT") {
                    $scope.displayMessage("Pagamento autorizado!");
                    if (CartService.cart.customer.offerCustomerInsurance && CartService.cart.insuranceData.authorizationNumber) {
                        CartService.cart.insuranceData.contractNumber = result.nsu;
                    }
                    $scope.paymentDone(result);
                } else {
                    $scope.displayMessage("Pagamento não autorizado!");
                
                    AlertService.error('Pagamento não autorizado!')
                    .then($scope.paymentError);
                }
            }, function(error) {
                $scope.clearDisplay();
                
                AlertService.restError(error, 'Erro ao autorizar pagamento!')
                .then($scope.paymentError);
            });
        };

        /**
         * Função para tratamento após finalização do pagamento com cartão próprio
         */
        $scope.paymentDone = function(paymentResults) {
            $scope.processPayment(paymentResults, CartService.cart, PaymentService.currentPayment());
            PaymentService.currentPaymentDone();
            // timeout para segurar a mensagem Pagamento Autorizado na tela.
            $timeout(function() {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(function() {
                     UtilService.navigateAfterReceivedPayment();
                });
            }, 2000);
        };

        /**
         * Processa o pagamento atual e o resultado da transação TEF no objeto enviado para integração e geração da NFC-e.
         */
        $scope.processPayment = function(paymentResults, cart, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SalePrivateLabelPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.paymentCodeMethod           = currentPayment.paymentMethod;
            salePayment.paymentValue                = parseFloat(new Number(currentPayment.paymentValue).toFixed(2));

            if (currentPayment.plan) {
                salePayment.paymentCodePlan             = currentPayment.plan.code;
                salePayment.paymentDescriptionPlan      = currentPayment.plan.description;
                salePayment.paymentCodeEvent            = currentPayment.plan.paymentCodeEvent;
                salePayment.paymentPlanExternalCode     = currentPayment.plan.externalCode;
                salePayment.paymentPlanProvider         = currentPayment.plan.provider;
                salePayment.numberOfInstallments        = currentPayment.plan.instalments.length;
                salePayment.dateTofirstInstalment       = currentPayment.plan.firstInstalment.dueDate;
                salePayment.firstInstalmentValue        = new Number(currentPayment.plan.firstInstalment.instalmentValue).toFixed(2);
                salePayment.installments                = [];
                salePayment.discountValue               = 0;   

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { // Convertendo Instalment em InstallmentsDetails
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = new Number(instalment.instalmentValue).toFixed(2);
                    instalmentDetails.instalmentOriginalValue   = new Number(instalment.instalmentOriginalValue ? instalment.instalmentOriginalValue : 0).toFixed(2);
                    instalmentDetails.insuranceValue            = new Number(instalment.insuranceValue ? instalment.insuranceValue : 0).toFixed(2);

                    this.push(instalmentDetails);
                }, salePayment.installments)

                if (currentPayment.plan.discount) {
                    var discount = {}

                    discount.sequenceId         = cart.discounts.length;
                    discount.value              = currentPayment.plan.discount;
                    discount.source             = 'PLAN';

                    salePayment.paymentValue    -= currentPayment.plan.discount; //O desconto deve ser contabilizado no valor do pagamento enviado para o WS.
                    salePayment.discountValue   += currentPayment.plan.discount;

                    salePayment.discountValue = new Number(salePayment.discountValue).toFixed(2);
                    // atualiza valor do pagamento
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));
                    cart.discounts.push(discount);
                }
                if (currentPayment.plan.charges) {
                    cart.additionalCharge       += currentPayment.plan.charges;
                    salePayment.chargeValue     = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }
                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            }

            salePayment.paymentValue                = Math.round(new Number(salePayment.paymentValue) * 100) / 100;
            salePayment.paymentAuthorizationNSU     = paymentResults.nsu;
            salePayment.contractNumber              = paymentResults.nsu;
            salePayment.cardNumber                  = cart.customer.customerId;
            salePayment.customerPaymentVoucher      = paymentResults.customerPaymentVoucher;
            salePayment.bussinessPaymentVoucher     = paymentResults.bussinessPaymentVoucher;
            
            salePayment.installments.sort(function(a,b) {
                return a.paymentSequence - b.paymentSequence;
            });

            cart.salePayments.push(salePayment);
        };

       /**
        * Função chamada por callback. Desfaz os pagamentos.
        */
        $scope.paymentError = function() {
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(CartService.undoCashbackDiscountPartial)
            .then(()=> {
                checkUndoInsuranceAuth();            
                undoPaymentCampaigns();
                CartService.undoFirtsPurchaseDiscount();
                UtilService.navigateWithoutHistory('easyorder.paymentmethods');
            })          
        };
       
       async function undoPaymentCampaigns() {
           const campaigns = PaymentService.currentPayment.campaigns;

           if (campaigns && campaigns.length) {
               campaigns.forEach(campaign => {
                   LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                   CartService.undoCampaignDiscountsPartial(campaign);
               });
           }
       };

       /**
        * 
        */
       var checkUndoInsuranceAuth = () => {
           if (InsuranceService.currentSimulation && CartService.cart.insuranceData.authorizationNumber) {
               LogService.info(`Desfazendo autorização de Seguro: ${CartService.cart.insuranceData.authorizationNumber}`);
               
               InsuranceService.undoInsuranceAuth(CartService.cart).finally(() => {
                   InsuranceService.clearSimutaionData();
                   CartService.cart.insuranceData = {};
               });
           }
       };

        /**
         * Função de inicialização.
         */
        (function() {
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.options = [];
            $scope.nfceAuthResponse = {};
            $scope.transactionInitiated = true;

            $scope.clearDisplay();

            $timeout(function() {
                $scope.displayMessage("Digite a senha para autorização da venda");
            }, 800); //Firula.
            $scope.inputText("Senha do Cartão", $scope.platformNumberInputType, $scope.authPrivateLabelSale, { minSize: 4, maxSize: 15 });
        })();
    });
})();
