class CeaBarcodeHandler extends BarcodeHandler {
    constructor($injector) {
        super($injector);
        this.LogService = $injector.get('LogService');;
        this.ProductService = $injector.get('ProductService');
        this.TagsReaderModal = $injector.get('TagsReaderModal');
        this.LogService.info(`Inicializando CeaBarcodeHandler.`);
    }

    isCustomBarcode(code) {
        return code && code.length === 18;
    }

    extractProductCode(barcode) {
        if (this.isCustomBarcode(barcode)) {
            const tag = new CeaTag(barcode);
            return tag.productCode;
        } else {
            return barcode;
        }
    }

    handleBarcode(barcode) {
        // no caso específico da CEA, sempre que se tem 18 posições o código é interpretado como uma etiqueta
        return new Promise((resolve, reject) => {
                if (!this.isCustomBarcode(barcode)) {
                    // código de tamanho diferente de 18 não é interpretado como etiqueta
                    return resolve(barcode);
                } else {
                    this.LogService.info(`Etiqueta encontrada de código: ${barcode}!`);
                    // Verifica o número de ítens do produto (é usado tanto no fluxo da etiqueta interna quanto do kit)
                    const tag = new CeaTag(barcode);
                    return this.searchProductTotalItemsKit(tag.productCode)
                        .then((totalItemsKit) => {
                            if (totalItemsKit < 1) {
                                reject(`Total de ítens da etiqueta inválido na base: ${totalItemsKit}!`);
                            } else {
                                const tag = new CeaTag(barcode);
                                // se tiver apenas 1 ítem e não for etiqueta externa, já é o ítem
                                if (totalItemsKit === 1 && !tag.isExternal()) {
                                    this.LogService.info(`Tag não é externa e o total de ítens no kit é 1, não será exibido modal.`);
                                    return resolve(tag.productCode);
                                } else {
                                    // caso contrário solicita a leitura da etiqueta interna ou dos ítens do kit
                                    this.TagsReaderModal.open(tag, totalItemsKit)
                                        .then(() => {
                                            // se toda a validação foi efetuada com sucesso (etiqueta interna / kit), devolve o código
                                            return resolve(tag.productCode);
                                        })
                                        .catch((error) => {
                                            return reject(error);
                                        });
                                }
                            }
                        })
                        .catch((error) => {
                            return reject(error);
                        });
            }
        })
    }

    searchProductTotalItemsKit(productCode) {
        return this.ProductService.searchProductAdditionalInfo({
            productCode : productCode
        }).then((productAdditionalInfo) => {
            if (productAdditionalInfo && productAdditionalInfo.additionalInfo['QTD_ITENS_KIT']) {
                const totalItemsKit = parseInt(productAdditionalInfo.additionalInfo['QTD_ITENS_KIT']);
                if (totalItemsKit < 1) {
                    return this.$q.reject(`Total de ítens do produto inválido: ${totalItemsKit}`);
                } else {
                    return this.$q.resolve(totalItemsKit);
                }
            } else {
                this.LogService.warn(`Não foram encontradas informações sobre a quantidade de ítens do produto '${productCode}'! Será assumido o valor 1!`);
                return this.$q.resolve(1);
            }
        }).catch((error) => {
            const message = `Não foi possível obter informações sobre a quantidade de ítens do produto '${productCode}'!`;
            this.LogService.error(message);
            this.LogService.error(JSON.stringify(error));
            return this.$q.reject(message);
        });
        
    }
}
