(function() {
    'use strict';

    angular.module('easyorder.controllers.barcode.camera', [
        'easyorder.services.cart',
        'easyorder.services.barcode',
        'easyorder.services.product',
        'easyorder.services.alert',
        'easyorder.services.log'
    ])

    .controller('ProductSearchCameraCtrl', function($scope,
                                        $ionicLoading,
                                        $ionicPopup,
                                        $ionicSideMenuDelegate,
                                        $timeout,
                                        $rootScope,
                                        CartService,
                                        BarcodeService,
                                        ProductService,
                                        UserService,
                                        ConfigService,
                                        AlertService,
                                        UtilService,
                                        ServiceSaleService,
                                        LogService,
                                        DiscountService,
                                        $q) {
        var promise = null;
        
        $scope.productImageEnabled = ConfigService.config.business.product_images_enabled === true;
        $scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
        $scope.barcodeService = BarcodeService;
        /**
         * Registrando função para o caso de reinicialização do carrinho.
         */
        CartService.initCartObservers.add(function() {
            $scope.reset(true);
        });
         
        $scope.title = () => {
            if (!$scope.firstPartnerCode) {
                return `<span style='font-family: Dosis;font-size: 24px'>${APP_NAME_DESC}</span>`;
            } else {
                return `<span style='font-family: Dosis;font-size: 87%'>Por favor, informe a próxima etiqueta.</span>`;
            }
        };

        /**
         *
         */
        $scope.reset = function(doNotInitCam) {
            $scope.item = {};
            $scope.scannedCode = '';
            $scope.partnership = null;
            $scope.firstPartnerCode = false;
            $scope.previousItem = null;
            $scope.loadingState = false;
            $scope.productDetails = undefined;
            $scope.sellingProduct = false;

            if ($scope.popup) {
                $scope.popup.close();
                $scope.popup = undefined;
            }
            if (!doNotInitCam) {
                BarcodeService.reInitScan();
            }
        };

        /**
         *
         */
        $scope.$on('$destroy', function(){
            if (promise) {
                $timeout.cancel(promise);
            }
        });

        /**
         * A camera deve ser inicializada com esse evento ou ela aparecerá antes da tela ser exibida.
         */
        $scope.$on('$ionicView.enter', function() {
            $scope.init();
            if ($ionicSideMenuDelegate && $ionicSideMenuDelegate.isOpen()) {
                $ionicSideMenuDelegate.toggleLeft(false);
            }
        });

        $scope.init = function(param = '2/44/2/128') {
            $scope.item = {};
            $scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
            $scope.automatic_addition = ConfigService.config.business.enables_automatic_addition_to_cart;

            BarcodeService.init(function success(session) {
                BarcodeService.cancel();
                $scope.loadingState = true;

                $scope.originalCode = session[0];
                var separatorIndex = $scope.originalCode.indexOf('$');
                $scope.digitoVerificador = $scope.originalCode.substring(separatorIndex);
                var partnerStartIndex = $scope.originalCode.indexOf('$');

                $scope.scannedCode = partnerStartIndex > -1 ? $scope.originalCode.substring(0, partnerStartIndex) : $scope.originalCode;

                const barcode = $rootScope.customBarcodeHandler.extractProductCode($scope.scannedCode);
                // manipula o código se necessário (default = retornar o mesmo código)
                ProductService.search({
                    code : barcode
                }, function(result) {
                    $scope.loadingState = false;

                    if (result.items.length > 0) {
                        var saleInfo = result.items[0];

                        $scope.item = saleInfo;
                        if ($scope.item.productType == 'MINIMUM_SALE_UNIT') {
                            $scope.item.quantitySold = $scope.item.minimumSaleUnit;
                        } else {
                            $scope.item.quantitySold = 1;
                        }
                        $scope.firstPartnerCode = false;

                        var partnerProductCode = new PartnerProductCode($scope.originalCode);

                        if (partnerProductCode.isValid()) {
                            //Produto lido foi parceiro...
                            if ($scope.partnership == null) {
                                // Produto anterior não foi parceiro, salva para conferir com a próxima leitura...
                                $scope.firstPartnerCode = true;
                                $scope.partnership = {
                                    lastCode: $scope.originalCode,
                                    partnerCode: partnerProductCode.getReversePartnerCode()
                                };
                                $scope.partnerChecked = false;
                                $scope.previousItem = $scope.item;
                                if($scope.automatic_addition){
                                    $scope.addToCart($scope.item);
                                }else{
                                    $scope.openProductDetails($scope.item);
                                }
                            } else {
                                // O produto anterior é parceiro, verifica se o item atual é o parceiro do anterior...
                                if ($scope.originalCode == $scope.partnership.partnerCode) {
                                    // Fluxo normal, leu os dois parceiros corretamente...
                                    $scope.partnership = null;
                                    $scope.firstPartnerCode = false;
                                    $scope.isPartnerItemReady = true;
                                    $scope.partnerChecked = true;
                                    if($scope.automatic_addition){
                                        $scope.addToCart($scope.item);
                                    }else{
                                        $scope.openProductDetails($scope.item);
                                    }
                                } else if ($scope.originalCode == $scope.partnership.lastCode) {
                                    BarcodeService.cancel();

                                    AlertService.warn('Item parceiro inválido').finally(function() {
                                        $scope.reset(true);
                                        $scope.init();
                                    });
                                } else {
                                    BarcodeService.cancel();
                                    AlertService.warn('Código inválido').finally(function() {
                                        $scope.reset(true);
                                        $scope.init();
                                    });
                                }
                            }
                        } else {
                            if ($scope.partnership != null) {
                                // Leu um parceiro e em seguida um não parceiro...
                                BarcodeService.cancel();

                                AlertService.warn('Código inválido').finally(function() {
                                    $scope.reset(true);
                                    $scope.init();
                                });
                            } else {
                                $rootScope.customBarcodeHandler.handleBarcode($scope.scannedCode)
                                    .then((barcode) => {
                                        if($scope.automatic_addition) {
                                            $scope.addToCart($scope.item);
                                        } else {
                                            $scope.openProductDetails($scope.item);
                                        }
                                    })
                                    .catch((error) => { 
                                        $scope.loadingState = false;
                                        AlertService.error(error).finally(function() {
                                            $scope.reset(true);
                                            $scope.init();
                                        });
                                    });
                            }
                        }
                    } else {
                        AlertService.warn('Produto não encontrado!<br/>Código: <b>' + barcode + '</b>').finally(function() {
                            $scope.reset(true);
                            $scope.init();
                        });
                    }
                }, function(error) {
                    $scope.loadingState = false;
                    AlertService.restError(error, 'Erro ao consultar produtos.').finally(function() {
                        $scope.reset();
                    });
                });
            }, { inSizeCam: [param, '2/64/2/148'], outSizeCam: ['275/44/0/128', '275/64/0/148'] });
            $scope.item = {};
            $scope.loadingState = false;
        };

        var defaultAddItemErrorCallback = function(error) {
            AlertService.error(error.message);
        };

        var defaultAddItemSuccessCallback = function(item) {
            $scope.reset();
        };

        $scope.addToCartImpl = function(item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback) {
            // Adiciona ao carrinho quando o item não é parceiro ($scope.partnership == null) ou
            // caso o item atual seja o parceiro do produto anterior.
            if ($scope.partnership == null || ($scope.partnership != null && $scope.originalCode == $scope.partnership.partnerCode)) {
                CartService.checkInjectItemBatch(item)
                    .then(() => CartService.addItem(item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback));
                return;
            }

            if ($scope.partnership != null) {
                if ($scope.originalCode != $scope.partnership.partnerCode) {
                    // Produto escaneado não foi o parceiro do item anterior. O item parceirado será ignorado.
                    $scope.requestauth = {};
                    if (!$scope.checkPermission()){
                        BarcodeService.cancel();
                        $ionicPopup.show({
                            templateUrl: 'views/requestauth.html',
                            scope: $scope,
                            buttons: [
                                { text: 'Cancelar',
                                    onTap: function(event) {
                                        $scope.reset(true);
                                        $scope.init();
                                    }
                                },
                                {
                                    text: '<b>Autorizar</b>',
                                    type: 'button-theme',
                                    onTap: function(event) {
                                        var sucesso = function(result) {
                                            LogService.info("Authorized: ", result.authorized);

                                            if (result.authorized) {
                                                $scope.firstPartnerCode = false;
                                                $scope.isPartnerItemReady = true;
                                                $scope.openProductDetails( $scope.ItemConfirm);
                                            } else {
                                                BarcodeService.cancel();
                                                AlertService.error('Autorização negada!').finally(function() {
                                                    $scope.reset(true);
                                                    $scope.init();
                                                });
                                            }
                                        };

                                        var erro = function(error) {
                                            LogService.error(error);
                                            BarcodeService.cancel();
                                            AlertService.restError(error, "Erro ao consultar autorização!")
                                                .finally(function() {
                                                    $scope.reset(true);
                                                    $scope.init();
                                                });
                                        };

                                        if (!$scope.requestauth.user) {
                                            BarcodeService.cancel();
                                            AlertService.error('Usuário inválido!').finally(function() {
                                                $scope.reset(true);
                                                $scope.init();
                                        });
                                        } else if (!$scope.requestauth.password) {
                                            BarcodeService.cancel();
                                            AlertService.error('Senha inválida!').finally(function() {
                                                $scope.reset(true);
                                                $scope.init();
                                            });
                                        } else {
                                            UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [INCLOMPLETE_PARTNER_PRODUCT_PERMISSION], sucesso, erro);
                                        }
                                    }
                                }
                            ]
                        });
                    } else {
                        $scope.requestauth.user = UserService.currentUser.username;
                        $scope.firstPartnerCode = false;
                        $scope.isPartnerItemReady = true;
                        $scope.openProductDetails( $scope.ItemConfirm);
                    }
                }
            }
        };

        $scope.getProductImageUrl = function(item) {
            if (item) {
                return ProductService.getProductImageUrl(item);
            } else {
                return null;
            }
        };

        $scope.openProductSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.productsearch');
        };

        $scope.confirm = () => {
            $scope.firstPartnerCode = false;
            // $scope.partnership = undefined;
            BarcodeService.cancel();
            $scope.validateparner = {};
            $scope.validateparner.barCode = $scope.ItemConfirm.productCode;

            if (!$scope.checkPermission()){
                $ionicPopup.show({
                    templateUrl: 'views/validatepartner.html',
                    title: 'Produto Parceirado',
                    scope: $scope,
                    buttons: [
                        {
                            text: 'Cancelar',
                            onTap: (e) => {
                                $scope.init();
                            }
                        },
                        {
                            text: '<b>Validar</b>',
                            type: 'button-theme',
                            onTap: (e) => {
                                // Validações
                                if (!$scope.validateparner.barCode) {
                                    AlertService.warn('Código inválido');
                                    return;
                                }
                                // Precisa de permissão?
                                if ((!$scope.validateparner.dig2 && !$scope.validateparner.dig1) || // Dídigo 1 em branco e dígito 2 em branco
                                    ($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                    ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco

                                    // Se informou ao menos um dígito verificador, deve validar...
                                    if (($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                        ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco
                                        var partnerProductCode = {};

                                        if ($scope.validateparner.dig1) {
                                            partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                + $scope.validateparner.dig1);
                                        } else if ($scope.validateparner.dig2) {
                                            partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                + $scope.validateparner.dig2);
                                        }

                                        if (!partnerProductCode.isValid()) {
                                            AlertService.warn('Item parceiro inválido').then(() => {
                                                $scope.init();
                                            });
                                            return;
                                        }
                                    }
                                    $scope.addToCartImpl($scope.ItemConfirm, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
                                } else {
                                    const partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                        + $scope.validateparner.dig1);
                                    if (partnerProductCode.isValid()) {
                                        var reversePartnerCode = partnerProductCode.getReversePartnerCode();
                                        if (reversePartnerCode != $scope.validateparner.barCode + '$'
                                            + $scope.validateparner.dig2) {
                                            //Código + dig2 não batem com o código do parceiro reverso do dig1
                                            AlertService.warn('Item parceiro inválido').then(() => {
                                                $scope.init();
                                            });
                                        } else {
                                            // Validação OK, pode adicionar ao carrinho
                                            $scope.addToCartImpl($scope.ItemConfirm, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
                                        }
                                    } else {
                                        // Código + dig1 não formam item parceirado
                                        AlertService.warn('Item parceiro inválido').then(() => {
                                            $scope.init();
                                        });
                                    }
                                }
                            }
                        }]
                });
            }else{
                $scope.init();
                $scope.firstPartnerCode = false;
                $scope.isPartnerItemReady = true;
                $scope.openProductDetails( $scope.ItemConfirm);
            }
        };

        /**
         *
         */
        $scope.checkPermission=()=>{
            const permission = UserService.checkPermission(INCLOMPLETE_PARTNER_PRODUCT_PERMISSION);
            LogService.info(permission);
           if(permission){
               return true;
           }else{
               return false;
           }
        };

        $scope.openProductDetails = function(item) {
            if ($scope.firstPartnerCode) {
                $scope.ItemConfirm = item;
                $scope.init('2/44/2/128');
            }else{
                UtilService.navigateWithoutHistory('easyorder.productdetail', { currentItem : item, prevState : 'easyorder.barcode', isPartnerItemReady : $scope.isPartnerItemReady });
            }
        };

        $scope.addToCart = function(item) {
            if (ProductService.shouldProcessPartnerProductOf(item) && !$scope.partnerChecked) {
              $scope.butonOpenFollow = true;
              $scope.searching = false;
              $scope.ItemConfirm = item;
              $scope.init('2/44/2/128');
            } else {
              if (item.uuidInfo && item.uuidInfo.inputRequestType === 'REQUIRED') {
                $scope.butonOpenFollow = false;
                UtilService.navigateWithoutHistory('easyorder.productdetail', { currentItem: item, prevState: 'easyorder.barcode', isPartnerItemReady: $scope.isPartnerItemReady });
              } else {
                $scope.lookNavigation(item);
              }
            }
        };

        $scope.lookNavigation = function(item){
            return additionalServices(item)
                .then(() => {
                    return $ionicLoading.show({
                        template : 'Aguarde...',
                        duration : 0
                    }).then(function() {
                        return ServiceSaleService.fetchServices(item, resp => {
                            $ionicLoading.hide();
                            if (resp.services.length > 0){
                                return UtilService.navigateWithoutHistory('easyorder.servicesale', {
                                    "item": item,
                                    "services": resp.services
                                });
                            }
                        }, error => {
                            LogService.error(error);
                            $ionicLoading.hide();
                            return doAddToCart(item);
                        });
                    });
                })
                .catch(() => {
                    return doAddToCart(item);
                });
        };

        const doAddToCart = (item) => {
            return DiscountService.processAutomaticDiscount(item).then((resp) => {
                if (resp && resp.result && resp.item) {
                    return CartService.doAddToCart(item)
                        .then((cartItem) => UtilService.navigateWithoutHistory('easyorder.productdiscounts', {
                            item: cartItem,
                            automatic: true,
                            discountResult: resp.result,
                            prevState : 'easyorder.barcode'
                        }));
                } else {
                    return CartService.doAddToCart(item).then(() => {
                        $ionicLoading.hide();
                        $scope.reset();
                    });
                }
            }).catch(() => {
                return CartService.doAddToCart(item).then(() => {
                    $ionicLoading.hide();
                    $scope.reset(true);
                    $scope.init();
                });
            });
        };

        const additionalServices = function(item) {
            if (ConfigService.config.business.additional_services) {
                if (item.serviceItemsAvailable) {
                    if (ConfigService.config.business.additional_services && ConfigService.config.business.item_multiplication_enabled) {
                        return $ionicPopup.alert({
                            title: "Aviso",
                            template: "Para efetuar a compra de serviços adicionais será necessário desabilitar o multiplicador de itens.",
                            okType: "button-theme",
                            okText: 'OK'
                        }).then(() => $q.reject());
                    }
                    return $q.resolve();
                } else {
                    return $q.reject();
                }
            } else {
                return $q.reject();
            }
        };

        $scope.typeItems = () => {
            UtilService.navigateWithoutHistory('easyorder.productsearch');
        }
    });
})();
