(function() {
    'use strict';

    angular.module('easyorder.controllers.barcode.scanner', [
        'easyorder.services.cart',
        'easyorder.services.barcode',
        'easyorder.services.product',
        'easyorder.services.alert',
        'easyorder.services.log'
    ])

    .directive('keypressEvents', function ($document) {
        return {
            restrict: 'A',
            link: function (scope) {
                $document.bind('keypress', function (e) {
                    scope.$broadcast('keypress', e, String.fromCharCode(e.which));
                });
            }
        };
    })

    .controller('ProductSearchScannerCtrl', function($scope,
                                                    $rootScope,
                                                    $document,
                                                    $timeout,
                                                    $state,
                                                    $ionicPopup,
                                                    CartService,
                                                    ProductService,
                                                    ConfigService,
                                                    AlertService,
                                                    UtilService,
                                                    LogService,
                                                    UserService,
                                                    DiscountService,
                                                    ServiceSaleService,
                                                    $ionicLoading) {
        var promise = null;
        $scope.searching = false;
        $scope.productImageEnabled = ConfigService.config.business.product_images_enabled === true;
        $scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
        $scope.butonOpenFollow = false;

        /**
         * A camera deve ser inicializada com esse evento ou ela aparecerá antes da tela ser exibida.
         */
        $scope.$on('$ionicView.enter', function() {
            $scope.reset();
            $scope.registeredKeypressHandler = function (e) {
                $scope.$broadcast('keypress', e, String.fromCharCode(e.which));
            };
            $scope.deregisterKeypressListener = $document.on('keypress', $scope.registeredKeypressHandler);
        });

        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        $scope.initScanner = () => {
            if (window.bluebirdBarcodeScanner) {
                bluebirdBarcodeScanner.register((barcode) => {
                    if ($scope.searching) {
                        return;
                    }
                    $scope.searching = true;

                    $scope.scannedCode = barcode;
                    $scope.doSearch();
                }, (argument) => {
                    LogService.warn("Erro ao registrar Scanner: ", argument);
                });
            }
            listenCT60Scanner();
        };

        function listenCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            if ($rootScope.ct60ScannerStatus === 'DISABLED') {
                reEnableCT60Scanner();
                return;
            }

            window.plugins.honeywell.nativeListen(ct60Search, LogService.error);
            $rootScope.ct60ScannerStatus = 'LISTENING';
        }


        function ct60Search(data) {
            if (!$state.is('easyorder.scanner-product-search')) {
                return;
            }

            disableCT60Scanner();
            $scope.scannedCode = data;
            $scope.doSearch();
        }

        function reEnableCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            window.plugins.honeywell.nativeClaim();
            $rootScope.ct60ScannerStatus = 'CLAIMING';
            listenCT60Scanner();
        }

        function disableCT60Scanner() {
            if (!isCT60()) {
                return;
            }

            window.plugins.honeywell.nativeRelease();
            $rootScope.ct60ScannerStatus = 'DISABLED';
        }

        function isCT60() {
            return device.model === 'CT60';
        }

        $scope.initDirectProductSearch = function() {
            UtilService.navigateWithoutHistory('easyorder.productsearch');
        };

        /**
         *
         */
        $scope.$on('$destroy', () => {
            $scope.deregisterKeypressListener = $document.off('keypress', $scope.registeredKeypressHandler);

            if (window.bluebirdBarcodeScanner) {
                bluebirdBarcodeScanner.unregister(() => console.log('BlueBird => unregister'), (argument) => {
                    LogService.warn("Erro ao remover callback do pluign do scanner: ", argument);
                });
            }
        });

        /**
         *
         */
        $scope.$on('keypress', function (evt, obj, key) {
            if (event.keyCode === 13) {
                $scope.doSearch();
            } else {
                $scope.$apply(() => {
                    $scope.scannedCode += key;
                });
            }
        });

        /**
         *
         */
        $scope.doSearch = () => {
            $scope.butonOpenFollow = false;
            $scope.originalCode = $scope.scannedCode;
            var separatorIndex = $scope.originalCode.indexOf('$');
            $scope.digitoVerificador = $scope.originalCode.substring(separatorIndex);
            var partnerStartIndex = $scope.originalCode.indexOf('$');

            $scope.scannedCode = partnerStartIndex > -1 ? $scope.originalCode.substring(0, partnerStartIndex) : $scope.originalCode;
            $scope.loadingState = true;

            ProductService.search({
                code : $scope.scannedCode
            }, function(result) {
                $scope.loadingState = false;

                if (result.items.length > 0) {
                    var saleInfo = result.items[0];

                    $scope.item = saleInfo;
                    if ($scope.item.productType == 'MINIMUM_SALE_UNIT') {
                        $scope.item.quantitySold = $scope.item.minimumSaleUnit;
                    } else {
                        $scope.item.quantitySold = 1;
                    }
                    $scope.scannedCode = '';

                    var partnerProductCode = new PartnerProductCode($scope.originalCode);
                    $scope.automatic_addition = ConfigService.config.business.enables_automatic_addition_to_cart;

                    if (partnerProductCode.isValid()) {
                        //Produto lido foi parceiro...
                        if ($scope.partnership == null) {
                            // Produto anterior não foi parceiro, salva para conferir com a próxima leitura...
                            $scope.partnership = {
                                lastCode: $scope.originalCode,
                                partnerCode: partnerProductCode.getReversePartnerCode()
                            };
                            $scope.previousItem = $scope.item;
                            if($scope.automatic_addition){
                                $scope.addToCart($scope.item);
                            }else{
                                $scope.openProductDetails($scope.item);
                            }
                        } else {
                            // O produto anterior é parceiro, verifica se o item atual é o parceiro do anterior...
                            if ($scope.originalCode == $scope.partnership.partnerCode) {
                                // Fluxo normal, leu os dois parceiros corretamente...
                                $scope.partnership = null;
                                $scope.partnerChecked = true;
                                $scope.isPartnerItemReady = true;
                                if($scope.automatic_addition){
                                    $scope.addToCart($scope.item);
                                }else{
                                    $scope.openProductDetails($scope.item);
                                }
                            } else if ($scope.originalCode == $scope.partnership.lastCode) {
                                AlertService.warn('Item parceiro inválido').finally(function() {
                                    $scope.reset();
                                    reEnableCT60Scanner();
                                });
                            } else {
                                AlertService.warn('Código inválido').finally(function() {
                                    $scope.reset();
                                    reEnableCT60Scanner();
                                });
                            }
                        }
                    } else {
                        if ($scope.partnership != null) {
                            // Leu um parceiro e em seguida um não parceiro...
                            AlertService.warn('Código inválido')
                            .finally(function() {
                                $scope.reset();
                                reEnableCT60Scanner();
                            });
                        } else {
                            if($scope.automatic_addition){
                                $scope.addToCart($scope.item);
                            }else{
                                $scope.openProductDetails($scope.item);
                            }
                        }
                    }
                } else {
                    AlertService.warn('Produto não encontrado!<br/>Código: <b>' + $scope.scannedCode + '</b>').finally(function() {
                        $scope.reset();
                        reEnableCT60Scanner();
                    });
                }
            }, function(error) {
                $scope.loadingState = false;

                AlertService.restError(error, 'Erro ao consultar produtos.').finally(function() {
                    $scope.reset();
                    reEnableCT60Scanner();
                });
            });
        };

        /**
         *
         */
        $scope.reset = function() {
            $scope.item = {};
            $scope.scannedCode = '';
            $scope.partnership = null;
            $scope.previousItem = null;
            $scope.loadingState = false;
            $scope.productDetails = undefined;
            $scope.sellingProduct = false;
            $scope.searching = false;
            $scope.butonOpenFollow = false;

            if ($scope.popup) {
                $scope.popup.close();
                $scope.popup = undefined;
            }
        };

        /**
         * Registrando função para o caso de reinicialização do carrinho.
         */
        CartService.initCartObservers.add(function() {
            $scope.reset();
        });

        /**
         *
         */
        $scope.$on('$destroy', function(){
            if (promise) {
                $timeout.cancel(promise);
            }
        });

        /**
         *
         */
        $scope.confirm = ()=>{
            $scope.validateparner = {};
            $scope.validateparner.barCode = $scope.ItemConfirm.productCode;
            const permission = UserService.checkPermission(INCLOMPLETE_PARTNER_PRODUCT_PERMISSION);
            if (!permission) {
                $ionicPopup.show({
                    templateUrl: 'views/validatepartner.html',
                    title: 'Produto Parceirado',
                    scope: $scope,
                    buttons: [
                        {
                            text: 'Cancelar',
                            onTap: (e) => {
                                $scope.reset();
                                reEnableCT60Scanner();
                            }
                        },
                        {
                            text: '<b>Validar</b>',
                            type: 'button-theme',
                            onTap: (e) => {
                                 // Validações
                                 if (!$scope.validateparner.barCode) {
                                    AlertService.warn('Código inválido');
                                    return;
                                }
                                // Precisa de permissão?
                                if ((!$scope.validateparner.dig2 && !$scope.validateparner.dig1) || // Dídigo 1 em branco e dígito 2 em branco
                                    ($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                    ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco

                                    // Se informou ao menos um dígito verificador, deve validar...
                                    if (($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                        ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco
                                        var partnerProductCode = {};

                                        if ($scope.validateparner.dig1) {
                                            partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                + $scope.validateparner.dig1);
                                        } else if ($scope.validateparner.dig2) {
                                            partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                + $scope.validateparner.dig2);
                                        }

                                        if (!partnerProductCode.isValid()) {
                                            AlertService.warn('Item parceiro inválido').then(() => {
                                                $scope.reset();
                                                reEnableCT60Scanner();
                                            });
                                            return;
                                        }
                                    }
                                    $scope.authorization();
                                } else {
                                    const partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                        + $scope.validateparner.dig1);
                                    if (partnerProductCode.isValid()) {
                                        const reversePartnerCode = partnerProductCode.getReversePartnerCode();
                                        if (reversePartnerCode != $scope.validateparner.barCode + '$'
                                            + $scope.validateparner.dig2) {
                                            //Código + dig2 não batem com o código do parceiro reverso do dig1
                                            AlertService.warn('Item parceiro inválido').then(() => {
                                                $scope.reset();
                                                reEnableCT60Scanner();
                                            });
                                        } else {
                                            // Validação OK, pode adicionar ao carrinho
                                            $scope.authorization();
                                        }
                                    } else {
                                        // Código + dig1 não formam item parceirado
                                        AlertService.warn('Item parceiro inválido').then(() => {
                                            $scope.reset();
                                            reEnableCT60Scanner();
                                        });
                                    }
                                }
                            }
                        }
                    ]
                });
            }else{
                $scope.isPartnerItemReady = true;
                $scope.partnerChecked = true;
                $scope.addToCart($scope.ItemConfirm);
            }
        };

        /**
         * autorização
         */
        $scope.authorization = () =>{
            const permission = UserService.checkPermission(INCLOMPLETE_PARTNER_PRODUCT_PERMISSION);
            LogService.info(permission);
            if (!permission) {
                $scope.reset();
                $scope.requestauth = {};
                const authPopup = $ionicPopup.show({
                    templateUrl: 'views/requestauth.html',
                    scope: $scope,
                    buttons: [
                        {
                            text: 'Cancelar',
                            onTap: function (event) {
                                $scope.reset();
                            }
                        },
                        {
                            text: '<b>Autorizar</b>',
                            type: 'button-theme',
                            onTap: function (event) {
                                var sucesso = function (result) {
                                    LogService.info("Authorized: ", result.authorized);
                                    if (result.authorized) {
                                        $scope.isPartnerItemReady = true;
                                        $scope.partnerChecked = true;
                                        $scope.addToCart($scope.ItemConfirm);
                                    } else {
                                        AlertService.error('Autorização negada!').finally(()=>{
                                            $scope.reset();
                                        });
                                    }
                                };

                                var erro = function (error) {
                                    LogService.error(error);
                                    AlertService.restError(error, "Erro ao consultar autorização!")
                                        .finally(function () {
                                            $scope.reset();
                                        });
                                };

                                if (!$scope.requestauth.user) {
                                    AlertService.error('Usuário inválido!').finally(()=>{
                                        $scope.reset();
                                    });
                                } else if (!$scope.requestauth.password) {
                                    AlertService.error('Senha inválida!').finally(()=>{
                                        $scope.reset();
                                    });
                                } else {
                                    UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [INCLOMPLETE_PARTNER_PRODUCT_PERMISSION], sucesso, erro);
                                }
                            }
                        }
                    ]
                });
            } else {
                $scope.requestauth.user = UserService.currentUser.username;
                CartService.addItem($scope.item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
            }
        };


        /**
         * Exibe o popup com os dados do produto.
         */
        $scope.openProductDetails = function(item) {
            if (ProductService.shouldProcessPartnerProductOf(item) && !$scope.partnerChecked) {
                $scope.butonOpenFollow = true;
                $scope.searching = false;
                $scope.ItemConfirm = item;
                reEnableCT60Scanner();
            } else {
                $scope.butonOpenFollow = false;
                UtilService.navigateWithoutHistory('easyorder.productdetail', { currentItem: item, prevState: 'easyorder.barcode', isPartnerItemReady: $scope.isPartnerItemReady });
            }
        };

        $scope.addToCart = function(item) {
            if (ProductService.shouldProcessPartnerProductOf(item) && !$scope.partnerChecked) {
              $scope.butonOpenFollow = true;
              $scope.searching = false;
              $scope.ItemConfirm = item;
              reEnableCT60Scanner();
            } else {
              if (item.uuidInfo && item.uuidInfo.inputRequestType === 'REQUIRED') {
                $scope.butonOpenFollow = false;
                UtilService.navigateWithoutHistory('easyorder.productdetail', { currentItem: item, prevState: 'easyorder.barcode', isPartnerItemReady: $scope.isPartnerItemReady });
              } else {
                $scope.lookNavigation(item);
              }
            }
        };

        $scope.lookNavigation = function(item){
            if (!additionalServices(item)) {
                DiscountService.processAutomaticDiscount(item).then((resp) => {
                    if (resp && resp.result && resp.item) {
                        CartService.doAddToCart(item).then((cartItem) => UtilService.navigateWithoutHistory('easyorder.productdiscounts', { item: cartItem, automatic: true, discountResult: resp.result, prevState : 'easyorder.barcode' }));
                    } else {
                        CartService.doAddToCart(item).then(() => {
                            $ionicLoading.hide()
                            $scope.reset();
                            reEnableCT60Scanner();
                        });
                    }
                }).catch(() => {
                    CartService.doAddToCart(item).then(() => {
                        $ionicLoading.hide()
                        $scope.reset();
                        reEnableCT60Scanner();
                    });
                });
            } 
        };

        var additionalServices = function(item) {
            if (ConfigService.config.business.additional_services) {
                if (ConfigService.config.business.additional_services && ConfigService.config.business.item_multiplication_enabled) {
                    $ionicPopup.alert({
                        title: "Aviso",
                        template: "Para efetuar a compra de serviços adicionais será necessário desabilitar o multiplicador de itens.",
                        okType: "button-theme",
                        okText: 'OK'
                    });
                    return;
                }
                if (item.serviceItemsAvailable) {
                    return $ionicLoading.show({
                        template : 'Aguarde...',
                        duration : 0
                    }).then(function() {
                        ServiceSaleService.fetchServices(item, resp => {
                            $ionicLoading.hide();

                            if (resp.services.length > 0){
                                UtilService.navigateWithoutHistory('easyorder.servicesale', { "item": item, "services": resp.services });
                            }
                        }, error => {
                            LogService.error(error);
                            $ionicLoading.hide();
                            CartService.doAddToCart(item).then(() => {
                                $ionicLoading.hide()
                                $scope.reset();
                                reEnableCT60Scanner();
                            });
                        });
                    });
                } else {
                    return false;
                }
            } else {
                return false;
            }
        };

        (() => {
            $scope.butonOpenFollow = false;
            document.addEventListener("deviceready", $scope.initScanner); //Registrando função no plugin do scanner.
        })();
    });
})();
