(function() {
    'use strict';

    angular.module('easyorder.controllers.sled.barcode.scanner', [
        'easyorder.services.cart',
        'easyorder.services.barcode',
        'easyorder.services.product',
        'easyorder.services.alert',
        'easyorder.services.log'
    ])
    
    .controller('SledBarcodeScannerCtrl', function($ionicLoading,
                                                AlertService,
                                                BluebirdCaptureService,
                                                UtilService,
                                                LogService,
                                                ProductService,
                                                CartService) {
        /**
         *
         */
        function callbackResult(data) {
            return $ionicLoading.show({
                template:`Consultando '${data.raw}'`,
                duration: 0
            }).then(() => {
                return onSearch(data.raw)
                .then(item => {
                    if (item) {
                        CartService.doAddToCart(item);
                    }
                });
            })
            .finally($ionicLoading.hide);
        }
        /**
         *
         */
        function callbackError(data) {
            LogService.info(`Erro na leitura SLED: ${data.message}`);
        }
        /**
         *
         */
        function onSearch(productId) {
            return ProductService.search({
                code: productId
            })
            .then(result => {
                if (result.items.length > 0) {
                    var saleInfo = result.items[0];
                    
                    if (saleInfo.productType == 'MINIMUM_SALE_UNIT') {
                        saleInfo.quantitySold = saleInfo.minimumSaleUnit;
                    } else {
                        saleInfo.quantitySold = 1;
                    }

                    return saleInfo;
                } else {
                    return $ionicLoading.hide()
                    .then(() => AlertService.warn('Produto não encontrado!<br/>Código: <b>' + productId + '</b>'))
                    .then(() => null);
                }
            })
            .catch(error => {
                return $ionicLoading.hide() 
                .then(() => AlertService.restError(error, `Não foi possível consultar item: ${productId}`))
                .then(() => null);
            });
        }
        /**
         *
         */
        (() => {
            $ionicLoading.show({
                template: 'Conectando ao SLED em modo de código de barras...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.barcodeConnect())
            .then(() => BluebirdCaptureService.startReading(callbackResult, callbackError))
            .finally($ionicLoading.hide)
            .catch(error => {
                let promise;
                
                if (error.errorCode == '-32') {
                   promise = AlertService.warn('Leitor SLED não conectado.');
                } else {
                   promise = AlertService.warn('Erro de conexão do Leitor SLED.');
                }
                
                promise.then(() => UtilService.navigateToMainScreen())
                .finally(BluebirdCaptureService.disconnect);
            });
        })();
    });
})();