(function() {
	'use strict';

     angular.module('easyorder.controllers.product', [
        'easyorder.services.product',
        'easyorder.services.user',
        'easyorder.services.alert',
		'easyorder.components.accordion',
		'easyorder.services.log'
	])

    .controller('ProductCtrl', function($scope,
										$rootScope,
                                        $ionicListDelegate,
                                        $ionicModal,
                                        $ionicPopup,
                                        $ionicLoading,
                                        $sce,
                                        $q,
                                        ProductService,
                                        CartService,
                                        UserService,
                                        ConfigService,
                                        AlertService,
                                        UtilService,
										FreightService,
										DiscountService,
                                        LogService,
										StyleSearchModal) {

		$scope.searchResult = [];
		$scope.productSearchParams = [];
		$scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
		$scope.uuidEnabled = ConfigService.config.business.uuid_enabled;
		$scope.saleByStyleEnabled = ConfigService.config.business.sale_by_style === true;
		$scope.currentItem = {};
		$scope.inventorySearchResult = [];
		$scope.productCodeType = ConfigService.config.business.product_code_character_type;
		$scope.productImageEnabled = ConfigService.config.business.product_images_enabled === true;
		$scope.storeInfoModalShown = false;
		$scope.deliveryAddress = CartService.cart.deliveryAddress ? CartService.cart.deliveryAddress : {};
		$scope.showroomingLabel = ConfigService.config.business.showrooming_label;

		const AUTH_TYPE_ALLOWED = 1;
		const AUTH_TYPE_NOT_ALLOWED = 2;
		const AUTH_TYPE_REQUIRES_PERMISSION = 3;

		$scope.openModal = function() {
			$scope.modal.show();
		};

		$scope.closeModal = function() {
			$ionicListDelegate.closeOptionButtons();
			$scope.modal.hide();
			$scope.searchResult = [];
			$scope.productSearchParams = [];
		};

		// Modal de busca de estoque
		$ionicModal.fromTemplateUrl('views/productinventoryresult.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.inventoryModal = modal;
		});

		$scope.openInventoryModal = function() {
			$scope.inventoryModal.show();
		};

		$scope.closeInventoryModal = function() {
			$ionicListDelegate.closeOptionButtons();
			$scope.inventoryModal.hide();
			$scope.inventorySearchResult = [];
		};

		// Modal de dados da loja
		$ionicModal.fromTemplateUrl('views/storeinfo.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.storeInfoModal = modal;
		});

		$scope.openStoreInfoModal = function(store) {
			$scope.currentStore = store;
			$scope.storeInfoModalShown = true;
			$scope.storeInfoModal.show();
		};

		/**
		 *
		 */
		$scope.storeMapAddrress = function(currentStore) {
		    var ret = 'https://www.google.com/maps/embed/v1/place?key=AIzaSyD241usOdsnkPzXCoIMvyc53ieJ7S90AiI&';
		    ret += 'language=pt&';
		    ret += 'region=BR&';

            if (currentStore && currentStore.address) {
                var address = currentStore.address;

                var addSep = address.streetAddress.indexOf(',');
                addSep =  addSep > 0 ? addSep : address.streetAddress.length;

                ret += 'zoom=15&';
                ret += 'q=' + (address.streetAddress ? address.streetAddress.substring(0, addSep).replace(/\s+/g, '+') : '')
                    		+ (address.streetNumber ? ',' + address.streetNumber.replace(/\s+/g, '+') : '')
                        + (address.neighborhood ? ',' + address.neighborhood.replace(/\s+/g, '+') : '')
                        + (address.postalCode ? ',' + address.postalCode.replace(/\D/g, '').replace(/\s+/g, '+') : '');
                        + (address.city ? ',' + address.city.replace(/\s+/g, '+') : '')
                        + (address.state ? ',' + address.state.replace(/\s+/g, '+') : '');
            } else {
                ret += 'zoom=3&';
                ret += 'q=Brasil';
            }
            return $sce.trustAsResourceUrl(ret);
        };

		$scope.closeStoreInfoModal = function() {
			$ionicListDelegate.closeOptionButtons();
			$scope.storeInfoModalShown = false;
			$scope.storeInfoModal.hide();
		};

		$scope.$on('$destroy', function() {
			// $scope.modal.remove();
			$scope.searchResult = [];
			$scope.inventoryModal.remove();
			$scope.inventorySearchResult = [];
			$scope.currentItem = {};
			$scope.storeInfoModal.remove();
		});
        /**
         *
         */
        $scope.dosummarySearchByDescription = productSearchParams => {
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            })
            .then(() => $scope.loadingState = true)
            .then(() => { return { description: productSearchParams.productDescription }})
            .then(request => ProductService.summarySearch(request))
            .then(result => {
                result.items.forEach(saleInfo => {
					if (saleInfo.productType == 'MINIMUM_SALE_UNIT') {
						saleInfo.quantitySold = saleInfo.minimumSaleUnit;
					} else {
						saleInfo.quantitySold = 1;
					}
                });

                if (result.items.length) {
                    if (result.items.length === 1) {
                        $scope.openProductDetails(result.items[0]);
                    } else {
                        $scope.searchResult = result.items;
                        $scope.openProductSearchResults($scope.searchResult);
                    }
                } else {
					$scope.productSearchParams = [];
                    AlertService.warn('Nenhum produto foi encontrado com o filtro informado!');
                }
            })
            .catch(error => {
				AlertService.restError(error, 'Erro consultando produtos no servidor!');
				LogService.error('Erro consultando produtos no servidor: ', error);
            })
            .finally(() => $scope.loadingState = false)
            .finally($ionicLoading.hide);

        };

		function itemCallback(productCode) {
            LogService.info(`Produto por modelo escolhido: ${productCode}.`);
			$scope.doSearch({ automationCode: productCode }, { soldByStyle: true});
        }

		function requireSaleByStylePermission(styleInfo) {
			$scope.requestauth = {};
			var permission = UserService.checkPermission(SALE_BY_STYLE);

			if (permission) {
				StyleSearchModal.open(styleInfo, itemCallback);
			} else {
				var authPopup = $ionicPopup
					.show({
						templateUrl : 'views/requestauth.html',
						scope : $scope,
						buttons : [
								{
									text : 'Cancelar'
								},
								{
									text : '<b>Autorizar</b>',
									type : 'button-theme',
									onTap : function(event) {
										var sucesso = function(result) {
											if (result.authorized) {
												StyleSearchModal.open(styleInfo, itemCallback);
											} else {
												AlertService.error('Autorização negada!')
											}
										};

										var erro = function(error) {
											AlertService.restError(error,
													"Erro ao consultar autorização!");
										};

										if (!$scope.requestauth.user) {
											AlertService.error('Usuário inválido!');
										} else if (!$scope.requestauth.password) {
											AlertService.error('Senha inválida!');
										} else {
											UserService.checkAuthorization($scope.requestauth.user,
													$scope.requestauth.password,
													[SALE_BY_STYLE],
													sucesso, erro);
										}
									}
								}]
					});
			}
		}
        /**
         *
         */
		$scope.doSearch = function(productSearchParams, params) {
            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(function() {
                var error = function(error) {
                    $scope.loadingState = false;
                    $ionicLoading.hide();

					AlertService.restError(error, 'Erro consultando produtos no servidor!');
					LogService.error('Erro consultando produtos no servidor: ', error);
                };
               var uuidCode;

               if (productSearchParams.automationCode) {
                   uuidCode = productSearchParams.automationCode;
               } else if (productSearchParams.productId) {
                   uuidCode = productSearchParams.productId;

                   var UuidParams = {
                       storeNumber : ConfigService.config.storeNumber,
                       productUuid : uuidCode
                   }

				   /**
					 * Valida se já existe no carrinho um item com o Código Identificador Único informado pelo usuário
					 */
				   const found = CartService.cart.fiscalItems.some(cartItem => cartItem.itemUuid && cartItem.itemUuid.type === 'IMEI' && cartItem.itemUuid.value === UuidParams.productUuid);
				   if (found) {
						$ionicLoading.hide();
					   	AlertService.error("Já existe um item com o código " + " [" + uuidCode + "] no carrinho.");
						return;
				   }
               }

                var productQueryParams = {
                    description : productSearchParams.productDescription,
                    code : uuidCode
                }
                $scope.searchResult = [];

               if (productSearchParams.productId) {
                   ProductService.lookupByUUID(UuidParams, function(result) {
					productQueryParams.code = result.productInventories[0].productCode;
					productQueryParams.itemUuid = { value: result.productInventories[0].uuid };
					$scope.productServiceSearch(productQueryParams, error, { UuidParams: UuidParams });
                   }, error);
				} else if (productSearchParams.styleCode) {
					ProductService.searchStyleInfo({ styleCode: productSearchParams.styleCode}, function(styleInfo) {
						$ionicLoading.hide();
						if (styleInfo) {
							LogService.debug(JSON.stringify(styleInfo));
							const authType = (styleInfo && styleInfo.additionalInfo && styleInfo.additionalInfo['IND_PERMITE_VENDA_MODELO']) ? parseInt(styleInfo.additionalInfo['IND_PERMITE_VENDA_MODELO'].trim()) : 3;
							LogService.info(`Modelo ${productSearchParams.styleCode} (${styleInfo.name}) com autorização tipo ${authType}.`);
							switch (authType) {
								case AUTH_TYPE_NOT_ALLOWED:
									AlertService.warn(`Produto de código '${productSearchParams.styleCode}' não pode ser vendido por modelo!`);
									break;
								case AUTH_TYPE_REQUIRES_PERMISSION:
									requireSaleByStylePermission(styleInfo);
									break;
								case AUTH_TYPE_ALLOWED:
								default:
									StyleSearchModal.open(styleInfo, itemCallback);
									break;
							}
						} else {
							AlertService.error(`Informações de modelo não retornadas!`);
						}
					}, error);
				} else {
					$scope.productServiceSearch(productQueryParams, error, params)
				}
            });
        };

        /**
         *
         */
       $scope.productServiceSearch = function(productQueryParams, error, params) {

			let finalProductQueryParams = null;
				// A C&A manipula etiquetas compostas onde o código do produto é uma parte dela
			if (productQueryParams.code && $rootScope.customBarcodeHandler.isCustomBarcode(productQueryParams.code)) {
				const productCode = $rootScope.customBarcodeHandler.extractProductCode(productQueryParams.code);
				finalProductQueryParams = JSON.parse(JSON.stringify(productQueryParams));
				delete finalProductQueryParams.automationCode;
				finalProductQueryParams.code = productCode;
			} else {
				finalProductQueryParams = productQueryParams;
			}

            ProductService.search(finalProductQueryParams, (result) => {
               $ionicLoading.hide();

               if (result.items.length > 0) {
                   result.items.forEach(function(saleInfo) {
						if (saleInfo.productType == 'MINIMUM_SALE_UNIT') {
							saleInfo.quantitySold = saleInfo.minimumSaleUnit;
						} else {
							saleInfo.quantitySold = 1;
						}

                        if (params) {
						   if (params.UuidParams) {
								saleInfo.itemUuid = {
									type: saleInfo.uuidInfo?saleInfo.uuidInfo.type:'IMEI',
									value: params.UuidParams.productUuid
								}
							}
							if (params.soldByStyle) {
								saleInfo.soldByStyle = params.soldByStyle;
							}
                        }
                   }, this);

                   if (result.items.length === 1) {
					   if (productQueryParams.code) {
						    // trata as especificidades do código, se houverem
							$rootScope.customBarcodeHandler.handleBarcode(productQueryParams.code)
								.then((barcode) => {
									$scope.openProductDetails(result.items[0]);
								})
								.catch((error) => AlertService.error(error));
					   } else {
							$scope.openProductDetails(result.items[0]);
					   }
                   } else {
                       $scope.searchResult = result.items;
                       $scope.openProductSearchResults($scope.searchResult);
                   }
                   $scope.productSearchParams = [];
               } else {
				   AlertService.warn('Nenhum produto foi encontrado com o filtro informado!');
                   $scope.productSearchParams = [];
               }
            }, error);
        };

		/**
		 *
		 */
        $scope.discountScreenPopup = function(item) {
            var error = function(error) {
                $scope.loadingState = false;
                $ionicLoading.hide();
                $scope.productSearchParams.productId = null;
                AlertService.restError(error, 'Erro consultando desconto no servidor!');
            }
            var UuidParams = {
                storeNumber : ConfigService.config.storeNumber,
                productUuid : $scope.productSearchParams.productId
            }
            ProductService.lookupByUUID(UuidParams, function(result) {
                $ionicLoading.hide();

                if (item.productCode == result.productInventories[0].productCode) {
					var discountParams = {
						productCode: item.productCode,
						employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false,
						ownBrand: item.ownBrand,
						companyNumber: ConfigService.config.companyNumber,
						marketingCode: item.marketingCode,
						promotionalPrice: item.promotionalPrice,
						automatic: true,
						unitValue: item.unitValue,
						storeNumber : ConfigService.config.storeNumber
					};

                    DiscountService.searchDiscountItem(discountParams, function(result) {
                        $ionicLoading.hide();
						LogService.info('Resultado do desconto: ', result);

						if (result && result.discountRuleTypes[0].discountRules.length > 0) {
						    $scope.addProductToCart(item).then((cartItem) => $scope.discountPopup(result, cartItem));
							$scope.productSearchParams.productId = null;
						} else {
							$scope.addProductToCart(item);
							$scope.productSearchParams.productId = null;
						}
                    }, error);
                } else {
                    $scope.loadingState = false;
                    $ionicLoading.hide();
                    $scope.productSearchParams.productId = null;

                    AlertService.restError(null, 'Identificador Único especificado não pertence ao produto selecionado');
                }
            }, error);
        };

		$scope.addToCart = function(index, item) {
			if (ProductService.shouldProcessPartnerProductOf(item)) {
				$scope.addPatnerToCart(index, item);
			} else if (ConfigService.config.business.additional_services) {
                if (ConfigService.config.business.additional_services && ConfigService.config.business.item_multiplication_enabled) {
                    $ionicPopup.alert({
                        title: "Aviso",
                        template: "Para efetuar a compra de serviços adicionais será necessário desabilitar o multiplicador de itens.",
                        okType: "button-theme",
                        okText: 'OK'
                    });
                    return;
                }

                return $ionicLoading.show({
					template : 'Aguarde...',
					duration : 0
				}).then(function() {
					ServiceSaleService.fetchServices(item,
					resp => {
						$ionicLoading.hide();
						if(resp.services.length > 0){
							UtilService.navigateWithoutHistory('easyorder.servicesale', { "item": item, "services": resp.services });
						}
					}, error => {
						LogService.error(error);
						$ionicLoading.hide();
						$scope.doAddToCart(item);
					});
                });
            } else {
				$scope.doAddToCart(item);
			}
		};

		$scope.doAddToCart = function(item) {
            if ($scope.productSearchParams.productId) {
                $scope.discountScreenPopup(item);
            } else {
                if (item.uuidInfo) {
					if (item.uuidInfo.inputRequestType == 'REQUIRED') {
						$scope.uuidPopup(item);
					} else {
						$scope.addProductToCart(item);
					}
				} else{
					$scope.addProductToCart(item);
				}
            };
        };

        function reserve(item) {
            return $q((resolve, reject) => {
                if (ConfigService.config.business.enable_reserve_products
                    && (item.uuidInfo && item.uuidInfo.inputRequestType == 'REQUIRED')) {

                    const reserveParams = {
                        storeNumber : ConfigService.config.storeNumber,
                        orderNumber: CartService.sessionId,
                        terminalNumber: ConfigService.config.terminalNumber,
                        operatorName: UserService.currentUser.name,
                        products: [
                            {
                                productCode: item.productCode,
                                uuid: item.automationCode,
                                quantity: 1
                            }
                        ]
                    };

                    ProductService
                        .reserveProduct(reserveParams)
                        .then(() => {
                            $scope.closeModal();
                            resolve();
                        }).catch(err => {
                            reject(err);
                        });
                } else {
                    resolve();
                }
            });
        }

        $scope.addProductToCart = (item) => {
            var defered = $q.defer();

            reserve(item).then(() => {
                const errorCallback = (error) => {
                    AlertService.error(error.message);
                    defered.reject(error);
                };

                if ('GROUP' === item.productType || 'BUNDLE' === item.productType) {
                    $ionicLoading.show({
                        template : 'Consultando itens. Aguarde...',
                        duration : 0
                    }).then(() => {
                        ProductService.search({
                            groupCode : item.productCode
                        }, (result) => {
                            if (result.items.length > 0) {
                                const groupCode = ConfigService.config.business.item_multiplication_enabled ? item.productCode : new Date().getTime(); // Com itens mutiplicados, os grupos devem se sobrepor.
                                const quantitySold = item.quantitySold
                                var cartItem;

                                result.items.forEach(function(itemResult, index) {
                                    itemResult.groupCode = groupCode;
                                    itemResult.quantitySold = quantitySold;

                                    if (itemResult.productCode === item.productCode) {
                                        itemResult.itemUuid = item.itemUuid;
                                        CartService.addItem(itemResult, item => cartItem = item, errorCallback);
                                    } else {
                                        CartService.addItem(itemResult, null, errorCallback);
                                    }
                                });
                                defered.resolve(cartItem);
                            }
                        }, errorCallback).finally($ionicLoading.hide);
                    });
                } else {
                    CartService.checkInjectItemBatch(itemResult)
						.then(() => CartService.addItem(item, defered.resolve, errorCallback));
                }
            }).catch(errorCallback);

            return defered.promise;
        };

        $scope.uuidPopup = function(item) {
            $scope.data = {};

            $ionicPopup.show({
                template: '<input type="text" ng-model="productSearchParams.productId">',
                title: 'Identificador único',
                scope: $scope,
                buttons: [
                    {
                        text: 'Cancelar',
                        onTap: function() {
                            $scope.productSearchParams.productId = null;
                        }
                    }, {
                        text: '<b>OK</b>',
                        type: 'button-theme',
                        onTap: function(e) {
                            if (!$scope.productSearchParams.productId) {
                                //don't allow the user to close unless he enters uuid
                                e.preventDefault();
                            } else {
                                $scope.discountScreenPopup(item);
                                return $scope.productSearchParams.productId;
                            }
                        }
                    }
                ]
            });
        };

        $scope.selectDiscount = function(item) {
            $scope.selectedDiscount = item;
            $scope.isSelectedDiscount = true;
            $scope.discountAccordionOpen = false;

            if (item.hasOwnProperty('percentage')) {
                $scope.percentageDiscount = item.percentage.max;
            } else {
                $scope.percentageDiscount = null;
            }
        };

        $scope.discountPopup = function(result, item) {
				UtilService.navigateWithoutHistory('easyorder.productdiscounts', { item : item, discountResult: result, prevState : 'easyorder.productsearch'});
        };

		$scope.addPatnerToCart = function(index, item){
			// consultou um produto parceirado
			$scope.validateparner = {};
			$scope.validateparner.barCode = item.productCode;
			$ionicPopup.show({
				templateUrl : 'views/validatepartner.html',
				title : 'Produto Parceirado',
				scope : $scope,
				buttons : [
						{ text : 'Cancelar' },
						{
							text : '<b>Validar</b>',
							type : 'button-theme',
							onTap : function(e) {
								// Validações
								if (!$scope.validateparner.barCode) {
									AlertService.warn('Código inválido');
									return;
								}

								// Precisa de permissão?
								if ((!$scope.validateparner.dig2 && !$scope.validateparner.dig1) || // Dídigo 1 em branco e dígito 2 em branco
										($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
										($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco

									// Se informou ao menos um dígito verificador, deve validar...
									if (($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
											($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco
										var partnerProductCode
										if ($scope.validateparner.dig1) {
											partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
													+ $scope.validateparner.dig1);
										} else if ($scope.validateparner.dig2) {
											partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
													+ $scope.validateparner.dig2);
										}
										if (!partnerProductCode.isValid()) {
											AlertService.warn('Item parceiro inválido');
											return;
										}
									}

									$scope.requestauth = {};
									var permission = UserService.checkPermission(INCLOMPLETE_PARTNER_PRODUCT_PERMISSION);
									LogService.info(permission);

									if (permission) {
										$scope.doAddToCart(item);
									} else {
										var authPopup = $ionicPopup
											.show({
												templateUrl : 'views/requestauth.html',
												scope : $scope,
												buttons : [
														{
															text : 'Cancelar'
														},
														{
															text : '<b>Autorizar</b>',
															type : 'button-theme',
															onTap : function(event) {
																var sucesso = function(result) {
																	if (result.authorized) {
																		$scope.doAddToCart(item);
																	} else {
																		AlertService.error('Autorização negada!')
																	}
																};

																var erro = function(error) {
																	BarcodeService.cancel();
																	AlertService.restError(error,
																			"Erro ao consultar autorização!");
																};

																if (!$scope.requestauth.user) {
																	BarcodeService.cancel();
																	AlertService.error('Usuário inválido!');
																} else if (!$scope.requestauth.password) {
																	BarcodeService.cancel();
																	AlertService.error('Senha inválida!');
																} else {
																	UserService.checkAuthorization($scope.requestauth.user,
																			$scope.requestauth.password,
																			[INCLOMPLETE_PARTNER_PRODUCT_PERMISSION],
																			sucesso, erro);
																}
															}
														}]
											});
									}
								} else {
									var partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
											+ $scope.validateparner.dig1);
									if (partnerProductCode.isValid()) {
										var reversePartnerCode = partnerProductCode.getReversePartnerCode();
										if (reversePartnerCode != $scope.validateparner.barCode + '$'
												+ $scope.validateparner.dig2) {
											//Código + dig2 não batem com o código do parceiro reverso do dig1
											AlertService.warn('Item parceiro inválido');
										} else {
											// Validação OK, pode adicionar ao carrinho
											$scope.doAddToCart(item);
										}
									} else {
										// Código + dig1 não formam item parceirado
										AlertService.warn('Item parceiro inválido');
									}
								}
							}
						}]
					});
		};

		$scope.validateFields = function(productSearchParams) {
			if (productSearchParams.automationCode || productSearchParams.productId || productSearchParams.styleCode || productSearchParams.productDescription && productSearchParams.productDescription.length >= 3) {
				return false;
			}
			return true;
		};

		$scope.addExternalItemToCart = function(inventoryItem) {
			var successCallback = function(item) {
				$scope.closeInventoryModal();
				$scope.closeModal();
			};

			var errorCallback = function(error) {
				AlertService.error(error.message);
			};
			// TODO O CartService precisa ser reestruturado.
			var added = true;
			for (var i = 0; added === true && i < inventoryItem.quantitySold; i++) {
				var item = JSON.parse(JSON.stringify($scope.currentItem));

				if (item.productType == 'MINIMUM_SALE_UNIT') {
					item.quantitySold = item.minimumSaleUnit || 1;
				} else {
					item.quantitySold = 1;
				}

				added = CartService.addItem(item, successCallback, errorCallback, inventoryItem.store.number);
			}
		}

        $scope.lookupInventory = function(item) {
        	    $scope.currentItem = item;
        	    $scope.currentDestinationStoreNumber = CartService.cart.destinationStoreNumber;
        	    $ionicLoading.show({
				template : 'Consultando estoque...',
				duration : 0
			}).then(function() {
				var error = function(error) {
					$scope.loadingState = false;
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro consultando estoque de produto no servidor!');
					$ionicListDelegate.closeOptionButtons();
				};

				var productInventoryLookupParams = {
					companyNumber : ConfigService.config.companyNumber,
					storeNumber : ConfigService.config.storeNumber,
					productCode : item.productCode
				};
				$scope.inventorySearchResult = [];

				ProductService.lookupInventory(productInventoryLookupParams,
					function(result) {
						$ionicLoading.hide();

						if (result.productInventories.length > 0) {
							result.productInventories.forEach(function(productInventory) {
								if (productInventory && productInventory.availableQuantity && productInventory.store && productInventory.store.number ) {
									var tmpInventoryItem = productInventory;

									if (item.productType == 'MINIMUM_SALE_UNIT') {
										tmpInventoryItem.quantitySold = item.minimumSaleUnit;
									} else {
										tmpInventoryItem.quantitySold = 1;
									}

									$scope.inventorySearchResult.push(tmpInventoryItem);
								} else {
									LogService.warn('Estoque retornado inválido, não será adicionado: ', JSON.stringify(productInventory))
								}
							}, this);
							$scope.inventoryModal.show();
						} else {
							error({
								data : {
									message : 'Estoque não encontrado!'
								}
							});
						}
					}, error);
			});
        };

        $scope.getProductImageUrl = function(item) {
            if (item){
                return ProductService.getProductImageUrl(item);
            }
            return null;
        };

        $scope.showrooming = function(item){
        	if (item.weight && item.weight > 0) {
            	// se não existirem ítens de 'showrooming' no carrinho, solicita a confirmação do CEP
                if (!CartService.hasShowrooming()) {
                    UtilService.navigateWithoutHistory('easyorder.showroomingzipcode', { currentItem : item, prevState : "easyorder.productsearch" });
                } else {
                	// se existem ítens de showrooming
                	searchItemFreight(item);
                }
        	} else {
        		AlertService.error('Showrooming não pode ser efetuado pois o item possui o peso inválido!');
        	}

        };

        const searchItemFreight = (item) => {
	          var cart = CartService.cart;
	      	  var isPickupInStore =  cart.showroomingInfo.searchOption === 'pickup_in_store';
	      	  var loadingMessage;
	      	  var zipcode;
	      	  if (isPickupInStore) {
	      		  loadingMessage = "Consultando lojas disponíveis para retirada do produto '" + item.productDescription + "'...";
	      	  } else {
	      		  zipcode = $scope.cart.deliveryAddress.zipCode;
	      		  loadingMessage = "Consultando opções de entrega para o CEP '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'...";
	      	  }

	          $ionicLoading.show({
	              template :  loadingMessage,
	              duration : 0
	          }).then(function() {
	              var error = function(error) {
	                  $scope.loadingState = false;
	                  $ionicLoading.hide();
	                  AlertService.restError(error, 'Erro ao consultar opções de entrega!');
	              };

	              var success = function(freights) {
	                  $scope.loadingState = false;
	                  $ionicLoading.hide();

	                  var hasShowrooming = false;

	                  if (freights && freights.length > 0)  {
                        for (const f of freights) {
                            if (f && f.groups && f.groups.length > 0) {
                                var group = f.groups[0];
                                if  (group.pickups && group.pickups.length > 0) {
                                    hasShowrooming = true;
                                }
                                // se for apenas pickup in store, não considera os shipments. E remove-os para não serem expostos e escolhidos.
                                if (isPickupInStore) {
                                    group.shipments = [];
                                } else {
                                    if (group.shipments && group.shipments.length > 0) {
                                        hasShowrooming = true;
                                    }
                                }
                            }
                        }
                    }
	                  if (hasShowrooming) {
	                      UtilService.navigateWithoutHistory('easyorder.showroomingitemfreightdetails', { currentItem : item, freightQuote: freights});
	                  } else {
	                      if (isPickupInStore) {
	                          AlertService.info("Não foram encontradas lojas próximas disponíveis para entregar o produto: '" + item.productDescription + "'.")
	                      } else {
	                          AlertService.info("Nenhuma opção de entrega disponível para o produto: '" + item.productDescription + "', no CEP: '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'.")
	                      }
	                  }
	              }

                  const orderValue = MathUtils.add(
                    MathUtils.subtract(cart.totalValue, cart.totalDiscount),
                    MathUtils.multiply(item.unitValue, item.quantitySold)
                  );

	              if (isPickupInStore) {
	                  FreightService.searchSingleItemByStore(cart.companyNumber, cart.storeNumber, orderValue, item, success, error);
	              } else {
	                  FreightService.searchSingleItemByZipcode(cart.companyNumber, cart.storeNumber, zipcode, orderValue, item, success, error);
	              }
	          });
        };

        $scope.reset = function() {
            if ($scope.popup) {
                $scope.popup.close();
                $scope.popup = undefined;
            }
        };

        $scope.openSearchBarcode = function() {
            UtilService.navigateWithoutHistory('easyorder.barcode');
        };

        $scope.openProductDetails = function(item) {
            UtilService.navigateWithoutHistory('easyorder.productdetail', { currentItem : item, productSearchParams: $scope.productSearchParams, prevState: 'easyorder.productsearch'});
        };

        $scope.openProductSearchResults = function(items) {
            UtilService.navigateWithoutHistory('easyorder.productsearchresult', { searchResult : items, productSearchParams: $scope.productSearchParams, prevState: 'easyorder.productsearch'});
        };
	});
})();
