
(function () {
    'use strict';

    angular.module('easyorder.controllers.productdetail', [
        'easyorder.services.product',
        'easyorder.services.user',
        'easyorder.services.alert',
        'easyorder.services.log'
    ])

        .controller('ProductDetailCtrl', function ($scope,
            $ionicListDelegate,
            $ionicPopup,
            $ionicLoading,
            $ionicModal,
            $stateParams,
            $q,
            ProductService,
            CartService,
            UserService,
            ConfigService,
            AlertService,
            FreightService,
            ServiceSaleService,
            DiscountService,
            UtilService,
            LogService,
            BarcodeService,
            $rootScope) {

            $scope.productCodeType = ConfigService.config.business.product_code_character_type;
            $scope.item = $stateParams.currentItem;
            $scope.prevState = $stateParams.prevState;
            $scope.productImageEnabled = ConfigService.config.business.product_images_enabled === true;
            $scope.cart = CartService.cart;
            $scope.isPartnerItemReady = $stateParams.isPartnerItemReady;
            $scope.itemSelected = null;
            $scope.openScan = false;
            $scope.showroomingLabel = ConfigService.config.business.showrooming_label;

            // Modal de busca de estoque
            $ionicModal.fromTemplateUrl('views/productinventoryresult.html', {
                scope: $scope,
                animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.inventoryModal = modal;
            });

            $scope.openInventoryModal = function () {
                $scope.inventoryModal.show();
            };

            $scope.closeInventoryModal = function () {
                $ionicListDelegate.closeOptionButtons();
                $scope.inventoryModal.hide();
                $scope.inventorySearchResult = [];
            };

            $scope.addExternalItemToCart = function (inventoryItem) {
                var successCallback = function (item) {
                    $scope.closeInventoryModal();
                    UtilService.navigateWithoutHistory($scope.prevState);
                };

                var errorCallback = function (error) {
                    AlertService.error(error.message);
                };
                // TODO O CartService precisa ser reestruturado.
                var added = true;
                for (var i = 0; added === true && i < inventoryItem.quantitySold; i++) {
                    var item = JSON.parse(JSON.stringify($scope.currentItem));

                    if (item.productType == 'MINIMUM_SALE_UNIT') {
                        item.quantitySold = item.minimumSaleUnit || 1;
                    } else {
                        item.quantitySold = 1;
                    }

                    added = CartService.addItem(item, successCallback, errorCallback, inventoryItem.store.number);
                }
            };

            $scope.listConditions = [];
            $scope.showrooming = function (item) {
                if (item.weight && item.weight > 0) {
                    // se não existirem ítens de 'showrooming' no carrinho, solicita a confirmação do CEP
                    if (!CartService.hasShowrooming()) {
                        UtilService.navigateWithoutHistory('easyorder.showroomingzipcode', { currentItem: item, prevState: "easyorder.productdetail" });
                    } else {
                        // se existem ítens de showrooming
                        searchItemFreight(item);
                    }
                } else {
                    AlertService.error('Showrooming não pode ser efetuado pois o item possui o peso inválido!');
                }
            };

            /**
             *
             */
            const searchItemFreight = (item) => {
                var cart = CartService.cart;
                var isPickupInStore = cart.showroomingInfo.searchOption === 'pickup_in_store';
                var loadingMessage;
                var zipcode;
                if (isPickupInStore) {
                    loadingMessage = "Consultando lojas disponíveis para retirada do produto: '" + item.productDescription + "'...";
                } else {
                    zipcode = $scope.cart.deliveryAddress.zipCode;
                    loadingMessage = "Consultando opções de entrega para o CEP: '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'...";
                }

                $ionicLoading.show({
                    template: loadingMessage,
                    duration: 0
                }).then(function () {
                    var error = function (error) {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Erro ao consultar opções de entrega!');
                    };
                    var success = function (freights) {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                        var hasShowrooming = false;

                        if (freights && freights.length > 0) {
                            for (const f of freights) {
                                if (f && f.groups && f.groups.length > 0) {
                                    var group = f.groups[0];
                                    if (group.pickups && group.pickups.length > 0) {
                                        hasShowrooming = true;
                                    }
                                    // se for apenas pickup in store, não considera os shipments. E remove-os para não serem expostos e escolhidos.
                                    if (isPickupInStore) {
                                        group.shipments = [];
                                    } else {
                                        if (group.shipments && group.shipments.length > 0) {
                                            hasShowrooming = true;
                                        }
                                    }
                                }
                            }
                        }
                        if (hasShowrooming) {
                            UtilService.navigateWithoutHistory('easyorder.showroomingitemfreightdetails', { currentItem: item, freightQuote: freights });
                        } else {
                            if (isPickupInStore) {
                                AlertService.info("Não foram encontradas lojas próximas disponíveis para entregar o produto: '" + item.productDescription + "'.")
                            } else {
                                AlertService.info("Nenhuma opção de entrega disponível para o produto: '" + item.productDescription + "', no CEP: '" + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + "'.")
                            }
                        }
                    }

                    const orderValue = MathUtils.add(
                        MathUtils.subtract(cart.totalValue, cart.totalDiscount),
                        MathUtils.multiply(item.unitValue, item.quantitySold)
                    );

                    if (isPickupInStore) {
                        FreightService.searchSingleItemByStore(cart.companyNumber, cart.storeNumber, orderValue, item, success, error);
                    } else {
                        FreightService.searchSingleItemByZipcode(cart.companyNumber, cart.storeNumber, zipcode, orderValue, item, success, error);
                    }
                });
            };

            $scope.getProductImageUrl = function (item) {
                if (item) {
                    return ProductService.getProductImageUrl(item);
                }
                return null;
            };


            $scope.getItemDisplayCode = (item) => {
                if (!item) return '';

                if (ConfigService.config.business.product_code_display_type == 'EXTERNAL_CODE') {
                    return item.externalCode;
                } else if (ConfigService.config.business.product_code_display_type == 'AUTOMATION_CODE') {
                    return item.automationCode;
                } else {
                    return item.productCode;
                }
            };

            /**
             *
             */
            $scope.lookupInventory = function (item) {
                $scope.currentItem = item;
                $scope.currentDestinationStoreNumber = CartService.hasItems() ? CartService.cart.destinationStoreNumber : null;
                $ionicLoading.show({
                    template: 'Consultando estoque...',
                    duration: 0
                }).then(function () {
                    var error = function (error) {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Erro consultando estoque de produto no servidor!');
                        $ionicListDelegate.closeOptionButtons();
                    };

                    var productInventoryLookupParams = {
                        companyNumber: ConfigService.config.companyNumber,
                        storeNumber: ConfigService.config.storeNumber,
                        productCode: item.productCode
                    };
                    $scope.inventorySearchResult = [];

                    ProductService.lookupInventory(productInventoryLookupParams,
                        function (result) {
                            $ionicLoading.hide();

                            if (result.productInventories.length > 0) {
                                result.productInventories.forEach(function (productInventory) {
                                    if (productInventory && productInventory.availableQuantity && productInventory.store && productInventory.store.number) {
                                        var tmpInventoryItem = productInventory;

                                        if (item.productType == 'MINIMUM_SALE_UNIT') {
                                            tmpInventoryItem.quantitySold = item.minimumSaleUnit;
                                        } else {
                                            tmpInventoryItem.quantitySold = 1;
                                        }

                                        $scope.inventorySearchResult.push(tmpInventoryItem);
                                    } else {
                                        LogService.warn('Estoque retornado inválido e não será adicionado: ', JSON.stringify(productInventory))
                                    }
                                }, this);
                                $scope.openInventoryModal();
                            } else {
                                error({
                                    data: {
                                        message: 'Estoque não encontrado!'
                                    }
                                });
                            }
                        }, error);
                });
            };

            /**
             *
             */
            var additionalServices = function (item) {
                if (ConfigService.config.business.additional_services) {
                    if (ConfigService.config.business.additional_services && ConfigService.config.business.item_multiplication_enabled) {
                        $ionicPopup.alert({
                            title: "Aviso",
                            template: "Para efetuar a compra de serviços adicionais será necessário desabilitar o multiplicador de itens.",
                            okType: "button-theme",
                            okText: 'OK'
                        });
                        return;
                    }
                    if (item.serviceItemsAvailable) {
                        return $ionicLoading.show({
                            template: 'Aguarde...',
                            duration: 0
                        }).then(function () {
                            ServiceSaleService.fetchServices(item, resp => {
                                $ionicLoading.hide();

                                if (resp.services.length > 0) {
                                    UtilService.navigateWithoutHistory('easyorder.servicesale', { "item": item, "services": resp.services });
                                }
                            }, error => {
                                LogService.error(error);
                                $ionicLoading.hide();
                                CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                            });
                        });
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            };

            /**
             *
             */
            $scope.addToCart = function (index, item) {

                if (item.restrictions && item.restrictions.find(e => e.type != null && e.type == 'DOF')) {
                    if (!CartService.cart.customer) {
                        AlertService.warn("Produto de Origem Florestal. Identificação do cliente obrigatória.");
                        return;
                    }
                }

                if (item.restrictions && item.restrictions.find(e => e.type != null && e.type == 'SALE_RESTRICTION')) {
                    if (!CartService.cart.customer) {
                        AlertService.warn("VENDA PROIBIDA PARA MENORES DE 18 ANOS. Identificação do cliente OBRIGATÓRIA");
                        return;
                    }
                }

                let promise = $q.resolve(item);
                // regra adicionada do ProductSearchResultCtrl.js
                if (!item.productType) { // Os dados estão resumidos?
                    promise = $ionicLoading.show()
                        .then(() => ProductService.search({ code: item.productCode }))
                        .then(result => result.items[0])
                        .then(itemSrc => itemSrc ? itemSrc : $q.reject({ data: { message: 'Nenhum produto encontrado' } }))
                        .then(itemSrc => {
                            itemSrc.quantitySold = item.quantitySold;
                            return itemSrc;
                        })
                        .finally($ionicLoading.hide);
                }
                promise.then(itemSrc => {
                    item = itemSrc;

                    if (ProductService.shouldProcessPartnerProductOf(item)) {
                        $scope.addPatnerToCart(index, item);
                    } else {
                        if (item.uuidInfo && item.uuidInfo.inputRequestType === 'REQUIRED') {
                            $scope.itemSelected = item;
                            $scope.uuidPopup(item.uuidInfo.type).then((uuid) => {
                                if (uuid) {
                                    item.itemUuid = {
                                        type: item.uuidInfo.type,
                                        value: uuid
                                    };

                                    /**
                                     * Valida se já existe no carrinho um item com o Código Identificador Único informado pelo usuário
                                     */
                                    let finded = CartService.cart.fiscalItems.find(cartItem => cartItem.itemUuid && cartItem.itemUuid.type === item.itemUuid.type && cartItem.itemUuid.value === item.itemUuid.value);
                                    if (finded) {
                                        AlertService.error("Já existe um item com o código " + item.itemUuid.type + " [" + uuid + "] no carrinho.");
                                        return;
                                    }

                                    $ionicLoading.show({
                                        template: 'Aguarde...',
                                        duration: 0
                                    }).then(function () {
                                        ProductService.search({ code: item.productCode, itemUuid: item.itemUuid }).then((result) => {
                                            $ionicLoading.hide();

                                            if (result.items.length && result.items[0].productCode === item.productCode) {
                                                result.items[0].quantitySold = item.quantitySold
                                                result.items[0].itemUuid = {
                                                    type: item.uuidInfo.type,
                                                    value: uuid
                                                };
                                                item = result.items[0];

                                                if (ConfigService.config.business.enable_reserve_products) {
                                                    $scope.processInventoryLookupByUUID(item).then(() => {
                                                        DiscountService.processAutomaticDiscount(item).then((resp) => {
                                                            if (resp && resp.result && resp.item) {
                                                                $scope.processInventoryReservation(item).then(() => {
                                                                    CartService.doAddToCart(item).then((cartItem) => $scope.discountState(resp.result, cartItem));
                                                                });
                                                            } else {
                                                                $scope.processInventoryReservation(item).then(() => {
                                                                    CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                                                                });
                                                            }
                                                        }).catch(() => {
                                                            $scope.processInventoryReservation(item).then(() => {
                                                                CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                                                            });
                                                        });
                                                    });
                                                } else {
                                                    $scope.lookNavigation(item);
                                                }
                                            } else {
                                                AlertService.error("Código de item não localizado.[" + uuid + "]");
                                            }
                                        }, (error) => {
                                            $ionicLoading.hide();
                                            AlertService.restError(error, 'Erro consultando Produto por Identificador Único!');
                                        });
                                    });
                                }
                            });
                        } else {
                            $scope.lookNavigation(item);
                        }
                    }
                })
                    .catch(error => {
                        AlertService.warn('Não foi possível inserir produto no carrinho.')
                            .then(() => {
                                if (error.data) {
                                    LogService.error(error.data.message);
                                }
                            });
                    });

            };

            /**
             *
             */
            $scope.processInventoryReservation = (item) => {
                var reserveParams = {
                    storeNumber: ConfigService.config.storeNumber,
                    orderNumber: CartService.sessionId,
                    terminalNumber: ConfigService.config.terminalNumber,
                    operatorName: UserService.currentUser.name,
                    products: [{
                        productCode: item.productCode,
                        uuid: item.itemUuid.value,
                        quantity: item.quantitySold
                    }]
                };

                $scope.loadingState = true;

                return $ionicLoading.show({
                    template: 'Realizando reserva no estoque...',
                    duration: 0
                }).then(function () {
                    const defer = $q.defer();

                    ProductService.reserveProduct(reserveParams).then(() => {
                        defer.resolve();
                    }, (error) => {
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Erro ao reservar produto!');
                        defer.reject();
                    }).finally(() => {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                    });

                    return defer.promise;
                });
            };

            /**
             *
             */
            $scope.lookNavigation = function (item) {
                if (!additionalServices(item)) {
                    DiscountService.processAutomaticDiscount(item).then((resp) => {
                        if (resp && resp.result && resp.item) {
                            CartService.doAddToCart(item).then((cartItem) => $scope.discountState(resp.result, cartItem));
                        } else {
                            CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                        }
                    }).catch(() => {
                        CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                    });
                }
            };

            /**
             *
             */
            $scope.uuidPopup = (uuidType) => {
                $scope.uuidInfo = (() => {
                    if (!$scope.productSearchParams
                        || !$scope.productSearchParams.productId
                        || !$scope.productSearchParams.productId.length) {
                        return {};
                    } else {
                        return {
                            uuid: $scope.productSearchParams.productId
                        };
                    }
                })();

                $scope.popupUuid = $ionicPopup.show({
                    templateUrl: "views/popupSearchByUUI.html",
                    title: 'Identificador único',
                    scope: $scope,
                    buttons: [
                        {
                            text: 'Cancelar',
                            onTap: function () {
                                $scope.openScan = false;
                                BarcodeService.cancel();
                            }
                        }, {
                            text: '<b>OK</b>',
                            type: 'button-theme',
                            onTap: function (e) {
                                if (ConfigService.config.business.enable_imei_iccid_validation
                                    && ((uuidType === 'IMEI' && !UtilService.checkImeiNumber($scope.uuidInfo.uuid))
                                     || (uuidType === 'ICCID' && !UtilService.checkIccidNumber($scope.uuidInfo.uuid)))) {
                                    AlertService.error('Número inválido');
                                    e.preventDefault();
                                    return;
                                } else {
                                    $scope.openScan = false;
                                    BarcodeService.cancel();
                                    return $scope.uuidInfo.uuid;
                                }
                            }
                        }]
                });

                return $scope.popupUuid;
            };

            $scope.showBarcodeScanner = () => {
                $scope.openScan = true;
                LogService.info("Iniciando Scanner na Consulta de UUID");
                $scope.initBarcodeScanner();
            };

            $scope.initBarcodeScanner = () => {
                BarcodeService.init(barcode => {
                    let uuid = null;
                    if (barcode[0].length > 20) {
                        uuid = barcode[0].substr(0, 20);
                    } else {
                        uuid = barcode[0];
                    }

                    if (uuid == null || uuid.length === 0) {
                        return;
                    }

                    $scope.openScan = false;
                    BarcodeService.cancel();
                    if (ConfigService.config.business.enable_imei_iccid_validation
                        && (($scope.itemSelected.uuidInfo.type === 'IMEI' && !UtilService.checkImeiNumber(uuid))
                         || ($scope.itemSelected.uuidInfo.type === 'ICCID' && !UtilService.checkIccidNumber(uuid)))) {
                        AlertService.error('Número inválido');
                    } else {
                        $scope.popupUuid.close(uuid);
                    }
                }, { screenName: "popupSearchByUUI.html" });
            };

            /**
             *
             */
            $scope.processInventoryLookupByUUID = (item) => {
                var UuidParams = {
                    storeNumber: ConfigService.config.storeNumber,
                    productUuid: item.itemUuid.value
                };
                $scope.loadingState = true;

                return $ionicLoading.show({
                    template: 'Reservando produto no estoque...',
                    duration: 0
                }).then(function () {
                    return ProductService.lookupByUUID(UuidParams).then((result) => {
                        if (item.productCode != result.productInventories[0].productCode) {
                            $ionicLoading.hide();
                            AlertService.restError(null, 'Identificador Único especificado não pertence ao produto selecionado');
                            return $q.reject();
                        }
                        
                        return true;
                    }, (error) => {
                        $scope.loadingState = false;
                        AlertService.restError(error, 'Erro consultando servidor de estoque!');
                        return $q.reject();
                    }).finally(() => {
                        $scope.loadingState = false;
                        $ionicLoading.hide();
                    });
                });
            };

            /**
             *
             */
            $scope.discountState = function (result, item) {
                if ($stateParams.prevState === "easyorder.barcode") {
                    UtilService.navigateWithoutHistory('easyorder.productdiscounts', { item: item, automatic: true, discountResult: result, prevState: 'easyorder.barcode' });
                } else {
                    UtilService.navigateWithoutHistory('easyorder.productdiscounts', { item: item, automatic: true, discountResult: result, prevState: 'easyorder.productsearch' });
                }
            };


            $scope.validateRequestAuthPartner = function (item) {
                $scope.requestauth = {};
                var permission = UserService.checkPermission(INCLOMPLETE_PARTNER_PRODUCT_PERMISSION);
                LogService.info(permission);

                if (permission) {
                    CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                } else {
                    var authPopup = $ionicPopup
                        .show({
                            templateUrl: 'views/requestauth.html',
                            scope: $scope,
                            buttons: [
                                {
                                    text: 'Cancelar'
                                },
                                {
                                    text: '<b>Autorizar</b>',
                                    type: 'button-theme',
                                    onTap: function (event) {
                                        var sucesso = function (result) {
                                            LogService.info("Authorized: ", result.authorized);
                                            if (result.authorized) {
                                                CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                                            } else {
                                                AlertService.error('Autorização negada!')
                                            }
                                        };

                                        var erro = function (error) {
                                            LogService.error(error);
                                            BarcodeService.cancel();
                                            AlertService.restError(error,
                                                "Erro ao consultar autorização!");
                                        };

                                        if (!$scope.requestauth.user) {
                                            BarcodeService.cancel();
                                            AlertService.error('Usuário inválido!');
                                        } else if (!$scope.requestauth.password) {
                                            BarcodeService.cancel();
                                            AlertService.error('Senha inválida!');
                                        } else {
                                            UserService.checkAuthorization($scope.requestauth.user,
                                                $scope.requestauth.password,
                                                [INCLOMPLETE_PARTNER_PRODUCT_PERMISSION],
                                                sucesso, erro);
                                        }
                                    }
                                }]
                        });
                }
            };
            /**
             *
             */
            $scope.addPatnerToCart = function (index, item) {
                if ($scope.isPartnerItemReady) {
                    CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                } else {
                    $scope.validateparner = {};
                    $scope.validateparner.barCode = item.productCode;
                    $ionicPopup.show({
                        templateUrl: 'views/validatepartner.html',
                        title: 'Produto Parceirado',
                        scope: $scope,
                        buttons: [
                            { text: 'Cancelar' },
                            {
                                text: '<b>Validar</b>',
                                type: 'button-theme',
                                onTap: function (e) {
                                    // Validações
                                    if (!$scope.validateparner.barCode) {
                                        AlertService.warn('Código inválido');
                                        return;
                                    }

                                    // Precisa de permissão?
                                    if ((!$scope.validateparner.dig2 && !$scope.validateparner.dig1) || // Dídigo 1 em branco e dígito 2 em branco
                                        ($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                        ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco

                                        // Se informou ao menos um dígito verificador, deve validar...
                                        if (($scope.validateparner.dig1 && !$scope.validateparner.dig2) || // Dídigo 1 preenchido e dígito 2 em branco
                                            ($scope.validateparner.dig2 && !$scope.validateparner.dig1)) { // Dídigo 2 preenchido e dígito 1 em branco
                                            var partnerProductCode
                                            if ($scope.validateparner.dig1) {
                                                partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                    + $scope.validateparner.dig1);
                                            } else if ($scope.validateparner.dig2) {
                                                partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                                    + $scope.validateparner.dig2);
                                            }
                                            if (!partnerProductCode.isValid()) {
                                                AlertService.warn('Item parceiro inválido');
                                                return;
                                            }
                                        }

                                        $scope.validateRequestAuthPartner(item);
                                    } else {
                                        var partnerProductCode = new PartnerProductCode($scope.validateparner.barCode + '$'
                                            + $scope.validateparner.dig1);
                                        if (partnerProductCode.isValid()) {
                                            var reversePartnerCode = partnerProductCode.getReversePartnerCode();
                                            if (reversePartnerCode != $scope.validateparner.barCode + '$'
                                                + $scope.validateparner.dig2) {
                                                //Código + dig2 não batem com o código do parceiro reverso do dig1
                                                AlertService.warn('Item parceiro inválido');
                                            } else {
                                                // Validação OK, pode adicionar ao carrinho
                                                CartService.doAddToCart(item).then(() => UtilService.navigateWithoutHistory($scope.prevState));
                                            }
                                        } else {
                                            // Código + dig1 não formam item parceirado
                                            AlertService.warn('Item parceiro inválido');
                                        }
                                    }
                                }
                            }]
                    });
                }
            };

            /**
             *
             */
            $scope.reset = function () {
                if ($scope.popup) {
                    $scope.popup.close();
                    $scope.popup = undefined;
                }
            };

            $scope.showSuggestions = () => {
                AlertService.showAlertList($scope, "Sugestões", CartService.cart.suggestions, [{ "field": "descriptor" },]);
            }

            $scope.getCountSuggestions = () => {
                return CartService.countSuggestions();
            }

            $scope.setConditionsPromotion = (returnCampaign) => {
                if (returnCampaign.suggestions && returnCampaign.suggestions.length) {
                    angular.forEach(returnCampaign.suggestions, (sugg) => {
                        $scope.listConditions.push({ "icon": "comment-alt", "description": sugg.descriptor });
                    })
                }
            }

            (function () {
                $scope.productCodeType = ConfigService.config.business.product_code_character_type;
                $scope.item = $stateParams.currentItem;
                $scope.prevState = $stateParams.prevState;
                $scope.productImageEnabled = ConfigService.config.business.product_images_enabled === true;
                $scope.cart = CartService.cart;

                if ($stateParams.productSearchParams && $stateParams.productSearchParams.productId) {
                    $scope.productSearchParams = $stateParams.productSearchParams;
                }
                $scope.listConditions = [];
                if ($scope.item && $rootScope.promotionEngine) {
                    let listItemsTemp = [];
                    listItemsTemp.push($scope.item);
                    CartService.processRulesDiscounts(null, null, null, listItemsTemp).then((returnCampaign) => {
                        $scope.setConditionsPromotion(returnCampaign);
                    });
                }
            })();
        });
})();
