(function () {
    'use strict';
    angular.module('easyorder.controllers.purchasesummary', [
        'easyorder.services.log'
    ])
        .controller('purchaseSummaryCtrl', function ($scope,
            $ionicLoading,
            ConfigService,
            UtilService,
            DiscountService,
            CartService,
            $ionicPopup,
            AlertService,
            LogService) {

            $scope.showDiscountsTypes = () => {

                var discountParams = {
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    automatic: false,
                    employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false,
                    fiscalItems: DiscountService.prepareFiscalItems(CartService.cart.fiscalItems)
                };

                $ionicLoading.show({
                    template: 'Por favor, aguarde...',
                    duration: 0
                });
                DiscountService.searchDiscountFiscalCoupon(discountParams).then((result) => {
                    LogService.info(result);
                    $ionicLoading.hide();

                    if (result.discountRuleTypes.length == 0) {
                        $ionicPopup.alert({
                            title: 'Erro!',
                            template: 'Não existe descontos cadastrados.',
                            buttons: [{
                                text: 'Ok',
                                type: 'button button-theme'
                            }]
                        });
                    } else {
                        UtilService.navigateWithoutHistory('easyorder.discountstypes', {
                            "result": result,
                            "cart": true,
                            "automatic": null
                        });
                    }
                }).catch(err => {
                    $ionicLoading.hide();
                    LogService.error(err);
                });
            };

            /**
             * 
             */
            $scope.doAction = () => {

                var currentCartAction = CartService.getCurrentCartAction();

                switch (currentCartAction) {
                    case "ORDER_PAYMENT": // pedido resgatado para pagamento
                        $scope.payOrder();
                        break;
                    case "SALE_PAYMENT": // venda
                        $scope.payOrder();
                        break;
                    default:
                        break;
                }
            };

            $scope.validateFields = function () {
                var isValid = false;

                switch ($scope.cart.customer.documentType) {
                    case "CPF":
                        isValid = validaCPF($scope.cart.customer.document);
                        break;
                    case "CNPJ":
                        isValid = validaCNPJ($scope.cart.customer.document);
                        break;
                    case "RNE":
                        isValid = validaRNE($scope.cart.customer.document);
                        break;
                    case "PASSPORT":
                        isValid = validaPassport($scope.cart.customer.document);
                        break;
                }
                return isValid;
            };

            $scope.isMandatoryParameterValid = function () {
                var isValid = true;

                if ($scope.mandatoryParameterFromServer == "SET2") {
                    if (!$scope.cart.customer || !$scope.cart.customer.document || !$scope.validateFields() || !$scope.cart.customer.email) {
                        AlertService.warn('Para realizar o pagamento é necessário informar o CPF e o E-mail do cliente!');
                        isValid = false;
                    }
                }

                return isValid;
            };


            $scope.payOrder = function () {
                if ($scope.isMandatoryParameterValid()) {
                    var customerRequired = ConfigService.config.business.customer_identification_required;
                    var documentEmpty = (!$scope.cart.customer || !$scope.cart.customer.document || $scope.cart.customer.document.length == 0);

                    var isMobileRecharge = CartService.getCurrentServiceOperationType() === 'MOBILE_RECHARGE';

                    if (ConfigService.config.tefserver && ConfigService.config.tefserver.host != '' && ConfigService.config.tefserver.port != 0) {
                        if ((customerRequired && !documentEmpty && $scope.validateFields()) || !customerRequired || isMobileRecharge || (CartService.hasNonFiscalItems && !documentEmpty)) {
                            if (CartService.predefinedPaymentMethodsDetails && CartService.predefinedPaymentMethodsDetails.length) {
                                CartService.processedPredefinedPayments.length = 0; //limpando.
                                UtilService.navigateWithoutHistory('easyorder.predefinedpaymentmethods');
                            } else {
                                UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                            }
                        } else {
                            AlertService.warn('Favor identificar cliente!');
                        }
                    } else {
                        AlertService.warn('TEF não configurado!');
                    }
                }
            };

            (function () {
                $scope.cart = CartService.cart;
                //Primeira função a ser chamada.
            })();
        })
})();
