(function () {
    'use strict';
    angular.module('easyorder.controllers.consolidatedstoresalesdetails', [])
        .controller('ConsolidatedStoreSalesDetailsCtrl', function (
            $scope,
            $ionicLoading,
            $ionicModal,
            $stateParams,
            ConfigService,
            AlertService,
            MailService,
            LogService,
            UserService,
            UtilService,
            PrinterService,
            ReportService) {

            $scope.printingEnabled = ConfigService.config.business.printing_enabled;
            $scope.sendMailEnabled = ConfigService.config.business.send_mail_enabled;
            $scope.saleSummaries = $stateParams.saleSummaries;
            $scope.octNumber = $scope.saleSummaries[0].octNumber;
            $scope.date = $stateParams.searchParams.date;
            $scope.operatorName = UserService.operatorName();
            $scope.storeName = 'Loja ' + ConfigService.config.storeNumber;
            $scope.reportService = ReportService;

            const pdf = $stateParams.pdf;
            const printContents = $stateParams.printContents;
            const documentTitle = $stateParams.documentTitle;

           

            $scope.goBack = () => {
                UtilService.navigateWithoutHistory('easyorder.consolidatedstoresalesresult', $stateParams);
            };

            $scope.doSendMailAction = function(){
                if ($scope.sendMailEnabled && pdf){
                    $scope.attachmentContent = [];
                    $scope.mails = [];
                    $scope.textMail = {
                        value: ""
                    };
                    
                    $scope.attachmentContent.push(new Attachment(pdf, documentTitle + ".pdf"));
                    $scope.openModalMail();
                }
            };
    
            $scope.sendMail = function(){
                $scope.useDestinationInputValue();
                if ($scope.mails && $scope.mails.length > 0) {
                    $ionicLoading.show({
                        template : 'Aguarde, enviando e-mail...',
                        duration : 0
                    }).then(function() {
                        const mailRequest = new MailRequest();
                        mailRequest.type = MailRequest.TYPES.REPORT;
                        mailRequest.subject = "Relatório Consolidado da Loja";
                        mailRequest.attachmentContent = $scope.attachmentContent;
                        mailRequest.destinations = $scope.mails;
    
                        MailService.send(mailRequest, () => {
                            $ionicLoading.hide();
                            AlertService.info('Email enviado!').then(() => {
                                $scope.closeModalMail();
                            });
                        }, error => {
                            LogService.error(error);
                            $ionicLoading.hide();
                            AlertService.info('Ocorreu um erro ao tentar enviar comprovante por email!');
                        });
    
                    }, error => {
                        LogService.error(error);
                        $ionicLoading.hide();
                        AlertService.info('Ocorreu um erro ao tentar enviar comprovante por email!');
                    });
                } else {
                    AlertService.error('Informe ao menos um e-mail válido para o envio');
                }
            };
            
            $scope.useDestinationInputValue = function() {
                if ($scope.textMail.value && $scope.textMail.value !== "") {
                    LogService.info("Adicionando e-mail informado no input à lista de destino");
                    $scope.addMail($scope.textMail.value);
                }
            }

            $scope.addMail = function(mail){
                var validador = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/gi;
                if (validador.test(mail)){
                    if (!$scope.mails.includes(mail)){
                        $scope.mails.push(mail);
                         $scope.textMail.value = "";
                    }else{
                        AlertService.error("Endereço de email já inserido!");
                    }
                }else{
                    AlertService.error("Endereço de email inválido!");
                }
            };
    
            $scope.removeMail = function(mail){
                var index = $scope.mails.indexOf(mail);
                if (index !== -1) $scope.mails.splice(index, 1);
            };

            $scope.openModalMail = function() {
                $scope.modalMail.show();
            };

            $scope.closeModalMail = function() {
                $scope.modalMail.hide();
            };

            // Modal de emails
            $ionicModal.fromTemplateUrl('views/sendmailorder.html', {
                scope: $scope,
                animation: 'slide-in-up'
            }).then(function(modalMail) {
                $scope.modalMail = modalMail;
            });
            
            $scope.$on('$destroy', function() {
                if ($scope.modalMail){
                    $scope.modalMail.remove();
                }
                $scope.modal.remove();
            });


            $ionicModal.fromTemplateUrl('views/printerslist.html', {
                scope: $scope,
                animation: 'slide-in-up'
            }).then(function(modal) {
                $scope.modal = modal;
            });
            
            function showPrintersModal() {
                return $scope.modal.show();
            }
    
            function hidePrintersModal() {
                return $scope.modal.hide();
            }

            $scope.closeModal = () => {
                hidePrintersModal();
            };
    

            $scope.listPrinters = () => {
                $ionicLoading.show({
                    template : 'Consultando impressoras...',
                    duration : 0
                }).then(function() {
                    PrinterService.search({
                        connection_status: 'ONLINE'
                    }).then(printers => {
                        $ionicLoading.hide();
                        $scope.loadingState = false;
        
                        $scope.pluginPrinters = [];
                        $scope.wsPrinters = [];
        
                        const supportedPrinters = printers.filter(printer => printerFilter(printer, printContents));
        
                        const pluginPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_PLUGIN);
                        const wsPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_WS);
        
                        if (supportedPrinters.length == 1) {
                            $scope.printingDoc = printContents;
                            $scope.printersSearchResult = supportedPrinters;
                            $scope.pluginPrinters = pluginPrinters;
                            $scope.wsPrinters = wsPrinters;
                            $scope.print(supportedPrinters[0], $scope.voucher);
                        } else if (supportedPrinters.length > 0) {
                            $scope.printingDoc = printContents;
                            $scope.printersSearchResult = supportedPrinters;
                            $scope.pluginPrinters = pluginPrinters;
                            $scope.wsPrinters = wsPrinters;
                            showPrintersModal();
                        } else {
                            AlertService.error('Nenhuma impressora disponível para o documento escolhido!');
                        }
                    }).catch(error => {
                        $ionicLoading.hide();
                        AlertService.error('Nenhuma impressora disponível');
                    });
                });
            };

            function printerFilter(printer, printContents) {
                const length = printContents.length;
    
                for (let i = 0; i < length; ++i) {
                    if (printer.paperType === printContents[i].paperType) {
                        return true;
                    }
                }
                return false;
            }
    
            $scope.print = function(printer) {
                $ionicLoading.show({
                    template : 'Imprimindo...',
                    duration : 0
                }).then(() => {
                    const paperType = printer.paperType;
                    const printContent = printContents.find(content => {
                        return paperType === content.paperType;
                    });
    
                    const params = {
                        'printerid':    printer.id,
                        'contentType':  printer.paperType === 'A_SERIES' ? 'application/pdf' 
                                                                : 'application/vnd.linx.printer-text',
                        'content':      printContent.text,
                        'title':        'Doc',
                        'terminal':     ConfigService.config.terminalNumber,
                        'origin':       printer.origin,
                        'paperType':    printer.paperType
                    };
                    return doPrint(params, printer);
                });
            };
    
            function doPrint(params, printer) {
                return PrinterService.submit(params)
                    .then(() => {
                        return $ionicLoading.hide().then(() => {
                            $scope.loadingState = false;
                        });
                    }).then(() => {
                        return AlertService.info(`Documento impresso na impressora ${printer.name}`, 'OK');
                    }).then(() => {
                        return hidePrintersModal();
                    }).catch(error => {
                        $ionicLoading.hide().then(() => {
                            $scope.loadingState = false;
                        }).then(() => {
                            AlertService.restError(error, 'Erro no envio da impressão');
                        });
                    });
            }
        });
})();
