(function () {
    'use strict';
    angular.module('easyorder.controllers.consolidatedstoresalessearch', [])
        .controller('ConsolidatedStoreSalesSearchCtrl', function (
            $rootScope,
            $scope,
            $ionicLoading,
            ConfigService,
            UtilService,
            AlertService,
            UserService,
            ReportService) {

            $scope.operatorName = UserService.operatorName();
            $scope.storeName = 'Loja ' + ConfigService.config.storeNumber;
            $scope.todayDate = new Date();
            const receiptVersion = 2;
            
            $scope.searchParams = {
                date: formatDate(new Date())
            };
           
            function searchSales() {
                return ReportService.consolidatedsales(
                    ConfigService.config.companyNumber,
                    ConfigService.config.storeNumber,
                    ConfigService.config.terminalNumber,
                    UserService.operatorCode(),
                    toDate($scope.searchParams.date),
                    {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    },
                    receiptVersion
                );
            }

            function toDate(value) {
                if (!value) {
                    return;
                }
                const match = value.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
                return new Date(match[3], match[2] - 1, match[1]);
            }
                    
            function formatDate(date) {
                var d = new Date(date),
                    month = "" + (d.getMonth() + 1),
                    day = "" + d.getDate(),
                    year = d.getFullYear();

                if (month.length < 2) month = "0" + month;
                if (day.length < 2) day = "0" + day;

                return [day, month, year].join("");
            }

            $scope.search = () => {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(() => {
                    return searchSales();
                }).then(result => {
                    $ionicLoading.hide().then(() => {
                        if (result.saleSummaries && result.saleSummaries.length) {
                            const params = Object.assign({}, $scope.searchParams);
                            params.date = toDate($scope.searchParams.date);

                            UtilService.navigateWithoutHistory('easyorder.consolidatedstoresalesresult', {
                                saleSummaries: result.saleSummaries,
                                searchParams: params,
                                pdf: result.pdf,
                                printContents: result.printContents,
                                documentTitle: result.documentTitle || 'Relatorio'
                            });
                        } else {
                            AlertService.error("Nenhuma informação encontrada!");
                        }
                    });
                }).catch(err => {
                    return $ionicLoading.hide().then(() => {
                        $scope.loadingState = false;
                        return AlertService.restError(err, err);  
                    });
                });
            };

        });
})();
