(function () {
    'use strict';
    angular.module('easyorder.controllers.mysalessearch', [])
        .controller('MySalesSearchCtrl', function (
            $rootScope,
            $scope,
            $ionicLoading,
            ConfigService,
            UtilService,
            AlertService,
            ReportService,
            UserService) {

            $scope.operatorName = UserService.operatorName();
            $scope.storeName = 'Loja ' + ConfigService.config.storeNumber;
            $scope.todayDate = new Date();

            $scope.searchParams = {
                initialDate: formatDate(new Date()),
                finalDate: formatDate(new Date()),
                terminalType: "ALL"
            };

            function searchSales() {
                return ReportService.operatorsales(
                    ConfigService.config.companyNumber,
                    ConfigService.config.storeNumber,
                    ConfigService.config.terminalNumber,
                    UserService.operatorCode(),
                    [ UserService.operatorCode() ],
                    toDate($scope.searchParams.initialDate),
                    toDate($scope.searchParams.finalDate),
                    $scope.searchParams.terminalType == 'ALL'? null : [ $scope.searchParams.terminalType ],
                    {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    }
                );
            }

            function toDate(value) {
                if (!value) {
                    return;
                }
                const match = value.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
                return new Date(match[3], match[2] - 1, match[1]);
            }
                    
            function formatDate(date) {
                var d = new Date(date),
                    month = "" + (d.getMonth() + 1),
                    day = "" + d.getDate(),
                    year = d.getFullYear();

                if (month.length < 2) month = "0" + month;
                if (day.length < 2) day = "0" + day;

                return [day, month, year].join("");
            }

            $scope.search = () => {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(() => {
                    return searchSales();
                }).then(result => {
                    $ionicLoading.hide().then(() => {
                        if (result.saleSummaries && result.saleSummaries.length) {
                            const params = Object.assign({}, $scope.searchParams);
                            params.initialDate = toDate($scope.searchParams.initialDate);
                            params.finalDate   = toDate($scope.searchParams.finalDate);

                            UtilService.navigateWithoutHistory('easyorder.mysalesresult', {
                                saleSummaries: result.saleSummaries,
                                searchParams: params
                            });
                        } else {
                            AlertService.error("Nenhuma informação encontrada!");
                        }
                    });
                }).catch(err => {
                    return $ionicLoading.hide().then(() => {
                        $scope.loadingState = false;
                        return AlertService.restError(err, err);  
                    });
                });
            };

        });
})();
