(function () {
    'use strict';
    angular.module('easyorder.controllers.reprintoptions', [])
        .controller('reprintOptionsCtrl', function (
            $scope,
            $state,
            $stateParams,
            $ionicLoading,
            ReprintService,
            UtilService,
            AlertService,
            LogService,
            ConfigService) {

            $scope.options = $stateParams.options;

            $scope.goBack = () => {
                if ($stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();
                    
                    UtilService.navigateWithoutHistory(backState, params);
                }
            };

            $scope.searchTransactions = (option) => {
                // option === 'LAST_TERMINAL_TRANSACTIONS, filtra pelo terminal em questão. 'LAST_STORE_TRANSACTIONS', todos os terminais da loja.
                const params = {
                    storeNumber: ConfigService.config.storeNumber,
                    terminalNumber: option === 'LAST_TERMINAL_TRANSACTIONS' ? ConfigService.config.terminalNumber : null
                }
                
                LogService.info(`Iniciando consulta das últimas vendas da loja '${params.storeNumber}' e terminal '${option === 'LAST_TERMINAL_TRANSACTIONS' ? params.terminalNumber : 'todos'}'.`);
                $ionicLoading.show({
                    template: "Consultando vendas, por favor aguarde...",
                    duration: 0
                }).then(() => {
                    return ReprintService.searchLastTransactions(params);
                }).then((result) => {
                    $ionicLoading.hide().then(() => {
                        if (result.receipts == null || result.receipts.length <=0) {
                            AlertService.error('Não há cupons disponíveis para reimpressão com os dados pesquisados!');
                        } else {
                            LogService.info("Retornados {"+result.receipts.length+"} cupons.");
                            navigateToState('easyorder.reprintreceiptlist', { receipts: result.receipts });
                        }
                    });
                }).catch((error) => {
                    $ionicLoading.hide().then(() => {
                        AlertService.restError(error, error);
                    });
                });
            };

            $scope.searchBy = (option) => {
                navigateToState('easyorder.reprintsearch', { option: option });
            };
            
            function navigateToState(state, params) {
                $stateParams.flowHistory.flow.push($state.current.name);
                $stateParams.flowHistory.params.push($stateParams);
                
                const transitionParams = {
                    flowHistory: $stateParams.flowHistory,
                };
                angular.merge(transitionParams, params || {});
                
                UtilService.navigateWithoutHistory(state, transitionParams);
            };

            $scope.hasOption = (option) => {
                return $scope.options.includes(option);
            };

        });
})();
