(function () {
    'use strict';
    angular.module('easyorder.controllers.reprintreceiptlist', [
    ]).controller('reprintReceiptListCtrl', function (
        $scope,
        $stateParams,
        UtilService,
        PrinterService,
        $ionicModal,
        $ionicLoading,
        AlertService,
        ConfigService,
        LogService
    ) {

        const printContents = [];

        /**
        *
        */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };

        $scope.selectItem = item => {
            LogService.info(`Cupom selecionado. Data: ${item.date}, valor: ${item.totalValue}.`);
            item.status = !item.status;
            if (item.status) {
                printContents.push(item);
            } else {
                printContents.pop(item);
            }
        };

        $scope.buttonActive = () => {
            return (printContents.length > 0);
        };

        $ionicModal.fromTemplateUrl('views/printerslist.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function (modal) {
            $scope.modal = modal;
        });

        function showPrintersModal() {
            return $scope.modal.show();
        }

        function hidePrintersModal() {
            return $scope.modal.hide();
        }

        $scope.$on('$destroy', () => {
            $scope.modal.remove();
        });

        $scope.closeModal = () => {
            hidePrintersModal();
        };

        $scope.listPrinters = () => {
            $ionicLoading.show({
                template: 'Consultando impressoras...',
                duration: 0
            }).then(() => {
                PrinterService.search({
                    connection_status: 'ONLINE'
                }).then(printers => {
                    LogService.info("iniciando listagem de impressoras");
                    $ionicLoading.hide();
    
                    $scope.pluginPrinters = [];
                    $scope.wsPrinters = [];
    
                    const supportedPrinters = printers;
    
                    const pluginPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_PLUGIN);
                    const wsPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_WS);
    
                    if (supportedPrinters.length == 1) {
                        $scope.printingDoc = printContents;
                        $scope.printersSearchResult = supportedPrinters;
                        $scope.pluginPrinters = pluginPrinters;
                        $scope.wsPrinters = wsPrinters;
                        $scope.print(supportedPrinters[0], printContents);
                    } else if (supportedPrinters.length > 0) {
                        $scope.printingDoc = printContents;
                        $scope.printersSearchResult = supportedPrinters;
                        $scope.pluginPrinters = pluginPrinters;
                        $scope.wsPrinters = wsPrinters;
                        showPrintersModal();
                    } else {
                        AlertService.error('Nenhuma impressora disponível para o documento escolhido!');
                    }
    
                }).catch(error => {
                    $ionicLoading.hide();
                    AlertService.error('Nenhuma impressora disponível');
                });
            });
          

        };

        $scope.print = function (printer) {
            $ionicLoading.show({
                template: 'Imprimindo...',
                duration: 0
            }).then(() => {
                LogService.info("preparando dados para impressão");
                const content = printContents.filter(c => c.printContent != null)
                                             .map(c => atob(c.printContent))
                                             .reduce((acc, txt) => acc.concat(txt), '');

                if (content == null || content.length == 0) {
                    return $ionicLoading.hide().then(() =>
                        AlertService.error('Dados para impressão não foram retornados'));
                }

                const params = {
                    'printerid': printer.id,
                    'contentType': printer.paperType === 'A_SERIES' ? 'application/pdf'
                        : 'application/vnd.linx.printer-text',
                    'content': btoa(content),
                    'title': 'reprint - coupon',
                    'terminal': ConfigService.config.terminalNumber,
                    'origin': printer.origin,
                    'paperType': printer.paperType
                };
                LogService.info("enviando dados para impressão");
                return doPrint(params).then(() => {
                    return $ionicLoading.hide();
                }).then(() => {
                    hidePrintersModal();
                    return AlertService.info(`Documento impresso na impressora ${printer.name}`, 'OK');
                }).then(() => {
                    LogService.info("Documento impresso");
                    $scope.nextPage();
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
                        AlertService.restError(error, 'Erro no envio da impressão');
                    });
                });
            })
            .finally(() => $ionicLoading.hide())
            .catch(err => {
                LogService.error(err);
                AlertService.error('Falha inesperada durante impressão');
            });
        };

        function doPrint(params) {
            return PrinterService.submit(params);
        }

        $scope.nextPage = () => {
            $scope.receipts = [];
            UtilService.navigateToMainScreen();
        };


        (function () {
            $scope.receipts = [];
            $stateParams.receipts.forEach(element => {
                element.status = false;
                $scope.receipts.push(element);
            });
            console.log($scope.receipts);
        })();
    });
})();
