(function () {
    'use strict';
    angular.module('easyorder.controllers.reprintsearch', [])
        .controller('reprintSearchCtrl', function (
            $scope,
            $stateParams,
            $state,
            $ionicLoading,
            ConfigService,
            ReprintService,
            UtilService,
            AlertService,
            LogService,
            $q) {

            $scope.formData = {
                initialDate: new Date(new Date().setHours(0,0,0,0)).toISOString(),
                finalDate: new Date(new Date().setHours(0,0,0,0)).toISOString(),
            };
            $scope.initialDate = moment().format('DD/MM/YYYY');
            $scope.finalDate   = moment().format('DD/MM/YYYY');
            const maxIntervalDays = ConfigService.config.business.reprint_max_interval_days;



            $scope.option = $stateParams.option ? $stateParams.option : 'DOCUMENT';

            $scope.goBack = () => {
                if ($stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();

                    UtilService.navigateWithoutHistory(backState, params);
                }
            };

            $scope.search = () => {
                LogService.info(`Iniciando consulta de cupom: '${JSON.stringify($scope.formData)}.'`);
                $ionicLoading.show({
                    template: "Aguarde...",
                    duration: 0
                }).then(() => {

                    if ($scope.option === 'DOCUMENT' && isEmpty($scope.formData.document)) {
                        return $q.reject('CPF/CNPJ vazio ou inválido');
                    } else if ($scope.option === 'NSU' && !UtilService.isInteger($scope.formData.storeNumber)) {
                        return $q.reject('Número da loja vazio ou inválido!');
                    } else if ($scope.option === 'NSU' && (!UtilService.isNullOrEmpty($scope.formData.nsu) && !UtilService.isInteger($scope.formData.nsu))) {
                        return $q.reject('Número do terminal inválido!');
                    } else if ($scope.option != 'NSU' && UtilService.isNullOrEmpty($scope.formData.initialDate)) {
                        return $q.reject('Data (início) inválida!');
                    } else {
                        if ($scope.formData.initialDate) {
                            const initialDate = moment($scope.formData.initialDate);
                            const finalDate   = moment($scope.formData.finalDate);

                            if (!initialDate.isValid()) {
                                return $q.reject('Data (início) inválida!');
                            } else if (!finalDate.isValid()) {
                                return $q.reject('Data (fim) inválida!');
                            } else {
                                var now = moment();
                                if (initialDate > now) {
                                    return $q.reject('Data (início) superior à data atual!');
                                } else if ( finalDate < now.startOf('day')) {
                                    return $q.reject('Data (fim) inferior à data atual!');
                                } else if (maxIntervalDays && Math.abs(initialDate.diff(finalDate, 'days')) > maxIntervalDays) {
                                    return $q.reject('Intervalo entre datas deve ser de no máximo ' + maxIntervalDays + ' dias!');
                                }
                            }
                        }

                        return ReprintService.search({
                            companyNumber: "00001",
                            storeNumber: $scope.formData.storeNumber,
                            terminalNumber: $scope.formData.terminalNumber,
                            nsu: $scope.formData.nsu,
                            document: $scope.formData.document,
                            initialDate: $scope.formData.initialDate,
                            finalDate: $scope.formData.finalDate
                        });
                    }

                }).then((result) => {
                    $ionicLoading.hide().then(() => {
                        if (!result.receipts || result.receipts.length <= 0) {
                            AlertService.error('Não há cupons disponíveis para reimpressão com os dados pesquisados!');
                        } else {
                            LogService.info(`Retornados ${result.receipts.length} cupons para o CPF/CNPJ informado.`);
                            navigateToState('easyorder.reprintreceiptlist', { receipts: result.receipts });
                        }
                    });
                }).catch((error) => {
                    $ionicLoading.hide().then(() => {
                        AlertService.restError(error, error);
                    });
                });
            };



            $scope.formatInitialDate = (value) => {
                if (!value) {
                    $scope.formData.initialDate = null;
                    return;
                }
                const match = value.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
                $scope.formData.initialDate = new Date(match[3], match[2] - 1, match[1]).toISOString();
            };

            $scope.formatFinalDate = (value) => {
                if (!value) {
                    $scope.formData.finalDate = null;
                    return;
                }
                const match = value.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
                $scope.formData.finalDate = new Date(match[3], match[2] - 1, match[1]).toISOString();
            };


            function isEmpty(str) {
                return (str == undefined || str == '');
            }

            function navigateToState(state, params) {
                $stateParams.flowHistory.flow.push($state.current.name);
                $stateParams.flowHistory.params.push($stateParams);
                
                const transitionParams = {
                    flowHistory: $stateParams.flowHistory,
                };
                angular.merge(transitionParams, params || {});
                
                UtilService.navigateWithoutHistory(state, transitionParams);
            };

        });
})();
