(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange.document.search', [])
    .controller('DocumentExchangeCtrl', function ($scope,
                                                $state,
                                                $stateParams,
                                                $ionicLoading,
                                                $q,
                                                UtilService,
                                                TransactionSearchService,
                                                CustomerService,
                                                LogService,
                                                AlertService,
                                                CartService,
                                                $timeout) {

        $scope.searchCustomer = {};

        /**
         *
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();

                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.doSearch = () => {
            $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            }).then(() => {
                LogService.info(`Consultando transações de troca do CPF ${$scope.searchCustomer.document}`);

                getCustomer().then(() => {
                    getTransacationList(transactions => {
                        goToTransactionCupomList(transactions);
                    });
                });
            });
        };
        $scope.doSearchV3 = () => {
            $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            }).then(() => {
                LogService.info(`Consultando transações de troca do CPF ${$scope.searchCustomer.document}`);

                getCustomer().then(() => {
                    getTransacationList(transactions => {
                        goToTransactionCupomListV3(transactions);
                    });
                });
            });
        };
        /**
         *
         */
        $scope.reset = () => {
            $scope.searchCustomer = {};
        };
       /**
        *
        */
        $scope.validateDocument = () => {
            var document = $scope.searchCustomer.document;
            $scope.documentInvalid = false;

            if (document) {
                if (document.length == 11 && !validaCPF(document)) {
                    $scope.documentInvalid = true;
                    AlertService.warn("CPF inválido").finally(putFocusOnInput);
                } else {
                    return removeFocusFromInput();
                }
            }
        };
        /**
         *
         */
        async function putFocusOnInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_registrationCPF');
                var angularEl = angular.element(myEl);
                angularEl[0].focus();
            });
        }
        /**
         *
         */
        async function removeFocusFromInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_registrationCPF');
                var angularEl = angular.element(myEl);
                angularEl[0].blur();
            });
        }
        /**
         *
         */
        function getCustomer() {
            var defered = $q.defer();

            CustomerService.searchByType($scope.searchCustomer.document, 'CPF').then(result => {
                if (result.length) {
                    $scope.searchCustomer = {
                        customerId: result[0].customerId,
                        document: result[0].document,
                        documentType: result[0].documentType,
                        email: result[0].email,
                        name: result[0].name,
                        address: result[0].address
                    };
                }
            }).catch((error) => {
                LogService.warn(`Não foi possível consultar CPF [${$scope.searchCustomer.document}]: ${error.message}`);
                $scope.searchCustomer.documentType = 'CPF';
            }).finally(defered.resolve);

            return defered.promise;
        }
        /**
         *
         */
        async function getTransacationList(callback) {
            $ionicLoading.show({
                template : 'Consultando transações do cliente...',
                duration : 0
            });
            let params = {
                customer: {
                    document: $scope.searchCustomer.document
                },
            }

            let resultStorex = null;
            let resultLogic = null;

            try {
                params.provider = "STOREX"
                resultStorex = await TransactionSearchService.list(params);
            }catch (error) {
                LogService.error(error);
            }

            try {
                params.provider = "LOGIC"
                resultLogic = await TransactionSearchService.list(params);
            }catch (error) {
                LogService.error(error);
            }

            if (resultStorex == null && resultLogic == null){
                $ionicLoading.hide().then(error  => {
                    AlertService.restError(error, "Não foi possível consultar transações do cliente.").then(() => $scope.reset());
                });
            }

            const mergedResult = mergTransactions(resultStorex, resultLogic);
            let filtredResult = filterAvailableTransactions(mergedResult)

            $ionicLoading.hide()
            .then(() => {
                if (filtredResult && filtredResult.length) {
                    callback(filtredResult);
                } else {
                    AlertService.warn("Não há transação disponível para troca.");
                }
            });
        }

        function mergTransactions(result1, result2){
            if(result1 && result1.transactions && result2 && result2.transactions) {
                let ret = result1;
                let transactions = result1.transactions.concat(result2.transactions.filter((item) => result1.transactions.indexOf(item) < 0));
                ret.transactions = transactions;
                return ret;
            }
            if(result1 && result1.transactions) return result1;
            if(result2 && result2.transactions) return result2;

            return null;
        }
        /**
         *
         */
        function filterAvailableTransactions(result) {
            var ret;

            if (result.transactions && result.transactions.length) {
                ret = result.transactions.filter(trns => trns.availableFiscalItems.length);
            }

            return ret;
        }
        /**
         *
         */
        function goToTransactionCupomList(transactions) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);

            if (transactions.length > 1) {
                redirectTransactionCupomList(transactions);
            } else {
                redirectTransactionProductList(transactions[0]);
            }
        }
        /**
         *
         */
        function goToTransactionCupomListV3(transactions) {
            $stateParams.flowHistory.flow.push($state.current.name);
            $stateParams.flowHistory.params.push($stateParams);

            if (transactions.length > 1) {
                redirectTransactionCupomListV3(transactions);
            } else {
                redirectTransactionProductList(transactions[0]);
            }
        }
        function redirectTransactionCupomListV3(transactions) {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.searchCustomer = $scope.searchCustomer || {};
            transitionParams.transactions = transactions;
            transitionParams.authorizations = [];
            transitionParams.customer = {};

            return UtilService.navigateWithoutHistory('easyorder.exchangetransactioncouponslist', transitionParams);
        }
        /**
         *
         */
        function redirectTransactionCupomList(transactions) {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.searchCustomer = $scope.searchCustomer || {};
            transitionParams.transactions = transactions;

            return UtilService.navigateWithoutHistory('easyorder.exchangetransactioncouponslist', transitionParams);
        }
        /**
         *
         */
        function redirectTransactionProductList(transaction) {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData.items = transaction.availableFiscalItems
            .map(item => {
                return {
                    transaction: transaction,
                    item: item,
                    quantity: 1
                };
            });
            return UtilService.navigateWithoutHistory('easyorder.exchangetransactionproductlist', transitionParams);
        }

        (() => {
            $scope.validateDocument();

            if (CartService.cart.customer) {
                $scope.searchCustomer = CartService.cart.customer;
            } else if ($stateParams.customer) {
                $scope.searchCustomer = {...$stateParams.customer};
            }
        })();
    });
})();
