(function() {
    'use strict';

    angular.module('easyorder.controllers.returns.exchange.rfid.tag.transaction.search', [
    ])
    .controller('ExchangeRFIDTagTransactionSearchCtrl', function($scope,
                                            $state,
                                            $stateParams,
                                            $q,
                                            $timeout,
                                            $ionicLoading,
                                            UtilService,
                                            UserService,
                                            ConfigService,
                                            LogService,
                                            AlertService,
                                            ProductService,
                                            TransactionSearchService,
                                            BluebirdCaptureService) {
        var searchTimeoutPromise;
        var tagForSearching = new Map();
                                            
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.goScannedCouponless = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);
    
            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData.items = $scope.items;
            
            UtilService.navigateWithoutHistory('easyorder.rfidscannedexchangecouponless', transitionParams);
        };
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastPowerSet = $scope.data.defaultPower)
            .then(() => ConfigService.config.business.rfid_reading_power = $scope.data.lastPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultPower = $scope.data.lastPowerSet);
            });
        }; 
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function callbackResult(data) {
            
            if (!alreadyRead(data.raw)) {
                LogService.info(`Leitura SLED: ${data.raw}`);

                const tagCore = data.raw.substring(3);
                const opositeHeader = data.raw.substring(0,3) === ConfigService.config.business.epc_enable_prefix ?
                    ConfigService.config.business.epc_disable_prefix : ConfigService.config.business.epc_enable_prefix;
                
                const ref_1 = data.raw;
                const ref_2 = opositeHeader + tagCore;
                
                tagForSearching.set(data.raw, [ref_1, ref_2]);
            } else {
                AlertService.info(`Etiqueta já informada para esta troca: <b>${data.raw}</b>`);
            }
        }
        /**
         *
         */
        $scope.$watch("sledTrigger.status", function(status) {
            $timeout.cancel(searchTimeoutPromise);

            if ('TRIGGER_RELEASED' === status && BluebirdCaptureService.connected()) {
                searchTimeoutPromise = $timeout(() => {
                    if (tagForSearching.size) {
                        searchTransactions()
                        .finally(() => tagForSearching.clear());       
                    } else {
                        LogService.info(`Nenhum item lido para busca.`);
                    }
                }, 500);
            }
        });
        /**
         *
         */
        function searchTransactions() {
            const searchingTuples = [...tagForSearching.values()];
            const flatSearchingTuples = [...tagForSearching.values()].flat();
            
            LogService.info(`Dados usados para consulta de transaçãoes: ${flatSearchingTuples.join(', ')}`);
            
            return $ionicLoading.show()
            .then(() => TransactionSearchService.list({epcTags: flatSearchingTuples}))
            .finally($ionicLoading.hide)
            .then(result => {
                var ret = $q.resolve();
                
                if (result.transactions && result.transactions.length) {
                    var resultTransactions = result.transactions
                    .sort((e1, e2) => new Date(e2.key.date).getTime() - new Date(e1.key.date).getTime()); // Ordenando por "Mais recentes" primeiro (desc).
                    
                    var couponlessItems = [];
                    
                    searchingTuples.forEach(tuple => {
                        let availableItem;
                        
                        for (const trans of resultTransactions) {
                            availableItem = trans.availableFiscalItems.find(item => !!tuple.find(t => t === item.epcTag));
                            
                            if (availableItem) {
                                LogService.info(`Um item da Transação [${trans.key.storeNumber}|${trans.key.terminal}|${trans.key.date}|${trans.key.nsu}] foi selecionado para troca pelo EPC ${tuple[0]}`);
                                addItemToExchangeData(trans, availableItem);
                                break;
                            } else if (trans.returnedFiscalItems.find(item => !!tuple.find(t => t === item.epcTag))) { // já Foi retornado anteriormente? Então é sem cupom.
                                break;
                            }
                        }
                        if (!availableItem) {
                            couponlessItems.push(tuple[0]);
                        }
                    });
                    
                    if (couponlessItems.length) {
                        LogService.info(`Lista de EPCs ${couponlessItems.join(', ')} que não estão associados a uma transação.`);
                        
                        ret = AlertService.confirm(`Deseja incluir os itens?`, 40000, `Existem produtos sem cupom associado`, `Incluir itens`, `Ignorar itens`)
                        .then(result => result ? UserService.requestExternalAuthorization([{id: EXCHANGE_OPERATION, description: 'Operação de Troca'}]) : $q.reject())
                        .then(async () => {
                            for(const epc of couponlessItems) {
                                await getItem(epc)
                                 .then(item =>  addItemToExchangeData(undefined, item))
                                 .catch(AlertService.restError);
                            }
                        });
                    }
                } else {
                    LogService.info(`Nenhuma transação foi retornda para a consulta.`);
                    
                    ret = AlertService.confirm(`Deseja incluir os itens?`, 40000, `Existem produtos sem cupom associado`, `Incluir itens`, `Ignorar itens`)
                    .then(result => result ? UserService.requestExternalAuthorization([{id: EXCHANGE_OPERATION, description: 'Operação de Troca'}]) : $q.reject())                    
                    .then(async () => {
                        for (const tuple of searchingTuples) {
                            await getItem(tuple[0])
                            .then(item =>  addItemToExchangeData(undefined, item))
                            .catch(AlertService.restError);
                        }
                    });
                }
                return ret;
            })
            .then(() => {
                if ($scope.items && $scope.items.length) {
                    goToProductList($scope.items);
                }
            })
            .catch(AlertService.restError);
        }
        /**
         *
         */
        function alreadyRead(epc) {
            return $scope.items
            .map(added => added.item)
            .find(item => item.epcTag === epc);
        }
        /**
         *
         */
        function addItemToExchangeData(transaction, item) {
            $scope.items.push({
                transaction: transaction, 
                item: item,
                quantity: 1
            });
        }
        /**
         *
         */
        function getItem(epc) {
            return $ionicLoading.show({
                template : 'Consultando Produto...',
                duration : 0
            }).then(() => {
                const productCode = parseInt(epc.substring(4, 16), 16);
                
                return ProductService.search({
                    code : productCode
                })
                .finally($ionicLoading.hide)
                .then(result => {
                    if (result.items.length > 0) {
                        var saleInfo = result.items[0];
                        saleInfo.epcTag = epc;
                        
                        if (saleInfo.productType == 'MINIMUM_SALE_UNIT') {
                            saleInfo.quantitySold = saleInfo.minimumSaleUnit;
                        } else {
                            saleInfo.quantitySold = 1;
                        }

                        saleInfo.totalValue = saleInfo.unitValue * saleInfo.quantitySold;
    
                        return saleInfo;
                    } else {
                        return $q.reject({data: { message: `Nenhum item foi encontrado com código ${productCode}`}});
                    }
                });
            });
        };
        /**
         * 
         */
        function goToProductList(items) {
            var transitionParams = UtilService.cloneDeep($stateParams);
            
            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData.items = items;
            transitionParams.exchangeData.showAddMoreItems = true;
            
            return UtilService.navigateWithoutHistory('easyorder.exchangetransactionproductlist', transitionParams);
        }
        /**
         *
         */
        (() => {
            $scope.items = $stateParams.exchangeData.items || [];    
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.data = {
                lastPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultPower: ConfigService.config.business.rfid_reading_power,
            };
            
            $ionicLoading.show({
                template: 'Conectando ao Leitor RFID...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.connect($scope.data.defaultPower))
            .then(() => BluebirdCaptureService.startReading(callbackResult, error => LogService.info(`Erro na leitura SLED: ${error.message}`)))
            .finally($ionicLoading.hide)
            .catch(error => {
                if (ConfigService.config.business.enables_exchange_by_rfid_only) {
                    AlertService.alert('Leitor RFID não conectado', 'Retorne após conectar o leitor RFID para prosseguir com a operação')
                    .then($scope.goBack);
                } else {
                    let promise;
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                    promise = AlertService.confirm('Leitor RFID não conectado. Deseja prosseguir?');
                    } else {
                    promise = AlertService.confirm(`Erro de conexão do Leitor RFID [${data.errorCode}]. Deseja prosseguir?`);
                    }
                    
                    promise.then(result => {
                        if (!result) {
                            return $scope.goBack();
                        }
                    });
                }
            });
        })();
    });
})();