(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange', [])
    .controller('ExchangeMenuCtrl', function ($scope,
                                    $state,
                                    $stateParams,
                                    $ionicLoading,
                                    $timeout,
                                    $q,
                                    AlertService,
                                    UtilService,
                                    ConfigService,
                                    BluebirdCaptureService) {
        /**
         *
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();

                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.coupomSearch = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData = {};
            transitionParams.exchangeData.rfidReaderConnected = $scope.rfidReaderConnected;

            if ($scope.rfidReaderConnected) {
                return UtilService.navigateWithoutHistory('easyorder.rfidscannedexchangetransaction', transitionParams);
            } else {
                return UtilService.navigateWithoutHistory('easyorder.scannedexchange', transitionParams);
            }
        };
        /**
         *
         */
         $scope.coupomSearchV3 = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData = {};
            transitionParams.exchangeData.rfidReaderConnected = $scope.rfidReaderConnected;
            transitionParams.customer = $stateParams.customer;

            return UtilService.navigateWithoutHistory('easyorder.typedexchangev3', transitionParams);
        };
        /**
         *
         */
        $scope.customerDocumentSearch = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData = {};
            transitionParams.exchangeData.rfidReaderConnected = $scope.rfidReaderConnected;

            return UtilService.navigateWithoutHistory('easyorder.documentexchange', transitionParams);
        };
        /**
         *
         */
        $scope.scannedProductSearch = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData = {};
            transitionParams.exchangeData.couponlessFlowFromEPC = false;
            transitionParams.exchangeData.rfidReaderConnected = $scope.rfidReaderConnected;

            if ($scope.rfidReaderConnected) {
                return UtilService.navigateWithoutHistory('easyorder.rfidscannedexchangecouponless', transitionParams);
            } else {
                return UtilService.navigateWithoutHistory('easyorder.scannedexchangecouponless', transitionParams);
            }
        };
        /**
         *
         */
        $scope.rfidProductSearch = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData = {};
            transitionParams.exchangeData.couponlessFlowFromEPC = true;
            transitionParams.exchangeData.rfidReaderConnected = $scope.rfidReaderConnected;

            return UtilService.navigateWithoutHistory('easyorder.exchangerfidtagtransactionsearch', transitionParams);
        };

        (async () => {
            const methods = ConfigService.config.business.available_exchange_operation_methods;

            if (ConfigService.config.business.enables_exchange_rfid_reading) {
                await $ionicLoading.show()
                .then(() => BluebirdCaptureService.connect(ConfigService.config.business.rfid_reading_power))
                .then(() => BluebirdCaptureService.disconnect())
                .finally($ionicLoading.hide)
                .then(() => $scope.rfidReaderConnected = true)
                .catch(() => {
                    if (ConfigService.config.business.enables_exchange_by_rfid_only) {
                        AlertService.alert('Leitor RFID não conectado', 'Retorne após conectar o leitor RFID para prosseguir com a operação')
                        .then($scope.goBack);
                    } else {
                        let prom = $q.resolve();

                        if ($scope.previousState === "easyorder.exchangetransactioncustomerIdentification") {
                            prom = AlertService.confirm(`Leitor RFID não conectado. Deseja prosseguir?`)
                            .then(async confirm => {
                                if (!confirm) {
                                    return $q.reject();
                                }
                            });
                        }
                        prom
                        .then(() => $scope.rfidReaderConnected = false)
                        .catch(() => UtilService.navigateWithoutHistory('easyorder.operations', {}));
                    }
                })
                .finally(() => $scope.loading = false);
            } else {
                $scope.rfidReaderConnected = false;
            }
            let animationIndex = 1;

            if (methods.find(method => method === "COUPON")) {
                $timeout(() => $scope.coupomSearchActive = true, 50 * animationIndex++);
            }
            if (methods.find(method => method === "CPF")) {
                $timeout(() => $scope.customerDocumentSearchActive = true, 50  * animationIndex++);
            }
            if (methods.find(method => method === "COUPONLESS")) {
                $timeout(() => $scope.scannedProductSearchActive = true, 50  * animationIndex++);
            }
            if (methods.find(method => method === "EPC")) {
                $timeout(() => $scope.rfidProductSearchActive = $scope.rfidReaderConnected, 50  * animationIndex++);
            }
        })();
    })
})();
