(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange.coupons', [])
    .controller('ExchangeTransactionCouponsCtrl', function ($scope,
                                                        $state,
                                                        $stateParams,
                                                        $q,
                                                        UtilService,
                                                        ConfigService) {
        
        $scope.searchCustomer = $stateParams.searchCustomer;
        $scope.transactions = $stateParams.transactions
            .sort((t1, t2) => UtilService.compareTo(() => new Date(t2.key.date).getTime(), () => new Date(t1.key.date).getTime()));
        $scope.transactionFilter = $scope.transactions;
        $scope.productCodeType = ConfigService.config.business.product_code_character_type;
        
        if ($scope.searchCustomer && $scope.searchCustomer.document) {
            if ($scope.searchCustomer.documentType == "CPF") {
                $scope.formatDocument = applyCpfMask($scope.searchCustomer.document);
            } else if ($scope.cart.customer.documentType == "CNPJ") {
                $scope.formatDocument = applyCnpjMask($scope.searchCustomer.document);
            } else {
                $scope.formatDocument = $scope.searchCustomer.document;
            }
        };
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.goNext = transaction => {
            var transitionParams = UtilService.cloneDeep($stateParams);
    
            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData.items = transaction.availableFiscalItems
            .map(item => {
                return {
                    transaction: transaction,
                    item: item,
                    quantity: 1
                };
            });
            delete transitionParams.searchCustomer;
            delete transitionParams.transactions;
        
            UtilService.navigateWithoutHistory('easyorder.exchangetransactionproductlist', transitionParams);
        };
        /**
         * 
         */
        $scope.searchProductCupon = value => {
            $q.resolve()
            .then(() => {
                $scope.transactionFilter = $scope.transactions
                    .filter(trns => trns.availableFiscalItems)
                    .filter(trns => productCodeFilter(trns.availableFiscalItems, value));
            });
        };
        /**
         * 
         */
        function productCodeFilter(fiscalItems, code) {
            if (!fiscalItems || !fiscalItems.length) {
                return false;
            }
            if (!code || !code.length) {
                return true;
            }
            return fiscalItems.some(fiscalItem => fiscalItem.productCode.search(new RegExp(code, "gi")) >= 0);
        }
    });
})();
