(function () {
    'use strict';

    angular.module('easyorder.controllers.returns.exchange.customer', [])
    .controller('ExchangeTransactionCustomerIdentificationCtrl', function ($scope,
                                    $q,
                                    $stateParams,
                                    $state,
                                    $ionicLoading,
                                    AlertService,
                                    UtilService,
                                    CartService,
                                    LogService,
                                    UserService,
                                    CustomerService) {

        /**
         *
         */
        $scope.validateDocument = () => {
            var document = $scope.customer.document;
            $scope.documentInvalid = false;

            if (document) {
                if (document.length == 11 && !validaCPF(document)) {
                    $scope.documentInvalid = true;
                    AlertService.warn("CPF inválido");
                }
            }
        };
        /**
         *
         */
        $scope.isUserDoingSelfService = () => {
            const user = UserService.currentUser;

            if (user && user.document && $scope.customer.document && user.document ===  $scope.customer.document) {
                return true;
            }
            return false;
        };
        /**
         *
         */
        $scope.goBack = () => {
            UtilService.navigateWithoutHistory("easyorder.operations");
        };
        /**
         *
         */
        $scope.goNext = () => {
            if ($scope.isUserDoingSelfService()) {
                AlertService.warn('Proibido autosserviço!');
                return;
            }

            $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            }).then(() => {
                getCustomer()
                .then($ionicLoading.hide)
                .then(() => {
                    var auths = {};
                    auths[EXCHANGE_OPERATION] = $stateParams.authorizer;

                    const transitionParams = {
                        flowHistory: {
                            flow: [$state.current.name],
                            params: [$stateParams]
                        },
                        authorizations: auths,
                        customer: $scope.customer
                    };
                    UtilService.navigateWithoutHistory('easyorder.exchangemenu', transitionParams);
                });
            });

        };
        /**
         *
         */
         $scope.goNextV3 = () => {
            if ($scope.isUserDoingSelfService()) {
                AlertService.warn('Proibido autosserviço!');
                return;
            }

            $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            }).then(() => {
                getCustomer()
                .then($ionicLoading.hide)
                .then(() => {
                    const transitionParams = {
                        flowHistory: {
                            flow: [$state.current.name],
                            params: [$stateParams]
                        },
                        customer: $scope.customer
                    };
                    UtilService.navigateWithoutHistory('easyorder.exchangemenuv3', transitionParams);
                });
            });

        };

        function getCustomer() {
            var defered = $q.defer();

            CustomerService.searchByType($scope.customer.document, 'CPF').then(result => {
                if (result.length) {
                    $scope.customer = {
                        customerId: result[0].customerId,
                        document: result[0].document,
                        documentType: result[0].documentType,
                        email: $scope.customer.email,
                        name: result[0].name,
                        address: result[0].address
                    };
                }
            }).catch((error) => {
                LogService.warn(`Não foi possível consultar CPF [${$scope.customer.document}]: ${error.message}`);
            }).finally(defered.resolve);

            return defered.promise;
        }

        (() => {
            $scope.customer = {}
            $scope.customer.document = $stateParams.customer && $stateParams.customer.document ? $stateParams.customer.document : '';
            $scope.customer.documentType = 'CPF';

            if (CartService.cart.customer) {
                $scope.customer.email = CartService.cart.customer.email;

                if (CartService.cart.customer.documentType === 'CPF') {
                    $scope.customer.document = CartService.cart.customer.document;
                }
            }
            $scope.validateDocument();
        })();
    });
})();
