(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange.products.list', [])
    .controller('ExchangeTransactionProductListCtrl', function ($scope, $rootScope,
                                    $q,
                                    $state,
                                    $stateParams,
                                    $ionicPopup,
                                    $ionicListDelegate,
                                    $ionicLoading,
                                    AlertService,
                                    UtilService,
                                    ConfigService,
                                    LogService,
                                    TransactionReturnService) {


        /**
         * Ao voltar a lista de podutos é limpa
         */
        $scope.goBack = () => {
            var defer = $q.defer();

            if ($scope.displayItems && $scope.displayItems.length) {
                AlertService.confirm('Deseja desistir dos itens já informados?<br><b>Eles serão descartados.</b>').then(defer.resolve, defer.reject);
            } else {
                defer.resolve(true);
            }

            defer.promise.then((result) => {
                if (result && $stateParams.flowHistory) {
                    const backState = $stateParams.flowHistory.flow.pop();
                    const params = $stateParams.flowHistory.params.pop();

                    if (params.exchangeData.items) {
                        params.exchangeData.items.length = 0;
                    }
                    UtilService.navigateWithoutHistory(backState, params);
                }
            });
        };
        /**
         *
         */
        $scope.addMoreItems = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                params.exchangeData.items = $scope.displayItems;

                if  (!$stateParams.exchangeData.couponlessFlowFromEPC) {
                    UtilService.navigateWithoutHistory(backState, params);
                } else {
                    UtilService.navigateWithoutHistory('easyorder.exchangerfidtagtransactionsearch', params);
                }
            }
        };
        /**
         *
         */
        $scope.goNext = () => {
            const selectedItems = $scope.displayItems.filter(displayItem => displayItem.selected);

            if (selectedItems.length) {
                var transitionParams = UtilService.cloneDeep($stateParams);

                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.exchangeData.items = selectedItems;

                if (ConfigService.config.business.enables_exchange_rfid_reading && $stateParams.exchangeData.rfidReaderConnected) {
                    UtilService.navigateWithoutHistory('easyorder.exchangetransactionrfidlinkage', transitionParams);
                } else {
                    UtilService.navigateWithoutHistory('easyorder.exchangetransactionrequest', transitionParams);
                }
            } else {
                AlertService.warn("Selecione um produto da lista para prosseguir.");
            }
        };
        /**
         *
         */
        $scope.removeItem = (displayItem) => {
            AlertService.confirm(`Deseja remover o item da troca?`, undefined, `Atenção`, `Sim, remover`)
            .then(result => {
                if (result) {
                    LogService.info(`Removendo item da lista da troca: ${displayItem.item.epcTag ? displayItem.item.epcTag : displayItem.item.productCode}`);
                    let index;

                    if ((index = $scope.couponItems.indexOf(displayItem)) > -1) {
                        $scope.couponItems.splice(index, 1);
                    } else if ((index = $scope.couponlessItems.indexOf(displayItem)) > -1) {
                        $scope.couponlessItems.splice(index, 1);
                    }
                    $scope.displayItems.splice($scope.displayItems.indexOf(displayItem), 1);
                    update();
                    if (!$scope.displayItems.length) {
                        $scope.addMoreItems();
                    }
                } else {
                    $ionicListDelegate.closeOptionButtons();
                }
            })
        }
        /**
         *
         */
        $scope.anySelectedItem = () => {
            if ($scope.displayItems) {
                return $scope.displayItems
                .some(displayItem => !!displayItem.reason);
            }
            return false;
        };
        /**
         *
         */
        function closePopup() {
            if ($scope.popup) {
                $scope.popup.close();
                $scope.popup = undefined;
            }
        };
        /**
         *
         */
        function update() {
            const selectedItems = $scope.displayItems.filter(displayItem => displayItem.selected);
            $scope.selectedCount = selectedItems.length;
            $scope.totalValue = selectedItems.map(displayItem => displayItem.item.unitValue * displayItem.quantity)
                                .reduce((acc, val) => acc + val, 0);
        }
        /**
         *
         */
        $scope.showReason = (displayItem) => {
            $ionicListDelegate.closeOptionButtons();

            if ($scope.reasons.length) {
                $scope.reasonItemSelected = displayItem.item;

                $scope.reasons.forEach(it => {
                    it.selected = false;
                });

                $ionicPopup.show({
                    templateUrl: 'partials/productrexchangereason.html',
                    scope: $scope,
                    cssClass: 'exchange-reason-popup-class',
                    buttons: [{
                        text: 'CANCELAR',
                        onTap: _ => {
                            closePopup();
                        }
                    },
                    {
                        text: '<b>OK</b>',
                        type: 'button-theme',
                        onTap: _ => {
                            const reason = $scope.reasons.find(it => it.selected);

                            if (reason) {
                                LogService.info(`Motivo ${reason.description} escolhido para ${$scope.reasonItemSelected.productCode}-${$scope.reasonItemSelected.epcTag ? $scope.reasonItemSelected.epcTag : 'N/A'}`);
                                displayItem.reason = reason;
                            }
                            displayItem.selected = !!displayItem.reason;
                            update();
                            closePopup();
                        }
                    }]
                });
            } else {
                LogService.warn(`Nenhum motivo está presente na lista. A tela de motivos não pode ser exibida.`)
            }
        };
        /**
         * Callback da seleção de motivos. Controla a seleção única na tela.
         */
        $scope.reasonSelected = (item) => {
            $scope.reasons.forEach(it => it.selected = item === it);
        };

        $scope.getItemDisplayCode = (item) => {
            return $rootScope.getItemDisplayCode(item);
        }

        /**
         *
         */
        function listReasons() {
            return TransactionReturnService.searchReasons()
            .then(result => $scope.reasons = result.data.reasons)
            .catch(error => {
                AlertService.restError(error, "Não foi possível obter lista de motivos para troca!")
                .then($scope.goBack);
            });
        }
        /**
         *
         */
         function listReasonsV3() {
            return TransactionReturnService.searchReasons({operatioType: "VOUCHER_RETURN"})
            .then(result => $scope.reasons = result.data.reasons)
            .catch(error => {
                AlertService.restError(error, "Não foi possível obter lista de motivos para troca!")
                .then($scope.goBack);
            });
        }
        /**
         *
         */
        (() => {
            if ($stateParams.exchangeData.items && $stateParams.exchangeData.items.length) {
                $scope.showAddMoreItems = $stateParams.exchangeData.showAddMoreItems;
                $scope.selectedCount = 0;

                $ionicLoading.show()
                .then(() => $scope.displayItems = $stateParams.exchangeData.items.map(item => {return {...item}})) //Clonando os itens.
                .then(() => {
                    $scope.couponItems = $scope.displayItems.filter(item => !!item.transaction);
                    $scope.couponlessItems = $scope.displayItems.filter(item => !item.transaction);
                })
                .then(listReasons)
                .then(update)
                .finally($ionicLoading.hide);
            } else {
                AlertService.warn('Não há itens para troca.')
                .then(() => UtilService.navigateWithoutHistory('easyorder.exchangemenu'));
            }
        })();
    })
})();
