(function () {
    'use strict';

    angular.module('easyorder.controllers.returns.exchange.requestv3', [])
    .controller('ExchangeTransactionRequestCtrlV3', function ($scope,
                                    $rootScope,
                                    $stateParams,
                                    $timeout,
                                    $q,
                                    $ionicPopup,
                                    $ionicLoading,
                                    ConfigService,
                                    AlertService,
                                    UtilService,
                                    CartService,
                                    LogService,
                                    TransactionReturnService,
                                    UserService) {

        /**
         *
         */
        $scope.clearDisplay = () => {
            return $timeout(() => {
                $scope.message = "";
                LogService.info("clearDisplay");
            });
        };
        /**
        *
        */
       $scope.displayMessage = (message) => {
           return $timeout(() => {
               $scope.message = message;
               LogService.info("displayMessage: " + $scope.message);
           });
       };
       /**
        *
        */
       $scope.goBack = () => {
           if ($stateParams.flowHistory) {
               const backState = $stateParams.flowHistory.flow.pop();
               const params = $stateParams.flowHistory.params.pop();

               UtilService.navigateWithoutHistory(backState, params);
           }
       };
       /**
        *
        * @returns
        */
       function doLog() {
           var itensTxt = `Itens para a troca:`;

           $stateParams.exchangeData.items
           .map(rt => `[${rt.transaction ? rt.transaction.key.storeNumber + "|" + rt.transaction.key.terminal + "|" + rt.transaction.key.date + "|" + rt.transaction.key.nsu + "\t"
                            : 'SEM CUPOM\t\t\t'}]: Produto ${rt.item.productCode}-${rt.item.epcTag}`)
           .forEach(strItem => itensTxt += `\n\t ${strItem}`);

           LogService.info(itensTxt);
        }
        /**
         *
         */
        function doRequest() {
            const dataMap = groupItemsByTransaction($stateParams.exchangeData.items);
            const requestList = [...dataMap].map(arr => createRequest(arr[0] === 'NO-TR' ? undefined: arr[0], arr[1]));
            const request = requestList[0];
            TransactionReturnService.return(request)
            .then(result => {
                return $scope.displayMessage("VT gerado com sucesso!")
                .then(() => {
                    CartService.cart.customer = $stateParams.customer;

                    return UtilService.navigateWithoutHistory('easyorder.couponview', {
                        pdf: result.pdf,
                        printContents: result.printContents,
                        documentTitle: result.documentTitle || 'Cupom_vale_troca'
                    });
                });
            })
            .finally($scope.clearDisplay)
            .catch(error => {
                AlertService.restError(error, 'Não foi possível prosseguir com a geração do VT.')
                .then($scope.goBack);
            });

            // const requestList = [...dataMap].map(arr => createRequest(arr[0] === 'NO-TR' ? undefined: arr[0], arr[1]));

            // $scope.displayMessage("Validando operação de troca...")
            // .then(() => validateReturnRequest(requestList))
            // .then(() => {return {returnRequests: requestList}})
            // .finally(() => $scope.displayMessage("Realizando operação de troca..."))
            // .then(batchRequest => TransactionReturnService.return(batchRequest[0]))
            // .then(result => {
            //     return $scope.displayMessage("VT gerado com sucesso!")
            //     .then(() => {
            //         CartService.cart.customer = $stateParams.customer;

            //         return UtilService.navigateWithoutHistory('easyorder.couponview', {
            //             pdf: result.pdf,
            //             printContents: result.printContents,
            //             documentTitle: result.documentTitle || 'Cupom_vale_troca'
            //         });
            //     });
            // })
            // .finally($scope.clearDisplay)
            // .catch(error => {
            //     AlertService.restError(error, 'Não foi possível prosseguir com a geração do VT.')
            //     .then($scope.goBack);
            // });
        }
        /**
         *
         */
        function groupItemsByTransaction(items) {
            let dataMap = new Map();

            for (const retItem of items) {
                const key = retItem.transaction ? retItem.transaction : 'NO-TR'
                let existingItemList = dataMap.get(key);

                if (!existingItemList) {
                    existingItemList = [];
                    dataMap.set(key, existingItemList);
                }
                existingItemList.push(retItem);
            }
            return dataMap;
        }
        /**
         *
         */
        function createRequest(sourceTransaction, returnItems) {
            let request = {};

            request.appInfo = {
                name: $rootScope.APP_NAME,
                version: $rootScope.appVersion
            };
            if (sourceTransaction && sourceTransaction.key) {
                request.sourceTransactionKey = sourceTransaction.key;
                request.sourceTransactionTotalValue = sourceTransaction.totalValue;
            }
            request.transactionKey = {
                companyNumber: ConfigService.config.companyNumber,
                storeNumber: ConfigService.config.storeNumber,
                date: new Date(),
                nsu: ConfigService.nsuControll.incrementNSU(),
                terminal: ConfigService.config.terminalNumber,
            };
            request.operationType = 'VOUCHER_RETURN';
            request.operator = {
                code: UserService.operatorCode(),
                name: UserService.operatorName()
            };
            request.authorizations = null;
            const authorizer = $stateParams.authorizations[EXCHANGE_OPERATION];
            request.authorizer = { //A validação da troca ainda usa essa propriedade.
                code: (isEmpty(authorizer)) ? request.operator.code : authorizer
            };
            request.customer = $stateParams.customer;
            request.totalValue = returnItems.reduce((acc, item) => acc + (item.quantity * item.item.unitValue), 0);
            request.fiscalItems = processFiscalItems(returnItems);

            if (sourceTransaction) {
                request.orderNumber = sourceTransaction.orderNumber;
            }
            return request;
        }
        /**
         *
         */
        function processFiscalItems(returnItems) {
            const availableFiscalItemsToReturn = returnItems
            .map(returnItem => {
                const ret = returnItem.item;
                ret.quantitySold = returnItem.quantity;
                ret.reason = returnItem.reason;

                return ret;
            });
            return availableFiscalItemsToReturn;
        }
        /**
         *
         */
        function isEmpty(string) {
            return string == null || string.length == 0
        }

        function getFormmatedReason(rejected403) {
            const rejectedUniqueValues = [...new Set([...rejected403.values()])];
            let message = rejectedUniqueValues.join(", ");
            message = (isEmpty(message)) ? 'Operação não permitida.' : message.trim();
            message = "Motivo: ".concat(message);
            if (!message.endsWith('.')) {
                message = message.concat('.');
            }
            message = message.concat(' Deseja solicitar autorização?');
            return message;
        };

        function validateReturnRequest(requestList) {
            const promises = [];

            requestList.forEach(request => promises.push(TransactionReturnService.validateReturnRequest(request).catch(error => error)));

            return $q.all(promises)
            .then(results => {
                let ret = $q.resolve();
                const rejected403 = new Map();

                for (var index = 0; index < results.length; index++) {
                    let result = results[index];

                    if (result.status) {
                        if (result.status === 403) {
                            rejected403.set(index, result.data.message);
                        } else {
                            ret = $q.reject(result);
                            rejected403.clear();
                            break;
                        }
                    }
                }

                if (rejected403.size) {
                    ret = $ionicPopup.alert({
                        title: 'Operação não permitida',
                        template: getFormmatedReason(rejected403),
                        scope: $scope,
                        okType: "button-theme",
                        buttons: [
                            {   text: 'Cancelar',
                                onTap: function(e) { return false; }
                            }, {
                                text: '<b>Ok</b>',
                                type: "button-theme",
                                onTap: function(e) { return true; }
                            }]
                    })
                    .then(accepted => {
                        if (accepted) {
                            return processRequestAuth().then(userCode => {
                                let newRequestList = requestList
                                .filter((_, index) => !!rejected403.get(index))
                                .map(request => {
                                    request.authorizations[EXCHANGE_OPERATION] = userCode.code;
                                    request.authorizer = { //A validação da troca ainda usa essa propriedade.
                                        code: request.authorizations[EXCHANGE_OPERATION]
                                    };
                                    return request;
                                });
                                return validateReturnRequest(newRequestList);
                            });
                        } else {
                            return $q.reject();
                        }
                    });
                }
                return ret;
            })
        }
        /**
         *
         */
        async function processRequestAuth() {
            return await UserService.requestExternalAuthorization([{id: EXCHANGE_OPERATION, description: 'Operação de Troca'}])
            .then(userCode => {
                return {
                    code: userCode
                };
            })
            .finally($scope.clearDisplay);
        }
        /**
         *
         */
        (() => {
            $ionicLoading.hide()
            .then(() => {
                $scope.showWatermark = true;
                $scope.message = "";
                $scope.authorizer = undefined;
                $scope.authorizations = $stateParams.authorizations;

                doLog();
                doRequest();
            });
        })();
    })
})();
