(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange.rfid.activation', [])
    .controller('ExchangeTransactionRfidActivationCtrl', function ($scope,
                                                                $q,
                                                                $timeout,
                                                                $stateParams,
                                                                $ionicLoading,
                                                                UtilService,
                                                                ConfigService,
                                                                LogService,
                                                                AlertService,
                                                                UserService,
                                                                BluebirdCaptureService) {
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * Essa função não registra este controlador no FlowHistory porque essa tela não deve estar na pilha do histórico.
         * Além disso, modifica o histórico para que o controlador de retorno seja a seleção dos motivos para troca.
         */
        $scope.goNext = () => {
            return $timeout(() => {}, 1000)
            .then($ionicLoading.show)
            .then(() => {
                $stateParams.flowHistory.flow.pop(); // Removendo a tela de vinculo de rfid do histórico
                $stateParams.flowHistory.params.pop();
            })
            .then(() => UtilService.navigateWithoutHistory('easyorder.exchangetransactionrequest', getTransitionParams()));
        };
        /**
         *
         */
        $scope.goNextWithoutActivation = () => {
            return AlertService.confirm('Produtos ainda não ativados. Deseja prosseguir?')
            .then(result => result ? $q.resolve() : $q.reject())
            .then(() => $scope.skipActivation = true)
            .then(() => UserService.requestAuthorization(EXCHANGE_RFID_TAGS_ACTIVATION, 'Troca sem ativação RFID'))
            .then(userCode => $stateParams.authorizations[EXCHANGE_RFID_TAGS_ACTIVATION] = userCode)
            .then($scope.goNext);
        };
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultWritingPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastWritingPowerSet = $scope.data.defaultWritingPower)
            .then(() => ConfigService.config.business.defaultReadingPower = $scope.data.lastWritingPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultWritingPower = $scope.data.lastWritingPowerSet);
            });
        };
        /**
         *
         */
        $scope.startWriting = () => {
            let epcs = [...$scope.data.activationMap.keys()];
            
            if (epcs.length) {
                LogService.info(`Cabeçalho configurado para ativação: '${$scope.data.enablePrefix}'`);
                
                var epcsSelectionForWriting = epcs.map(epc => $scope.data.disablePrefix + epc);
                LogService.info(`EPCs usados para seleção na ativação: ${epcsSelectionForWriting.join(', ')}`);

                return $ionicLoading.show({
                    template: 'Conectando ao Leitor RFID...',
                    duration: 0
                })
                .then(() => BluebirdCaptureService.connect($scope.data.defaultWritingPower))
                .then(() => BluebirdCaptureService.changeEPCHeader(
                    epcsSelectionForWriting, // O plugin irá ignorar as tags que caso já estejam fisicamente habilitadas.
                    $scope.data.enablePrefix,
                    changeHeaderSuccessCallback,
                    changeHeadererrorCallback))
                .then(() => $scope.writingInProgress = true)
                .finally($ionicLoading.hide)
                .catch(error => {
                    let promise;
                    let data = JSON.parse(error);
                    $scope.writingInProgress = false;
                    
                    if (data.errorCode == '-32') {
                        promise = AlertService.warn('Leitor RFID não conectado.');
                    } else {
                        promise = AlertService.warn(`Não foi possível conectar ao Sled [${data.errorCode}]`);
                    }
                    return promise.finally(() => $q.reject());
                });
            } else {
               return $ionicLoading.hide();
            } 
        };
        /**
         *
         */
        $scope.clearActivationMap = () => {
            LogService.info(`Reiniciando processo de reativação de items...`);

            let epcs = [...$scope.data.activationMap.keys()];
            epcs.forEach(epc => $scope.data.activationMap.set(epc, undefined)); 
            
            BluebirdCaptureService.disconnect()
            .then($scope.startWriting);
        };
        /**
         *
         */
        $scope.epcTagActivationStatus = (epcTag) => {
            if (epcTag) {
                return $scope.data.activationMap.get(epcTag.substring($scope.data.enablePrefix.length));
            } else { 
                return null;
            }
        };
        /**
         *
         */
        function setEPCTagActivationStatus(epcTag, status) {
            return $timeout(() => {
                $scope.data.activationMap.set(epcTag.substring($scope.data.enablePrefix.length), status);
            });
        }
        /**
         *
         */
        async function changeHeaderSuccessCallback(data) {
            const epc = data.originalEPC;
            LogService.info(`Retorno da alteração usando seleção '${epc}' foi recebido.`);
            
            await setEPCTagActivationStatus(epc, data.status);
            updateWritingStatus();
            LogService.info(`EPC '${epc}' teve o cabeçalho alterado.`);
            
            return isAllDone()
            .then($scope.goNext);
        }
        /**
         *
         */
        async function changeHeadererrorCallback(error) {
            if (error.data) {
                let data = JSON.parse(error.data);
                const epcUsedInSelection = data.originalEPC;
                
                if (error.errorCode == "-32") {
                    LogService.info(`Tempo para obter acesso à tag foi excedido para seleção '${epcUsedInSelection}'. Nenhuma alteração realizada.`);
                } else {
                    LogService.info(`'A seleção ${epcUsedInSelection}' não teve o cabeçalho alterado: ${error.message} [${error.errorCode}]`);
                }
                await setEPCTagActivationStatus(epcUsedInSelection, data.status);
                updateWritingStatus();
            }
            return isAllDone()
            .then($scope.goNext);
        }
        /**
         *
         */
        function updateWritingStatus() {
            let status = [...$scope.data.activationMap.values()];
            $scope.writingInProgress = status.find(st => !st);
        }
        /**
         *
         */
        function isAllDone() {
            let itemsToReturn = $scope.data.itemsToReturn;
            let ret = $q.resolve();
            
            const items = itemsToReturn
            .map(retItem => retItem.item)
            .filter(item => !!item.epcTag); //Apenas os itens que possuem EPC.
            
            for (const item of items) {
                const epcActivaionStatus = $scope.epcTagActivationStatus(item.epcTag);
                
                if (!epcActivaionStatus || epcActivaionStatus === 'ERROR') {
                    ret = $q.reject();
                    break;
                }
            }
            return ret;            
        };
        /**
         *
         */
        async function loadActivationMap(items) {
            items = items
            .map(itemRet => itemRet.item)
            .filter(item => !!item.epcTag); //Apenas os itens que possuem EPC.
            
            for (const item of items) {
                await setEPCTagActivationStatus(item.epcTag, undefined);
            }
        }
        
        function getTransitionParams() {
            let transitionParams = UtilService.cloneDeep($stateParams);
            if ($scope.skipActivation) {
                resetEpcItems(transitionParams.exchangeData.items);
            }
            return transitionParams;
        }

        function resetEpcItems(items) {
            for (const exchangeItem of items) {
                if (exchangeItem && exchangeItem.item && exchangeItem.item.epcTag) {
                    exchangeItem.item.epcTag = $scope.data.disablePrefix + exchangeItem.item.epcTag.substring($scope.data.disablePrefix.length);
                }
            }
        }
        /**
         *
         */                                               
        (() => {
            $scope.writingInProgress = true;
            $scope.skipActivation = false;
            $scope.data = {
                itemsToReturn: $stateParams.exchangeData.items,
                lastWritingPowerSet: ConfigService.config.business.rfid_writing_power,
                defaultWritingPower: ConfigService.config.business.rfid_writing_power,
                activationMap: new Map(),
                enablePrefix: ConfigService.config.business.epc_enable_prefix,
                disablePrefix: ConfigService.config.business.epc_disable_prefix,
            };
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            
            loadActivationMap($scope.data.itemsToReturn)
            .then($scope.startWriting)
            .catch($scope.goBack);
        })();
    })
})();