(function() {
    'use strict';

    angular.module('easyorder.controllers.returns.exchange.rfid.tag.linkage', [
    ])
    .controller('ExchangeTransactionRFIDLinkageCtrl', function($scope,
                                            $stateParams,
                                            $q,
                                            $state,
                                            $timeout,
                                            $ionicLoading,
                                            UtilService,
                                            ConfigService,
                                            LogService,
                                            AlertService,
                                            UserService,
                                            RFIDTransactionService,
                                            BluebirdCaptureService) {
        var tagsReadCount = 0;
        /**
         *
         */
        $scope.$watch("sledTrigger.status", function(status) {
            if ('TRIGGER_PRESSED' === status) {
                tagsReadCount = 0;
            }
        });
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        function goNext() {
            return $timeout(() => {}, 1000)
            .then($ionicLoading.show)
            .then(() => {
                var transitionParams = UtilService.cloneDeep($stateParams);
                
                if ($scope.anyMissingEPCItems) {
                    transitionParams.flowHistory.flow.push($state.current.name);
                    transitionParams.flowHistory.params.push($stateParams);
                }
                
                return transitionParams;
            })
            .then(transitionParams => UtilService.navigateWithoutHistory('easyorder.exchangetransactionrfidactivation', transitionParams));
        };
        /**
         *
         */
        $scope.goNextWithoutLinkage = () => {
            return AlertService.confirm('Produtos ainda não verificados/vinculados. Deseja prosseguir?')
            .then(result => result ? $q.resolve() : $q.reject())
            .then(() => UserService.requestAuthorization(EXCHANGE_RFID_TAGS_ACTIVATION, 'Troca sem vínculo RFID'))
            .then(userCode => $stateParams.authorizations[EXCHANGE_RFID_TAGS_ACTIVATION] = userCode)
            .then(goNext);
        };
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultReadingPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastReadingPowerSet = $scope.data.defaultReadingPower)
            .then(() => ConfigService.config.business.defaultReadingPower = $scope.data.lastReadingPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultReadingPower = $scope.data.lastReadingPowerSet);
            });
        };
        /**
         *
         */
        $scope.epcTagReadingStatus = epcTag => {
            if (epcTag) {
                return $scope.data.epcReadingMap.get(epcTag.substring($scope.data.enablePrefix.length));
            } else {
                return null;
            }
        };
        /**
         *
         */
        function setEPCTagReadingStatus(epcTag, status) {
            return $timeout(() => {
                $scope.data.epcReadingMap.set(epcTag.substring($scope.data.enablePrefix.length), status);
            }, 50 * tagsReadCount);
        }
        /**
         *
         */
        $scope.startReading = () => {
            return $ionicLoading.show({
                template: 'Conectando ao Leitor RFID...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.connect($scope.data.defaultReadingPower))
            .then(() => BluebirdCaptureService.startReading(
                readingCallbackResult, 
                data => LogService.info(`Erro na leitura SLED: ${data.message}`)))
            .finally($ionicLoading.hide)
            .catch(error => {
                if (ConfigService.config.business.enables_rfid_only) {
                    AlertService.alert('Leitor RFID não conectado', 'Retorne após conectar o leitor RFID para prosseguir com a operação')
                    .then($scope.goBack);
                } else {
                    let promise;
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                    promise = AlertService.warn('Leitor RFID não conectado.');
                    } else {
                    promise = AlertService.warn(`Erro de conexão do Leitor RFID [${data.errorCode}].`);
                    }
                    return promise.finally(() => $q.reject());
                }
            });
        };
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        async function readingCallbackResult(data) {
            LogService.info(`Leitura SLED: ${data.raw}`);
            tagsReadCount++;
            
            const existingItem = $scope.data.itemsToReturn
            .map(retItem => retItem.item)
            .find(item => item.epcTag && item.epcTag.substring($scope.data.enablePrefix.length) === data.raw.substring($scope.data.enablePrefix.length));
            
            if (!existingItem) {
                const elegibleItems = getElegibleItemsForLinkage(data.raw, data.productCode);
                let foundItem = elegibleItems[0];
                                
                if (foundItem) {
                    LogService.info(`Criando vínculo entre produto ${foundItem.productCode} e EPC ${data.raw}.`);
                    foundItem.epcTag = data.raw;
                    
                    await setEPCTagReadingStatus(foundItem.epcTag, "NEW");
                } else {
                    LogService.info(`A etiqueta ${data.raw} não pertence a esta troca e não é possível atribuí-la a um item.`);
                }
            } else {
                LogService.info(`Produto ${existingItem.productCode} confirmado com EPC ${data.raw}.`);
                await setEPCTagReadingStatus(existingItem.epcTag, "OK");
            }
            await linkageCatalog()
            .then(processItemLinkage)
            .finally(() => {
                isAllDone()
                .then(goNext);
            });
        }
        /**
         *
         */
        function isAllDone() {
            let ret = $q.resolve();
            
            for (const retItem of $scope.data.itemsToReturn) {
                if (!$scope.epcTagReadingStatus(retItem.item.epcTag)) {
                    ret = $q.reject();
                    break;
                }
            }
            return ret;            
        }
        /**
         *
         */
        function linkageCatalog() {
            let items = $scope.data.itemsToReturn;
            var ret = $q.reject();
            
            if (!hasMissingEPCItems()) {
                let itemsToLinkage = items
                .filter(retItem => retItem.transaction && $scope.epcTagReadingStatus(retItem.item.epcTag) === "NEW"); //Tem transação e é novo? Associar.
                
                if (itemsToLinkage.length) {
                    itemsToLinkage.forEach(retItem => {
                        retItem.item.epcTagAlarmDisabled = retItem.item.epcTag.startsWith($scope.data.disablePrefix); 
                        delete retItem.item.onlineInventoryIntegration; //Forçando o uso valor default da API REST
                    });
                    ret = $q.resolve(itemsToLinkage);
                }
            }
            return ret;
        }
        /**
         *
         */
        function hasMissingEPCItems() {
            return !!$scope.data.itemsToReturn
                .map(retItem => retItem.item)
                .filter(item => !item.epcTag).length;
        }
        /**
         * Essa função busca itens sem EPC na lista para de itens da troca, cuja a transação de venda, se houver, não possui  
         * o EPC num item já trocado, pois isto faria com que o EPC fosse atribuído duas vezes à mesma venda.
         */
        function getElegibleItemsForLinkage(epc, productCode) {
            const ret = getMissingEPCItems()
            .filter(returnItem => !returnItem.transaction 
                || !returnItem.transaction.returnedFiscalItems
                || !returnItem.transaction.returnedFiscalItems.length
                || !returnItem.transaction.returnedFiscalItems
                    .find(item => item.epcTag && item.epcTag.substring($scope.data.enablePrefix.length) === epc.substring($scope.data.enablePrefix.length)))
            .map(returnItem => returnItem.item)
            .filter(item => item.productCode === productCode);
            
            LogService.info(`Itens elegíveis para '${epc}-${productCode}': ${ret.length}`);
            
            return ret;
        }
        /**
         *
         */
        function getMissingEPCItems() {
            const ret = $scope.data.itemsToReturn
            .filter(returnItem => !returnItem.item.epcTag);

            return ret;
        }
        /**
         *
         */
        function processItemLinkage(retItems) {
            var ret = $q.resolve();
            
            if (retItems && retItems.length) {
                let promises = [];
                let dataMap = new Map();
                
                for (const retItem of retItems) { // Agrupando itens por transação.
                    let existingItemList = dataMap.get(retItem.transaction.key);
                    
                    if (!existingItemList) {
                        existingItemList = [];
                        dataMap.set(retItem.transaction.key, existingItemList);
                    }
                    existingItemList.push(retItem.item);
                }
                
                for (let [key, items] of dataMap) {
                    LogService.info(`Enviando dados do vínculo para a transação: ${key}`);
                    
                    let promise = $ionicLoading.show()
                    .then(() => {
                        return {
                            transactionKey: {
                                companyNumber:  ConfigService.config.companyNumber,
                                storeNumber:    ConfigService.config.storeNumber,
                                nsu:            ConfigService.nsuControll.incrementNSU(),
                                terminal:       ConfigService.config.terminalNumber,
                                date:           new Date().toISOString()
                            },
                            sourceTransactionKey: {
                                companyNumber:  key.companyNumber,
                                storeNumber:    key.storeNumber,
                                nsu:            key.nsu,
                                terminal:       key.terminal,
                                date:           key.date
                            },
                            fiscalItems: items,
                            operator: {
                                code: UserService.operatorCode(),
                                name: UserService.operatorName()
                            },
                            authorizer: {
                                code: UserService.operatorCode(),
                                name: UserService.operatorName()
                            },
                            operation: 'EXCHANGE'
                        }
                    })
                    .then(RFIDTransactionService.itemLinkage)
                    .then(() => items.forEach(item => {
                        LogService.info(`A tag '${item.epcTag}' foi vinculada ao produto ${item.productCode} da transação [${key.storeNumber}|${key.terminal}|${key.date}|${key.nsu}]`);
                        setEPCTagReadingStatus(item.epcTag, "LINKED");
                     }))
                    .finally($ionicLoading.hide)
                    .catch(() => {
                        return AlertService.confirm(`Falha ao integrar dados de vínculo de EPC, deseja tentar novamente?`)
                        .then(tryAgain => {
                            if (tryAgain) {
                                return processItemLinkage(retItems);
                            } else {
                                LogService.info(`O operador desistiu do vínculo de EPC...`);
                                return $q.resolve();
                            }
                        });
                    });
                    promises.push(promise);
                }
                ret = $q.all(promises);
            }
            return ret;
        }
        /**
         *
         */
        $scope.clearReadingMap = () => {
            $scope.data.epcReadingMap.forEach((value, key) => {
                if (value === "NEW") {
                    const item = $scope.data.itemsToReturn
                    .map(retItem => retItem.item)
                    .find(item => item.epcTag.substring($scope.data.enablePrefix.length) === key);
                    
                    delete item.epcTag;
                }
                $scope.data.epcReadingMap.set(key, undefined);
            }); 
        };
        /**
         *
         */                                               
        (() => {
            $scope.data = {
                itemsToReturn: $stateParams.exchangeData.items,
                lastReadingPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultReadingPower: ConfigService.config.business.rfid_reading_power,
                epcReadingMap: new Map(),
                enablePrefix: ConfigService.config.business.epc_enable_prefix,
                disablePrefix: ConfigService.config.business.epc_disable_prefix
            };
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.anyMissingEPCItems = getMissingEPCItems().length > 0;

            if ($scope.anyMissingEPCItems) {
                $scope.startReading()
                .catch($scope.goBack);
            } else {
                goNext();
            }
        })();
    });
})();