(function() {
    'use strict';

    angular.module('easyorder.controllers.returns.exchange.scanned.rfid.couponless', [
    ])

    .controller('RFIDScannedExchangeCouponlessCtrl', function($scope,
                                        $state,
                                        $stateParams,
                                        $q,
                                        $timeout,
                                        ProductService,
                                        AlertService,
                                        UtilService,
                                        LogService,
                                        ConfigService,
                                        $ionicLoading,
                                        BluebirdCaptureService) {
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.goNext = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);
    
            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.exchangeData.items = $scope.items;
            transitionParams.exchangeData.showAddMoreItems = true;
                        
            return UtilService.navigateWithoutHistory('easyorder.exchangetransactionproductlist', transitionParams);
        };
        /**
         * 
         */
        $scope.addProduct = () => {
            removeFocusFromInput();
            
            onSearch($scope.formParm.automationCode)
            .then($scope.goNext)
            .catch(() => $scope.formParm.automationCode = "");
        };
        /**
         *
         */
        async function removeFocusFromInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_productSearchAutomationCode');
                var angularEl = angular.element(myEl);
                angularEl[0].blur();
            });
        }
        /**
         *
         */
        function onSearch(productId) {
            $scope.loadingState = true;
            
            return $ionicLoading.show({
                template:`Consultando '${productId}'`,
                duration: 0
            })
            .then(() => ProductService.search({ code: productId }))
            .finally($ionicLoading.hide)
            .finally(() => $scope.loadingState = false)
            .then(result => {
                if (result && result.items.length > 0) {
                    var item = result.items[0];

                    if (item.productType == 'MINIMUM_SALE_UNIT') {
                        item.quantitySold = item.minimumSaleUnit;
                    } else {
                        item.quantitySold = 1;
                    }

                    item.totalValue = item.unitValue * item.quantitySold;
                    
                    $scope.items.push({
                        transaction: undefined, 
                        item: item,
                        quantity: 1
                    });
                } else {
                    return $q.reject({data: {message: 'Produto não encontrado!<br/>Código: <b>' + productId + '</b>'}});
                }
            })
            .catch(error => AlertService.restError(error, `Não foi possível consultar item: ${productId}`).then($q.reject));
        }
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function callbackResult(data) {
            LogService.info(`Leitura SLED: ${data.raw}`);
            
            if (!$scope.loadingState) {
                onSearch(data.raw)
                .then($scope.goNext);
            }
        }
        /**
         *
         */
        (() => {
            $scope.loadingState = true;
            $scope.productCodeType = ConfigService.config.business.product_code_character_type;
            $scope.formParm = {};
            $scope.items = $stateParams.exchangeData.items || [];
            $ionicLoading.show({
                template: 'Habilitando ao Leitor. Aguarde...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.barcodeConnect())
            .then(() => BluebirdCaptureService.startReading(callbackResult, error => LogService.info(`Erro na leitura SLED: ${error.message}`)))
            .then(() => $scope.loadingState = false)
            .finally($ionicLoading.hide)
            .catch(error => {
                if (ConfigService.config.business.enables_exchange_by_rfid_only) {
                    AlertService.alert('Leitor RFID não conectado', 'Retorne após conectar o leitor RFID para prosseguir com a operação')
                    .then($scope.goBack);
                } else {
                    let promise;
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                    promise = AlertService.warn('Leitor não conectado.');
                    } else {
                    promise = AlertService.warn(`Não foi possível conectar ao Leitor: [${data.errorCode}]`);
                    }
                    promise.finally(() => UtilService.navigateWithoutHistory('easyorder.operations'));
                }
            });
        })();
    });
})();
