(function () {
    'use strict';
    angular.module('easyorder.controllers.returns.exchange.scanned.search', [])
    .controller('ScannedExchangeCtrl', function ($scope,
                                            $state,
                                            $stateParams,
                                            $ionicLoading,
                                            $q,
                                            ConfigService,
                                            AlertService,
                                            UtilService,
                                            LogService,
                                            TransactionSearchService,
                                            BluebirdCaptureService,
                                            CustomerService) {

          /**
         *
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();

                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.goTyped = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);

            UtilService.navigateWithoutHistory('easyorder.typedexchange', transitionParams);
        };
        /**
         *
         */
         $scope.goTypedV3 = () => {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.customer = $stateParams.customer;

            UtilService.navigateWithoutHistory('easyorder.typedexchangev3', transitionParams);
        };
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        async function callbackResult(data) {
            const dataInput = data.raw;

            if (!$scope.loadingState) {
                return await $q.resolve()
                .then(() => LogService.info(`Leitura SLED-Barcode: ${dataInput}`))
                .then(() => $scope.loadingState = true)
                .then(() => {
                    if (dataInput.length == 44) {
                        return doLogicSearch(dataInput);
                    } else if (dataInput.length == 22) {
                        return doStorexSearch(dataInput);
                    } else if (dataInput.length <= 18) {
                        return doCustomerSearch(dataInput)
                    } else {
                        return $q.reject({data: {message: 'Código de Barras não atende aos parâmetros dessa funcionalidade.'}});
                    }
                })
                .finally(() => $scope.loadingState = false)
                .then((transactions) => {
                    if (!transactions || transactions.length == 0) {
                        return;
                    }
                    if (transactions.length == 1) {
                        return goToTransactionProductList(transactions[0]);
                    }
                    return goToTransactionCupomList(transactions);
                })
                .catch(error => {
                    if (error) {
                        return AlertService.restError(error);
                    }
                });
            }
        }
        /**
         *
         */
        async function doLogicSearch(dataInput) {
            const defer = $q.defer();
            const nsu = dataInput.substr(25, 9);
            const date = new Date();
            const year = `${date.getFullYear().toString().substr(0, 2)}${dataInput.substr(2, 2)}`;
            const month = dataInput.substr(4, 2);
            const maxDay = new Date(year, month - 1, 0).getDate();

            $scope.date = {};
            $scope.date.minDate = `${year}-${month}-01`;
            $scope.date.maxDate = `${year}-${month}-${maxDay}`;
            $scope.date.value = new Date(year, month - 1, 1);

            $scope.bsConfirm = () => {
                $scope.bottomsheet = false;

                const key = {
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    terminal: ConfigService.config.terminalNumber,
                    nsu: nsu,
                    date: $scope.date.value
                };

                $q.resolve()
                .then(() => $ionicLoading.show({ template : 'Consultando transação...', duration : 0 }))
                .then(() =>
                    TransactionSearchService.search({
                        transactionKey: key,
                        provider: 'LOGIC',
                    })
                )
                .finally(() => $ionicLoading.hide())
                .then((result) => defer.resolve([
                    result.transaction,
                ]))
                .catch(err => defer.reject(err));
            };

            $scope.bsCancel = () => {
                $scope.bottomsheet = false;
                defer.reject();
            };

            $scope.bottomsheet = true;

            return defer.promise;
        }
        /**
         *
         */
        async function doStorexSearch(dataInput) {
            const key = {};
            key.storeInfo = {};
            key.storeInfo.companyNumber = '00001';
            key.storeInfo.storeNumber = dataInput.substring(0, 4);

            key.date = dataInput.substring(4, 12);
            const match = key.date.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
            key.date = new Date(match[3], match[2] - 1, match[1], 12, 0, 0, 0).toISOString();

            key.terminal = parseInt(dataInput.substring(12, 15));
            key.nsu = parseInt(dataInput.substring(15, 22));

            return $q.resolve()
            .then(() => $ionicLoading.show({ template : 'Consultando transação...', duration : 0 }))
            .then(() =>
                TransactionSearchService.search({
                    transactionKey: key,
                    provider: 'STOREX',
                })
            )
            .finally($ionicLoading.hide)
            .then((result) => ([
                result.transaction,
            ]));
        }
        /**
         *
         */
        function doCustomerSearch(dataInput) {
            return $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            })
            .then(() => CustomerService.searchByType(dataInput, 'TYPED_CARD'))
            .finally($ionicLoading.hide)
            .then(result => {
                if (result.length) {
                    $scope.searchCustomer = {
                        customerId: result[0].customerId,
                        document: result[0].document,
                        documentType: result[0].documentType,
                        email: result[0].email,
                        name: result[0].name,
                        address: result[0].address
                    };
                }
            })
            .then(() => searchCustomerTransacations({customer: {document: $scope.searchCustomer.document}}))
            .then(filterAvailableTransactions);
        }
        /**
         *
         */
        function searchCustomerTransacations(customer) {
            $ionicLoading.show({
                template : 'Consultando transações do cliente...',
                duration : 0
            });
            return TransactionSearchService.list({
                ...customer,
                provider: "STOREX",
            }).finally(() => $ionicLoading.hide());
        };
        /**
         *
         */
        function filterAvailableTransactions(result) {
            var deferred = $q.defer();
            var ret;

            if (result.transactions && result.transactions.length) {
                ret = result.transactions.filter(trns => trns.availableFiscalItems.length);
            }

            if (ret && ret.length) {
                deferred.resolve(ret);
            } else {
                deferred.reject({data: {message: "Não há transação disponível para troca."}});
            }
            return deferred.promise;
        }
        /**
         *
         */
        function goToTransactionCupomList(transactions) {
            var transitionParams = UtilService.cloneDeep($stateParams);

            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.searchCustomer = $scope.searchCustomer || {};
            transitionParams.transactions = transactions;
            transitionParams.exchangeData.items = [];

            UtilService.navigateWithoutHistory('easyorder.exchangetransactioncouponslist', transitionParams);
        }
        /**
         *
         */
        function goToTransactionProductList(transaction) {
            LogService.info(`Transação selecionada: [${transaction.key.storeNumber}, ${transaction.key.terminal}, ${transaction.key.nsu}, ${transaction.key.date}]`);

            validateRFIDItems(transaction)
            .then(() => {
                var transitionParams = UtilService.cloneDeep($stateParams);

                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.exchangeData.items = transaction.availableFiscalItems
                .map(item => {
                    return {
                        transaction: transaction,
                        item: item,
                        quantity: 1
                    };
                });
                UtilService.navigateWithoutHistory('easyorder.exchangetransactionproductlist', transitionParams);
            });
        }
        /**
         *
         */
        function validateRFIDItems(transaction) {
            if (!transaction.availableFiscalItems.length) {
                return AlertService.error('A transação consultada não possui itens disponívéis para troca.')
               .then($q.reject);
            }
            return $q.resolve();
        }
        /**
         *
         */
        (() => {
            $scope.loadingState = true;
            $ionicLoading.show({
                template: 'Habilitando ao Leitor. Aguarde...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.androidBarcodeConnect())
            .then(() => BluebirdCaptureService.startReading(callbackResult, error => LogService.info(`Erro na leitura SLED-Scanner: ${error.message}`)))
            .then(() => $scope.loadingState = false)
            .finally($ionicLoading.hide)
            .catch(error => {
                let promise;
                let data = JSON.parse(error);

                if (data.errorCode == '-32') {
                    promise = AlertService.warn(`Leitor óptico não conectado: ${data.message}`);
                } else if (data.errorCode == '-99') {
                    promise = AlertService.warn(`${data.message}. Leitura ótipca indisponível.`);
                } else {
                    promise = AlertService.warn(`Não foi possível conectar ao Leitor: [${data.errorCode}]`);
                }
                promise.finally(() => UtilService.navigateWithoutHistory('easyorder.typedexchange', $stateParams));
            });
        })();
    });
})();
