(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.document.transaction.list.tag.deactivation', [
    ])
    .controller('RFIDDocumentTransactionListTagDeactivationCtrl', function ($scope,
                                                                $state,
                                                                $stateParams,
                                                                $q,
                                                                UtilService,
                                                                LogService,
                                                                ConfigService,
                                                                AlertService) {
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.goNext = transaction => {
            LogService.info(`Transação selecionada: [${transaction.key.storeNumber}, ${transaction.key.terminal}, ${transaction.key.nsu}, ${transaction.key.date}]`);
            
            validateRFIDItems(transaction)
            .then(() => {
                var transitionParams = {};
                angular.copy($stateParams, transitionParams);
                
                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.transaction = transaction;
                
                return UtilService.navigateWithoutHistory('easyorder.rfid-tag-deactivation', transitionParams);
            });
        }
        /**
         * 
         */
        $scope.searchProductCupon = value => {
            $q(resolve => {
                $scope.transactionFilter = $scope.transactions
                    .filter(trns => trns.availableFiscalItems)
                    .filter(trns => productCodeFilter(trns.availableFiscalItems, value));
                resolve();
            }).then(() => {});
        };
        /**
         *
         */
        function validateRFIDItems(transaction) {
            const enablePrefix = ConfigService.config.business.epc_enable_prefix;
            
            if (!transaction.availableFiscalItems.length) {
                return AlertService.warn(`Não há item disponível para desativação no cupom consultado.`)
                .then($q.reject);
            } else if (!transaction.availableFiscalItems.find(item => !item.epcTag 
                    || (item.epcTag && !item.epcTagAlarmDisabled && item.epcTag.startsWith(enablePrefix)))) {
                return AlertService.warn(`Não há pendências de desativação de alarme.`)
                .then($q.reject);
            }
            return $q.resolve();
        }
        /**
         * 
         */
        function productCodeFilter(fiscalItems, code) {
            if (!fiscalItems || !fiscalItems.length) {
                return false;
            }
            if (!code || !code.length) {
                return true;
            }
            return fiscalItems.some(fiscalItem => fiscalItem.productCode.search(new RegExp(code, "gi")) >= 0);
        }
        /**
         * 
         */
        (() => {
            $scope.searchCustomer = $stateParams.searchCustomer;
            $scope.transactions = $stateParams.transactions
                .sort((t1, t2) => {
                    var ret = UtilService.compareTo(() => new Date(new Date(t2.key.date).setHours(0,0,0,0)).getTime(), () => new Date(new Date(t1.key.date).setHours(0,0,0,0)).getTime());
                    
                    if (!ret) {
                        ret = UtilService.compareTo(() => parseInt(t1.key.nsu), () => parseInt(t2.key.nsu));
                    }
                    return ret;
                 });
            $scope.transactionFilter = $scope.transactions;
            $scope.productCodeType = ConfigService.config.business.product_code_character_type;
            
            if ($scope.searchCustomer && $scope.searchCustomer.document) {
                if ($scope.searchCustomer.documentType == "CPF") {
                    $scope.formatDocument = applyCpfMask($scope.searchCustomer.document);
                } else if ($scope.cart.customer.documentType == "CNPJ") {
                    $scope.formatDocument = applyCnpjMask($scope.searchCustomer.document);
                } else {
                    $scope.formatDocument = $scope.searchCustomer.document;
                }
            };
        })();
    });
})();