(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.document.transaction.tag.deactivation', [
    ])
    .controller('RFIDDocumentTransactionTagDeactivationCtrl', function($scope,
                                                            $timeout,
                                                            $state,
                                                            $stateParams,
                                                            $ionicLoading,
                                                            $q,
                                                            UtilService,
                                                            TransactionSearchService,
                                                            CustomerService,
                                                            LogService,
                                                            AlertService,
                                                            CartService,
                                                            ConfigService) {
        
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.search = () => {
            $ionicLoading.show({
                template : 'Consultando cliente...',
                duration : 0
            })
            .then(() => $scope.loading = true)
            .then(() => CustomerService.searchByType($scope.searchCustomer.document, 'CPF'))
            .catch(error => {
                LogService.warn(`Não foi possível consultar CPF [${$scope.searchCustomer.document}]: ${error.message}`);
                return [{document: $scope.searchCustomer.document, documentType: 'CPF'}];
            })
            .then(result => result.length ? $q.resolve(result[0]) : $q.reject(`Usuário não encontrado.`))
            .then(data => $scope.searchCustomer = data)
            .then($scope.doTransactionsSearch)
            .finally(() => $scope.loading = false)
            .catch(AlertService.warn);
        };
        /**
         * 
         */
        $scope.doTransactionsSearch = (customer) => {
            return $ionicLoading.show({
                template : 'Por favor aguarde...',
                duration : 0
            })
            .then(() => {
                LogService.info(`Consultando transações de venda do CPF ${$scope.searchCustomer.document}`);
                
                return $q.resolve(customer)
                .then(getTransacationList)
                .then(goToTransactionCupomList);
            })
        }
        /**
         * 
         */
        $scope.reset = () => {
            $scope.searchCustomer = {};
        };
       /**
        *
        */
        $scope.validateDocument = () => {
            var document = $scope.searchCustomer.document;
            $scope.documentInvalid = false;
        
            if (document) {
                if (document.length == 11 && !validaCPF(document)) {
                    $scope.documentInvalid = true;
                    return AlertService.warn("CPF inválido").finally(putFocusOnInput);
                } else {
                    return removeFocusFromInput();
                }
            }
        };
        /**
         *
         */
        async function putFocusOnInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_registrationCPF');
                var angularEl = angular.element(myEl);
                angularEl[0].focus();
            });
        }
        /**
         *
         */
        async function removeFocusFromInput() {
            $timeout(() => {
                var myEl = document.getElementById('input_registrationCPF');
                var angularEl = angular.element(myEl);
                angularEl[0].blur();
            });
        }
        /**
         * 
         */
        function getTransacationList(customer) {
            return $ionicLoading.show({
                template : 'Consultando transações do cliente...',
                duration : 0
            })
            .then(() => {
                let params = {};
                params.customer = {
                    document: customer.document
                };
                params.daysToSearch = ConfigService.config.business.transaction_search_days_threshold;
                params.provider = "STOREX";
                
                LogService.debug(`Consultando transações Storex: provider=${params.provider} daysToSearch=${params.daysToSearch} customer=${params.customer.document}`);
                
                return TransactionSearchService.list(params)
                .then(result => result ? result.transactions : null)
                .catch(error => LogService.error(`Não foi possível obter transações STOREX para CPF ${customer.document}: ${error.data.message}`));
            })
            .then(transactions => {
                let params = {};
                params.customer = {
                    document: customer.document
                };
                params.daysToSearch = ConfigService.config.business.transaction_search_days_threshold;
                params.provider = "LOGIC";
                
                LogService.debug(`Consultando transações Logic: provider=${params.provider} daysToSearch=${params.daysToSearch} customer=${params.customer.document}`);
                
                return TransactionSearchService.list(params)
                .then(result => mergTransactions(transactions, result ? result.transactions : null))
                .catch(error => {
                    LogService.error(`Não foi possível obter transações LOGIC para CPF ${customer.document}: ${error.data.message}`);
                    return transactions;
                });
            })
            .finally($ionicLoading.hide)
            .then(transactions => {
                if (transactions && transactions.length) {
                    return transactions;
                } else {
                    return $q.reject("Nenhuma transação encontrada.")
                }
            })
            .then(transactions => transactions.filter(trns => trns.availableFiscalItems.length))
            .then(transactions => {
                if (transactions && transactions.length) {
                   return transactions;
                } else {
                    return $q.reject("Não há transação disponível para desativação.");
                }
            });
        }
        /**
         *
         */
        function mergTransactions(result1, result2){
            if (result1 && result2) {
                return result1.concat(result2.filter((item) => result1.indexOf(item) < 0));
            }
            if (result1 && result1.transactions) return result1;
            if (result2 && result2.transactions) return result2;
            
            return null;
        }
        /**
         * 
         */
        function goToTransactionCupomList(transactions) {
            var transitionParams = {};
            angular.copy($stateParams, transitionParams);
            
            transitionParams.flowHistory.flow.push($state.current.name);
            transitionParams.flowHistory.params.push($stateParams);
            transitionParams.transactions = transactions;
            transitionParams.searchCustomer = $scope.searchCustomer;
            
            return UtilService.navigateWithoutHistory('easyorder.rfid-document-transaction-list-tag-deactivation', transitionParams);
        }
        /**
         *
         */
        (() => {
            $scope.searchCustomer = {};
            $scope.loading = false;
            
            if (CartService.cart.customer) {
                $scope.searchCustomer = CartService.cart.customer;
                $scope.doTransactionsSearch($scope.searchCustomer)
                    .catch(AlertService.warn);               
            }
        })();
    });
})();