(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.epc.transaction.tag.deactivation', [
    ])
    .controller('RFIDEPCTransactionTagDeactivationCtrl', function($scope,
                                            $state,
                                            $q,
                                            $stateParams,
                                            $ionicLoading,
                                            UtilService,
                                            ConfigService,
                                            LogService,
                                            AlertService,
                                            TransactionSearchService,
                                            BluebirdCaptureService) {
        var seletecTag;
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastPowerSet = $scope.data.defaultPower)
            .then(() => ConfigService.config.business.rfid_reading_power = $scope.data.lastPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultPower = $scope.data.lastPowerSet);
            });
        }; 
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function callbackResult(data) {
            LogService.info(`Leitura SLED: ${data.raw}`);
            
            if (!seletecTag) {
                seletecTag = data.raw;
                
                LogService.info(`Leitura SLED selecionada: ${data.raw}`);
                
                var tagCore = data.raw.substring(3);
                var validTags = [ConfigService.config.business.epc_enable_prefix + tagCore, ConfigService.config.business.epc_disable_prefix + tagCore];
                
                $ionicLoading.show({ template : 'Consultando transação...', duration : 0 })
                .then(() => TransactionSearchService.list({epcTags: validTags}))
                .then(result => {
                    let trans = undefined;
                    
                    if (result.transactions && result.transactions.length) {
                        trans = selectActualTransaction(result.transactions, validTags);
                    }
                    
                    if (trans) {
                        return goToTransactionProductList(trans);
                    } else {
                        return $q.reject({data: {message: `Nenhuma transação foi encontrada para EPC: ${data.raw}.`}});
                    }
                })
                .finally($ionicLoading.hide)
                .finally(() => seletecTag = undefined)
                .catch(AlertService.restError);
            }
        }
        /**
         *
         */
        function selectActualTransaction(transactions, validTags) {
            transactions.sort((e1, e2) => new Date(e2.key.date).getTime() - new Date(e1.key.date).getTime()); // Ordenando por data desc.
            let trans = transactions.find(t => !!t.availableFiscalItems.find(item => !!validTags.find(t => t === item.epcTag))); //Primeira transação que tenha epc na lista de itens disponíveis.
            
            return trans;
        }
        /**
         *
         */
        function callbackError(data) {
            LogService.info(`Erro na leitura SLED: ${data.message}`);
        }
        /**
         * 
         */
        function goToTransactionProductList(transaction) {
            LogService.info(`Transação selecionada: [${transaction.key.storeNumber}, ${transaction.key.terminal}, ${transaction.key.nsu}, ${transaction.key.date}]`);
            
            validateRFIDItems(transaction)
            .then(() => {
                var transitionParams = {};
                angular.copy($stateParams, transitionParams);
                
                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.transaction = transaction;
                
                return UtilService.navigateWithoutHistory('easyorder.rfid-tag-deactivation', transitionParams);
            });
        }
        /**
         *
         */
        function validateRFIDItems(transaction) {
            const enablePrefix = ConfigService.config.business.epc_enable_prefix;
            
            if (!transaction.availableFiscalItems.length) {
                return AlertService.warn(`Não há item disponível para desativação no cupom consultado.`)
                .then($q.reject);
            } else if (!transaction.availableFiscalItems.find(item => !item.epcTag 
                    || (item.epcTag && !item.epcTagAlarmDisabled && item.epcTag.startsWith(enablePrefix)))) {
                return AlertService.warn(`Não há pendências de desativação de alarme.`)
                .then($q.reject);
            }
            return $q.resolve();
        } 
        /**
         *
         */
        (() => {
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.data = {
                lastPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultPower: ConfigService.config.business.rfid_reading_power
            };
            $ionicLoading.show({
                template: 'Conectando ao Leitor RFID...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.connect($scope.data.defaultPower))
            .then(() => BluebirdCaptureService.startReading(callbackResult, callbackError))
            .finally($ionicLoading.hide)
            .catch(error => {
                let data = JSON.parse(error);
                
                AlertService.error(`Não foi possível conectar ao leitor RFID. [${data.errorCode}]`)
                .then($scope.goBack);
            });
        })();                                                     
    });
})();