(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.tag.deactivation', [
    ])
    .controller('RFIDTagDeactivationCtrl', function($rootScope,
                                            $scope,
                                            $stateParams,
                                            $q,
                                            $ionicLoading,
                                            UtilService,
                                            ConfigService,
                                            AlertService,
                                            LogService,
                                            BluebirdCaptureService,
                                            RFIDTransactionService,
                                            CartService,
                                            UserService) {
        let unkownEPCList = new Set();
       
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower(getCurrentPowerFunction($scope.readingInProgress ? 1 : 2)))
            .finally($ionicLoading.hide)
            .then(() => updateCurrentPowerFunction($scope.readingInProgress ? 1 : 2))
            .catch(error => {
                AlertService.error(error.message)
                .then(() => recoverCurrentPowerFunction($scope.readingInProgress ? 1 : 2));
            });
        };
        /**
         *
         */
        $scope.starMissingEPCCapture = () => {
            return $ionicLoading.show({
                template: 'Conectando ao Leitor RFID...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.connect(getCurrentPowerFunction(1)))
            .then(() => BluebirdCaptureService.startReading(readingCallbackResult, data => LogService.info(`Erro na leitura SLED: ${data.message}`)))
            .then(() => $scope.readingInProgress = true)
            .finally($ionicLoading.hide)
            .catch(error => {
                let data = JSON.parse(error);
                
                if (data.errorCode == '-32') {
                   promise = AlertService.confirm('Leitor RFID não conectado.');
                } else {
                   promise = AlertService.confirm(`Erro de conexão do Leitor RFID [${data.errorCode}].`);
                }
            });
        };
        /**
         *
         */
        $scope.startWriting = () => {
            let epcs = [...$scope.data.deactivationMap]
            .filter(item => !item[1] || (item[1] !== "ALREADY_DISABLED" && item[1] !== "OK"))
            .map(item => item[0]);
            
            if (epcs.length) {
                epcs.forEach(epc => $scope.data.deactivationMap.set(epc, null));
                return initDeactivationProcessFor(epcs);
            } else {
                return $q.resolve()
                .then(() => $scope.writingInProgress = false)
                .then(() => $scope.allDone = !([...$scope.data.deactivationMap].find(elem => elem[1] != "OK" && elem[1] != "ALREADY_DISABLED")))
                .then(processDeactivationAndFinish);
            } 
        }
        /**
         *
         */
        $scope.changeReportVisibility = () => {
            $scope.reportShown = !$scope.reportShown;
        };
        /**
         *
         */
        $scope.shutdownReport = () => {
            $scope.reportShown = false;
        };
        /**
         *
         */
        function getCurrentPowerFunction(type) {
            if (type === 1) {
                return $scope.data.defaultReadingPower;
            } else if (type === 2) {
                return $scope.data.defaultWritingPower;
            } else {
                LogService.warn(`Potência do RFID foi solicitada num estado inválido. Retornando 'undefined'...`)
                return undefined;
            }
        }
        /**
         *
         */
        function updateCurrentPowerFunction(type) {
            if (type === 1) {
                $scope.data.lastReadingPowerSet = $scope.data.defaultReadingPower;
                ConfigService.config.business.rfid_reading_power = $scope.data.lastReadingPowerSet;
            } else if (type === 2) {
                $scope.data.lastWritingPowerSet = $scope.data.defaultWritingPower;
                ConfigService.config.business.rfid_writing_power = $scope.data.lastWritingPowerSet;
            } else {
                LogService.warn(`Potência do RFID foi alterada num estado inválido. Nada será feito.`)
            }
        }
        /**
         *
         */
        function recoverCurrentPowerFunction(type) {
            if (type === 1) {
                $scope.data.defaultReadingPower = $scope.data.lastReadingPowerSet; 
            } else if (type === 2) {
                $scope.data.defaultWritingPower = $scope.data.lastWritingPowerSet;
            } else {
                LogService.warn(`Potência do RFID foi alterada num estado inválido. Nada será feito.`)
            }
        }
        /**
         *
         */
        function loadDeactivationMap(transaction) {
            transaction.availableFiscalItems
            .filter(item => !!item.epcTag) //Apenas os itens que possuem EPC.
            .forEach(item => {
                var currentEPCDeactivationStatus = null;
                
                if (item.epcTagAlarmDisabled || item.epcTag.startsWith($scope.data.disablePrefix)) { //O EPC está inativo.
                    currentEPCDeactivationStatus = "ALREADY_DISABLED"; //O EPC foi desativado durante a venda.
                }
                $scope.data.deactivationMap.set(item.epcTag, currentEPCDeactivationStatus);
            });
        }
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function readingCallbackResult(data) {
            const transaction = $scope.data.transaction;
            LogService.info(`Leitura SLED: ${data.raw}`);
            
            if (!transaction.availableFiscalItems.find(item => item.epcTag && item.epcTag === data.raw)) {
                const missingEPCItems = transaction.availableFiscalItems.filter(item => !item.epcTag);
                
                if (missingEPCItems.length) {
                    let item = missingEPCItems.find(item => item.productCode === data.productCode);
                    
                    if (item) {
                        LogService.info(`Criando vínculo entre produto ${item.productCode} e EPC ${data.raw}.`);
                        item.epcTag = data.raw;
                        $scope.data.deactivationMap.set(item.epcTag, item.epcTag.startsWith($scope.data.disablePrefix) ? "OK" : null); //"OK" aqui irá persistir tags que forem lidas já desabilitadas.
                        
                        if (!transaction.availableFiscalItems.find(item => !item.epcTag)) {
                            LogService.info(`Todos os item foram vinulados.`)
                            stopMissingEPCCapture();
                        }
                    } else {
                        LogService.info(`EPC ${data.raw} foi lido mas é estranho à venda (1).`);
                        unkownEPCList.add(data.raw);
                    }
                } else {
                    LogService.info(`EPC ${data.raw} foi lido mas é estranho à venda (2).`);
                    unkownEPCList.add(data.raw);
                }
            } else {
                LogService.info(`Etiqueta já presente na venda: ${data.raw}`);
            }
        }
        /**
         *
         */
        function stopMissingEPCCapture() {
            let promise = $q.resolve();
            
            if (unkownEPCList.size) {
                promise = AlertService.warn(`Existem produtos não registrados no cupom. Estes produtos não serão desativados!`);
                unkownEPCList.clear();
            }
            promise
            .then(BluebirdCaptureService.disconnect)
            .then(() => $scope.missingEPC = false)
            .then(() => $scope.readingInProgress = false);
        }
        /**
         *
         */
        function initDeactivationProcessFor(epcs) {
            if (epcs.length) {    
                LogService.info(`Cabeçalho configurado para desativação: '${$scope.data.disablePrefix}'`);
        
                return $ionicLoading.show({
                    template: 'Conectando ao Leitor RFID...',
                    duration: 0
                })
                .then(() => BluebirdCaptureService.connect(getCurrentPowerFunction(2)))
                .then(() => $scope.writingInProgress = true)
                .then(() => BluebirdCaptureService.changeEPCHeader(
                    epcs,
                    $scope.data.disablePrefix,
                    changeHeaderSuccessCallback,
                    changeHeadererrorCallback))
                .finally($ionicLoading.hide)
                .catch(error => {
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                       AlertService.warn('Leitor RFID não conectado.');
                    } else {
                       AlertService.warn(`Não foi possível conectar ao Sled [${data.errorCode}]`);
                    }
                    return $q.reject();
                });
            } else {
                return $q.resolve();
            }
        }
        /**
         *
         */
        function changeHeaderSuccessCallback(data) {
            LogService.info(`EPC '${data.originalEPC}' teve o cabeçalho alterado.`);
            $scope.data.deactivationMap.set(data.originalEPC, data.status);
            
            updateItemStatus(data);

            disconnetIfAllResponsesWereGiven()
            .then(processDeactivationAndFinish);
        }

        function updateItemStatus(data){

            let epc = $scope.data.disablePrefix + data.originalEPC.substring($scope.data.disablePrefix.length);
            if($scope.data.deactivationMap.has(epc)){
                epc = $scope.data.enablePrefix + epc.substring($scope.data.disablePrefix.length);
            }
            $scope.data.deactivationMap.set(epc, data.status);
        }
        /**
         *
         */
        function changeHeadererrorCallback(error) {
            if (error.data) {
                let data = JSON.parse(error.data);
                
                if (error.errorCode == "-32") {
                    LogService.info(`Tempo para obter acesso à tag foi excedido para '${data.originalEPC}'. Nenhuma alteração realizada.`);
                    $scope.data.deactivationMap.set(data.originalEPC, data.status);
                } else {
                    LogService.info(`'${data.originalEPC}' não teve o cabeçalho alterado: ${error.message} [${error.errorCode}]`);
                    $scope.data.deactivationMap.set(data.originalEPC, data.status);
                }
            }
            disconnetIfAllResponsesWereGiven()
            .finally(finishIfDone);
        }
        /**
         *
         */
         function finishIfDone() {
            let ret = $q.reject();
            
            if ($scope.allDone) {
                const msg = `Todos os alarmes foram ${$scope.data.disableOp ? 'desativados': 'ativados'} com sucesso!`;
                
                ret = AlertService.info(msg)
                .then(() => UtilService.navigateToMainScreen());
            }
            return ret;
        }        
        function disconnetIfAllResponsesWereGiven() {
            let ret = $q.resolve();
            let dataMapArr = [...$scope.data.deactivationMap];
            
            if (!dataMapArr.find(elem => elem[1] === null)) {
                ret = BluebirdCaptureService.disconnect()
                .finally(() => {
                    $scope.writingInProgress = false;
                    $scope.allDone = !(dataMapArr.find(elem => elem[1] != "OK"));                
                });
            }
            return ret;
        }
        /**
         *
         */
        function processDeactivationAndFinish() {
            return ($scope.allDone ? $q.resolve() : $q.reject())
            .then(() => {
                let items = $scope.data.transaction.availableFiscalItems
                .filter(item => "OK" === $scope.data.deactivationMap.get(item.epcTag)); //Apenas Tags que foram sensibilizadas pelo controlador devem mudar o valor.
                
                items.forEach(item => {
                    item.epcTagAlarmDisabled = true; 
                    delete item.onlineInventoryIntegration; //Forçando o uso valor default da API REST
                });
                return items;
            })
            .then(integrate)
            .then(() => AlertService.info(`Desativação de alarme executada com sucesso.`))
            .then(() => CartService.initCart())
            .then(() => $rootScope.$emit("screen-total-value",  0))
            .then(() => UtilService.navigateToMainScreen());
        }
        /**
         *
         */
        function integrate(items) {
            var ret = $q.resolve();
            
            if (items && items.length) {
                ret = $ionicLoading.show()
                .then(() => {
                    return {
                        transactionKey: {
                            companyNumber:  ConfigService.config.companyNumber,
                            storeNumber:    ConfigService.config.storeNumber,
                            nsu:            CartService.cart.nsu,
                            terminal:       ConfigService.config.terminalNumber,
                            date:           new Date().toISOString()
                        },
                        sourceTransactionKey: {
                            companyNumber:  $scope.data.transaction.key.companyNumber,
                            storeNumber:    $scope.data.transaction.key.storeNumber,
                            nsu:            $scope.data.transaction.key.nsu,
                            terminal:       $scope.data.transaction.key.terminal,
                            date:           $scope.data.transaction.key.date
                        },
                        fiscalItems: items,
                        operator: {
                            code: UserService.operatorCode(),
                            name: UserService.operatorName()
                        },
                        authorizer: {
                            code: UserService.operatorCode(),
                            name: UserService.operatorName()
                        },
                        operation: 'EPC_DEACTIVATION'
                    }
                })
                .then(RFIDTransactionService.itemLinkage)
                .finally($ionicLoading.hide)
                .catch(() => {
                    return AlertService.confirm(`Falha ao integrar dados de desativação, deseja tentar novamente?`)
                    .then(tryAgain => {
                        if (tryAgain) {
                            return integrate(items);
                        } else {
                            LogService.info(`O operador desistiu da desativação...`);
                            $scope.allDone = false;
                            return $q.reject();
                        }
                    });
                });
            }
            return ret;
        }
        /**
         *
         */                                        
        (() => {
            $scope.data = {
                transaction: $stateParams.transaction,
                lastReadingPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultReadingPower: ConfigService.config.business.rfid_reading_power,
                lastWritingPowerSet: ConfigService.config.business.rfid_writing_power,
                defaultWritingPower: ConfigService.config.business.rfid_writing_power,
                deactivationMap: new Map(),
                disablePrefix: ConfigService.config.business.epc_disable_prefix,
                enablePrefix: ConfigService.config.business.epc_enable_prefix
            };
            $scope.allDone = false;
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.reportShown = false;
            $scope.readingInProgress = false;
            $scope.writingInProgress = false;
            
            loadDeactivationMap($scope.data.transaction);
            
            $scope.missingEPC = !!$scope.data.transaction.availableFiscalItems.find(item => !item.epcTag);
            
            var ret = $q.resolve();
            
            if ($scope.missingEPC) {
                ret = $scope.starMissingEPCCapture();
            } else if ($scope.data.transaction.availableFiscalItems.find(item => !item.epcTagAlarmDisabled)) {
                ret = $scope.startWriting();
            }
            
            ret.finally(() => $scope.initialLoadingFinished = true);
        })();
    });
})();