(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.scanned.transaction.tag.deactivation', [
    ])
    .controller('RFIDScannedTransactionTagDeactivationCtrl', function($scope,
                                            $state,
                                            $stateParams,
                                            $q,
                                            $ionicLoading,
                                            UtilService,
                                            ConfigService,
                                            LogService,
                                            AlertService,
                                            TransactionSearchService,
                                            BluebirdCaptureService) {
        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();
                
                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.goTyped = () => {
            var stateParams = {};
            angular.copy($stateParams, stateParams);
            
            stateParams.flowHistory.flow.push($state.current.name);
            stateParams.flowHistory.params.push($stateParams);
            
            return UtilService.navigateWithoutHistory('easyorder.rfid-typed-transaction-tag-deactivation', stateParams);
        };
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function callbackResult(data) {
            const dataInput = data.raw;
            
            return $q.resolve()
            .then(() => LogService.info(`Leitura SLED-Barcode: ${dataInput}`))
            .then(() => $scope.initialLoading = true)
            .then(() => {
                if (dataInput.length == 44) {
                    return doLogicSearch(dataInput);
                } else if (dataInput.length == 22) {
                    return doStorexSearch(dataInput);
                } else {
                    return $q.reject({data: {message: 'Código de Barras não atende aos parâmetros dessa funcionalidade.'}});
                }
            })
            .then(result => goToTransactionProductList(result.transaction))
            .finally(() => $scope.initialLoading = false)
            .catch(error => {
                if (error) {
                    return AlertService.restError(error);
                }   
            });
        }
        /**
         *
         */
        function callbackError(data) {
            LogService.info(`Erro na leitura SLED-Scanner: ${data.message}`);
        }
        /**
         *
         */
        async function doLogicSearch(dataInput) {
            const defer = $q.defer();
            const nsu = dataInput.substr(25, 9);
            const date = new Date();
            const year = `${date.getFullYear().toString().substr(0, 2)}${dataInput.substr(2, 2)}`;
            const month = dataInput.substr(4, 2);
            const maxDay = new Date(year, month - 1, 0).getDate();

            $scope.date = {};
            $scope.date.minDate = `${year}-${month}-01`;
            $scope.date.maxDate = `${year}-${month}-${maxDay}`;
            $scope.date.value = new Date(year, month - 1, 1);

            $scope.bsConfirm = () => {
                $scope.bottomsheet = false;
                console.log($scope.date);
                const key = {
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    terminal: ConfigService.config.terminalNumber,
                    nsu: nsu,
                    date: $scope.date.value
                };
                
                $q.resolve()
                .then(() => $ionicLoading.show({ template : 'Consultando transação...', duration : 0 }))
                .then(() => {
                    return TransactionSearchService.search({ 
                        transactionKey: key,
                        provider: 'LOGIC',
                    })
                    .then(defer.resolve)
                    .catch(defer.reject);
                })
                .finally(() => $ionicLoading.hide());
            };
            $scope.bsCancel = () => {
                $scope.bottomsheet = false;
                defer.reject();
            };
            $scope.bottomsheet = true;
            
            return defer.promise;
        }
        /**
         *
         */
        async function doStorexSearch(dataInput) {
            const key = {};
            key.storeInfo = {};
            key.storeInfo.companyNumber = '00001';
            key.storeInfo.storeNumber = dataInput.substring(0, 4);

            key.date = dataInput.substring(4, 12);
            const match = key.date.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
            key.date = new Date(match[3], match[2] - 1, match[1], 12, 0, 0, 0).toISOString();

            key.terminal = parseInt(dataInput.substring(12, 15));
            key.nsu = parseInt(dataInput.substring(15, 22));

            return $q.resolve()
            .then(() => $ionicLoading.show({ template : 'Consultando transação...', duration : 0 }))
            .then(() => {
                return TransactionSearchService.search({ 
                    transactionKey: key,
                    provider: 'STOREX',
                });
            })
            .finally($ionicLoading.hide);
        }
        /**
         * 
         */
        function goToTransactionProductList(transaction) {
            LogService.info(`Transação selecionada: [${transaction.key.storeNumber}, ${transaction.key.terminal}, ${transaction.key.nsu}, ${transaction.key.date}]`);
            
            validateRFIDItems(transaction)
            .then(() => {
                var transitionParams = {};
                angular.copy($stateParams, transitionParams);
                
                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.transaction = transaction;
                
                return UtilService.navigateWithoutHistory('easyorder.rfid-tag-deactivation', transitionParams);
            });
        }
        /**
         *
         */
        function validateRFIDItems(transaction) {
            const enablePrefix = ConfigService.config.business.epc_enable_prefix;
            
            if (!transaction.availableFiscalItems.length) {
                return AlertService.warn(`Não há item disponível para desativação no cupom consultado.`)
                .then($q.reject);
            } else if (!transaction.availableFiscalItems.find(item => !item.epcTag 
                    || (item.epcTag && !item.epcTagAlarmDisabled && item.epcTag.startsWith(enablePrefix)))) {
                return AlertService.warn(`Não há pendências de desativação de alarme.`)
                .then($q.reject);
            }
            return $q.resolve();
        }
        /**
         *
         */
        (() => {
            $scope.initialLoading = true;
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.data = {
                lastPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultPower: ConfigService.config.business.rfid_reading_power
            };
            $ionicLoading.show({
                template: 'Habilitando ao Leitor. Aguarde...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.barcodeConnect())
            .then(() => BluebirdCaptureService.startReading(callbackResult, callbackError))
            .then(() => $scope.initialLoading = false)
            .finally($ionicLoading.hide)
            .catch(error => {
                let promise;
                let data = JSON.parse(error);
                
                if (data.errorCode == '-32') {
                   promise = AlertService.warn('Leitor não conectado.');
                } else {
                   promise = AlertService.warn(`Não foi possível conectar ao Leitor: [${data.errorCode}]`);
                }
                promise.finally(() => UtilService.navigateWithoutHistory('easyorder.operations'));
            });
        })();                      
    });
})();