(function() {
    'use strict';
    angular.module('easyorder.controllers.rfid.typed.transaction.tag.deactivation', [])
    .controller('RFIDTypedTransactionTagDeactivationCtrl', function($scope,
                                                            $q,
                                                            $state,
                                                            $stateParams,
                                                            AlertService,
                                                            UtilService,
                                                            ConfigService,
                                                            LogService) {

        /**
         * 
         */
        $scope.goBack = () => {
            if ($stateParams.flowHistory) {
                const backState = $stateParams.flowHistory.flow.pop();
                const params = $stateParams.flowHistory.params.pop();

                UtilService.navigateWithoutHistory(backState, params);
            }
        };
        /**
         * 
         */
        $scope.onSearchResult = (resultPromise) => {
            resultPromise
            .then(result => {
                if (result.transaction.availableFiscalItems.length) {
                    return goToTransactionProductList(result.transaction);
                } else {
                    return AlertService.error('A transação consultada não possui itens disponívéis para troca.');
                }
            })
            .catch(error => {
                return AlertService.restError(error, 'Não foi possível consultar transação para troca!');
            });
        };
        /**
         * 
         */
        function goToTransactionProductList(transaction) {
            LogService.info(`Transação selecionada: [${transaction.key.storeNumber}, ${transaction.key.terminal}, ${transaction.key.nsu}, ${transaction.key.date}]`);
            
            validateRFIDItems(transaction)
            .then(() => {
                var transitionParams = {};
                angular.copy($stateParams, transitionParams);
                
                transitionParams.flowHistory.flow.push($state.current.name);
                transitionParams.flowHistory.params.push($stateParams);
                transitionParams.transaction = transaction;
                
                return UtilService.navigateWithoutHistory('easyorder.rfid-tag-deactivation', transitionParams);
            });
        }
        /**
         *
         */
        function validateRFIDItems(transaction) {
            const enablePrefix = ConfigService.config.business.epc_enable_prefix;
            
            if (!transaction.availableFiscalItems.length) {
                return AlertService.warn(`Não há item disponível para desativação no cupom consultado.`)
                .then($q.reject);
            } else if (!transaction.availableFiscalItems.find(item => !item.epcTag 
                    || (item.epcTag && !item.epcTagAlarmDisabled && item.epcTag.startsWith(enablePrefix)))) {
                return AlertService.warn(`Não há pendências de desativação de alarme.`)
                .then($q.reject);
            }
            return $q.resolve();
        }
    });
})();
