(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.detached.change.tag.header', [
        'easyorder.services.alert',
        'easyorder.services.log',
        'easyorder.services.util'
    ])

    .controller('RFIDDetachedChangeTagHeaderCtrl', function($scope,
                                            $stateParams,
                                            $q,
                                            $ionicLoading,
                                            ConfigService,
                                            UtilService,
                                            LogService,
                                            BluebirdCaptureService,
                                            AlertService) {
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Aguarde. Configurando Potência...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower(getCurrentPowerFunction($scope.readingInProgress ? 1 : 2)))
            .finally($ionicLoading.hide)
            .then(() => updateCurrentPowerFunction($scope.readingInProgress ? 1 : 2))
            .catch(error => {
                AlertService.error(error.message)
                .then(() => recoverCurrentPowerFunction($scope.readingInProgress ? 1 : 2));
            });
        };
        /**
         *
         */
        $scope.startReading = function() {
            return $ionicLoading.show({
                template: 'Aguarde. Iniciando leitura...',
                duration: 0
            })
            .then(() => $scope.data.changeMap.clear())
            .then(() => $scope.data.epcs.length = 0)
            .then(() => $scope.allDone = true)
            .then(() => BluebirdCaptureService.connect(getCurrentPowerFunction(1)))
            .then(() => BluebirdCaptureService.startReading(data => {
                if (!$scope.data.epcs.find(epc => epc === data.raw)) {
                    LogService.info(`EPC Lido: ${data.raw}`);
                    $scope.data.changeMap.set(data.raw, data.raw.startsWith($scope.data.changeHeadersTo) ? "OK" : null);
                    $scope.data.epcs.push(data.raw);
                }
                if ($scope.allDone) { //Verifica se a tag lida deve ser alterada.
                    $scope.allDone = data.raw.startsWith($scope.data.changeHeadersTo);
                }
            }, error => {
                LogService.info(`Erro na leitura RFID: ${error.message}`);
            }))
            .then(() => $scope.readingInProgress = true)
            .finally($ionicLoading.hide)
            .catch(error => {
                let promise;
                let data = JSON.parse(error);
                
                if (data.errorCode == '-32') {
                   promise = AlertService.warn('Leitor RFID não conectado.');
                } else {
                   promise = AlertService.warn(`Não foi possível conectar ao Sled [${data.errorCode}]`);
                }
                promise
                .finally(() => UtilService.navigateWithoutHistory('easyorder.operations'));
            });
        }
        /**
         *
         */
        $scope.startWriting = ()  => {
            return $q.resolve($scope.readingInProgress = false)
            .then(() => {
                let epcs = [...$scope.data.changeMap]
                    .filter(item => item[1] === "ERROR" || !item[1])
                    .map(item => item[0]);
                
                epcs.forEach(epc => $scope.data.changeMap.set(epc, null))
                
                return epcs;
            })
            .then(initHearderChangeProcessFor)
            .catch(error => {
                let data = JSON.parse(error);
                    
                if (data.errorCode == '-32') {
                   AlertService.warn('Leitor RFID não conectado.');
                } else if (error.errorCode == "-4") {
                    LogService.info(`O dispositivo está ocupado. O gatilho não pode estar pressionado para inciar a alteração.`);
                } else {
                   AlertService.warn(`Não foi possível conectar ao Sled [${data.errorCode}]`);
                }
            });   
        };
        /**
         *
         */
        function initHearderChangeProcessFor(epcs) {
            if (epcs.length) {    
                LogService.info(`Cabeçalho configurado para alteração: '${$scope.data.changeHeadersTo}'`);
        
                return $ionicLoading.show({
                    template: 'Conectando ao Leitor RFID...',
                    duration: 0
                })
                .then(() => BluebirdCaptureService.connect(getCurrentPowerFunction(2)))
                .then(() => $scope.writingInProgress = true)
                .then(() => BluebirdCaptureService.changeEPCHeader(
                    epcs,
                    $scope.data.changeHeadersTo,
                    changeHeaderSuccessCallback,
                    changeHeadererrorCallback))
                .finally($ionicLoading.hide)
                .catch($q.reject);
            } else {
                LogService.warn(`Não há items para alteração.`);
                return $q.resolve();
            }
        }
        /**
         *
         */
        function getCurrentPowerFunction(type) {
            if (type === 1) {
                return $scope.data.defaultReadingPower;
            } else if (type === 2) {
                return $scope.data.defaultWritingPower;
            } else {
                LogService.warn(`Potência do RFID foi solicitada num estado inválido. Retornando 'undefined'...`)
                return undefined;
            }
        }
        /**
         *
         */
        function updateCurrentPowerFunction(type) {
            if (type === 1) {
                $scope.data.lastReadingPowerSet = $scope.data.defaultReadingPower;
                ConfigService.config.business.rfid_reading_power = $scope.data.lastReadingPowerSet;
            } else if (type === 2) {
                $scope.data.lastWritingPowerSet = $scope.data.defaultWritingPower;
                ConfigService.config.business.rfid_writing_power = $scope.data.lastWritingPowerSet;
            } else {
                LogService.warn(`Potência do RFID foi alterada num estado inválido. Nada será feito.`)
            }
        }
        /**
         *
         */
        function recoverCurrentPowerFunction(type) {
            if (type === 1) {
                $scope.data.defaultReadingPower = $scope.data.lastReadingPowerSet; 
            } else if (type === 2) {
                $scope.data.defaultWritingPower = $scope.data.lastWritingPowerSet;
            } else {
                LogService.warn(`Potência do RFID foi alterada num estado inválido. Nada será feito.`)
            }
        }
        /**
         *
         */
        function changeHeaderSuccessCallback(data) {
            LogService.info(`EPC '${data.originalEPC}' teve o cabeçalho alterado.`);
            $scope.data.changeMap.set(data.originalEPC, data.status);
            
            disconnetIfAllResponsesWereGiven()
            .finally(finishIfDone);
        }
        /**
         *
         */
        function changeHeadererrorCallback(error) {
            if (error.data) {
                let data = JSON.parse(error.data);
                
                if (error.errorCode == "-32") {
                    LogService.info(`Tempo para obter acesso à tag foi excedido para '${data.originalEPC}'. Nenhuma alteração realizada.`);
                } else {
                    LogService.info(`'${data.originalEPC}' não teve o cabeçalho alterado: ${error.message} [${error.errorCode}]`);
                }
                $scope.data.changeMap.set(data.originalEPC, data.status);
            }
            disconnetIfAllResponsesWereGiven()
            .finally(finishIfDone);
        }
        /**
         *
         */
        function disconnetIfAllResponsesWereGiven() {
            let ret = $q.resolve();
            let dataMapArr = [...$scope.data.changeMap];
            
            if (!dataMapArr.find(elem => elem[1] === null)) {
                ret = BluebirdCaptureService.disconnect()
                .finally(() => {
                    LogService.info(`Todas as tags obtiveram resposta. Sled foi desconectado.`)
                    $scope.writingInProgress = false;
                    $scope.allDone = !(dataMapArr.find(elem => elem[1] != "OK"));                
                });
            }
            return ret;
        }
        /**
         *
         */
        function finishIfDone() {
            let ret = $q.reject();
            
            if ($scope.allDone) {
                const msg = `Todos os alarmes foram ${$scope.data.disableOp ? 'desativados': 'ativados'} com sucesso!`;
                
                ret = AlertService.info(msg)
                .then(() => UtilService.navigateToMainScreen());
            }
            return ret;
        }
        /**
         *
         */
        (() => {
            $ionicLoading.hide();
            $scope.data = {
                lastReadingPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultReadingPower: ConfigService.config.business.rfid_reading_power,
                lastWritingPowerSet: ConfigService.config.business.rfid_writing_power,
                defaultWritingPower: ConfigService.config.business.rfid_writing_power,
                changeMap: new Map(),
                changeHeadersTo: $stateParams.changeHeadersTo,
                disableOp:  $stateParams.changeHeadersTo === ConfigService.config.business.epc_disable_prefix,
                epcs: []
            };
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.startReading();
        })();                                                
    });
})();