(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.disable.tag', [
        'easyorder.services.cart',
        'easyorder.services.alert',
        'easyorder.services.log',
        'easyorder.services.bluebird.capture',
        'easyorder.services.util'
    ])

    .controller('RFIDDisableTagsCtrl', function($scope,
                                            $timeout,
                                            $q,
                                            $ionicLoading,
                                            CartService,
                                            ConfigService,
                                            UtilService,
                                            LogService,
                                            BluebirdCaptureService,
                                            UserService,
                                            AlertService) {
        /**
         *
         */
        function changeHeaderSuccessCallback(data) {
            LogService.info(`EPC '${data.originalEPC}' teve o cabeçalho alterado.`);
            $scope.data.deactivationMap.set(data.originalEPC, data.status);
            
            disconnetIfAllResponsesWereGiven()
            .finally(finishIfDone);
        }
        /**
         *
         */
        function changeHeadererrorCallback(error) {
            if (error.data) {
                let data = JSON.parse(error.data);
                
                if (error.errorCode == "-32") {
                    LogService.info(`Tempo para obter acesso à tag foi excedido para '${data.originalEPC}'. Nenhuma alteração realizada.`);
                } else {
                    LogService.info(`'${data.originalEPC}' não teve o cabeçalho alterado: ${error.message} [${error.errorCode}]`);
                }
                $scope.data.deactivationMap.set(data.originalEPC, data.status);
            }
            disconnetIfAllResponsesWereGiven()
            .finally(finishIfDone);
        }
        /**
         *
         */
        function disconnetIfAllResponsesWereGiven() {
            let ret = $q.resolve();
            let dataMapArr = [...$scope.data.deactivationMap];
            
            if (!dataMapArr.find(elem => elem[1] === null)) {
                ret = BluebirdCaptureService.disconnect()
                .finally(() => {
                    $scope.inProgress = false;
                    $scope.allDone = !(dataMapArr.find(elem => elem[1] != "OK" && elem[1] != "ALREADY_DISABLED"));                
                });
            }
            return ret;
        }
        /**
         *
         */
        function finishIfDone() {
            let ret = $q.reject();
            
            if ($scope.allDone) {
                ret = $ionicLoading.show()
                .then($timeout(finished, 2000)); //Faz com que a tela pare por alguns instantes antes de prosseguir
            }
            return ret;
        }
        /**
         *
         */
        function initDeactivationProcessFor(epcs) {
            if (epcs.length) {    
                LogService.info(`Cabeçalho configurado para desativação: '${$scope.data.disablePrefix}'`);
        
                return $ionicLoading.show({
                    template: 'Conectando ao Leitor RFID...',
                    duration: 0
                })
                .then(() => BluebirdCaptureService.connect($scope.data.defaultPower))
                .then(() => $scope.inProgress = true)
                .then(() => BluebirdCaptureService.changeEPCHeader(
                    epcs,
                    $scope.data.disablePrefix,
                    changeHeaderSuccessCallback,
                    changeHeadererrorCallback))
                .finally($ionicLoading.hide)
                .catch(error => {
                    let data = JSON.parse(error);
                    
                    if (data.errorCode == '-32') {
                       AlertService.warn('Leitor RFID não conectado.');
                    } else {
                       AlertService.warn(`Não foi possível conectar ao Sled [${data.errorCode}]`);
                    }
                    return $q.reject();
                });
            } else {
                return $q.resolve();
            }
        }
        /**
         *
         */
        function validateItens() {
            if (!$scope.allDone) {
                return AlertService.confirm('Existe produto com alarme ativo. Deseja finalizar?')
                .then(result => result ? $q.resolve() : $q.reject())
                .then(() => UserService.requestAuthorization(SELL_ENABLED_RFID_TAGS, '"Venda Tag RFID ainda Habilitada"'))
                .then(userCode => CartService.cart.changeAuthorizations[SELL_ENABLED_RFID_TAGS] = userCode);
            } else {
                return $q.resolve();    
            }      
        }
        /**
         *
         */
        function finished() {
            return $q.resolve() 
            .then(() =>{
                CartService.cart.fiscalItems.forEach(item => {
                    item.epcTagAlarmDisabled = "OK" === $scope.data.deactivationMap.get(item.epcTag); //Apenas Tags que foram sensibilizadas pelo controlador devem mudar o valor.
                });
                return UtilService.navigateWithoutHistory('easyorder.paymentresult');
            })
        };
        /**
         *
         */
        $scope.goNext = () => {
            validateItens()
            .then($ionicLoading.show)
            .then(finished);
        }
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastPowerSet = $scope.data.defaultPower)
            .then(() => ConfigService.config.business.rfid_writing_power = $scope.data.lastPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultPower = $scope.data.lastPowerSet);
            });
        };
        /**
         *
         */
        $scope.restartDeactivation = function() {
            $timeout(() => {
                let epcs = [...$scope.data.deactivationMap]
                    .filter(item => item[1] === "ERROR" || !item[1])
                    .map(item => item[0]);
                
                epcs.forEach(epc => $scope.data.deactivationMap.set(epc, null)) 
                
                LogService.info(`Reniciando processo de desativação para ${epcs.join(', ')}`);                
                initDeactivationProcessFor(epcs);
            });   
        };
        /**
         *
         */
        (() => {
            $ionicLoading.hide();
            $scope.data = {
                cart: CartService.cart,
                lastPowerSet: ConfigService.config.business.rfid_writing_power,
                defaultPower: ConfigService.config.business.rfid_writing_power,
                deactivationMap: new Map(),
                disablePrefix: ConfigService.config.business.epc_disable_prefix
            };
            var epcs = [];
            $scope.allDone = true;
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.inProgress = false;
            
            CartService.cart.fiscalItems
            .filter(item => !!item.epcTag) //Apenas os itens que possuem EPC.
            .forEach(item => {
                var currentEPCDeactivationStatus = null;
                
                if (item.epcTagAlarmDisabled) {
                     currentEPCDeactivationStatus = "OK"; //Em caso de retentativa da venda.
                } else if (item.epcTag.startsWith($scope.data.disablePrefix)) {
                    currentEPCDeactivationStatus = "ALREADY_DISABLED"; //O EPC foi desativado numa venda anterior.
                } else {
                    $scope.allDone = false;
                    epcs.push(item.epcTag);
                }
                $scope.data.deactivationMap.set(item.epcTag, currentEPCDeactivationStatus);
            });
            finishIfDone().catch(() => initDeactivationProcessFor(epcs));
        })();
    });
})();
