(function() {
    'use strict';

    angular.module('easyorder.controllers.rfid.search', [
        'easyorder.services.cart',
        'easyorder.services.alert',
        'easyorder.services.log',
        'easyorder.services.bluebird.capture',
        'easyorder.services.util'
    ])


    .controller('RFIDScannerCtrl', function($scope,
                                        $q,
                                        CartService,
                                        ProductService,
                                        ConfigService,
                                        AlertService,
                                        UtilService,
                                        LogService,
                                        $ionicLoading,
                                        BluebirdCaptureService) {
                                            
        let srchChainPromise = $q.resolve();
        
        /**
         *
         */        
        function isEPCNewInCart(epc) {
            const fiscalItems = CartService.cart.fiscalItems;
            
            return !fiscalItems.find(item => item.epcTag === epc);
        }
        /**
         *
         */
        function onSearch(productId, epc) {
            return $ionicLoading.show({
                template:`Consultando '${productId}'`,
                duration: 0
            })
            .then(() => ProductService.search({ code: productId }))
            .finally($ionicLoading.hide)
            .then(result => {
                if (result.items.length > 0) {
                    var saleInfo = result.items[0];
                    
                    if (saleInfo.productType == 'MINIMUM_SALE_UNIT') {
                        saleInfo.quantitySold = saleInfo.minimumSaleUnit;
                    } else {
                        saleInfo.quantitySold = 1;
                    }

                    saleInfo.epcTag = epc;

                    return saleInfo;
                } else {
                    return $q.reject({data: {message: 'Produto não encontrado!<br/>Código: <b>' + productId + '</b>'}});
                }
            });
        }
        /**
         * Registra função de callback para leitura do scanner via plugin.
         */
        function callbackResult(data) {
            LogService.info(`Leitura SLED: ${data.raw}`);
            
            if (isEPCNewInCart(data.raw)) {
                srchChainPromise = srchChainPromise
                .then(() => onSearch(data.productCode, data.raw))
                .then(item => CartService.doAddToCart(item))
                .catch(error => AlertService.restError(error, `Não foi possível consultar item '${data.productCode}'`));
            }
        }
        /**
         *
         */
        function callbackError(data) {
            LogService.info(`Erro na leitura SLED: ${data.message}`);
        }
        /**
         *
         */
        $scope.changePowerTo = () => {
            $ionicLoading.show({
                template: 'Configurando SLED...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.setPower($scope.data.defaultPower))
            .finally($ionicLoading.hide)
            .then(() => $scope.data.lastPowerSet = $scope.data.defaultPower)
            .then(() => ConfigService.config.business.rfid_reading_power = $scope.data.lastPowerSet)
            .catch(error => {
                AlertService.error(error.message)
                .then($scope.data.defaultPower = $scope.data.lastPowerSet);
            });
        };
        /**
         *
         */
        $scope.changeInputMode = () => {
            if (window.cordova) {
                return UtilService.navigateWithoutHistory('easyorder.barcode');
            } else {
                return UtilService.navigateWithoutHistory('easyorder.productsearch');
            }
        }
        /**
         *
         */
        (() => {
            $scope.sledTrigger = BluebirdCaptureService.sledTrigger;
            $scope.data = {
                lastPowerSet: ConfigService.config.business.rfid_reading_power,
                defaultPower: ConfigService.config.business.rfid_reading_power
            };
            $ionicLoading.show({
                template: 'Conectando ao Leitor RFID...',
                duration: 0
            })
            .then(() => BluebirdCaptureService.connect($scope.data.defaultPower))
            .then(() => BluebirdCaptureService.startReading(callbackResult, callbackError))
            .finally($ionicLoading.hide)
            .catch(error => {
                if (ConfigService.config.business.enables_reading_by_rfid_only) {
                    AlertService.alert('Leitor RFID não conectado', 'Retorne após conectar o leitor RFID para prosseguir com a operação').then(result => {
                        return UtilService.navigateToMainScreen();
                    });
                } else {
                    let promise;
                    let data = JSON.parse(error);
                    if (data.errorCode == '-32') {
                        promise = AlertService.confirm('Leitor RFID não conectado. Deseja prosseguir?');
                     } else {
                        promise = AlertService.confirm(`Erro de conexão do Leitor RFID [${data.errorCode}]. Deseja prosseguir?`);
                     }
                     
                     promise.then(result => {
                         if (result) {
                             return UtilService.navigateWithoutHistory('easyorder.barcode');
                         } else {
                             return UtilService.navigateToMainScreen();
                         }
                     });
                }
            });
        })();
    });
})();
