(function() {
    'use strict';

    angular.module('easyorder.controllers.scannedgiftcardpayment', [])
    .controller('ScannedGiftCardPaymentCtrl', function(
                                                $scope,
                                                $q,
                                                UtilService,
                                                PaymentService,
                                                AlertService,
                                                GiftCardService,
                                                BarcodeService) {

        AlertService.restError = new Proxy(AlertService.restError, {
            apply: (target, __, argumentsList) => applyProxy(target, __, argumentsList)
        });

        AlertService.confirm = new Proxy(AlertService.confirm, {
            apply: (target, __, argumentsList) => applyProxy(target, __, argumentsList, true)
        });

        function applyProxy(target, __, argumentsList, load) {
            $scope.loadingState = false;

            return target(...argumentsList).then((res) => {
                $scope.loadingState = load;
                return res;
            });
        }

        $scope.loadingState = false;
        $scope.barcodeService = BarcodeService;

        BarcodeService.init((scanData) => {
            check(scanData);
        },  { inSizeCam: ['2/44/2/44', '2/64/2/64'], outSizeCam: ['275/44/0/44', '275/64/0/64'] });

        let lock = false;
        function check(scanData) {
            if (lock) {
                return;
            }
            lock = true;
            BarcodeService.cancel();
            $scope.loadingState = true;

            const length = scanData[0].length;
            const scanned = scanData[0];
            const barcode = length % 2 == 0 && scanned.charAt(0) == "0" ?
                scanned.substr(1, length - 1) : scanned;

            GiftCardService.search(barcode).then((res) => {
                const giftCard = res.giftCard;

                if (giftCard == null) {
                    return $q.reject("Cartão presente não retornado");
                }
                
                if (!giftCard.activated) {
                    return $q.reject("Cartão presente não está ativo");
                }

                return checkBalance(giftCard);
            }).then((giftCard) =>
                navigate(giftCard)
            ).catch((err) => {
                if (err == null) { // apertou em cancelar no popup
                    $scope.loadingState = false;
                    return BarcodeService.reInitScan();
                }
                return AlertService.restError(err.message || err.data ? err : { message: err }, "Não foi possível consultar o Cartão Presente")
                    .then(BarcodeService.reInitScan);
            }).finally(() => {
                lock = false;
                $scope.loadingState = false;
            });
        }

        function checkBalance(giftCard) {
            return $q((resolve, reject) => {
                if (giftCard.balanceAvailable <= 0) {                    
                    reject("Cartão presente já utilizado");
                    return;
                }

                const currentPayment = PaymentService.currentPayment();

                if (giftCard.balanceAvailable < currentPayment.paymentValue) {
                    const msg = `Saldo do cartão é ${currencyFormat(giftCard.balanceAvailable)} inferior ao valor do pagamento. Deseja prosseguir com o saldo como parte do pagamento?`;
                    AlertService
                        .confirm(msg)
                        .then((res) => {
                            if (!res) {
                                reject();
                                return;
                            }
                            currentPayment.paymentValue = giftCard.balanceAvailable;
                            resolve(giftCard);
                        });
                    return;
                }

                if (giftCard.balanceAvailable > currentPayment.paymentValue) {
                    const msg = `Você utilizará ${currencyFormat(PaymentService.currentPayment().paymentValue)}, restando ${currencyFormat(giftCard.balanceAvailable - currentPayment.paymentValue)} no saldo do cartão presente. Deseja prosseguir?`;
                    AlertService
                        .confirm(msg)
                        .then((res) => {
                            if (!res) {
                                reject();
                                return;
                            }
                            resolve(giftCard);
                        });
                    return;
                }

                resolve(giftCard);
            });
        }

        function currencyFormat(value) {
            const formatter = new Intl.NumberFormat('pt-BR', {
                style: 'currency',
                currency: 'BRL',
                minimumFractionDigits: 2
            });
            return formatter.format(value);
        };

        function navigate(giftCard) {
            UtilService.navigate('easyorder.passwordgiftcardpayment', {
                giftCard: giftCard
            });
        }
    });
})();

