(function () {
    'use strict';
    angular.module('easyorder.controllers.servicesale', ['easyorder.services.log'])
        .controller('servicesaleCtrl', function ($scope,
            $ionicLoading,
            $stateParams,
            UtilService,
            CartService,
            $ionicListDelegate,
            $ionicPopup,
            ServiceSaleService,
            AlertService,
            LogService) {

            $scope.groupsValues = [];
            $scope.selectedServices = [];
            $scope.services = [];
            $scope.updateServices = null;

            $scope.contractorDocument = {
                contractorDocument: '',
                contractorDocumentType: 'CPF'
            };

            $scope.data = {};

            /**
             * Get the radio button properties
             * @param {*} $event 
             * @param {*} parent 
             * @param {*} item 
             */
            $scope.getService = function (service, indexParam) {

                let span = document.getElementById("titleSpan" + indexParam);
                let input = document.getElementById("check" + indexParam);

                if ($scope.selectedServices.length !== 0) {

                    let index = null;
                    let notEquals = null;
                    let lastObject = null;

                    $scope.selectedServices.forEach((element, indexE) => {
                        if (element.type == service.type) {
                            index = indexE;
                            if (element.uid !== service.uid) {
                                lastObject = element;
                                notEquals = true;
                            }
                        }
                    });

                    if (index === null) {
                        removeClass(span);
                        span.className += ' icon-lbl-show';
                        service.checked = true;
                        span.textContent = "Selecionado";
                        service.indexParam = indexParam;
                        $scope.selectedServices.push(service);
                    } else if (index !== null && notEquals) {

                        service.indexParam = indexParam;

                        let input2 = document.getElementById("check" + lastObject.indexParam);
                        let span2 = document.getElementById("titleSpan" + lastObject.indexParam);

                        removeClass(span2);
                        input2.checked = false;
                        $scope.selectedServices[index].checked = false;
                        span2.textContent = "Selecione";

                        removeClass(span);
                        service.checked = true;
                        span.className += ' icon-lbl-show';
                        span.textContent = "Selecionado";

                        $scope.selectedServices.splice(index, 1);
                        $scope.selectedServices.push(service);

                    } else {
                        removeClass(span);
                        span.textContent = "Selecione";
                        service.checked = false;
                        $scope.selectedServices.splice(index, 1);
                    }

                } else {
                    service.indexParam = indexParam;
                    removeClass(span);
                    span.className += ' icon-lbl-show';
                    service.checked = true;
                    span.textContent = "Selecionado";
                    $scope.selectedServices.push(service);
                }
                input.checked = service.checked;
            };


            let removeClass = function (span) {
                let lClass = ' ' + span.className;
                if (span.className.indexOf(' icon-lbl-show') !== -1) {
                    lClass = span.className.replace(' icon-lbl-show', '');
                }
                span.className = lClass;
            };

            /**
             * Set the total value
             */
            $scope.getTotalServices = function () {

                if (!$scope.item) return 0;

                let total = 0;
                for (let index = 0; index < $scope.selectedServices.length; index++) {
                    const item = $scope.selectedServices[index];
                    total += parseFloat(item.value);
                }

                return total + $scope.item.unitValue;
            };

            /**
             *  Convert Response
             */
            $scope.prepareResponse = function (response) {

                let array = [];
                response.forEach((plan) => {

                    let index = null;
                    array.forEach((element, indexE) => {
                        if (element.type == plan.type) {
                            index = indexE;
                        }
                    });
                    if (index === null) {
                        array.push({
                            "name": additionalServices(plan.type),
                            "type": plan.type,
                            "plans": [plan]
                        });
                    } else {
                        array[index].plans.push(plan);
                    }

                    $scope.services = array;
                });
            };

            /**
             * Set the selected services in nonFiscalItems array
             */
            $scope.prepareNonFiscalItems = function () {
                return new Promise(async (res, rej) => {
                    if ($scope.selectedServices.length > 0) {
                        try {
                            for (let index = 0; index < $scope.selectedServices.length; index++) {

                                let nonFiscalItem = ServiceSaleService.prepareServices(null, {
                                    item: $scope.selectedServices[index],
                                    document: $scope.contractorDocument
                                });
                                $scope.selectedServices[index] = nonFiscalItem;
                            }

                            res($scope.selectedServices);
                        } catch (e) {
                            rej(e);
                        }
                    }
                });
            };


            /**
             * Method that gets the erro message
             * @param {*} error 
             */
            var defaultAddItemErrorCallback = function (error) {
                AlertService.error(error.message);
            };

            /**
             * 
             * @param {*} item 
             */
            var defaultAddItemSuccessCallback = function (item) {
                LogService.info('Adicionado item ao carrinho: ', item);
            };

            /**
             * Add the item to cart
             */
            $scope.confirm = function () {

                if ($scope.selectedServices.length <= 0) {
                    if ($scope.updateAdditionalServices) {
                        $scope.updateItemCart($scope.item);
                        CartService.reindexCartItems();
                    } else {
                        CartService.addItem($scope.item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
                    }
                    UtilService.navigateWithoutHistory('easyorder.cart');
                    return;
                }

                //Adicionando as garantias selecionadas no item fiscal
                let services = [];                    
                for (let index = 0; index < $scope.selectedServices.length; index++) {

                    let nonFiscalItem = ServiceSaleService.prepareServices(null, {
                        item: $scope.selectedServices[index],
                        document: {}
                    });
                    services.push(nonFiscalItem);
                }
                $scope.item.nonFiscalItems = services;
                if ($scope.updateAdditionalServices) {
                    $scope.updateItemCart($scope.item);
                    CartService.reindexCartItems();
                } else {
                    CartService.addItem($scope.item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
                }
                UtilService.navigateWithoutHistory('easyorder.cart');

                return;//Evitando que o sistema peca a leitura do documento do cliente  
                
                $ionicPopup.show({
                    title: 'DOCUMENTO DO CONTRATANTE',
                    templateUrl: 'views/contractor-document/contractor-document.html',
                    scope: $scope,
                    buttons: [{
                        text: 'Cancelar',
                        onTap: function (e) {
                            $ionicListDelegate.closeOptionButtons();
                        }
                    }, {
                        text: '<b>Adicionar</b>',
                        type: 'button-theme',
                        onTap: function (e) {

                            if ($scope.contractorDocument.contractorDocument &&
                                $scope.contractorDocument.contractorDocumentType &&
                                !$scope.invalidLength() &&
                                !$scope.data.validationError) {

                                $ionicLoading.show({
                                    template: 'Aguarde ...',
                                    duration: 0
                                }).then(async function () {

                                    try {
                                        let services = await $scope.prepareNonFiscalItems();
                                        if (services) {
                                            $scope.item.nonFiscalItems = services;
                                        }

                                        if ($scope.updateAdditionalServices) {
                                            $scope.updateItemCart($scope.item);
                                        } else {
                                            CartService.addItem($scope.item, defaultAddItemSuccessCallback, defaultAddItemErrorCallback);
                                        }

                                        $ionicLoading.hide();
                                    } catch (error) {
                                        $ionicLoading.hide();
                                        AlertService.error(error.data.message);
                                    }

                                    UtilService.navigateWithoutHistory('easyorder.cart');
                                });

                            } else {
                                e.preventDefault();
                            }
                        }
                    }]
                });
            };

            /**
             * 
             */
            $scope.updateItemCart = function (item) {
                CartService.cart.fiscalItems.forEach(element => {
                    if (element.sequenceId == item.sequenceId) {
                        element.nonFiscalItems = item.nonFiscalItems;
                    }
                });
            };

            /**
             * 
             */
            $scope.updateSelectedServices = function (item, services) {
                if ($scope.updateAdditionalServices && $scope.item.nonFiscalItems && $scope.item.nonFiscalItems.length > 0) {
                    let selectedServices = [];
                    item.nonFiscalItems.forEach(nonFiscalItem => {
                        services.find((service, index) => {
                            if (service.id == nonFiscalItem.automationCode) {
                                service.indexParam = index;
                                selectedServices.push(service);
                            }
                        })
                    });
                    selectedServices.forEach(selected => {
                        $scope.getService(selected, selected.indexParam);
                    });
                }
            };

            /**
             *
             */
            $scope.validateDocument = function (documentType) {

                $scope.data.validationError = false;

                let document = $scope.contractorDocument.contractorDocument;
                if (documentType && document) {
                    if ((documentType === "CPF" && document.length == 11 && !validaCPF(document)) ||
                        (documentType === "CNPJ" && document.length == 14 && !validaCNPJ(document)) ||
                        (documentType === "PASSPORT" && document.length == 9 && !validaPassport(document))) {
                        $scope.data.validationError = true;
                    }
                }
                return $scope.data.validationError;
            };

            /**
             * 
             */
            $scope.invalidLength = function () {
                let document = $scope.contractorDocument.contractorDocument;
                let documentType = $scope.contractorDocument.contractorDocumentType;
                if (documentType && document) {
                    if ((documentType === "CPF" && document.length < 11) ||
                        (documentType === "CNPJ" && document.length < 14) ||
                        (documentType === "PASSPORT" && document.length < 9)) {
                        return true;
                    }
                }
                return false;
            };

            /**
             * 
             */
            $scope.getServiceName = function (name) {
                return additionalServices(name);
            };

            $scope.$on('$ionicView.enter', function () {
                // Any thing you can think of
                $scope.updateSelectedServices($scope.item, $scope.services);
            });

            (function () {
                $scope.item = $stateParams.item;
                $scope.services = $stateParams.services;
                $scope.updateAdditionalServices = $stateParams.updateAdditionalServices;
                //Primeira função a ser chamada.
            })();
        })
})();
