﻿(function() {
	'use strict';

	angular.module('easyorder.controllers.showroomingfulfillments', [
        'ln.currency',
		'easyorder.services.cart',
		'easyorder.services.util',
		'easyorder.services.config',
		'easyorder.services.user',
		'easyorder.services.alert',
		'easyorder.services.customer',
		'easyorder.services.log'
    ])
    .controller('ShowroomingFulfillmentsCtrl', function($scope,
                                     $ionicModal,
									 $ionicLoading,
									 $stateParams,
									 $sce,
                                     CartService,
                                     ConfigService,
									 CustomerService,
                                     FreightService,
                                     ProductService,
									 AlertService,
									 UtilService,
									 LogService,
									 $q) {


    	$scope.showPickups = false;
        $scope.showShipments = false;
        // a cotação do frete deve vir na transição de estado
        $scope.freightQuote = $stateParams.freightQuote;
        $scope.freightQuoteWithShipments = $stateParams.freightQuote.find(fq => fq.groups[0].shipments != null);
        // exibe ou oculta a lista de lojas
        $scope.storesShown = false;
        $scope.freightSelected = {};
        $scope.isOnlyPickupInStore = $scope.cart.showroomingInfo.searchOption === 'pickup_in_store';
        $scope.deliveryAddress = $scope.cart.deliveryAddress;
		$scope.customer = $scope.cart.customer;
		$scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
        $scope.freeShipment = "GRÁTIS";
        const freightLabels = ConfigService.config.business.showrooming_freight_labels;
        $scope.shipment_label = freightLabels['SHIPMENT'] || freightLabels['DEFAULT'];
        $scope.pickup_label = freightLabels['PICKUP'] || freightLabels['DEFAULT'];

		$scope.getZipCodeFormatter = function(zipcode){
			if (zipcode) {
				return zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3');
			}
			return null;
		};

		$scope.zipCodeFormat = $scope.getZipCodeFormatter($scope.deliveryAddress.zipCode);

        if ($scope.freightQuote && $scope.freightQuote.length > 0) {
        	// só exibe os dados se foi passado a cotação de fretes
            var hasShowrooming = false;
            for (const fq of $scope.freightQuote) {
                if (fq && fq.groups && fq.groups.length > 0) {
                    var group = fq.groups[0];
                    if (group.pickups && group.pickups.length > 0) {
                        $scope.showPickups = true;
                    }
                    if (group.shipments && group.shipments.length > 0) {
                        $scope.showShipments = true;
                    }

                    if ($scope.showPickups && !$scope.showShipments) {
                        $scope.storesShown = true;
                    }
                }
            }
        }

        const getDeliveryDate = function(days) {
            let now = new Date();
            now.setDate(now.getDate() + days);
            return now.toLocaleDateString();
        }

        $scope.getFreightMessage = function(freight) {
            const label = freight.type === 'SHIPMENT' ? $scope.shipment_label : $scope.pickup_label;
            return label
                .replace("${FREIGHT_DAYS}", freight.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(freight.time))
                .replace("\\n", "\n");
        }

        $scope.getPickupMessage = function(pickup) {
            return $scope.pickup_label
                .replace("${FREIGHT_DAYS}", pickup.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(pickup.time))
                .replace("\\n", "\n");
        }

        $scope.getShipmentMessage = function(ship) {
            return $scope.shipment_label
                .replace("${FREIGHT_DAYS}", ship.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(ship.time))
                .replace("\\n", "\n");
        }

		$scope.getFreightDescription = function(ship) {
			return ship.method.replace("Entrega", "");
		}

        $scope.toggleStores = function() {
        	$scope.storesShown = !$scope.storesShown;
        	if ($scope.storesShown) {
        		$scope.freightSelected = {};
			   	CartService.removeFreightValue();
        	}
        }

        $scope.selectFreightDeliveryShipment = function(groupId, delivery, quoteId) {
            if (delivery) {
                $scope.freightSelected = {};
                $scope.freightSelected.type = "SHIPMENT";
                $scope.freightSelected.description = delivery.method;
                $scope.freightSelected.price = delivery.price;
                $scope.freightSelected.time = delivery.time;
                $scope.freightSelected.id = delivery.id;
                $scope.freightSelected.groupId = groupId;
                $scope.freightSelected.quoteId = quoteId;
                CartService.updateFreightValue($scope.freightSelected.price);
            	$scope.storesShown = false;
            	$scope.isOnlyPickupInStore = false;
            }
        };
        $scope.selectFreightDeliveryPickup = function(groupId, pickup, quoteId) {
            if (pickup) {
                $scope.freightSelected = {};
                $scope.freightSelected.type = "PICKUP";
                $scope.freightSelected.description = "Retirada na Loja";
                $scope.freightSelected.storeName = pickup.store.name;
                $scope.freightSelected.time = pickup.time;
                $scope.freightSelected.id = pickup.id;
                $scope.freightSelected.groupId = groupId;
                $scope.freightSelected.quoteId = quoteId;
			   	CartService.removeFreightValue();
            	$scope.storesShown = false;
            	$scope.isOnlyPickupInStore = true;
            }
        };

        // Modal de confirmação de dados do cliente
		$ionicModal.fromTemplateUrl('views/showrooming/showroomingcustomerinfoconfirmation.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.showroomingAddressCompleteModal = modal;
		});

		$scope.openModal = function() {
			let searchPromise = $q.resolve({});

			if ($scope.isOnlyPickupInStore) {
				$scope.address = $scope.customer.address || {};
			} else {
				$scope.address = $scope.deliveryAddress || {};
			}

			// criando cópia
			$scope.address = JSON.parse(JSON.stringify($scope.address));

			if (ConfigService.config.business.address_service_provider
			 	 && isAddressIncomplete($scope.address)
				 && $scope.address.zipCode) {
				searchPromise = $ionicLoading.show({
					template: 'Consultando endereço...',
					duration: 0
				}).then(() =>
					CustomerService.getAddress($scope.address.zipCode)
				).finally(() =>
					$ionicLoading.hide()
				);
			}

			searchPromise.then((addr) => {
				const dAddr = $scope.address;

				dAddr.addressName = chkstr(dAddr.addressName, addr.address, "");
				dAddr.neighborhood = chkstr(dAddr.neighborhood, addr.district, "");
				dAddr.state = chkstr(dAddr.state, addr.state, "");
				dAddr.city = chkstr(dAddr.city, addr.city, "");
				dAddr.cityCode = chkstr(dAddr.cityCode, addr.cityCode, "");
                dAddr.addressNumber = chkstr( // Se o CEP for o mesmo, utilizará o mesmo número do endereço do cliente. O operador poderá mudar no momento da confirmação dos dados do cliente no final do processo
					dAddr.addressNumber,
					$scope.cart.customer && $scope.cart.customer.address && $scope.cart.customer.address.zipCode == $scope.address.zipCode ? $scope.cart.customer.address.addressNumber : "",
					""
				);
				dAddr.addressComplement = chkstr( // Se o CEP for o mesmo, utilizará o mesmo complemento do endereço do cliente. O operador poderá mudar no momento da confirmação dos dados do cliente no final do processo
					dAddr.addressComplement,
					$scope.cart.customer && $scope.cart.customer.address && $scope.cart.customer.address.zipCode == $scope.address.zipCode ? $scope.cart.customer.address.addressComplement : "",
					""
				);
                dAddr.phone = chkstr(
					dAddr.phone,
					$scope.cart.customer && $scope.cart.customer.address ? $scope.cart.customer.address.phone : "",
					""
				);
			}).finally(() =>
				checkLoadedAdressFields()
			).then(() =>
				$scope.showroomingAddressCompleteModal.show()
			).catch((err) =>
				//Futuramente será necessário enviar o cityCode para vendas acima de X valor
				//Nesse caso se ultrapassar o valor X, não devemos abrir o modal
				AlertService.restError(err, 'Falha ao consultar CEP').then(() =>
					$scope.showroomingAddressCompleteModal.show()
				)
			);
		};

        $scope.findAddress = function() {
            if (!$scope.address.zipCode) {
				resetAddressFields();
				checkLoadedAdressFields();
            }

			if ($scope.address.zipCode && $scope.address.zipCode.length == 8) {
                $ionicLoading.show({
                    template : 'Buscando endereço por CEP...',
                    duration : 0
                }).then(function() {
                    CustomerService.getAddress($scope.address.zipCode, function(result) {
                        $ionicLoading.hide();
                        LogService.info(result);
                        $scope.address.addressName = result.address;
						$scope.address.addressNumber = "";
						$scope.address.addressComplement = "";
                        $scope.address.neighborhood = result.district;
                        $scope.address.state = result.state;
                        $scope.address.city = result.city;
                        $scope.address.cityCode = result.cityCode;
						checkLoadedAdressFields();
					}).catch(error => {
                        $ionicLoading.hide();
                        var errMsg = error.data.message ? error.data.message : "Falha ao realizar consulta de CEP";
                        AlertService.warn(errMsg);
                        LogService.warn(error);
                    });
                });
            }
        }

		function isAddressIncomplete(addr) {
			return addr == null
				|| !addr.addressName
				|| !addr.neighborhood
				|| !addr.state
				|| !addr.city
				|| !addr.cityCode
		}

		function checkLoadedAdressFields() {
			const addr = $scope.address;

			$scope.addressAddressNameLoaded = addr.addressName && addr.addressName.length > 0;
			$scope.addressNeighborhoodLoaded = addr.neighborhood && addr.neighborhood.length > 0;
			$scope.addressStateLoaded = addr.state && addr.state.length > 0;
			$scope.addressCityLoaded = addr.city && addr.city.length > 0;
		};

		function resetAddressFields() {
			$scope.address.addressName = "";
			$scope.address.addressNumber = "";
			$scope.address.addressComplement = "";
			$scope.address.neighborhood = "";
			$scope.address.state = "";
			$scope.address.city = "";
			$scope.address.cityCode = "";
		}

		function chkstr(str1, str2, str3) {
			return str1 || str2 || str3;
		}

		$scope.closeModal = function() {
			$scope.showroomingAddressCompleteModal.hide();
		};

       	$scope.confirmCustomerData = function() {
    	   if ($scope.freightSelected && $scope.freightSelected && $scope.freightSelected.type) {
    		   if ($scope.address && $scope.address.phone) {
        		   var isShipment = ($scope.freightSelected.type === 'SHIPMENT');
        		   if (isShipment) {
    	    			FreightService.validateDeliveryAddress($scope.address).then(function() {
    	    				LogService.info('Endereço válido!');
							$scope.cart.deliveryAddress = $scope.address;
    	    	            $scope.closeModal();
    	    				$scope.confirm();
    					}).catch(errorMessage => {
    						AlertService.warn(errorMessage);
    					})
        		   } else {
        			   // pickup
					   FreightService.validateDeliveryAddress($scope.address).then(function() {
    	    				LogService.info('Pickup válido!');
							$scope.cart.customer.address = $scope.address;
    	    	            $scope.closeModal();
    	    				$scope.confirm();
    					}).catch(errorMessage => {
    						AlertService.warn(errorMessage);
    					})
        		   }
    		   } else {
    			   AlertService.warn('Por favor, preencha o celular do cliente.');
    		   }
    	   }
	   };

	   $scope.storeMapAddrress = function(currentStore) {
			var ret = 'https://www.google.com/maps/embed/v1/place?key=AIzaSyD241usOdsnkPzXCoIMvyc53ieJ7S90AiI&';
			ret += 'language=pt&';
			ret += 'region=BR&';

			if (currentStore && currentStore.address) {
				var address = currentStore.address;

				var addSep = address.streetAddress.indexOf(',');
				addSep =  addSep > 0 ? addSep : address.streetAddress.length;

				ret += 'zoom=15&';
				ret += 'q=' + (address.streetAddress ? address.streetAddress.substring(0, addSep).replace(/\s+/g, '+') : '')
							+ (address.streetNumber ? ',' + address.streetNumber.replace(/\s+/g, '+') : '')
						+ (address.neighborhood ? ',' + address.neighborhood.replace(/\s+/g, '+') : '')
						+ (address.postalCode ? ',' + address.postalCode.replace(/\D/g, '').replace(/\s+/g, '+') : '');
						+ (address.city ? ',' + address.city.replace(/\s+/g, '+') : '')
						+ (address.state ? ',' + address.state.replace(/\s+/g, '+') : '');
			} else {
				ret += 'zoom=3&';
				ret += 'q=Brasil';
			}
			return $sce.trustAsResourceUrl(ret);
		};

	   $scope.confirm = function() {
			UtilService.navigateWithoutHistory('easyorder.showroomingsummary', { freightSelected : $scope.freightSelected });
	   };

       $scope.getProductImageUrl = function(item) {
           if (item) {
               return ProductService.getProductImageUrl(item);
           }
           return null;
       };
	});
})();
