(function() {
	'use strict';

	angular.module('easyorder.controllers.showroomingitemfreightdetails', [
        'ln.currency',
		'easyorder.services.cart',
		'easyorder.services.util',
		'easyorder.services.config',
		'easyorder.services.user',
		'easyorder.services.alert',
		'easyorder.services.customer',
    ])
	.controller('ShowroomingItemFreightCtrl', function($scope,
									 $stateParams,
                                     $q,
                                     CartService,
                                     $ionicModal,
                                     ConfigService,
                                     $ionicLoading,
                                     ProductService,
                                     FreightService,
                                     $sce,
									 AlertService,
                                     UtilService,
                                     DiscountService,
                                     LogService) {

        $scope.item = $stateParams.currentItem;
        $scope.freightQuote = $stateParams.freightQuote;
        $scope.freightQuoteWithShipments = $stateParams.freightQuote.find(fq => fq.groups[0].shipments != null);
    	$scope.showPickups = false;
        $scope.showShipments = false;
        $scope.freeShipment = "GRÁTIS";
        const freightLabels = ConfigService.config.business.showrooming_freight_labels;
        $scope.shipment_label = freightLabels['SHIPMENT'] || freightLabels['DEFAULT'];
        $scope.pickup_label = freightLabels['PICKUP'] || freightLabels['DEFAULT'];

        const getDeliveryDate = function(days) {
            let now = new Date();
            now.setDate(now.getDate() + days);
            return now.toLocaleDateString();
        }

        if ($scope.item && $scope.freightQuote && $scope.freightQuote.length > 0) {
        	// só exibe os dados se foi passado um ítem e cotação de fretes
            var hasShowrooming = false;
            for(const fq of $scope.freightQuote) {
                if (fq && fq.groups && fq.groups.length > 0) {
  	  			    var group = fq.groups[0];
                    if (group.pickups && group.pickups.length > 0) {
                        $scope.showPickups = true;
                    }
                    if (group.shipments && group.shipments.length > 0) {
                        $scope.showShipments = true;
                    }
                }
            }

		}

        $scope.getPickupMessage = function(pickup) {
            return $scope.pickup_label
                .replace("${FREIGHT_DAYS}", pickup.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(pickup.time))
                .replace("\\n", "\n");
        }

        $scope.getShipmentMessage = function(ship) {
            return $scope.shipment_label
                .replace("${FREIGHT_DAYS}", ship.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(ship.time))
                .replace("\\n", "\n");
        }

        const searchItemListFreight = () => {
	          var cart = CartService.cart;
	          var item = $scope.item;
	          // obtém uma cópia da lista de showrooming
	          var showroomingItems = CartService.getShowroomingItems();
	          // e adiciona o novo ítem
	          showroomingItems.push({
					sku:		item.productCode,
					quantity:	1,
					price:		item.unitValue,
					weight:		item.weight
				});

	      	  var isPickupInStore =  cart.showroomingInfo.searchOption === 'pickup_in_store';
	      	  var loadingMessage;
	      	  var zipcode;
	      	  if (isPickupInStore) {
	      		  loadingMessage = "Consultando lojas disponíveis para retirada de todos produtos do carrinho...";
	      	  } else {
	      		  zipcode = cart.deliveryAddress.zipCode;
	      		  loadingMessage = "Consultando opções de entrega para o CEP '" + $scope.getZipCodeFormatter(zipcode) + "' de todos os produtos do carrinho...";
	      	  }

	          $ionicLoading.show({
	              template :  loadingMessage,
	              duration : 0
	          }).then(function() {
	              var error = function(error) {
	                  $scope.loadingState = false;
	                  $ionicLoading.hide();
	                  AlertService.restError(error, 'Erro ao consultar opções de entrega!');
	              };

	            	var success = function(freights) {
	                  $scope.loadingState = false;
	                  $ionicLoading.hide();

	                  var hasShowrooming = false;
	                  if (freights && freights.length > 0)  {
                        for (const f of freights) {
                            if (f && f.groups && f.groups.length > 0) {
                                const group = f.groups[0];
                                if  (group.pickups && group.pickups.length > 0) {
                                    hasShowrooming = true;
                                }
                                // se for apenas pickup in store, não considera os shipments. E remove-os para não serem expostos e escolhidos.
                                if (isPickupInStore) {
                                    group.shipments = [];
                                } else {
                                    if (group.shipments && group.shipments.length > 0) {
                                        hasShowrooming = true;
                                    }
                                }
                            }
                        }
                    }
                      if (hasShowrooming) {
                        // adiciona o ítem apenas houverem opções de entrega.
                        if (!item.productType) { // Os dados estão resumidos?
                            $scope.getItemDetailsAndAddToCart(item);
                        } else {
        		            $scope.doAddToCart(item);
                        }
                      } else {
                        if (isPickupInStore) {
                            AlertService.error("Nenhuma loja disponível para entrega de todos os produtos do carrinho!")
                        } else {
                            AlertService.error("Não existem mais opções de entrega para os itens atuais no CEP: '" + $scope.getZipCodeFormatter(zipcode) + "'.")
                        }
                      }
	            	}

                    const orderValue = cart.totalValue - cart.totalDiscount;

	            	if (isPickupInStore) {
	            		FreightService.searchItemListByStore(cart.companyNumber, cart.storeNumber, orderValue, showroomingItems, success, error);
	            	} else {
	            		FreightService.searchItemListByZipcode(cart.companyNumber, cart.storeNumber, zipcode, orderValue, showroomingItems, success, error);
	            	}
	          });
        };

        $scope.addToCart = function(item){
        	if (CartService.hasShowrooming()) {
        		searchItemListFreight();
        	} else {
                if (!item.productType) { // Os dados estão resumidos?
                    $scope.getItemDetailsAndAddToCart(item);
                } else {
        		    $scope.doAddToCart(item);
                }
        	}
        };

        $scope.storeMapAddrress = function(currentStore) {
		    var ret = 'https://www.google.com/maps/embed/v1/place?key=AIzaSyD241usOdsnkPzXCoIMvyc53ieJ7S90AiI&';
		    ret += 'language=pt&';
		    ret += 'region=BR&';

            if (currentStore && currentStore.address) {
                var address = currentStore.address;

                var addSep = address.streetAddress.indexOf(',');
                addSep =  addSep > 0 ? addSep : address.streetAddress.length;

                ret += 'zoom=15&';
                ret += 'q=' + (address.streetAddress ? address.streetAddress.substring(0, addSep).replace(/\s+/g, '+') : '')
                    		+ (address.streetNumber ? ',' + address.streetNumber.replace(/\s+/g, '+') : '')
                        + (address.neighborhood ? ',' + address.neighborhood.replace(/\s+/g, '+') : '')
                        + (address.postalCode ? ',' + address.postalCode.replace(/\D/g, '').replace(/\s+/g, '+') : '');
                        + (address.city ? ',' + address.city.replace(/\s+/g, '+') : '')
                        + (address.state ? ',' + address.state.replace(/\s+/g, '+') : '');
            } else {
                ret += 'zoom=3&';
                ret += 'q=Brasil';
            }
            return $sce.trustAsResourceUrl(ret);
		};

         /**
         * Adição do produto selecionado ao carrinho após confirmação do endereço de entrega ou CEP.
         */
        $scope.doAddToCart = (item) => {
		    const successCallback = (prd) => {
                const nextScreen = () => {
                    if ($stateParams.prevState) {
                        UtilService.navigateWithoutHistory($stateParams.prevState);
                    } else {
                        UtilService.navigateToMainScreen();
                    }
                };

                processAutomaticDiscount(item)
                .then((resp) => {
                    if (!resp || resp.result == null) {
                        nextScreen();
                        return;
                    }

                    UtilService.navigateWithoutHistory('easyorder.productdiscounts', {
                        item: prd,
                        automatic: true,
                        discountResult: resp.result,
                        prevState: 'easyorder.showroomingitemfreightdetails'
                    });
                }).catch((err) => {
                    LogService.error(err);
                    nextScreen();
                });
            };

            const errorCallback = function(error) {
                AlertService.error(error.message);
            };

            if ('GROUP' === item.productType || 'BUNDLE' === item.productType) {
                ProductService.search({
                    groupCode : item.productCode
                }, (result) => {
                    if (result.items.length > 0) {
                        const groupCode = item.productCode;
                        const quantitySold = item.quantitySold

                        result.items.forEach((itemResult, index) => {
                            itemResult.groupCode = groupCode;
                            itemResult.quantitySold = quantitySold;

                            if (index === result.items.length - 1) {
                                itemResult.showrooming = true; // TODO grupo acho q nao deve entrar no showrooming
                                CartService.addItem(itemResult, successCallback, errorCallback);
                            } else {
                                CartService.addItem(itemResult, null, errorCallback);
                            }
                        });
                    }
                }, (error) => {
                    AlertService.restError(error, 'Erro consultando produtos do agrupamento!');
                });
            } else {
                item.showrooming = true;
                CartService.addItem(item, successCallback, errorCallback);
            }
        };

        function processAutomaticDiscount(item) {
            const defer = $q.defer();

            if (ConfigService.config.business.enable_screen_discount) {
                const discountParams = {
                    productCode: item.productCode,
                    employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false,
                    ownBrand: item.ownBrand,
                    companyNumber: ConfigService.config.companyNumber,
                    marketingCode: item.marketingCode,
                    promotionalPrice: item.promotionalPrice,
                    automatic: true,
                    unitValue: item.unitValue,
                    storeNumber: ConfigService.config.storeNumber
                };

                $ionicLoading.show({
                    template: 'Verificando desconto para produto...',
                    duration: 0
                }).then(() => {
                    DiscountService.searchDiscountItem(discountParams, function (result) {
                        LogService.info("Resultado do desconto: ", result);

                        $ionicLoading.hide();

                        if (result && result.discountRuleTypes && result.discountRuleTypes[0] && result.discountRuleTypes[0].discountRules.length) {
                            defer.resolve({ result: result, item: item });
                        } else {
                            AlertService.info("Não há regras de desconto disponíveis para o item.").then(defer.resolve);
                        }
                    }, function (error) {
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Erro consultando desconto no servidor!').then(function () {
                            defer.resolve({ result: undefined, item: item });
                        });
                    }).finally(() => {
                        $scope.loadingState = false;
                    });
                });
            } else {
                defer.resolve();
            }

            return defer.promise;
        };

        $scope.getItemDetailsAndAddToCart = function(item) {
            $ionicLoading.show({
                    template : 'Aguarde... Consultando detalhes do produto',
                    duration : 0
                })
                .then(() => ProductService.search({code: item.productCode}))
                .then(result => result.items[0])
                .then(itemSrc => itemSrc ? itemSrc : $q.reject({data : {message: 'Nenhum produto encontrado'}}))
                .then(itemSrc => {
                    itemSrc.quantitySold = item.quantitySold;
                    return itemSrc;
                })
                .then(itemSrc => $scope.doAddToCart(itemSrc))
                .finally($ionicLoading.hide)
                .catch(error => {
                    AlertService.show('Erro ao carregar detalhes do produto', error.message, 'Voltar');
                });
        }

		$scope.getZipCodeFormatter = function(zipcode) {
			return zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3');
		};

        $scope.getProductImageUrl = function(item) {
            if (item){
                return ProductService.getProductImageUrl(item);
            }
            return null;
        };
        $scope.getFreightDescription = function(ship) {
			return ship.method.replace("Entrega", "");
		}
	});
})();
