(function() {
	'use strict';

	angular.module('easyorder.controllers.showroomingsummary', [
        'ln.currency',
		'easyorder.services.cart',
		'easyorder.services.util',
		'easyorder.services.config',
		'easyorder.services.user',
		'easyorder.services.alert',
		'easyorder.services.customer',
    ])
    .controller('ShowroomingSummaryCtrl', function($scope,
                                     $ionicModal,
                                     $ionicLoading,
                                     $stateParams,
                                     CartService,
                                     ConfigService,
                                     FreightService,
                                     ProductService,
									 AlertService,
                                     UtilService,
                                     OrderService) {

        $scope.cart = CartService.cart;
        $scope.freightSelected = $stateParams.freightSelected;
		$scope.isOnlyPickupInStore = $scope.freightSelected.type === 'PICKUP';
		$scope.itemMultiplication = ConfigService.config.business.item_multiplication_enabled;
		$scope.showroomingGenerateOrderEnabled = ConfigService.config.business.showrooming_generate_order_enabled;
        const freightLabels = ConfigService.config.business.showrooming_freight_labels;
        $scope.shipment_label = freightLabels['SHIPMENT'] || freightLabels['DEFAULT'];
        $scope.pickup_label = freightLabels['PICKUP'] || freightLabels['DEFAULT'];

		if ($scope.cart.deliveryAddress && $scope.cart.deliveryAddress.zipCode) {
			$scope.zipCodeFormat = $scope.cart.deliveryAddress.zipCode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3');
		}

        if ($scope.cart.customer && $scope.cart.customer.document) {
			if ($scope.cart.customer.documentType == "CPF") {
				$scope.documentoFormatado = applyCpfMask($scope.cart.customer.document);
			} else if ($scope.cart.customer.documentType == "CNPJ") {
				$scope.documentoFormatado = applyCnpjMask($scope.cart.customer.document);
			}else {
				$scope.documentoFormatado = $scope.cart.customer.document;
			}
		};

    	$scope.createOrder = function() {
			$ionicLoading.show({
				template: 'Gerando pedido...',
				duration: 0
			}).then(function() {
				var successFn = function(order) {
					$ionicLoading.hide();

					AlertService.info("Pedido: " + order.orderNumber + " gerado com sucesso!").then(function() {
				        CartService.initCart();
						UtilService.navigateToMainScreen();
					});
				};

				var errorFn = function(error) {
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro gerando pedido!');
				};

				// Clona e converte o carrinho no order
				var order = OrderService.cartToOrderConverter(CartService.cart);

				if (order) {
					var isShipment = ($scope.freightSelected.type === 'SHIPMENT');
					// injeta campos do showrooming
					order.status = 'CHECKOUT';
					order.type = 'SHOWROOMING';

					// remove mascara do documento
					if (order.customer && order.customer.document) {
						order.customer.document = removeMask(order.customer.document);
					}

					if ($scope.cart.customer.customerId){
						order.customer.customerId =  $scope.cart.customer.customerId;
					}

					// ajusta sequencial e dados do delivery
					var i;
					for (i = 0; i < order.fiscalItems.length; i++) {
						var fiscalItem = order.fiscalItems[i];

						if (!fiscalItem.sequenceId) {
							fiscalItem.sequenceId = i + 1;
						}
						if (fiscalItem.showrooming) {
							fiscalItem.deliveryOption = (isShipment ? 'WAREHOUSE_DELIVERY' : 'PICK_UP_IN_STORE');
	                        fiscalItem.showrooming = undefined;
						} else {
							fiscalItem.deliveryOption = 'SOLD_IN_STORE';
						}
						fiscalItem.totalValue = Math.round(fiscalItem.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.
					}
					order.totalValue =  Math.round(order.totalValue * 100) / 100; // Eliminando casas decimais para apenas duas.

					// cria a cotação de frete escolhida
					var freightInfo = {
			                quoteId: $scope.freightSelected.quoteId,
			                freightQuotes : [{
			                        groupId: $scope.freightSelected.groupId,
			                        id: $scope.freightSelected.id,
			                        type: $scope.freightSelected.type
			                    }
			                ]
			            }
					OrderService.reserveAndCreate(order, freightInfo, successFn, errorFn);
				} else {
					$ionicLoading.hide();
					AlertService.error('Não foi possível converter objeto de requisição do pedido!');
				}
			});
		};

        $scope.payOrder = function() {
			// se optar por pagar, o frete selecionado é armazenado nos dados de showrooming para uso posterior
				$scope.cart.showroomingInfo.freightSelected = $scope.freightSelected;
				UtilService.navigateWithoutHistory('easyorder.paymentmethods');
        };

        $scope.getProductImageUrl = function(item) {
            if (item){
                return ProductService.getProductImageUrl(item);
            }
            return null;
        };

        const getDeliveryDate = function(days) {
            let now = new Date();
            now.setDate(now.getDate() + days);
            return now.toLocaleDateString();
        }

        $scope.getFreightMessage = function(freight) {
            const label = freight.type === 'SHIPMENT' ? $scope.shipment_label : $scope.pickup_label;
            return label
                .replace("${FREIGHT_DAYS}", freight.time)
                .replace("${FREIGHT_DATE}", getDeliveryDate(freight.time))
                .replace("\\n", "\n");
        };
	});
})();
