(function() {
	'use strict';

	angular.module('easyorder.controllers.showroomingzipcode', [
        'ln.currency',
		'easyorder.services.cart',
		'easyorder.services.util',
		'easyorder.services.config',
		'easyorder.services.user',
		'easyorder.services.alert',
		'easyorder.services.customer',
    ])
	.controller('ShowroomingZipcodeCtrl', function($scope,
									 $stateParams,
                                     CartService,
                                     ConfigService,
                                     $ionicLoading,
                                     FreightService,
									 AlertService,
                                     UtilService) {

        $scope.customer = CartService.cart.customer;
        $scope.cart = CartService.cart;
        $scope.cart.showroomingInfo.searchOption = ($scope.cart.showroomingInfo.searchOption?$scope.cart.showroomingInfo.searchOption:'zipcode');
        $scope.showroomingInfo = $scope.cart.showroomingInfo;
        $scope.item = $stateParams.currentItem;
        $scope.customerHasDeliveryAddresses =  ($scope.customer && $scope.customer.deliveryAddresses && $scope.customer.deliveryAddresses.length > 0);

        $scope.selectCustomerAddress = function(address){
        	if (address && address.zipCode) {
                var confirmPopup = AlertService.confirm("Utilizar o endereço de CEP '"+ address.zipCode +"' como o de entrega?");
    			confirmPopup.then(
    				function(result) {
    					if (result) {
    						// sobrescreve o endereço de entrega atual
                            $scope.cart.deliveryAddress = address;
                            $scope.doSearchFreight();
                        }
                    }
    			);
        	} else {
        		AlertService.error('Endereço de entrega não possui um CEP válido!');
        	}
        };

		$scope.getZipCodeFormatter = function(zipcode){
			if (zipcode){
				return zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3');
			}
			return null;
		};

        $scope.doSearchFreight = function(){
        	var currentZipcode = $scope.cart.deliveryAddress.zipCode;
        	switch ($scope.showroomingInfo.searchOption) {
        		case 'pickup_in_store':
        			// se for pickup in store, remove o endereço de entrega atual
        			$scope.cart.deliveryAddress = {};
        			searchItemFreight($scope.item);
        			break;
        		case 'zipcode':
        			if (currentZipcode) {
        				// remove o endereço de entrega atual e insere apenas o zipcode
						$scope.cart.deliveryAddress = {};
						$scope.cart.deliveryAddress.zipCode = currentZipcode;
        				searchItemFreight($scope.item);
        			}
        			break;
        		case 'customer_address':
        			if (currentZipcode) {
        				// endereço já sobrescrito na confirmação
        				searchItemFreight($scope.item);
        			}
        			break;
        	}
        };

        const searchItemFreight = (item) => {
			var isPickupInStore =  $scope.showroomingInfo.searchOption === 'pickup_in_store';
			var loadingMessage;
			var zipcode;
			if (isPickupInStore) {
				loadingMessage = "Consultando lojas disponíveis para retirada do produto '" + item.productDescription + "'...";
			} else {
				zipcode = $scope.cart.deliveryAddress.zipCode;
				loadingMessage = "Consultando opções de entrega para o CEP " + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3')  + "...";
			}

			$ionicLoading.show({
				template :  loadingMessage,
				duration : 0
			}).then(function() {
				var error = function(error) {
					$scope.loadingState = false;
					$ionicLoading.hide();
					AlertService.restError(error, 'Erro ao consultar opções de entrega!');
				};

				var success = function(freights) {
					$scope.loadingState = false;
					$ionicLoading.hide();

					var hasShowrooming = false;
                    if (freights && freights.length > 0)  {
                        for (const f of freights) {
                            if (f && f.groups && f.groups.length > 0) {
                                const group = f.groups[0];
                                if  (group.pickups && group.pickups.length > 0) {
                                    hasShowrooming = true;
                                }
                                // se for apenas pickup in store, não considera os shipments. E remove-os para não serem expostos e escolhidos.
                                if (isPickupInStore) {
                                    group.shipments = [];
                                } else {
                                    if (group.shipments && group.shipments.length > 0) {
                                        hasShowrooming = true;
                                    }
                                }
                            }
                        }
                    }

					if (hasShowrooming) {
						UtilService.navigateWithoutHistory('easyorder.showroomingitemfreightdetails', { currentItem : item, freightQuote: freights});
					} else {
						if (isPickupInStore) {
							AlertService.info("Não foram encontradas lojas próximas disponíveis para entregar o produto: '" + item.productDescription + "'.")
						} else {
							AlertService.info("Nenhuma opção de entrega disponível para o produto: '" + item.productDescription + "', no CEP: " + zipcode.replace(/^(\d{2})(\d{3})(\d{3}).*/, '$1.$2-$3') + ".")
						}
					}
				}

                const orderValue = MathUtils.add(
                    MathUtils.subtract($scope.cart.totalValue, $scope.cart.totalDiscount),
                    MathUtils.multiply(item.unitValue, item.quantitySold)
                );

				if (isPickupInStore) {
                    FreightService.searchSingleItemByStore($scope.cart.companyNumber, $scope.cart.storeNumber, orderValue, item, success, error);
				} else {
					FreightService.searchSingleItemByZipcode($scope.cart.companyNumber, $scope.cart.storeNumber, zipcode, orderValue, item, success, error);
				}
			});
      	}

	});
})();
