(function () {
    'use strict';
    angular.module('easyorder.controllers.supply', ['easyorder.services.log'])
        .controller('supplyCtrl', function ($scope,
            $ionicLoading,
            $state,
            ConfigService,
            CartService,
            $ionicModal,
            $ionicPopup,
            UserService,
            PrinterService,
            AlertService,
            LogService,
            UtilService) {
            (function () {
                //Primeira função a ser chamada.
                $scope.supply = {};
                $scope.supply.supplyValue = 0.00;       
            })();

            $scope.userAuthorized = false;
            $scope.imprimir = false;
            var documents = new Array();

            $ionicModal.fromTemplateUrl('views/printerslist.html', {
                scope: $scope,
                animation: 'slide-in-up'
            }).then(function(modal) {
                $scope.modal = modal;
            });
            $scope.openModal = function() {
                $scope.modal.show();
            };
            $scope.closeModal = function() {
                $scope.modal.hide();
                $scope.printersSearchResult = [];
                $scope.pluginPrinters = [];
                $scope.wsPrinters = [];
            };
            $scope.$on('$destroy', function() {
                $scope.modal.remove();
            });


            $scope.confirmar = function() {    
                $scope.requestauth = {};
                var permission = UserService.checkPermission(CASH_FLOAT_ENTRY);
                LogService.info(permission);
                if (!permission) {
                    var authPopup = $ionicPopup.show({
                        templateUrl: 'views/requestauth.html',
                        scope: $scope,
                        buttons: [
                            { text: 'Cancelar' },
                            {
                                text: '<b>Autorizar</b>',
                                type: 'button-theme',
                                onTap: function(event) {
                                    var sucesso = function(result) {
                                        LogService.info("Autorização do usuário: ", result);
                                        authPopup.close();
                                        if (result.authorized == true) {
                                            $scope.userAuthorized = true;
                                            $scope.userAuthorizedName = result.user.name;
                                            $scope.userAuthorizedCode = result.user.id;
                                            LogService.info('Usuário autorizado após inserir senha e login')
                                            $scope.confirmarPopup();
                                        } else {
                                            LogService.warn('Usuário negado')
                                            AlertService.error('Autorização negada!')
                                        }
                                    };

                                    var erro = function(error) {
                                        LogService.error(error);
                                        AlertService.restError(error, "Erro ao consultar autorização!");
                                    };

                                    if (!$scope.requestauth.user) {
                                        AlertService.error("Usuário inválido!");
                                    } else if (!$scope.requestauth.password) {
                                        AlertService.error("Senha inválida!");
                                    } else {
                                        UserService.checkAuthorization($scope.requestauth.user, $scope.requestauth.password, [CASH_FLOAT_ENTRY], sucesso, erro);
                                    }
                                    event.preventDefault();
                                }
                            }
                        ]
                    });
                } else {
                    $scope.requestauth.user = UserService.currentUser.username;
                    LogService.info('Usuário autorizado por default. ', $scope.requestauth)
                    $scope.confirmarPopup();
                }
            }


            $scope.confirmarPopup = function() {    
                var confirmPopup = AlertService.confirm('Confirmar o valor do suprimento?<br><b>Suprimento não confirmado não será efetuado .</b>');
                confirmPopup.then((result) => {
                    if (result) {
                        callCashInFlow();
                    } else {
                        // fazer outro algo
                    }
                });
            };
            // operatorCode: UserService.operatorCode(),

            var callCashInFlow = function () {

                var params = {
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    terminalNumber: ConfigService.config.terminalNumber, 
                    operatorCode: UserService.operatorCode(),
                    operatorName: UserService.operatorName(),
                    authorizerCode: $scope.userAuthorized ? $scope.userAuthorizedCode : UserService.operatorCode(), 
                    authorizerName: $scope.userAuthorized ? $scope.userAuthorizedName : UserService.operatorName(),
                    nsu: ConfigService.nsuControll.currentNSU(),
                    amount: $scope.supply.supplyValue,
                    date: new Date(),
                    appInfo: {
                        name: $scope.APP_NAME,
                        version: $scope.appVersion
                    }
                };
                $ionicLoading.show({
                    template: 'Por favor, aguarde...',
                    duration: 0
                }).then(function () {
                    UserService.getCashInFlow(params,
                        function (result) {
                            try {
                                $scope.imprimir = true;
                                LogService.info(result);
                                $scope.loadSupplyCouponPDF(result.pdfDocumentBase64PrintData, result.printData);
                                initializeDocumentSlides();
                                $ionicLoading.hide();
                            }
                            catch (err) {
                                $ionicLoading.hide();
                                AlertService.restError(err, 'Não foi possível obter saldo do operador.');
                            }
                            finally {
                                CartService.reloadNSU(); //  Atualizando a NSU do carrinho.
                            }
                        },
                        function (err) {
                            $ionicLoading.hide();
                            AlertService.restError(err, 'Não foi possível obter saldo do operador.');
                        });
                });
            }


            $scope.loadSupplyCouponPDF = function(pdfDocumentBase64PrintData, printData) {
                const printContents = printData.map(data => {
                    return new PrintContent(data.documentTitle, data.paperType, data.base64PrintData);
                });
                const pdf = new PaymentResultPDF(pdfDocumentBase64PrintData, printContents);
                documents.push(pdf);    
            };
            function initializeDocumentSlides() {
                showAnyPDF();
                $scope.documents = documents;
            }
            function showAnyPDF() {
                const doc = documents.find(doc => doc instanceof PDF);
                $scope.printDocument = doc;
                if (doc) {
                    $scope.pdfUrl = doc.toURL();
                }
            }
            class PaymentResultPDF extends PDF {
                constructor(base64, printContents) {
                    super(base64);
                    this.printContents = printContents;
                    this.printed = false;
                }
            }         
            class PrintContent {
                constructor(title, paperType, data) {
                    this.title = title;
                    this.paperType = paperType;
                    this.data = data;
                }
            }


            $scope.onPrintDocument = function() {
                if (ConfigService.config.business.printing_enabled){
                    var document = $scope.printDocument;
                    if (document instanceof PaymentResultPDF) {
                        if (document.printed) {
                            AlertService.info('Documento já impresso!');
                        } else {
                            $scope.loadPrinters(document);
                        }
                    }
                } 
            };  


        /**
         * Função para busca de impressoras com status de 'ONLINE'.
         */
        $scope.loadPrinters = function(doc) {
            $ionicLoading.show({
                template : 'Consultando impressoras...',
                duration : 0
            }).then(function() {
                var error = function(error) {
                    $scope.loadingState = false;
                    $ionicLoading.hide();

                    AlertService.restError(error, 'Não foi possível obter impressoras disponíveis!');
                }

                var searchParams = {
                    connection_status: 'ONLINE'
                }

                PrinterService.search(searchParams)
                    .then(printers => {
                        $scope.printersSearchResult =  [];
                        $scope.pluginPrinters = [];
                        $scope.wsPrinters = [];
                        $ionicLoading.hide();
                        $scope.loadingState = false;

                        const printingContents = doc.printContents;
                        const supportedPrinters = printers.filter(printer => printerFilter(printer, printingContents));

                        const pluginPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_PLUGIN);
                        const wsPrinters = printers.filter(printer => printer.origin == PrinterService.ORIGIN_WS);

                        if (supportedPrinters.length == 1) {
                            $scope.printingDoc = doc;
                            $scope.printersSearchResult = supportedPrinters;
                            $scope.pluginPrinters = pluginPrinters;
                            $scope.wsPrinters = wsPrinters;

                            $scope.print(supportedPrinters[0], doc);

                        } else if (supportedPrinters.length > 0) {
                            $scope.printingDoc = doc;
                            $scope.printersSearchResult = supportedPrinters;
                            $scope.pluginPrinters = pluginPrinters;
                            $scope.wsPrinters = wsPrinters;
                            $scope.modal.show(); // lists the printers
                        } else {
                            error({
                                data : {
                                    message : "Nenhuma impressora disponível para o documento escolhido!"
                                }
                            });
                        }
                })
                .catch(error);
            });
        };

        function printerFilter(printer, printContents) {
            const length = printContents.length;

            for (let i = 0; i < length; ++i) {
                if (printer.paperType === printContents[i].paperType) {
                    return true;
                }
            }
            return false;
        }

        $scope.print = function(printer, printingDoc) {
            $ionicLoading.show({
                template : 'Imprimindo...',
                duration : 0
            }).then(function() {
                const paperType = printer.paperType;
                const printContents = printingDoc.printContents;
                const printContent = printContents.find(printContent => {
                    return paperType === printContent.paperType;
                });

                var params = {
                    'printerid':    printer.id,
                    'contentType':  printer.paperType === 'A_SERIES' ? 'application/pdf' : 'application/vnd.linx.printer-text',
                    'content':      printContent.data,
                    'title':        printContent.title,
                    'terminal':     ConfigService.config.terminalNumber,
                    'origin':       printer.origin,
                    'paperType':    printer.paperType
                };
                $scope.doPrint(params, printer, printingDoc);
            });
        };

        $scope.doPrint = function(params, printer, printingDoc) {
            var error = function(error) {
                $scope.loadingState = false;
                $ionicLoading.hide();

                AlertService.restError(error, 'Erro no envio da impressão');
            }

            PrinterService.submit(params)
                .then(() => {
                    $ionicLoading.hide();
                    $scope.loadingState = false;
                    printingDoc.printed = true;

                    const message = "Document impresso na impressora '" + printer.name + "'";
                    const okText = "Sair";
                    const alertPopup = AlertService.info(message, okText);

                    alertPopup.then(function(res) {
                        $scope.closeModal();
                        UtilService.navigateToMainScreen();
                    });
                })
                .catch(error);
        };
    })
})();