(function() {
    'use strict';

    angular.module('easyorder.controllers.tefpayment', [
        'easyorder.services.cart',
        'easyorder.services.eft',
        'easyorder.services.nfce',
        'easyorder.services.payment',
        'sprintf',
        'easyorder.services.log'
    ])

    .controller('TefPaymentCtrl', function($scope,
                                           $ionicLoading,
                                           $q,
                                           CartService,
                                           UtilService,
                                           EFTService,
                                           ConfigService,
                                           PaymentService,
                                           InsuranceService,
                                           UserService,
                                           AlertService,
                                           InfoService,
                                           LogService) {

        const SMARTPOS_DEVICES = [
            "GPOS700",
            "GPOS700X",
            "P2-B",
            "APOS A8OVS"
        ];

        /* 
         * caso o callback $scope.paymentdone for chamado não podemos permitir voltar
         */
        let done = false;
        let goingBack = false;

        $scope.goBack = () => {
            if ($scope.isBackButtonDisabled()) {
                return;
            }
            goingBack = true;
            
            $scope.message = "Cancelando operação...";
            
            $q((resolve, reject) => {
                EFTService.finalizeTransaction().then(resolve, reject);
            })
            .then(rewind)
            .catch(() => {
                $scope.message = "Não foi possível cancelar operação";
                goingBack = false
            });
        };
        
        function rewind() {
            return $q.resolve()
                .then(CartService.undoCashbackDiscountPartial)
                .then(CartService.undoFirtsPurchaseDiscount)
                .then(undoPaymentCampaigns)
                .then(() => UtilService.navigateWithoutHistory('easyorder.paymentmethods'));
        }
        
        async function undoPaymentCampaigns() {
            const campaigns = PaymentService.currentPayment.campaigns;

            if (campaigns && campaigns.length) {
                campaigns.forEach(campaign => {
                    LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                    CartService.undoCampaignDiscountsPartial(campaign);
                });
            }
        };

        $scope.isBackButtonDisabled = () => {
            return done || goingBack;
        };

        $scope.isBackButtonVisible = () => {
            return SMARTPOS_DEVICES.includes(ionic.Platform.device().model)
                && $scope.isTransactionInitiated() 
                && $scope.options.length == 0;
        };

        if ($scope.isModoPedido) {
            $scope.NEW_ORDER = "Novo Pedido";
        } else {
            $scope.NEW_ORDER = "Nova Venda";
        }

        $scope.isTransactionInitiated = function() {
            return $scope.transactionInitiated;
        };

        /*
         * Funções referentes ao tratamento de retorno do plugin de TEF (callback)
         */
        $scope.clearDisplay = function() {
            $scope.$apply(function () {
                $scope.message = "";
            });
            LogService.info("clearDisplay");
        };

        /**
         *
         */
        $scope.displayMessage = function(message) {
            /*
             * plugin de tef não atualiza a mensagem caso seja requisitado
             * o cancelamento da operação, por isto bloqueamos outras mensagens
             */
            if (goingBack) {
                return;
            }
            $scope.$apply(function () {
                $scope.message = message ? message : $scope.message;
            });
            LogService.info("displayMessage: " + $scope.message);
        };

        /**
         *
         */
        $scope.selectOption = function(title, options, responseHandler) {
            $scope.$apply(function () {
                $scope.message = title;
            });
            $scope.processOptions(options, responseHandler);
            LogService.info("Opção tef selecionada: ", title,  " Oções: ",  options);
        };

        /**
         *
         */
        $scope.selectOptionExtended = function(title, options, responseHandlerExtended) {
            $scope.$apply(function () {
                $scope.message = title;
            });
            $scope.processOptions(options, responseHandlerExtended);
            LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
        };

        /**
         *
         */
        $scope.numberInput = function(label, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               'number',
                maxSize:            fieldSizeRange.maxSize,
                valuePattern:       '[0-9]*'
            });
            $scope.headerOptions = "Informe";

            $scope.$apply();
        };

        /**
         *
         */
        $scope.stringInput = function(label, responseHandler, fieldSizeRange) {
            $scope.options.length = 0;
            $scope.showWatermark = false;

            $scope.options.push({
                id:                 1, //Dummy
                label:              label,
                handler:            responseHandler,
                type:               'string',
                maxSize:            fieldSizeRange.maxSize,
                valuePattern:       '[A-Za-z0-9_]*'
            });
            $scope.headerOptions = "Informe";

            $scope.$apply();
        };

        

        /**
         *
         */
        $scope.processOptions = function(options, responseHandler) {    //(1,"Magnetico/Chip")#(2,"Digitado")
            $scope.options.length = 0;
            $scope.showWatermark = false;

            var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
            splitedOpt = splitedOpt.split("#"); // Separando as opções.

            for (var i = 0; i < splitedOpt.length; i++) {
                var split = splitedOpt[i].split(",");

                $scope.options.push({
                    id:         split[0].trim(),
                    label:      split[1].trim().replace(/\"/g, ""),
                    value:      split[split.length-1],
                    handler:    responseHandler,
                    type:        "single-option"
                });
                $scope.$apply();
            }
            $scope.headerOptions = "Opções";
        };

        /**
         * Função executa a função com os parâmetros passados a esta.
         */
        $scope.tickOptionHandler = function(func, params) {
            $scope.options.length = 0;
            $scope.showWatermark = true;

            func.apply(this, params);
        };

        /**
         *
         */
        $scope.tickNumberInputHandler = function(inputOption) {
            var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);

            $scope.tickOptionHandler(inputOption.handler, [value]);
        };

        /**
         *
         */
        $scope.tickStringInputHandler = function(inputOption) {
            const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);

            $scope.tickOptionHandler(inputOption.handler, [value]);
        };

        /**
         * Função para tratamento após finalização do pagamento TEF Chama autorização de NFC-e
         */
        $scope.paymentDone = function(paymentResults) {
            done = true;

            $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            }).then(function() {
                var predefinedDetails = PaymentService.currentPayment().predefinedDetails;
                var paymentAuthorizerCodeChoice = predefinedDetails ? predefinedDetails.paymentAuthorizerCodeChoice : undefined;
                
                InfoService.checkCTFAuthorizerCodeSimilarity(paymentAuthorizerCodeChoice, paymentResults.CodigoRede).then(function() {
                    let salePayment = $scope.processTefPayment(paymentResults, CartService.cart, PaymentService.currentPayment());

                    PaymentService.isCardBinEnabled(salePayment.paymentCodeMethod, salePayment.cardNumber).then(() => {
                        PaymentService.tefOnlineAuthConfirmationStatus(salePayment).then(() => {
                            PaymentService.currentPaymentDone();
                            UtilService.navigateAfterReceivedPayment();
                        }, error => {
                            $ionicLoading.hide();
                            PaymentService.currentPaymentDone(); // Faz com que o pagamento fiquei disponível para ser desfeito.
                            $scope.paymentErrorUndoPayments(error); // Se o pagamento não é válido, deve ser desfeito.
                        });
                    }, errorMessage => {
                        $ionicLoading.hide();
                        PaymentService.currentPaymentDone(); // Faz com que o pagamento fiquei disponível para ser desfeito.
                        $scope.paymentErrorUndoPaymentsMsg(errorMessage); // Se o pagamento não é válido, deve ser desfeito.
                    });
                 
                }, error => {
                    $ionicLoading.hide();
                    PaymentService.currentPaymentDone(); // Faz com que o pagamento fiquei disponível para ser desfeito.
                    $scope.paymentErrorUndoPayments(error); // Se o pagamento não é válido, deve ser desfeito.
                });
            });
        };


        /**
         * Processa o pagamento atual e o resultado da transação TEF no objeto enviado para integração e geração da NFC-e.
         */
        $scope.processTefPayment = function(tefPaymentResults, cart, currentPayment) {
            var salePayment = {};

            salePayment.type                        = 'SaleTEFPayment';
            salePayment.paymentSequence             = PaymentService.payments().length;
            salePayment.mideReferenceID             = currentPayment.mideReferenceID;
            salePayment.sefazNSU                    = currentPayment.sefazNSU;
            salePayment.fiscalIntegratorAuthModeOff = currentPayment.fiscalIntegratorAuthModeOff;

            if (currentPayment.paymentMethod === 'CREDIT' ||
                currentPayment.paymentMethod === 'CREDIT_WITH_INSTALMENTS') {
                salePayment.paymentCodeMethod = 'CREDIT';
            } else {
                salePayment.paymentCodeMethod = currentPayment.paymentMethod;
            }

            salePayment.paymentValue = currentPayment.paymentValue;
            salePayment.installments = [];

            if (currentPayment.plan) {
                salePayment.paymentCodePlan         = currentPayment.plan.code;
                salePayment.paymentCodeEvent        = currentPayment.plan.paymentCodeEvent;
                salePayment.paymentPlanExternalCode = currentPayment.plan.externalCode;
                salePayment.paymentPlanProvider     = currentPayment.plan.provider;
                salePayment.numberOfInstallments    = currentPayment.plan.instalments ? currentPayment.plan.instalments.length : 0;
                salePayment.dateTofirstInstalment   = currentPayment.plan.firstInstalment ? currentPayment.plan.firstInstalment.dueDate : null;

                currentPayment.plan.instalments.forEach(function(instalment, index, arr) { // Convertendo Instalment em InstallmentsDetails
                    var instalmentDetails = {};

                    instalmentDetails.paymentSequence           = instalment.number;
                    instalmentDetails.installmentsNumber        = arr.length;
                    instalmentDetails.instalmentDateExpiration  = instalment.dueDate;
                    instalmentDetails.installmentValue          = instalment.instalmentValue;

                    this.push(instalmentDetails);
                }, salePayment.installments)

                if (currentPayment.plan.discount) {
                    var discount = {}

                    discount.sequenceId     = cart.discounts.length;
                    discount.value          = currentPayment.plan.discount;
                    discount.source         = 'PLAN';

                    /* O desconto deve ser contabilizado no valor do pagamento enviado para o WS. */
                    salePayment.paymentValue -= currentPayment.plan.discount; 
                    // atualiza valor do pagamento
                    currentPayment.paymentValue = parseFloat(new Number(salePayment.paymentValue).toFixed(2));
                    cart.discounts.push(discount);
                }

                if (currentPayment.plan.charges) {
                    cart.additionalCharge += currentPayment.plan.charges;
                    salePayment.chargeValue = currentPayment.plan.charges;

                    if (ConfigService.config.business.apply_charges_on_payment_value) {
                        salePayment.paymentValue += currentPayment.plan.charges;
                    }
                }

                salePayment.iofValue = currentPayment.plan.iof || 0;
                salePayment.rates = currentPayment.plan.rates;
            } else {
            	// se não for um plano de pagamento
            	salePayment.numberOfInstallments = tefPaymentResults.NumeroParcelas;
            }

            salePayment.paymentAuthorizerBrandType  = tefPaymentResults.IndiceCodigoBandeira;
            salePayment.paymentFiscalBrandCode = tefPaymentResults.CodigoBandeiraSefaz;

            if (tefPaymentResults.NomeBandeiraCartao) {
                salePayment.paymentAuthorizerBrandName  = tefPaymentResults.NomeBandeiraCartao.trim();
            }

            if (ConfigService.device.lio) {
                salePayment.paymentAuthorizerCode = 0;
            } else {
                const tefProvider = ConfigService.config && 
                                    ConfigService.config.tefserver ? ConfigService.config.tefserver.provider : undefined;
                switch(tefProvider) {
                    case 'Auttar':
                        // No caso do CTF, o código do autorizador é obtido no campo 'CodigoAutorizadora'.
                        salePayment.paymentAuthorizerCode = tefPaymentResults.CodigoAutorizadora;
                        salePayment.vanCode               = tefPaymentResults.CodigoVAN;
                        break;
                    case 'Linx':
                        // !!!!! ATENCAO !!!!!
                        // Este campo não está preenchido com o valor correto, deveria ser CodigoRede.
                        // Alteração temporária devido a urgência do cliente TIM.
                        // Campo correto para receber este valor deveria ser productCode, uma vez que no DTEF
                        // operadora é o nome do produto (join de Bandeira+Tipo de Produto)
                        salePayment.paymentAuthorizerCode = tefPaymentResults.CodigoOperadora;
                        // !!!!! ATENCAO !!!!!
                        // Este campo não está preenchido com o valor correto, deveria ser NomeBandeira.
                        // Alteração temporária devido a urgência do cliente TIM.
                        // Campo correto para receber este valor deveria ser productName, uma vez que no DTEF
                        // operadora é o nome do produto (join de Bandeira+Tipo de Produto)
                        if (tefPaymentResults.NomeOperadora) {
                            salePayment.paymentAuthorizerBrandName = tefPaymentResults.NomeOperadora.trim();
                        }

                        if (tefPaymentResults.CodigoRede) {
                            salePayment.vanCode           = '' + parseInt(tefPaymentResults.CodigoRede, 10);
                        }

                        break;
                    case 'SoftwareExpress':
                        salePayment.paymentAuthorizerCode = tefPaymentResults.IndiceCodigoRede;

                        if (tefPaymentResults.IndiceCodigoRede) {
                            salePayment.vanCode           = '' + parseInt(tefPaymentResults.IndiceCodigoRede, 10);
                        }

                        break;
                    default:
                        throw new Error('TEF provider not implemented!');
                }
            }
            
            salePayment.paymentAuthorizationCode    = tefPaymentResults.CodigoAutorizacao;
            salePayment.paymentMethodIndicator      = tefPaymentResults.TipoFinanciamento;
            salePayment.paymentAuthorizationNSU     = tefPaymentResults.NSU;
            salePayment.paymentAuthorizerNSU        = tefPaymentResults.NSURede;
            salePayment.cardNumber                  = tefPaymentResults.NumeroCartao;
            salePayment.customerPaymentVoucher      = tefPaymentResults.ComprovanteCliente;
            salePayment.bussinessPaymentVoucher     = tefPaymentResults.ComprovanteEstabelecimento;
            salePayment.acquirerName                = tefPaymentResults.NomeAdquirente;
            salePayment.lastCardDigits              = tefPaymentResults.UltimosDigitosCartao;
            salePayment.cardExpirationDate          = tefPaymentResults.DataValidadeCartao;
            salePayment.paymentAuthorizerCNPJ       = tefPaymentResults.CNPJRede;
            
            cart.salePayments.push(salePayment);

            return salePayment;
        };

       /**
        * Função chamada por callback do TEF.
        */
        $scope.paymentError = errorMsg => {
            switch (errorMsg) {
                case window.plugins.lio.PMNT_CANCELLED:
                    AlertService.error('Transação cancelada')
                        .then(rewind);
                    break;
                default:
                    AlertService.error(errorMsg)
                        .then(rewind);
                    break;
            }
        };
       
        /**
         * Tratador de erro que desfaz pagamentos.
         */
        $scope.paymentErrorUndoPayments = function(error) {
            PaymentService.cancel(true).then(() => {
                AlertService.restError(error, "Não foi possível processar pagamento TEF").then(() => {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                });
            });
        };


        $scope.paymentErrorUndoPaymentsMsg = function(errorMessage) {
            PaymentService.cancel(true).then(() => {
                AlertService.error(errorMessage).then(() => {
                    UtilService.navigateWithoutHistory('easyorder.cart');
                });
            });
        };

       /**
        * Processa o valor a ser pago pelo cliente.
        */
       $scope.processPaymentValue = function() {
           var currentPayment = PaymentService.currentPayment();
           var ret = new Number(currentPayment.paymentValue);
           var paymentPlan = currentPayment.plan;

           LogService.info("Processando valor do Pagamento: [" + ret + "]");

           if (paymentPlan) { // Tem plano de pagamento?
                // Os cartões privateLabel (RENNER) não utilizam o totalValue
                // do plano, pois os juros são calculados por fora, logo
                // damos apenas o desconto, porque o mesmo vem antes dos juros.
                if (currentPayment.privateLabel && paymentPlan.discount) {
                    ret -= new Number(paymentPlan.discount);
                }
                if (!currentPayment.privateLabel) {
                    ret = MathUtils.subtract(
                        paymentPlan.totalValue,
                        paymentPlan.downPaymentValue
                    );
                }
           }
           LogService.info("Valor do pagamento: [" + ret + "]");

           return ret;
       };

       /**
        * Inicia processamento de pagamento por voucher.
        */
       $scope.clientVirtualCard = function() {
           $scope.displayMessage("Consultando cartão virtual...");
           
           return PaymentService.getVirtualCard().then(function(response) {
               $scope.clearDisplay();
               
               var cardNumber = response.cardNumber;
                       
               if (!cardNumber) {
                   AlertService.error('Cartão virtual não retornado!').then(function() {
                       UtilService.navigateWithoutHistory('easyorder.cart');
                   });
               }
               return cardNumber;
           }, function(response) {
               $scope.clearDisplay();
               
               AlertService.restError(response, 'Erro ao consultar cartão virtual.').then(function() {
                   UtilService.navigateWithoutHistory('easyorder.cart');
               });
           });
       };

        /**
         * Função de inicialização.
         */
        (function(scopeObject) {
            $scope.transactionInitiated = false;
            $scope.showWatermark = true;
            $scope.message = "";
            $scope.options = [];
            $scope.transactionType = EFTService.transactionType;
            $scope.nfceAuthResponse = {};

            $scope.transactionInitiated = true;
            const currentPayment = PaymentService.currentPayment(); 
            var plan = currentPayment.plan;
            var params = {
                transactionType: (function(paymentMethod) {
                    var ret = undefined;
                    
                    switch (paymentMethod) {
                        case 'CREDIT':
                            ret = EFTService.transactionType.CREDIT;
                            break;
                        case 'CREDIT_WITH_INSTALMENTS':
                            ret = EFTService.transactionType.CREDIT_WITH_INSTALMENTS;
                            break;
                        case 'DEBIT':
                            ret = EFTService.transactionType.DEBIT;
                            break;
                        case 'CONDUCTOR_CREDIT_BOOK':
                            ret = EFTService.transactionType.CONDUCTOR_CREDIT_BOOK;
                            break;
                        case 'CONDUCTOR_CARD_STATEMENT':
                            ret = EFTService.transactionType.CONDUCTOR_CARD_STATEMENT;
                            break;
                        case 'VIRTUAL_CARD':
                            ret = EFTService.transactionType.CREDIT;
                            break;
                        case 'CEA_PAY':
                            // é usado um tipo de transação genérico pois é da mesma forma para outros clientes que usem conductor.
                            ret = EFTService.transactionType.CREDIT;
                            break;
                    }
                    return ret;
                })(currentPayment.paymentMethod),
                operatorCode:       UserService.operatorCode(),
                instalmentsNumber:  plan ? (plan.instalments ? plan.instalments.length : 1) : 1,
                amount:             $scope.processPaymentValue(),
                hasInsurance:       InsuranceService.isPlanInsured(plan),
                charges:            plan ? plan.charges : undefined,
                processorCode:      plan ? plan.processorCode : undefined,
                interestType:       plan ? plan.interestType : undefined,
                typedCardNumber:    currentPayment.cardNumber,
                cardExpirationDate: currentPayment.cardExpirationDate,
                cardProcessor:      currentPayment.paymentMethod === 'CEA_PAY' ? 'CONDUCTOR' : undefined,
                paymentPlanCode:    plan ? plan.externalCode : undefined
            };
            
            if (currentPayment.paymentMethod === 'VIRTUAL_CARD') {
                $scope.clientVirtualCard().then(function(virtualCard) {
                    params.cardNumber = virtualCard;
                    EFTService.executeTransaction(params, scopeObject);
                });
            } else {
                EFTService.executeTransaction(params, scopeObject);
            }
        })($scope);
    })
})();
