(function () {
    'use strict';
    angular.module('easyorder.controllers.transactionjustification', [])
        .controller('TransactionJustificationCtrl', function (
            $rootScope,
            $scope,
            ConfigService,
            AlertService,
            UtilService,
            $ionicLoading,
            TransactionReturnService,
            $stateParams,
            CartService,
            UserService,
            DonationService,
            $q
        ) {

            $scope.op = $stateParams.op;
            $scope.data = $stateParams.data;

            $scope.goBack = () => {
                switch ($scope.op) {
                    case 'RETURN':
                    case 'EXCHANGE':
                    case 'ANNULMENT':
                        UtilService.navigateWithoutHistory('easyorder.transactionproductlist', {
                            op: $scope.op,
                            authorizer: $stateParams.authorizer,
                            sourceTransaction: $scope.data.sourceTransaction
                        });
                        break;
                    case 'DONATION':
                        UtilService.navigateWithoutHistory('easyorder.donation', {
                            op: $scope.op,
                            authorizer: $stateParams.authorizer,
                        });
                        break;
                    default:
                        throw new Error('Operation not supported');
                }
            };

            $scope.goNext = () => {
                if (!$scope.reason) {
                    AlertService.warn("Você deve escolher ao menos um motivo.");
                    return;
                }

                $ionicLoading.show({
                    template: 'Aguarde...',
                    duration: 0
                }).then(() => {
                    switch ($scope.op) {
                        case 'RETURN':
                            return goToReturnScreen();
                        case 'EXCHANGE':
                            return goToExchangeScreen();
                        case 'DONATION':
                            return donate();
                        case 'ANNULMENT':
                            return goToAnnulmentScreen(); 
                        default:
                            throw new Error('OperationType not supported');
                    }
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
                        AlertService.restError(error, error);
                    });
                });
                
            };

            function goToReturnScreen() {
                return $ionicLoading.hide().then(() => {
                    return UtilService.navigateWithoutHistory('easyorder.voucher', {
                        op: $scope.op,
                        authorizer: $stateParams.authorizer,
                        sourceTransaction: $scope.data.sourceTransaction,
                        returnReason: $scope.reason,
                        returnData: $scope.data,
                    });
                });
            }

            function goToAnnulmentScreen() {
                return $ionicLoading.hide().then(() => {
                    return UtilService.navigateWithoutHistory('easyorder.annulment', {
                        op: $scope.op,
                        authorizer: $stateParams.authorizer,
                        sourceTransaction: $scope.data.sourceTransaction,
                        returnReason: $scope.reason,
                        returnData: $scope.data,
                    });
                });
            }


            function goToExchangeScreen() {
                return $ionicLoading.hide().then(() => {
                    return UtilService.navigateWithoutHistory('easyorder.exchangeuuidinputs', {
                        op: $scope.op,
                        authorizer: $stateParams.authorizer,
                        sourceTransaction: $scope.data.sourceTransaction,
                        returnReason: $scope.reason,
                        exchangeData: $scope.data,
                    });
                });
            }

            function donate() {
                const fiscalItems = $scope.data.fiscalItems;
                return DonationService.donate({
                    appInfo: {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    },
                    transactionKey: {
                        storeInfo: {
                            companyNumber: ConfigService.config.companyNumber,
                            storeNumber: ConfigService.config.storeNumber,
                        },
                        date: new Date(),
                        nsu: ConfigService.nsuControll.incrementNSU(),
                        terminal: ConfigService.config.terminalNumber,
                    },
                    reason: $scope.reason,
                    operator: {
                        code: UserService.operatorCode(),
                        name: UserService.operatorName()
                    },
                    authorizer: {
                        code: $stateParams.authorizer.id,
                        name: $stateParams.authorizer.name
                    },
                    customer: CartService.cart.customer,
                    fiscalItems: fiscalItems,
                    totalValue: fiscalItems.reduce((acc, item) => new Number((acc + item.totalValue).toFixed(2)), 0),
                }).then(result => {
                    return $ionicLoading.hide().then(() => {
                        return UtilService.navigateWithoutHistory('easyorder.couponview', {
                            pdf: result.data.pdf || " ",
                            printContents: result.data.printContents || [],
                            documentTitle: 'donation',
                            showPdf: result.data.pdf != null && result.data.pdf.length > 0
                        });
                    });
                });
            }

            $scope.selectReason = reason => {
                $scope.reason = reason;
            };

            function searchReasons() {
                $ionicLoading.show({
                    template: "Aguarde...",
                    duration: 0
                }).then(() => {
                    switch ($scope.op) {
                        case 'RETURN':
                        case 'EXCHANGE':
                        case 'ANNULMENT':
                            const params = {
                                companyNumber: ConfigService.config.companyNumber,
                                storeNumber: ConfigService.config.storeNumber,
                                operationType: $scope.op
                            };
                            return TransactionReturnService.searchReasons(params);
                        case 'DONATION':
                            return DonationService.searchReasons();
                        default:
                            throw new Error(`OperationType ${$scope.op} not supported`);
                    }
                }).then(result => {
                    $ionicLoading.hide();
                    if (UtilService.isNotEmptyArray(result.data.reasons)) {
                        $scope.options = result.data.reasons;
                    } else {
                        return $q.reject({ message: 'Nenhuma justificativa foi retornada!' } );
                    }
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
                        return AlertService.restError(error, "Erro ao buscar a lista de Justificativas!");
                    }).then(() => {
                        $scope.goBack();  
                    });
                });
            };
            searchReasons();
        })
})();
