(function () {
    'use strict';
    angular.module('easyorder.controllers.transactionsearch', [
        'ui.mask',
        'ui.utils.masks',
    ]).controller('TransactionSearchCtrl', function (
            $scope,
            $rootScope,
            ConfigService,
            AlertService,
            $ionicLoading,
            UtilService,
            TransactionReturnService,
            TransactionSearchService,
            $stateParams,
            $q) {

            $scope.op = $stateParams.op;
            $scope.formData = {};

            const isAnnulment = $scope.op === 'ANNULMENT';

            if (isAnnulment) {
                // se for anulação, já preenche a loja com a atual
                $scope.formData.storeNumber = ConfigService.config.storeNumber;
            }

            $scope.goBack = function () {
                UtilService.navigateWithoutHistory('easyorder.operations');
            };

            $scope.search = () => {
                
                $ionicLoading.show({
                    template: "Aguarde...",
                    duration: 0
                }).then(() => {
                    if (isEmpty($scope.formData.date) || isEmpty($scope.formData.storeNumber) ||
                        isEmpty($scope.formData.terminalNumber) || isEmpty($scope.formData.nsu)) {
                        return $q.reject('Todos os campos devem ser preenchidos.');
                    }

                    if ($scope.op === 'EXCHANGE') {
                        if (!$rootScope.exchangeOtherStore && ConfigService.config.storeNumber != $scope.formData.storeNumber) {
                            return $q.reject('Compra realizada em outra loja');
                        }
                        const trnsDate = new Date($scope.formData.date);
                        const maxDays = $rootScope.exchangeMaxDays;
                        if (Math.abs(dateDiffInDays(new Date(), trnsDate)) > maxDays) {
                            return $q.reject('Cupom fora do período de garantia');
                        }
                    }

                    if ($scope.op === 'RETURN') {
                        if (ConfigService.config.storeNumber != $scope.formData.storeNumber) {
                            return $q.reject('Cupom Fiscal é de outra loja, impossível prosseguir');
                        }
                        const trnsDate = new Date($scope.formData.date);
                        const maxDays = $rootScope.returnMaxDays;
                        if (Math.abs(dateDiffInDays(new Date(), trnsDate)) > maxDays) {
                            return $q.reject('Cupom fora do período de devolução, impossível prosseguir');
                        }
                    }

                    if (isAnnulment) {
                        if (ConfigService.config.storeNumber != $scope.formData.storeNumber) {
                            return $q.reject('A anulação da venda não é permitida em cupons fiscais de outra loja!');
                        }
                        const trnsDate = new Date($scope.formData.date);
                        // anulação só permite do mesmo dia
                        if (Math.abs(dateDiffInDays(new Date(), trnsDate)) > 0) {
                            return $q.reject('A anulação da venda só é permitida no dia atual!');
                        }

                    }

                    $rootScope.transactionDate = $scope.formData.date;
                    const service = isAnnulment ? TransactionSearchService : TransactionReturnService;
                    return service.search({
                        transactionKey: {
                            storeInfo: {
                                companyNumber: ConfigService.config.companyNumber,
                                storeNumber: $scope.formData.storeNumber,
                            },
                            terminal: $scope.formData.terminalNumber,
                            date: $scope.formData.date,
                            nsu: $scope.formData.nsu,
                        },
                        provider: "STOREX",
                    });
                }).then(result => {
                    $ionicLoading.hide().then(() => {
                        if (!result.transaction.availableFiscalItems || !result.transaction.availableFiscalItems.length) {
                            AlertService.error('Não existem produtos para o cupom fiscal.');
                        } else if (!isAnnulment && !result.transaction.availableFiscalItems.some(item => item.uuid)) {
                            AlertService.error('Produtos disponíveis nesse cupom não possuem Identificador. Não será possível prosseguir.');
                        } else if (isAnnulment && UtilService.isNotEmptyArray(result.transaction.returnedFiscalItems)) {
                            AlertService.error('Já existem produtos retornados nesse cupom, não será possível anular a venda!');
                        } else if (isAnnulment && hasForbiddenAnnulmentPayments(result.transaction.payments)) {
                            AlertService.error('Existem formas de pagamento não permitidas para anulação. Formas permitidas: Crédito, Débito, QR Linx e Link de Pagamentos!');
                        } else {
                            UtilService.navigateWithoutHistory("easyorder.transactionproductlist", {
                                op: $scope.op,
                                sourceTransaction: result.transaction,
                                authorizer: $stateParams.authorizer
                            });
                        }
                    });
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
    					AlertService.restError(error, error);
                    });
                });
            };

            function hasForbiddenAnnulmentPayments(payments) {
                return !UtilService.isNotEmptyArray(payments) || payments.some(
                    pmnt => !eftFilter(pmnt)
                        && !qrLinxFilter(pmnt)
                        && !paymentLinkFilter(pmnt)
                );
            }

            function eftFilter(payment) {
                const eftPaymentMethods = ['CREDIT', 'DEBIT'];
                return eftPaymentMethods.includes(payment.paymentMethod)
                    && !isLinkProvider(payment.provider)
            }

            function qrLinxFilter(payment) {
                return payment.paymentMethod == 'QR_LINX';
            }

            function paymentLinkFilter(payment) {
                return payment.paymentMethod == 'PAYMENT_LINK'
                    || isLinkProvider(payment.provider);
            }

            function isLinkProvider(provider) {
                if (provider == null) {
                    return false;
                }

                const linkProviders = [
                    'PAGAR_ME',
                    'LINX_PAYHUB_LINK',
                    'E_SITEF_WEB_CHECKOUT'
                ];

                return linkProviders.includes(provider.id);
            }

            $scope.formatDate = value => {
                if (!value) {
                    return;
                }
                const match = value.match(/^(\d{1,2})(\d{1,2})(\d{4})$/);
                $scope.formData.date = new Date(match[3], match[2] - 1, match[1]).toISOString();
            };

            const _MS_PER_DAY = 1000 * 60 * 60 * 24;
            function dateDiffInDays(a, b) {
                // Discard the time and time-zone information.
                const utc1 = Date.UTC(a.getFullYear(), a.getMonth(), a.getDate());
                const utc2 = Date.UTC(b.getFullYear(), b.getMonth(), b.getDate());

                return Math.floor((utc2 - utc1) / _MS_PER_DAY);
            }

            function isEmpty(str) {
                return str == undefined || str == '';
            }

        });
})();
