(function () {
    'use strict';
    angular.module('easyorder.controllers.voucherbarcodepayment', ["angular-uuid"])
        .controller('VoucherBarcodePaymentCtrl', function (
            $scope,
            AlertService,
            LogService,
            $ionicPopup,
            BarcodeService,
            UtilService,
            ConfigService,
            VoucherService,
            PaymentService,
            CartService,
            $q,
            $stateParams,
            PaymentProcessor,
            uuid
        ) {
            $scope.scanning_method = ConfigService.config.business.voucher_barcode_scanning_method;
            $scope.paymentValue = $stateParams.paymentValue;
            $scope.voucherInput = {};
            const voucherType = $stateParams.voucherType;
            $scope.voucherLabel = (voucherType === 'VOUCHER_OMNI' ? 'Vale OMNI' : 'Voucher');

            const auths = new Map();

            initBarcodeScanner();

            function checkVoucherUsed(voucherNumber) {
                return VoucherService.checkVoucherUsed(voucherType, voucherNumber, CartService.cart.salePayments);
            }

            function searchVoucher(voucherNumber) {
                return VoucherService.search(voucherNumber, ConfigService.config.storeNumber, voucherType, ConfigService.config.companyNumber, ConfigService.config.terminalNumber);
            }

            function searchVouchersByDocument(document, password) {
                const type = (voucherType === 'VOUCHER_OMNI') ? ConfigService.config.business.giftcard_service_provider : voucherType;
                if (voucherType === 'VOUCHER_OMNI') {
                    return VoucherService.search(document, ConfigService.config.storeNumber, type,
                        ConfigService.config.companyNumber, ConfigService.config.terminalNumber, 
                        ConfigService.nsuControll.currentNSU(), password);
                } else {
                    return VoucherService.searchByDocument(ConfigService.config.companyNumber, ConfigService.config.storeNumber,
                                                           ConfigService.config.terminalNumber, type, document, 'CPF');
                }
            }


            function authorizeVoucher(response) {
                const voucher = response.vouchers[0];

                if (auths.has(voucher.number)) {
                    return $q.reject(`Aguarde o processamento do Voucher '${voucher.number}' antes de tentar utilizá-lo`);
                }

                showLoadingMessage();

                const currentPaymentValue = $scope.paymentValue > voucher.value ? voucher.value : $scope.paymentValue;
                LogService.info(`Saldo do cartão é ${currencyFormat(voucher.value)} e o pagamento solicitado ${currencyFormat($scope.paymentValue)}. Será utilizado ${currencyFormat(currentPaymentValue)}.`);

                const contextUid = uuid.v4();
                LogService.info(`Contexto do Voucher '${contextUid}'`);
                auths.set(voucher.number, contextUid);

                PaymentProcessor.pushSalePayment({
                    type: 'SaleVoucherGivexPayment',
                    paymentCodeMethod: 'VOUCHER_GIVEX',
                    voucherNumber: voucher.number,
                    paymentValue:  currentPaymentValue,
                    companyNumber: ConfigService.config.companyNumber,
                    terminalNumber: ConfigService.config.terminalNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    voucherOwner: $scope.customer,
                    contextUid: contextUid,
                });

                return VoucherService.authorize(
                    ConfigService.nsuControll.currentNSU(),
                    voucher.number,
                    ConfigService.config.storeNumber,
                    voucherType, currentPaymentValue,
                    ConfigService.config.companyNumber,
                    ConfigService.config.terminalNumber,
                    contextUid,
                    null,
                    null,
                    null,
                    null,
                    null,
                    null,
                    null
                ).catch(err => {
                    // não sabemos se foi uma queda de rede
                    // de toda forma já marcamos pra desfazer o pagamento
                    PaymentProcessor.undoPayment((payment) =>
                        payment.data.undo.contextUid == contextUid
                    );
                    auths.delete(voucher.number);
                    return $q.reject(err);
                });
            }

            const currencyFormat = function (value) {
                let formatter = new Intl.NumberFormat('pt-BR', {
                    style: 'currency',
                    currency: 'BRL',
                    minimumFractionDigits: 2
                });
                return formatter.format(value);
            };

            function onVouchersFound(res) {
                const payments = CartService.cart.salePayments ? CartService.cart.salePayments : [];
                const vouchers = res.vouchers ? res.vouchers: [];
                const availableVouchers = vouchers.filter(voucher => voucher.status == 'AVAILABLE');

                payments.filter(pmnt => pmnt.paymentCodeMethod == voucherType).forEach(pmnt => {
                    const usedVoucher = availableVouchers.find(voucher => VoucherService.isSameVoucher(voucher.cardISO, voucher.serialNumber, pmnt.voucherNumber));
                    if (usedVoucher) {
                        var index = availableVouchers.indexOf(usedVoucher);
                        if (index > -1) {
                            availableVouchers.splice(index, 1);
                        }
                    }
                });

                if (availableVouchers && availableVouchers.length > 0) {
                    var params = {
                        vouchers: availableVouchers,
                        containsUsedVouchers: false
                    };
                    return $q.resolve(params);
                } else {
                    return $q.reject(new VoucherNotFoundError('Sem cupons disponíveis!'));
                }
            }

            function onAvailableVouchers(params, currentDocument) {
                hideLoadingMessage();
                var customer = null;
                var currentCredentials = null;

                if ($scope.customer && $scope.customer.documentType === 'CPF' &&  $scope.customer.document === currentDocument) {
                    customer = $scope.customer;
                } else {
                    customer = {
                        documentType: 'CPF',
                        document: currentDocument
                    };
                }

                if ($stateParams.isPasswordRequired && $scope.voucherInput.password) {
                    currentCredentials = $scope.voucherInput.password;
                }

                UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearch', {
                    customer: customer,
                    vouchers:  params.vouchers,
                    containsUsed:  params.containsUsedVouchers,
                    voucherType: voucherType,
                    paymentValue: $scope.paymentValue,
                    voucherCredentials: currentCredentials,
                });
            }

            function checkBalance(response) {
            	return $q((resolve, reject) => {
                    if (response && response.vouchers && response.vouchers.length > 0) {
                        const voucher = response.vouchers[0];
                        if (voucher.value <= 0) {
                            reject(new VoucherAlreadyUsedError('Voucher já utilizado!'));
                            return;
                        }

                        if (voucher.value < $scope.paymentValue) {
                            hideLoadingMessage();
                            const msg = `Saldo do cartão é ${currencyFormat(voucher.value)} inferior ao valor do pagamento. Deseja prosseguir com o saldo como parte do pagamento?`;
                            AlertService
                                .confirm(msg)
                                .then((res) => {
                                    if (!res) {
                                        reject(new VoucherOperationAbortedError());
                                        return;
                                    } else {
                                        showLoadingMessage();
                                        resolve(response);
                                        return;
                                    }

                                });
                            return;
                        } else {
                            if (voucherType === 'VOUCHER_OMNI') {
                                resolve(response);
                                return;
                            }
                            hideLoadingMessage();
                            $ionicPopup.confirm({
                                templateUrl: 'views/voucherbarcodeconfirmation.html',
                                title: 'Confirmação',
                                scope: $scope,
                                buttons: [
                                    {
                                        text: '<b>Confirmar</b>',
                                        type: 'button-theme',
                                        onTap: function(event) {
                                            showLoadingMessage();
                                            resolve(response);
                                        }
                                    },
                                    { text: 'Cancelar',
                                        onTap: function(event) {
                                            reject(new VoucherOperationAbortedError());
                                        }
                                    },
                                ]
                            });
                        }


                    } else {
                        reject(new VoucherNotFoundError('Voucher não encontrado!'));
                    }
            	});
            }

            function onAuthorizationError(error) {
                hideLoadingMessage();
                if (error instanceof VoucherAlreadyUsedError) {
                    return AlertService.error(error.message);
                } else if (error instanceof VoucherNotFoundError) {
                    return AlertService.error(error.message);
                } else if (error instanceof VoucherOperationAbortedError) {
                    // operação abortada
                    initBarcodeScanner();
                } else {
                    return AlertService.restError(error, typeof error == 'string' ? error : 'Falha ao autorizar voucher');
                }
            }

            function showLoadingMessage() {
                $scope.loadingState = true;
            }

            function hideLoadingMessage() {
                $scope.loadingState = false;
            }

            function paymentDone(authorizationResult){
                initializePayment(voucherType, authorizationResult.value, false);
                PaymentService.currentPaymentDone();

                const salePayment = {
                    type: 'SaleVoucherGivexPayment',
                    paymentCodeMethod: voucherType,
                    paymentSequence: PaymentService.payments().length,
                    paymentValue:  authorizationResult.value,
                    installments: [],
                    voucherNumber:  authorizationResult.voucherNumber,
                    cardNumber: authorizationResult.voucherNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    paymentAuthorizationCode: authorizationResult.authorizationCode,
                    expirationDate: authorizationResult.expirationDate,
                    currentBalance: authorizationResult.currentBalance
                };
                CartService.cart.salePayments.push(salePayment);

                const contextUid = auths.get(authorizationResult.voucherNumber);

                if (contextUid) {
                    PaymentProcessor.updateSalePayment(
                        (payment) => payment.data.confirm.contextUid == contextUid,
                        { ...salePayment, contextUid: contextUid },
                    );
                    auths.delete(authorizationResult.voucherNumber);
                }

                hideLoadingMessage();

                UtilService.navigateAfterReceivedPayment();
            }
            function initializePayment (paymentMethod, paymentValue, privateLabel) {
                var ret = paymentValue;

                CartService.processPaymentCampaigns(paymentMethod, paymentValue);
                CartService.updateCartTotalValue();

                if (CartService.cart.totalValue < paymentValue) {
                    ret = CartService.cart.totalValue;
                }
                PaymentService.initPayment(paymentMethod, ret, privateLabel);

                return ret;
            }

            function askForPassword() {
                return $q((resolve, reject) => {
                    $ionicPopup.confirm({
                        templateUrl: 'views/voucherbarcodepassword.html',
                        title: `Autenticar ${$scope.voucherLabel}`,
                        scope: $scope,
                        buttons: [
                            {
                                text: '<b>Confirmar</b>',
                                type: 'button-theme',
                                onTap: function(_evt) {
                                    showLoadingMessage();
                                    resolve($scope.voucherInput.password);
                                }
                            },
                            { text: 'Cancelar',
                                onTap: function(_evt) {
                                    reject(new VoucherOperationAbortedError());
                                }
                            },
                        ]
                    });
                });
            }


            function doAuthorization(voucherNumber) {
                if (voucherType === 'VOUCHER_OMNI') {
                    voucherNumber = voucherNumber.slice(1); // Remove o primeiro caractere
                    hideBarcodeScanner()
                        .then(() => askForPassword())
                        .then((voucherPassword) => searchVouchersByDocument(voucherNumber, voucherPassword))
                        .then(vouchers => checkBalance(vouchers))
                        .then(response => onVouchersFound(response))
                        .then(params => onAvailableVouchers(params, voucherNumber))
                        .catch(error => {
                            $scope.voucherInput = {};
                            onAuthorizationError(error)
                                .then(() => initBarcodeScanner());
                        })
                        .finally(hideLoadingMessage);
                } else {
                    hideBarcodeScanner()
                        .then(() => checkVoucherUsed(voucherNumber))
                        .then(showLoadingMessage)
                        .then(() => searchVoucher(voucherNumber))
                        .then(response => checkBalance(response))
                        .then(response => authorizeVoucher(response))
                        .then(paymentDone)
                        .catch(error => {
                            onAuthorizationError(error)
                                .then(() => initBarcodeScanner());
                        })
                        .finally(hideLoadingMessage);
                }
            }

            $scope.goBack = () => {
                UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchoptions', $stateParams);
            };

            function initBarcodeScanner() {
                if($scope.scanning_method !== "CAMERA"){
                    if (window.bluebirdBarcodeScanner) {
                        bluebirdBarcodeScanner.register((barcodeScanned) => {
                            onBarcodeScanned(barcodeScanned);
                        }, (argument) => {
                            LogService.info("Erro ao registrar Scanner: " + argument);
                        });
                    }
                } else {
                   const barcodeParams = { inSizeCam: ['0/88/0/150', '0/88/0/150'], outSizeCam: ['275/44/0/300', '275/44/0/300'] };
                   BarcodeService.cancel();
                   BarcodeService.init(onCameraScanned, barcodeParams);
                }

            }

            function hideBarcodeScanner() {
                return Promise.resolve(BarcodeService.cancel());
            }

            function onBarcodeScanned(scanned) {
                const length = scanned.length;
                const barcode = length % 2 == 0 && scanned.charAt(0) == "0" ?
                    scanned.substr(1, length - 1) : scanned;
                $scope.voucherNumber = barcode;
                doAuthorization($scope.voucherNumber);
            }

            function onCameraScanned(scanResult) {
                const barcodeScanned = scanResult[0].replace(/^0+/,'');
                onBarcodeScanned(barcodeScanned);
            }
        });
})();
