(function () {
    'use strict';
    angular.module('easyorder.controllers.voucher', [])
        .controller('VoucherCtrl', function (
            $rootScope,
            $scope,
            $timeout,
            TransactionReturnService,
            ConfigService,
            UtilService,
            $ionicModal,
            AlertService,
            $ionicLoading,
            CartService,
            $snackbar,
            $stateParams,
            UserService,
            EFTService,
            EWalletService,
            $ionicPopup,
            LogService,
            $q,
            PaymentLinkService) {

            $snackbar.init();
            $scope.printContents = $stateParams.printContents;

            function returns(reversedPayments) {
                const returnData = $stateParams.returnData;

                return TransactionReturnService.return({
                    appInfo: {
                        name: $rootScope.APP_NAME,
                        version: $rootScope.appVersion
                    },
                    sourceTransactionKey: returnData.sourceTransaction.key,
                    transactionKey: {
                        storeInfo: {
                            companyNumber: ConfigService.config.companyNumber,
                            storeNumber: ConfigService.config.storeNumber,
                        },
                        date: new Date(),
                        nsu: ConfigService.nsuControll.incrementNSU(),
                        terminal: ConfigService.config.terminalNumber,
                    },
                    operationType: $scope.op,
                    returnReason: $stateParams.returnReason,
                    operator: {
                        code: UserService.operatorCode(),
                        name: UserService.operatorName()
                    },
                    authorizer: {
                        code: $stateParams.authorizer.id,
                        name: $stateParams.authorizer.name
                    },
                    customer: CartService.cart.customer,
                    fiscalItems: returnData.fiscalItems,
                    sourceTransactionTotalValue: returnData.sourceTransaction.totalValue,
                    totalValue: returnData.fiscalItems
                        .reduce((acc, item) => new Number((acc + item.totalValue).toFixed(2)), 0),
                    reversedPayments: reversedPayments,
                });
            }

            $scope.createVoucher = () => {
                $scope.op = 'VOUCHER_RETURN';

                $ionicLoading.show({
                    template : 'Aguarde...',
                    duration : 0
                }).then(() => {
                    return returns();
                }).then(result => {
                    $ionicLoading.hide().then(() => {
                        UtilService.navigateWithoutHistory('easyorder.couponview', {
                            pdf: result.pdf,
                            printContents: result.printContents,
                            documentTitle: 'voucher'
                        });
                    });
                }).catch(err => {
                    return $ionicLoading.hide().then(() => {
                        $scope.loadingState = false;
                        return AlertService.restError(err, err);  
                    });
                });
            };

            $scope.returnCash = () => {
                $scope.op = 'CASH_RETURN';

                $ionicPopup.show({
                    template: 'Tem certeza que deseja realizar o estorno total da compra?',
                    title: 'Confirmação',
                    scope: $scope,
                    buttons: [
                        { text: 'Não' },
                        {
                            text: 'Sim',
                            type: 'button-theme',
                            onTap: function(e) {
                                return true;
                            }
                        }
                    ]
                }).then(res => {
                    if (!res) {
                        return $q.reject();
                    }
                    return $q.resolve();
                }).then(() => {
                    return $ionicLoading.show({
                        template : 'Aguarde...',
                        duration: 0,
                    });
                }).then(() => {
                    return returnPayments($stateParams.sourceTransaction);
                }).then(reversedPayments => {
                    return returns(reversedPayments);
                }).then(result => {
                    return confirmTransactions().then(() => result);
                }).then(result => {
                    $ionicLoading.hide().then(() => {
                        UtilService.navigateWithoutHistory('easyorder.couponview', {
                            pdf: result.pdf,
                            printContents: result.printContents,
                            documentTitle: 'voucher'
                        });
                    });
                }).catch(err => {
                    if (!err) {
                        return $ionicLoading.hide();
                    }
                    return undoTransactions().finally(() => {
                        $ionicLoading.hide().then(() => {
                            $scope.loadingState = false;
                            AlertService.restError(err, err);
                        });
                    });
                });
            };

            function isPartialReturn(srcTrns, returnedFiscalItems) {
                const availableCount = srcTrns.availableFiscalItems.filter(item => item.uuid).reduce(quantityAccumulator, 0);
                const returnedCount = returnedFiscalItems.reduce(quantityAccumulator, 0);

                if (availableCount > returnedCount) {
                    return true;
                }

                return false;
            }

            function quantityAccumulator(acc, item) {
                return acc + new Number(item.quantity);
            }

            function returnPayments(srcTrns) {
                const nonAllowedPmnt = srcTrns.payments.some(
                    pmnt => !eftFilter(pmnt)
                        && !qrLinxFilter(pmnt)
                        && !paymentLinkFilter(pmnt)
                );
                if (nonAllowedPmnt) {
                    return $q.reject('Só é possível cancelamento de Crédito, Débito, QR Linx e Link de Pagamentos, favor efetuar operação no PDV');
                }

                const rawSrcTrnsDate = srcTrns.key.date;
                const timeIdx = rawSrcTrnsDate.indexOf('T');
                const srcTrnsDate = timeIdx != -1 ? moment(rawSrcTrnsDate.substring(0, timeIdx))
                                                  : moment(rawSrcTrnsDate);

                if (srcTrnsDate.isSame(moment(), 'day')) {
                    if (isPartialReturn(srcTrns, $stateParams.returnData.fiscalItems)) {
                        return $q.reject(`
                            Não é possível realizar estorno parcial para para pagamento em Cartão, QR Linx ou Link de Pagamentos na data atual
                        `);
                    }
                    // Se esta variável estiver setada
                    // significa que o CXR estornou os
                    // pagamentos presentes no array.
                    $scope.reversedPayments = [];
                    // Estornaremos primeiro os TEFS, pois
                    // o QRLinx não dá pra desfazer a operação
                    // de estorno, TEFS podem existir perguntas
                    // onde o usuário pode desistir da operação
                    return srcTrns.payments.sort((p1, p2) => {
                        const isP1EFT = eftFilter(p1);
                        const isP2EFT = eftFilter(p2);

                        if (isP1EFT) {
                            return isP2EFT ? 0 : -1;
                        }

                        return isP2EFT ? 1 : 0;
                    }).reduce((acc, pmnt) =>
                        acc.then(_ =>
                            returnPayment(pmnt, srcTrnsDate)
                        ),
                        $q.resolve()
                    ).then(() =>
                        $scope.reversedPayments
                    );
                }

                return $q.resolve(srcTrns.payments);
            }

            function returnPayment(payment, srcTrnsDate) {
                if (qrLinxFilter(payment)) {
                    return returnQRLinxPayment(payment);
                }
                if (eftFilter(payment)) {
                    return returnEFTPayment(payment, srcTrnsDate);
                }
                if (paymentLinkFilter(payment)) {
                    return returnPaymentLink(payment);
                }
                return $q.reject(`FATAL: Forma de pagamento ${payment.paymentMethod}`
                                 + `não é esperada neste trecho, contate o suporte`);
            }

            function returnQRLinxPayment(payment) {
                return EWalletService.cancelPayment({
                    paymentId: payment.id,
                    remoteTerminalId: payment.remoteTerminalId,
                    walletType: payment.paymentMethod
                }).then(() => {
                    $scope.reversedPayments.push(payment);
                    return payment;
                });
            }

            function returnEFTPayment(payment, srcTrnsDate) {
                const deferred = $q.defer();

                EFTService.executeTransaction(
                    {
                        amount: new Number(payment.value),
                        transactionType: EFTService.transactionType.CANCEL,
                        originTransactionType: EFTService.transactionType[payment.paymentMethod],
                        operatorCode: UserService.operatorCode(),
                        transactionDate: srcTrnsDate.toISOString(),  
                        transactionNSU: payment.id,
                    },
                    {
                        paymentDone: res => {
                            $ionicLoading.show({
                                template : 'Aguarde...',
                                duration : 0
                            });
                            const pmnt = {
                                id: res.NSU,
                                paymentMethod: payment.paymentMethod,
                                date: new Date(),
                                value: new Number(payment.value),
                                customerReceipt: res.ComprovanteCliente,
                                businessReceipt: res.ComprovanteEstabelecimento,
                            };
                            // salvando pagamento que foi estornado
                            $scope.reversedPayments.push(pmnt);
                            deferred.resolve(pmnt);
                        },
                        paymentError: error => {
                            deferred.reject(error);
                        },
                        displayMessage: message => {
                            $ionicLoading.show({
                                template : message || 'Aguarde...',
                                duration : 0
                            });
                            LogService.info("displayMessage: " + message);
                        },
                        clearDisplay: () => {
                            $ionicLoading.show({
                                template : 'Aguarde...',
                                duration : 0
                            });
                            LogService.info("clearDisplay");
                        },
                        selectOption: (title, options, responseHandler) => {
                            $q((resolve) => {
                                $scope.message = title;
                                processOptions(options, responseHandler);
                                LogService.info("Opção tef selecionada: ", title, " Opções: ", options);
                                $ionicModal.fromTemplateUrl('views/tef-modal.html', {
                                    scope: $scope,
                                    animation: 'slide-in-up',
                                    hardwareBackButtonClose: false,
                                    backdropClickToClose: false,
                                })
                                .then(modal => {
                                    $scope.modal = modal;
                                    $ionicLoading.hide()
                                    .then(() => {
                                        $scope.modal.show();
                                    });
                                });
                                resolve();
                            }).then(() => {
                                $ionicLoading.show({
                                    template : 'Aguarde...',
                                    duration : 0
                                });
                            });
                        },
                        selectOptionExtended: (title, options, responseHandler) => {
                            $q((resolve) => {
                                $scope.message = title;
                                processOptions(options, responseHandler);
                                LogService.info("Opção extendida selecionada: ", title, " Opções: ", options);
                                $ionicModal.fromTemplateUrl('views/tef-modal.html', {
                                    scope: $scope,
                                    animation: 'slide-in-up',
                                    hardwareBackButtonClose: false,
                                    backdropClickToClose: false,
                                })
                                .then(modal => {
                                    $scope.modal = modal;
                                    $ionicLoading.hide()
                                    .then(() => {
                                        $scope.modal.show();
                                    });
                                });
                                resolve();
                            }).then(() => {});
                        },
                        confirmAction: (message) => {
                            return $ionicLoading.hide().then(() =>
                                $ionicPopup.confirm({
                                    title: 'Transação TEF',
                                    template: message,
                                    okType: "button-theme"
                                })
                            ).finally(() =>
                                $ionicLoading.show({
                                    template : 'Aguarde...',
                                    duration : 0
                                })
                            );
                        },
                    }
                );
                return deferred.promise;
            }

            function confirmTransactions() {
                if ($scope.op != 'CASH_RETURN') {
                    return $q.resolve();
                }

                if ($scope.reversedPayments == null ||
                        $scope.reversedPayments.length == 0) {
                    return $q.resolve();
                }

                let ret = $q.resolve();
                const eftPayments = $scope.reversedPayments.filter(eftFilter);

                if (eftPayments.length > 0) {
                    if (EFTService.isDTEF()) {
                        ret = ret.then(() => eftPayments.reduce(
                            (acc, payment) => acc.then(() => EFTService.confirmTransaction(payment.id)),
                            $q.resolve()
                        ));
                    } else {
                        ret = EFTService.confirmTransaction(eftPayments[0].id);
                    }
                }
                    
                return ret.then(() => {
                    delete $scope.reversedPayments;
                });
            }

            function undoTransactions() {
                if ($scope.op != 'CASH_RETURN') {
                    return $q.resolve();
                }

                if ($scope.reversedPayments == null ||
                        $scope.reversedPayments.length == 0) {
                    return $q.resolve();
                }

                let ret = $q.resolve();
                const eftPayments = $scope.reversedPayments.filter(eftFilter);

                if (eftPayments.length > 0) {
                    if (EFTService.isDTEF()) {
                        ret = ret.then(() => eftPayments.reduce(
                            (acc, payment) => acc.then(() => EFTService.undoTransaction(payment.id)),
                            $q.resolve()
                        ));
                    } else {
                        ret = EFTService.undoTransaction(eftPayments[0].id);
                    }
                }

                return ret.then(() => {
                   delete $scope.reversedPayments;
                });
            }

            function returnPaymentLink(payment) {
                const paymentLinkId = findPaymentLinkId(payment);
                const returned = $scope.reversedPayments
                    .filter(paymentLinkFilter)
                    .some(p => paymentLinkId == findPaymentLinkId(p));

                if (returned) {
                    $scope.reversedPayments.push(payment);
                    return $q.resolve(payment);
                }

                return PaymentLinkService
                    .disablePaymentLink(paymentLinkId)
                    .then(() => {
                        $scope.reversedPayments.push(payment);
                        return payment;
                    });
            }

            function findPaymentLinkId(payment) {
                return payment.paymentMethod == 'PAYMENT_LINK' ?
                    payment.id : payment.groupId;
            }

            function eftFilter(payment) {
                const eftPaymentMethods = ['CREDIT', 'DEBIT'];
                return eftPaymentMethods.includes(payment.paymentMethod)
                    && !isLinkProvider(payment.provider)
            }

            function qrLinxFilter(payment) {
                return payment.paymentMethod == 'QR_LINX';
            }

            function paymentLinkFilter(payment) {
                return payment.paymentMethod == 'PAYMENT_LINK'
                    || isLinkProvider(payment.provider);
            }

            function isLinkProvider(provider) {
                if (provider == null) {
                    return false;
                }

                const linkProviders = [
                    'PAGAR_ME',
                    'LINX_PAYHUB_LINK',
                    'E_SITEF_WEB_CHECKOUT'
                ];

                return linkProviders.includes(provider.id);
            }

            function processOptions(options, responseHandler) {
                $scope.showWatermark = false;
                $scope.options = [];

                var splitedOpt = options.replace(/\(|\)/g, ""); //removendo "(" e ")"
                splitedOpt = splitedOpt.split("#"); // Separando as opções.

                for (var i = 0; i < splitedOpt.length; i++) {
                    var split = splitedOpt[i].split(",");
                    $scope.options.push({
                        id:         split[0].trim(),
                        label:      split[1].trim().replace(/\"/g, ""),
                        value:      split[split.length-1],
                        handler:    responseHandler,
                        type:        "single-option"
                    });
                }
                $scope.headerOptions = "Opções";
            };

            $scope.tickNumberInputHandler = function(inputOption) {
                var value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 'd', inputOption.value);

                $scope.tickOptionHandler(inputOption.handler, [value]);
            };

            $scope.tickStringInputHandler = function(inputOption) {
                const value = sprintf('%1$' + (inputOption.maxSize ? '0' + inputOption.maxSize : '') + 's', inputOption.value);

                $scope.tickOptionHandler(inputOption.handler, [value]);
            };

            $scope.tickOptionHandler = function(func, params) {
                $scope.options.length = 0;
                $scope.showWatermark = true;

                $scope.modal.hide();
                $ionicLoading.show();

                func.apply(this, params);
            };

        })
})();
