
(function () {
    'use strict';
    angular.module('easyorder.controllers.voucherpayment', [])
        .controller('VoucherPaymentCtrl', function (
            $scope,
            ConfigService,
            UtilService,
            AlertService,
            $ionicLoading,
            CartService,
            $snackbar,
            VoucherService,
            PaymentService,
            $q) {

            $scope.formData = {};

            $scope.formData.label = '';

            $snackbar.init();

            $scope.useVoucher = () => {
                $scope.voucherCode = $scope.formData.voucherCode;
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(() => {
                    return VoucherService.search($scope.voucherCode, ConfigService.config.storeNumber, 'VOUCHER_STOREX');
                }).then(result => {
                    const voucher = result.vouchers[0];
                    const cmpVal = new Number(voucher.value) + new Number(PaymentService.totalPayments());
                    const cartTotalValue = new Number(CartService.cart.totalValue);
                    if (cmpVal > cartTotalValue) {
                        return $q.reject(new OverPaymentError(`
                            Valor do crédito devolução (R$ ${voucher.value}) somado com os pagamentos existentes supera o total
                        `));
                    }
                    return $q.resolve();
                }).then(() => {
                    return VoucherService.authorize(
                        ConfigService.nsuControll.incrementNSU(),
                        $scope.voucherCode,
                        ConfigService.config.storeNumber,
                        'VOUCHER_STOREX',
                        null,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null
                    );
                }).then(result => {
                    _initPayment(result.paymentMethod, result.value, false);
                    PaymentService.currentPaymentDone();

                    const salePayment = {
                        type: 'SaleVoucherPayment',
                        paymentCodeMethod: result.paymentMethod,
                        paymentSequence: PaymentService.payments().length,
                        paymentValue: result.value,
                        installments: [],
                        voucherNumber: result.voucherNumber,
                        cardNumber: result.voucherNumber,
                        storeNumber: result.storeNumber,
                        bussinessPaymentVoucher: result.businessReceipt,
                    };
                    CartService.cart.salePayments.push(salePayment);

                    $snackbar.show({
                        message: `Adicionado crédito devolução #${result.voucherNumber}, R$${result.value}`,
                        buttonName: "Ok",
                        textAlign: 'center',
                        buttonFunction: null
                    });

                    $ionicLoading.hide().then(() => {
                         UtilService.navigateAfterReceivedPayment();
                    });
                }).catch(error => {
                    $ionicLoading.hide().then(() => {
                        if (error instanceof OverPaymentError) {
                            AlertService.error(error.message).then(() => {
                                UtilService.navigateWithoutHistory('easyorder.productsearch');
                            });
                        } else {
                            AlertService.restError(error, error);
                        }
                    });
                });
            };

            $scope.goBack = () => {
                if (CartService.hasUnprocessedPredefinedPaymentMethod()) {
                    UtilService.navigateWithoutHistory('easyorder.predefinedpaymentmethods');
                } else {
                    UtilService.navigateWithoutHistory('easyorder.paymentmethods');
                }
            };

            function _initPayment(paymentMethod, paymentValue, privateLabel) {
                var ret = paymentValue;

                CartService.processPaymentCampaigns(paymentMethod, paymentValue);
                CartService.updateCartTotalValue();

                if (CartService.cart.totalValue < paymentValue) {
                    ret = CartService.cart.totalValue;
                }
                PaymentService.initPayment(paymentMethod, ret, privateLabel);

                return ret;
            };
            const customName = ConfigService.config.business.available_payment_methods_names.find(pmnt => pmnt.id === 'VOUCHER' || pmnt.id === 'VOUCHER_STOREX');
            if(customName){
                $scope.formData.label = customName.name;
            }else{
                $scope.formData.label = 'Voucher';
            }

        })
})();

class OverPaymentError extends Error {
    constructor(message) {
        super(message);
        this.name = 'OverPaymentError';
    }
}
