
(function () {
    'use strict';
    angular.module('easyorder.controllers.voucherpaymentsearch', [
        "angular-uuid"
    ])
    .controller('VoucherPaymentSearchCtrl', function (
        $scope,
        $ionicLoading,
        $stateParams,
        $q,
        $filter,
        AlertService,
        VoucherService,
        CartService,
        UtilService,
        UserService,
        PaymentService,
        ConfigService,
        LogService,
        PaymentProcessor,
        uuid
    ) {
        $scope.vouchersSelected = [];
        $scope.totalValueVoucher = 0;
        $scope.voucherCountSelected = 0;
        $scope.customer = $stateParams.customer;
        $scope.containsUsed = $stateParams.containsUsed;
        const voucherType = $stateParams.voucherType;
        const voucherCredentials = $stateParams.voucherCredentials;
        const [singularLabel, pluralLabel] = ConfigService.config.business.promo_voucher_label;
        $scope.paymentValue = $stateParams.paymentValue;
        $scope.voucherId = voucherType;
        $scope.vouchersLabel = (voucherType === 'VOUCHER_STOREX' ? 'Créditos Devoluções' :
                                voucherType === 'VOUCHER_OMNI' ? pluralLabel : 'Vouchers');
        $scope.voucherLabel = (voucherType === 'VOUCHER_STOREX' ? 'Crédito Devolução' :
                               voucherType === 'VOUCHER_OMNI' ? singularLabel : 'Voucher');
        $scope.vouchers = $stateParams.vouchers.sort( (voucher1, voucher2) => {
            return new Date(voucher1.expirationDate) - new Date(voucher2.expirationDate);
        });

        $scope.dateFormat = function(date){
            if (date){
                return $filter('date')(moment.parseZone(date).format('DD/MM/YY'));
            }
            return null;
        };

        $scope.select = (item) => {
            if (!item.used){
                if (!item.selected && voucherType === 'VOUCHER_GIVEX' && (Number($scope.totalValueVoucher) >= $scope.paymentValue)) {
                    AlertService.error(`Valor do(s) voucher(s) (R$ ${$scope.totalValueVoucher }) já superam o valor do pagamento (R$ ${$scope.paymentValue}) .`);
                } else {
                    item.selected = !item.selected;
                    if (item.selected){
                        $scope.vouchersSelected.push(item);
                    }else{
                        $scope.vouchersSelected.pop(item);
                    }
                    $scope.updateTotalValue(item);
                }
            }
        };

        $scope.updateTotalValue = (item) => {
            if (item.selected) {
                $scope.voucherCountSelected++;
                $scope.totalValueVoucher = Number($scope.totalValueVoucher) +  Number(item.value);
            } else {
                $scope.voucherCountSelected--;
                $scope.totalValueVoucher = Number($scope.totalValueVoucher) - Number(item.value);
            }
        };

        $scope.confirm = () => {
            if ($scope.voucherCountSelected > 0) {
                $ionicLoading.show({
                    template : 'Por favor, aguarde...',
                    duration : 0
                }).then(() => {
                    const cmpVal = new Number($scope.totalValueVoucher) + new Number(PaymentService.totalPayments());
                    const remainingTotal = new Number(CartService.cart.totalValue) - $scope.paymentCampaignTotalDiscount;
                    if (cmpVal >= remainingTotal  && voucherType == 'VOUCHER_ASSURANT') {
                        return $q.reject(new OverPaymentVoucherError(`
                            Valor do(s) voucher(s) (R$ ${$scope.totalValueVoucher}) somado(s) com os pagamentos existentes supera o total do pedido,
                            reveja seu carrinho de produtos ou escolha mais produtos para utilizar o voucher.
                        `));
                    } else if (cmpVal > remainingTotal && voucherType == 'VOUCHER_STOREX') {
                        return $q.reject(new OverPaymentVoucherError(`
                            Valor do(s) crédito(s) devolução(ões) (R$ ${$scope.totalValueVoucher}) somado(s) com os pagamentos existentes supera o total do pedido,
                            reveja seu carrinho de produtos ou escolha mais produtos para utilizar o(s) crédito(s) devolução(ões).
                    `   ));
                    } else if (cmpVal > remainingTotal && voucherType == 'VOUCHER_SAFE') {
                        return $q.reject(new OverPaymentVoucherError(`
                            Valor do(s) crédito(s) devolução(ões) (R$ ${$scope.totalValueVoucher}) somado(s) com os pagamentos existentes supera o total do pedido,
                            reveja seu carrinho de produtos ou escolha mais produtos para utilizar o(s) crédito(s) devolução(ões).
                    `   ));
                    }
                    return $q.resolve();
                }).then(() => {
                    if (voucherType === 'VOUCHER_GIVEX') {
                        return authorizeGivex();
                    } else if (voucherType === 'VOUCHER_OMNI') {
                        return authorizePromo();
                    } else {
                        return authorizeGeneralVouchers();
                    }
                })
                .then(UtilService.navigateAfterReceivedPayment)
                .catch(error => {
                    cancelPayments()
                        .then(() => $ionicLoading.hide())
                        .then(() => {
                            if (error instanceof OverPaymentVoucherError) {
                                AlertService.error(error.message).then(() => {
                                    UtilService.navigateWithoutHistory('easyorder.cart');
                                });
                            } else {
                                AlertService.restError(error, error);
                            }
                        });
                });
            } else {
                AlertService.warn(`Por favor, selecione ao menos um ${$scope.voucherLabel}!`);
            }
        };

        function authorizeGivex() {
            $scope.remainingPaymentValue = $scope.paymentValue;

            return $scope.vouchersSelected.reduce((acc, voucher) => {
                return acc.then(() => {
                    var currentPaymentValue = voucher.value <= $scope.remainingPaymentValue ? voucher.value : $scope.remainingPaymentValue;
                    currentPaymentValue = new Number(currentPaymentValue.toFixed(2));
                    LogService.info('Efetuando pagamento de voucher de valor '+ currentPaymentValue);

                    const contextUid = uuid.v4();
                    LogService.info(`Contexto do Voucher '${contextUid}'`);

                    const paymentSequence = PaymentService.payments().length;

                    PaymentProcessor.pushSalePayment({
                        type: 'SaleVoucherGivexPayment',
                        paymentCodeMethod: 'VOUCHER_GIVEX',
                        paymentValue:  currentPaymentValue,
                        storeNumber: ConfigService.config.storeNumber,
                        voucherOwner: $scope.customer,
                        contextUid: contextUid,
                    });

                    return VoucherService.authorizeBySerialNumber(
                        ConfigService.config.companyNumber,
                        ConfigService.config.storeNumber,
                        ConfigService.config.terminalNumber,
                        voucherType,
                        voucher.cardISO,
                        voucher.serialNumber,
                        currentPaymentValue,
                        CartService.cart.nsu,
                        $scope.customer.document,
                        $scope.customer.documentType,
                        contextUid,
                    ).then(authorizationResult => {
                        initializePayment('VOUCHER_GIVEX', currentPaymentValue, false);
                        PaymentService.currentPaymentDone();

                        const salePayment = {
                            type: 'SaleVoucherGivexPayment',
                            paymentCodeMethod: 'VOUCHER_GIVEX',
                            paymentValue:  currentPaymentValue,
                            paymentSequence: paymentSequence,
                            installments: [],
                            voucherNumber:  authorizationResult.voucherNumber,
                            cardNumber: authorizationResult.voucherNumber,
                            storeNumber: ConfigService.config.storeNumber,
                            paymentAuthorizationCode: authorizationResult.authorizationCode,
                            expirationDate: authorizationResult.expirationDate,
                            currentBalance: authorizationResult.currentBalance,
                            voucherOwner: $scope.customer
                        };
                        CartService.cart.salePayments.push(salePayment);

                        PaymentProcessor.updateSalePayment(payment =>
                            payment.data.confirm.contextUid == contextUid,
                            { ...salePayment, contextUid: contextUid }
                        );

                        $scope.remainingPaymentValue = $scope.remainingPaymentValue - currentPaymentValue;
                        LogService.info('Valor remanescente de pagamento '+ $scope.remainingPaymentValue);
                    }).catch(err => {
                        // não sabemos se foi uma queda de rede de toda forma já marcamos pra desfazer o pagamento
                        PaymentProcessor.undoPayment(payment =>
                            payment.data.undo.contextUid == contextUid
                        );
                        return $q.reject(err);
                    });
                }
                );
            }, $q.resolve());
        }

        function authorizePromo() {
            $scope.remainingPaymentValue = $scope.paymentValue;

            return $scope.vouchersSelected.reduce((acc, voucher) => {
                return acc.then(() => {
                    var currentPaymentValue = voucher.value <= $scope.remainingPaymentValue ? voucher.value : $scope.remainingPaymentValue;
                    currentPaymentValue = new Number(currentPaymentValue.toFixed(2));
                    LogService.info('Efetuando pagamento de voucher de valor '+ currentPaymentValue);
                    const contextUid = uuid.v4();
                    LogService.info(`Contexto do Voucher '${contextUid}'`);

                    const paymentSequence = PaymentService.payments().length;

                    PaymentProcessor.pushSalePayment({
                        type: 'SaleVoucherOmniPayment',
                        paymentCodeMethod: ConfigService.config.business.giftcard_service_provider,
                        paymentValue:  currentPaymentValue,
                        storeNumber: ConfigService.config.storeNumber,
                        voucherOwner: $scope.customer,
                        contextUid: contextUid,
                    });

                    return VoucherService.authorize(
                        CartService.cart.nsu,
                        $scope.customer.document,
                        ConfigService.config.storeNumber,
                        ConfigService.config.business.giftcard_service_provider,
                        currentPaymentValue,
                        ConfigService.config.companyNumber,
                        ConfigService.config.terminalNumber,
                        contextUid,
                        voucherCredentials
                    ).then(authorizationResult => {
                        initializePayment('VOUCHER_OMNI', currentPaymentValue, false);
                        PaymentService.currentPaymentDone();

                        // A API acrescenta o prefixo no retorno, então precisa cortar
                        const voucherNumber = typeof authorizationResult.voucherNumber === 'string'?
                                              authorizationResult.voucherNumber.substring(1) :
                                              authorizationResult.voucherNumber;
                        const salePayment = {
                            type: 'SaleVoucherOmniPayment',
                            paymentCodeMethod: 'VOUCHER_PROMO',
                            paymentValue:  currentPaymentValue,
                            paymentSequence: paymentSequence,
                            installments: [],
                            voucherNumber: voucherNumber,
                            cardNumber: authorizationResult.voucherNumber,
                            storeNumber: ConfigService.config.storeNumber,
                            paymentAuthorizationCode: authorizationResult.authorizationCode,
                            expirationDate: authorizationResult.expirationDate,
                            currentBalance: authorizationResult.currentBalance,
                            voucherOwner: $scope.customer
                        };
                        CartService.cart.salePayments.push(salePayment);

                        PaymentProcessor.updateSalePayment(payment =>
                            payment.data.confirm.contextUid == contextUid,
                            { ...salePayment, contextUid: contextUid, password: voucherCredentials, }
                        );

                        $scope.remainingPaymentValue = $scope.remainingPaymentValue - currentPaymentValue;
                        LogService.info('Valor remanescente de pagamento '+ $scope.remainingPaymentValue);

                        // Se o cliente foi previamente identificado, atualiza o saldo do vale
                        if (CartService.cart.customer &&
                            voucherNumber === CartService.cart.customer.document &&
                            ConfigService.config.business.promo_voucher_balance_enabled) {
                            CartService.cart.customer.vouchersBalance = authorizationResult.currentBalance;
                        }
                    }).catch(err => {
                        // não sabemos se foi uma queda de rede de toda forma já marcamos pra desfazer o pagamento
                        PaymentProcessor.undoPayment(payment =>
                            payment.data.undo.contextUid == contextUid
                        );
                        return $q.reject(err);
                    });
                }
                );
            }, $q.resolve());
        }

        function authorizeGeneralVouchers() {
            return $scope.vouchersSelected.reduce((acc, voucher) => {
                const contextUid = uuid.v4();

                let nsu = ConfigService.nsuControll.incrementNSU();
                let undoNsu = null;

                if (voucherType == 'VOUCHER_STOREX') {
                    undoNsu = ConfigService.nsuControll.incrementNSU();
                }

                return acc.then(() =>
                    VoucherService.authorize(
                        nsu,
                        voucher.number,
                        ConfigService.config.storeNumber,
                        voucherType,
                        voucher.value,
                        ConfigService.config.companyNumber,
                        ConfigService.config.terminalNumber,
                        contextUid,
                        null,
                        {code: UserService.operatorCode(), name: UserService.operatorName()},
                        {code: UserService.operatorCode(), name: UserService.operatorName()},
                        CartService.cart.customer ? CartService.cart.customer.document : null,
                        CartService.cart.customer ? CartService.cart.customer.documentType : null,
                        voucher.mac,
                        new Date().toJSON(),
                        undoNsu
                    ).then(authorization => {
                        const paymentSequence = PaymentService.payments().length;
                        const value = voucher.value ? voucher.value : authorization.value;

                        initializePayment(authorization.paymentMethod, value, false);
                        PaymentService.currentPaymentDone();

                        const salePayment = {
                            type: getType(voucherType),
                            paymentCodeMethod: authorization.paymentMethod,
                            paymentSequence: paymentSequence,
                            paymentValue: value,
                            installments: [],
                            voucherNumber: voucher.number,
                            cardNumber: voucher.number,
                            storeNumber: ConfigService.config.storeNumber,
                            model: voucher.model,
                            imei: voucher.imei,
                            bussinessPaymentVoucher: authorization.businessReceipt,
                            safeNsu: authorization.safeNsu,
                            expirationDate: authorization.expirationDate,
                            currentBalance: authorization.currentBalance
                        };

                        // não deve ser serializado
                        Object.defineProperty(salePayment, 'contextUid', {
                            enumerable: false,
                            value: contextUid
                        });

                        CartService.cart.salePayments.push(salePayment);
                    })
                );
            }, $q.resolve());
        }
        /**
         * As campanhas desse pagamento serão aplicadas nos itens, mesmo que sejam campanhas por forma. Então,
         * o registro da campanha é alterado que tal comportamento seja executado.
         */
        function initializePayment(paymentMethod, paymentValue, privateLabel) {
            var ret = paymentValue;

            doPaymentCampaignTotalDiscount();

            if ($scope.paymentCampaignTotalDiscount) {
                $scope.campaignForPayment.forEach(c => {
                    c.campaignPayment = false;
                    c.couponDiscount = 0;
                });
                CartService.applyCampaignsDiscounts($scope.campaignForPayment);
                CartService.updateCartTotalValue();
            }

            if (CartService.cart.totalValue < paymentValue) {
                ret = CartService.cart.totalValue;
            }
            PaymentService.initPayment(paymentMethod, ret, privateLabel);

            return ret;
        };

        function getType(voucherType) {
            let ret;
            switch (voucherType) {
                case 'VOUCHER_GIVEX':
                    ret = 'SaleVoucherGivexPayment';
                    break;
                case 'VOUCHER_ASSURANT':
                    ret = 'SaleVoucherAssurantPayment';
                    break;
                case 'VOUCHER_SAFE':
                    ret = 'SaleVoucherSafePayment'
                    break;
                default:
                    ret = 'SaleVoucherPayment';
                    break;
            }
            return ret;
        }

        function cancelPayments() {
            if (CartService.cart.salePayments.length > 0) {
                return PaymentService.cancel();
            }
            return $q.resolve();
        }

        $scope.goBack = () => {
            if (CartService.hasUnprocessedPredefinedPaymentMethod()) {
                UtilService.navigateWithoutHistory('easyorder.predefinedpaymentmethods');
            } else {
                UtilService.navigateWithoutHistory('easyorder.paymentmethods');
            }
        };

       function campaignForPayment() {
           var ret = [];
           var campaignsPayment = CartService.cart.campaignsPayment;

           if (campaignsPayment && campaignsPayment.length) {
               campaignsPayment.forEach(campaign => {
                   if ((!CartService.cart.promotions || !CartService.cart.promotions.length)
                           && !CartService.cart.promotions.filter(p => campaign.codeCampaing).length) { //Ainda nao foi aplicada?
                       if (campaign.payments
                               && campaign.payments.length
                               && campaign.payments.filter(p => p.formCode == voucherType).length) {

                           ret.push(campaign);
                       }
                   }
               });
           }
           return ret;
        };

        function doPaymentCampaignTotalDiscount() {
            $scope.campaignForPayment = campaignForPayment();
            $scope.paymentCampaignTotalDiscount = $scope.campaignForPayment
                .map(c => c.products.map(p => p.unitDiscount * p.quantitySold))
                .reduce((total, productsValues) => total + productsValues.reduce((pTotal, value) => pTotal + value , 0), 0);
            LogService.info(`Desconto total de campanha aplicável ao pagamento em ${voucherType}: ${$scope.paymentCampaignTotalDiscount}`);
        };

        /**
         *
         */
        (() => {
            doPaymentCampaignTotalDiscount();
        })();
    })
})();

class OverPaymentVoucherError extends Error {
    constructor(message) {
        super(message);
        this.name = 'OverPaymentVoucherError';
    }
}
