(function () {
    'use strict';
    angular.module('easyorder.controllers.voucherpaymentsearchoptions', ["angular-uuid"])
    .controller('VoucherPaymentSearchOptionsCtrl', function (
                                                        $scope,
                                                        AlertService,
                                                        $ionicLoading,
                                                        UserService,
                                                        UtilService,
                                                        ConfigService,
                                                        VoucherService,
                                                        PaymentService,
                                                        CartService,
                                                        LogService,
                                                        $q,
                                                        $stateParams,
                                                        PaymentProcessor,
                                                        uuid) {

        $scope.requestauth = {};
        $scope.customer = $stateParams.customer;
        $scope.paymentValue = $stateParams.paymentValue;

        $scope.popupOptionShown = 'NONE';
        $scope.voucherInput = {};
        $scope.voucherId = $stateParams.voucherType;
        const voucherType = $stateParams.voucherType;
        const [singularLabel, pluralLabel] = ConfigService.config.business.promo_voucher_label;
        $scope.isPasswordRequired = $stateParams.isPasswordRequired;
        $scope.voucherLabel = (voucherType === 'VOUCHER_OMNI' ? singularLabel : 'Voucher');
        $scope.vouchersLabel = (voucherType === 'VOUCHER_OMNI' ? pluralLabel : 'Vouchers');

        const auths = new Map();

        $scope.requestAuth = () => {
            if (!$scope.requestauth.user || !$scope.requestauth.password) {
                AlertService.error('Todos os campos devem ser preenchidos');
            } else {
                UserService.checkSelfService($scope.requestauth.user)
                .then(
                    () => userAuthorization(),
                    error => AlertService.error(error));
            }
        };
        $scope.paymentBarcode = () => {
            UtilService.navigateWithoutHistory('easyorder.voucherbarcodepayment', { paymentValue: $scope.paymentValue, voucherType: voucherType, isPasswordRequired: $scope.isPasswordRequired });
        };

        $scope.enableBarcodePopup = () => {
            $scope.clearSearch();
            $scope.popupOptionShown = 'BARCODE';
        };

        $scope.enableCustomerPopup = () => {
            $scope.clearSearch();
            if ($scope.customer) {
                if ($scope.customer && $scope.customer.documentType === 'CPF' &&  $scope.customer.document) {
                    $scope.voucherInput.value = $scope.customer.document;
                }
            }
            $scope.popupOptionShown = 'CPF';
        };

        $scope.enableBarcodePopupV3 = () => {
            $scope.clearSearch();
            $scope.popupOptionShown = 'SAFE_VOUCHER';
        };

        $scope.clearSearch = () => {
            $scope.voucherInput.value = "";
            $scope.voucherInput.password = "";
            $scope.popupOptionShown = 'NONE';
        };

        $scope.confirmAction = (option, popupNumber) => {
            switch (option) {
                case 'BARCODE':
                    checkVoucherUsed(popupNumber.value)
                        .then(showLoadingMessage)
                        .then(() => searchVoucher(popupNumber.value))
                        .then(vouchers => checkBalance(vouchers))
                        .then(voucher => authorizeVoucher(voucher))
                        .then(paymentDone)
                        .catch(onAuthorizationError)
                        .finally(hideLoadingMessage);
                    break;
                case 'SAFE_VOUCHER':
                    checkVoucherUsed(popupNumber.value)
                        .then(showLoadingMessage)
                        .then(() => searchVoucher(popupNumber.value))
                        .then(vouchers => onVouchersFoundV3(vouchers))
                        .then(voucher => onAvailableVouchersV3(voucher, popupNumber.value))
                        .catch(onSearchError)
                        .finally(hideLoadingMessage);
                    break;
                case 'CPF':
                    showLoadingMessage()
                        .then(() => searchVouchersByDocument(popupNumber.value, popupNumber.password))
                        .then(response => onVouchersFound(response))
                        .then(params => onAvailableVouchers(params, popupNumber.value, popupNumber.password))
                        .catch(onSearchError)
                        .finally(hideLoadingMessage);
                    break;
            }
        };

        $scope.disableConfirmButton = () => {
            return (!$scope.voucherInput.value && !$scope.isPasswordRequired)
                || ($scope.isPasswordRequired && !$scope.voucherInput.value && !$scope.voucherInput.password);
        }

        function checkVoucherUsed(voucherNumber) {
            return VoucherService.checkVoucherUsed(voucherType, voucherNumber, CartService.cart.salePayments);
        }

        function getType(voucherType) {
            let ret;
            switch (voucherType) {
                case 'VOUCHER_GIVEX':
                    ret = 'SaleVoucherGivexPayment';
                    break;
                case 'VOUCHER_ASSURANT':
                    ret = 'SaleVoucherAssurantPayment';
                    break;
                case 'VOUCHER_OMNI':
                    ret = 'SaleVoucherOmniPayment';
                case 'VOUCHER_SAFE':
                    ret = 'SaleVoucherSafePayment'
                    break;
                default:
                    ret = 'SaleVoucherPayment';
                    break;
            }
            return ret;
        }

        function authorizeVoucher(response) {
            const voucher = response.vouchers[0];
            if (auths.has(voucher.number)) {
                return $q.reject(`Aguarde o processamento do ${$scope.voucherLabel} '${voucher.number}' antes de tentar utilizá-lo`);
            }

            const currentPaymentValue = $scope.paymentValue > voucher.value ? voucher.value : $scope.paymentValue;
            LogService.info(`Saldo do cartão é ${currencyFormat(voucher.value)} e o pagamento solicitado ${currencyFormat($scope.paymentValue)}. Será utilizado ${currencyFormat(currentPaymentValue)}.`);

            const contextUid = uuid.v4();
            LogService.info(`Contexto do Voucher '${contextUid}'`);
            auths.set(voucher.number, contextUid);

            PaymentProcessor.pushSalePayment({
                type: getType(voucherType),
                paymentCodeMethod: voucherType,
                voucherNumber: voucher.number,
                paymentValue:  currentPaymentValue,
                companyNumber: ConfigService.config.companyNumber,
                terminalNumber: ConfigService.config.terminalNumber,
                storeNumber: ConfigService.config.storeNumber,
                voucherOwner: $scope.customer,
                contextUid: contextUid,
            });

            return VoucherService.authorize(
                ConfigService.nsuControll.currentNSU(),
                voucher.number,
                ConfigService.config.storeNumber,
                voucherType,
                currentPaymentValue,
                ConfigService.config.companyNumber,
                ConfigService.config.terminalNumber,
                contextUid,
                null,
                {code: UserService.operatorCode(), name: UserService.operatorName()},
                {code: UserService.operatorCode(), name: UserService.operatorName()},
                CartService.cart.customer ? CartService.cart.customer.document : null,
                CartService.cart.customer ? CartService.cart.customer.documentType : null,
                voucher.mac

            ).catch(err => {
                // não sabemos se foi uma queda de rede
                // de toda forma já marcamos pra desfazer o pagamento
                PaymentProcessor.undoPayment((payment) =>
                    payment.data.undo.contextUid == contextUid
                );
                auths.delete(voucher.number);
                return $q.reject(err);
            });
        }

        function searchVoucher(voucherNumber) {
            return VoucherService.search(voucherNumber, ConfigService.config.storeNumber, voucherType, ConfigService.config.companyNumber, ConfigService.config.terminalNumber);
        }

        var currencyFormat = function (value) {
            let formatter = new Intl.NumberFormat('pt-BR', {
                style: 'currency',
                currency: 'BRL',
                minimumFractionDigits: 2
            });
            return formatter.format(value);
        };

        function checkBalance(response) {
        	return $q((resolve, reject) => {
                if (response && response.vouchers && response.vouchers.length > 0) {
                    const voucher = response.vouchers[0];
                    if (voucher.value <= 0) {
                        reject(new VoucherAlreadyUsedError(`${$scope.voucherLabel} já utilizado!`));
                        return;
                    }

                    if (voucher.value < $scope.paymentValue) {
                        hideLoadingMessage();
                        const msg = `Saldo do ${$scope.voucherLabel} é ${currencyFormat(voucher.value)} inferior ao valor do pagamento. Deseja prosseguir com o saldo como parte do pagamento?`;
                        AlertService
                            .confirm(msg)
                            .then((res) => {
                                if (!res) {
                                    reject(new VoucherOperationAbortedError());
                                    return;
                                }
                                showLoadingMessage();
                                resolve(response);
                            });
                        return;
                    }

                    resolve(response);
                } else {
                    reject(new VoucherNotFoundError(`${$scope.voucherLabel} não encontrado!`));
                }
        	});
        }

        function onAuthorizationError(error) {
            hideLoadingMessage();
            if (error instanceof VoucherAlreadyUsedError) {
                return AlertService.error(error.message);
            } else if (error instanceof VoucherNotFoundError) {
                return AlertService.error(error.message);
            } else if (error instanceof VoucherOperationAbortedError) {
                // operação abortada
            } else {
                return AlertService.restError(error, typeof error == 'string' ? error : `Falha ao autorizar ${$scope.voucherLabel}`);
            }
        }

        function onSearchError(error) {
            $scope.voucherInput.password = "";
            hideLoadingMessage();
            if (error instanceof VoucherNotFoundError) {
                return AlertService.error('Sem cupons disponíveis!');
            } else {
                return AlertService.restError(error, `Falha ao consultar ${$scope.vouchersLabel}`);
            }

        }

        function showLoadingMessage() {
            return $ionicLoading.show({
                template : 'Por favor, aguarde...',
                duration : 0
            });
        }

        function hideLoadingMessage() {
            $ionicLoading.hide();
        }
        function paymentDone(authorizationResult){
            initializePayment(voucherType, authorizationResult.value, false);
            PaymentService.currentPaymentDone();

            const salePayment = {
                type: getType(voucherType),
                paymentCodeMethod: voucherType,
                paymentSequence: PaymentService.payments().length,
                paymentValue:  authorizationResult.value,
                installments: [],
                voucherNumber:  authorizationResult.voucherNumber,
                cardNumber: authorizationResult.voucherNumber,
                storeNumber: ConfigService.config.storeNumber,
                paymentAuthorizationCode: authorizationResult.authorizationCode,
                expirationDate: authorizationResult.expirationDate,
                currentBalance: authorizationResult.currentBalance,
                safeNsu: authorizationResult.safeNsu
            };
            CartService.cart.salePayments.push(salePayment);

            const contextUid = auths.get(authorizationResult.voucherNumber);

            if (contextUid) {
                PaymentProcessor.updateSalePayment(
                    (payment) => payment.data.confirm.contextUid == contextUid,
                    { ...salePayment, contextUid: contextUid },
                );
                auths.delete(authorizationResult.voucherNumber);
            }

            UtilService.navigateAfterReceivedPayment();
        }
        function initializePayment (paymentMethod, paymentValue, privateLabel) {
            var ret = paymentValue;

            CartService.processPaymentCampaigns(paymentMethod, paymentValue);
            CartService.updateCartTotalValue();

            if (CartService.cart.totalValue < paymentValue) {
                ret = CartService.cart.totalValue;
            }
            PaymentService.initPayment(paymentMethod, ret, privateLabel);

            return ret;
        }

        $scope.goBack = () => {
            UtilService.navigateWithoutHistory('easyorder.paymentmethods', $stateParams);
        };

        function searchVouchersByDocument(document, password) {
            const type = (voucherType === 'VOUCHER_OMNI') ? ConfigService.config.business.giftcard_service_provider : voucherType;
            if (voucherType === 'VOUCHER_OMNI') {
                return VoucherService.search(document, ConfigService.config.storeNumber, type,
                                             ConfigService.config.companyNumber, ConfigService.config.terminalNumber, 
                                             ConfigService.nsuControll.currentNSU(), password);
            } else {
                return VoucherService.searchByDocument(ConfigService.config.companyNumber, ConfigService.config.storeNumber,
                                                       ConfigService.config.terminalNumber, type, document, 'CPF');
            }
        }

        function onVouchersFoundV3(res) {
            var params = {
                vouchers: res.vouchers,
                containsUsedVouchers: false
            };
            return $q.resolve(params);
        }
        
        function onVouchersFound(res) {
            const payments = CartService.cart.salePayments ? CartService.cart.salePayments : [];
            const vouchers = res.vouchers ? res.vouchers: [];
            const availableVouchers = vouchers.filter(voucher => voucher.status == 'AVAILABLE' && voucher.value > 0);

            payments.filter(pmnt => pmnt.paymentCodeMethod == voucherType).forEach(pmnt => {
                const usedVoucher = availableVouchers.find(voucher => VoucherService.isSameVoucher(voucher.cardISO, voucher.serialNumber, pmnt.voucherNumber));
                if (usedVoucher) {
                    var index = availableVouchers.indexOf(usedVoucher);
                    if (index > -1) {
                        availableVouchers.splice(index, 1);
                    }
                }
            });

            if (availableVouchers && availableVouchers.length > 0) {
                var params = {
                    vouchers: availableVouchers,
                    containsUsedVouchers: false
                };
                return $q.resolve(params);
            } else {
                return $q.reject(new VoucherNotFoundError('Sem cupons disponíveis!'));
            }
        }

        function onAvailableVouchers(params, currentDocument, currentCredentials) {
            hideLoadingMessage();
            var customer = null;

            if ($scope.customer && $scope.customer.documentType === 'CPF' &&  $scope.customer.document === currentDocument) {
                customer = $scope.customer;
            } else {
                customer = {
                    documentType: 'CPF',
                    document: currentDocument
                };
            }

            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearch', {
                customer: customer,
                vouchers:  params.vouchers,
                containsUsed:  params.containsUsedVouchers,
                voucherType: voucherType,
                paymentValue: $scope.paymentValue,
                voucherCredentials: currentCredentials,
            });
        }

        function onAvailableVouchersV3(params) {
            hideLoadingMessage();

            UtilService.navigateWithoutHistory('easyorder.voucherpaymentsearchv3', {
                vouchers:  params.vouchers,
                containsUsed:  params.containsUsedVouchers,
                voucherType: voucherType,
                paymentValue: $scope.paymentValue });
        }

        function checkAutoSearchField (autoSearchType) {
            return ConfigService.config.business.voucher.auto_search_fields && ConfigService.config.business.voucher.auto_search_fields.indexOf(autoSearchType) >= 0;
        }


        (() => {
            if ($scope.customer && checkAutoSearchField('CUSTOMER_DOCUMENT')) {
                LogService.info(`Fluxo de recebimento desviado automaticamente para consulta de vouchers por CPF de usuário logado: ${$scope.customer.document}`);
                const customer = {
                    value: $scope.customer.document,
                    password: undefined
                }
                $scope.confirmAction("CPF", customer);
            }
        })();
    });
})();
