
(function () {
    'use strict';

    angular.module('easyorder.services.ewallet', [
    ]).service('EWalletService', function(
        ConfigService,
        $http,
        $q
    ) {
        const HTTP = 'http://';
        const HTTPS = 'https://';

        const SERVICE = '/linx-webservices/v1/e-wallet';

        const PAY = '/pay';
        const WAIT_PAYMENT = '/wait-payment';
        const UNDO_PAYMENT = '/undo-payment';
        const CANCEL_PAYMENT = '/cancel-payment';
        const CONFIRM_PAYMENT = '/confirm-payment';

        function url(action) {
            const lws = ConfigService.config.linxwebservices;
            return `${protocol()}${lws.host}:${lws.port}${SERVICE}${action}`;
        }

        function protocol() {
            const lws = ConfigService.config.linxwebservices;
            return lws.secure ? HTTPS : HTTP;
        }

        const service = {
            pay: (paymentRequest) => {
                return $http.post(url(PAY), paymentRequest);
            },
            // long-polling the result
            waitPayment: (waitPaymentRequest) => {
                return $http.post(url(WAIT_PAYMENT), waitPaymentRequest)
                .catch((res) => {
                    if (res.config.timedOut || res.status == 408) {
                        return service.waitPayment(waitPaymentRequest);
                    }
                    return $q.reject(res);
                });
            },
            cancelPayment: (cancelPaymentRequest) => {
                return $http.post(url(CANCEL_PAYMENT), cancelPaymentRequest);
            },
            undoPayment: (undoPaymentRequest) => {
                return $http.post(url(UNDO_PAYMENT), undoPaymentRequest);
            },
            confirmPayment: (confirmPaymentRequest) => {
                return $http.post(url(CONFIRM_PAYMENT), confirmPaymentRequest);
            }
        };

        return service;
    });
})();
