(function() {
	'use strict';

	angular.module('easyorder.services.barcode', [])

	.factory('BarcodeService', function(LogService, UserService) {
		var barcodeScan = {};

		barcodeScan.cancel = function() {
			if (barcodeScan.scanActive) {
				barcodeScan.scanActive = false;
				cordova.exec(null, null, "ScanditSDK", "cancel", []);
			}
		};

		barcodeScan.initScan = function(barcodeCallback, extraOpts) {
			if (!barcodeScan.scanActive) {
				barcodeScan.scanActive = true;

				let opts = {
					"beep": true,
					"codeDuplicateFilter": 2000,
					"continuousMode": true,
					"portraitMargins": barcodeScan.inSizeCam,
					"guiStyle": "laser",
					"scanningHotSpotHeight": 0.15,
					"torch": false
				};

				if (extraOpts == null) {
					opts = {
						...opts,
						qr: false,
						itf: true,
						code39: true
					};
				} else {
					opts = {
						...opts,
						...extraOpts
					};
				}

				cordova.exec(session => {
				    	LogService.info(`Código de Barras lido na camera: ${session[0]}`);
				        barcodeCallback(session);
					},
					null,
					"ScanditSDK",
					"scan",
					[
						"6QQhFpy3HzyVbB8vChDre3IH1H/qhFQThxtnX9wgdLs",
						opts
					]
				);
			}
		};

		/**
		 * Religa a camera usando a função de callback definida na inicialização.
		 *
		 */
		barcodeScan.reInitScan = function() {
			if (barcodeScan.callBackFunction) {
				barcodeScan.initScan(barcodeScan.callBackFunction);
			}
		};

		barcodeScan.scanActivated = function() {
            return barcodeScan.scanActive;
        };

		barcodeScan.stop = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "stop", []);
			}
		};

		barcodeScan.start = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "start", []);
			}
		};

		barcodeScan.pause = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "pause", []);
			}
		};

		barcodeScan.resume = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "resume", []);
			}
		};

		barcodeScan.resizeIn = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "resize", [{
					"portraitMargins": barcodeScan.inSizeCam,
					"animationDuration": 0.1
				}]);
				if (barcodeScan.isAndroid) {
					barcodeScan.stop();
					barcodeScan.start();
				}
			}
		};

		barcodeScan.resizeOut = function() {
			if (barcodeScan.scanActive) {
				cordova.exec(null, null, "ScanditSDK", "resize", [{
					"portraitMargins": barcodeScan.outSizeCam,
					"animationDuration": 0.1
				}]);
			}
		};

		barcodeScan.init = function(barcodeCallback, params) {
			barcodeScan.scanActive = false;

			barcodeScan.isAndroid = devicePlatform === "Android";
			barcodeScan.inSizeCam = barcodeScan.isAndroid ? "0/44/0/300" : "0/64/0/300";
			barcodeScan.outSizeCam = barcodeScan.isAndroid ? "275/44/0/300" : "275/64/0/300";

			// para as telas que exigem tratamento específico pelo modelo do device
			if (params && params.screenName) {
				const rect = barcodeScan.createRect(devicePlatform, device.model, params.screenName);
				if (rect) {
					params.inSizeCam = rect.inSizeCam;
					params.outSizeCam = rect.outSizeCam;
				}
				
			}
			if (params && params.inSizeCam) {
                barcodeScan.inSizeCam = barcodeScan.isAndroid ? params.inSizeCam[0] : params.inSizeCam[1];
			}
			if (params && params.outSizeCam) {
                barcodeScan.outSizeCam = barcodeScan.isAndroid ? params.outSizeCam[0] : params.outSizeCam[1];
            }
			barcodeScan.callBackFunction = barcodeCallback;
			barcodeScan.initScan(barcodeScan.callBackFunction, params.extraOpts);
		};

		barcodeScan.createRect = function(platform, deviceModel, screenName) {
			let rect = null;
			switch (screenName) {
				case 'popupSearchByUUI.html': 
					const androidInSizeCam = (platform === 'Android' && deviceModel === 'SM-T575' ? '30/280/30/270' : '20/250/20/220');
					rect = {
						inSizeCam: [androidInSizeCam, '55/420/55/390'],
						outSizeCam: ['275/110/20/100', '275/80/20/80'],
					};
					break;
			}
			return rect;
		}

		return barcodeScan;
	});
})();
