(function() {
    'use strict';

    angular.module('easyorder.services.billing.payment', [
        'ngResource',
        'easyorder.services.log'
    ])

    .factory('BillingPaymentService', function($resource, 
            ConfigService,
            UserService,
            CustomerService) {
        
         var service = {};
         
         service.SEARCH_TYPE_CREDIT_BOOK = 'CREDIT_BOOK';
         service.SEARCH_TYPE_CARD_STATEMENT = 'CARD_STATEMENT';
         service.SEARCH_TYPE_PROCESSOR_TOTVS = 'TOTVSCARD';
         service.SEARCH_TYPE_PROCESSOR_CONDUCTOR = 'CONDUCTOR';
        
        /**
         *
         */
        service.getURL = () => {
            let url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
            url += '/linx-webservices/v1/billing-payment';
          
            return url;
        };
        
        /**
         *
         */
        service.search = (params, success,  error) => {
            params.companyNumber = params.companyNumber || ConfigService.config.companyNumber;
            params.storeNumber = params.storeNumber || ConfigService.config.storeNumber;
            params.terminalNumber =  params.terminalNumber || ConfigService.config.terminalNumber
            params.nsu = params.nsu || ConfigService.nsuControll.currentNSU();
            params.operatorCode = params.operatorCode || UserService.operatorCode();
            
            return $resource(service.getURL() + "/search", {}, {
                process: {
                    method: 'POST'
                }
            }).process({}, params, success,  error).$promise;
        };
        
        /**
         *
         */
        service.integrate = (request) => {
            return $resource(service.getURL() + "/integrate", {}, {
                process: {
                    method: 'POST'
                }
            }).process({}, request).$promise;
        };
        
        /**
         *
         */
        service.confirmTransaction = (request) => {
            return $resource(service.getURL() + "/confirm", {}, {
                process: {
                    method: 'POST'
                }
            }).process({}, request).$promise;
        };
        
        /**
         *
         */
        service.getCustomer = (document) => {
            if (document) {
                return CustomerService.searchByType(document, 'CPF')
                .then(result => {
                    if (result.length) {
                        return {
                            cardNumber: result[0].cardNumber,
                            document: result[0].document,
                            documentType: result[0].documentType,
                            email: result[0].email,
                            name: result[0].name,
                            address: result[0].address
                        };
                    }
                });
            } else {
                return Promise.reject("Documento não informado.");
            }
        };
        
        /**
         *
         */
        service.filterConductorCardStatements = (bills) => {
            const ret = [];
            bills.forEach(data => {
                ret.push(... (data.items.filter(b => b.type === 'CARD_STATEMENT' 
                    && b.processorCode === service.SEARCH_TYPE_PROCESSOR_CONDUCTOR)));
            });
            return ret;
        };

        return service;
    });
})();