(function() {
    'use strict';

    angular.module('easyorder.services.biometrics', [
		'ngResource',
		'easyorder.services.config',
		'easyorder.services.log'
	])

    .factory('BiometricsService', function($http, ConfigService,$q) {
        const HTTP = 'http://';
        const HTTPS = 'https://';

        const SERVICE = '/linx-webservices/v1/biometrics';
        const FACE_RECOGNITION = '/face/recognition';

		const service = {};
        
        function url(action) {
            const lws = ConfigService.config.linxwebservices;
            return `${protocol()}${lws.host}:${lws.port}${SERVICE}${action}`;
        }

        function protocol() {
            const lws = ConfigService.config.linxwebservices;
            return lws.secure ? HTTPS : HTTP;
        }
		
		service.faceRecognition = (params) => {
			return $http.post(url(FACE_RECOGNITION), params);
		};

        service.getCustomerFaceImage = (customer) => {
            return customer && customer.biometrics && customer.biometrics.faceImage ? customer.biometrics.faceImage : null;
		};

        service.isBiometricPaymentMethod = (paymentMethod) => {
            const biometricsPaymentMethods = ConfigService.config.business.biometric_payment_methods;
            return paymentMethod && biometricsPaymentMethods && biometricsPaymentMethods.includes(paymentMethod);
        };

        service.isCustomerBiometricsEnabled = (customer) => {
            return customer != null && customer.biometrics != null && customer.biometrics.enabled === true;
        };

        service.hasCustomerBiometricsInfo = (customer) => {
            return customer != null && customer.biometrics != null;
        };
		return service;

    })
})();