
(function() {
    'use strict';

    angular.module('easyorder.services.bluebird.capture', [])
    .factory('BluebirdCaptureService', function($q, $timeout, LogService) {
        var service = {
            registered: false,
            sledTrigger: {
                status: "TRIGGER_RELEASED" //TRIGGER_RELEASED, TRIGGER_PRESSED
            }
        };
        
        class PluginCallbackManager {
            constructor(successCallback, errorCallback) {
                this.mainSuccessCallback = successCallback;
                this.maniErrorCallback = errorCallback;
                this.commandPromiseChain = $q.resolve();
            }        
            success(result) {
                result = JSON.parse(result);
                
                if ("reading" == result.command) {
                    this.commandPromiseChain = this.commandPromiseChain.finally(() => this.mainSuccessCallback(JSON.parse(result.data)));
                } else if ("power" == result.command) {
                    if (this.powerChangeDeferedResult) {
                        this.powerChangeDeferedResult.resolve();
                    } else {
                        LogService.warn(`Não há promessa configurada para mudança de potência do SLED.`);
                    }
                } else if ("changeEPCHeader" == result.command) {
                    this.commandPromiseChain = this.commandPromiseChain.finally(() => this.mainSuccessCallback(JSON.parse(result.data)));
                } else if ("triggerEvent" == result.command) {
                    $timeout(() => service.sledTrigger.status = result.data);
                    LogService.debug(`SLED: ${result.data}`);
                }
            }
            error(error) {
                error = JSON.parse(error);
                
                if ("reading" == error.command) {
                    this.commandPromiseChain = this.commandPromiseChain.finally(() => this.maniErrorCallback(error));
                } else if ("power" == error.command) {
                    if (this.powerChangeDeferedResult) {
                        this.powerChangeDeferedResult.reject(error);
                    } else {
                        LogService.warn(`Não há promessa configurada para mudança de potência do SLED.`);
                    }
                } else if ("sledRemoved" == error.command) {
                    LogService.error(error.message);
                    service.registered = false;
                } else if ("changeEPCHeader" == error.command) {
                    this.commandPromiseChain = this.commandPromiseChain.finally(() => this.maniErrorCallback(error));
                }
            }
            initPowerChangeDeferedResult() {
                return this.powerChangeDeferedResult = $q.defer();
            }
        }
        /**
         *
         */
        service.androidBarcodeConnect = () => {
            LogService.info(`Iniciando dispositivo android para leitura de códigos de barras...`);
            let args= [window.plugins.bluebirdConnector.Type.ANDROID_SCANNER];
            
            if (window.plugins.bluebirdConnector) {
                return $q((resolve, reject) => window.plugins.bluebirdConnector.connect(resolve, reject, args))
                .then(() => service.registered = true);
            } else {
                return $q.reject('{"command":"connect","errorCode":"-32","message":"Não foi possível conectar ao dispositivo."}');
            }
        };
        /**
         *
         */
        service.barcodeConnect = () => {
            LogService.info(`Iniciando conexão com o SLED-SCANNER...`);
            let args= [window.plugins.bluebirdConnector.Type.SLED_SCANNER];
            service.sledTrigger.status = "TRIGGER_RELEASED";
            
            if (window.plugins.bluebirdConnector) {
                return $q((resolve, reject) => window.plugins.bluebirdConnector.connect(resolve, reject, args))
                .then(() => service.registered = true);
            } else {
                return $q.reject('{"command":"connect","errorCode":"-32","message":"Não foi possível conectar ao SLED"}');
            }
        };
        /**
         *
         */
        service.connect = (power) => {
            power = power || 5;
            power = power < 5 ? 5 : power;
            power = power > 30 ? 30 : power;
            service.sledTrigger.status = "TRIGGER_RELEASED";
            
            LogService.info(`Iniciando conexão com o SLED-RFID potência '${power}'`);
            let args= [window.plugins.bluebirdConnector.Type.SLED_RFID, power];
            
            if (window.plugins.bluebirdConnector) {
                return $q((resolve, reject) => window.plugins.bluebirdConnector.connect(resolve, reject, args))
                .then(() => service.registered = true);
            } else {
                return $q.reject('{"command":"connect","errorCode":"-32","message":"Não foi possível conectar ao SLED"}');
            }
        };
        /**
         *
         */
        service.startReading = (successCallback, errorCallback) => {
            LogService.info(`Habilitando leitura...`);
            service.callbackManager = new PluginCallbackManager(successCallback, errorCallback);
            let initResultPromise = $q.defer();
            
            window.plugins.bluebirdConnector.startReading(
                result => {
                    if (initResultPromise.promise.$$state.status === 0) {
                        initResultPromise.resolve(result);
                    } else {
                       service.callbackManager.success(result);
                    }
                },
                error => {
                    if (initResultPromise.promise.$$state.status === 0) {
                        initResultPromise.reject(error);
                    } else {
                        service.callbackManager.error(error);
                    }
                }, []);
            return initResultPromise.promise;
        };
        /**
         *
         */
        service.changeEPCHeader = (epcs, changeTo, successCallback, errorCallback) => {
            if (!Array.isArray(epcs)) {
                throw `Os EPCs devem ser uma lista (array)!`;
            }
            LogService.info(`Iniciando mudança de cabeçalho de tags com o SLED...`);
            service.callbackManager = new PluginCallbackManager(successCallback, errorCallback);
            let initResultPromise = $q.defer();
            
            window.plugins.bluebirdConnector.changeEPCHeader(
                result => {
                    if (initResultPromise.promise.$$state.status === 0) {
                        initResultPromise.resolve(result);
                    } else {
                       service.callbackManager.success(result);
                    }
                },
                error => {
                    if (initResultPromise.promise.$$state.status === 0) {
                        initResultPromise.reject(error);
                    } else {
                        service.callbackManager.error(error);
                    }
                }, [epcs.join(","), changeTo]);
            return initResultPromise.promise;
        };
        /**
         *
         */
        service.setPower = (value) => {
            if (service.registered) {
                LogService.info(`Potência do SLED para '${value}'...`);
                
                let defer = service.callbackManager.initPowerChangeDeferedResult();
                window.plugins.bluebirdConnector.setPower(service.callbackManager.success, service.callbackManager.error, [value]);
                return defer.promise;
            } else {
                return $q.reject({message:`Potência do SLED não pode ser alterada sem que o serviço esteja inicializado.`});
            }
        };
        /**
         *
         */
        service.disconnect = () => {
            if (service.registered) {
                return $q((resolve, reject) => window.plugins.bluebirdConnector.disconnect(resolve, reject, []))
                .then(() => service.registered = false)
                .then(() => LogService.info(`SLED desconectado.`));
            } else {
                return $q.resolve();
            }
        }
        /**
         *
         */
        service.connected = () => {
            return service.registered;
        }
        return service;
    });
})();
