(function() {
    'use strict';

    angular.module('easyorder.services.cart', [
        'ngResource',
        'easyorder.services.config',
        'easyorder.services.log',
        'easyorder.services.cashback'
    ])

    .factory('CartService', function($rootScope,
                                    $ionicPopup,
                                    $state,
                                    $q,
                                    $ionicLoading,
                                    $resource,
                                    ConfigService,
                                    UserService,
                                    AlertService,
                                    $timeout,
                                    $injector,
                                    ServiceSaleService,
                                    LoyaltyService,
                                    LogService,
                                    CashbackService,
                                    ProductService,
                                    UtilService) {
        var service = {
            cart: {},
            seller: undefined,
            initCartObservers: new Observer()
        };
        $rootScope.processDiscount = null;
        const ONE_FACTOR_DISCOUNT = ["CAMPAING", "EXTERNAL_TOTAL_VALUE"];
        const EXTENDED_WARRANTY = "EXTENDED_WARRANTY";
        const THEFT_FURTH_INSURANCE = "THEFT_FURTH_INSURANCE";

        /**
         *
         */
        function Observer() {
            var observers = [];

            this.add = function(callback) {
                observers.push(callback);
            };
            this.remove = function(callback) {
                for (var i = 0; i < observers.length; i++) {
                    if (observers[i] === callback) {
                        delete(observers[i]);
                    }
                }
            };
            this.notify = function () {
                for (var i = 0; i < observers.length; i++) {
                    if (typeof observers[i] === 'function') {
                        observers[i]();
                    }
                }
            };
            this.clear = function() { // Chamado no tratamento de mudança de estados no HomeCtrl.
                observers.length = 0;
            };
        };

        var dispatchErrorCallback = function (errorCallback, message) {
            if (errorCallback) {
				var error = {
					message: message
				}
				errorCallback(error);
			}
			return false;
        };

        var isItemValid = function (item, destinationStoreNumber, errorCallback) {
        	if (service.getCurrentOperation() === 'SERVICE') {
                    var message = "Já existem itens de serviço adicionados! Item não pode ser adicionado!";
                    return dispatchErrorCallback(errorCallback, message);
        	} else {
    			var currentDestinationStoreNumber = service.cart.destinationStoreNumber;
    			// se já houver ítem para loja externa e for diferente da loja de destino do ítem a ser adicionado
    			if (currentDestinationStoreNumber && currentDestinationStoreNumber !== destinationStoreNumber) {
    				var message = "Já existe um(a) '" + item.productDescription + "' no carrinho para outra loja: " + currentDestinationStoreNumber + "! <br> Portanto, o produto não poderá ser adicionado para outra loja!";
    				return dispatchErrorCallback(errorCallback, message);
    			} else {
				 	var isShowrooming = (item.showrooming === true);

				 	if (isShowrooming && 'ASSORTED' === item.productType) {
				 		var message = "Ítem '" + item.productDescription + "' não pode ser vendido via showrooming! <br> Portanto, o produto não poderá ser adicionado para outra loja!";
	    				return dispatchErrorCallback(errorCallback, message);
				 	} else if (!isShowrooming && 'ECOMMERCE' === item.productType) {
				 		var message = "Ítem '" + item.productDescription + "' só pode ser vendido via showrooming! <br> Portanto, o produto não poderá ser adicionado para outra loja!";
	    				return dispatchErrorCallback(errorCallback, message);
				 	} else {
				 		var fiscalItems = service.cart.fiscalItems;
				 		if (fiscalItems && fiscalItems.length > 0) {
				 			var message;
				 			fiscalItems.forEach(function(currentItem) {
				 				if (currentItem.productCode === item.productCode) {
				 					var currentItemShowroomimg = (currentItem.showrooming === true);
				 					if (currentItemShowroomimg !== isShowrooming) {
				 						if (isShowrooming) {
				 							message = "Já existe um(a) '" + item.productDescription + "' no carrinho para venda auto-serviço! <br> Portanto, o produto não poderá ser adicionado para Showrooming!";
				 						} else {
				 							message = "Já existe um(a) '" + item.productDescription + "' no carrinho para Showrooming! <br> Portanto, o produto não poderá ser adicionado para venda auto-serviço!";
				 						}

				 					}
				 				}
				 			});
				 			if (message) {
				 				return dispatchErrorCallback(errorCallback, message);
				 			}

				 		}
				 	}
    			}
        	}
			return true;
        };

        /**
         * Verifica se o produto possui um código de mercadológico válido para inclusão de itens avulsos em resgate de pedidos
         *
         * @param {Item do carrinho} item
         * @returns TRUE -> Para itens válidos | FALSE para itens inválidos
         */
        service.isAValidMarketingCode = (item, ignoreOrderItems) => {
            if(!ConfigService.config.business.enables_control_of_single_items_by_marketing || item.orderNumber) {
                return true;
            }
            const hasValids = ConfigService.config.business.marketing_codes_to_control_the_items_inclusion.filter(code => {
                if(code.includes('*')) {
                    const markWithoutAsterisc = code.replace('.*', '');
                    if(item.marketingCode && item.marketingCode.indexOf(markWithoutAsterisc) == 0) {
                        return true;
                    }
                    return false;
                } else {
                    return item.marketingCode == code;
                }
            })

            const noOrderItems = ignoreOrderItems ||
                                            (service.cart.fiscalItems.filter(cartItem => cartItem.orderNumber != null && cartItem.orderNumber != undefined).length == 0
                                            && service.cart.nonFiscalItems.filter(cartItem => cartItem.orderNumber != null && cartItem.orderNumber != undefined).length == 0
                                            && service.cart.serviceItems.filter(cartItem => cartItem.orderNumber != null && cartItem.orderNumber != undefined).length == 0);

            return hasValids.length > 0 || noOrderItems;
        }

        /**
         * Adiciona produto ao carrinho.
         * Produtos adicionados ao carrinho recebem funções para controle de sua totalização.
         */
        service.addItem = (item, successCallback, errorCallback, destinationStoreNumber) => {
            var ret = false;
            if (!service.isAValidMarketingCode(item, false)) {
                AlertService.info("Produto selecionado não permitido para inclusão!");
            } else if(ConfigService.config.business.ignore_mid_settings && !item.showrooming) {
                AlertService.info("O Caixa Rápido não está configurado para este tipo de operação, apenas showrooming é permitido.");
            } else {
                LogService.info('Adicionando item no carrinho: ', item);

                if (destinationStoreNumber == null) {
                    destinationStoreNumber = service.cart.storeNumber;
                }
                if (ConfigService.config.business.seller_identification_mode === 'venda') {
                    service.registerSellerToSaleItems(service.seller, [item]);
                }

                if (isItemPriceValid(item, errorCallback) && isItemValid(item, destinationStoreNumber, errorCallback)) {
                    service.undoCampaignDiscounts();
                    service.cart.destinationStoreNumber = destinationStoreNumber;

                    const matches = service.cart.fiscalItems.filter(element => {
                        return element.productCode == item.productCode
                            && item.groupCode == element.groupCode // Só é o mesmo item se for do mesmo grupo (kit/cesta).
                            && item.minimumSaleUnit == element.minimumSaleUnit // evitando agrupar itens que o incremento são diferentes
                            && element.orderNumber == null; // não deve ser um item de pedido
                    });

                    item = new CartItem(item, ConfigService);
                    item.discounts = !item.discounts ? [] : item.discounts;

                    //Incrementa o item fiscal
                    let sequenceIdAux = item.sequenceId || getNextSequenceId();
                    item.sequenceId = sequenceIdAux;
                    //Itens nao fiscais associados ao item fiscal sao incrementados manualmente
                    //a partir do sequencial do pai, para nao receberem o mesmo sequencial e dar
                    //problemas ao tentar persistir o dado
                    if (item.nonFiscalItems) {
                        item.nonFiscalItems.forEach((nonFiscalItem) => {
                            nonFiscalItem.sequenceId = ++sequenceIdAux;
                        });
                    }

                    // Adiciona o vendedor ao item automativamente, caso parametro esteja ativo
                    if (ConfigService.config.business.seller_identification_mode == 'item' 
                        && ConfigService.config.business.automatic_seller_identification_enabled === true) {
                        const loggedOperator = {
                            login: UserService.operatorCode(),
                            name: UserService.operatorName(),
                        }  
                        item.seller = loggedOperator;
                        item.salesAssociateNumber = UserService.operatorCode();
                    }


                    const mul = ConfigService.config.business.item_multiplication_enabled;

                    if (matches.length > 0) {
                        const cartItem = matches[0];

                        if (mul) {
                            cartItem.quantitySold = MathUtils.add(cartItem.quantitySold, item.quantitySold, 3);
                            item = cartItem;
                        } else {
                            // não entendi por qual motivo apenas este tipo de desconto é copiado
                            cartItem.discounts.filter(disc => disc.source === 'PRICE_CHANGE').forEach(disc => {
                                item.removeDiscountBySource('PRICE_CHANGE');
                                item.addDiscount(disc);
                            });
                            service.cart.fiscalItems.push(item);
                        }
                    } else {
                        service.cart.fiscalItems.push(item);
                    }

                    item.processTotalValue();
                    service.updateCartTotalValue();

                    if (successCallback) {
                        successCallback(item);
                    }
                    ret = true;
                }
            }
            $rootScope.$emit("cart-item-quantity"); //ver homectrl.

            return ret;
        };

        service.addServiceItem = function(item) {
            if (!service.isAValidMarketingCode(item)) {
                AlertService.info("Produto selecionado não permitido para inclusão!");
            }
            service.cart.serviceItems.push(item);
            service.updateCartTotalValue();

            $rootScope.$emit("cart-item-quantity"); //ver homectrl.

            return true;
        };

        service.addDiscount = function(discount) {
            var discounts = service.cart.discounts;

            if (!discounts) {
                service.cart.discounts = [];
            }

            discount.sequenceId = service.cart.discounts.length;
            service.cart.discounts.push(discount);
        };

        service.removeDiscountBySource = (discount) => {
            var discounts = service.cart.discounts;

            if (discounts) {
                for (var i = 0; i < discounts.length; i++) { // Removendo desconto já existente.
                    if (discounts[i].source === discount.source) {
                        discounts.splice(i--, 1);
                    }
                }
            }
        }

        /**
         * Valida o preço do item
         */
        var isItemPriceValid = function(item, errorCallback) {
            var ret = true;

            if (!item.unitValue || item.unitValue <= 0) {
                let message = `O preço do item '${item.productDescription}' é inválido.`;
                ret = dispatchErrorCallback(errorCallback, message);
            } else {
                ret = isSamePriceOrder(item, errorCallback);
            }
            return ret;
        };

        /**
         * TODO: O CartService precisa ser reestruturado.
         */
        service.incrementInCart = function(item) {
            var currItem = JSON.parse(JSON.stringify(item));
            if (currItem.productType === 'MINIMUM_SALE_UNIT') {
                currItem.quantitySold = item.minimumSaleUnit;
            } else {
                currItem.quantitySold = 1;
            }

            service.addItem(currItem);
            service.updateInCart(item);
            $rootScope.$emit("cart-item-quantity"); //ver homectrl.
        };

        service.updateInCart = function(item, isIncrement) {
            if (isIncrement){
                var currItem = JSON.parse(JSON.stringify(item));
                currItem.quantitySold = Number(item.quantitySold);
                service.addItem(currItem, null, null, null);
            }else{
                service.undoCampaignDiscounts();
                item.processTotalValue();
                service.updateCartTotalValue();
            }
        };

        service.decrementInCart = function(item) {
            service.undoCampaignDiscounts();
            if ((item.productType != 'MINIMUM_SALE_UNIT' && item.quantitySold == 1)
                    || (item.productType == 'MINIMUM_SALE_UNIT' && item.quantitySold == item.minimumSaleUnit)) {
                let index = service.cart.fiscalItems.indexOf(item);
                if (index > -1) {
                    service.removeItem(index);
                }
            } else {
                if (item.productType == 'MINIMUM_SALE_UNIT') {
                    item.quantitySold = MathUtils.subtract(item.quantitySold, item.minimumSaleUnit, 3);
                } else {
                    item.quantitySold--;
                }
                item.processTotalValue();
            }


            service.updateCartTotalValue();
            $rootScope.$emit("cart-item-quantity"); //ver homectrl.
        };

        service.updateQuantitySoldInCart = function(item) {
            service.undoCampaignDiscounts();
            item.processTotalValue();
            service.updateCartTotalValue();
        };

        service.updateCartTotalValue = () => {
            LogService.info("Iniciando atualização do Total do Carrinho...");
            service.cart.totalValue = 0;
            service.cart.cashChange = 0;

            if (service.cart.fiscalItems) {
                service.cart.fiscalItems.forEach((item) => {
                    const itemValue = item.processedUnitValue();
                    service.cart.totalValue = parseFloat((service.cart.totalValue + itemValue).toFixed(2));
                    LogService.info("Valor do item fiscal adicionado ao total: ["  + itemValue + "]");
                    LogService.info("Total após adição: [" + service.cart.totalValue + "]");
                    if (item.nonFiscalItems) {
                        item.nonFiscalItems.forEach((nonFiscalItem) => {
                            service.cart.totalValue = parseFloat((service.cart.totalValue + nonFiscalItem.value).toFixed(2));
                            LogService.info("Valor do item não fiscal adicionado ao total: ["  + nonFiscalItem.value + "]");
                            LogService.info("Total após adição: [" + service.cart.totalValue + "]");
                        });
                    }
                });
            }

            if (service.cart.serviceItems) {
                service.cart.serviceItems.forEach((item) => {
                    service.cart.totalValue = parseFloat((service.cart.totalValue + item.value).toFixed(2));
                    LogService.info("Valor do item de serviço adicionado ao total: ["  + item.value + "]");
                    LogService.info("Total após adição: [" + service.cart.totalValue + "]");
                });
            }

            if (service.cart.nonFiscalItems) {
                service.cart.nonFiscalItems.forEach((item) => {
                    service.cart.totalValue = parseFloat((service.cart.totalValue + item.value).toFixed(2));
                    LogService.info("Valor do item não fiscal adicionado ao total: ["  + item.value + "]");
                    LogService.info("Total após adição: [" + service.cart.totalValue + "]");
                });
            }

            service.cart.totalValue = parseFloat(service.cart.totalValue.toFixed(2));
            LogService.info("Total após arredondamento: [" + service.cart.totalValue + "]");

            service.processCouponDiscounts();

            LogService.info("Finalizado atualização do Total do Carrinho, Total: [" + service.cart.totalValue + "]");
        };

        service.processCouponDiscounts = () => {
            // inicio modificação (calcular descontos de itens e cupons)
            const total = service.cart.fiscalItems.reduce((a, b) => a + b.processedUnitValue(), 0);

            service.cart.totalDiscount = 0; //Reiniciando

            const discounts = service.cart.discounts;
            service.cart.totalDiscount = 0;

            if (discounts !== 'undefined' && discounts != null && discounts.length > 0) {
                LogService.info("Adicionando descontos de cupom: ", JSON.stringify(discounts));

                discounts.forEach((discount) => {
                    let value = discount.type && discount.type == "PERCENT" ? (discount.value * total) / 100 : discount.value;
                    value = value <= service.cart.totalValue ? value : service.cart.totalValue; // Descontar até o valor do carrinho, no máximo.
                    value = parseFloat(value.toFixed(2));

                    service.cart.totalDiscount = parseFloat((service.cart.totalDiscount + value).toFixed(2));
                    service.cart.totalValue    = parseFloat((service.cart.totalValue - value).toFixed(2));
                    LogService.info("Valor do Desconto de Cupom adicionado ao total: ["  + value + "]");
                    LogService.info("Total de Descontos de Cupom: [" + service.cart.totalDiscount + "]");
                    LogService.info("Total após adição: [" + service.cart.totalValue + "]");
                });

                service.cart.totalValue = parseFloat(service.cart.totalValue.toFixed(2));
                LogService.info("Total após arredondamento: [" + service.cart.totalValue + "]");
            }
        };

        function getAttachedNonFiscalItemsCount(index) {
            let result = service.cart.fiscalItems[index].nonFiscalItems ? service.cart.fiscalItems[index].nonFiscalItems.length : 0;
            if (index > 0) {
                result += getAttachedNonFiscalItemsCount(index - 1);
            }
            return result;
        }

        function getNextSequenceId() {
            const fiscalItemsCount = service.cart.fiscalItems ? service.cart.fiscalItems.length : 0;
            const nonFiscalItemsCount = service.cart.nonFiscalItems ? service.cart.nonFiscalItems.length : 0;
            const attachedNonFiscalItemsCount = fiscalItemsCount > 0 ? getAttachedNonFiscalItemsCount(fiscalItemsCount - 1) : 0;
            return fiscalItemsCount + nonFiscalItemsCount + attachedNonFiscalItemsCount + 1;
        }

        service.reindexCartItems = function () {
            let index = 1;
            //fiscal items
            if (service.cart.fiscalItems) {
                service.cart.fiscalItems.forEach((FiscalItem) => {
                    FiscalItem.sequenceId = index;
                    index++;
                    //attached non fiscal items
                    if (FiscalItem.nonFiscalItems) {
                        FiscalItem.nonFiscalItems.forEach((nonFiscalItem) => {
                            nonFiscalItem.sequenceId = index;
                            index++;
                        });
                    }
                });
            }
            //non fiscal items
            if (service.cart.nonFiscalItems) {
                service.cart.nonFiscalItems.forEach((nonFiscalItem) => {
                    nonFiscalItem.sequenceId = index;
                    index++;
                });
            }
        }

        /**
         *
         */
        service.removeItem = function(itemIndex) {
            if (service.getCurrentOperation() === 'SERVICE') {
                var item = service.cart.serviceItems[itemIndex];

                if (item) {
                    service.cart.serviceItems.splice(itemIndex, 1);
                    service.updateCartTotalValue();
                }
            } else {
                var item = service.cart.fiscalItems[itemIndex];

                if (item) {
                    service.cart.fiscalItems.splice(itemIndex, 1);
                    service.updateCartTotalValue();
                }
                service.reindexCartItems();
            }
            if (!service.hasItems()) {
                service.cart.destinationStoreNumber = undefined;
            }
            $rootScope.$emit("cart-item-quantity"); //ver homectrl.
        };

       /**
        * Remove o item passado e todos os itens do carrinho que tenham o mesmo
        * agrupamento, caso estejam em um.
        */
       service.removeItemsByItemGroup = function(itemIndex) {
           var cartItems = service.cart.fiscalItems.length > 0 ? service.cart.fiscalItems : service.cart.serviceItems;
           var groupCode = cartItems[itemIndex].groupCode;

           service.removeItem(itemIndex);

           if (groupCode) {
               var removeItems = cartItems.filter(function(item) {
                  return item.groupCode ? item.groupCode === groupCode : false;
               });
               removeItems.forEach(function(itemRev) {
                   for (var i = 0; i < cartItems.length; i++) {
                       if (cartItems[i].productCode
                               && cartItems[i].productCode === itemRev.productCode
                               && cartItems[i].groupCode === itemRev.groupCode) {
                           service.removeItem(i);
                           break;
                       }
                   }
               });
           }
       };

        /**
         *
         */
        service.updateItemPrice = function(item, newPrice, reason, permissions, newLineItemTaxes) {
            var discountObj = undefined;

            service.undoCampaignDiscounts();
            service.removeCustomerRelatedDiscounts();

            try {
                var priceOverride = ConfigService.config.business.price_override === true;
                if (!priceOverride && item.unitValue - newPrice > 0) {
                    discountObj = {
                        value:  item.unitValue - newPrice,
                        source: 'PRICE_CHANGE',
                        factor: undefined
                    };
                } else {
                    // se o preço original já foi setado é porque já houve alteração de preço
                    item.originalPrice = item.originalPrice ? item.originalPrice : item.unitValue;
                    item.unitValue = newPrice;
                    if (reason) {
                       item.priceChangeReason   = reason;
                    }

                    if (newLineItemTaxes) {
                        item.lineItemTaxes = newLineItemTaxes;
                    }
                }
                if (discountObj) {
                    service.cart.fiscalItems.filter(element => element.sequenceId == item.sequenceId)
                        .forEach(fiscalItem => {
                            fiscalItem.removeDiscountBySource('PRICE_CHANGE');
                            fiscalItem.addDiscount(discountObj);
                            fiscalItem.unitValue = item.unitValue; //atualizando o valor para todos os itens já presentes no carrinho.
                            fiscalItem.processTotalValue();
                        });
                } else {
                    service.cart.fiscalItems.filter(element => element.sequenceId == item.sequenceId)
                        .forEach(fiscalItem => {
                            fiscalItem.removeDiscountBySource('PRICE_CHANGE');
                            // não deve ser alterado para todos
                            // TODO parametrizar
                            fiscalItem.processTotalValue();
                        });
                }
                service.updateCartTotalValue();

                for (var attrname in permissions) {
                    service.cart.changeAuthorizations[attrname] = permissions[attrname];
                }
            } finally {
                service.applyLoyaltyDiscount()
                    .finally(() =>	service.processCampaigns())
                    .finally(() => {
                        service.applyEmployeeDiscount();
                    });
            }
        };


        /**
         *
         */
        service.isCartValid = function() {
            return service.hasItems() && !service.customerValidationWarn() &&
                (!service.isRemoteOrder() ? true : service.cart.customer && service.cart.customer.name && service.cart.customer.document);
        };

        service.validateMandatoryParameter = function() {
            var mandatoryParameterFromServer = ConfigService.config.business.customer_mandatory_fields;
			if(mandatoryParameterFromServer == "DEFAULT"){
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
            } else if ($scope.mandatoryParameterFromServer == "SET1"){
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryzipCode = true;
                $scope.mandatoryaddressName = true;
                $scope.mandatoryaddressNumber = true;
                $scope.mandatoryneighborhood = true;
                $scope.mandatorycity = true;
                $scope.mandatoryState = true;
            } else if ($scope.mandatoryParameterFromServer == "SET2"){
                var message = "";
                if (!service.cart.customer.document){
                    message = "Para continuar a operação é necessário informar o documento do cliente.";
                    return dispatchErrorCallback(errorCallback, message);
                }
                if (!service.cart.customer.email){
                    message = "Para continuar a operação é necessário informar o email do cliente.";
    	    		return dispatchErrorCallback(errorCallback, message);
                }
            } else if ($scope.mandatoryParameterFromServer == "SET3"){
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryMail = true;
            } else if ($scope.mandatoryParameterFromServer == "SET4") {
                $scope.mandatoryDocument = true;
                $scope.mandatoryName = true;
                $scope.mandatoryMail = true;
                $scope.mandatoryPhone = true;
            } else if ($scope.mandatoryParameterFromServer == "DOCUMENT") {
                $scope.mandatoryDocument = true;
            }
		};

        /**
         *
         */
        service.loadSettings = function() {
              service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
              service.url += ConfigService.config.linxwebservices.host + ':';
              service.url += ConfigService.config.linxwebservices.port;
              service.url += '/linx-webservices';
        };

        /**
         *
         */
        service.processCampaigns = function(paymentMethod, paymentValue, payments) {
            var deferred = $q.defer();

            if (!service.cart.orderRetrieved) {
                if (!$rootScope.promotionEngine) {
                    if ($rootScope.campaignEnabled) {
                        var fiscalItems = service.cart.fiscalItems;

                        if (fiscalItems && fiscalItems.length) {
                            $ionicLoading.show({
                                template : 'Por favor, aguarde...',
                                duration : 0
                            }).then(function() {
                                var success = function(response) {
                                    $ionicLoading.hide();

                                    if (typeof response !== 'undefined' && response != null) {
                                        service.applyCampaignsDiscounts(response.promotions);
                                    }
                                };
                                var error = function(response) {
                                    $ionicLoading.hide();

                                    $ionicPopup.alert({
                                        title: "Processamento campanhas",
                                        template: response.data.message,
                                        okType: "button-theme"
                                    });
                                };
                                service.executeCampaigns(success, error).then(deferred.resolve, deferred.reject); //Fazendo com que a promessa da campanha controle a promessa retornada pelo método.
                            });
                        } else {
                            deferred.resolve(); //Marcando como sucesso.
                        }
                    } else {
                        deferred.resolve(); //Marcando como sucesso.
                    }
                } else {
                    service.updateDiscountCart(paymentMethod, paymentValue, payments).then(deferred.resolve, deferred.reject);
                }
            } else {
                deferred.resolve(); //Marcando como sucesso.
            }
            return deferred.promise;
        };

        /**
         *
         */
        service.executeCampaigns = function(successCallback, errorCallback){
            if (ConfigService.config.business.additional_services) {
                let cloneCart = ServiceSaleService.prepareServices(service.cart);
                var formData = {
                    customer : cloneCart.customer,
                    salePayments : cloneCart.salePayments,
                    fiscalItems : cloneCart.fiscalItems,
                    totalValue : cloneCart.totalValue,
                    storeNumber : cloneCart.storeNumber
                };
            } else {
                var formData = {
                    customer : service.cart.customer,
                    salePayments : service.cart.salePayments,
                    fiscalItems : service.cart.fiscalItems,
                    totalValue : service.cart.totalValue,
                    storeNumber : service.cart.storeNumber
                };
            }
            service.loadSettings();
            service.url += '/v1/campaign';

            var campaignService = $resource(service.url, {}, {
                'process': {method: 'POST'}
            });
            return campaignService.process({}, formData, successCallback, errorCallback).$promise;
        };

        /** Desfaz uso do cashback */
        service.undoUseCashbackDiscount = function() {
            return new Promise((resolve, reject) => {
                service.cart.discounts.forEach(function(disc, index, array){
                    if (disc.source === "CASHBACK"){
                        var cancelRequest = {
                            cashbackAuthorization: {},
                            customer: {}
                        };
                        cancelRequest.cashbackAuthorization.saleId = service.cart.usedCashback.saleId;
                        cancelRequest.cashbackAuthorization.baseValue = service.cart.totalValue;
                        cancelRequest.cashbackAuthorization.cashbackAmount = service.cart.usedCashback.value;
                        cancelRequest.customer = service.cart.usedCashback.customer;

                        CashbackService.cancelCashbackAuthorizationRequest(cancelRequest)
                        .then(() => {
                            LogService.info("Uso de cashback desfeito com sucesso para o identificador ::  " + cancelRequest.cashbackAuthorization.saleId);
                            resolve();
                        })
                        .catch(err => {
                            LogService.error(err);
                            reject(err)
                        });
                    }
                });
            })
        }

        service.undoCashbackDiscountPartial = async () => {
            if (service.cart.usedCashback) {
                try {
                    await service.undoUseCashbackDiscount();
                } catch (error) {
                    AlertService.restError(error, 'Não foi possível fazer o cancelamento o saldo cashback.')
                }
            }

            if (service.cart.discounts) {
                service.cart.discounts.forEach((disc, index) => {
                    if (disc.source === "CASHBACK") {
                        service.cart.discounts.splice(index);
                    }
                });
            }
            service.updateCartTotalValue();
        };

        service.undoPlanDiscount = () => {
            if (service.cart.discounts) {
                service.cart.discounts.forEach((disc, index) => {
                    if (disc.source === "PLAN") {
                        service.cart.discounts.splice(index,1);
                    }
                });
            }
        }

        service.undoFirtsPurchaseDiscount = async () => {
            if (service.cart.discounts) {
                service.cart.discounts.forEach((disc, index) => {
                    if (disc.source === "FIRST_PURCHASE") {
                        service.cart.discounts.splice(index);
                    }
                });
            }
            service.updateCartTotalValue();
        };

        /**
         * Essa função DEVE desfazer APENAS o "impacto" associado à campanha passada como parâmetro!!
         */
        service.undoCampaignDiscountsPartial = function(campaign) {
            if ($rootScope.campaignEnabled) {
                var totalPaymentDiscountUnmake = 0;
                var totalGiftsUnamke = 0;
                var fiscalItems = service.cart.fiscalItems;

                if (fiscalItems) {
                    fiscalItems.forEach(fiscalItem => {
                        var discounts = fiscalItem.discounts;

                        if (discounts && discounts.length) {
                            for (var i = 0; i < discounts.length; i++) {
                                if (discounts[i].source === 'CAMPAING' && discounts[i].indentifierCode == campaign.codeCampaign) {
                                    totalPaymentDiscountUnmake = totalPaymentDiscountUnmake + discounts[i].value;
                                    discounts.splice(i--, 1);
                                    fiscalItem.processTotalValue();
                                }
                            }
                        }
                    });
                }

                //Desfaz descontos no carrinho
                var couponDiscounts = service.cart.discounts;
                if (couponDiscounts && couponDiscounts.length) {
                    for(var j = 0; j < couponDiscounts.length; j++) {
                        if (couponDiscounts[j].source === 'CAMPAING' && couponDiscounts[j].indentifierCode == campaign.codeCampaign) {
                            var discountValue       = couponDiscounts[j].value;
                            totalPaymentDiscountUnmake = totalPaymentDiscountUnmake + discountValue;
                            service.cart.totalValue = service.cart.totalValue + discountValue;
                            couponDiscounts.splice(j--, 1);
                            service.cart.discounts = couponDiscounts;
                        }
                    }
                }
                service.cart.campaignPaymentTotalDiscount = service.cart.campaignPaymentTotalDiscount - totalPaymentDiscountUnmake;

                //Desfazer promocao
                var campaigns = service.cart.promotions;
                if (campaigns && campaigns.length) {
                    for (var i = 0; i < campaigns.length; i++) {
                         if (campaigns[i].codeCampaign == campaign.codeCampaign) {
                            campaigns.splice(i, 1);
                            break;
                        }
                    }
                }
                service.updateCartTotalValue();
            }
        };

        service.undoCampaignDiscounts = function(){
            if ($rootScope.campaignEnabled || $rootScope.promotionEngine) {
                 var fiscalItems = service.cart.fiscalItems;

                 if (fiscalItems) {
                     fiscalItems.forEach(fiscalItem => {
                         var discounts = fiscalItem.discounts;

                         if (discounts && discounts.length) {
                             for (var i = 0; i < discounts.length; i++) {
                                 if (discounts[i].source === 'CAMPAING') {
                                     discounts.splice(i--, 1);
                                     fiscalItem.processTotalValue();
                                 }
                             }
                         }
                     });
                 }
                 var couponDiscounts = service.cart.discounts;

                 if (couponDiscounts && couponDiscounts.length) {
                     for (var j = 0; j < couponDiscounts.length; j++) {
                         if (couponDiscounts[j].source === 'CAMPAING') {
                             var discountValue       = couponDiscounts[j].value;
                             service.cart.totalValue = service.cart.totalValue + discountValue;
                             couponDiscounts.splice(j--, 1);
                             service.cart.discounts = couponDiscounts;
                         }
                     }
                 }
                 service.cart.campaignPaymentTotalDiscount = 0;
                 service.cart.promotions = [];
                 service.cart.campaignsPayment = [];
                 service.cart.giftsQuantity = 0;
                 service.updateCartTotalValue();
            }
        };

        function searchCampaignDiscount(element, campaign) {
            var discounts = element.discounts;
            var discountCampaign = null;

            if (discounts) {
                discounts.forEach(discount => {
                    var source = discount.source;

                    if (source === 'CAMPAING' && discount.indentifierCode == campaign.codeCampaign) {
                        discountCampaign =  discount;
                    }
                });
            }
            return discountCampaign;
        }

        var applyDiscount = function(discount, sequencialItem, campaign) {
    	    if ($rootScope.campaignEnabled) {
    		    var fiscalItems = service.cart.fiscalItems;

    		    if (fiscalItems && discount) {
                    fiscalItems.forEach(fiscalItem => {
                        const sequenceId = fiscalItem.sequenceId;

                        if (sequenceId == sequencialItem) {
                           var campaignDiscount = {};
                           campaignDiscount = searchCampaignDiscount(fiscalItem, campaign);

                           if (!campaignDiscount) {
                                campaignDiscount = {
                                    "source": "CAMPAING",
                                    "type": "VALUE",
                                    "value": discount,
                                    "factor": 1,
                                    "indentifierCode": campaign.codeCampaign
                                };
                               fiscalItem.addDiscount(campaignDiscount);
                           } else {
                               campaignDiscount.value = campaignDiscount.value + discount;
                           }

                           fiscalItem.processTotalValue();
                        }
                    });
                }
    	    }
        };

        var processCampaignCouponDiscount = function(campaign) {
            var appliedDiscount = 0;

            if ($rootScope.campaignEnabled) {
        		if (campaign.couponDiscount) {
                    var couponDiscount = searchCampaignDiscount(service.cart, campaign);

                    if (!couponDiscount) {
                        couponDiscount        = {};
                        couponDiscount.value  = campaign.couponDiscount;
                        couponDiscount.source = 'CAMPAING';
                        couponDiscount.type = 'VALUE';
                        couponDiscount.indentifierCode = campaign.codeCampaign;

                        service.addDiscount(couponDiscount);
                    } else {
                        couponDiscount.value  =  couponDiscount.value + campaign.couponDiscount;
                    }
                    appliedDiscount = appliedDiscount + couponDiscount.value;
                }
            }
            return appliedDiscount;
        };

        var processCampaignProductDiscount = (campaign) => {
            var appliedDiscount = 0;

            if ($rootScope.campaignEnabled) {
                var productsDiscounts = campaign.products;

                if (productsDiscounts && productsDiscounts.length) {
                    productsDiscounts.forEach(productPromotion => {
                        var discount       = productPromotion.unitDiscount;
                        var sequencialItem = productPromotion.sequencialItem;

                        applyDiscount(discount, sequencialItem, campaign);
                        appliedDiscount = appliedDiscount + discount;
                    });
                }
        	}
            return appliedDiscount;
        };

        var processCampaign = function(campaign) {
            if ($rootScope.campaignEnabled) {
                service.cart.giftsQuantity = (service.cart.giftsQuantity || 0) + campaign.giftsQuantity;

                if (!campaign.campaignPayment) {
                    var appliedDiscount = processCampaignCouponDiscount(campaign);

                    if (!appliedDiscount) {
                        processCampaignProductDiscount(campaign);
                    }
                    service.cart.promotions.push(campaign);
                } else {
                    if (!service.cart.campaignsPayment) {
                        service.cart.campaignsPayment = [];
                        service.cart.campaignsPayment.push(campaign);
                    } else {
                        service.cart.campaignsPayment.push(campaign);
                    }
                }
            }
        };

        var processPaymentCampaign = function(campaign){
             // só modifica campanhas se a configuração estiver habilitada.
         	if ($rootScope.campaignEnabled) {
                var appliedDiscount = processCampaignCouponDiscount(campaign);

                if (!appliedDiscount) {
                    appliedDiscount = processCampaignProductDiscount(campaign);
                }
                service.cart.campaignPaymentTotalDiscount = (service.cart.campaignPaymentTotalDiscount || 0) + appliedDiscount;
         	}
        };

        service.applyCampaignsDiscounts = function(campaigns){
        	// só modifica campanhas se a configuração estiver habilitada.
        	if ($rootScope.campaignEnabled) {
                if (campaigns) {
                    campaigns.forEach(function(campaign, index, array) {
                        campaign.type = "ProductPromotion";

                        try {
                            processCampaign(campaign);
                        } catch(err) {
                            LogService.error(err);
                            service.undoCampaignDiscountsPartial(campaign);
                        }
                     });
                    service.updateCartTotalValue();
                }
         	}
        };

        var isPaymentMethodValid = function(paymentMethod, campaignsPayments) {
            if (typeof campaignsPayments !== 'undefined' && campaignsPayments !== null) {
                for (var i = 0; i < campaignsPayments.length; i++) {
                    if (campaignsPayments[i].formCode == paymentMethod) {
                        return true;
                    }
                }
            }
            return false;
        };

        /**
         *
         */
        service.processPaymentCampaigns = function(paymentMethod, paymentValue) {
            let ret = null;

    	    if ($rootScope.campaignEnabled) {
    	        if (paymentValue == service.cart.totalValue) {
    	            ret = service.processNonConstrainedPaymentCampaigns(paymentMethod, paymentValue);
                }
    	        service.updateCartTotalValue();
            }

            return ret;
        };

        /**
         * Processa campanhas de pagamento em que o valor de pagamento não é verificado contra o valor total da campanha.
         */
        service.processNonConstrainedPaymentCampaigns = function(paymentMethod, paymentValue) {
            let ret = [];

            if ($rootScope.campaignEnabled) {
                LogService.info("Vai processar as campanhas com recebimento, forma de recebimento: ", paymentMethod);
                var campaignsPayment = service.cart.campaignsPayment;

                if (campaignsPayment && campaignsPayment.length) {
                    campaignsPayment.forEach(campaign => {
                        try {
                            if (isPaymentMethodValid(paymentMethod, campaign.payments)) {
                                processPaymentCampaign(campaign);
                                service.cart.promotions.push(campaign);
                                ret.push(campaign);
                                LogService.info("Promoção com forma de recebimento ativada: ", campaign.codeCampaign);
                            }
                        } catch(err) {
                            service.undoCampaignDiscountsPartial(campaign);
                        }
                    });
                }
                service.updateCartTotalValue();
            }
            return ret;
        };

        /**
        *
        */
        service.applyEmployeeDiscount = function() {
           if (!service.cart.orderRetrieved) {
               var customer = service.cart.customer;

               if (customer && customer.employeeDiscountPercentage && customer.employeeDiscountPercentage > 0) {
                   var totalValueFiscaisItens = changeDiscountsToFiscalItems();
                   var totalValueWithEmployeeDiscountPercentage = (totalValueFiscaisItens * customer.employeeDiscountPercentage / 100);
                   var truncatedTotalValue = Math.floor((totalValueWithEmployeeDiscountPercentage * 100).toFixed(2)) / 100;
                   var discount = {};

                   discount.factor = customer.employeeDiscountPercentage;
                   discount.source = "EMPLOYEE";
                   discount.value = truncatedTotalValue;

                   service.removeDiscountBySource(discount.source);
                   service.addDiscount(discount);
               }
               service.updateCartTotalValue();
           }
        };


        service.applyLoyaltyDiscount = function() {
            if (!ConfigService.config.business.loyalty_enabled || service.orderRetrieved) {
				return $q.resolve();
			}
			var customer = service.cart.customer;

			if (customer
				&& customer.document
				&& (customer.documentType == 'CPF' || customer.documentType == 'CNPJ' || customer.documentType == 'PASSPORT')
				&& (service.cart.fiscalItems && service.cart.fiscalItems.length > 0)) {
				const loyaltyDiscountRequest = new DiscountRequest();
				loyaltyDiscountRequest.documentNumber = customer.document;
				let updateCartTotalValue = false;
				service.cart.fiscalItems.forEach(fiscalItem => {
					for (var i = 0; i < fiscalItem.discounts.length; i++) {
						if (fiscalItem.discounts[i].source === 'LOYALTY_PROGRAM') {
							fiscalItem.discounts.splice(i, 1);
							updateCartTotalValue = true;
                            fiscalItem.processTotalValue();
							break;
						}
					}

					loyaltyDiscountRequest.items.push({
						identifier: fiscalItem.sequenceId,
						sku: fiscalItem.productCode,
						quantity: fiscalItem.quantitySold,
						unitValue: fiscalItem.totalExhib
					})
				});
				return $ionicLoading.show({
					template :  "Verificando programa de fidelidade",
					duration : 0
				}).then(function() {
					return LoyaltyService.getDiscount(loyaltyDiscountRequest)
					.then((ret) => {
						$ionicLoading.hide();
						if (ret.items) {
							ret.items.forEach(item => {
								if (item.originalUnitValue > item.unitValue) {
									updateCartTotalValue = true;
									const discount = {
										source: "LOYALTY_PROGRAM",
										value: parseFloat(item.originalUnitValue - item.unitValue),
                                        type: "VALUE",
									}
									service.cart.fiscalItems.forEach(fiscalItem => {
										if (fiscalItem.sequenceId == item.identifier) {
                                            if(discount.value >= fiscalItem.totalExhib) {
                                                // Esta validação é necessária porque a API da Renner pode retornar um valor maior que o item, fazendo com que o mesmo fique zerado ou negativado
                                                discount.value = fiscalItem.totalExhib - 0.01;
                                                LogService.info('O valor do desconto Loyalty é maior que o preço do item. Desconto recalculado :: ' + discount.value);
                                            }
											fiscalItem.discounts.push(discount);
											fiscalItem.processTotalValue();
										}
									});
								}
							})
						}
					}).catch(e => {
						LogService.error('Falha ao consultar o programa de fidelidade', e);
						$ionicLoading.hide();
                        return $q.resolve();
					}).finally(() => {
						if (updateCartTotalValue) {
							service.updateCartTotalValue();
						}
					});
				});
			} else {
                return $q.resolve();
            }
        }

        /**
         *
         */
        let changeDiscountsToFiscalItems = function () {
            var totalValueFiscaisItens = 0;
            service.cart.fiscalItems.forEach((item) => {
                totalValueFiscaisItens += item.processedUnitValue();
            });
            return totalValueFiscaisItens;
        };

        /**
         *
         */
        service.removeCustomerRelatedDiscounts = function() {
            // se for resgate de pedido, não modifica descontos.
         	if (!service.cart.orderRetrieved) {
                var discounts = service.cart.discounts;

                if (discounts) {
                    for (var i = 0; i < discounts.length; i++) {
                        if ((discounts[i].source === "EMPLOYEE") || (discounts[i].source === "FIRST_PURCHASE")) {
                            discounts.splice(i--, 1);
                        }
                    }
                }
                service.cart.firstPurchaseDiscountValue = undefined;
                service.updateCartTotalValue();
         	}
        };

        // retorna 'true' se existirem ítens, seja fiscais ou de serviços.
        service.hasItems = function() {
            return (service.cart.fiscalItems && service.cart.fiscalItems.length > 0) || service.hasServiceItems();
        };

        /**
         *
         */
        service.hasServiceItems = function() {
            return service.cart.serviceItems && service.cart.serviceItems.length > 0;
        };

        // retorna a operação corrente.
        service.getCurrentOperation = function() {
        	if (service.cart.fiscalItems && service.cart.fiscalItems.length > 0) {
        		return 'FISCAL';
        	} else if (service.cart.serviceItems && service.cart.serviceItems.length > 0) {
        		return 'SERVICE';
        	} else {
        		return 'NONE';
        	}
        };

        // retorna o tipo de serviço, caso seja uma operação de serviço.
        service.getCurrentServiceOperationType = function() {
        	var operationType = '';

        	if (service.cart.serviceItems && service.cart.serviceItems.length > 0) {
        		operationType = service.cart.serviceItems[0].operationType;
        	}
    	    return operationType;
        };

        /**
         *
         */
        service.getCurrentCartAction = function() {
            // montagem do carrinho. Se loja <> destino = CHECKOUT. Cao contrário, SALE.
        	// resgate do pedido. Se loja <> destino, nem resgata. Se loja = destino (obrigatorio), verifica o status.

            const fiscalOrderOperations = {
                V1: {
                    ON_HOLD: "ORDER_FULFILMENT",
                    READY_TO_PACK: "ORDER_PACKING",
                    CHECKOUT: "ORDER_PAYMENT",
                    DEFAULT: "ORDER_PAYMENT",
                },
                V2: {
                    READY_TO_PACK: "ORDER_PACKING_V2",
                }
            }

            const cfgFiscalOrderOperations = ConfigService.config.business.fiscal_cart_order_operations ||
                { ON_HOLD: 'V1', READY_TO_PACK: 'V1', CHECKOUT: 'V1' };

            const findOrderAction = (orderStatus) => {
                let version = cfgFiscalOrderOperations[orderStatus] || 'V1';
                version = version.toUpperCase();
                const operations = fiscalOrderOperations[version];
                return operations[orderStatus] || operations['DEFAULT'];
            };

        	if (service.getCurrentOperation !== 'SERVICE') {
    			// se for geração resgate de pedido, verifica o status.
    			if (service.cart.orderRetrieved) {
                    return findOrderAction(service.cart.orderStatus);
    			} else {
                    // se for caixa rápidp
                    if (service.isRemoteOrder()) {
                        // se tem ítens para outra loja para outra loja, cria um pedido com status de 'ON_HOLD'.
                        return "ORDER_REMOTE_CREATION";
                        } else if (service.hasShowrooming()) {
                            // se tem ítens para criação de showrooming
                        return "SHOWROOMING_CREATION";
                    } else {
                        // caso contrário, é uma venda.
                        return "SALE_PAYMENT";
                    }
    			}
        	} else {
        		return null;
        	}
		};

        /**
         *
         */
        service.isRemoteOrder = function() {
            return (service.cart.destinationStoreNumber && service.cart.storeNumber !== service.cart.destinationStoreNumber) ? true : false;
        };

        /**
         * Inicializa as propriedades do objeto do objeto do Carrinho.
         */
        service.initCart = function(preserveCustomer) {
            LogService.clearKeys();

            LogService.info("Inicializando carrinho...");

            for (const key in service.cart) {
                if (service.cart.hasOwnProperty(key)) {
                	if (preserveCustomer) {
                		if (key != 'customer'){
							service.cart[key] = undefined;
                		}
                	} else {
                		service.cart[key] = undefined;
                	}
                }
            }

            LogService.info("Inicializando carrinho...");
            service.reloadNSU();

            const currentTime = new Date();

            let sessionId = currentTime.toLocaleDateString();
            sessionId = parseInt(sessionId.match(/\d+$/)[0], 10);
            sessionId = String(sessionId);
            sessionId = sessionId.substring(sessionId.length, 2);
            sessionId += currentTime.toLocaleTimeString().replace(/[^0-9]*/g, '');
            sessionId = parseInt(sessionId);
            service.sessionId =  ConfigService.config.terminalNumber + sessionId;
            service.params = {
                createPdf: true,
                createPrinterData: true
            };
            service.preConfirmedPayments = undefined;

            service.cart.orderRetrieved = false;
            service.cart.companyNumber = ConfigService.config.companyNumber;
            service.cart.storeNumber = ConfigService.config.storeNumber;
            service.cart.terminalNumber = ConfigService.config.terminalNumber;
            service.cart.status = undefined;
            service.cart.operatorCode = UserService.operatorCode();
            service.cart.changeAuthorizations = {};
            service.cart.totalValue = 0;
            service.cart.customer = (preserveCustomer ? service.cart.customer : undefined);
            service.cart.fiscalItems = [];
            service.cart.serviceItems = [];
            service.cart.salePayments = [];
            service.cart.discounts = [];
            service.cart.additionalCharge = 0;
            service.cart.campaignPayment = [];
            service.cart.giftsQuantity = 0;
            service.cart.totalDiscount = 0;
            service.cart.promotions = [];
            service.cart.insuranceData = {};
            if (ConfigService.config.business.seller_identification_mode == 'venda'
                && ConfigService.config.business.automatic_seller_identification_enabled === true) {
                const loggedOperator = {
                    login: UserService.operatorCode(),
                    name: UserService.operatorName(),
                }   
                service.seller = loggedOperator;
            } else {
                service.seller = undefined;
            }
            service.predefinedPaymentMethodsDetails = undefined;
            service.processedPredefinedPayments = undefined;
            service.cart.nonFiscalItems = [];
            service.cart.deliveryAddress = {};
            service.cart.showroomingInfo = {};
            service.cart.usedCashback = undefined;
            service.initCartObservers.notify();
            service.cart.cashChange = 0;
            service.cart.currentOrder = undefined;
            service.cart.suggestions = [];
            service.cart.giftBenefits = [];
            service.cart.coupons = [];
            service.cart.fiscalCustomer = undefined;
            service.cart.orderChecking = undefined;

            if(ConfigService.config.business.loyalty_enabled) {
                service.cart.loyaltyData = undefined;
            }
            service.cart.paymentLinkSettings = {
                messageReceiverCustomer: {},
                meanOfSendingMessage: 'WHATSAPP'
            };
            service.cart.confirmedPayments = undefined;
            // Validando login
            if (!$state.is('login') && ConfigService.config.business.check_signin_before_sale) {
                UserService.userAutheticationValidation().then(() => {}, error => {
                    AlertService.restError(error, 'Autenticação inválida. Favor realizar login novamente.').finally (() => {
                        UserService.logout();
                    });
                });
            }
            $rootScope.$emit("cart-item-quantity"); //ver homectrl.
        };

        /**
         * Recarrega o NSU do carrinho
         */
        service.reloadNSU = function() {
            ConfigService.nsuControll.incrementCartNSU(); // Atualizando a NSU corrente.
            service.cart.nsu = ConfigService.nsuControll.currentNSU();
        }

        /**
         *
         */
        service.loadCart = function(originalOrder, mergedOrders, customer) {
        	// inicializa os valores 'default' do carrinho.
            service.initCart();

            const order = JSON.parse(JSON.stringify(originalOrder));

            const ignoredPaymentStates = ['NONE', 'CANCELED', 'UNDONE', 'ERROR'];

            if (order.orderPayments) {
                order.orderPayments = order.orderPayments
                    .filter(p => !ignoredPaymentStates.includes(p.paymentState));
            }

            service.cart.currentOrder = JSON.parse(JSON.stringify(order));

        	service.cart.orderRetrieved = true;
            service.cart.totalValue = parseFloat(order.totalValue);
            service.cart.companyNumber = order.companyNumber;
            service.cart.storeNumber = order.storeNumber;
            service.cart.orderNumber = order.orderNumber;
            service.cart.destinationStoreNumber = order.destinationStoreNumber;
            service.cart.orderStatus = order.status;
            service.cart.originSystem = order.originSystem;
            service.cart.orderDate = order.orderDate;
            service.cart.operatorCode = order.operatorCode;
            service.cart.externalOrderCode = order.externalOrderCode;
            service.cart.cupomFooterNote = order.cupomFooterNote;

            customer = customer || order.customer;
            if (customer && customer.document) {
                service.cart.customer = customer;
            }

            service.cart.fiscalCustomer = order.fiscalCustomer;
            service.cart.discounts = order.discounts;
            service.cart.additionalInformations = order.additionalInformations;
            service.cart.fiscalItems = [];
            service.cart.nonFiscalItems = order.nonFiscalItems;
            service.cart.cashChange = 0;
            service.cart.mergedOrders = mergedOrders;

            // se tiver habilitado a conferência de ítens do pedido e tiverem ítens RL, inclui objeto de controle
            if (service.mustScanOrderItems()) {
                const orders = UtilService.isNotEmptyArray(mergedOrders) ? JSON.parse(JSON.stringify(mergedOrders)) : [];
                orders.push(UtilService.cloneObject(order));

                const soldInStoreItems = OrderChecking.extractOrderSoldInStoreItems(orders);
                if (UtilService.isNotEmptyArray(soldInStoreItems)) {
                    let clonedOrders = UtilService.cloneObject(orders);
                    service.cart.orderChecking = {
                        orders: clonedOrders
                    }
                    LogService.info(`Existem ${soldInStoreItems.length} registros de ítens vendidos em loja que precisarão ser bipados.`);
                    LogService.info(`Total de pedidos inseridos para validação: ${orders.length}.`);
                } else {
                    LogService.info(`Não existem ítens vendidos em loja, não haverá conferência de ítens!`);
                }
            }

            LogService.clearKey('transactionsIds');
            LogService.clearKey('externalTransactionsIds');

            if (mergedOrders) {
                mergedOrders.forEach(o => {
                    LogService.putKey('transactionsIds', o.transactionId, 'APPEND');
                    LogService.putKey('externalTransactionsIds', o.externalTransactionId, 'APPEND');
                });
            } else {
                LogService.putKey('transactionsIds', order.transactionId, 'APPEND');
                LogService.putKey('externalTransactionsIds', order.externalTransactionId, 'APPEND');
            }

            var index;

	        for (index in order.fiscalItems) {
        	    var fiscalItem = order.fiscalItems[index];

                if (order.type !== 'SHOWROOMING') {
                    // a validação para garantir o mesmo tipo é efetuada anteriormente. Se for nulo, o padrão é 'SOLD_IN_STORE'.
                    service.cart.deliveryOption = fiscalItem.deliveryOption && service.cart.deliveryOption !== 'SOLD_IN_STORE' ? fiscalItem.deliveryOption : 'SOLD_IN_STORE';
                }

                if (service.mustScanOrderItems() && service.isSoldInStore(fiscalItem)) {
                    LogService.info(`Ítem não será adicionado automaticamente no carrinho pois precisa ser bipado: '${fiscalItem.productCode}'.`);
                } else {
                    if (order.type === 'SHOWROOMING') {
                        if (fiscalItem.deliveryOption === 'WAREHOUSE_DELIVERY' || fiscalItem.deliveryOption === 'PICK_UP_IN_STORE') {
                            fiscalItem.showrooming = true;
                        }
                    }
                    if (fiscalItem.imei) {
                        fiscalItem.itemUuid = {
                            type:     'IMEI',
                            value:    fiscalItem.imei
                        };
                        delete fiscalItem.imei;
                    }
                    if (order.orderNumber && !fiscalItem.orderNumber) {
                        fiscalItem.orderNumber = order.orderNumber;
                    }
                    if (order.externalOrderCode && !fiscalItem.externalOrderCode) {
                        fiscalItem.externalOrderCode = order.externalOrderCode;
                    }
                    if (order.originSystem && !fiscalItem.originSystem) {
                        fiscalItem.originSystem = order.originSystem;
                    }
                    service.addItem(fiscalItem, null, null, service.cart.destinationStoreNumber);
                }
	        }

	        service.predefinedPaymentMethodsDetails = processMandatoryOrderPaymentsDetails(order.orderPayments);
	        service.processedPredefinedPayments = [];

            if (order.orderPayments != null) {
                const preConfirmedPayments = order.orderPayments.filter(
                    op => op.paymentState == 'CONFIRMED'
                ).map(
                    op => parseSalePayment(op, order)
                );
                service.preConfirmedPayments = preConfirmedPayments;
            }

            service.reindexCartItems();
            service.updateCartTotalValue();
        };

        service.loadOrders = function(orders, initCart) {
            // carrega os pedidos sem fazer merge com o carrinho (comportamento atual)
            if (initCart) {
                service.initCart();
            }
            service.cart.orderRetrieved = true;

            const order = orders[0];
            service.cart.companyNumber = order.companyNumber;
            service.cart.storeNumber = order.storeNumber;
            service.cart.orderNumber = order.orderNumber;
            service.cart.destinationStoreNumber = order.destinationStoreNumber;
            service.cart.orderStatus = order.status;
            service.cart.originSystem = order.originSystem;
            service.cart.orderDate = order.orderDate;
            service.cart.operatorCode = order.operatorCode;
            service.cart.cupomFooterNote = order.cupomFooterNote;
            service.cart.customer = order.customer || service.cart.customer;
            service.cart.fiscalCustomer = order.fiscalCustomer || service.cart.fiscalCustomer;
            service.cart.additionalInformations = order.additionalInformations;
            service.cart.mergedOrders = orders;

            if (service.cart._orderFiscalCustomer) {
                service.cart._orderFiscalCustomer = order.fiscalCustomer;
            } else {
                Object.defineProperty(service.cart, '_orderFiscalCustomer', {
                    enumerable: false,
                    value: order.fiscalCustomer
                });
            }
            // se tiver habilitado a conferência de ítens do pedido e tiverem ítens RL, inclui objeto de controle
            if (service.mustScanOrderItems()) {
                const soldInStoreItems = OrderChecking.extractOrderSoldInStoreItems(orders);
                if (UtilService.isNotEmptyArray(soldInStoreItems)) {
                    let clonedOrders = UtilService.cloneObject(orders);
                    service.cart.orderChecking = {
                        orders: clonedOrders
                    }
                    LogService.info(`Existem ${soldInStoreItems.length} registros de ítens vendidos em loja que precisarão ser bipados.`);
                    LogService.info(`Total de pedidos inseridos para validação: ${orders.length}.`);
                } else {
                    LogService.info(`Não existem ítens vendidos em loja, não haverá conferência de ítens!`);
                }
            }

            LogService.clearKey('transactionsIds');
            LogService.clearKey('externalTransactionsIds');

            orders.forEach(o => {
                LogService.putKey('transactionsIds', o.transactionId, 'APPEND');
                LogService.putKey('externalTransactionsIds', o.externalTransactionId, 'APPEND');
            });



            for (var i = 0; i < orders.length; i++) {
                const order = orders[i];
                for (var index in order.fiscalItems) {
                    var fiscalItem = order.fiscalItems[index];

                    if (order.type !== 'SHOWROOMING') {
                        // a validação para garantir o mesmo tipo é efetuada anteriormente. Se for nulo, o padrão é 'SOLD_IN_STORE'.
                        service.cart.deliveryOption = fiscalItem.deliveryOption && service.cart.deliveryOption !== 'SOLD_IN_STORE' ? fiscalItem.deliveryOption : 'SOLD_IN_STORE';
                    }

                    if (service.mustScanOrderItems() && service.isSoldInStore(fiscalItem)) {
                        LogService.info(`Ítem não será adicionado automaticamente no carrinho pois precisa ser bipado: '${fiscalItem.productCode}' do pedido '${order.orderNumber}'.`);
                    } else {
                        if (order.type === 'SHOWROOMING') {
                            if (fiscalItem.deliveryOption === 'WAREHOUSE_DELIVERY' || fiscalItem.deliveryOption === 'PICK_UP_IN_STORE') {
                                fiscalItem.showrooming = true;
                            }
                        }
                        if (fiscalItem.imei) {
                            fiscalItem.itemUuid = {
                                type:     'IMEI',
                                value:    fiscalItem.imei
                            };
                            delete fiscalItem.imei;
                        }
                        if (order.orderNumber && !fiscalItem.orderNumber) {
                            fiscalItem.orderNumber = order.orderNumber;
                        }
                        service.addItem(fiscalItem, null, null, service.cart.destinationStoreNumber);
                    }
                }
            }

	        service.predefinedPaymentMethodsDetails = null;
	        service.processedPredefinedPayments = [];

            orders
                .filter(order => order.orderPayments != null)
                .forEach(order => {
                    const preConfirmedPayments = order.orderPayments.filter(
                        op => op.paymentState == 'CONFIRMED'
                    ).map(
                        op => parseSalePayment(op, order)
                    );

                    if (preConfirmedPayments == null || preConfirmedPayments.length == 0) {
                        return;
                    }

                    if (service.preConfirmedPayments == null) {
                        service.preConfirmedPayments = [];
                    }

                    service.preConfirmedPayments.push(...preConfirmedPayments);
                });
            
            service.reindexCartItems();
            service.updateCartTotalValue();
        };

        function isSamePriceOrder(item, errorCallback) {
            // só fará a verificação de preços, por enquanto, na utilização do resgate de pedidos
            if (service.cart.orderRetrieved) {
                const itemDifPriceFound = service.cart.fiscalItems
                    .find(fiscalItem => fiscalItem.productCode === item.productCode
                        && fiscalItem.unitValue.toFixed(2) !== item.unitValue.toFixed(2));

                if (itemDifPriceFound) {
                    const message = `Produto já existe no carrinho com preço (${itemDifPriceFound.unitValue.toFixed(2)}) diferente do produto sendo adicionado: ${item.unitValue.toFixed(2)}!`;
                    return dispatchErrorCallback(errorCallback, message);
                } else {
                    // verifica nos pedidos a serem resgatados
                    if (service.cart.orderChecking && UtilService.isNotEmptyArray(service.cart.orderChecking.orders)) {
                        const orders = service.cart.orderChecking.orders;

                        const itemDifPriceOrderFound = OrderChecking.extractOrderItems(orders)
                            .find(fiscalItem => fiscalItem.productCode === item.productCode
                                && fiscalItem.unitValue.toFixed(2) !== item.unitValue.toFixed(2));

                        if (itemDifPriceOrderFound) {
                            const message = `Produto já existe num pedido resgatado com preço (${itemDifPriceOrderFound.unitValue.toFixed(2)}) diferente do produto sendo adicionado: ${item.unitValue.toFixed(2)}!`;
                            return dispatchErrorCallback(errorCallback, message);
                        }
                    }
                }
            }

            return true;
        }

        /**
         * Processa os pagamentos obrigatórios na lista passada. Caso não haja algum, a função da retorno indefinido.
         */
        const processMandatoryOrderPaymentsDetails = function(orderPayments) {
            let ret = undefined;

            if (orderPayments && orderPayments.length) {
                const filtered = orderPayments.filter(function (orderPayment) {
                    return orderPayment.mandatoryMethod
                        && orderPayment.paymentState != 'CONFIRMED';
                });

                if (filtered.length > 0) {
                    ret = [];

                    filtered.forEach(function(orderPayment) {
                       const paymentDetails = {
                           sequence:                    new Number(orderPayment.paymentSequence),
                           paymentMethod:               orderPayment.paymentCodeMethod,
                           billedPaymentMethod:         orderPayment.billedPaymentMethod,
                           paymentValue:                orderPayment.paymentValue,
                           paymentAuthorizerCodeChoice: orderPayment.paymentAuthorizerCodeChoice,
                           paymentAuthorizerBrandName:  orderPayment.paymentAuthorizerBrandName,
						   paymentAuthorizationCode: 	orderPayment.paymentAuthorizationCode,
						   paymentAuthorizationNSU: 	orderPayment.paymentAuthorizationNSU,
						   paymentAuthorizerNSU: 		orderPayment.paymentAuthorizerNSU,
                           paymentAuthorizerName:       orderPayment.paymentAuthorizerName,
                           paymentTerminalId:           orderPayment.paymentTerminalId,
						   customerPaymentVoucher: 		orderPayment.customerPaymentVoucher,
						   bussinessPaymentVoucher: 	orderPayment.bussinessPaymentVoucher,
						   cardNumber: 					orderPayment.cardNumber,
						   acquirerName: 				orderPayment.acquirerName,
                           paymentState:                orderPayment.paymentState,
                           paymentDate:                 orderPayment.paymentDate,
                           billedBankDocument:          orderPayment.billedBankDocument,
                           billedBankCode:              orderPayment.billedBankCode,
                           billedBankName:              orderPayment.billedBankName,
                           paymentCustomer:             orderPayment.customer,
                           paymentId:                   orderPayment.paymentId,
                           paymentTransactionId:        orderPayment.paymentTransactionId,
                           walletDocument:              orderPayment.eWalletDocument,
                           walletName:                  orderPayment.eWalletName,
                           walletId:                    orderPayment.eWalletCode,
                           plan: {
                               code:            orderPayment.paymentCodePlan,
                               totalValue:      orderPayment.paymentValue,
                               instalments:    (function(orderInstalments) {
                                   if (orderInstalments) {
                                       var res = [];
                                       orderInstalments.forEach(function(instalment) {
                                           res.push({    //Covertendo do parcela do pedido para a parcela usada.
                                               number:          new Number(instalment.paymentSequence),
                                               dueDate:         instalment.instalmentDateExpiration,
                                               instalmentValue: instalment.installmentValue
                                           });
                                       });
                                       return res;
                                   } else {
                                       return undefined;
                                   }
                               })(orderPayment.installments),
                               firstInstalment: (function(orderInstalments) {
                                   if (orderInstalments && orderInstalments.length) {
                                       orderInstalments.sort(function(a, b) {
                                           return new Number(a.paymentSequence) - new Number(b.paymentSequence);
                                       });

                                       const instalment = orderInstalments[0];

                                       return {
                                           number:          new Number(instalment.paymentSequence),
                                           dueDate:         instalment.instalmentDateExpiration,
                                           instalmentValue: instalment.installmentValue
                                       };
                                   } else {
                                       return undefined;
                                   }
                               })(orderPayment.installments)
                           }
                       };
                       ret.push(paymentDetails);
                    });
                }
            }
            return ret;
        };

        function parseSalePayment(orderPayment, order) {
            // não salvamos o paymentSequence, em casos de múltiplos pedidos
            // seria sobrescrito
            const sp = {
                type:                           parseSalePaymentType(orderPayment.paymentCodeMethod),
                paymentCodeMethod:              orderPayment.paymentCodeMethod,
                genericMethodId:                orderPayment.genericMethodId,
                billedPaymentMethod:            orderPayment.billedPaymentMethod,
                paymentValue:                   orderPayment.paymentValue,
                paymentAuthorizerBrandName:     orderPayment.paymentAuthorizerBrandName,
                paymentAuthorizationCode: 	    orderPayment.paymentAuthorizationCode,
                paymentAuthorizationNSU: 	    orderPayment.paymentAuthorizationNSU,
                paymentAuthorizerNSU: 		    orderPayment.paymentAuthorizerNSU,
                paymentAuthorizerName:          orderPayment.paymentAuthorizerName,
                paymentTerminalId:              orderPayment.paymentTerminalId,
                customerPaymentVoucher: 		orderPayment.customerPaymentVoucher,
                bussinessPaymentVoucher: 	    orderPayment.bussinessPaymentVoucher,
                cardNumber: 					orderPayment.cardNumber,
                acquirerName: 				    orderPayment.acquirerName,
                paymentState:                   orderPayment.paymentState,
                paymentDate:                    orderPayment.paymentDate,
                billedBankDocument:             orderPayment.billedBankDocument,
                billedBankCode:                 orderPayment.billedBankCode,
                billedBankName:                 orderPayment.billedBankName,
                paymentCustomer:                orderPayment.customer,
                paymentId:                      orderPayment.paymentId,
                paymentTransactionId:           orderPayment.paymentTransactionId,
                walletDocument:                 orderPayment.eWalletDocument,
                walletName:                     orderPayment.eWalletName,
                walletId:                       orderPayment.eWalletCode,
                installments:                   orderPayment.installments || [],
                paymentOrigin:                  {
                                                    type: 'ORDER',
                                                    id: order.orderNumber,
                                                    application: {
                                                        id: 'LINX_SERVIDOR_PEDIDOS',
                                                        vendor: {
                                                            id: 'LINX'
                                                        }
                                                    }
                                                }
            };

            Object.defineProperty(sp, '_privateLabel', {
                enumerable: false,
                value: isPrivateLabel(sp.paymentCodeMethod)
            });

            return sp;
        }

        function parseSalePaymentType(paymentMethod) {
            switch(paymentMethod) {
                case 'CREDIT':
                case 'DEBIT':
                case 'CONDUCTOR_CREDIT_BOOK':
                case 'CONDUCTOR_CARD_STATEMENT':
                    return 'SaleTEFPayment';
                case 'TOTVSCARD_CREDIT_BOOK':
                case 'TOTVSCARD_CARD_STATEMENT':
                case 'LEGACY_CLIENT':
                    return 'SalePrivateLabelPayment';
                case 'GENERIC':
                case 'CASH':
                    return 'SaleGenericPayment';
                case 'PIX':
                    return 'SalePixPayment';
                case 'QR_SITEF':
                case 'QR_LINX':
                    return 'SaleEWalletPayment';
                case 'VOUCHER_STOREX':
                    return 'SaleVoucherPayment';
                case 'VOUCHER_ASSURANT':
                    return 'SaleVoucherAssurantPayment';
                case 'VOUCHER_SAFE':
                    return 'SaleVoucherSafePayment';
                case 'VOUCHER_GIVEX':
                    return 'SaleVoucherGivexPayment';
                case 'VOUCHER_PROMO':
                    return 'SaleVoucherOmniPayment';
                case 'CHECK':
                    return 'SaleCheckPayment';
                case 'INSTALLMENT_PLAN':
                    return 'SaleInstallmentPlanPayment';
                case 'GIFT_CARD':
                case 'GIFT_CARD_GIVEX':
                case 'GIFT_CARD_GIVEX_LEGACY':
                    return 'SaleGiftCardPayment';
                case 'COVENANT':
                    return 'SaleCovenantPayment';
                case 'PAYMENT_LINK':
                    // não temos pagamentos desta forma mapeado, resgate de link de pagamento
                    // é retornado esta forma, porém como não integramos vendas neste fluxo
                    // não é necessário ter um tipo
                    return null;
                default:
                    throw new Error(`Forma de pagamento '${paymentMethod}' não possui tipo mapeado`);
            }
        }

        function isPrivateLabel(paymentMethod) {
            return paymentMethod == 'CONDUCTOR_CREDIT_BOOK'
                || paymentMethod == 'CONDUCTOR_CARD_STATEMENT'
                || paymentMethod == 'TOTVSCARD_CREDIT_BOOK'
                || paymentMethod == 'TOTVSCARD_CARD_STATEMENT'
                || paymentMethod == 'LEGACY_CLIENT';
        }

        service.isSoldInStore = function(fiscalItem) {
            return fiscalItem != null && (fiscalItem.deliveryOption == null || fiscalItem.deliveryOption == 'SOLD_IN_STORE');
        }

        service.mustScanOrderItems = function () {
            return ConfigService.config.business.scan_order_sold_in_store_items === true;
        }
        /**
         * Verifica se há pagamento predefinido pendente comparando as informações deste serviço.
         */
        service.hasUnprocessedPredefinedPaymentMethod = function() {
            var predefined = service.predefinedPaymentMethodsDetails && service.predefinedPaymentMethodsDetails.length ?
                    service.predefinedPaymentMethodsDetails : [];

            predefined = predefined.filter(function(pre){
                return !service.processedPredefinedPayments || !service.processedPredefinedPayments.includes(pre);
            });
            return !!predefined.length;
        };

        /**
         *
         */
        service.showSellerDialog = function(buttons, callScope) {
            callScope.sellerDetails = service.seller || {};

            $ionicPopup.confirm({
                title : 'Vendedor',
                templateUrl : './partials/sellers.html',
                subTitle : 'Dados do Vendedor',
                scope: callScope,
                buttons : buttons || [{ text: '<b>Fechar</b>', type: 'button-theme', onTap: function(e) { return true; } }]
            })
            .then(function() { callScope.sellerDetails = undefined; });
        };

        /**
         *
         */
        service.registerSellerToSaleItems = function(seller, items) {
            const login = (seller && seller.login) ? seller.login : undefined;
            items = items || service.cart.fiscalItems;
            angular.forEach(items, fiscalItem => fiscalItem.salesAssociateNumber = login);
            service.seller = seller;
        };

        /**
         *
         */
        service.customerValidationWarn = function() {
            // TODO remover função de todas as telas
            return false;
        };

        //Verifica se tem alguma receita adicionada.
        service.existPrescriptionControlled = function(){
            if(service.cart.prescriptions && service.cart.prescriptions.length > 0) return true;
            return false;
        }
        //Retorna a quantidade de remédios controlados no carrinho.
        service.countProductsControlled = function(){
            if(service.cart.fiscalItems && service.cart.fiscalItems.length > 0){
                return service.cart.fiscalItems.reduce(function(total, product){
                    return product.productType == "PRESCRIPTION_CONTROLLED" ? total + product.quantitySold : total + 0;
                }, 0);
            }
            return 0;

        }
        //Retorna a quantidade de itens nas receitas
        service.countProductsInPrescriptionControlled = function(){
            if(service.existPrescriptionControlled()){
                return service.cart.prescriptions.reduce(function(totalProducts, prescription){
                    return totalProducts + prescription.items.reduce(function(totalItems, item){
                        return totalItems + item.quantity;
                    }, 0);
                }, 0);
            }
            return 0;
        };

        service.isPickupOrder = function () {
            return service.hasItems() && (service.cart.destinationStoreNumber && service.cart.destinationStoreNumber !== service.cart.storeNumber);
        };

        service.hasShowrooming = function (){
			var hasShowrooming = false;
			for (var i = 0; i < service.cart.fiscalItems.length && !hasShowrooming; i++) {
				var item = service.cart.fiscalItems[i];
				if (item.showrooming) {
					hasShowrooming = true;
				}
			}
			return hasShowrooming;
		};

		service.hasSoldInStoreItems = function (){
			var hasSoldInStoreItems = false;
			for (var i = 0; i < service.cart.fiscalItems.length && !hasSoldInStoreItems; i++) {
				var item = service.cart.fiscalItems[i];
				if (!item.showrooming) {
					hasSoldInStoreItems = true;
				}
			}
			return hasSoldInStoreItems;
		};

		service.getShowroomingItems = function (){
			var ret = [];
			var fiscalItems = service.cart.fiscalItems;
			if (fiscalItems) {
				fiscalItems.forEach(function(fis){
					if (fis.showrooming === true) {
						const discount = fis.discounts && Array.isArray(fis.discounts) ? fis.discounts.map((d) => d.value).reduce((t, v) => t + v, 0) : 0;
						ret.push({
							sku:		fis.productCode,
							quantity:	fis.quantitySold,
							price:		fis.unitValue - discount,
							weight:		fis.weight
						});
					}
				});
			}
			return ret;
		};

		service.updateFreightValue = function (freightValue){
			service.removeFreightValue();

			if (!service.cart.nonFiscalItems) {
				service.cart.nonFiscalItems = [];
			}
            if(!service.cart.fiscalItems) {
                service.cart.fiscalItems = [];
            }
			var nonFiscalFreightItem = {
				sequenceId: getNextSequenceId() + "",
				nonFiscalOperationType: "FREIGHT",
				value: freightValue,
			};
			LogService.info('Valor do frete informado: [' + freightValue + "]");
			service.cart.nonFiscalItems.push(nonFiscalFreightItem);
			service.updateCartTotalValue();

		};

		service.removeFreightValue = function () {
			if (service.cart.nonFiscalItems) {
				var nonFiscalItems = service.cart.nonFiscalItems;
                for (var i = 0; i < nonFiscalItems.length; i++) {
                    if (nonFiscalItems[i].nonFiscalOperationType === "FREIGHT") {
                    	LogService.info('Valor do frete removido!');
                    	nonFiscalItems.splice(i, 1);
                        service.updateCartTotalValue();
                        break;
                    }
                }
			}
		}

        service.hasNonFiscalItems = function() {
            return service.cart.nonFiscalItems && service.cart.nonFiscalItems.length > 0;
        }

        service.setCompareProductInPrescriptions = function(product){
            var prescriptions = service.cart.prescriptions;
            var quantity = product.quantitySold * -1;
            for(var x = 0; x < prescriptions.length; x++){
                var prescriptionItems = prescriptions[x].items;
                for(var s = 0; s < prescriptionItems.length; s++){
                    var prodPrescription = prescriptionItems[s];
                    if(product.automationCode == prodPrescription.automationCode){
                        quantity += prodPrescription.quantity;
                    }
                }
            }
            product.comparePrescription = quantity;
        }

        service.setComparePrescriptionInProducts = function(productPrescription, pruductsCart){
            var quantity = productPrescription.quantity * -1;
            for(var x = 0; x < pruductsCart.length; x++){
                var productCart = pruductsCart[x];
                if(productPrescription.automationCode == productCart.automationCode){
                    quantity += productCart.quantitySold;
                    if(productPrescription.quantity >= productCart.quantitySold){
                        productCart.quantitySold = 0;
                    }else{
                        productCart.quantitySold -= productPrescription.quantity ;
                        quantity = 0
                    }
                }
            }
            productPrescription.compareProduct = quantity;
        }


        service.getCompareListProductPrescription = function(){
            var items = service.cart.fiscalItems;
            for(var i = 0; i < items.length;i++){
                var item = items[i];
                service.setCompareProductInPrescriptions(item);
            }
            return items;
        }

        service.getCompareListPrescriptionProduct = function(){
            var prescriptions = service.cart.prescriptions;
            var listProductCart = angular.copy(service.cart.fiscalItems);
            for(var i = 0; i < prescriptions.length;i++){
                var prescription = prescriptions[i];
                var prescriptionItems = prescription.items;
                for(var x = 0; x < prescriptionItems.length; x++){
                    var productPrescription = prescriptionItems[x];
                    service.setComparePrescriptionInProducts(productPrescription, listProductCart);
                }
            }
            return prescriptions;
        }

        service.getDiscountsRules = function(params, success, error) {
            var error = function(response) {
                $ionicPopup.alert({
                    title: "Processamento dos Descontos",
                    template: response.data.message,
                    okType: "button-theme"
                });
            };
            service.loadSettings();
            service.url += '/v1/promotion/';

            return $resource(service.url, {}, {
                process: {
                    method: 'POST'
                }
            }).process({}, params, success,  error).$promise;
		}

        /**
         *
         */
         service.processRulesDiscounts = function(paymentMethods, paymentValue, payments, fiscalItemsTemp, firstPurchase) {
            return $q(function(resolve, reject) {
                let containsItemsTemp = (fiscalItemsTemp && fiscalItemsTemp.length && fiscalItemsTemp.length > 0);
                if ($rootScope.processDiscount) {
                    $timeout.cancel($rootScope.processDiscount);
                }
                if ($rootScope.promotionEngine) {
                    var cart = service.cart;

                    if (cart.fiscalItems.length > 0 || containsItemsTemp) {
                        $ionicLoading.show({
                            template : 'Por favor, aguarde...',
                            duration : 0
                        }).then(function() {
                            $rootScope.processDiscount = $timeout(function() {
                                var params = {
                                    sale: {
                                        fiscalItems: [],
                                        discounts: clearFactor(cart.discounts),
                                        totalValue: cart.totalValue,
                                        companyNumber: ConfigService.config.companyNumber,
                                        storeNumber: ConfigService.config.storeNumber,
                                        coupons: service.cart.coupons,
                                        sessionData: service.getSessionData()
                                    }
                                };

                                if (paymentMethods) {
                                    params.sale.paymentInitRequest = {
                                        paymentCodeMethod: paymentMethods.id,
                                        paymentCodePlan: '',
                                        paymentValue: paymentValue,
                                        firstPurchase: firstPurchase
                                    }
                                }

                                if (payments) {
                                    var __payments = [];
                                    payments.forEach(element => {
                                        var pay = null;
                                        pay = {
                                            paymentSequence: element.paymentSequence,
                                            paymentCodeMethod: element.paymentMethod,
                                            paymentCodePlan: element.plan != null ?
                                                element.plan.code : null,
                                            paymentValue: element.paymentValue,
                                            firstPurchase: element.firstPurchase
                                        };
                                        __payments.push(pay);
                                    });
                                    params.sale.salePayments = __payments;
                                }

                                params.sale.customers = [];

                                if (cart.customer && cart.customer.documentType) {
                                    var loyaltyPrograms = null;
                                    if (cart.customer.loyaltyPrograms && cart.customer.loyaltyPrograms.length) {
                                        loyaltyPrograms = cart.customer.loyaltyPrograms.map(loyaltyProgram => {
                                            return {
                                                name: loyaltyProgram.name,
                                                category: loyaltyProgram.category
                                            };
                                        })
                                    }

                                    params.sale.customers.push({
                                        documentType: cart.customer.documentType,
                                        addtionalDiscountRules: cart.customer.additionalDiscountRules,
                                        additionalCampaigns: cart.customer.additionalCampaigns,
                                        groups: [
                                            {
                                                groupCode: cart.customer.groupCode
                                            }
                                        ],
                                        registerType: 'CUSTOMER_REGISTER',
                                        document: cart.customer.document,
                                        loyaltyPrograms: loyaltyPrograms
                                    });
                                }
                                var covenantService = $injector.get('CovenantService');

                                if (covenantService.currentCovenantAuth()) {
                                    params.sale.customers.push({
                                        groups: [{
                                            groupCode: covenantService.currentCovenant().code
                                        }],
                                        registerType: 'COVENANT'
                                    });
                                }

                                params.sale.fiscalItems = cart.fiscalItems.map((item) => ({
                                    sequenceId: item.sequenceId,
                                    productCode: item.productCode,
                                    automationCode: item.automationCode,
                                    marketingCode: item.marketingCode,
                                    unitValue: item.unitValue,
                                    quantitySold: item.quantitySold,
                                    unitQuantity: 1,
                                    totalValue: item.totalValue,
                                    ownBrand: item.ownBrand,
                                    promotionalPrice: item.promotionalPrice,
                                    discounts: clearFactor(item.discounts)
                                }));

                                service.getDiscountsRules(params).then((resp) => {
                                    resolve(resp);
                                }).finally(() => {
                                    $rootScope.processDiscount = null;
                                    $ionicLoading.hide()
                                });
                            }, 1000);
                        });
                    } else {
                          $rootScope.processDiscount = null;
                          resolve({});
                    }
                } else {
                    $rootScope.processDiscount = false;
                }
            });
        };

        let clearFactor = function (discounts) {
            let resp = [];
            discounts.forEach((discount) => {
                let object = angular.copy(discount);
                if(object["factor"]){
                    delete object["factor"]
                }
                resp.push(object);
            });
            return resp;
        };

        /**
         *
         */
        service.updateDiscountCart = function(paymentMethod, paymentValue, payments, firstPurchase) {
            service.cart.suggestions = [];
            service.cart.giftBenefits = [];
            return service.processRulesDiscounts(paymentMethod, paymentValue, payments, null, firstPurchase).then((resp) => {
                let listCampaigns = []; //Garantindo unicidade dos dados
                let campaignTemp;
                // LINMPAR DESCONTO NO CUPOM, APENAS QUANDO ESTIVER NO CARRINHO, QUANDO ESTIVER NO PAGAMENTO DEVERÁ CONTINUAR O ÚLTIMO INSERIDO.
                if (!paymentMethod || !payments && payments.length <= 0) {
                    service.cart.discounts = [];
                }

                // o motor de promoção recalcula tudo, logo devemos limpar todos descontos
                if ($rootScope.promotionEngine) {
                    service.cart.discounts.length = 0;
                    service.cart.fiscalItems
                        .forEach(fiscalItem => {
                            fiscalItem.discounts.length = 0;
                            fiscalItem.processTotalValue();
                        });
                    service.cart.promotions.length = 0;
                }


                if (resp.discounts && resp.discounts.length > 0) {

                    resp.discounts.forEach((discount) => {
                        var source = discount.source;
                        var target = discount.target;
                        var discountObj = {
                            value: discount.value,
                            type: discount.type
                        };

                        if (source.type === 'CAMPAING') {
                            campaignTemp = listCampaigns.find( (c) => c.codeCampaign == source.code);//Verificando se a cammpanha já existe na lista.
                            if(!campaignTemp){
                                campaignTemp = {
                                    codeCampaign: source.code,
                                    promotionSourceCode: source.promotionSource ? source.promotionSource : "STOREX",
                                    description: source.description,
                                    externalPromotionCode: (source.promotionSource && source.promotionSource == "PROMO") ? target.externalId : undefined,
                                    triggers: [],
                                    products: [],
                                }
                                listCampaigns.push(campaignTemp);
                            }
                        }

                        switch (target.type) {
                            case 'COUPON':
                                if (campaignTemp) {
                                    campaignTemp["type"] = "CouponPromotion";
                                    campaignTemp["code"] = undefined;
                                }

                                service.cart.discounts = service.cart.discounts.filter(item => item.indentifierCode != source.code); //Removendo o desconto

                                discountObj.source = source.type;
                                discountObj.indentifierCode = source.code;
                                discountObj.externalId = source.code;

                                service.removeDiscountBySource(discountObj.source);
                                service.addDiscount(discountObj);
                                break;

                            case 'PRODUCT':

                                if (campaignTemp) {
                                    campaignTemp["type"] = "ProductPromotion";
                                    campaignTemp["code"] = target.code;
                                }


                                var prod = service.cart.fiscalItems.find((item, index) => item.productCode === target.code && index+1 === target.sequenceId);

                                if (prod) {
                                    prod.discounts = prod.discounts.filter(disc => disc.externalId != source.code); //Removendo o desconto
                                    discountObj.sequenceId = target.sequenceId;
                                    discountObj.source = source.type;
                                    discountObj.externalId = source.code;

                                    prod.addDiscount(discountObj);
                                    prod.processTotalValue();

                                    if (prod.isTotalValid()) {//Verificando se o desconto é maior que o valor do produto.
                                        if (campaignTemp) {
                                            campaignTemp.products.push({
                                                unitDiscount: discountObj.value,
                                                // practicedPrice: prod.unitValue, - Campo ainda sem conhecimento.
                                                quantitySold: discountObj.factor,
                                                sequencialItem: discountObj.sequenceId
                                            });
                                        }
                                    }
                                }
                                break;
                            default:
                                break;
                        }
                    });
                 }
                if (listCampaigns.length) {
                    service.cart.promotions = listCampaigns;
                }
                service.setSuggestions(resp);
                service.setGiftBenefits(resp);
                service.updateCartTotalValue();
            });
        };

        service.getSessionData = (status = "total", newSession = true) => {
            return {
                storeNumber: ConfigService.config.storeNumber,
                terminalNumber: ConfigService.config.terminalNumber,
                status: status,
                messageId: service.cart.nsu,
                newSession: newSession
            }
        }

        service.countSuggestions = () => {
            if(service.cart.suggestions) return service.cart.suggestions.length;
            return 0;
        };

        service.countGiftBenefits = () => {
            if(service.cart.giftBenefits) return service.cart.giftBenefits.length;
            return 0;
        };

        service.countCoupons = () => {
            if(service.cart.coupons) return service.cart.coupons.length;
            return 0;
        };

        service.addCoupon = (coupon) => {
            if(coupon && coupon.barcode){
                service.cart.coupons = service.cart.coupons ? service.cart.coupons : [];
                service.cart.coupons = service.cart.coupons.filter( (c) => c.barcode && c.barcode != coupon.barcode);
                service.cart.coupons.push(coupon);
            }
        }

        service.removeCoupon = (barcode) => {
            if(barcode && service.cart.coupons && service.cart.coupons.length){
                service.cart.coupons = service.cart.coupons.filter( (c) => c.barcode && c.barcode != barcode);
            }
        }

        service.setSuggestions = (returnCampaign) => {
            service.cart.suggestions = returnCampaign.suggestions;
        }

        service.setGiftBenefits = (returnCampaign) => {
            service.cart.giftBenefits = returnCampaign.giftBenefits;
        }

        service.checkInjectItemBatch = (item) => {
			var defered = $q.defer();
			if (item && item.batchCodeRequired === true) {
				AlertService.prompt("Lote ", "", "text", "Informe o lote do produto").then(batchCode => {
					if (!UtilService.isNullOrEmpty(batchCode)) {
						return $q.resolve(batchCode);
					} else {
                        // se clicou em cancelar
                        if (batchCode == undefined) {
                            return $q.reject();
                        } else {
                            // se o lote é vazio
                            return AlertService.error("Lote inválido!")
                                .then(() => $q.reject());
                        }
					}
				}).then((batchCode) => {
					return $ionicLoading.show({
						template : 'Consultando lote do produto. Aguarde...',
						duration : 0
					}).then(() => $q.resolve(batchCode))
				}).then((batchCode) => {
					const onResponseError = (error) => {
						LogService.error(error);
						$ionicLoading.hide();
						AlertService.confirm('Falha na comunicação com o serviço de lote! Deseja continuar?')
							.then(okButton => {
								if (okButton) {
									item.batchCode = batchCode;
									defered.resolve(item);
								} else {
									defered.reject();
								}
							});
					};

					const onResponseOk = (batchValidationResponse) => {
						$ionicLoading.hide();
						if (batchValidationResponse && batchValidationResponse.validated === true) {
							item.batchCode = batchCode;
                            defered.resolve(item);
						} else {
							AlertService.confirm('Lote não encontrado para os dados informados! Deseja continuar?')
								.then(okButton => {
									if (okButton) {
										item.batchCode = batchCode;
										defered.resolve(item);
									} else {
										defered.reject();
									}
								});
						}
					}

					ProductService.batchValidation({
						storeNumber : ConfigService.config.storeNumber,
						productCode : item.productCode,
						batchCode: batchCode
					}, onResponseOk, onResponseError);
				});
			} else {
				defered.resolve(item);
			}
			return defered.promise;
		}

        service.addExchangeCoupon = (checkCouponQuantitiy, scope) => {
            if (!checkCouponQuantitiy) {
                return $q.resolve(0);
            }

            const totalCartItems = service.cart.fiscalItems.filter(
                i => service.isSoldInStore(i)
            ).reduce(
                (total, current) => MathUtils.add(total, current.displayQuantity, 0), 0
            );

            if (totalCartItems == 0) {
                return $q.resolve(0);
            }

            const defered = $q.defer();

            $ionicLoading.hide();

            AlertService.showNumericPopup(scope, "Cupons de Troca / Presente", `Informe a quantidade de cupons. Mín 1 e Máx.: ${totalCartItems}`)
            .then(couponQuantity => {                
                couponQuantity = parseInt(couponQuantity || 0);

                if (couponQuantity == 0 || couponQuantity > totalCartItems) {
                        AlertService.confirm(`O número de cupons de troca deve ser no mínimo 1 e no máximo ${totalCartItems}. Por favor informe novamente.`)
                                .then(okButton => {
                                    if (okButton) {
                                        service.addExchangeCoupon(checkCouponQuantitiy, scope)
                                        .then(res => defered.resolve(res))
                                        .catch(err => defered.reject(err));
                                    } else {
                                        defered.resolve(0);
                                    }
                                });
                } else {
                    defered.resolve(couponQuantity);
                }
            });

			return defered.promise.then((couponQuantity) =>
                service.params.exchangeReceiptQuantity = couponQuantity
            );
		}

        /**
         *
         */
        service.doAddToCart = function(item) {
            var defered = $q.defer();

            var errorCallback = (error) => {
                AlertService.error(error.message)
                .then(() => defered.reject(error));
            }

            if ('GROUP' === item.productType || 'BUNDLE' === item.productType) {
                $ionicLoading.show({
                    template : 'Consultando itens. Aguarde...',
                    duration : 0
                }).then(() => {
                    ProductService.search({
                        groupCode : item.productCode
                    }, (result) => {
                        if (result.items.length > 0) {
                            const groupCode = ConfigService.config.business.item_multiplication_enabled ? item.productCode : new Date().getTime(); // Com itens mutiplicados, os grupos devem se sobrepor.
                            var quantitySold = item.quantitySold
                            var cartItem;

                            result.items.forEach(itemResult => {
                                itemResult.groupCode = groupCode;
                                itemResult.quantitySold = quantitySold;

                                if (itemResult.productCode === item.productCode) {
                                    itemResult.itemUuid = item.itemUuid;
                                    service.addItem(itemResult, item => cartItem = item, errorCallback);
                                } else {
                                    service.addItem(itemResult, null, errorCallback);
                                }
                            });
                            defered.resolve(cartItem);
                        }
                    }, errorCallback).finally($ionicLoading.hide);
                });
            } else {
                service.checkInjectItemBatch(item)
                    .then(() => service.addItem(item, defered.resolve, errorCallback, null));
            }
            return defered.promise;
        };
        /**
         *
         */
        service.validateIfCartHasPartnerProduct = () => {
            var ret = $q.resolve();

            if (service.cart.fiscalItems.find(item => item.productType === "PARTNER")
                    && ConfigService.config.business.enables_rfid_reading) {
                ret = AlertService.info(`Existe item parceirado! Valide!`);
            }
            return ret;
        };

        service.undoDiscounts = () => {
            if (service.cart.discounts) {
                service.cart.discounts.length = 0;
            }
            if (service.cart.fiscalItems) {
                service.cart.fiscalItems.forEach(fiscalItem => fiscalItem.discounts.length = 0);
            }
            service.updateCartTotalValue();
        };

        service.validateIfCartHasDofOrSaleRestrictionProduct = () => {
            var ret = $q.resolve();

            for (var i = 0; i < service.cart.fiscalItems.length; i++) {
                let item = service.cart.fiscalItems[i];
                //Verificando se ha itens com garantia estendida e exigindo a identificacao do cliente
                if (item.nonFiscalItems) {
                    let isExtendedWarrantyPresent = false;
                    let isTheftFurthInsurance = false;
                    item.nonFiscalItems.forEach((nonFiscalItem) => {
                        if (nonFiscalItem.nonFiscalOperationType == EXTENDED_WARRANTY) {
                            isExtendedWarrantyPresent = true;
                        }
                        if (nonFiscalItem.nonFiscalOperationType == THEFT_FURTH_INSURANCE) {
                            isTheftFurthInsurance = true;
                        }
                    });
                    if (isExtendedWarrantyPresent) {
                        if (!service.cart.customer) {
                            ret = $q.reject(new SaleRestrictionError("Produto com garantia estendida. Identificação do cliente obrigatória."));
                            break;
                        } else if (!service.cart.customer.address) {
                            ret = $q.reject(new SaleRestrictionError("Produto com garantia estendida. Favor informar o endereço do cliente."));
                            break;
                        }
                    } else if (isTheftFurthInsurance) {
                        if (!service.cart.customer) {
                            ret = $q.reject(new SaleRestrictionError("Produto com seguro contra roubo e furto. Identificação do cliente obrigatória."));
                            break;
                        } else if (!service.cart.customer.address) {
                            ret = $q.reject(new SaleRestrictionError("Produto com seguro contra roubo e furto. Favor informar o endereço do cliente."));
                            break;
                        }
                    }
                }

                if (item.restrictions && item.restrictions.find(e => e.type != null && e.type == 'DOF')){
                    if (!service.cart.customer) {
                        ret = $q.reject(new SaleRestrictionError("Produto de Origem Florestal. Identificação do cliente obrigatória."));
                        break;
                    }
                }

                if (item.restrictions && item.restrictions.find(e => e.type != null && e.type == 'SALE_RESTRICTION')) {
                    if (!service.cart.customer) {
                        ret = $q.reject(new SaleRestrictionError("VENDA PROIBIDA PARA MENORES DE 18 ANOS. Identificação do cliente OBRIGATÓRIA"));
                        break;
                    }
                }
            }
            return ret;
        };

        service.findItemByProductCode = function(fiscalItems, productCode) {
            return UtilService.isNotEmptyArray(fiscalItems) ? fiscalItems.find(fiscalItem => fiscalItem.productCode === productCode) : null;
        }

        return service;
    });
})();

class CartItem {
    constructor(item, ConfigService) {
        angular.merge(this, item);
        this.__proto__.ConfigService = ConfigService;
    }

    processedUnitValue() {
        let discountValue = 0;
        if (this.discounts){
            this.discounts.forEach(function(disc) {
                if (disc) {
                    let value = disc.type && disc.type == "PERCENT" ? MathUtils.divide(MathUtils.multiply(disc.value, this.unitValue), 100) : disc.value;
                    discountValue += MathUtils.multiply(value, this.isOneFactorDiscount(disc.source) ? 1 : this.quantitySold);
                }
            }, this);
        }
        if (this.__proto__.ConfigService.config.business.disable_order_redemption_calculation && (this.orderNumber || this.externalOrderCode)) {
            return MathUtils.subtract(this.totalValue, discountValue);
        }
        let total = MathUtils.multiply(this.unitValue, this.quantitySold);
        return MathUtils.subtract(total, discountValue);
    }

    processTotalValue() {
        if (!this.__proto__.ConfigService.config.business.disable_order_redemption_calculation || (!this.orderNumber && !this.externalOrderCode)) {
            this.totalValue = MathUtils.multiply(this.unitValue, this.quantitySold);
        }
        this.totalExhib = this.processedUnitValue();
    }

    addDiscount(discount) {
        if (this.isDiscountValid(this, discount)) {
            discount.sequenceId = this.discounts.length;
            this.discounts.push(discount);
        } else {
            AlertService.error('Desconto inválido');
            throw "Desconto inválido";
        }
    }

    removeDiscount(discount) {
        if (this.discounts == null || this.discounts.length == 0) {
            return;
        }

        this.discounts = this.discounts.filter(d => d != discount);
    }

    removeDiscountBySource(source) {
        for (var i = 0; i < this.discounts.length; i++) {
            if (this.discounts[i].source === source) {
                this.discounts.splice(i--, 1);
            }
        }
    }

    isDiscountValid(discount) {
        var discounts = this.discounts;
        var itemTotalValue = this.totalValue;

        if (discounts !== 'undefined' && discounts !== null) {
            var totalDiscount = 0;

            discounts.forEach(function(discount) {
                var value = discount.value;
                totalDiscount = MathUtils.add(totalDiscount, value);
            });
            if (discount !== 'undefined' && discount !== null) {
                totalDiscount = MathUtils.add(totalDiscount, (discount.value || 0));
            }
            if (totalDiscount >=  itemTotalValue) {
                return false;
            }
        }
        return true;
    }

    set displayQuantity(newQuantity) {
        newQuantity = MathUtils.round(newQuantity, 0);

        if (this.productType == 'MINIMUM_SALE_UNIT') {
            this.quantitySold = MathUtils.multiply(newQuantity, this.minimumSaleUnit, 3);
        } else {
            this.quantitySold = MathUtils.round(newQuantity, 3);
        }
    }

	get displayQuantity() {
        if (this.productType != 'MINIMUM_SALE_UNIT') {
            return this.quantitySold;
        }
        return MathUtils.divide(this.quantitySold, this.minimumSaleUnit, 3);
    }

    isTotalValid(){
        if(this.totalExhib && this.totalExhib <= 0){
            this.discount = [];
            this.processTotalValue();
            AlertService.error(`O produto: ${item.productDescription} não pode ter o desconto maior que seu valor!`);
            return false;
        }
        return true;
    }

    isOneFactorDiscount(source){
        const ONE_FACTOR_DISCOUNT = [
            "CAMPAING",
            "EXTERNAL_TOTAL_VALUE",
            "PLAN",
            "COUPON",
            "CASHBACK",
            "PRORATED"
        ];
        return ONE_FACTOR_DISCOUNT.includes(source);
    }
}
