(function () {
    'use strict';

    angular.module('easyorder.services.cashback', [
            'ngResource',
            'easyorder.services.config'
        ])
        .factory('CashbackService', function ($http, $resource, ConfigService) {
            function CashbackService() {
                
                var service = {
                    companyNumber: 0,
                    storeNumber: 0,
                    url: ''
                }

                var observerCallbacks = [];
                /**
                 * register an observer
                 */
                this.registerObserverCallback = function(id, callback, event) {
                    if (!event || !id || !callback) {
                        throw 'Observador inválido'; 
                    }
                    
                    for (var i = 0; i < observerCallbacks.length; i++) {
                        if (observerCallbacks[i].id === id) {
                            delete(observerCallbacks.splice(i, 1));
                        }
                    }
                    observerCallbacks.push({ id: id, callback: callback, event: event });
                }

                /**
                 * call this when you know 'foo' has been changed
                 */
                this.notifyObservers = function(event) {
                    if (!event) {
                        throw 'Evento não definido';
                    }
                    observerCallbacks.forEach(c => {
                        if (c.event === event && typeof c.callback === 'function') {
                            c.callback();
                        }
                    })
                }

                service.loadSettings = function () {
                    service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                    service.url += ConfigService.config.linxwebservices.host + ':';
                    service.url += ConfigService.config.linxwebservices.port;
                    service.url += '/linx-webservices';
                    service.companyNumber = ConfigService.config.companyNumber;
                    service.storeNumber = ConfigService.config.storeNumber;
                }

                this.getCashbackInfo = function (cashbackInfoRequest, success, error) {
                    service.loadSettings();
                    return $resource(service.url + '/v1/cashback?documentType=' + cashbackInfoRequest.documentType + '&documentValue=' + cashbackInfoRequest.documentValue + '&baseValue=' + cashbackInfoRequest.baseValue).get({}, success, error).$promise;
                }

                this.createCashbackAuthorizationRequest = function (createRequest, success, error) {
                    service.loadSettings();

                    createRequest.cashbackAuthorization.companyNumber = service.companyNumber;
                    createRequest.cashbackAuthorization.storeNumber = service.storeNumber;
                    return $resource(service.url + '/v1/cashback', {}, { send: { method: 'POST' } }).send({}, createRequest, success, error).$promise;
                }

                this.cancelCashbackAuthorizationRequest = function (cancelRequest, success, error) {
                    service.loadSettings();

                    cancelRequest.cashbackAuthorization.companyNumber = service.companyNumber;
                    cancelRequest.cashbackAuthorization.storeNumber = service.storeNumber;
                    
                    return $resource(service.url + '/v1/cashback/undo', {}, { send: { method: 'POST' } }).send({}, cancelRequest, success, error).$promise;
                }
            }
            var service = new CashbackService();
            return service;
        })
})();


/**
 * Objeto de requisição para consulta de saldo Cashback 
 */
class CashbackInfoRequest {
    constructor(docType, docValue, baseValue) {
        this.documentType = docType;
        this.documentValue = docValue;
        this.baseValue = baseValue;
    }
}
class CashbackCreatenRequest {
    constructor() {
        this.cashbackAuthorization = {};
        this.customer = new Customer();
    }
}

/**
 * Objeto de resposta da consulta de Cashback
 */
class CashbackInfo {
    constructor() {
        this.availableValue = null;
        this.minimumPurchaseAmount = null;
        this.customer = new Customer();
        this.email = null;
        this.phoneNumber = null;
    }
}

/**
 * Cliente para objetos de cashback
 */
class Customer {
    constructor() {
        this.name = null;
        this.document = new Document();
    }
}

/**
 * Documento para cliente de cashback
 */
class Document {
    constructor() {
        this.type = 'CPF';
        this.value = null;
    }
}