
(function () {
    'use strict';
    angular.module('easyorder.services.configwatchdog', 
    []).factory('ConfigWatchdog', function($q,
                                           $ionicLoading,
                                           $ionicPopup,
                                           $ionicBackdrop,
                                           $ionicBody,
                                           $timeout,
                                           CartService,
                                           UserService,
                                           AlertService,
                                           LogService,
                                           BarcodeService,
                                           EFTService,
                                           PaymentService) {
        const WARN_MSG = "Um arquivo necessário para execução da aplicação foi excluído, é necessário a reconfiguração do terminal";
        const ANDROID_ROOT_DIR = "/storage/emulated/0/Linx/CaixaRapido/bin";

        document.addEventListener("deviceready", () => {
            // só android hoje
            if (ionic.Platform.isAndroid()) {
                window.plugins.FSWatchdog.watch({
                    id: 1,
                    path: ANDROID_ROOT_DIR,
                    events: window.plugins.FSWatchdog.EVENT.DELETE,
                    success: (arg) => {
                        if (arg.path == null || arg.path.indexOf(".dat") == -1) {
                            return;
                        }

                        LogService.warn(`[ConfigWatchdog] arquivo ${arg.path} excluído...`);

                        if (arg.path == "nsustorage.dat") {
                            logout();
                        }
                    },
                    error: LogService.error
                });
            }
        });

        function logout() {
            $ionicLoading.show({
                template: "Por favor, aguarde...",
                duration: 0
            }).then(BarcodeService.cancel)
              .then(closePopups)
              .then(EFTService.finalizeTransaction)
              .then(() => {
                  // antes de inicializar o primeiro carrinho
                  if (CartService.cart.salePayments == null) {
                      return;
                  }
                  return PaymentService.cancel();
              })
              .then(() => UserService.logout(true))
              .then($ionicLoading.hide)
              .finally(() => {
                  AlertService.warn(WARN_MSG);
              });
        }

        function closePopups() {
            const popupStack = $ionicPopup._popupStack;

            if (popupStack.length > 0) {
                popupStack.forEach(function(popup) {
                    if (popup.isShown === true) {
                        popup.remove();
                        popupStack.pop();
                    }
                });
            }

            $ionicBackdrop.release();

            $timeout(() => {
                $ionicBody.removeClass('popup-open');
                $ionicBody.removeClass('modal-open');
            }, 400, false);
            
            ($ionicPopup._backButtonActionDone || angular.noop)();
        }

        return this;
    });
})();

