﻿(function() {
    "use strict";

    angular.module('easyorder.services.config.update', [
        'ngResource',
        'easyorder.services.config',
        'easyorder.services.log'
    ]).factory("ConfigUpdateService", function(
        $resource, $http, $q, $rootScope, $sce, Idle, ConfigService, LogService, $injector, UtilService
    ) {
        var service = {};


        const loadBaseSSlPinning = (url, obj) => {
            const deferred = $q.defer();

            const headers = {
				'Message-Key': UtilService.createMessageKey(obj),
				'Request-Key': UtilService.createRequestKey(url)
			};

            const options = {
                method: 'post',
                data: obj,
                headers: headers,
                serializer: 'json',
                responseType: 'json'
              };


			window.cordova.plugin.http.sendRequest(url,
				options,
				// onSuccess
				function (result) {
                    configTerminal(result.data);
                    deferred.resolve();
				}, // onFail
				function (response) {
                    const error = UtilService.convertSslPluginErrorResponse(response);
					deferred.reject(error);
				});

			return deferred.promise;
        }
        /**
         *
         */
        service.loadBaseConfigurations = function () {
            var url = ConfigService.config.linxwebservices.secure ? "https://" : "http://";
            url += ConfigService.config.linxwebservices.host + ":";
            url += ConfigService.config.linxwebservices.port;
            url += "/linx-webservices";

            var resDeviceConfig = $resource(url + "/v1/devices/register", {}, {
                queryConfig: {
                    method: "POST"
                }
            });

            var deviceUuid = null;

            if (window.cordova && device.uuid) {
                deviceUuid = device.uuid;
            } else {
                deviceUuid = "1";
            }

            const obj = { type: "UUID", uid: deviceUuid, storeNumber: ConfigService.config.storeNumber };

            if (SSLP) {
                return loadBaseSSlPinning(url + "/v1/devices/register", obj);
            } else {
                return resDeviceConfig.queryConfig({}, obj).$promise
                    .then(function (deviceData) {
                        configTerminal(deviceData);
                    })
                    .catch(function (error) {
                        LogService.error("Erro carregando configurações: ", JSON.stringify(error));
                        return $q.reject(error);
                    });
            }


        };

        /**
         *
         */
        service.loadConfigServerAdd = function() {
            var url = ConfigService.config.linxwebservices.secure ? "https://" : "http://";
            url += ConfigService.config.linxwebservices.host + ":";
            url += ConfigService.config.linxwebservices.port;
            url += "/linx-webservices";

            return $resource(url + "/v1/configuration").get({}).$promise
                .then(function(configData) {
                    configTerminal2(configData);
                })
                .catch(function(error) {
                    LogService.error("Erro carregando configurações: ", JSON.stringify(error));
                    return $q.reject(error);
                });
        };

        service.nextNsu = () => {
            const nextNsuFn = ConfigService.config.business.remote_nsu_sync ?
                remoteNextNsu : localNextNsu;
            const date = new Date();

            return nextNsuFn(date).then(data => {
                ConfigService.nsuControll.updateStorageNSU({
                    nsu: parseInt(data.nsu),
                    date: date.toISOString()
                });
            });
        };

        const localNextNsu = () => {
            return $q.resolve({
                nsu: 99
            });
        };

        const remoteNextNsu = (date) => {
            var url = ConfigService.config.linxwebservices.secure ? "https://" : "http://";
            url += ConfigService.config.linxwebservices.host + ":";
            url += ConfigService.config.linxwebservices.port;
            url += "/linx-webservices";

            return $http.post(`${url}/v1/nsu/next`, {
                    storeInfo: {
                        storeNumber: ConfigService.config.storeNumber,
                        companyNumber: ConfigService.config.companyNumber
                    },
                    terminalNumber: ConfigService.config.terminalNumber,
                    date: date.toJSON()
                })
                .then(res => res.data);
        };


        const loadAppConfigurationsSSlPinning = (url, obj) => {
            const deferred = $q.defer();

            const finalUrl = url + '?' + UtilService.encodeObjectURI(obj)

            const headers = {
                'Request-Key': UtilService.createRequestKey(finalUrl)
            };

            const options = {
                method: 'get',
                headers: headers,
                responseType: 'json'
            };

            window.cordova.plugin.http.sendRequest(finalUrl,
                options,
                // onSuccess
                function (result) {
                    service.loadParameters(result.data);
                    deferred.resolve();
                }, // onFail
                function (response) {
                    const error = UtilService.convertSslPluginErrorResponse(response);
                    deferred.reject(error);
                });

            return deferred.promise;
        }

       /**
        *
        */
        service.loadAppConfigurations = function () {
            var url = ConfigService.config.linxwebservices.secure ? "https://" : "http://";
            url += ConfigService.config.linxwebservices.host + ":";
            url += ConfigService.config.linxwebservices.port;
            url += "/linx-webservices/v1/configuration/app-config";

             const key = {
                 storeNumber: ConfigService.config.storeNumber,
                 terminalNumber: ConfigService.config.terminalNumber,
                 companyNumber: ConfigService.config.companyNumber,
                 appType: ConfigService.config.appType,
                 appVersion: $rootScope.appVersion
             };
            if (SSLP) {
                return loadAppConfigurationsSSlPinning(url, key);
            } else {
                const resServConfig = $resource(url, {}, {
                         queryParams: {
                             method: "GET",
                             isArray: true
                         }
                     });
                     return resServConfig.queryParams(key, service.loadParameters, LogService.error).$promise;
            }
        };

        /**
         *
         */
        var configTerminal = function(deviceData) {
            ConfigService.config.appType = APP_TYPE;
            ConfigService.config.companyNumber = deviceData.companyNumber;
            ConfigService.config.storeNumber = deviceData.storeNumber;
            ConfigService.config.store = deviceData.store;
            ConfigService.config.terminalNumber = deviceData.terminal;
            ConfigService.config.eftStoreNumber = deviceData.eftStoreNumber;
        };

        function emptyObject(arg) {
            return arg || {};
        }
        /**
         *
         */
        service.loadParameters = (result) => {
            var newConfig = new ConfigService.BaseConfig();

            newConfig.configLoaded = true;
            newConfig.appType = ConfigService.config.appType;
            newConfig.companyNumber = ConfigService.config.companyNumber;
            newConfig.store = ConfigService.config.store;
            newConfig.storeNumber = ConfigService.config.storeNumber;
            newConfig.eftStoreNumber = ConfigService.config.eftStoreNumber;
            newConfig.terminalNumber = ConfigService.config.terminalNumber;
            newConfig.linxwebservices = ConfigService.config.linxwebservices;
            newConfig.configurationserver = ConfigService.config.configurationserver;
            newConfig.business = emptyObject(newConfig.business);

            result.forEach(function(element) {
                switch (element.fqdn) {
                    case FQDN_ORDERSERVER_HOST:
                        newConfig.orderserver.host = element.value;
                        break;
                    case FQDN_ORDERSERVER_PORT:
                        newConfig.orderserver.port = element.value;
                        break;
                    case FQDN_ORDERSERVER_SECURE:
                        newConfig.orderserver.secure = element.value;
                        break;
                    case FQDN_TEFSERVER_HOST:
                        newConfig.tefserver.host = element.value;
                        break;
                    case FQDN_TEFSERVER_PORT:
                        newConfig.tefserver.port = element.value;
                        break;
                    case FQDN_TEFSERVER_SECONDARY_HOST:
                        newConfig.tefserver.secondaryHost = element.value;
                        break;
                    case FQDN_TEFSERVER_SECONDARY_PORT:
                        newConfig.tefserver.secondaryPort = element.value;
                        break;
                    case FQDN_TEFSERVER_ESTABLISHMENT_CODE:
                        newConfig.tefserver.establishmentCode = element.value;
                        break;
                    case FQDN_TEFSERVER_PROVIDER:
                        newConfig.tefserver.provider = element.value;
                        break;
                    case FQDN_TEFSERVER_SECURE_COMMUNICATION:
                        newConfig.tefserver.secure = element.value;
                        break;
                    case FQDN_TEFSERVER_CERT_PASS:
                        newConfig.tefserver.certificatePassword = element.value;
                        break;
                    case FQDN_TEFSERVER_CERT_ACTIVATION_KEY:
                       newConfig.tefserver.certificateActivationKey = element.value;
                       break;
                    case FQDN_TEFSERVER_STORE_CNPJ:
                        newConfig.tefserver.storeCnpj = element.value;
                        break;
                    case FQDN_TEFSERVER_STORE_NUMBER:
                        newConfig.tefserver.storeNumber = element.value;
                        break;
                    case FQDN_TEFSERVER_HTTPS_PASSWORD:
                        newConfig.tefserver.httpsPassword = element.value;
                        break;
                    case FQDN_TEFSERVER_HTTPS_PASSWORD_TYPE:
                        newConfig.tefserver.httpsPasswordType = element.value;
                        break;
                    case FQDN_TEFSERVER_SSL_PROVIDER:
                        newConfig.tefserver.sslProvider = element.value;
                        break;
                    case FQDN_THEME:
                        newConfig.theme = element.value;
                        break;
                    case FQDN_PRICE_CHANGE_BEHAVIOR:
                        newConfig.business.price_change_behavior = element.value;
                        break;
                    case FQDN_ITEM_MULTIPLICATION_ENABLED:
                        newConfig.business.item_multiplication_enabled = element.value;
                        break;
                    case FQDN_PRICE_CHANGE_REASONS:
                        newConfig.business.price_change_reasons_list = element.value ? element.value.split("|") : [];
                        break;
                    case FQDN_PAYMENT_PLANS_ENABLED:
                        newConfig.business.payment_plan_enabled = element.value;
                        break;
                    case FQDN_PARCIALS_PAYMENTS_ENABLED:
                        newConfig.business.parcials_payment_enabled = element.value;
                        break;
                    case FQDN_DISPLAYS_INTEREST_INSTALLMENT_AMOUNTS:
                        newConfig.business.displays_interest_installment_amounts = element.value != null ? element.value : true;
                        break;
                    case FQDN_PAYMENT_PLAN_DISPLAY_ORDER:
                        newConfig.business.payment_plan_display_order = element.value;
                        break;
                    case FQDN_POS_CAN_RETRIEVE_ORDER:
                        newConfig.business.pos_can_retrieve_order = element.value;
                        break;
                    case FQDN_CUSTOMER_IDENTIFICATION_TYPES:
                        newConfig.business.customer_identification_types = element.value;
                        break;
                    case FQDN_LOGIN_KEYBOARD_TYPE:
                        newConfig.login.keyboard_type = element.value;
                        break;
                    case FQDN_CAMPAIGN_ENABLED:
                        newConfig.business.campaign_enabled = element.value;
                        break;
                    case FQDN_PRMOTION_ENGINE_ENABLED:
                        newConfig.business.promotion_engine_enabled = element.value;
                        break;
                    case FQDN_GIFTCARD_RECHARGE_METHODS:
                        newConfig.business.giftcard_recharge_methods = (element.value ? element.value.split(/\s|;|,/) : []);
                        break;
                    case FQDN_ORDER_SEARCH_TYPES:
                        newConfig.business.order_search_types = element.value;
                        break;
                    case FQDN_MUST_RELOAD_CUSTOMER:
                        newConfig.business.must_reload_customer = element.value;
                        break;
                    case FQDN_PRODUCT_CODE_CHARACTER_TYPE:
                        newConfig.business.product_code_character_type = element.value.toLowerCase();
                        break;
                    case FQDN_APPLICATION_IDLE_TIMEOUT:
                        const timeout = (element.value && element.value > 60 ? element.value : 60);
                        newConfig.business.application_idle_timeout = timeout;
                        Idle.setTimeout(timeout);
                        break;
                    case FQDN_SELLER_IDENTIFICATION_MODE:
                        newConfig.business.seller_identification_mode = element.value.toLowerCase();
                        break;
                    case FQDN_ENABLE_LOGGED_USER_AS_IDENTIFIED_SELLER:
                        newConfig.business.automatic_seller_identification_enabled = element.value;
                        break;
                    case FQDN_SMS_INFO_ORDER_READY_SENDER_ID:
                        newConfig.sms_info.order_ready.sender_id = element.value;
                        break;
                    case FQDN_SMS_INFO_ORDER_READY_MESSAGE:
                        newConfig.sms_info.order_ready.message = element.value;
                        break;
                    case FQDN_ALLOW_ITEM_AS_GIFT:
                        newConfig.business.allow_item_as_gift = element.value;
                        break;
                    case FQDN_PRODUCT_IMAGES_ENABLED:
                        newConfig.business.product_images_enabled = element.value;
                        break;
                    case FQDN_PRINTING_ENABLED:
                        newConfig.business.printing_enabled = element.value;
                        break;
                    case FQDN_SEND_MAIL_ENABLED:
                        newConfig.business.send_mail_enabled = element.value;
                        break;
                    case FQDN_CUSTOMER_MANDATORY_FIELDS:
                        newConfig.business.customer_mandatory_fields = element.value;
                        break;
                    case FQDN_SHOWROOMING_ENABLED:
                        newConfig.business.showrooming_enabled = element.value;
                        break;
                    case FQDN_ALLOW_GENERATE_ORDER:
                        newConfig.business.allow_generate_order = element.value;
                        break;
                    case FQDN_AVAILABLE_PAYMENT_METHODS:
                        newConfig.business.available_payment_methods = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_AVAILABLE_PAYMENT_METHODS_WITH_CASHBACK:
                        newConfig.business.payment_methods_with_cashback = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_BARCODE_SCANNING_METHOD:
                        newConfig.business.barcode_scanning_method = element.value;
                        break;
                    case FQDN_VOUCHER_BARCODE_SCANNING_METHOD:
                        newConfig.business.voucher_barcode_scanning_method = element.value;
                        break;
                    case FQDN_AVAILABLE_EXCHANGE_OPERATION_METHODS:
                        newConfig.business.available_exchange_operation_methods = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_CUSTOMER_IDENTIFICATION_REQUIRED_ENABLED:
                        newConfig.business.customer_identification_required = element.value;
                        break;
                    case FQDN_COVENANT_IDENTIFICATION_TYPES:
                        newConfig.business.covenant_identification_types = element.value;
                        break;
                    case FQDN_DISCOUNT_COUPON_MODE:
                        newConfig.business.discount_coupon_mode = element.value;
                        break;
                    case FQDN_PRESCRIPTION:
                        newConfig.business.prescription = element.value;
                        break;
                    case FQDN_DISCOUNT_SCREEN_REQUIRED_ENABLED:
                        newConfig.business.enable_screen_discount = element.value;
                        break;
                    case FQDN_ENABLE_TYPE_NFE:
                        newConfig.business.enable_type_nfce = element.value;
                        break;
                    case FQDN_ACCEPT_MORE_DOCUMENTS:
                        newConfig.business.enable_more_documents = element.value;
                        break;
                    case FQDN_PRODUCT_RESERVE_REQUIRED_ENABLED:
                        newConfig.business.enable_reserve_products = element.value;
                        break;
                    case FQDN_PRODUCT_INVENTORY_ENABLED:
                        newConfig.business.product_inventory_enabled = element.value;
                        break;
                    case FQDN_SEARCH_ORDERS_RETURN_LIMITED_TO:
                        newConfig.business.search_orders_return_limited_to = element.value;
                        break;
                    case FQDN_SEARCH_ORDERS_RETURN_DATE_MINUS:
                        newConfig.business.search_orders_return_date_minus = (element.value || 0);
                        break;
                    case FQDN_CUSTOMER_MANDATORY_FIELDS:
                        newConfig.business.customer_mandatory_fields = element.value;
                        break;
                    case FQDN_CPF_IN_NOTE:
                        newConfig.business.cpf_in_note = element.value;
                        break;
                    case FQDN_MOBILE_RECHARGE_ENABLED:
                        newConfig.business.mobile_recharge_enabled = element.value;
                        break;
                    case FQDN_MOBILE_RECHARGE_MODE:
                        newConfig.business.mobile_recharge_mode = element.value;
                        break;
                    case FQDN_SOX_ENABLED:
                        newConfig.business.sox_enabled = element.value;
                        break;
                    case FQDN_CHECK_SIGNIN_BEFORE_SALE:
                        newConfig.business.check_signin_before_sale = element.value;
                        break;
                    case FQDN_PRODUCT_URL_CODE_TYPE:
                        newConfig.business.product_url_code_type = element.value;
                        break;
                    case FQDN_UUID_ENABLED:
                        newConfig.business.uuid_enabled = element.value;
                        break;
                    case FQDN_DEFAULT_HOME_SCREEN:
                        newConfig.business.default_home_screen = element.value;
                        break;
                    case FQDN_ADDITIONAL_SERVICES_ENABLED:
                        newConfig.business.additional_services = element.value;
                        break;
                    case FQDN_DISCOUNT_RULE_AUTOMATIC_APPLICATION:
                        newConfig.business.discount_rule_automatic_application = element.value;
                        break;
                    case FQDN_MANUAL_DISCOUNT:
                        newConfig.business.manual_discount = element.value;
                        break;
                    case FQDN_INTERSTATE_SELLING:
                        newConfig.business.interstate_selling = element.value;
                        break;
                    case FQDN_FISCAL_AUTH_TYPE:
                        newConfig.business.fiscal_auth_type = element.value;
                        break;
                    case FQDN_PRODUCT_CODE_DISPLAY_TYPE:
                        newConfig.business.product_code_display_type = element.value;
                        break;
                    case FQDN_EMAIL_AUTOCOMPLETE_DOMAIN_LIST:
                        newConfig.business.email_autocomplete_domain_list = (element.value ? element.value.split(/\s|;|,/) : []);
                    case FQDN_REQUEST_EMPLOYEE_ID_COMFIRMATION:
                        newConfig.business.request_employee_id_comfirmation = element.value;
                        break;
                    case FQDN_OPERATIONS:
                        newConfig.business.operations = element.value;
                        break;
                    case FQDN_EXCHANGE_OTHER_STORE:
                        newConfig.business.exchange_other_store = element.value;
                        break;
                    case FQDN_EXCHANGE_MAX_DAYS:
                        newConfig.business.exchange_max_days = element.value;
                        break;
                    case FQDN_RETURN_MAX_DAYS:
                        newConfig.business.return_max_days = element.value;
                        break;
                    case FQDN_AVAILABLE_REPORTS:
                        newConfig.business.available_reports = element.value;
                        break;
                    case FQDN_DEFAULT_REQUEST_TIMEOUT:
                        newConfig.business.default_request_timeout = (element.value && element.value > 60 ? element.value : 60);
                        break;
                    case FQDN_AVAILABLE_PAYMENT_METHODS_NAME:
                        newConfig.business.available_payment_methods_names = element.value ?
                            element.value.split(";").reduce((acc, element) => {
                                const splittedElement = element.split("=");
                                acc.push({
                                    id: splittedElement[0],
                                    name: splittedElement[1]
                                });
                                return acc;
                            }, []) : [];
                        break;
                    case FQDN_CUSTOMER_IDENTIFICATION_NAMES:
                        newConfig.business.customer_identification_names = element.value ?
                            element.value.split(";").reduce((acc, element) => {
                                const splittedElement = element.split("=");
                                acc.push({
                                    id: splittedElement[0],
                                    name: splittedElement[1]
                                });
                                return acc;
                            }, []) : [];
                        break;
                    case DISABLED_CREDIT_CARD_BINS:
                        newConfig.business.disabled_credit_card_bins = element.value ?
                            element.value.split(";").reduce((acc, value) => {
                                if (value && value.length == 6) {
                                    acc.push(value);
                                } else {
                                    LogService.warn('Bin do cartão inválido e será ignorado: ' + value);
                                }
                                return acc;
                            }, []) : [];
                        break;
                    case FQDN_CHECK_PERMISSION_MANUAL_DISCOUNT:
                        newConfig.business.check_permission_manual_discount = element.value;
                        break;
                    case FQDN_ENABLE_CUSTOMER_CRUD_BY_REST_API:
                        newConfig.business.enable_customer_crud_by_rest_api = element.value;
                        break;
                    case FQDN_ENABLE_GIVEX_VOUCHER_REDIRECTION_REQUEST:
                        newConfig.business.enable_givex_voucher_redirection_request = element.value;
                        break;
                    case FQDN_ALLOWS_DISCOUNT_RULE_OVERLAP:
                        newConfig.business.allows_discount_rule_overlap = element.value;
                        break;
                    case FQDN_ALLOW_CURRENT_CUSTOMER_EDITING:
                        newConfig.business.allow_current_customer_editing = element.value;
                        break;
                    case FQDN_SHOW_MARKETING_FIELDS:
                        newConfig.business.show_marketing_fields = element.value;
                        break;
                    case FQDN_CUSTOMER_DATA_MARKETING_FIELDS:
                        newConfig.business.customer_data_marketing_fields = element.value;
                        break;
                    case FQDN_REMOTE_NSU_SYNC:
                        newConfig.business.remote_nsu_sync = element.value;
                        break;
                    case FQDN_MOBILE_RECHARGE_OPERATORS:
                        newConfig.business.mobile_recharge_operators = element.value;
                        break;
                    case FQDN_SELF_AUTHORIZER:
                        newConfig.business.self_authorizer = element.value;
                        break;
                    case FQDN_SITEF_QR_NOT_ACCEPTED_VOUCHERS:
                        newConfig.business.sitef_qr_not_accepted_vouchers = (element.value ? element.value.split(/\s|;|,/) : undefined);
                        break;
                    case FQDN_APPLY_CHARGES_ON_PAYMENT_VALUE:
                        newConfig.business.apply_charges_on_payment_value = element.value;
                        break;
                    case FQDN_BUTTON_CHOICE_TIMEOUT:
                        newConfig.business.gui_button_choice_timeout = element.value;
                        break;
                    case FQDN_GIFT_CARD_MANDATORY_CUSTOMER_IDENTIFICATION:
                        newConfig.business.gift_card_mandatory_customer_identification = element.value;
                        break;
                    case FQDN_UNIFIED_CUSTOMER_IDENTIFICATION_ENABLED:
                        newConfig.business.unified_customer_identification_enabled = element.value;
                        break;
                    case FQDN_PAYMENT_LINK_SEND_METHODS:
                        newConfig.business.payment_link_send_methods = (element.value ? element.value.split(/\s|;|,/) : ['WHATSAPP']);
                        break;
                    case FQDN_ENABLES_AUTOMATIC_ADDITION_TO_CART:
                        newConfig.business.enables_automatic_addition_to_cart = element.value;
                        break;
                    case FQDN_DISCOUNT_EMPLOYEE_SEARCH_ENABLED:
                        newConfig.business.discount_employee_search_enabled = element.value;
                        break;
                    case FQDN_AVAILABLE_BILLING_PAYMENT_METHODS:
                        newConfig.business.available_billing_payment_methods = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_ELASTIC_ENABLED:
                        newConfig.business.elastic_enabled = element.value;
                        break;
                    case FQDN_ELASTIC_SERVERS:
                        newConfig.business.elastic_servers = element.value;
                        break;
                    case FQDN_TRANSACTION_SEARCH_DAYS_THRESHOLD:
                        newConfig.business.transaction_search_days_threshold = element.value;
                        break;
                    case FQDN_ENABLES_RFID_READING:
                        newConfig.business.enables_rfid_reading = element.value;
                        break;
                    case FQDN_ENABLES_EXCHANGE_RFID_READING:
                        newConfig.business.enables_exchange_rfid_reading = element.value;
                        break;
                    case FQDN_RFID_READING_POWER:
                        newConfig.business.rfid_reading_power = element.value;
                        break;
                    case FQDN_RFID_WRITING_POWER:
                        newConfig.business.rfid_writing_power = element.value;
                        break;
                    case FQDN_ENABLES_READING_BY_RFID_ONLY:
                        newConfig.business.enables_reading_by_rfid_only = element.value;
                        break;
                    case FQDN_ENABLES_EXCHANGE_BY_RFID_ONLY:
                        newConfig.business.enables_exchange_by_rfid_only = element.value;
                        break;
                    case FQDN_EPC_ENABLE_PREFIX:
                        newConfig.business.epc_enable_prefix = element.value;
                        break;
                    case FQDN_EPC_DISABLE_PREFIX:
                        newConfig.business.epc_disable_prefix = element.value;
                        break;
                    case FQDN_SHOWROOMING_GENERATE_ORDER_ENABLED:
                        newConfig.business.showrooming_generate_order_enabled = element.value;
                      	break;
                    case FQDN_DEFAULT_PRODUCT_SEARCH_OPERATION:
                        newConfig.business.default_product_search_operation = element.value ? element.value : "Barcode";
                        break;
                    case FQDN_IGNORE_MID_SETTINGS:
                        newConfig.business.ignore_mid_settings = element.value;
                        break;
                    case FQDN_CUSTOMER_SEARCH_PROVIDER:
                        newConfig.business.customer_search_provider = element.value;
                        break;
                    case FQDN_MINIMUM_VALUE_FOR_MANDATORY_IDENTIFICATION:
                        newConfig.business.minimum_value_for_mandatory_identification = element.value;
                        break;
                    case FQDN_ITEM_EXCLUSION_BY_BARCODE_SCANNING:
                        newConfig.business.item_exclusion_by_barcode_scanning = element.value;
                        break;
                    case FQDN_ITEM_EXCLUSION_BY_SLIDING:
                        newConfig.business.item_exclusion_by_sliding = element.value;
                        break;
                    case FQDN_ORDER_WITH_DELIVERY_OF_MERGED_ITEMS_ALLOWED:
                        newConfig.business.order_with_delivery_of_merged_items_allowed = element.value;
                        break;
                    case FQDN_SALE_BY_STYLE:
                        newConfig.business.sale_by_style = element.value;
                        break;
                    case FQDN_MULTIPLE_ORDERS_ENABLED:
                        newConfig.business.multiple_orders_enabled = element.value;
                        break;
                    case FQDN_CUSTOM_BARCODE_HANDLER:
                        newConfig.business.custom_barcode_handler = element.value;
                        break;
                    case FQDN_EULA_ENABLED:
                        newConfig.business.enable_end_user_license_agreement = element.value;
                        break;
                    case FQDN_PRICE_OVERRIDE:
                        newConfig.business.price_override = element.value;
                        break;
                    case FQDN_PAYMENT_LINK_ENABLE_COPY_URL_BUTTON:
                        newConfig.business.payment_link_enable_copy_url_button = element.value;
                        break;
                    case FQDN_PAYMENT_PLAN_SEARCH_PROVIDERS:
                        newConfig.payment_plan.search.providers =
                            parsePaymentPlanSearchProviders(element.value);
                        break;
                    case FQDN_PAYMENT_PLAN_SEARCH_FB_PROVIDERS:
                        newConfig.payment_plan.search.fb_providers =
                            parsePaymentPlanSearchProviders(element.value);
                        break;
                    case FQDN_ADDRESS_SERVICE_PROVIDER:
                        newConfig.business.address_service_provider = element.value;
                        break;
                    case FQDN_PAYMENTLINK_SERVICE_PROVIDER:
                        newConfig.business.paymentlink_service_provider = element.value;
                        break;
                    case FQDN_PAYMENTLINK_AVAILABLE_PAYMENT_METHODS:
                        newConfig.business.paymentlink_available_payment_methods = element.value ? element.value.split(/\s|;|,/) : ['CREDIT_CARD'];
                        break;

                    case FQDN_PAYMENTLINK_CONTACT_CONFIRMATION_ENABLED:
                        newConfig.business.paymentlink_contact_confirmation_enabled = element.value;
                        break;
                    case FQDN_VOUCHER_AUTO_SEARCH_FIELDS:
                        newConfig.business.voucher = {
                            auto_search_fields: element.value ? element.value.split(/\s|;|,/) : []
                        }
                        break;
                    case FQDN_REDIRECTS_NEW_SALE_WHEN_PRINTING:
                        newConfig.business.redirects_new_sale_when_printing = element.value;
                        break;
                    case FQDN_PASSWORD_CHANGE_ENABLED:
                        newConfig.business.password_change_enabled = element.value;
                        break;
                    case FQDN_MANDATORY_FIELDS_FOR_FISCAL_IDENTIFICATION:
                        newConfig.business.mandatory_fiscal_fields = element.value;
                        break;
                    case FQDN_MINIMUM_VALUE_FOR_FISCAL_IDENTIFICATION:
                        newConfig.business.minimum_value_for_fiscal_identification = element.value;
                        break;
                    case FQDN_MAXIMUM_SALE_VALUE:
                        newConfig.business.maximum_sale_value = element.value;
                        break;
                    case FQDN_FISCAL_MENU_UFS:
                        newConfig.business.fiscal_menu_ufs = element.value;
                        break;
                    case FQDN_PAF_TIMEOUT:
                        newConfig.business.paf_timeout = element.value;
                        break;
                    case FQDN_PAYMENT_LINK_SCREEN_MODE:
                        newConfig.business.payment_link_screen_mode = element.value;
                        break;
                    case FQDN_ORDER_SEARCH_SCREEN_OPERATIONS:
                        newConfig.business.order_search_screen_operations = element.value;
                        break;
                    case FQDN_FISCAL_CART_ORDER_OPERATIONS:
                        newConfig.business.fiscal_cart_order_operations = parseKeyValueToUppercase(element.value, "Operação de Pedidos no Carrinho Fiscal");
                        break;
                    case FQDN_UNALLOWED_PHONE_NUMBERS:
                        newConfig.business.unallowed_phone_numbers =
                            element.value ? element.value.split(";") : [];
                        break;
                    case FQDN_ORDER_SEARCH_RELOAD_CUSTOMER_BEHAVIOR:
                        newConfig.business.order_search_reload_customer_behavior =
                            element.value || 'V1';
                        break;
                    case FQDN_TRIPLE_DES_KEY:
                        newConfig.business.triple_des_key = element.value;
                        break
                    case FQDN_GIFTCARD_SERVICE_PROVIDER:
                        newConfig.business.giftcard_service_provider = element.value || 'GIVEX';
                        break;
                    case FQDN_GIFTCARD_TYPE:
                        newConfig.business.giftcard_type = element.value;
                        break;
                    case FQDN_PAYMENT_AUTH_METHODS:
                        newConfig.business.payment_auth_methods = parseKeyValueToUppercase(element.value, "Métodos de autorização do pagamento");
                        break;
                    case FQDN_PAYMENT_AUTH_PROVIDERS:
                        newConfig.business.payment_auth_providers = parseKeyValueToUppercase(element.value, "Provedores de autorização do pagamento");
                        break;
                    case FQDN_PAYMENT_AUTH_CONTIGENCY_PROVIDERS:
                        newConfig.business.payment_auth_contingency_providers = parseKeyValueToUppercase(element.value, "Forma de contingência para autorização do pagamento");
                        break;
                    case FQDN_LOYALTY_PROVIDER:
                        newConfig.business.loyalty_provider = element.value;
                        break;
                    case FQDN_LOYALTY_ENABLED:
                        newConfig.business.loyalty_enabled = element.value;
                        break;
                    case FQDN_EXTERNAL_ORDER_NUMBER_KEYBOARD_TYPE:
                        newConfig.business.external_order_number_keyboard_type = element.value || 'Alphanumeric';
                        break;
                    case FQDN_USES_CUSTOMER_IDENTIFIED_AS_FISCAL_CUSTOMER:
                        newConfig.business.use_customer_as_fiscal_customer = element.value;
                        break;
                    case FQDN_PAF_FILE_STORAGE_MODE:
                        newConfig.business.paf_file_storage_mode = element.value;
                        break;
                    case FQDN_AUTO_SELECT_UNIQUE_EVENT:
                        newConfig.business.auto_select_unique_event = element.value;
                        break;
                    case FQDN_PRODUCT_QUERY_SHORTCUT_ON_THE_CART_SCREEN_ENABLED:
                        newConfig.business.product_query_shortcut_on_the_cart_screen_enabled = element.value;
                        break;
                    case FQDN_PRODUCT_REMOVE_SHORTCUT_ON_THE_CART_SCREEN_ENABLED:
                        newConfig.business.product_remove_shortcut_on_the_cart_screen_enabled = element.value;
                        break;
                    case FQDN_MUST_SCAN_ORDER_SOLD_IN_STORE_ITEMS:
                        newConfig.business.scan_order_sold_in_store_items = element.value;
                        break;
                    case FQDN_ALLOW_DIFFERENT_CUSTOMERS_ON_ORDERS:
                        newConfig.business.allow_different_customers_on_orders = element.value;
                        break;
                    case FQDN_REPRINT_OPTIONS:
                        newConfig.business.reprint_options = element.value;
                        break;
                    case FQDN_ENABLES_CONTROL_OF_SINGLE_ITEMS_BY_MARKETING:
                        newConfig.business.enables_control_of_single_items_by_marketing = element.value;
                        break;
                    case FQDN_MARKETING_CODES_TO_CONTROL_THE_ITEMS_INCLUSION:
                        newConfig.business.marketing_codes_to_control_the_items_inclusion = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_DISABLE_ORDER_REDEMPTION_CALCULATION:
                        newConfig.business.disable_order_redemption_calculation = element.value;
                        break;
                    case FQDN_ORDER_STATUS_FILTER:
                        newConfig.business.order_status_filter = element.value ? element.value.split(/\s|;|,/) : [];
                        break;
                    case FQDN_BIOMETRIC_PAYMENT_METHODS:
                        newConfig.business.biometric_payment_methods = element.value ? element.value.trim().toUpperCase().split(';') : [];
                        break;
                    case FQDN_ADD_FISCAL_DOCUMENT_MSG:
                        if (element.value) {
                            newConfig.business.add_fiscal_document_msg = element.value;
                        }
                        break;
                    case FQDN_PAYMENT_INSURANCE_PROVIDER:
                        newConfig.business.payment_insurance_provider = element.value || 'LREN';
                        break;
                    case FQDN_INSTALLMENT_PLAN_PROVIDER:
                        newConfig.business.installment_plan_provider = element.value;
                        break;
                    case FQDN_PROMO_VOUCHER_LABEL:
                        newConfig.business.promo_voucher_label = (element.value) ? element.value.trim().split(',').map(l => l.trim()) : ['Voucher Promo', 'Vouchers Promo'];
                        break;
                    case FQDN_PROMO_VOUCHER_BALANCE_ENABLED:
                        newConfig.business.promo_voucher_balance_enabled = element.value;
                        break;
                    case FQDN_SHOWROOMING_LABEL:
                        newConfig.business.showrooming_label = element.value ? element.value.trim().substr(0, 25) : 'Showrooming';
                        break;
                    case FQDN_ENABLE_EDITING_OF_EXCHANGE_COUPON_QUANTITY:
                        newConfig.business.enable_editing_of_exchange_coupon_quantity = element.value;
                        break;
                    case FQDN_RETURN_SERVICE_PROVIDER:
                        newConfig.business.return_service_provider = element.value;
                        break;
                    case FQDN_INTERSTATE_SELLING_ALLOWED_MODES:
                        newConfig.business.interstate_selling_allowed_modes = element.value ? element.value.split("/") : [];
                        break;
                    case FQDN_NO_FISCAL_IDENTIFICATION_MARKETING_CODES:
                        newConfig.business.no_fiscal_identification_marketing_codes = element.value ? element.value.split(";") : [];
                        break;
                    case FQDN_SKIP_FISCAL_DIALOG_WHEN_OVERLIMIT:
                        newConfig.business.skip_fiscal_dialog_when_overlimit = element.value;
                        break;
                    case FQDN_REPRINT_MAX_INTERVAL_DAYS:
                        newConfig.business.reprint_max_interval_days = element.value;
                        break;
                    case FQDN_STATE_REGISTRATION_ERROR_MSG:
                        if (element.value) {
                            newConfig.business.state_registration_error_msg = element.value;
                        }
                        break;
                        case FQDN_SHOWROOMING_FREIGHT_LABELS:
                                                    newConfig.business.showrooming_freight_labels = parseKeyValueRaw(element.value, "Mensagens de Frete em Showrooming")
                                                        || {PICKUP: "Disponível em ${FREIGHT_DAYS} dia(s)",
                                                            SHIPMENT: "Prazo de ${FREIGHT_DAYS} dia(s)",
                                                            DEFAULT: "Prazo de ${FREIGHT_DAYS} dia(s)"};
                                                    break;
                    case FQDN_ENABLE_IMEI_ICCID_VALIDATION:
                        newConfig.business.enable_imei_iccid_validation = element.value;
                    break;
                    default:
                        break;
                }
            });
            LogService.info("Parâmetros de configuração atuais: ", JSON.stringify(newConfig));
            ConfigService.updateStorage(newConfig);
        };

        function parsePaymentPlanSearchProviders(rawProviders) {
            if (rawProviders == null || rawProviders.length == 0) {
                return null;
            }
            const splitted = rawProviders.split(";");
            const ret = {};

            for (const rawProvider of splitted) {
                const props = rawProvider.split("=");

                if (props.length < 2) {
                    LogService.warn(`Payment Plan Provider configurado incorretamente: ${rawProvider}`);
                    continue;
                }

                const paymentMethod = props[0];

                if (paymentMethod.length == 0) {
                    LogService.warn(`Payment Plan Provider possui PaymentMethod nulo: ${rawProvider}`);
                    continue;
                }

                const rawValues = props[1].split(",");
                const id = rawValues[0];

                if (id.length == 0) {
                    LogService.warn(`Payment Plan Provider possui ID nulo: ${rawProvider}`);
                    continue;
                }

                ret[paymentMethod] = {
                    id: id,
                };

                if (rawValues.length == 1) {
                    continue;
                }

                const timeout = rawValues[1];

                if (isNaN(timeout)) {
                    LogService.warn(`Payment Plan Provider possui timeout invalido, serÃ¡ ignorado: ${rawProvider}`);
                    continue;
                }

                ret[paymentMethod].timeout = timeout;
            }

            return Object.keys(ret).length > 0 ? ret : null;
        }

        function parseKeyValueToUppercase(raw, entityName) {
            if (raw == null || raw.length == 0) {
                return null;
            }
            if (!entityName) {
                throw new Error("Titulo do parâmetro a ser convertido não informado");
            }
            const rawProps = raw.split(";");

            const ret = {};

            for (const rawProp of rawProps) {
                const values = rawProp.trim().split("=");

                if (values.length < 2 || values.some(v => v.trim().length == 0)) {
                    LogService.warn(`${entityName} configurado incorretamente: ${rawProp}`);
                    continue;
                }

                ret[values[0].trim().toUpperCase()] = values[1].trim().toUpperCase();
            }

            return Object.keys(ret).length > 0 ? ret : null;
        }

        function parseKeyValueRaw(raw, entityName) {
            if (raw == null || raw.length == 0) {
                return null;
            }
            if (!entityName) {
                throw new Error("Titulo do parâmetro a ser convertido não informado");
            }
            const rawProps = raw.split(";");

            const ret = {};

            for (const rawProp of rawProps) {
                const values = rawProp.trim().split("=");

                if (values.length < 2 || values.some(v => v.trim().length == 0)) {
                    LogService.warn(`${entityName} configurado incorretamente: ${rawProp}`);
                    continue;
                }

                ret[values[0].trim().toUpperCase()] = values[1].trim();
            }

            return Object.keys(ret).length > 0 ? ret : null;
        }

        return service;
    });
})();
