(function () {
    'use strict';

    angular.module('easyorder.services.covenant', [
        'ngResource',
        'easyorder.services.util',
        'easyorder.services.config',
        'easyorder.services.log'
    ])

    .factory('CovenantService', function ($resource,
                                            $injector,
                                            ConfigService,
                                            CartService,
                                            UtilService,
                                            UserService,
                                            LogService) {
            
        function CovenantService() {
            this.pageSize = 10;
            
            var _url =  '';
            var _currentCovenant = undefined;
            
            /**
             * 
             */
            var _loadSettings = function () {
                _url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                _url += ConfigService.config.linxwebservices.host + ':';
                _url += ConfigService.config.linxwebservices.port;

                _url += '/linx-webservices';
            };
            
            /**
             * Callcback usado para processamento de convênios que nao foram usados como pagamento.
             */
            var processCovenantNotUsedAsPayment = function() {
                let paymentService = $injector.get('PaymentService'); //Evitando referência circular.
                let thisService = $injector.get('CovenantService');
                
                if (!CartService.cart.salePayments.filter(p => p.type === paymentService.SALE_COVENANT_PAYMENT).length) {
                    try {
                        thisService.covenantSaleAuth({}, function(result) {
                            thisService.covenantSaleAuthConfirmation(result.nsu);
                        });
                    } catch(err) {
                        LogService.error(err);
                    }
                }
            };
            
            /**
             * 
             */
            this.currentCovenant = function(covenant) {
                if (covenant) {
                    _currentCovenant = covenant;
                }
                
                if (_currentCovenant) {
                    let paymentService = $injector.get('PaymentService'); //Evitando referência circular.
                    paymentService.registerObserverCallback('covenantOnPaymentComplete', processCovenantNotUsedAsPayment, paymentService.ON_PAYMENT_COMPLETE);
                }
                return _currentCovenant;
            };
            
            /**
             * 
             */
            this.currentCovenantAuth = function(covenantAuth) {
               if (covenantAuth && _currentCovenant.covenantAuth) {
                   _currentCovenant.covenantAuth = covenantAuth;
                   return _currentCovenant.covenantAuth;
               }else if(!covenantAuth && _currentCovenant && _currentCovenant.covenantAuth){
                   return _currentCovenant.covenantAuth;
               }
            }
            
            /**
             *  Limpa dados de convênio
             */
            this.clearCovenant = function() {
                _currentCovenant = undefined;
            };
            
            /**
             * Controle de exibição de opçoes de convênio
             * 
             */
            this.checkAvoidCovenantMenu = function () {
                var covenantState = 'easyorder.covenantmenu';
                var identification_types = ConfigService.config.business.covenant_identification_types;
                identification_types = identification_types ? identification_types.split(";") : [];

                if (identification_types.length == 1) {
                    var identification_type = identification_types[0];

                    switch (identification_type) {
                        case "DPSP":
                            covenantState = 'easyorder.covenant';
                            break;
                        default:
                            covenantState = 'easyorder.covenanttypesidentification';
                            break;
                    }
                } else {
                    covenantState = 'easyorder.covenanttypesidentification';
                }
                UtilService.navigateWithoutHistory(covenantState);
            };
            
            /**
             * Busca de convênios
             * 
             */
            this.searchCovenants = function (pageNumber, querySort, querryFilter, success, error) {
                _loadSettings();

                var params = {
                    pageSize: this.pageSize,
                    page: pageNumber ? pageNumber : 1,
                    sort: querySort ? querySort : { description: "ASC" },
                    filter: querryFilter ? querryFilter : undefined
                };
                return $resource(_url + '/v1/covenant/list/', {}, {
                    'process': {
                        method: 'POST'
                    }
                }).process({}, params, success, error).$promise;
            };
            
            /**
             * Autenticação do usuário em um convênio.
             */
            this.covenantAuthUser = function (params, success, error) {
                _loadSettings();
                let covenant = params.covenantCode.toString();
                (covenant.length == 1) ? covenant = "000" + covenant :
                (covenant.length == 2) ? covenant = "00" + covenant :
                (covenant.length == 3) ? covenant = "0" + covenant : console.log('covenant code ok');
                params.covenantCode = covenant;

                return $resource(_url + '/v1/covenant/user/auth').save({}, params, success, error).$promise;
            };
            
            /**
             *
             */
            this.covenantSaleAuth = function (params, success, error) {
                _loadSettings();
                
                params.covenantCode      = this.currentCovenant().code;
                params.cardNumber        = this.currentCovenantAuth() ? this.currentCovenantAuth().cardNumber : undefined;
                params.nsu               = ConfigService.nsuControll.currentNSU();
                params.operatorCode      = UserService.operatorCode();
                params.products          = CartService.cart.fiscalItems.map(f => {
                    let ret = {};
                    ret.productCode         = f.productCode;
                    ret.productDescription  = f.productDescription;
                    ret.automationCode      = f.automationCode;
                    ret.storeNumber         = CartService.cart.storeNumber;
                    ret.marketingCode       = f.marketingCode;
                    ret.unitValue           = f.unitValue;
                    ret.quantitySold        = f.quantitySold;
                    ret.discounts           = f.discounts;
                    ret.otherValues         = f.otherValues;
                    
                    return ret;
                });
                return $resource(_url + '/v1/covenant/auth').save({}, params, success, error).$promise;
            };
            /**
             *
             */
            this.undoCovenantSaleAuth = function (paymentAuthorizationNSU, success, error) {
                _loadSettings();

                var params = {};
                params.nsu = paymentAuthorizationNSU;

                return $resource(_url + '/v1/covenant/undo').save({}, params, success, error).$promise;
            };
            
            /**
             * Confirma a venda com pagamento de cliente próprio. Essa função também incrementa o NSU para usuá-lo
             * na integração do Log de confirmação no proctrans.
             */
            this.covenantSaleAuthConfirmation = function (paymentAuthorizationNSU, success, error) {
                _loadSettings();

                let params = {};
                if (typeof paymentAuthorizationNSU === 'object') {
                    params = paymentAuthorizationNSU;
                } else {
                    let clonedCart = JSON.parse(JSON.stringify(CartService.cart));
                    params.nsu              = paymentAuthorizationNSU;
                    params.saleNSU          = clonedCart.nsu;
                    params.terminalNumber   = clonedCart.terminalNumber;
                    params.payments         = clonedCart.salePayments.map(p => {
                        let ret = {};
                        ret.type = p.type;
                        return ret;
                    });
                }

                return $resource(_url + '/v1/covenant/confirm').save({}, params, success, error).$promise;
            };
        }
        var service = new CovenantService();
        /**
         * Registrando função para o caso de reinicialização do carrinho.
         */
        CartService.initCartObservers.add(service.clearCovenant);
        
        return service;
    });
})();
