(function () {
    'use strict';

    angular.module('easyorder.services.customer', [
            'ngResource',
            'easyorder.services.util',
            'easyorder.services.config',
            'easyorder.services.covenant'
        ])

        .factory('CustomerService', function ($resource,
            $ionicModal,
            $rootScope,
            ConfigService,
            UtilService,
            UserService,
            AlertService,
            $q) {
            var service = {
                companyNumber: 0,
                storeNumber: 0,
                url: ''
            };
            var observerCallbacks = [];

            //register an observer
            service.registerObserverCallback = function (id, callback) {
                for (var i = 0; i < observerCallbacks.length; i++) {
                    if (observerCallbacks[i].id === id) {
                        delete(observerCallbacks.splice(i, 1));
                    }
                }
                observerCallbacks.push({
                    id: id,
                    callback: callback
                });
            };

            // call this when you know 'foo' has been changed
            service.notifyObservers = function () {
                angular.forEach(observerCallbacks, function (reg) {
                    reg.callback();
                });
            };

            var getStoreAddress = function (store) {
                if (store && store.address && store.address.state && store.address.state.trim()) {
                    return store.address.state.trim().toUpperCase();
                } else {
                    return null;
                }
            }

            var getCustomerAddress = function (customer) {
                if (customer && customer.address && customer.address.state && customer.address.state.trim()) {
                    return customer.address.state.trim().toUpperCase();
                } else {
                    return null;
                }
            }


            var getCustomerDocumentType = function (customer) {
                if (customer && customer.documentType && customer.documentType.trim()) {
                    return customer.documentType.trim();
                } else {
                    return null;
                }
            }

            var isForeignDocumentType = function (documentType) {
                return (documentType && (documentType === 'PASSPORT' || documentType === 'RNE'));
            }

            var isInterStateSaleAllowed = function(){
                const interStateSelling         = ConfigService.config.business.interstate_selling;
                const interStateSellingModes    = ConfigService.config.business.interstate_selling_allowed_modes;
                const isNfceEnabled             = ConfigService.config.business.enable_type_nfce;

                var allow = false;

                if (interStateSellingModes.length > 0) {
                    interStateSellingModes.forEach( taxMode => {
                        if (taxMode === 'NFC-e' && isNfceEnabled ) {
                            allow = true;
                        } else if (taxMode === 'NF-e' && !isNfceEnabled ) {
                            allow = true;
                        }
                    })
                } else if (interStateSelling) {
                    allow = true;
                }

                return allow;
            }
            /**
             *
             */
            service.isCustomerSaleAllowed = function (customer, store) {
                var interStateSelling = isInterStateSaleAllowed();

                var customerSaleAllowed = true;
                // estrangeiro
                var foreignDocumentType = isForeignDocumentType(getCustomerDocumentType(customer));
                // só efetua a validação se das UF's se parâmetro tiver habilitado e não for estrangeiro
                if (!interStateSelling && !foreignDocumentType) {
                    var storeAddress = getStoreAddress(store);
                    // validação efetuada apenas se a loja tiver UF válida
                    if (storeAddress) {
                        var customerAddress = getCustomerAddress(customer);
                        if (customerAddress) {
                            // se cliente tem UF válida, checa
                            customerSaleAllowed = (customerAddress === storeAddress);
                        } else {
                            // se cliente não tem UF válida, não permite a venda.
                            customerSaleAllowed = false;
                        }
                    }
                }

                return customerSaleAllowed;
            };

            service.showCustomerDetails = function (customer, mode, canConfirm) {
                const isCustomerSaleAllowed = service.isCustomerSaleAllowed(customer, ConfigService.config.store);

                if (isCustomerSaleAllowed) {
                    const $scope = $rootScope.$new();
                    $scope.canConfirm = canConfirm;
                    $scope.employee_id_comfirmation =  ConfigService.config.business.request_employee_id_comfirmation;

                    if (mode === 'SALE') {
                        $scope.cancelDesc = 'Fechar';
                        $scope.confirmDesc = 'Remover';
                        $scope.employeeActvate = false;
                    } else if ('SELECTION') {
                        $scope.cancelDesc = 'Cancelar';
                        $scope.confirmDesc = 'Selecionar';
                        $scope.employeeActvate = true;
                    } else {
                        throw new Error('MODE NOT IMPLEMENTED');
                    }

                    $scope.customerDetails = customer;
                    $scope.crudIsEnabled = ConfigService.config.business.enable_customer_crud_by_rest_api;

                    $scope.editCustomer = () => {
                        $rootScope.$broadcast('editCustomerInProgress', true);
                        UtilService.navigateWithoutHistory('easyorder.customer', { customerToEdit : customer });
                        $scope.modal.remove();
                    };

                    $ionicModal.fromTemplateUrl('./partials/customerModalDetails.html', {
                        scope: $scope,
                        animation: 'slide-in-up'
                    }).then(modal => {
                        $scope.modal = modal;
                    }).then(() => {
                        $scope.modal.show();
                    });

                    $scope.$on('$destroy', () => {
                        $scope.modal.remove();
                    });

                    $scope.__closeModal = () => {
                        return $scope.modal.hide();
                    };

                    const deferred = $q.defer();
                    $scope.confirm = () => {
                        if ($scope.employeeActvate && $scope.employee_id_comfirmation) {
                            if ($scope.customerDetails.registration === $scope.customerDetails.registrationConfirm ) {
                              $scope.__closeModal().then(() => {
                                    deferred.resolve(true);
                              });
                            } else {
                                AlertService
                                    .warn('Matrícula inválida!');
                                }
                        } else {
                            $scope.__closeModal().then(() => {
                                deferred.resolve(true);
                            });
                        }
                    };
                    $scope.cancel = () => {
                        $scope.__closeModal().then(() => {
                            deferred.resolve(false);
                        });
                    };

                    return deferred.promise;
                } else {
                    return AlertService
                        .warn('Não é permitida a venda para clientes de outros estados!</b>')
                        .then(() => {
                            return false;
                        });
                }
            };

            service.getAddress = function (params, success, error) {
                service.loadSettings();
                const addressProvider = ConfigService.config.business.address_service_provider;
                return $resource(service.url + '/v1/address/search?zipcode=' + params, {}, {
                    search: {
                        method: 'GET',
                        headers: {
                            'Service-Provider': addressProvider
                        }
                    }
                }).search({}, success, error).$promise;
            };

            /**
             *
             */
            service.checkAvoidCustomerMenu = function () {
                var customerState = 'easyorder.customermenu';
                var identification_types = ConfigService.config.business.customer_identification_types;
                identification_types = identification_types ? identification_types.split(";") : [];

                if (ConfigService.config.business.unified_customer_identification_enabled){
                    customerState = 'easyorder.customer';
                } else if (identification_types.length == 1) {
                    var identification_type = identification_types[0];

                    switch (identification_type) {
                        case "CPF":
                            customerState = 'easyorder.customercpfsearch';
                            break;
                        case "TYPED_CARD":
                            customerState = 'easyorder.customercardnumbersearch';
                            break;
                        case "SCANNED_CARD":
                        case "SCANNED_CPF":
                            customerState = 'easyorder.customerbarcodesearch';
                            break;
                        case "CHIP":
                            customerState = 'easyorder.customerchipsearch';
                            break;
                        case "DIRECT":
                            customerState = 'easyorder.customer';
                            break;
                        case "REGISTRATION":
                            customerState = 'easyorder.customerregistrationnumbersearch';
                            break;
                        default:
                            break;
                    }
                }
                UtilService.navigateWithoutHistory(customerState);
            }

            /**
             *
             */
            service.loadSettings = function () {
                service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                service.url += ConfigService.config.linxwebservices.host + ':';
                service.url += ConfigService.config.linxwebservices.port;
                service.url += '/linx-webservices';
                service.companyNumber = ConfigService.config.companyNumber;
                service.storeNumber = ConfigService.config.storeNumber;
            };

            /**
             *
             */
            service.status = function () {
                service.loadSettings();
                return $resource(service.url + '/status').get();
            };

            /**
             *
             */
            service.search = function (params, success, error) {
                service.loadSettings();

                params.storeNumber = service.storeNumber;
                params.operatorCode = UserService.operatorCode();
                params.componentNumber = ConfigService.config.terminalNumber;
                params.transactionNSUNumber = ConfigService.nsuControll.currentNSU();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }

                return $resource(service.url + '/v1/customer/list').get(params, success, error).$promise;
            };

            service.searchDiscount = function(value, type){
                service.loadSettings();

                const params = {};
                params.document = value;
                params.documentType = 'CPF';
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }

                return $resource(service.url + '/v1/customer/discounts')
                    .get(params).$promise
            };

            service.searchByType = (value, type, provider) => {
                service.loadSettings();

                const params = {};
                params.searchValue = value;
                params.searchType = type;
                params.storeNumber = service.storeNumber;
                params.operatorCode = UserService.operatorCode();
                params.componentNumber = ConfigService.config.terminalNumber;
                params.transactionNSUNumber = ConfigService.nsuControll.currentNSU();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }
                if (provider) {
                    params.provider = provider;
                }

                return $resource(service.url + '/v1/customer/list')
                    .get(params)
                    .$promise
                    .then(ret => ret.items ? ret.items :
                        Promise.reject(new CustomerNotFoundError('Resposta inválida!')), error => {
                            switch (error.status) {
                                case 404:
                                    return Promise.reject(new CustomerNotFoundError(error.data.message));
                                case 500:
                                    return Promise.reject(new InternalServerError(error.data.message));
                                default:
                                    return Promise.reject(new Error(error.data ? error.data.message || 'Falha ao buscar cliente.' : 'Falha ao buscar cliente.'));
                            }
                        });
            };

            service.save = (customer, success, error) => {
                service.loadSettings();

                var response = $resource(service.url + '/v1/customer', {} , {
                    'process': {
                        method: 'POST'
                    }
                });

                return response.process({}, customer, success, error).$promise;
            }

            service.update = (customer, success, error) => {
                service.loadSettings();

                var response = $resource(service.url + '/v1/customer', {} , {
                    'process': {
                        method: 'PUT'
                    }
                });

                return response.process({}, customer, success, error).$promise;
            }

            service.searchCoupon = function (params, success, error) {
                service.loadSettings();
                service.url += '/v1/discountcoupon/search/';
                var respose = $resource(service.url, {}, {
                    'process': {
                        method: 'POST'
                    }
                });
                return respose.process({}, params, success, error).$promise;
            };

            /**
             *
             */
            service.authPrivateLabelPayment = function (params, success, error) {
                service.loadSettings();

                params.storeNumber = service.storeNumber;
                params.operatorCode = UserService.operatorCode();
                params.componentNumber = ConfigService.config.terminalNumber;
                params.transactionNSUNumber = ConfigService.nsuControll.currentCartNSU();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }

                return $resource(service.url + '/v1/customer/auth').save({}, params, success, error);
            };

            /**
             *
             */
            service.undoPrivateLabelAuthorization = function (sale, success, error) {
                service.loadSettings();

                var params = {};
                params.sale = sale;
                params.nsuLogTransaction = ConfigService.nsuControll.incrementNSU();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }

                if (sale.insuranceData) {
                    sale.insuranceData.nsu = ConfigService.nsuControll.incrementNSU();
                }
                return $resource(service.url + '/v1/customer/undo').save({}, params, success, error).$promise;
            };

            /**
             * Confirma a venda com pagamento de cliente próprio. Essa função também incrementa o NSU para usuá-lo
             * na integração do Log de confirmação no proctrans.
             */
            service.confirmPrivateLabelPayment = function (sale, success, error) {
                service.loadSettings();

                var params = {};
                params.sale = sale;
                params.nsuConfirmationLog = ConfigService.nsuControll.incrementNSU();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }

                return $resource(service.url + '/v1/customer/confirm').save({}, params, success, error).$promise;
            };

            service.isIdentified = customer => {
                return customer && (customer.customerId || service.isFiscalIdentified(customer));
            }

            service.isFiscalIdentified = customer => {
                return customer && service.isDocumentValid(customer.document, customer.documentType);
            }

            service.isDocumentValid = (document, documentType) => {
                switch (documentType) {
                    case 'PASSPORT':
                        return (!service.isEmptyOrSpaces(document) && validaPassport(document));
                    case 'CPF':
                        return (!service.isEmptyOrSpaces(document) && validaCPF(document));
                    case 'CNPJ':
                        return (!service.isEmptyOrSpaces(document) && validaCNPJ(document));
                    case 'RNE':
                        return (!service.isEmptyOrSpaces(document) && validaRNE(document));
                    default:
                        return false; // throw error?
                }
            }

            service.isEmptyOrSpaces = str => {
                return str != undefined ? str === null || str.match(/^\s*$/) !== null : true;
            }

            service.searchEula = function (params, success, error) {
                service.loadSettings();
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }
                return $resource(service.url + '/v1/customer/eula').get(params, success, error).$promise;
            };

            service.confirmEula = function (params, success, error) {
                service.loadSettings();
                service.url += '/v1/customer/eula';
                if(ConfigService.config.business.customer_search_provider != null){
                    params.provider = ConfigService.config.business.customer_search_provider;
                }
                return $resource(service.url, {}, { 'process': { method: 'POST' } }).process({}, params, success, error).$promise;
            };

            service.confirmCoupon = function (params, success, error) {
                service.loadSettings();
                service.url += '/v1/discountcoupon/confirmation/';
                var respose = $resource(service.url, {}, {
                    'process': {
                        method: 'POST'
                    }
                });
                return respose.process({}, params, success, error);
            };

            return service;
        })
})();
