/**
 * 
 */
function FileDataSourceProvider($injector) {
    const fileServConfig = "caixarapido.servconfig";
    const fileSystemData = "caixarapido.systemdata";
    const fileMideClientInfo = "caixarapido.mideclientinfo";
    const fileNsuStorage = "caixarapido.nsustorage";
    
    const fileServConfigDat = "servconfig.dat";
    const fileSystemDataDat = "systemdata.dat";
    const fileNsuStorageDat = "nsustorage.dat";
    const fileEftActivationDat = "eftactivation.dat"
    const fileExtCfg = "caixarapido.config";

    const key = "e$h#xR&x9O60d(2#jdE023";

    const $cordovaFile = $injector.get('$cordovaFile');
    const $q = $injector.get('$q');
    const logService = $injector.get('LogService');
    const service = $injector.get('ConfigService');
    const $rootScope = $injector.get('$rootScope');
    const eftService = $injector.get('EFTService');
    const utilService = $injector.get('UtilService');

    const rootFilePath = $rootScope.rootFilePath;
    const fullRootFilePathBin = $rootScope.fullRootFilePathBin;
    const fullRootFilePathPaf = `${$rootScope.fullRootFilePath}/paf`;
    const fullRootFilePathCfg = $rootScope.fullRootFilePathConfig;
    const FILE_PROTOCOL = "file://";
    let pafFolderCreated = false;
    
    if (!$rootScope.isCordova) {
        throw 'Provider de arquivos utiliza Cordova, mas ele não foi encontrado.';
    }
    
    this.persist = function() {
        var deferred = $q.defer();
        var loadFiles = [
            this.createServConfigStorage().then(() => this.updateServConfigStorage(), $q.reject),
            this.deleteMideClientInfoStorage(),
            this.createSystemDataStorage().then(() => this.updateSystemDataStorage(), $q.reject),
            this.createNSUStorage().then(() => this.updateNSUStorage(), $q.reject),
            this.updateEftActivationStorage()
        ];
        
        $q.all(loadFiles).then(deferred.resolve, deferred.reject);

        return deferred.promise;
    };
    
    /**
     * 
     */
    this.load = function() {
        var deferred = $q.defer();
        var loadFiles = [
            this.loadServConfigStorage(),
            this.loadSystemDataStorage(),
            this.loadNSUStorage(),
            this.loadEftActivationStorage(),
            this.loadExtCfgStorage()
        ];

        $q.all(loadFiles).then(deferred.resolve, deferred.reject);
        
        return deferred.promise;
    };
    

    this.removeFile = function(file) {
        var deferred = $q.defer();
        
        $cordovaFile.removeFile(fullRootFilePathBin, file).then(() => {
            logService.info(`Arquivo '${file}' removido.`);
            deferred.resolve();
        }, (error) => {
            logService.error(`Arquivo '${file}' não foi removido.`, error);
            // ignorando erro na remoção
            deferred.resolve();
        });
        return deferred.promise;
    };

    /**
     * 
     */
    this.remove = function() {
        var deferred = $q.defer();
        
        var removeFiles = [
            this.removeFile(fileServConfigDat),
            this.removeFile(fileSystemDataDat),
            this.removeFile(fileEftActivationDat)
        ];

        $q.all(removeFiles).then(deferred.resolve, deferred.reject);
        return deferred.promise;
    };
    
    /**
     * Executa leitura do arquivo de NSU, atualizando os dados na mémória.
     */
    this.loadNSUStorage = function() {
        const deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileNsuStorageDat).then(() => {
            $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, fileNsuStorageDat).then((encryptedNsuFile) => {
                const nsuFile = this.decryptArrayBufferContent(encryptedNsuFile);
                service.nsuControll.updateStorageNSU(JSON.parse(nsuFile));
                service.nsuControll.currentNSU(); // Garantindo que os dados estão atualizados na memória .
                
                logService.info("Informações do arquivo de NSU carregadas: " + nsuFile);
                deferred.resolve();
            }, (error) => {
                logService.error("Não foi possível carregar as informações do arquivo NSU", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("O arquivo para NSU não existe!", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };
    
    /**
     * Cria o arquivo de storage de NSU se não existir.
     */
    this.createNSUStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileNsuStorageDat).then(deferred.resolve, () => {
            $cordovaFile.createFile(fullRootFilePathBin, fileNsuStorageDat, true).then(() => {
                logService.info(`Arquivo '${fileNsuStorageDat}' criado.`);
                deferred.resolve();
            }, (error) => {
                logService.error("Não foi possivel criar o arquivo para NSU.", error);
                deferred.reject(error);
            });
        });
        return deferred.promise;
    };
    
    /**
     * Atualiza os dados do arquivo de NSU. A promessa retorna é rejeitada se o arquivo não existir.
     */
    this.updateNSUStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileNsuStorageDat).then(() => {
            const jsonStorage = JSON.stringify(service.nsuControll.getNSUStorage());
            const encryptedBlob = this.encryptContentAsBlob(jsonStorage);
            
            $cordovaFile.writeFile(fullRootFilePathBin, fileNsuStorageDat, encryptedBlob, true).then((success) => {
                logService.info("Informações salvas no arquivo para o nsu: " + jsonStorage);
                deferred.resolve();
            }, (error) => {
                logService.error("Ocorreu um erro ao gravar as informações no arquivo nsu", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("O arquivo para NSU não existe!", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };

    /**
     * 
     */
    this.checkNsuStorage = () => {
        const deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileNsuStorageDat).then(() => {
            $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, fileNsuStorageDat).then((encryptedNsuFile) => {
                try {
                    const nsuFile = this.decryptArrayBufferContent(encryptedNsuFile);
                    const storage = JSON.parse(nsuFile);
            
                    logService.info("Informações do arquivo de NSU válidas.");
                    deferred.resolve(storage);
                } catch (e) {
                    logService.error("Informações do arquivo de NSU inválidas: ", nsuFile, e);
                    deferred.reject(e);
                }
            }, (error) => {
                logService.error("Não foi possível carregar as informações do arquivo NSU", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("O arquivo para NSU não existe!", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };
    
    /**
     * Carrega os dados de configuracão na memória.
     */
    this.loadServConfigStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileServConfigDat).then((success) => {
            $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, fileServConfigDat).then((encryptedServFile) => {
                const servFile = this.decryptArrayBufferContent(encryptedServFile);
                if (servFile) {
                    angular.merge(service.config, JSON.parse(servFile));
                    logService.info("serv config carregados do arquivo: " + servFile);
                    deferred.resolve();
                } else {
                    deferred.reject(`Não foi possível descriptografar o arquivo '${fileServConfigDat}'.`);
                }
            }, function(error) {
                logService.error("Não foi possivel carregar as informações no arquivo serv config", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("O arquivo para serv config não existe.", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };
    
    /**
     * 
     */
    this.createServConfigStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileServConfigDat).then(deferred.resolve, () => {
            $cordovaFile.createFile(fullRootFilePathBin, fileServConfigDat, true).then((success) => {
                logService.info(`Arquivo '${fileServConfigDat}' criado.`);
                deferred.resolve();
            }, (error) => {
                logService.error("Não foi possivel criar arquivo Serv Config", error);
                deferred.reject(error);
            });
        });
        return deferred.promise;
    };
    
    /**
     * Atualiza os dados no arquivo de parâmetros.
     */
    this.updateServConfigStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileServConfigDat).then((success) => {
            const encryptedBlob = this.encryptContentAsBlob(JSON.stringify(service.config));
            $cordovaFile.writeFile(fullRootFilePathBin, fileServConfigDat, encryptedBlob, true).then((success) => {
                logService.info("Informações salvas no arquivo para o Serv Config");
                deferred.resolve();
            }, (error) => {
                logService.error("Ocorreu um erro ao gravar as informações no arquivo Serv Config", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("O arquivo para serv config não existe.", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };
    
    /**
     * Carrega os dados do arquivo do System Data na memória.
     */
    this.loadSystemDataStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileSystemDataDat).then((success) => {
            $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, fileSystemDataDat).then((encryptedSystemDataFile) => {
                const systemDataFile = this.decryptArrayBufferContent(encryptedSystemDataFile);
                if (systemDataFile) {
                    angular.merge(service.systemData, JSON.parse(systemDataFile))
                    logService.info("systemData carregados do arquivo: " + systemDataFile);
                    deferred.resolve();
                } else {
                    deferred.reject(`Não foi possível descriptografar o arquivo '${fileSystemDataDat}'.`);
                }
            }, (error) => {
                logService.error("Não foi possivel carregar as informações no arquivo system data", error);
                deferred.reject(error);
            });
        }, (error) => {
            logService.error("arquivo system data não existe!", error);
            deferred.reject(error);
        });
        return deferred.promise;
    };
    
    /**
     * Cria arquivo do System Data. 
     */
    this.createSystemDataStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileSystemDataDat).then(deferred.resolve, () => {
            $cordovaFile.createFile(fullRootFilePathBin, fileSystemDataDat, true).then((success) => {
                logService.info(`Arquivo '${fileSystemDataDat}' criado.`);
                deferred.resolve();
            }, (error) => {
                logService.error(error + "Não foi possivel criar arquivo systemdata");
                deferred.reject(error);
            });
        });
        return deferred.promise;
    };
    
    /**
     * Atualiza os dados no arquivo do System Data. 
     */
    this.updateSystemDataStorage = function() {
        var deferred = $q.defer();
        const encryptedBlob = this.encryptContentAsBlob(JSON.stringify(service.systemData));
        
        $cordovaFile.writeFile(fullRootFilePathBin, fileSystemDataDat, encryptedBlob, true).then(function(success) {
            logService.info("Informações salvas no arquivo para o systemdata");
            deferred.resolve();
        }, function(error) {
            logService.error(error + "Ocorreu um erro ao gravar as informações no arquivo systemdata");
            deferred.reject(error);
        });
        
        return deferred.promise;
    };

    this.deleteMideClientInfoStorage = function() {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileMideClientInfo).then((success) => {
            $cordovaFile.removeFile(fullRootFilePathBin, fileMideClientInfo).then((success) => {
                logService.info("Arquivo mideclientinfo removido!");
                deferred.resolve();
            }, (error) => {
                logService.error("Ocorreu um erro ao remover arquivo Mideclientinfo", error);
                deferred.resolve();
            });
        }, (error) => {
            // ignorando pois o arquivo pode já ter sido removido
            deferred.resolve();
        });
        return deferred.promise;
    };

    this.updateEftActivationStorage = async () => {
        const eftActivation = eftService.getActivation();

        if (eftActivation == null) {
            const isEftActivationCreated = await this.fileExists(fullRootFilePathBin, fileEftActivationDat);

            if (!isEftActivationCreated) {
                return
            }

            return $cordovaFile.removeFile(fullRootFilePathBin, fileEftActivationDat);
        }

        const encryptedEftActivation = this.encryptContentAsBlob(JSON.stringify(eftActivation));
        return this.createAndSaveFile(fullRootFilePathBin, fileEftActivationDat, encryptedEftActivation);
    };

    this.loadEftActivationStorage = async () => {
        const isEftActivationCreated = await this.fileExists(fullRootFilePathBin, fileEftActivationDat);

        if (!isEftActivationCreated) {
            logService.info('Ativação do TEF não será carregada, não existem arquivos de ativação criados');
            return;
        }

        const encryptedEftActivation = await $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, fileEftActivationDat);

        if (!encryptedEftActivation) {
            logService.info('Ativação do TEF não será setada, não existem dados de ativação criados');
            return;
        }

        const decryptedEftActivation = this.decryptArrayBufferContent(encryptedEftActivation);
        eftService.setActivation(JSON.parse(decryptedEftActivation));
    };

    this.loadExtCfgStorage = async () => {
        const isExtCfgCreated = await this.fileExists(fullRootFilePathCfg, fileExtCfg);
        $rootScope.linxWebservicesExternallyConfigured = false;

        if (!isExtCfgCreated) {
            logService.warn("Arquivo de configuração externo não encontrado");
            return;
        }

        try {
            const rawCfg = await $cordovaFile.readAsText(fullRootFilePathCfg, fileExtCfg);

            if (!rawCfg || rawCfg.length == 0) {
                logService.warn('Arquivo de configuração externo encontrado, porém vazio');
                return;
            }

            logService.info(`Arquivo de configuração externo encontrado: ${rawCfg}`);

            const lines = rawCfg.split('\n');
            let linxWebservicesHost = null;
            let linxWebservicesPort = null;

            // parse dos properties
            for (let lineNumber = 0; lineNumber < lines.length; lineNumber++) {
                const line = lines[lineNumber].trim();
                const values = line.split('=');

                if (values.length == 2) {
                    const key = values[0].trim();
                    const value = values[1].trim();

                    if (key && value) {
                        if (key === 'linx-webservices.host') {
                            linxWebservicesHost = value;
                        } else if (key === 'linx-webservices.port') {
                            if (utilService.isInteger(value)) {
                                linxWebservicesPort = value;
                            } else {
                                logService.info("Porta do Linx Webservices invalida: '" + value + "'.");
                            }
                        } else if (key == 'linx-webservices.storeNumber') {
                            if (utilService.isInteger(value)) {
                                linxWebservicesStoreNumber = value;
                            } else {
                                logService.info("Número da Loja do Linx Webservices inválida: '" + value + "'.");
                            }
                        } else {
                            logService.info("Chave '" + key + "' de valor '" + value + "' não reconhecida!");
                        }
                    } else {
                        logService.info("Chave ou valor inválidos na linha '" + line + "'.");
                    }
                }
            }

            if (linxWebservicesHost && linxWebservicesPort) {
                logService.info(
                    "Configurando info do LWS através do arquivo de configuração: Host: '"
                    + linxWebservicesHost + "' - Porta: '" + linxWebservicesPort + "'."
                );
                service.config.linxwebservices = {
                    host: linxWebservicesHost,
                    port: linxWebservicesPort,
                    secure: true
                };
                $rootScope.linxWebservicesExternallyConfigured = true;
            }

            if (linxWebservicesStoreNumber) {
                logService.info(`Configurando Número da Loja do LWS através do arquivo de configuração: ${linxWebservicesStoreNumber}.`);
                service.config.storeNumber = linxWebservicesStoreNumber;
                $rootScope.linxWebservicesExternallyConfigured = true;
            }
        } catch (err) {
            logService.error('Falha durante leitura do arquivo de configuração externo');
            logService.error(err);
            return;
        }
    };

    this.initialize = function() {
        if (!$rootScope.isCordova) {
            return $q.resolve();
        }
        const deferred = $q.defer();

        window.resolveLocalFileSystemURL($rootScope.rootFilePath, (fileSystem) => {
            //Cria o diretório Linx
            fileSystem.getDirectory('Linx', { create: true }, (subFile) => {
                //Cria o diretório Linx/CaixaRapido
                subFile.getDirectory('CaixaRapido', { create: true }, (subFileLog) => {
                    //Cria o diretório Linx/CaixaRapido/bin
                    subFileLog.getDirectory('bin', { create: true }, (_) => {
                        deferred.resolve();
                    }, function (error) {
                        deferred.reject(error);
                    });
                }, function(error) {
                    deferred.reject(error);
                });
            }, function(error) {
                deferred.reject(error);
            });
        }, function(error) {
            deferred.reject(error);
        });

        return deferred.promise;
    }

    this.doDataSourceMigration = function() {
        const migrationPromises = [
            this.moveFile(fileNsuStorage),
            this.moveFile(fileServConfig),
            this.moveFile(fileSystemData),
            this.migrateEncryptFile(fileNsuStorage, fileNsuStorageDat),
            this.migrateEncryptFile(fileServConfig, fileServConfigDat),
            this.migrateEncryptFile(fileSystemData, fileSystemDataDat),
            this.migrateEftActivation(fileServConfigDat, fileEftActivationDat)
        ];

        return $q.all(migrationPromises);
    }

    this.moveFile = (fileName) => {
        return $cordovaFile.checkFile(rootFilePath, fileName).then(() => {
            return $cordovaFile.moveFile(rootFilePath, fileName, fullRootFilePathBin, fileName).then(() => {
                logService.info("Arquivo " + fileName + "  movido com sucesso");
                return $q.resolve();
            }, (error)  => {
                logService.error("Ocorreu um erro ao mover arquivo " + fileName, error);
                return $q.reject(error);
            });
        }, () => {
            return $q.resolve();
        });
    }

    this.migrateEncryptFile = (fileName, cryptoFileName) => {
        var deferred = $q.defer();
        
        $cordovaFile.checkFile(fullRootFilePathBin, fileName).then(() => {
            $cordovaFile.readAsText(fullRootFilePathBin, fileName).then((loadedFile) => {
                if (loadedFile) {
                    logService.info(`Iniciando migração do arquivo '${fileName}' para '${cryptoFileName}'.`);
                    const encryptedBlob = this.encryptContentAsBlob(loadedFile);
                    this.createAndSaveFile(fullRootFilePathBin, cryptoFileName, encryptedBlob)
                        .then(() => this.removeFile(fileName)) 
                        .then(() => { 
                            logService.info(`Migração do arquivo '${fileName}' para '${cryptoFileName}' concluída com sucesso!`);
                            deferred.resolve();
                        })
                        .catch((error) => 
                            { 
                                logService.error(`Não foi possível salvar o arquivo criptografado '${cryptoFileName}'!`, error);
                                deferred.reject(`Não foi possível salvar o arquivo criptografado '${cryptoFileName}'!`) 
                            });
                    
                } else {
                    deferred.reject(`Arquivo '${fileName}' não foi carregado!`);
                }
            }, (error) => {
                logService.error(`Arquivo '${fileName}' não pode ser lido!`, error);
                deferred.reject(`Arquivo '${fileName}' não pode ser lido!`);
            });
        }, () => {
            // se não encontra o arquivo anterior, já foi migrado
            deferred.resolve();
        });
        return deferred.promise;
    }

    this.migrateEftActivation = async (configFile, eftActivationFile) => {
        try {
            const isConfigFileCreated = await this.fileExists(fullRootFilePathBin, configFile);

            if (!isConfigFileCreated) {
                logService.info('Ativação do TEF não será migrada, não existem arquivos de configuração criados');
                return;
            }

            const encryptedConfig = await $cordovaFile.readAsArrayBuffer(fullRootFilePathBin, configFile);

            if (!encryptedConfig) {
                logService.info('Ativação do TEF não será migrada, não existem dados de configuração criados');
                return;
            }

            const decryptedConfig = this.decryptArrayBufferContent(encryptedConfig);
            const configObj = JSON.parse(decryptedConfig);

            if (!containsActivation(configObj.tefserver)) {
                logService.info('Ativação do TEF não será migrada,'
                    + 'não existem dados de ativação previamente salvos nas configurações');
                return;
            }

            const eftActivation = {
                storexTerminal: {
                    companyNumber: configObj.companyNumber,
                    storeNumber: configObj.storeNumber,
                    terminalNumber: configObj.terminalNumber,
                    storeDocument: configObj.store.cnpj
                },
                // o tef não obrigatoriamente retornará os mesmos
                // dados do terminal do storex
                eftTerminal: {
                    companyNumber: configObj.tefserver.establishmentCode || configObj.companyNumber,
                    storeNumber: configObj.tefserver.storeNumber || configObj.storeNumber,
                    terminalNumber: configObj.tefserver.terminalNumber || configObj.terminalNumber,
                    storeDocument: configObj.tefserver.storeCnpj || configObj.store.cnpj
                },
                date: new Date().toISOString(),
                deviceUuid: device.uuid,
                provider: configObj.tefserver.provider
            };

            const encryptedActivation = this.encryptContentAsBlob(JSON.stringify(eftActivation));
            await this.createAndSaveFile(fullRootFilePathBin, eftActivationFile, encryptedActivation);

            // este config é o único que hoje é salvo a partir de uma ativação
            delete configObj.tefserver.terminalNumber;
            const encryptedConfigBlob = this.encryptContentAsBlob(JSON.stringify(configObj));
            await this.createAndSaveFile(fullRootFilePathBin, configFile, encryptedConfigBlob);

            logService.info("Ativação TEF migrada com sucesso");
        } catch(err) {
            logService.error(err);
            throw err;
        }
    };

    function containsActivation(tefserver) {
        return notEmpty(tefserver)
            && notEmpty(tefserver.terminalNumber)
            && notEmpty(tefserver.storeNumber)
            && notEmpty(tefserver.establishmentCode);
    }

    function notEmpty(arg) {
        if (arg == null) {
            return false
        }

        if (typeof arg == "string") {
            return arg.length > 0;
        }

        if (typeof arg == "object") {
            return Object.keys(arg).length > 0;
        }

        return true;
    }

    this.createPAFFolder = function() {
        const deferred = $q.defer();
        if (pafFolderCreated) {
            deferred.resolve();
        } else {
            window.resolveLocalFileSystemURL($rootScope.fullRootFilePath, (fileSystem) => {
                //Cria o diretório PAF
                fileSystem.getDirectory('paf', { create: true }, (subFile) => {
                    pafFolderCreated = true;
                    deferred.resolve();
                }, function(error) {
                    deferred.reject(error);
                });
            }, function(error) {
                deferred.reject(error);
            });
        }
        return deferred.promise;
    };

    this.createAndSaveFile = (folder, fileName, fileContent) => {
        const deferred = $q.defer();
        $cordovaFile.writeFile(folder, fileName, fileContent, { replace: true }).then(() => {
            deferred.resolve(fileName);
        }).catch((error) => {
            logService.error(`Erro ao gravar arquivo '${fileName}' no diretorio '${folder}'.`, error);
            deferred.reject(`Erro ao gravar arquivo '${fileName}' no diretorio '${folder}'.`);
        });
        return deferred.promise;
    }

    this.savePafFile = (fileName, fileContent) => {
        const deferred = $q.defer();
        this.createPAFFolder()
            .then(() => {
                writeLargeBlobs(fullRootFilePathPaf, fileName, fileContent, false)
                    .then((fileName) => {
                        logService.info(`Arquivo '${fileName}' salvo no diretorio '${fullRootFilePathPaf}'.`);
                        // não repassamos o caminho completo para não exibir o file:///storage/emulated/0
                        const friendlyPath = fullRootFilePathPaf.substring(fullRootFilePathPaf.indexOf('/Linx'), fullRootFilePathPaf.length);
                        deferred.resolve(`${friendlyPath}/${fileName}`);
                    }).catch((error) => {
                        deferred.reject(error);
                    });
            })
            .catch(error => {
                logService.error("Erro ao criar diretório do PAF!", error);
                deferred.reject("Erro ao criar diretório do PAF!");
            });

        return deferred.promise;
    }

    this.saveCertificateFile = (fileName, fileContent) => {
        const deferred = $q.defer();

        this.createAndSaveFile(fullRootFilePathBin, fileName, fileContent)
            .then((fileName) => {
                logService.info(`Arquivo '${fileName}' salvo no diretorio '${fullRootFilePathBin}'.`);
                deferred.resolve(`${fileName}`);
            }).catch((error) => {
                deferred.reject(error);
            });

        return deferred.promise;
    }

    this.getCertificateAbsolutePath = (fileName) => {
        const path = fullRootFilePathBin.startsWith(FILE_PROTOCOL) ? fullRootFilePathBin.substring(FILE_PROTOCOL.length, fullRootFilePathBin.length) : fullRootFilePathBin;
        return `${path}${fileName}`;
    }

    // https://stackoverflow.com/questions/40914978/using-cordova-writefile-for-a-20mb-blob-crashes-android
    function writeLargeBlobs(folder, fileName, blob, isAppend) {
        const csize = 4 * 1024 * 1024; // 4MB

        logService.info(`Gravando blob em '${fileName}', tamanho atual: ${blob.size}.`)
    
        if (!blob.size)
        {
            logService.info(`Blob gravado com sucesso no arquivo '${fileName}'.`)
            return $q.resolve(fileName); 
        }
    
        if (!isAppend)
        {
            return $cordovaFile.writeFile(folder, fileName, blob.slice(0,csize), true)
                .then (function (succ) {
                    return writeLargeBlobs(folder,fileName,blob.slice(csize),true);
                });
        }
        else
        {
            return $cordovaFile.writeExistingFile(folder, fileName, blob.slice(0,csize))
                .then (function (succ) {
                    return writeLargeBlobs(folder,fileName,blob.slice(csize),true);
                });   
        }
    }

    this.arrayBuffer2String = (arrayBuffer) => {
        const enc = new TextDecoder("utf-8");
        return enc.decode(arrayBuffer);
    }

    this.encryptContentAsBlob = (content) => {
        const encrypted = CryptoJS.AES.encrypt(content, key, {
            mode: CryptoJS.mode.ECB,
            padding: CryptoJS.pad.Pkcs7
        });

        return new Blob([encrypted], {type: 'application/octet-stream'});
    }

    this.decryptArrayBufferContent = (encryptedArrayBuffer) => {
        const enctryptedContent = this.arrayBuffer2String(encryptedArrayBuffer);
        const decrypted = CryptoJS.AES.decrypt(enctryptedContent, key, {
            mode: CryptoJS.mode.ECB,
            padding: CryptoJS.pad.Pkcs7
        }).toString(CryptoJS.enc.Utf8); 
        return decrypted;
    }

    this.fileExists = async (dir, file) => {
        try {
            await $cordovaFile.checkFile(dir, file);
            return true;
        } catch(err) {
            if (err.message == 'NOT_FOUND_ERR') {
                return false;
            }
            throw err;
        }
    };
}
