/**
 *
 */
function LocalStorageDataSourceProvider($injector) {

    const $localStorage = $injector.get('$localStorage');
    const $q = $injector.get('$q');
    const LogService = $injector.get('LogService');
    const service = $injector.get('ConfigService');
    const eftService = $injector.get('EFTService');


    if (!$localStorage) {
        throw 'Provider utiliza $localStorage, mas ele não foi encontrado.';
    }

    /**
     * Função não é mais utilizada
     */
    this.doDataSourceMigration = function() {
        return $q.resolve();
    }

    this.persist = () => {
        return $q.all([
            this.persistConfig(),
            this.persistStorage("systemData"),
            this.persistNsuStorage(),
            this.persistEftActivationStorage()
        ]);
    }

    this.persistConfig = () => {
        return this.persistStorage("config");
    };

    this.persistStorage = (property) => {
        return $q((resolve) => {
            $localStorage[property] = angular.copy(service[property]);
            resolve($localStorage[property]);
        });
    };

    this.persistNsuStorage = () => {
        return $q((resolve) => {
            $localStorage.nsuStorage = angular.copy(service.nsuControll.getNSUStorage());
            resolve($localStorage.nsuStorage);
        });
    };

    this.persistEftActivationStorage = () => {
        return $q((resolve) => {
            const eftActivation = eftService.getActivation();

            if (eftActivation == null) {
                delete $localStorage.eftActivation;
                resolve();
                return;
            }

            $localStorage.eftActivation = {
                ...eftActivation
            };

            resolve($localStorage.eftActivation);
        });
    };
    
    this.load = () => {
        return $q.all([
            this.loadStorage("config"),
            this.loadStorage("systemData"),
            this.loadNsuStorage(),
            this.loadEftActivationStorage()
        ]);
    }

    this.loadStorage = (property) => {
        return this.readStorage(property).then((val) => {
            if (val) {
                angular.merge(service[property], val);
            }
        });
    };

    this.loadNsuStorage = () => {
        return this.readStorage("nsuStorage").then((val) => {
            service.nsuControll.updateStorageNSU(val);
        });
    };

    this.readStorage = (property) => {
        return $q((resolve, reject) => {
            if (!$localStorage[property]) {
                const err = `Informação do $localStorage.${property} inexistente.`;
                LogService.error(err);
                reject(err);
                return;
            }
            resolve(angular.copy($localStorage[property]));
        });
    };

    this.checkNsuStorage = () => {
        return $q((resolve, reject) => {
            if (!$localStorage.nsuStorage) {
                const err = "Configuração de nsu inexistente.";
                LogService.error(err);
                reject(err);
                return;
            }
            resolve($localStorage.nsuStorage);
        });
    };

    this.remove = () => {
        return $q((resolve) => {
            LogService.info("Removendo configurações da aplicação");
            delete $localStorage.config;
            delete $localStorage.mideClientInfo;
            delete $localStorage.systemData;
            delete $localStorage.nsuStorage;
            LogService.info("Configurações da aplicação removidas.");
            resolve();
        });
    }

    this.updateServConfigStorage = () => {
        return this.persistConfig();
    }

    this.updateNSUStorage = () => {
        return this.persistNsuStorage();
    }

    this.updateEftActivationStorage = () => {
        return this.persistEftActivationStorage();
    };

    this.loadEftActivationStorage = () => {
        return $q((resolve) => {
            if ($localStorage.eftActivation == null) {
                resolve();
                return;
            }

            const eftActivation = {
                ...$localStorage.eftActivation
            };

            eftService.setActivation(eftActivation);

            resolve(eftActivation);
        });
    };

    this.initialize = function() {
        return $q.resolve();
    }

    this.savePafFile = (fileName, fileContent) => {
        return $q((resolve) => {
            $localStorage[`PAF-${fileName}`] = fileContent;
            resolve(fileName);
        });
    }

    this.saveCertificateFile = (fileName, fileContent) => {
        return $q((resolve) => {
            $localStorage[`CERTIFICATE-${fileName}`] = fileContent;
            resolve(fileName);
        });
    }

    this.getCertificateAbsolutePath = (fileName) => {
        return fileName;
    }
}
