(function () {
    'use strict';

    angular.module('easyorder.services.discount', [
            'ngResource',
            'easyorder.services.config'
        ])
        .factory('DiscountService', function ($resource,
            ConfigService,
            CartService,
            $rootScope,
            UserService,
            UtilService,
            $q,
            $ionicLoading,
            AlertService,
            LogService) {

            var service = {
                companyNumber: 0,
                storeNumber: 0,
                url: ''
            };

            service.loadSettings = function () {
                service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                service.url += ConfigService.config.linxwebservices.host + ':';
                service.url += ConfigService.config.linxwebservices.port;
                service.url += '/linx-webservices';
                service.companyNumber = ConfigService.config.companyNumber;
                service.storeNumber = ConfigService.config.storeNumber;
                service.productUrlCodeType = ConfigService.config.business.product_url_code_type;
            };


            /**
             *
             */
            service.prepareFiscalItem = function (item) {
                return {
                    "productCode": item.productCode,
                    "marketingCode": item.marketingCode,
                    "quantitySold": item.quantitySold,
                    "unitValue": item.unitValue,
                    "ownBrand": item.ownBrand,
                    "promotionalPrice": item.promotionalPrice,
                    "discounts": item.discounts,
                }
            };

            /**
             *
             */
            service.prepareFiscalItems = function (fiscalItems) {
                let res = [];
                fiscalItems.forEach(item => {
                    let object = {
                        "productCode": item.productCode,
                        "marketingCode": item.marketingCode,
                        "quantitySold": item.quantitySold,
                        "unitValue": item.unitValue,
                        "ownBrand": item.ownBrand,
                        "promotionalPrice": item.promotionalPrice,
                        "discounts": item.discounts,
                    };
                    res.push(object);
                });
                return res;
            };


            /**
             *
             */
            service.discountParams = function (authUsername, authPass, param) {

                let params = {
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,

                    discountRuleInfo: {
                        discountRuleCode: param.rule.code,
                        discountRuleType: param.type,
                        discountValue: (param.data.valueType == "PERCENT") ? param.data.dataPercent : param.data.dataValue,
                        discountType: param.data.valueType,
                        requiredExternalAuthorization: (param.rule.requiredExternalAuthorization) ? param.rule.requiredExternalAuthorization : false
                    },
                    user: {
                        username: UserService.currentUser.username
                    },
                    authorizer: (!authUsername) ? null : {
                        username: authUsername,
                        password: authPass
                    }
                };

                if (param.item) {
                    params["fiscalItem"] = service.prepareFiscalItem(param.item);
                } else {
                    params["fiscalItems"] = service.prepareFiscalItems(CartService.cart.fiscalItems)
                }

                return params;
            };

            /**
             *
             */
            service.searchDiscountItem = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/search', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };

            /**
             *
             */
            service.searchDiscountFiscalCoupon = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/search/coupon', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };

            /**
             *
             */
            service.applyInItem = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/authorization/item', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };

            /**
             *
             */
            service.applyInCouponFiscal = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/authorization/coupon', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };

            /**
             *
             */
            service.authorizeDiscountInItem = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/authorization/item', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };


            /**
             *
             */
            service.authorizeDiscountInCouponFiscal = function (params, success, error) {
                service.loadSettings();
                return $resource(service.url + '/v1/discount/rules/authorization/coupon', {}, {
                        discount: {
                            method: 'POST'
                        }
                    })
                    .discount({}, params, success, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
            };

            /**
             *
             */
            $rootScope.ruleDiscounts = function(discounts) {
                let haveRuleDiscounts = false;
                if ($rootScope.allowsDiscountRuleOverlap){
                    haveRuleDiscounts = false;
                }else{
                    discounts.forEach(discount => {
                        if (discount.source && discount.source == "RULE") {
                            haveRuleDiscounts = true;
                        }
                    });
                }
                return haveRuleDiscounts;
            };


            /**
             *
             */
            service.errMessage = function (params) {
                let errMessage = null;
                switch (params) {
                    case "UNAUTHORIZED":
                        errMessage = "O pedido não foi autorizado, por favor, verifique a alçada do usuário informado"
                        break;
                    case "INVALID_CREDENTIALS":
                        errMessage = "Usuário/Senha incorretos, por favor, verifique os dados informados"
                        break;
                    case "INVALID_DISCOUNT_RULE":
                        errMessage = "Regra de desconto inválida, contacte o administrador"
                        break;
                    case "INVALID_DISCOUNT_VALUE":
                        errMessage = "Valor do desconto inválido, verifique os dados informados"
                        break;
                    default:
                        errMessage = "Ocorreu um erro ao processar o desconto"
                        break;
                }
                return errMessage;
            };

        /**
         *
         */
        service.processAutomaticDiscount = (item) => {
            var defer = $q.defer();

            if (ConfigService.config.business.enable_screen_discount) {
                var discountParams = {
                    productCode: item.productCode,
                    employee: (CartService.cart.customer && CartService.cart.customer.employee) ? CartService.cart.customer.employee : false,
                    ownBrand: item.ownBrand,
                    companyNumber: ConfigService.config.companyNumber,
                    marketingCode: item.marketingCode,
                    promotionalPrice: item.promotionalPrice,
                    automatic: true,
                    unitValue: item.unitValue,
                    storeNumber : ConfigService.config.storeNumber
                };

                $ionicLoading.show({
                    template : 'Verificando desconto para produto...',
                    duration : 0
                }).then(() => {
                    service.searchDiscountItem(discountParams, function(result) {
                        LogService.info('Resultado do desconto: ', result);

                        $ionicLoading.hide();

                        if (result && result.discountRuleTypes && result.discountRuleTypes[0] && result.discountRuleTypes[0].discountRules.length) {
                            defer.resolve({result: result, item: item});
                        } else {
                            AlertService.info("Não há regras de desconto disponíveis para o item.").then(defer.resolve);
                        }
                    }, function(error) {
                        $ionicLoading.hide();
                        AlertService.restError(error, 'Erro consultando desconto no servidor!').then(function() {
                            defer.resolve({result: undefined, item: item});
                        });
                    }).finally(() => {
                        $scope.loadingState = false;
                    });
                });
            } else {
                defer.resolve();
            }
            return defer.promise;
        };

            return service;
        })
})
();
