
(function() {
    'use strict';
    angular.module('easyorder.services.externalloading', [
		'easyorder.services.config',
		'easyorder.services.log'
	])

    .factory('ExternalLoadingService', function($state, ConfigService, LogService) {
        var service = {
        };
        
        const checkAppReady = () => {
      	  return new Promise((resolve, reject) => {
      		if (ConfigService && !ConfigService.isConfigurationLoaded()) {
        		throw new Error("A configuração deste dispositivo está incompleta! Favor realizar uma nova configuração!");
      		} else if (ConfigService.config.linxwebservices.host != ConfigService.systemData.currentLinxWebservicesHost) {
    			throw new Error("O host do Linx WebServices atual '" + ConfigService.config.linxwebservices.host + "' difere da última configuração '" + ConfigService.systemData.currentLinxWebservicesHost+ "'.  Favor realizar uma reconfiguração.");
        	} else {
        		resolve();
        	}
      	  });
      	};
      	
    	const checkExternalLoadingEnabled = () => {
    		var currentState = $state.current;
    		if (currentState && currentState.disableExternalLoading) {
    			throw new Error("Não é permitido carregamento externo na tela atual!");
    		}
    	};
    	
        const checkValidQueryParams = (queryParams) => {
			if (!queryParams) {
				throw new Error("Dados de busca não enviados!");
			} else if (!queryParams['storeNumber']) {
				throw new Error("Número da loja não enviado!");
			} else if (!queryParams['externalOrderNumber']) {
				throw new Error("Número do pedido externo não enviado!");
			}
    	};
        	
        const checkValidUriRequest = (uriString) => {
      		if (!uriString) { 
        		throw new Error('Requisição inválida!');
        	} else {
            	var uriParsed = URI.parse(uriString);
            	if (uriParsed.protocol !== 'caixarapido') {
            		// não deve ocorrer, visto que só o evento só é disparado neste protocolo.
            		throw new Error("Protocolo inválido: '" + uriParsed.protocol + "'.");
            	} else {
                	if (uriParsed.hostname !== 'linx') {
                		throw new Error("Host inválido: '" + uriParsed.hostname + "'.");
                	} else {
                		var path = uriParsed.path;
                		if (!path) {
                			throw new Error("Requisição inválida: ação remota não enviada!");
                		} else if (path !== '/orderretrieval' && path !== '/orderretrieval/') {
                			throw new Error("Requisiçao inválida: ação remota desconhecida: '" + path + "'.");
                		}
						
                		var username = uriParsed.username;
                		var password = uriParsed.password;
                		if (!username || !password) {
                			throw new Error('Usuário / senha não enviados.');
                		} else {
                			var uri = new URI(uriString);
                			var queryParams = uri.search(true);
                        	checkValidQueryParams(queryParams);
                		}
                	}
            	}
        	}
    	};
        
		
		service.validate = function(uriString) {
        	LogService.info("URI externa a ser validada: ", uriString);
    		return checkAppReady()
    			.then(function () { return checkExternalLoadingEnabled() })
    			.then(function () { return checkValidUriRequest(uriString) });
		};
		
        	
        
        return service;
    });
})();