(function() {
	'use strict';

	angular.module('easyorder.services.freight', [
		'ngResource',
		'easyorder.services.config'])

	.factory('FreightService', function($resource, ConfigService, UtilService) {

		var service = {
			url: ''
		};

		service.loadSettings = function() {
            service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
            service.url += '/linx-webservices/v1/freight';
        };

		service.searchSingleItemByZipcode = function(companyNumber, storeNumber, zipcode, orderValue, fiscalItem, success, error) {
			service.loadSettings();
			var params = {
                    companyNumber: companyNumber,
					storeNumber: storeNumber,
					destinationZipcode: zipcode,
                    orderValue: orderValue,
					groups: [{
						groupId: "group01",
						items: [{
							sku:		fiscalItem.productCode,
							quantity:	1,
							price:		fiscalItem.unitValue,
							weight:		fiscalItem.weight
						}]
					}]
			};
			service.loadSettings();
			var url = service.url + '/search/zipcode/list';

			return $resource(url, {}, { searchByZipcode: { method: 'POST', isArray: true } }).searchByZipcode({}, params, success, error).$promise;
		};

		service.searchSingleItemByStore = function(companyNumber, storeNumber, orderValue, fiscalItem, success, error) {
			service.loadSettings();
			var params = {
					companyNumber: companyNumber,
					storeNumber: storeNumber,
                    orderValue: orderValue,
					groups: [{
						groupId: "group01",
						items: [{
							sku:		fiscalItem.productCode,
							quantity:	1,
							price:		fiscalItem.unitValue,
							weight:		fiscalItem.weight
						}]
					}]
			};
			service.loadSettings();
			var url = service.url + '/search/store/list';

			return $resource(url, {}, { searchByStoreNumber: { method: 'POST', isArray: true } }).searchByStoreNumber({}, params, success, error).$promise;
		};

		service.searchItemListByZipcode = function(companyNumber, storeNumber, zipcode, orderValue, showroomingItems, success, error) {
			service.loadSettings();
				var params = {
                    companyNumber: companyNumber,
					storeNumber: storeNumber,
					destinationZipcode: zipcode,
                    orderValue: orderValue,
					groups: [{
						groupId: "group01",
						items: showroomingItems
					}]
			};
			service.loadSettings();
			var url = service.url + '/search/zipcode/list';

			return $resource(url, {}, { searchByZipcode: { method: 'POST', isArray: true } }).searchByZipcode({}, params, success, error).$promise;
		};

		service.searchItemListByStore = function(companyNumber, storeNumber, orderValue, showroomingItems, success, error) {
			service.loadSettings();
				var params = {
					companyNumber: companyNumber,
					storeNumber: storeNumber,
                    orderValue: orderValue,
					groups: [{
						groupId: "group01",
						items: showroomingItems
					}]
			};
			service.loadSettings();
			var url = service.url + '/search/store/list';

			return $resource(url, {}, { searchByZipcode: { method: 'POST', isArray: true } }).searchByZipcode({}, params, success, error).$promise;
		};

		service.validateDeliveryAddress = function(deliveryAddress) {
        	  return new Promise((resolve, reject) => {
   			   if (!deliveryAddress) {
   				   reject("Endereço não informado!");
   			   } else if (!deliveryAddress.zipCode) {
   				   reject("CEP não informado!");
   	   		   } else if (!deliveryAddress.addressName) {
   	   			   reject("Logradouro não informado!");
   	   		   } else if (!deliveryAddress.addressNumber) {
   	   			   reject("Número não informado!");
   	   		   } else if (!deliveryAddress.neighborhood) {
   	   			   reject("Bairro não informado!");
   	   		   } else if (!deliveryAddress.city) {
   	   			   reject("Cidade não informada!");
   	   		   } else if (!deliveryAddress.state) {
   	   			   reject("Estado não informado!");
	           } else {
	        	   resolve();
	           }
    	  });
		};

		return service;
	})
})
();
