(function() {
    'use strict';
    
    angular.module('easyorder.services.giftcard', [
        'ngResource', 
        'easyorder.services.config'
    ])

    .factory('GiftCardService', function(ConfigService,
                                        LogService, 
                                        $http,
                                        $rootScope,
                                        $resource) {
 

        var service = {
            url: '',
            provider: ''
        };

        service.loadSettings = function() {
            service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
            service.url += '/linx-webservices/v1/giftcard';
            service.provider = ConfigService.config.business.giftcard_service_provider;
        };
        
        service.loadVirtualSettings = function() {
            service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
            service.url += '/linx-webservices/v1/virtual-giftcard';
        };
        
        /**
         * 
         */
        service.search = function(cardNumber, success, error) {
            var params = {
	            cardNumber : cardNumber,
	            companyNumber : ConfigService.config.companyNumber,
	            storeNumber : ConfigService.config.storeNumber,
	            terminalNumber : ConfigService.config.terminalNumber,
                nsu : ("000000" + ConfigService.nsuControll.currentNSU()).slice(-6),
                cardTypeId : ConfigService.config.business.giftcard_type
            };
            service.loadSettings();
            service.url += '/search';
            
            return $resource(service.url, {}, { 
                search: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).search({}, params, success, error).$promise;
        };
        
        /**
         * 
         */
        service.reload = function(cardNumber, amount, success, error) {
            var params = {
                cardNumber: cardNumber,
                amount: amount,
                companyNumber : ConfigService.config.companyNumber,
                storeNumber : ConfigService.config.storeNumber,
                terminalNumber : ConfigService.config.terminalNumber,
                nsu : ("000000" + ConfigService.nsuControll.currentNSU()).slice(-6),
                cardTypeId : ConfigService.config.business.giftcard_type
            };
            service.loadSettings();
            service.url += '/reload';
            
            return $resource(service.url, {}, { 
                reload: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).reload({}, params, success, error).$promise;
        };
        
        /**
         * 
         */
        service.atomicReload = function(cardNumber, contextUID, amount, success, error) {
            var params = {
                cardNumber: cardNumber,
                amount: amount,
                companyNumber : ConfigService.config.companyNumber,
                storeNumber : ConfigService.config.storeNumber,
                terminalNumber : ConfigService.config.terminalNumber,
                nsu : ("000000" + ConfigService.nsuControll.currentNSU()).slice(-6),
                cardTypeId : ConfigService.config.business.giftcard_type,
                contextUid: contextUID
            };
            service.loadSettings();
            service.url += '/atomicReload';
            
            return $resource(service.url, {}, { 
                atomicReload: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).atomicReload({}, params, success, error).$promise;
        };
        
        /**
         * 
         */
        service.authGiftCardPayment = function(params, success, error) {
            service.loadSettings();
            service.url += '/auth';
            
            return $resource(service.url, {}, { 
                auth: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).auth({}, params, success, error).$promise;
        };
        
        /**
         * 
         */
        service.confirmGiftCardPayment = function(salePayment, success, error) {
            var params = {
                cardNumber: salePayment.cardNumber,
                amount: salePayment.paymentValue,
                companyNumber : ConfigService.config.companyNumber,
                storeNumber : ConfigService.config.storeNumber,
                terminalNumber : ConfigService.config.terminalNumber,
                paymentAuthorizationNSU : salePayment.paymentAuthorizationNSU,
                nsu : ConfigService.nsuControll.currentCartNSU(),
                cardTypeId : ConfigService.config.business.giftcard_type,
                transactionType: "PAYMENT",
                contextUid: salePayment.contextUid
            };
            service.loadSettings();
            service.url += '/confirm';
            
            return $resource(service.url, {}, { 
                confirm: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).confirm({}, params, success, error).$promise;
        };
        
        /**
         * 
         */
        service.undoGiftCardPayment = function(salePayment, success, error) {
            var params = {
                cardNumber: salePayment.cardNumber,
                amount: salePayment.paymentValue,
                companyNumber : ConfigService.config.companyNumber,
                storeNumber : ConfigService.config.storeNumber,
                terminalNumber : ConfigService.config.terminalNumber,
                paymentAuthorizationNSU : salePayment.paymentAuthorizationNSU,
                nsu : ConfigService.nsuControll.currentNSU(),
                cardTypeId : ConfigService.config.business.giftcard_type,
                transactionType: "PAYMENT",
                contextUid: salePayment.contextUid
            };
            service.loadSettings();
            service.url += '/undo';
            
            return $resource(service.url, {}, { 
                undo: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).undo({}, params, success, error).$promise;
        };
        
        /**
         * Consulta imagens disponíveis para criação de Cartão Presente Virtual
         */
        service.queryVirtualCardLayouts = function(success, error) {
            var params = {
                companyNumber: ConfigService.config.companyNumber,
                storeNumber: ConfigService.config.storeNumber,
                terminalNumber: ConfigService.config.terminalNumber,
                nsu: ("000000" + ConfigService.nsuControll.currentNSU()).slice(-6),
            };
            service.loadVirtualSettings();
            service.url += '/queryLayouts';
            

            return $resource(service.url, {}, { 
                queryLayouts: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).queryLayouts({}, params, success, error).$promise;
        };
        
        /**
         * Consulta imagens disponíveis para criação de Cartão Presente Virtual
         */
        service.createVirtualCard = function(cardData, success, error) {
            var params = {
                companyNumber: ConfigService.config.companyNumber,
                storeNumber: ConfigService.config.storeNumber,
                terminalNumber: ConfigService.config.terminalNumber,
                nsu: ("000000" + ConfigService.nsuControll.currentNSU()).slice(-6),
                sender: cardData.sender,
                cardCreationEcerts: [{
                    amount: cardData.amount,
                    reference: cardData.imageRef,
                    recipient: cardData.recipient,
                    deliveryDate: cardData.deliveryDate,
                    deliveryMessage: cardData.deliveryMessage
                }]
            };
            service.loadVirtualSettings();
            service.url += '/create';
            
            return $resource(service.url, {}, { 
                create: { 
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                } 
            }).create({}, params, success, error).$promise;
        };
        return service;
    });
})();
