(function() {
    'use strict';
    angular.module('easyorder.services.insurance', [
        'ngResource',
        'easyorder.services.config',
        'easyorder.services.log'
    ])

    .factory('InsuranceService', function(ConfigService, CartService, $resource, $rootScope, $q, LogService, $http) {
        var service = {
            url : '',
            currentSimulation: undefined
        };

        const CHANNEL_NUMBER = 1;

        service.clearSimutaionData = function() {
            service.currentSimulation = undefined;
        };

        /**
         * Registrando função para o caso de reinicialização do carrinho.
         */
        CartService.initCartObservers.add(service.clearSimutaionData);

        /**
         *
         */
        service.isPlanInsured = function(paymentPlan) {
            if (paymentPlan) {
                if (service.currentSimulation) {
                    for (var i = 0; i < service.currentSimulation.length; i++) {
                        var insuranceSimulation = service.currentSimulation[i];

                        if (paymentPlan.code == insuranceSimulation.insuranceDatas[0].event) {
                            return insuranceSimulation;
                        }
                    }
                    LogService.warn("Não há simulações de seguro para o plano: ", paymentPlan.code);
                }
            }
            return undefined;
        };

        /**
         *
         */
        service.loadSettings = function() {
            service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
        };

        /**
         *
         */
        service.simulate = function(operatorCode, componentNumber, cupomNumber, processorCode, cpf, cardNumber, events,
                successCallback, errorCallback) {
            service.loadSettings();
            service.url += '/linx-webservices/v1/insurance/simulate';
            var params = {
                storeNumber : ConfigService.config.storeNumber,
                operatorCode : operatorCode,
                channelDate : new Date(),
                channelNumber : CHANNEL_NUMBER,
                componentNumber : componentNumber,
                sequenceNumber : 1,
                cupomNumber : cupomNumber,
                processorCode : processorCode,
                cpf : cpf,
                cardNumber : cardNumber,
                events : events
            };

            var respose = $resource(service.url, {}, {
                'process' : {
                    method : 'POST',
                    headers: {
                        'Service-Provider': ConfigService.config.business.payment_insurance_provider || 'LREN'
                    }
                }
            });
            return respose.process({}, params, successCallback, errorCallback);
        };

        service.simulatePaymentPlan = (request) => {
            service.loadSettings();
            service.url += '/linx-webservices/v1/insurance/simulate-plan';
            return $http.post(service.url, request, {
                headers: {
                    'Service-Provider': ConfigService.config.business.payment_insurance_provider || 'LREN'
                }
            });
        };

        /**
         *
         */
        service.authorize = function(operatorCode, channelDate,componentNumber, sequenceNumber, cupomNumber,
                processorCode, insuranceValue, insuranceCode, birthDate, genre, /* numeroParcelas, */numberOfInstallments,
                clientAffinity, occupationCode, cpfCnpj, insurancePlan, confirmationFlag, operationDate, successCallback,
                errorCallback) {
            service.loadSettings();
            service.url += '/linx-webservices/v1/insurance/authorize';
            var params = {
                storeNumber : ConfigService.config.storeNumber,
                operatorCode : operatorCode,
                channelDate : channelDate,
                channelNumber : CHANNEL_NUMBER,
                componentNumber : componentNumber,
                sequenceNumber : sequenceNumber,
                cupomNumber : cupomNumber,
                processorCode : processorCode,
                insuranceValue : insuranceValue,
                insuranceCode : insuranceCode,
                birthDate : birthDate,
                genre : genre,
                numberOfInstallments : numberOfInstallments,
                clientAffinity : clientAffinity,
                occupationCode : occupationCode,
                cpfCnpj : cpfCnpj,
                confirmationFlag : confirmationFlag,
                insurancePlan : insurancePlan,
                operationDate : operationDate
            };

            var respose = $resource(service.url, {}, {
                'process' : {
                    method : 'POST',
                    headers: {
                        'Service-Provider': ConfigService.config.business.payment_insurance_provider || 'LREN'
                    }
                }
            });
            return respose.process({}, params, successCallback, errorCallback);
        };

        /**
         *
         */
        service.getTicketText = function(cart, successCallback, errorCallback) {
            service.loadSettings();
            service.url += '/linx-webservices/v1/insurance/get-ticket-text';

            var salePayment = cart.salePayments[0];
            var parcelas = salePayment.installments;
            parcelas.sort(function(a, b) {
                return a.paymentSequence - b.paymentSequence;
            });

            var params = {
                sale : cart,
                appInfo : {
                    name : APP_NAME,
                    version : $rootScope.appVersion
                }
            };

            var respose = $resource(service.url, {}, {
                'process' : {
                    method : 'POST',
                    headers: {
                        'Service-Provider': ConfigService.config.business.payment_insurance_provider || 'LREN'
                    }
                }
            });
            return respose.process({}, params, successCallback, errorCallback);
        };

        /**
         * Defaz autorização de seguro.
         */
        service.undoInsuranceAuth = (cart) => {
            if (cart.insuranceData && cart.insuranceData.authorizationNumber && cart.customer) {
                service.loadSettings();
                service.url += '/linx-webservices/v1/insurance/cancel';

                var params = {
                    storeNumber:            ConfigService.config.storeNumber,
                    operatorCode:           cart.operatorCode,
                    channelDate:            new Date(),
                    channelNumber:          CHANNEL_NUMBER,
                    componentNumber:        cart.terminalNumber,
                    processorCode:          cart.customer.processor,
                    sequenceNumber:         1,
                    cupomNumber:            cart.nsu,
                    cpf:                    cart.customer.document,
                    operationNsu:           cart.insuranceData.authorizationNumber,
                    nsuLogTransaction:      ConfigService.nsuControll.incrementNSU(),
                    transactionNSUnumber:   cart.nsu
                };
                var resource = $resource(service.url, {}, {
                    'process' : {
                        method : 'POST',
                        headers: {
                            'Service-Provider': ConfigService.config.business.payment_insurance_provider || 'LREN'
                        }
                    }
                });
                return $q((resolve, reject) => {
                    resource.process({}, params, resolve, reject);
                });
            }
        };
        return service;
    });
})();
