(function () {
    'use strict';

    angular.module('easyorder.services.loyalty', [
            'ngResource',
            'easyorder.services.util',
            'easyorder.services.config'
        ])

        .factory('LoyaltyService', function (
            $resource,
            ConfigService,
            UtilService,
            UserService) {
            
            function LoyaltyService() {

                var service = {
                    companyNumber: 0,
                    storeNumber: 0,
                    terminalNumber: 0,
                    url: ''
                };

                function serviceHeaders() {
                    const providerName = ConfigService.config.business.loyalty_provider;
                    if(providerName) {
                        return { headers : { 
                            'Service-Provider': providerName  
                        }};
                    } else {
                        return undefined;
                    }
                }
                
                service.loadSettings = function () {
                    service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                    service.url += ConfigService.config.linxwebservices.host + ':';
                    service.url += ConfigService.config.linxwebservices.port;
                    service.url += '/linx-webservices';
                    service.companyNumber = ConfigService.config.companyNumber;
                    service.storeNumber = ConfigService.config.storeNumber;
                    service.terminalNumber = ConfigService.config.terminalNumber;
                }
                
                /**
                 * Consulta informações de participante do programa Loyalty
                 * @param {Requisição de participante} participantRequest 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                this.getParticipant = function (participantRequest, success, error) {
                    service.loadSettings();
                    participantRequest.operator = UserService.operatorCode();
                    participantRequest.store = service.storeNumber;
                    participantRequest.terminal = service.terminalNumber;
                    participantRequest.companyId = service.companyNumber;
                    return $resource(service.url + '/v1/loyalty/participant?' + UtilService.jsonToUriParam(participantRequest), {}, {
                        search: {
                            method: 'GET',
                            headers: serviceHeaders()
                        }
                    }).search({}, success, error).$promise;
                }

                
                /**
                 * Consulta informações de descontos do programa Loyalty
                 * @param {Requisição de desconto} discountRequest 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                this.getDiscount = function (discountRequest, success, error) {
                    service.loadSettings();
                    discountRequest.operator = UserService.operatorCode();
                    discountRequest.store = service.storeNumber;
                    discountRequest.terminal = service.terminalNumber;
                    discountRequest.companyId = service.companyNumber;
                    return $resource(service.url + '/v1/loyalty/discount', {}, { 
                        send: { 
                            method: 'POST', 
                            headers: serviceHeaders()
                        }
                    }).send({}, discountRequest, success, error).$promise;
                }
            }
            var service = new LoyaltyService();
            return service;
        })
})();
/**
 * Objeto de requisição para consulta de participante Loyalty
 * OBS: O objeto deve ser convertido em query parameter
 */
class ParticipantRequest {
    constructor() {
        this.documentNumber = undefined;
        this.companyId = undefined;
        this.store = undefined;
        this.terminal = undefined;
        this.operator = undefined;
        this.startDate = undefined;
        this.channel = 'StorexMobile';
        this.language = 'pt-BR';
    }
}

/**
 * Objeto de requisição para consulta de desconto Loyalty
 * OBS: O objeto deve ser convertido em query parameter
 */
class DiscountRequest {
    constructor() {
        this.documentNumber = undefined;
        this.companyId = undefined;
        this.store = undefined;
        this.terminal = undefined;
        this.operator = undefined;
        this.startDate = undefined;
        this.channel = 'StorexMobile';
        this.language = 'pt-BR';
        this.items = [];
    }
}