(function () {
    'use strict';

    angular.module('easyorder.services.nfce', [
        'ngResource',
        'easyorder.services.config',
        'easyorder.services.user'
    ])

    .factory('NFCeService', function ($resource, ConfigService, UserService) {
        var service = {
            companyNumber: 0,
            storeNumber: 0,
            url: '',
            request: {}
        };

        /**
         *
         */
        service.loadSettings = function () {
            service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
            service.url += '/linx-webservices';
            service.companyNumber = ConfigService.config.companyNumber;
            service.storeNumber   = ConfigService.config.storeNumber;
        };

        /**
         *
         */
        service.status = function () {
            service.loadSettings();
            return $resource(service.url + '/status').get();
        };

        /**
         *
         */
        service.auth = function (sale, appInfo, inventoryInfo, orderInfo, freightInfo, currentOrder, mergedOrders, params, success, error) {

            service.loadSettings();
            var url = service.url + '/v1/salesintegration/auth';
            var resNFCe = $resource(url, {}, {
                auth: {
                    method: 'POST'
                }
            });

            sale.salePayments.forEach(function(salePayment) {
                salePayment.installments.sort(function(a,b) {
                    return a.paymentSequence - b.paymentSequence;
                });
            }, this);

            sale.status = undefined;
            sale.companyNumber  = ConfigService.config.companyNumber;
            sale.storeNumber    = ConfigService.config.storeNumber;
            sale.terminalNumber = ConfigService.config.terminalNumber;
            sale.operatorCode   = UserService.operatorCode();

            return resNFCe.auth(
                {},
                {
                    sale: sale,
                    appInfo: appInfo,
                    inventoryInfo: inventoryInfo,
                    orderInfo: orderInfo, 
                    freightInfo: freightInfo, 
                    order: currentOrder, 
                    mergedOrders: mergedOrders, 
                    params: params
                },
                success,
                error
            );
        };

        service.integrate = function (sale, appInfo, orderInfo, freightInfo, currentOrder, mergedOrders, inventoryInfo, params, success, error) {
            service.loadSettings();

            var url = service.url + '/v1/salesintegration/integrate';
            var resNFCe = $resource(url, {}, {
                integrate: {
                    method: 'POST'
                }
            });

            sale.salePayments.forEach(function(salePayment) {
                salePayment.installments.sort(function(a,b) {
                    return a.paymentSequence - b.paymentSequence;
                });
            }, this);

            sale.status = undefined;
            sale.companyNumber  = ConfigService.config.companyNumber;
            sale.storeNumber    = ConfigService.config.storeNumber;
            sale.terminalNumber = ConfigService.config.terminalNumber;
            sale.operatorCode   = UserService.operatorCode();

            return resNFCe.integrate(
                {}, 
                {
                    sale: sale,
                    appInfo: appInfo,
                    orderInfo: orderInfo,
                    mergedOrders: mergedOrders,
                    freightInfo: freightInfo,
                    order: currentOrder,
                    inventoryInfo: inventoryInfo,
                    params: params
                }, 
                success,
                error
            );
        };

        service.confirmServiceIntegration = function (serviceOperation, appInfo, tefProvider, success, error) {
            service.loadSettings();

            var url = service.url + '/v1/services-integration/confirm';
            var resServiceIntegration = $resource(url, {}, {
                confirm: {
                    method: 'POST'
                }
            });

            return resServiceIntegration.confirm({}, {serviceOperation: serviceOperation, appInfo: appInfo, tefProvider: tefProvider}, success, error).$promise;
        };

        service.undoServiceIntegration = function (serviceOperation, success, error) {
            service.loadSettings();

            var url = service.url + '/v1/services-integration/undo';
            var resServiceIntegration = $resource(url, {}, {
                undo: {
                    method: 'POST'
                }
            });

            return resServiceIntegration.undo({}, {serviceOperation: serviceOperation}, success, error).$promise;
        };

        service.cancel = params => {
            return $resource(service.url + '/v1/salesintegration/cancel', {}, {
                cancel: {
                    method: 'POST'
                }
            }).cancel(params).$promise;
        };

        service.confirm = params => {
            service.loadSettings();
            const url = service.url + '/v1/salesintegration/confirm';
            return $resource(url, {}, {
            	confirm: {
            		method: 'POST'
            	}
            }).confirm({}, params).$promise;
        };

        service.undoServiceAuthorization = function (companyNumber, storeNumber, terminalNumber, nsu, serviceItem, success, error) {
            	service.loadSettings();

            	var url = service.url + '/v1/services-integration/undo-authorization';
            	var resServiceIntegration = $resource(url, {}, {
            		undo_auth: {
            			method: 'POST'
            		}
            	});
            	return resServiceIntegration.undo_auth({}, {companyNumber: companyNumber, storeNumber: storeNumber, terminalNumber: terminalNumber, nsu: nsu, serviceItem: serviceItem}, success, error).$promise;
        };
        return service;
    });

})();
