(function () {
    'use strict';

    angular.module('easyorder.services.pafservice', [
        'ngResource',
        'easyorder.services.config'
    ]).factory('PAFService', function ($resource, $http, ConfigService) {

        const service = {

        };

        let _localFile = false;

        const serviceURL = () => {
            const linxwebservices = ConfigService.config.linxwebservices;
            return (linxwebservices.secure ? 'https://' : 'http://') + linxwebservices.host + ':' + linxwebservices.port + '/linx-webservices/v1/paf';
        };

        service.setLocalFile = (localFile) => {
            _localFile = localFile;
        }

        let isLocalFile = () => {
            return _localFile;
        }

        service.pafNfceIdentification = () => {
            return $resource(serviceURL() + '/paf-info', {}, {
                pafNfceIdentification: {
                    method: 'GET'
                }
            }).pafNfceIdentification().$promise;
        };

        service.generatePafNfceRecords = (companyNumber, storeNumber, terminalNumber, initialDate, finalDate, productCodes, productDescriptions) => {
            const localFile = isLocalFile();
            const urlPath = localFile ? '/create-file/paf-nfce-records-file' : '/create-file/paf-nfce-records';
            return $http.post(serviceURL() + urlPath, {
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                initialDate: initialDate,
                finalDate: finalDate,
                productCodes: productCodes,
                productDescriptions: productDescriptions
            },
            {
                responseType: localFile ? 'blob' : 'json',
                transformResponse: function(data) {
                    if (localFile) {
                        const fileName = `regPAFNFCe-${ConfigService.config.storeNumber}.${getCurrentDateFormatted()}.tar.gz`;
                        return new PafLocalFileResponse(fileName, data);
                    } else {
                        return new PafRemoteFileResponse(data ? data.message : null);
                    }
                }
            });
        };

        service.generateIdentifiedSales = (companyNumber, storeNumber, terminalNumber, date, document, appInfo) => {
            const localFile = isLocalFile();
            const urlPath = localFile ? '/create-file/identified-sales-file' : '/create-file/identified-sales';
            return $http.post(serviceURL() + urlPath, {
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                date: date,
                document: document,
                appInfo: appInfo
            },
            {
                responseType: localFile ? 'blob' : 'json',
                transformResponse: function(data) {
                    if (localFile) {
                        const fileName = `vendasIdentCPFCNPJ-${ConfigService.config.storeNumber}.${getCurrentDateFormatted()}.tar.gz`;
                        return new PafLocalFileResponse(fileName, data);
                    } else {
                        return new PafRemoteFileResponse(data ? data.message : null);
                    }
                }
            });
        };

        service.generateExternalRequestsRegistered = (companyNumber, storeNumber, date, appInfo) => {
            const localFile = isLocalFile();
            const urlPath = localFile ? '/create-file/external-requests-file' : '/create-file/external-requests';
            return $http.post(serviceURL() + urlPath, {
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                date: date,
                appInfo: appInfo
            },
            {
                responseType: localFile ? 'blob' : 'json',
                transformResponse: function(data) {
                    if (localFile) {
                        const fileName = `reqExternasRegistradas-${ConfigService.config.storeNumber}.${getCurrentDateFormatted()}.tar.gz`;
                        return new PafLocalFileResponse(fileName, data);
                    } else {
                        return new PafRemoteFileResponse(data ? data.message : null);
                    }
                }
            });
        };


        service.generateControlDav = () => {
            const localFile = isLocalFile();
            const urlPath = localFile ? '/create-file/control-dav-file' : '/create-file/control-dav';
            return $http.post(serviceURL() + urlPath,{},
            {
                responseType: localFile ? 'blob' : 'json',
                transformResponse: function(data) {
                    if (localFile) {
                        const fileName = `controleDavPafNfce.${getCurrentDateFormatted()}.tar.gz`;
                        return new PafLocalFileResponse(fileName, data);
                    } else {
                        return new PafRemoteFileResponse(data ? data.message : null);
                    }
                }
            });
        };

        let getCurrentDateFormatted = () => {
            return moment().format('YYYY-MM-DD-HHmmss');
        }

        return service;
    });
})();

class PafResponse {
    constructor() {
        if (this.constructor == PafResponse) {
          throw new Error("Abstract classes can't be instantiated.");
        }
      }

      isLocalFile() {
        throw new Error("Method 'isLocalFile()' must be implemented.");
      }
}

class PafLocalFileResponse extends PafResponse {
    constructor(fileName, gzip) {
        super();
        this.fileName = fileName;
        this.gzip = gzip;
    }

    isLocalFile() {
        return true;
    }
}
class PafRemoteFileResponse extends PafResponse {
    constructor(message) {
        super();
        this.message = message;
    }

    isLocalFile() {
        return false;
    }
}
