class SalePaymentTypes {}
SalePaymentTypes.SALE_TEF_PAYMENT = "SaleTEFPayment";
SalePaymentTypes.SALE_PRIVATE_LABEL_PAYMENT = "SalePrivateLabelPayment";
SalePaymentTypes.SALE_GIFTCARD_PAYMENT = "SaleGiftCardPayment";
SalePaymentTypes.SALE_COVENANT_PAYMENT = "SaleCovenantPayment";
SalePaymentTypes.SALE_VOUCHER_ASSURANT_PAYMENT = "SaleVoucherAssurantPayment";
SalePaymentTypes.SALE_VOUCHER_GIVEX_PAYMENT = "SaleVoucherGivexPayment";
SalePaymentTypes.SALE_VOUCHER_SAFE_PAYMENT = "SaleVoucherSafePayment";
SalePaymentTypes.SALE_VOUCHER_STOREX_PAYMENT = "SaleVoucherStorexPayment";
SalePaymentTypes.SALE_VOUCHER_OMNI_PAYMENT = "SaleVoucherOmniPayment";
SalePaymentTypes.SALE_VOUCHER_PAYMENT = "SaleVoucherPayment";
SalePaymentTypes.SALE_E_WALLET_PAYMENT = "SaleEWalletPayment";
SalePaymentTypes.CARD_STATEMENT_PAYMENT = "CardStatementPayment";

(function() {
    'use strict';

    angular.module('easyorder.services.payment', [
        'ngResource',
        'easyorder.services.cart',
        'easyorder.services.user',
        'easyorder.services.config',
        'easyorder.services.covenant',
        'easyorder.services.servicesale',
        'easyorder.services.log'
    ])

    .factory('PaymentService', function(
        $resource,
        $http,
        $q,
        $rootScope,
        CartService,
        UserService,
        ConfigService,
        AlertService,
        LogService,
        PaymentProcessor) {

        function PaymentService() {
            this.ON_PAYMENT_COMPLETE = 'onPaymentComplete';
            this.SALE_TEF_PAYMENT = SalePaymentTypes.SALE_TEF_PAYMENT;
            this.SALE_PRIVATE_LABEL_PAYMENT = SalePaymentTypes.SALE_PRIVATE_LABEL_PAYMENT;
            this.SALE_GIFTCARD_PAYMENT = SalePaymentTypes.SALE_GIFTCARD_PAYMENT;
            this.SALE_COVENANT_PAYMENT = SalePaymentTypes.SALE_COVENANT_PAYMENT;
            this.SALE_VOUCHER_ASSURANT_PAYMENT = SalePaymentTypes.SALE_VOUCHER_ASSURANT_PAYMENT;
            this.SALE_VOUCHER_GIVEX_PAYMENT = SalePaymentTypes.SALE_VOUCHER_GIVEX_PAYMENT;
            this.SALE_VOUCHER_STOREX_PAYMENT = SalePaymentTypes.SALE_VOUCHER_STOREX_PAYMENT;
            this.SALE_VOUCHER_OMNI_PAYMENT = SalePaymentTypes.SALE_VOUCHER_OMNI_PAYMENT;
            this.SALE_VOUCHER_PAYMENT = SalePaymentTypes.SALE_VOUCHER_PAYMENT;

            var _url = '';
            var _payments = [];
            var _currentPayment = undefined;
            var observerCallbacks = [];

            var _confirmedPayments = [];
            const self = this;

            var _loadSettings = function() {
                _url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                _url += ConfigService.config.linxwebservices.host + ':';
                _url += ConfigService.config.linxwebservices.port;

                _url += '/linx-webservices';
            };
            const paymentsMethods = [
                {
                    id: "CREDIT",
                    code: '11',
                    description: "Crédito"
                }, {
                    id: "CREDIT_WITH_INSTALMENTS",
                    code: '11',
                    description: "Crédito Parcelado Loja"
                }, {
                    id: "DEBIT",
                    code: '9',
                    description: "Débito"
                }, {
                    id: "CASH",
                    code: '1',
                    description: "Dinheiro"
                }, {
                    id: "TOTVSCARD_CREDIT_BOOK",
                    code: '120',
                    description: "Carnê Cliente"
                }, {
                    id: "TOTVSCARD_CARD_STATEMENT",
                    code: '121',
                    description: "Fatura Cliente"
                }, {
                    id: "CONDUCTOR_CREDIT_BOOK",
                    code: '122',
                    description: "Carnê Cliente (Meu Cartão)"
                }, {
                    id: "CONDUCTOR_CARD_STATEMENT",
                    code: '123',
                    description: "Fatura Cliente (Meu Cartão)"
                }, {
                    id: "LEGACY_CLIENT",
                    code: '92',
                    description: "Cartão Cliente"
                }, {
                    id: "GIFT_CARD",
                    code: '1000',
                    description: "Cartão Presente"
                }, {
                    id: "EMPLOYEEE_CARD",
                    code: '6',
                    description: "Associado"
                }, {
                    id: "COVENANT",
                    code: '6',
                    description: "Convênio"
                }, {
                    id: "VOUCHER_ASSURANT",
                    code: '97',
                    description: "Recompra"
                }, {
                    id: "VOUCHER_STOREX",
                    code: '69',
                    description: "Voucher"
                }, {
                    id: "VOUCHER_GIVEX",
                    code: '102',
                    description: "Vale Troca"
                }, {
                    id: "VOUCHER_PROMO",
                    code: '103',
                    description: "Vale Troca"
                }, {
                    id: "VOUCHER_SAFE",
                    code: '102',
                    description: "Vale Troca Safe"
                }, {
                    id: "SITEF_QR",
                    code: '125',
                    description: "QR Sitef"
                }, {
                    id: "QR_LINX",
                    code: '130',
                    description: "QR Linx"
                }, {
                    id: "PAYMENT_LINK",
                    code: '132',
                    description: "Link de Pagamento"
                }, {
                    id: "PAYMENT_LINK_CBR",
                    code: '126',
                    description: "Link de Pagamento SiTef (CBR)"
                }, {
                    id: "PAYMENT_LINK_CCR",
                    code: '127',
                    description: "Link de Pagamento SiTef (CCR)"
                }, {
                    id: "PAYMENT_LINK_DEBIT",
                    code: '128',
                    description: "Link de Pagamento SiTef (Débito)"
                }, {
                    id: "PAYMENT_LINK_CREDIT",
                    code: '129',
                    description: "Link de Pagamento SiTef (Crédito)"
                }, {
                    id: "PIX",
                    code: '800',
                    description: "Pix"
                }
            ];

            /**
             * register an observer
             */
            this.registerObserverCallback = function(id, callback, event) {
                if (!event || !id || !callback) {
                    throw 'Observador inválido';
                }

                for (var i = 0; i < observerCallbacks.length; i++) {
                    if (observerCallbacks[i].id === id) {
                        delete(observerCallbacks.splice(i, 1));
                    }
                }
                observerCallbacks.push({ id: id, callback: callback, event: event });
            };

            /**
             * call this when you know 'foo' has been changed
             */
            this.notifyObservers = function(event) {
                if (!event) {
                    throw 'Evento não definido';
                }
                observerCallbacks.forEach(c => {
                    if (c.event === event && typeof c.callback === 'function') {
                        c.callback();
                    }
                });
            };

            this.paymentInProgress = function() {
                return this.isInParcialPayment() || _currentPayment;
            };

            this.isInParcialPayment = () => {
                return !!_payments.length;
            }

            this.payments = function() {
                return _payments;
            };

            this.totalPayments = function() {
                var ret = 0;

                _payments.forEach(payment => {
                    ret += payment.paymentValue;
                });
                return parseFloat(new Number(ret).toFixed(2));
            };

            this.totalCashPayments = function() {
                var ret = 0;
                // TODO ver se eh CASH
                _payments.forEach(payment => {
                    if (payment.paymentMethod === 'CASH') {
                        ret += payment.paymentValue;
                    }
                });
                return parseFloat(new Number(ret).toFixed(2));
            };

            this.currentPayment = function(current) {
                if (current) {
                    if (_currentPayment) {
                        throw 'Existe um pagamento em andamento.'
                    } else {
                        _currentPayment = current;
                    }
                }
                return _currentPayment
            };

            /**
             * Instancia um pagamento atual
             */
            this.initPayment = function(paymentMethod, paymentValue, privateLabel) {

                if (!paymentMethod || !(typeof paymentMethod === 'string' || paymentMethod instanceof String) || isNaN(paymentValue) || paymentValue === 0) {
                    throw 'Parâmetros inválidos: paymentMethod = ' + paymentMethod + " paymentValue = " + paymentValue;
                }
                _currentPayment = {
                    paymentMethod: paymentMethod,
                    paymentValue: paymentValue,
                    totalValue: CartService.cart.totalValue,
                    privateLabel: privateLabel,
                    companyNumber: ConfigService.config.companyNumber,
                    storeNumber: ConfigService.config.storeNumber,
                    userCode: UserService.operatorCode(),
                    paymentOperationType: (CartService.getCurrentOperation() === 'SERVICE' ? CartService.getCurrentServiceOperationType() : 'SALE'),
                    saleDate: new Date(),
                    authorizerCode: null,
                    fiscalItems: (function(cartItens) {
                        var ret = [];

                        cartItens.forEach(function(fis) {
                            ret.push({
                                productCode: fis.productCode,
                                marketingCode: fis.marketingCode,
                                quantitySold: fis.quantitySold,
                                unitValue: fis.unitValue,
                                discounts: fis.discounts,
                                nonFiscalItems: (function(cartnonFiscalItems) {
                                    if (cartnonFiscalItems) {
                                        let nonFiscalItens = [];
                                        cartnonFiscalItems.forEach(nonFis => {
                                            nonFiscalItens.push({
                                                value: nonFis.value,
                                                nonFiscalOperationType: nonFis.nonFiscalOperationType
                                            })
                                        })
                                        return nonFiscalItens;
                                    } else {
                                        return null;
                                    }
                                })(fis.nonFiscalItems)
                            });
                        });
                        return ret;
                    })(CartService.cart.fiscalItems),
                    paymentProtectionInsurance: false,
                    tefPayment: undefined,
                    plan: undefined,
                    campaigns: undefined,
                    instalments: []
                }
                $rootScope.$broadcast('paymentInProgress', true); // Note que a função this.paymentInProgress declara "pagamento em progresso" quando _currentPayment exista.
            };

            this.setPaymentOperationType = (paymentOperationType) => {
                 _currentPayment.paymentOperationType = paymentOperationType;
            };

            this.currentPaymentDone = function() {
                if (!_currentPayment.paymentSequence) {
                    _currentPayment.paymentSequence = _payments.length;
                }
                _payments.push(_currentPayment);

                if (_currentPayment.predefinedDetails) {
                    CartService.processedPredefinedPayments.push(_currentPayment.predefinedDetails);
                    delete _currentPayment.predefinedDetails;
                }

                LogService.info(`Pagamento Finalizado! [Forma: ${_currentPayment.paymentMethod} Valor: ${_currentPayment.paymentValue}]`);
                wipeInitializedPaymentData();
            };

            this.isPendingDownPayment = () => {
                if (_payments.length == 0) {
                    return false;
                }
                const lastPayment = _payments[_payments.length - 1];

                return lastPayment.plan
                    && lastPayment.plan.downPaymentValue > 0
            };

            this.downPaymentValue = () => {
                if (_payments.length == 0) {
                    return 0.00;
                }

                const lastPayment = _payments[_payments.length - 1];

                if (lastPayment.plan == null) {
                    return 0.00;
                }

                return lastPayment.plan.downPaymentValue || 0.00;
            };

            function downPaymentPlansMap() {
                if (_payments.length == 0) {
                    return [];
                }

                const lastPayment = _payments[_payments.length - 1];

                if (lastPayment.plan == null) {
                    return [];
                }

                const plans = lastPayment.plan.downPaymentPlans;

                if (plans == null || plans.length == 0) {
                    return [];
                }

                const map = {};

                for (const plan of plans) {
                    if (plan.paymentMethods == null || plan.paymentMethods.length == 0) {
                        continue;
                    }

                    for (const method of plan.paymentMethods) {
                        if (!map[method]) {
                            map[method] = [];
                        }
                        map[method].push(plan);
                    }
                }

                return map;
            };

            this.downPaymentMethods = () => {
                return Object.keys(downPaymentPlansMap());
            };

            function downPaymentEvent() {
                if (!service.isPendingDownPayment()) {
                    return null;
                }

                const plansMap = downPaymentPlansMap();
                const plans = _currentPayment ? plansMap[_currentPayment.paymentMethod]
                                              : plansMap;

                if (_payments.length == 0) {
                    return null;
                }

                const lastPayment = _payments[_payments.length - 1];

                if (lastPayment.event == null) {
                    return null;
                }

                const ev = JSON.parse(JSON.stringify(lastPayment.event));
                ev.paymentPlans = plans;
                ev.maxValue = _currentPayment.paymentValue;

                return ev;
            };

            var wipeInitializedPaymentData = () => {
                if (_currentPayment) {
                    LogService.info(`Removendo dados inicializados no Serviço de pagamento...`);
                }
                _currentPayment = undefined;

                if (!_payments.length) {
                    $rootScope.$broadcast('paymentInProgress', false);
                }
            };

            this.clearInitializedPaymentData = () => {
                wipeInitializedPaymentData();
            };

            this.clearPayments = function() {
                LogService.info(`Limpando a lista de pagamentos realizados e descontos associados a esses.`);
                _payments.length = 0;
                _confirmedPayments.length = 0;

                wipeInitializedPaymentData();

                // dependencias
                CartService.cart.salePayments.length = 0;
                CartService.cart.additionalCharge = 0;

                var discounts = CartService.cart.discounts;
                if (discounts) {
                    for (var i = 0; i < discounts.length; i++) {
                        if (discounts[i].source === 'PLAN') {
                            discounts.splice(i, 1);
                        }
                    }
                }
                LogService.info(`Lista de pagamentos limpa.`);
            };

            /**
             * Verifica se há pagamento de cliente próprio.
             */
            this.containsPrivateLabelPayment = function() {
                var ret = false;

                CartService.cart.salePayments.forEach(salePayment => {
                    if (salePayment.type === this.SALE_PRIVATE_LABEL_PAYMENT) {
                        ret = true;
                    }
                }, this);
                return ret;
            };

            /**
             * Registrando função para o caso de reinicialização do carrinho.
             */
            CartService.initCartObservers.add(this.clearPayments);

            this.pushMobileRecharge = (data) => {
                PaymentProcessor.pushMobileRecharge(data);
            }

            this.confirmPaymentsWith = async function(nsu) {
               await PaymentProcessor.confirm(nsu)
                    .then(() => self.notifyObservers(this.ON_PAYMENT_COMPLETE));

            };

            this.confirmPayments = async function() {
               await self.confirmPaymentsWith(CartService.cart.nsu);
            };

            this.cancelWith = (nsu, alerting) => {
                // não é checado no paymentProcessor pois existem formas que não são inseridas nele, como dinheiro
                const containsPayment = CartService.cart.salePayments != null
                    && CartService.cart.salePayments.length > 0
                    && CartService.cart.confirmedPayments !== true;

                let msg = 'Pagamentos existentes foram desfeitos.';

                if (alerting && containsPayment) {
                    for (var i=0; i < CartService.cart.salePayments.length; i++) {
                        if (CartService.cart.salePayments[i].type === SalePaymentTypes.SALE_E_WALLET_PAYMENT
                            && CartService.cart.salePayments[i].paymentProvider !== null
                            && CartService.cart.salePayments[i].paymentProvider.id === 'QR_LINX') {
                            msg = 'Pagamentos existentes foram desfeitos.<br>Não se preocupe!<br>Caso você tenha efetuado o PIX, ele será estornado em até 48 horas.<br>';
                            break;
                        }
                    }
                }

                return PaymentProcessor.undo(nsu)
                    .catch(LogService.error)
                    .then(CartService.undoFirtsPurchaseDiscount)
                    .then(CartService.undoCashbackDiscountPartial)
                    .then(undoPaymentCampaigns)
                    .then(this.clearPayments)
                    .then(() => alerting && containsPayment ?
                        AlertService.alert("Aviso!", msg) : $q.resolve());
            };

            this.cancel = alerting => {
                return this.cancelWith(CartService.cart.nsu, alerting);
            };

            async function undoPaymentCampaigns() {
                const campaigns = service.currentPayment.campaigns;

                if (campaigns && campaigns.length) {
                    campaigns.forEach(campaign => {
                        LogService.info(`Desfazendo desconto de campanha por pagamento: ${campaign.codeCampaign}`);
                        CartService.undoCampaignDiscountsPartial(campaign);
                    });
                }
            };

            this.search = function() {
                _loadSettings();

                const currentPayment = this.currentPayment();

                if (service.isPendingDownPayment()) {
                    const ev = downPaymentEvent();
                    return $q.resolve({ paymentPlanEvents: ev != null ? [ ev ] : [] });
                }

                const customer = CartService.cart.customer;

                const salePayments = CartService.cart.salePayments.map(
                    (salePayment) => ({
                        paymentSequence: salePayment.paymentSequence,
                        paymentCodeMethod: salePayment.paymentCodeMethod,
                        paymentCodePlan: salePayment.paymentCodePlan,
                        paymentCodeEvent: salePayment.paymentCodeEvent,
                        paymentValue: salePayment.paymentValue
                    })
                );

                const paymentPlanRequest = {
                    paymentValue: currentPayment.paymentValue,
                    totalValue: currentPayment.totalValue,
                    paymentMethod: currentPayment.paymentMethod,
                    cardBin: currentPayment.cardBin,
                    cardNumber: currentPayment.cardNumber,
                    saleDate: currentPayment.saleDate,
                    authorizerCode: currentPayment.authorizerCode,
                    userCode: currentPayment.userCode,
                    companyNumber: currentPayment.companyNumber,
                    storeNumber: currentPayment.storeNumber,
                    paymentProtectionInsurance: currentPayment.paymentProtectionInsurance,
                    paymentOperationType: currentPayment.paymentOperationType,
                    fiscalItems: currentPayment.fiscalItems,
                    privateLabel: currentPayment.privateLabel,
                    payments: salePayments,
                    includeDownPaymentPlans: true,
                };

                if (CartService.cart.nonFiscalItems) {
                    paymentPlanRequest.nonFiscalItems = CartService.cart.nonFiscalItems.map(
                        (nonFis) => ({
                            value: nonFis.value,
                            nonFiscalOperationType: nonFis.nonFiscalOperationType
                        })
                    );
                }

                if (customer) {
                    paymentPlanRequest.dueDate = customer.dueDate;
                    paymentPlanRequest.closeBillingDay = customer.closeBillingDay;
                    paymentPlanRequest.cardNumber = customer.cardNumber;
                }

                // valor caso o provider não tenha timeout configurado
                const timeout = $rootScope.defaultRequestTimeoutInMillis;

                const provider = currentPayment.provider ? { id: currentPayment.provider } : getMappedValue(
                    currentPayment.paymentMethod,
                    ConfigService.config.payment_plan.search.providers || {},
                    { id: 'STOREX', timeout: timeout } // valor default
                );

                const fbProvider = getMappedValue(
                    currentPayment.paymentMethod,
                    ConfigService.config.payment_plan.search.fb_providers || {},
                    null // por padrão não temos fallback em caso de erros
                );

                if (provider.timeout == null) {
                    provider.timeout = timeout;
                }
                if (fbProvider != null && fbProvider.timeout == null) {
                    fbProvider.timeout = timeout;
                }

                const url = _url + '/v1/paymentplan/search';

                const ret = _doSearchPlans(url, provider, paymentPlanRequest);

                if (fbProvider == null) {
                    return ret;
                }

                return ret.then((res) => {
                    // Alguns providers não lançam erro, apenas retornam uma lista vazia de eventos,
                    // logo deve ser tratado para repetir a requisição em algum provider de
                    // fallback.
                    if (res.paymentPlanEvents == null || res.paymentPlanEvents.length == 0) {
                        LogService.warn(
                            `Não foram retornados planos do tipo ${body.paymentMethod}, provider ${provider.id}`);
                        return $q.reject();
                    }
                    return res;
                }).catch(err =>
                    _doSearchPlans(url, fbProvider, paymentPlanRequest)
                );
            };

            function _doSearchPlans(url, provider, body) {
                LogService.info(`Consultando planos de tipo ${body.paymentMethod} utilizando provider ${provider.id}, timeout ${provider.timeout}`);
                body.provider = provider.id;
                const headers = { 'Service-Provider': provider.id };
                return $http.post(url, body, { timeout: provider.timeout, headers: headers }).then(res =>
                    // mantendo compatibilidade com controllers que já utilizam o search
                    res.data
                ).catch(err => {
                    LogService.error(`Falha ao consultar planos do tipo ${body.paymentMethod}, provider ${provider.id}`);
                    return $q.reject(err);
                });
            }

            function getMappedValue(paymentMethod, providerMap, defaultValue) {
                if (paymentMethod == null) return defaultValue;
                return providerMap[paymentMethod] || defaultValue;
            }

            /**
             * Consulta cartão virtual do cliente do carrinho.
             */
            this.getVirtualCard = function() {
                _loadSettings();

                const customer = CartService.cart.customer;
                var virtualCardResource = $resource(_url + '/v1/financial/virtualCardNumber/:clientId', { clientId: '@document' });

                return virtualCardResource.get(customer).$promise;
            };

            /**
             * Autorização de pagamento TEF
             */
            this.tefPaymentAuthorization = () => {
                var defer = $q.defer();

                if (ConfigService.config.business.fiscal_auth_type &&
                    ConfigService.config.business.fiscal_auth_type === 'INTEGRADOR_FISCAL' &&
                    CartService.getCurrentOperation() !== 'SERVICE') {
                    _loadSettings();

                    let req = {
                        companyNumber: _currentPayment.companyNumber,
                        storeNumber: _currentPayment.storeNumber,
                        terminalNumber: CartService.cart.terminalNumber,
                        icmsCalculationBasisTotalValue: (fiscalItems => {
                            let filteredItems = fiscalItems.filter(item => item.printerInput != null &&
                                item.printerInput.toUpperCase() !== 'S'); //S significa serviço.
                            return filteredItems.reduce((total, item) => total + item.unitValue, 0);
                        })(CartService.cart.fiscalItems),
                        paymentValue: _currentPayment.paymentValue
                    };
                    let resource = $resource(_url + '/v1/payment/tefAuthorization', {}, { tefAuthorization: { method: 'POST' } });

                    resource.tefAuthorization({}, req, results => {
                        if (results) {
                            _currentPayment.mideReferenceID = results.mideReferenceID;
                            _currentPayment.sefazNSU = results.sefazNSU;
                            _currentPayment.fiscalIntegratorAuthModeOff = results.fiscalIntegratorAuthModeOff;
                        }
                    }).$promise.then(defer.resolve, defer.reject);
                } else {
                    defer.resolve();
                }
                return defer.promise;
            };

            /**
             * Confirmação de Pagamento TEF
             */
            this.tefOnlineAuthConfirmationStatus = (salePayment) => {
                var defer = $q.defer();

                if (ConfigService.config.business.fiscal_auth_type &&
                    ConfigService.config.business.fiscal_auth_type === 'INTEGRADOR_FISCAL' &&
                    CartService.getCurrentOperation() !== 'SERVICE' &&
                    salePayment.mideReferenceID &&
                    salePayment.sefazNSU &&
                    !salePayment.fiscalIntegratorAuthModeOff) {
                    _loadSettings();
                    let resource = $resource(_url + '/v1/payment/tefOnlineAuthConfirmation', {}, { tefOnlineAuthConfirmation: { method: 'POST' } });

                    resource.tefOnlineAuthConfirmation({}, { tefpayment: salePayment }, defer.resolve, defer.reject);
                } else {
                    defer.resolve();
                }
                return defer.promise;
            };

            this.voucherAssurantPaymentAuthorization = () => {
                var defer = $q.defer();
                if (ConfigService.config.business.fiscal_auth_type &&
                    ConfigService.config.business.fiscal_auth_type === 'INTEGRADOR_FISCAL' &&
                    CartService.getCurrentOperation() !== 'SERVICE' &&
                    salePayment.mideReferenceID &&
                    salePayment.sefazNSU &&
                    !salePayment.fiscalIntegratorAuthModeOff) {
                    _loadSettings();
                    // let resource = $resource(_url + '/v1/payment/tefOnlineAuthConfirmation', {}, {  tefOnlineAuthConfirmation : { method : 'POST' } });

                    // resource.tefOnlineAuthConfirmation({}, { tefpayment: salePayment }, defer.resolve, defer.reject);
                } else {
                    defer.resolve();
                }
            };

            this.voucherGivexPaymentAuthorization = () => {
                var defer = $q.defer();
                if (ConfigService.config.business.fiscal_auth_type &&
                    ConfigService.config.business.fiscal_auth_type === 'INTEGRADOR_FISCAL' &&
                    CartService.getCurrentOperation() !== 'SERVICE' &&
                    salePayment.mideReferenceID &&
                    salePayment.sefazNSU &&
                    !salePayment.fiscalIntegratorAuthModeOff) {
                    _loadSettings();
                    // let resource = $resource(_url + '/v1/payment/tefOnlineAuthConfirmation', {}, {  tefOnlineAuthConfirmation : { method : 'POST' } });

                    // resource.tefOnlineAuthConfirmation({}, { tefpayment: salePayment }, defer.resolve, defer.reject);
                } else {
                    defer.resolve();
                }
            };

            this.voucherStorexPaymentAuthorization = () => {
                var defer = $q.defer();
                if (ConfigService.config.business.fiscal_auth_type &&
                    ConfigService.config.business.fiscal_auth_type === 'INTEGRADOR_FISCAL' &&
                    CartService.getCurrentOperation() !== 'SERVICE' &&
                    salePayment.mideReferenceID &&
                    salePayment.sefazNSU &&
                    !salePayment.fiscalIntegratorAuthModeOff) {
                    _loadSettings();
                    // let resource = $resource(_url + '/v1/payment/tefOnlineAuthConfirmation', {}, {  tefOnlineAuthConfirmation : { method : 'POST' } });

                    // resource.tefOnlineAuthConfirmation({}, { tefpayment: salePayment }, defer.resolve, defer.reject);
                } else {
                    defer.resolve();
                }
            };


            this.isCardBinEnabled = (paymentCodeMethod, cardNumber) => {
                var defer = $q.defer();

                if (paymentCodeMethod === 'CREDIT' && ConfigService.config.business.disabled_credit_card_bins && ConfigService.config.business.disabled_credit_card_bins.length) {
                    if (cardNumber && cardNumber.length > 6) {
                        cardNumber = cardNumber.substring(0, 6);
                    }
                    var disabled = ConfigService.config.business.disabled_credit_card_bins.includes(cardNumber);
                    if (disabled) {
                        defer.reject("Cartão não habilitado para esta forma de pagamento!");
                    } else {
                        defer.resolve();
                    }
                } else {
                    defer.resolve();
                }

                return defer.promise;
            };

            this.getPaymentDescription = (paymentCodeMethod) => {
                const payment = paymentsMethods.find(payment => payment.id == paymentCodeMethod);
                return payment ? payment.description : 'Forma não identificada';
            };
            this.getPaymentCode = (paymentCodeMethod) => {
                const payment = paymentsMethods.find(payment => payment.id == paymentCodeMethod);
                return payment ? payment.code : null;
            };
        }
        var service = new PaymentService();

        return service;
    });
})();
