(function() {
	'use strict';

	angular.module('easyorder.services.printer', ['ngResource', 'easyorder.services.config'])

	.factory('PrinterService', function($resource, ConfigService, $q) {

		var service = {
			url: ''
		};

		service.ORIGIN_WS = 0x01;
		service.ORIGIN_PLUGIN = 0x02;

		service.loadSettings = function() {
			service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
			service.url += ConfigService.config.linxwebservices.host + ':';
			service.url += ConfigService.config.linxwebservices.port;
			service.url += '/linx-webservices';
		};

		service.search = function(params) {
			service.loadSettings();
			return $resource(service.url + '/v1/printers/search', {}, { search: { method: 'POST' }})
					.search(params)
					.$promise
					.then(ret => {
						if (ret.success && ret.printers.length > 0) {
							ret.printers.forEach(printer => {
								printer.origin = service.ORIGIN_WS;
							});
							return ret.printers;
						} else {
							return new Array();
						}
					})
					.catch(() => {
						return new Array();
					})
                    .then(wsPrinters => {
                        if (ionic.Platform.isAndroid()) {
					        return $q((resolve) => {
						        window.plugins.linxPrint.listPrinters(printers => {
							        wsPrinters.push(...printers.map(pluginPrinterToWsPrinter));
								    resolve(wsPrinters);
							    }, () => resolve(wsPrinters));
						    });
                        } else {
                            return $q(resolve => resolve(wsPrinters));
                        }
					})
					.then(printers => {
						if (printers.length == 0) {
							return $q.reject(new Error('NO_PRINTERS_FOUND'));
						} else {
							return printers;
						}
					});
		};

		function pluginPrinterToWsPrinter(pluginPrinter) {
			return {
				origin: service.ORIGIN_PLUGIN,
				id: pluginPrinter.uuid,
				name: pluginPrinter.name,
				paperType: pluginPaperTypeToWsPaperType(pluginPrinter.paperType)
			};
		}

		function pluginPaperTypeToWsPaperType(pluginPaperType) {
			switch(pluginPaperType) {
				case window.plugins.linxPrint.PAPER_TYPE_A_SERIES:
					return 'A_SERIES';
				case window.plugins.linxPrint.PAPER_TYPE_THERMAL_42_COL:
					return 'THERMAL_42COLS';
				case window.plugins.linxPrint.PAPER_TYPE_THERMAL_56_COL:
					return 'THERMAL_56COLS';
				default:
					throw new Error('paper type not implemented');

			}
		}

		service.submit = function(params) {
			service.loadSettings();

			switch(params.origin) {
				case service.ORIGIN_PLUGIN:
					return $q((resolve, reject) => {
						window.plugins.linxPrint.print(wsParamsToPluginPrintRequest(params), resolve, reject);
					});
				case service.ORIGIN_WS:
					delete params.origin; // used internally for origin distinction
					delete params.paperType; // only necessary for plugin
					return $q((resolve, reject) => {
						$resource(service.url + '/v1/printers/submit', {}, { submit: { method: 'POST' } })
							.submit(params, ret => {
								if (ret.success) {
									resolve('SUCCESS');
								} else {
									reject({
										data : {
											message : "Erro enviando impresso: " + result.message
										}
									});
								}
							}, reject);
					});
				default:
					throw new Error('Origin not supported');
			}
		};

		function wsParamsToPluginPrintRequest(wsParams) {
			return {
				uuid: wsParams.printerid,
				mimeType: wsContentTypeToPluginMimeType(wsParams.contentType),
				data: wsParams.content,
				paperType: wsPaperTypeToPluginPaperType(wsParams.paperType),
				title: wsParams.title
			}
		}

		function wsContentTypeToPluginMimeType(contentType) {
			switch(contentType) {
				case 'application/pdf':
					return window.plugins.linxPrint.MIME_TYPE_PDF;
				case 'application/vnd.linx.printer-text':
					return window.plugins.linxPrint.MIME_TYPE_LINX;
				default:
					throw new Error('contentType not implemented');
			}
		}

		function wsPaperTypeToPluginPaperType(wsPaperType) {
			switch(wsPaperType) {
				case 'A_SERIES':
					return window.plugins.linxPrint.PAPER_TYPE_A_SERIES;
				case 'THERMAL_42COLS':
					return window.plugins.linxPrint.PAPER_TYPE_THERMAL_42_COL;
				case 'THERMAL_56COLS':
					return window.plugins.linxPrint.PAPER_TYPE_THERMAL_56_COL;
				default:
					throw new Error('paperType not implemented');
			}
		}

		return service;
	})
})();
