(function () {
    'use strict';

    angular.module('easyorder.services.privatelabel', [
            'ngResource',
            'easyorder.services.util',
            'easyorder.services.config'
        ])

        .factory('PrivateLabelService', function (
            $resource,
            ConfigService) {
            
            function PrivateLabelService() {

                var service = {
                    companyNumber: 0,
                    storeNumber: 0,
                    terminalNumber: 0,
                    url: ''
                };

                service.loadSettings = function () {
                    service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                    service.url += ConfigService.config.linxwebservices.host + ':';
                    service.url += ConfigService.config.linxwebservices.port;
                    service.url += '/linx-webservices/v1/privatelabel';
                    service.companyNumber = ConfigService.config.companyNumber;
                    service.storeNumber = ConfigService.config.storeNumber;
                    service.terminalNumber = ConfigService.config.terminalNumber;
                }
                
                /**
                 * Consulta de status da autorização de pagamento 
                 * @param {Id da autorização do pagamento} authorizationId 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                this.paymentStatus = function (authorizationId, success, error) {
                    service.loadSettings();

                    return $resource(service.url + '/payment/auth/status/:authorizationId', { authorizationId: authorizationId }, {
                        search: {
                            method: 'GET'
                        }
                    }).search({}, success, error).$promise;
                }

                
                /**
                 * Autorização de pagamento para cartões próprios Renner
                 * @param {Requisição de autorização} paymentAuthRequest 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                this.authorizePayment = function (paymentAuthRequest, success, error) {
                    service.loadSettings();
                    paymentAuthRequest.storeNumber      = service.storeNumber;
                    paymentAuthRequest.terminalNumber   = service.terminalNumber;
                    paymentAuthRequest.coupomNumber     = ConfigService.nsuControll.currentCartNSU();

                    return $resource(service.url + '/payment/auth', {}, { 
                        send: { 
                            method: 'POST'
                        }
                    }).send({}, paymentAuthRequest, success, error).$promise;
                }

                /**
                 * Confirmação do pagamento para cartõe próprio Renner
                 * @param {Pagamento do carrinho} salePayment 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                 this.confirmPayment = function (salePayment, success, error) {
                    service.loadSettings();
                    let paymentConfirmRequest                           = {};
                    paymentConfirmRequest.paymentAuthorizationCode      = salePayment.paymentAuthorizationCode;

                    return $resource(service.url + '/payment/confirm', {}, { 
                        send: { 
                            method: 'POST',
                            headers: createHeader(salePayment.paymentProvider)
                        }
                    }).send({}, paymentConfirmRequest, success, error).$promise;
                }

                /**
                 * Desfazimento do pagamento para cartõe próprio Renner
                 * @param {Pagamento do carrinho} salePayment 
                 * @param {Retorno bem sucedido} success 
                 * @param {Retorno com erro} error 
                 * @returns 
                 */
                 this.undoPayment = function (salePayment, success, error) {
                    service.loadSettings();
                    let paymentUndoRequest                              = {}
                    paymentUndoRequest.paymentAuthorizationCode         = salePayment.paymentAuthorizationCode;
                    paymentUndoRequest.storeNumber                      = service.storeNumber;
                    paymentUndoRequest.terminalNumber                   = service.terminalNumber;

                    return $resource(service.url + '/payment/undo', {}, { 
                        send: { 
                            method: 'POST',
                            headers: createHeader(salePayment.paymentProvider)
                        }
                    }).send({}, paymentUndoRequest, success, error).$promise;
                }

                function createHeader(provider) {
                    let headerData = {};
                    if (provider) {
                        headerData['Service-Provider'] = provider.id;
                    }
                    return headerData;
                }

            }
            var service = new PrivateLabelService();
            return service;
        })
})();