(function() {
	'use strict';

	angular.module('easyorder.services.product', [
		'ngResource',
		'easyorder.services.config'])

	.factory('ProductService', function($resource, ConfigService, UtilService, AlertService) {

		var service = {
			companyNumber: 0,
			storeNumber: 0,
			url: ''
		};

		service.loadSettings = function() {
			service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
			service.url += ConfigService.config.linxwebservices.host + ':';
			service.url += ConfigService.config.linxwebservices.port;
			service.url += '/linx-webservices';
			service.companyNumber = ConfigService.config.companyNumber;
			service.storeNumber   = ConfigService.config.storeNumber;
			service.productUrlCodeType   = ConfigService.config.business.product_url_code_type;
		};

		service.status = function() {
			service.loadSettings();
			return $resource(service.url + '/status').get();
		};

		service.search = function(params, success, error) {
			service.loadSettings();
			params.companyNumber = service.companyNumber;
			params.storeNumber   = service.storeNumber;
			
			return $resource(service.url + '/v1/products/saleinfo').get(params, success ? success : null, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};

		service.searchTaxes = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/taxes').query(params, success ? success : null, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};
        
		service.searchTaxes = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/taxes').query(params, success ? success : null, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};

        service.summarySearch = function(params) {
            service.loadSettings();
            params.companyNumber = service.companyNumber;
            params.storeNumber   = service.storeNumber;
            
            return $resource(service.url + '/v1/products/saleinfo/summary').get(params).$promise;
        };
		
		service.lookupInventory = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/inventory/lookup', {}, { inventoryLookup: { method: 'POST' } })
			    .inventoryLookup({}, params, success,  error).$promise;
		};

		service.lookupByUUID = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/inventory/lookup/uuid', {}, { lookupByUUID: { method: 'POST' } })
			    .lookupByUUID({}, params, success,  error).$promise;
		};

		service.reserveProduct = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/inventory/reserve', {}, { reserveProduct: { method: 'POST' } })
			    .reserveProduct({}, params, success,  error).$promise;
		};

		service.confirmSale = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/inventory/confirmsale', {}, { confirmSale: { method: 'POST' } })
			    .confirmSale({}, params, success,  error).$promise;
		};

		/**
         * Cancelamento da reserva
         */

		 service.cancelSale = function(params, success, error) {
			 service.loadSettings();
			 return $resource(service.url + '/v1/products/inventory/cancel', {}, {
				 cancelSale: { method: 'POST' }
			 }).cancelSale({}, params, success, error).$promise;
		 }

		/**
         *
         */
		service.discount = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/discount/rules/search', {}, { discount: { method: 'POST' } })
			    .discount({}, params, success,  UtilService.wrapCancelableResponseErrorHandler(error)).$promise;
		};

		service.getProductImageUrl = function(item) {
			service.loadSettings();
			if (service.productUrlCodeType === "PRODUCT_CODE") {
				return service.url + '/v1/images/products/code/' + item.productCode + '/' + item.productCode;
			} else {
				return service.url + '/v1/images/products/ean/' + item.automationCode + '/' + item.automationCode;
            }
		};
        /**
         *
         */
        service.shouldProcessPartnerProductOf = (item) => {
            if (item.productType === "PARTNER") {
                return !ConfigService.config.business.enables_rfid_reading;
            } else {
                return false;
            }
        };

		service.searchStyleInfo = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/style-info').get(params, success ? success : null, error ? 
				UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};

		service.searchProductAdditionalInfo = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/additional-info').get(params, success ? success : null, error ? 
				UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};
		
		service.batchValidation = function(params, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/products/batch-validation').get(params, success ? success : null, error ? UtilService.wrapCancelableResponseErrorHandler(error) : null).$promise;
		};
        return service;
	})
})
();
