(function () {
    'use strict';

    angular.module('easyorder.services.reportservice', [
        'ngResource',
        'easyorder.services.config'
    ]).factory('ReportService', function ($resource,
                                                ConfigService) {

        const service = {
       
        };
        const serviceURL = () => {
            const linxwebservices = ConfigService.config.linxwebservices;
            return (linxwebservices.secure ? 'https://' : 'http://') + linxwebservices.host + ':' + linxwebservices.port + '/linx-webservices/v1/reports';
        };

        service.consolidatedsales = (companyNumber, storeNumber, terminalNumber, operatorCode, date, appInfo, receiptVersion) => {
            return $resource(serviceURL() + '/sales/store/consolidated', {}, {
                consolidatedsales: {
                    method: 'POST'
                }
            }).consolidatedsales({
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                operatorCode: operatorCode,
                initialDate: date,
                finalDate: date,
                appInfo: appInfo,
                receiptVersion: receiptVersion
            }).$promise;
        };

        service.storesales = (companyNumber, storeNumber, terminalNumber, operatorCode, initialDate, finalDate, operatorCodes, terminalTypes, appInfo) => {
            return $resource(serviceURL() + '/sales/store', {}, {
                storesales: {
                    method: 'POST'
                }
            }).storesales({
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                operatorCode: operatorCode,
                initialDate: initialDate,
                finalDate: finalDate,
                operatorCodes: operatorCodes,
                terminalTypes: terminalTypes,
                appInfo: appInfo
            }).$promise;
        };

        service.operatorsales = (companyNumber, storeNumber, terminalNumber, operatorCode, operatorCodes, initialDate, finalDate, terminalTypes, appInfo) => {
            return $resource(serviceURL() + '/sales/operator', {}, {
                operatorsales: {
                    method: 'POST'
                }
            }).operatorsales({
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                operatorCode: operatorCode,
                operatorCodes: operatorCodes,
                initialDate: initialDate,
                finalDate: finalDate,
                terminalTypes: terminalTypes,
                appInfo: appInfo
            }).$promise;
        };

        service.getPaymentMethodIconClass = function (paymentMethodCode) {
            switch (paymentMethodCode) {
                case 1: // CASH
                    return "fa fa-money-bill-alt fa-fw";
                case 9: // DEBIT
                case 11: // CREDIT
                    return "fa fa-credit-card fa-fw";
                case 1000: // GIFT_CARD
                    return "fa fa-gift fa-fw";
                default:
                    return "fa fas fa-ticket-alt fa-fw";
            }  
        };

        service.sumValues = function(items) {
            return items
                .map(function(x) { return x.paymentValue; })
                .reduce(function(a, b) { return a + b; });
        };
        
        service.sumSaleValues = function(items) {
            return items
                .map(function(x) { 
                    return x.sale ? x.paymentValue : 0; 
                })
                .reduce(function(a, b) { return a + b; });
        };

        service.hasSaleItems = function(items) {
            return items.filter(item => item.sale).length > 0;
        };

        service.hasOtherItems = function(items) {
            return items.filter(item => !item.sale).length > 0;
        };
        
        return service;
    });
})();
