(function () {
    'use strict';
    angular.module('easyorder.services.servicesale', ['ngResource', 'easyorder.services.config'])

        .factory('ServiceSaleService', function (ConfigService, UserService, $http, $resource, $q, $localStorage) {

            var service = {
                url: ''
            };

            /**
             * 
             */
            service.loadSettings = function () {
                service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
                service.url += ConfigService.config.linxwebservices.host + ':';
                service.url += ConfigService.config.linxwebservices.port;
                service.url += '/linx-webservices/v1/';
            };

            /**
             * 
             * @param {*} params 
             * @param {*} success 
             * @param {*} error 
             */
            service.fetchServices = function (params, success, error) {
                service.loadSettings();
                let param = {
                    "storeNumber": ConfigService.config.storeNumber,
                    "terminalNumber": ConfigService.config.terminalNumber,
                    "productCode": params.productCode,
                    "unitValue": params.unitValue
                };

                return $resource(service.url + 'warranty/search', {}, {
                        services: {
                            method: 'POST'
                        }
                    })
                    .services({}, param, success, error).$promise;
            };

            /**
             * 
             * @param {*} params 
             * @param {*} success 
             * @param {*} error 
             */
            service.purchaseServices = function (param, success, error) {
                service.loadSettings();

                return $resource(service.url + 'warranty/purchase', {}, {
                        services: {
                            method: 'POST'
                        }
                    })
                    .services({}, param, success, error).$promise;
            };

            /**
             * 
             * @param {*} params 
             */
            service.prepareServices = function (cart, purchasedServices) {
                if (!purchasedServices) {
                    var clonedCart = angular.copy(cart);
                    clonedCart.fiscalItems.forEach(fiscalItem => {
                        if (fiscalItem.nonFiscalItems && fiscalItem.nonFiscalItems.length > 0) {
                            for (let index = 0; index < fiscalItem.nonFiscalItems.length; index++) {
                                fiscalItem.nonFiscalItems[index] = service.prepareNonFiscalItems(fiscalItem.nonFiscalItems[index]);
                            }
                        }
                    });
                    return clonedCart;
                } else {

                    //Concat the service selected with the response from purchase service. 
                    return service.prepareNonFiscalItems(purchasedServices.item, purchasedServices.purchased, purchasedServices.document);
                }
            };

            service.prepareNonFiscalItems = function (item, purchase, document) {

                let object = {};

                if (!purchase && !document) {
                    object = angular.copy(item);
                    delete object.description;
                    delete object.automationCode;
                } else {
                    Object.defineProperty(object, 'description', {
                        enumerable: false,
                        value: item.description
                    });

                    Object.defineProperty(object, 'automationCode', {
                        enumerable: false,
                        value: item.id
                    });

                    object.value = item.value; //productValue
                    object.externalCode = item.uid;
                    object.warrantyPeriod = item.warrantyPeriod;
                    object.warrantyValue = item.value; // transactionValue
                    object.contractorDocumentType = document.contractorDocumentType;
                    object.contractorDocument = document.contractorDocument;
                    object.nonFiscalOperationType = item.type;
                    object.branch = item.branch;
                    object.branchId = item.branchId;
                }

                return object;
            };

            service.loadSettings();

            return service;
        });
})();
