(function() {
    'use strict';
    angular.module('easyorder.services.sms', [
        'ngResource',
        'easyorder.services.config',
        'easyorder.services.log'
    ])

    .factory('SmsService', function(ConfigService, $resource, LogService) {
        var service = {
            url : ''
        };
        
        /**
         * 
         */
        service.loadSettings = function() {
            service.url = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            service.url += ConfigService.config.linxwebservices.host + ':';
            service.url += ConfigService.config.linxwebservices.port;
            service.url += '/linx-webservices/v1/sms';
        };
        
        /**
         * 
         */
        service.sendSms = function(from, to, msg, id, successCallback, errorCallback) {
            var params = {
                from : from,
                to : to,
                msg : msg,
                id : id
            };

            return $resource(service.url + '/send', {}, {
                send : {
                    method : 'POST'
                }
            }).send({}, params, successCallback, errorCallback).$promise;
        };
        
        /**
         * 
         */
        service.isSmsSent = function(smsSendResponse) {
            var sent = false;
            
            if (smsSendResponse && smsSendResponse.statusCode) {
                switch (smsSendResponse.statusCode) {
                    case "00" : // OK
                    case "01" : // SCHEDULED
                    case "02" : // SENT
                    case "03" : // DELIVERED
                        sent = true;
                        break;
                    default :
                        sent = false;
                        break;
                }
                LogService.info("Envio de SMS: ", smsSendResponse.statusCode, ' Envio: ', sent)
            }
            return sent;
        };

        /**
         * 
         */
        service.isSmsNumberValid = function(smsNumber) {
            var smsNumberValid = false;
            
            if (smsNumber) {
                var onlyNumbers = smsNumber.replace(/\D/g, '');
                smsNumberValid = (onlyNumbers.length > 9);
            }
            return smsNumberValid;
        };

        /**
         * 
         */
        service.getFinalSmsNumber = function(smsNumber) {
            var finalSmsNumber;
            
            if (smsNumber) {
                if (smsNumber.startsWith('+55')) {
                    finalSmsNumber = smsNumber;
                } else {
                    finalSmsNumber = "+55" + smsNumber;
                }
            }
            return finalSmsNumber;
        };

        /**
         * 
         */
        service.getSmsResponseMessage = function(smsSendResponse) {
            var message;
            
            if (smsSendResponse && smsSendResponse.statusCode && smsSendResponse.statusDescription) {
                message = smsSendResponse.statusCode + " - " + smsSendResponse.statusDescription;
            } else {
                message = "Erro desconhecido!";
            }
            return message;
        };

        service.loadSettings();

        return service;
    });
})();
