(function () {
    'use strict';

    angular.module('easyorder.services.transactionreturnservice', [
        'ngResource',
        'easyorder.services.config'
    ]).factory('TransactionReturnService', function ($resource,
                                                     ConfigService,
                                                     $http) {

        const linxwebservices = ConfigService.config.linxwebservices;
        const service = {
            url: () => `${linxwebservices.secure ? 'https://' : 'http://'}${linxwebservices.host}:${linxwebservices.port}/linx-webservices/v1/returns`,
            provider: () => ConfigService.config.business.return_service_provider
        };

        service.search = params => {
            return $resource(service.url() + '/search-transaction', {}, {
                searchTransactionRequest: {
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                }
            }).searchTransactionRequest(params).$promise;
        };

        service.searchReasons = params => {
            return $http.get(service.url() + '/reasons', {
                params: params,
                headers: {
                    'Service-Provider': service.provider
                }
            });
        };

        service.return = params => {
            return $resource(service.url() + '/return', {}, {
                returnRequest: {
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                }
            }).returnRequest(params).$promise;
        };

        service.bacthReturn = params => {
            const resource = $resource(`${service.url()}/batch-return`, {}, {
                doRequest: {
                    method: 'POST',
                    headers: {
                        'Service-Provider': service.provider
                    }
                }
            });
            return resource.doRequest(params).$promise;
        };

        service.validateReturnRequest = params => {
            const resource = $resource(`${service.url()}/validate-return`, {}, {
                doRequest: {
                    method: 'PUT',
                    headers: {
                        'Service-Provider': service.provider
                    }
                }
            });
            return resource.doRequest(params).$promise;
        };
        return service;
    });
})();
