(function() {
	'use strict';

	angular.module('easyorder.services.user', [
		'ngResource',
		'easyorder.services.config',
		'easyorder.services.log'
	])

	.factory('UserService', function($http, $ionicModal, ConfigService, UtilService, $q, $resource, $ionicLoading, $injector, $rootScope,
	        $timeout, ProductService, LogService, AlertService, ConfigUpdateService) {

		var service = {
			currentUser: {},
			url: ''
		};

		if(typeof device !== 'undefined'){
			var platform = device.platform === "Android" ? "ANDROID" : "IOS";
		} else {
			var platform = "WEB";
		}

		service.loadSettings = function() {
			service.url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
			service.url += ConfigService.config.linxwebservices.host + ':';
			service.url += ConfigService.config.linxwebservices.port;
			service.url += '/linx-webservices';
		};

		service.loadAuthorizationSettings = function() {
			service.authurl  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
			service.authurl += ConfigService.config.linxwebservices.host + ':';
			service.authurl += ConfigService.config.linxwebservices.port;
			service.authurl += '/linx-webservices/v1/authorization';
		};

		const loginSslPinning = (user, passwd) => {
			const deferred = $q.defer();
			// window.cordova.plugin.http.setDataSerializer('urlencoded');

			const obj = {
				appType: ConfigService.config.appType,
				appVersion: $rootScope.appVersion,
				companyNumber: ConfigService.config.companyNumber,
				hostAddress: window.ionic.ipAddress,
				ipAddress: window.ionic.ipAddress || '127.0.0.1',
				originSystem: 'CAIXA_RAPIDO',
				password: passwd,
				platform: platform,
				storeNumber: ConfigService.config.storeNumber,
				terminalNumber: ConfigService.config.terminalNumber,
				username: user
			};
			const url = service.url + '/signin-config';
			const headers = {
				'Message-Key': UtilService.createMessageKey(obj, true),
				'Request-Key': UtilService.createRequestKey(url)
			};

            const options = {
                method: 'post',
                data: obj,
                headers: headers,
                responseType: 'json'
              };
			window.cordova.plugin.http.sendRequest(url,
				options,
				// onSuccess
				function (result) {
					const data = result.data;

					let authorizationHeader = result.headers['authorization'];

					if (authorizationHeader) {
						var bearer = "Bearer ";
						if (authorizationHeader.startsWith(bearer)) {
							var token = authorizationHeader.substring(bearer.length, authorizationHeader.length);
							localStorage.setItem('id_token', token);
						}
					}

					service.currentUser = Object.assign({}, data.user);

					if (data.configAppData && data.configAppData.length) {
						ConfigUpdateService.loadParameters(data.configAppData);

						$injector.get('EFTService').activate().then(() => {
							$rootScope.dataSourceManager.updateEftActivationStorage()
								.then(() => deferred.resolve())
								.catch((err) => deferred.reject(err));
						}).catch((err) => deferred.reject(err));
					} else {
						deferred.resolve();
					}
				}, // onFail
				function (response) {
					const error = UtilService.convertSslPluginErrorResponse(response);
					deferred.reject(error);
				});

			return deferred.promise;
		}


		const loginNoCheck = (user, passwd) => {
			let tefserver = null;

			return $http({
				method : 'POST',
				headers : {
					'Content-Type' : 'application/x-www-form-urlencoded'
				},
				url : service.url + '/signin-config',
				transformRequest : function(obj) {
					return UtilService.encodeObjectURI(obj);
				},
				data : {
				    username: user,
                    password: passwd,
                    platform: platform,
                    originSystem: 'CAIXA_RAPIDO',
                    ipAddress: window.ionic.ipAddress || '127.0.0.1',
                    hostAddress: window.ionic.ipAddress,
                    storeNumber: ConfigService.config.storeNumber,
                    terminalNumber: ConfigService.config.terminalNumber,
                    companyNumber: ConfigService.config.companyNumber,
                    appType: ConfigService.config.appType,
                    appVersion: $rootScope.appVersion
				}
			}).then(async function(result) {
				service.currentUser = Object.assign({}, result.data.user);
				if (result.data.configAppData && result.data.configAppData.length) {
					tefserver = ConfigService.config.tefserver;

				    ConfigUpdateService.loadParameters(result.data.configAppData);

                    if (result.data.notifications && result.data.notifications.length > 0) {
                       for (const notification of result.data.notifications) {
                            await AlertService.warn(notification);
                       }
                    }
				}
			}).then(() =>
				$injector.get('EFTService')
					.activate()
					.then(() =>
						$rootScope.dataSourceManager.updateEftActivationStorage()
					)
            );
		}

		service.login = function(user, passwd) {
			service.loadSettings();
			if (SSLP) {
				return loginSslPinning(user, passwd);
			} else {
				return loginNoCheck(user, passwd)
			}
		};

		service.changePassword = function(username, currentPassword, password) {
			service.loadSettings();

			return $http({
				method : 'POST',
				headers : {
					'Content-Type' : 'application/x-www-form-urlencoded'
				},
				url : service.url + '/password-change',
				transformRequest : function(obj) {
					return UtilService.encodeObjectURI(obj);
				},
				data : {
				    username: username,
                    currentPassword: currentPassword,
                    password: password
				}
			});
		};

		/**
		 *
		 */
		service.authorizedLogin = function(user, passwd, superUser, superPasswd) {
			service.loadSettings();

			return $http({
				method : 'POST',
				headers : {
					'Content-Type' : 'application/x-www-form-urlencoded'
				},
				url : service.url + '/authorized-signin',
				transformRequest : function(obj) {
					return UtilService.encodeObjectURI(obj);
				},
				data : {
				    username: user,
					password: passwd,
					authUsername: superUser,
					authPassword: superPasswd,
					requiredPermissions: 'allow-contingency-login',
                    platform: platform,
                    originSystem: 'CAIXA_RAPIDO',
                    ipAddress: window.ionic.ipAddress,
                    hostAddress: window.ionic.ipAddress,
                    storeNumber: ConfigService.config.storeNumber,
                    terminalNumber: ConfigService.config.terminalNumber
				}
			}).then(function(user) {
				LogService.info("Usuário: ", user)
				service.currentUser = Object.assign({}, user.data);
			}, function(error) {
				LogService.error(error)
				return $q.reject(error);
			});
		};

		/**
		 *
		 */
		service.logout = function(doLogoutRestRequest) {
		    if (localStorage && localStorage.getItem('id_token')) {
    		    service.loadSettings();

                let params = {
                    username: service.currentUser.username,
                    ipAddress: window.ionic.ipAddress,
                    storeNumber: ConfigService.config.storeNumber,
                    terminalNumber: ConfigService.config.terminalNumber
                };

                return $ionicLoading.show({
                    template : 'Saindo... Aguarde.',
                    duration : 0
                }).then(function() {
                    let cartService = $injector.get('CartService'); //Evitando referência circular.
                    var cartItens = cartService.cart.fiscalItems;

            		var proceed = () => {
						if (doLogoutRestRequest) {
							return $resource(service.url + '/signout').save({}, params).$promise.finally(() => {
								service.currentUser = {};
								localStorage.removeItem('id_token');
								UtilService.navigateWithoutHistory("login");
								$ionicLoading.hide();
							});
						}
						else {
							service.currentUser = {};
							localStorage.removeItem('id_token');
							UtilService.navigateWithoutHistory("login");
							$ionicLoading.hide();
							return false;
						}
					};

                    if (ConfigService.config.business.enable_reserve_products === true && cartItens.length > 0
                            &&  (cartItens = cartItens.filter(item => item.itemUuid && item.itemUuid.value)).length) {
                        const cancelParams = {
                            storeNumber : ConfigService.config.storeNumber,
                            orderNumber: cartService.sessionId,
                            terminalNumber: ConfigService.config.terminalNumber,
                            operatorName: service.currentUser.name,
                            products: cartItens.map(item => {
                                return {
                                    productCode: item.productCode,
                                    uuid: item.itemUuid.value,
                                    quantity: item.quantitySold
                                };
                            })
                        };
                        return ProductService.cancelSale(cancelParams).finally(() => {
                            return proceed();
                        });
                    } else {
                        return proceed();
                    }
                });
		    } else {
		        service.currentUser = {};
		        localStorage.removeItem('id_token');
		        UtilService.navigateWithoutHistory("login");

		        return $q.resolve();
		    }
		};

		service.logoutUser = function() {
			service.currentUser = {};
			localStorage.removeItem('id_token');
		}

		/**
		 * Validação on-line da autenticação do usuário.
		 */
		service.userAutheticationValidation = function() {
		    let params = {
		        username: service.currentUser.username,
	            storeNumber: ConfigService.config.storeNumber,
	            terminalNumber: ConfigService.config.terminalNumber,
                ipAddress: window.ionic.ipAddress
            };
		    return $resource(service.url + '/user-auth-validation').save({}, params).$promise;
		};

		service.isAuthenticated = function() {
			return (service.currentUser && service.currentUser.username);
		};


		service.checkPermission = function(permission) {
			if (!ConfigService.config.business.self_authorizer){
				return false;
			}
			return service.checkNotConstrainedPermission(permission);
		};

         /**
          *
          */
        service.checkNotConstrainedPermission = function(permission) {
            return service.currentUser && service.currentUser.permissions != null && service.currentUser.permissions.indexOf(permission) >= 0;
        };

        /**
         * Essa função faz apenas a autorização de uma permissão.
         *
         * Se você precisa de algo mais, como validação de autosserviço, CRIE OUTRA FUNÇÃO.
         */
        service.requestAuthorization = (permission, permissionDesc, permissionVerificationFn) => {
            permissionVerificationFn = permissionVerificationFn || service.checkNotConstrainedPermission;

            LogService.info(`Solicitando autorização para '${permission}' ${permissionDesc}`);

            if (!permissionVerificationFn(permission)) {
                return service.requestExternalAuthorization([{ id: permission, description: permissionDesc }]);
            } else {
                LogService.info(`Autorização concedida para usuário '${service.currentUser.username}'`);
                return $q.resolve(service.currentUser.username);
            }
        };

		function ignorePermissions(permission) {
            return false;
        };

		service.checkRequestAuthorization = (permission, permissionDesc) => {
			// para manter os métodos do legado, se permitir a auto-autorização, retorna a função q verifica as permissões. Caso contrário, retorna a q ignora.
			const permissionVerificationFn = (ConfigService.config.business.self_authorizer === true ? service.checkNotConstrainedPermission : ignorePermissions);
            return service.requestAuthorization(permission, permissionDesc, permissionVerificationFn);
        };

        service.requestExternalAuthorization = (permissions) => {
            const ret = $q.defer();
            const $scope = $rootScope.$new();

            // erro de dev
            if (permissions == null) {
                throw new Error('Permissões devem ser informadas');
            }
            // erro de dev
            if (permissions.some(p => p.id == null)) {
                throw new Error('Uma das permissões não possui ID');
            }

            // utilizados no modal
            $scope.requestauth = {};
            $scope.permissionDesc = permissions.map(p => p.description).toString();

            $ionicModal.fromTemplateUrl('views/requestauthmodal.html', {
                scope: $scope,
                focusFirstInput: true,
                animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.modal = modal;
                $scope.modal.show();
                $rootScope.$on("IdleTimeout", () => $scope.closeModal());
            });
            $scope.closeModal = function () {
                $scope.modal.remove().then(() => ret.reject());
            };
            $scope.$on('$destroy', function () {
                $scope.modal.remove();
            });
            $scope.requestAuth = function () {
                $timeout(() => {
                    var myEl = document.getElementById('button_requestauthmodalConfirm');
                    var angularEl = angular.element(myEl);
                    angularEl[0].focus();
                });
                $ionicLoading.show({
                    template: 'Autorizando. Por favor, aguarde...',
                    duration: 0
                }).then(() => {
                    if (!$scope.requestauth.user) {
                        $ionicLoading.hide()
                            .then(() => AlertService.error("Usuário inválido!"));
                    } else if (!$scope.requestauth.password) {
                        $ionicLoading.hide()
                            .then(() => AlertService.error("Senha inválida!"));
                    } else {
                        return checkAuthorizationInternal($scope.requestauth.user, $scope.requestauth.password, permissions.map(p => p.id))
                            .then(result => {
                                $ionicLoading.hide();

                                if (result.authorized == true) {
                                    LogService.info(`Autorização concedida para usuário '${$scope.requestauth.user}'`);
                                    $scope.modal.remove().then(() => ret.resolve($scope.requestauth.user));
                                } else {
                                    LogService.info(`Autorização begada para usuário '${$scope.requestauth.user}'`);
                                    AlertService.warn('Autorização negada!')
                                }
                            })
                            .catch(error => {
                                $ionicLoading.hide()
                                    .then(() => LogService.error("Erro ao consultar autorização!", error))
                                    .then(() => AlertService.restError(error, "Erro ao consultar autorização!"));
                            });
                    }
                });
            };

            return ret.promise;
        }

		/**
		 *
		 * @param {*} user
		 * @param {*} passwd
		 * @param {*} permissions
		 * @param {*} success
		 * @param {*} error
		 *
		 */
		service.checkAuthorization = function(user, passwd, permissions, success, error) {
			if (!ConfigService.config.business.self_authorizer && service.currentUser.username === user){
				AlertService.error("Proibido auto serviço!");
				return $q.reject();
			}
			service.loadAuthorizationSettings();
			var formData = {username : user, password : passwd, requiredPermissions : permissions };
			return $resource(service.authurl, {}, { create: { method: 'POST' } }).create({}, formData, success, error).$promise;
		};

		function checkAuthorizationInternal(user, passwd, permissions, success, error) {
			if (!ConfigService.config.business.self_authorizer && service.currentUser.username === user){
				return $q.reject({ message: "Proibido auto serviço!" });
			}
			service.loadAuthorizationSettings();
			var formData = {username : user, password : passwd, requiredPermissions : permissions };
			return $resource(service.authurl, {}, { create: { method: 'POST' } }).create({}, formData, success, error).$promise;
		};

		service.checkSelfService = function(username) {
			if (!ConfigService.config.business.self_authorizer && service.currentUser.username === username){
				return $q.reject("Proibido auto serviço!");
			} else {
				return $q.resolve(username);
			}
		};
		/**
		 *
		 */
		service.operatorCode = () => {
			return !service.currentUser.id || !service.currentUser.id.length ? service.currentUser.username : service.currentUser.id;
		};

		/**
		 *
		 */
		service.operatorName = () => {
		    return service.currentUser.name;
		};

		service.searchSeller = function(params, success, error) {
		    var url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
            url += '/linx-webservices/v1/seller/show';

		    return $resource(url).get(params, success, error);
		};

		service.soxToast = function(success, error) {
		    var url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
            url += '/linx-webservices/v1/sox-info/login-messages';

		    return $resource(url).get(success, error);
		};

		service.getBalanceOperator = function(params, success, error) {
			var url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
			url += '/linx-webservices/v1/financial/balance/operator';
			return $resource(url, {}, { create: { method: 'POST' } }).create(params, success, error);
		};

		service.getBalanceStore = function(params, success, error) {
			var url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
			url += '/linx-webservices/v1/financial/balance/store';
			return $resource(url, {}, { create: { method: 'POST' } }).create(params, success, error);
		};

		service.getCashInFlow = function(params, success, error) {
			var url  = ConfigService.config.linxwebservices.secure ? 'https://' : 'http://';
            url += ConfigService.config.linxwebservices.host + ':';
            url += ConfigService.config.linxwebservices.port;
			url += '/linx-webservices/v1/financial/cash/float-entry';
			return $resource(url, {}, { create: { method: 'POST' } }).create(params, success, error);
		};

		service.listUsers = function(companyNumber, storeNumber, success, error) {
			service.loadSettings();
			return $resource(service.url + '/v1/users/list', {}, {
                listUsers: {
                    method: 'POST'
                }
            }).listUsers({
                companyNumber: companyNumber,
                storeNumber: storeNumber
            }).$promise;
		};
		return service;
	})
})();
