(function() {
	'use strict';

	angular.module('easyorder.services.util', [])

	.factory('UtilService', function($state, $stateParams, $ionicHistory, $ionicNavBarDelegate, $injector) {
		return {
		    previousStateData: {},
			navigateWithoutHistory : function(state, param) {
				$ionicHistory.nextViewOptions({
					disableBack : true
				})
				$ionicNavBarDelegate.showBackButton(false);

				if (!$state.is(state)) {
				    this.previousStateData.name = $state.$current.name;
				    this.previousStateData.params =  $stateParams;
				    				    
				    if (!param) {
				        return $state.go(state);
				    } else {
				        return $state.go(state, param);
				    }
				}
			},
            navigateToMainScreen: function(param) {
                var ConfigService = $injector.get('ConfigService');
                var UtilService = $injector.get('UtilService');
                
                if (ConfigService.config.business.default_home_screen === "CustomerIdentification") { 
                    if (ConfigService.config.business.unified_customer_identification_enabled) {
                        return UtilService.navigateWithoutHistory('easyorder.customer', param);
                    } else {
                        return UtilService.navigateWithoutHistory('easyorder.customermenu', param);
                    }
                } else {
                    return UtilService.navigateToMainSearchScreen(param);
                }
            },
			navigateToMainSearchScreen : function(param) {
				var ConfigService =  $injector.get('ConfigService');
                var UtilService =  $injector.get('UtilService');
				
                if (window.cordova) {
                    if (ConfigService.config.business.enables_rfid_reading) {
                        return UtilService.navigateWithoutHistory('easyorder.rfid-product-search', param);  
    				} else if (window.cordova && ConfigService.config.business.default_product_search_operation === "Barcode") {
                        return UtilService.navigateWithoutHistory('easyorder.barcode', param);
                    } else if (ConfigService.config.business.default_product_search_operation === "Typed") {
                        return UtilService.navigateWithoutHistory('easyorder.productsearch', param);
                    } else {
                        return UtilService.navigateWithoutHistory('easyorder.productsearch', param);
                    }
                } else {
                    return UtilService.navigateWithoutHistory('easyorder.productsearch');
                }
            },
            navigateAfterReceivedPayment: function() {
                var ConfigService =  $injector.get('ConfigService');
                var CartService =  $injector.get('CartService');
                var PaymentService =  $injector.get('PaymentService');
                var UtilService =  $injector.get('UtilService');
                var state = null;
                
                CartService.updateCartTotalValue(); //Atualizando total da dívida se planos de pagamentos com desconto.
                
                if (new Number(CartService.cart.totalValue) > new Number(PaymentService.totalPayments())) { // Há resto a pagar?
                    if (CartService.hasUnprocessedPredefinedPaymentMethod()) {
                        state = 'easyorder.predefinedpaymentmethods';
                    } else {
                        state = 'easyorder.paymentmethods';
                    }
                } else {
                    const showRFIDDisableView = ConfigService.config.business.enables_rfid_reading 
                        && CartService.getCurrentOperation() !== 'SERVICE'
                        && CartService.cart.fiscalItems.find(fis => !!fis.epcTag);
                    
                    if (showRFIDDisableView) {
                        state = 'easyorder.rfid-disable-tag';
                    } else {
                        state = 'easyorder.paymentresult';
                    }
                }
                return UtilService.navigateWithoutHistory(state);
            },
            navigate: (state, params) => {
                $ionicNavBarDelegate.showBackButton(true);
                $ionicHistory.nextViewOptions({
                    disableBack : false
                });
                if (params == null) {
                    return $state.go(state);   
                }
                return $state.go(state, params);
            },
            wrapCancelableResponseErrorHandler: function(errorhandler) { // Encapsula o tratamento de erro passado em uma verificação de cancelamento da requisição http.
                return function(response) {
                    if (!response.config.responseCanceled) {
                        errorhandler(response);
                    }
                };
            },
            isInteger: function(value) {
                const UtilService = $injector.get('UtilService');
            	return !UtilService.isNullOrEmpty(value) && /^[0-9]*$/.test(value);
            },
            isNumeric: function (str) {
                if (typeof str != "string") {
                    return false;
                }
                return !isNaN(str) && !isNaN(parseFloat(str));
            },
            removeEmptyAttributes: function(obj) {
            	  let newObj = {};
            	  Object.keys(obj).forEach((prop) => {
            	    if (obj[prop]) { newObj[prop] = obj[prop]; }
            	  });
            	  return newObj;
        	},
        	isNullOrEmpty: function(value){
				return (value == undefined || !value  || value == "" || value.length == 0);
			},
        	compareTo: (func1, func2) => {
        	    const value1 = func1();
        	    const value2 = func2();
        	    
        	    if (value1 > value2) {
        	        return 1; 
        	    } else if (value1 < value2) {
        	        return -1;
        	    }
        	    return 0;
			},
			uuidv4: function() {
				return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
				  var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
				  return v.toString(16);
				});
            },
            cloneDeep: obj => _.cloneDeep(obj), //Lodash
            jsonToUriParam: function(json) {
                let queryString = ""
                if (json) {
                    let params = new URLSearchParams();
                    for (let key in json) {
                        if (json[key]) {
                            params.set(key, json[key])
                        }
                    }
                    queryString = params.toString();
                }
                return queryString;
            },
            createRequestKey: (url) => {
                const UtilService = $injector.get('UtilService');
                const finalUrl = UtilService.getFinalUrl(url).toLowerCase();
                const secret = "$%pf#&RTw7SLYp$!";
                return CryptoJS.HmacSHA512(finalUrl, secret).toString();
            },
            createMessageKey: (obj, isForm) => {
                const UtilService = $injector.get('UtilService');
                const secret = "$%pf#&RTw7SLYp$!";
                const content = isForm ? UtilService.encodeObjectURI(obj) : angular.toJson(obj);
                return CryptoJS.HmacSHA512(content, secret).toString();
            },
            convertSslPluginErrorResponse: (response) => {
                // const LogService = $injector.get('LogService');
                // LogService.error(response);
                let errorMessage = null;
                let errorCode = null;
                if (response.error) {
                    let error = null;
                    try {
                        error = JSON.parse(response.error);
                    } catch (e) { }
                    
                    if (error && error.message) {
                        errorMessage = error.message;
                        errorCode = error.code;
                    }
                }
                if (errorMessage == null) {
                    // pelo plugin, os erros são:
                    // -1=Genérico, -2=SSL, -3=Host desconhecido, -4=Timeout
                    errorMessage = response.status === -4 ? 'Limite do tempo de espera excedido.' : 'Erro de comunicação com o Servidor.';
                }

                const error = {
                    data: {
                        code: errorCode,
                        message: errorMessage
                    }
                }
                return error;
            },
            encodeObjectURI: (obj) => {
                let str = [];
                const sortedObj = Object.keys(obj).sort().reduce((r, k) => (r[k] = obj[k], r), {})
                for (let p in sortedObj)
                    str.push(encodeURIComponent(p) + "=" + encodeURIComponent(sortedObj[p]));
                return str.join("&");
            },
            getFinalUrl: (rawUrl) => {
                let url = new URL(rawUrl);
                if (url.port === 443) {
                    url.port = '';
                }
                return url.toString();
            },
            isNotEmptyArray: (array) => {
                return Array.isArray(array) && array.length > 0;
            },
            cloneObject: (obj) => {
                return obj == null ? null : JSON.parse(JSON.stringify(obj));
            },
            /*
            * Each second digits are multiplied by two. These numbers seperate into single digits. 
            * All digits are summed. The result is rounded up to the nearest multiple of ten. 
            * If the difference between the result and rounded-up number is equal to last digit, 
            * imei number is valid.
            */
            checkImeiNumber: function (imeiNumber) {
                let result = false;
                let numbers = [];
                let length = 0;
                let newNumber = 0;
                let sum = 0;
                let nearestMultipleOfTen = 0;
                let difference = 0;
                while (imeiNumber) {
                    numbers.push(imeiNumber % 10);
                    imeiNumber = Math.floor(imeiNumber / 10);
                }
                numbers.reverse();
                length = numbers.length;
                if (length != 15)
                    return result;
                for (let i = 0; i < 15; i++) {
                    if (i % 2 != 0) {
                        switch (numbers[i]) {
                            case 0:
                                // 0 * 2 = 0
                                newNumber = 0;
                                break;
                            case 1:
                                // 1 * 2 = 2
                                newNumber = 2;
                                break;
                            case 2:
                                // 2 * 2 = 4
                                newNumber = 4;
                                break;
                            case 3:
                                // 3 * 2 = 6
                                newNumber = 6;
                                break;
                            case 4:
                                // 4 * 2 = 8
                                newNumber = 8;
                                break;
                            case 5:
                                // 5 * 2 = 10
                                // 1 + 0 = 1
                                newNumber = 1;
                                break;
                            case 6:
                                // 6 * 2 = 12
                                // 1 + 2 = 3
                                newNumber = 3;
                                break;
                            case 7:
                                // 7 * 2 = 14
                                // 1 + 4 = 5
                                newNumber = 5;
                                break;
                            case 8:
                                // 8 * 2 = 16
                                // 1 + 6 = 7
                                newNumber = 7;
                                break;
                            case 9:
                                // 9 * 2 = 18
                                // 1 + 8 = 9
                                newNumber = 9;
                                break;
                        }
                        numbers.splice(i, 1, newNumber);
                    }
                }
                for (let i = 0; i < 14; i++) {
                    sum += numbers[i];
                }
                nearestMultipleOfTen = Math.ceil(sum / 10) * 10;
                difference = nearestMultipleOfTen - sum;
                if (difference == numbers[14])
                    result = true;
                return result;
            },

            /*
            * Iccid validation is based on Luhn algorithm:
            * 1) If the number already contains the check digit, drop that digit to form the "payload." The check digit 
            *    is most often the last digit.
            * 2) With the payload, start from the rightmost digit. Moving left, double the value of every second digit 
            *    (including the rightmost digit).
            * 3) Sum the digits of the resulting value in each position (using the original value where a digit did not 
            *    get doubled in the previous step).
            * 4) The check digit is calculated by 10-(s mod 10).
            */
            checkIccidNumber: function (iccidNumber) {
                if (iccidNumber.length != iccidNumber.replace(/\D/g, "").length){
                    return false
                }
                let originalCheckDigit = iccidNumber.charAt(iccidNumber.length-1);
                let calculatedCheckDigit = 0;
                let nDigit = 0;
                let bEven = true;
                
                for (let n = iccidNumber.length - 2; n >= 0; n--) {                     
                    nDigit = parseInt(iccidNumber.charAt(n), 10);
                    if (bEven) {
                        if ((nDigit *= 2) > 9) nDigit -= 9;
                    }
                    calculatedCheckDigit += nDigit;
                    bEven = !bEven;
                }
                calculatedCheckDigit = 10 - (calculatedCheckDigit % 10);
                return calculatedCheckDigit == originalCheckDigit;
            }
		}
	});
})();
