(function () {
    'use strict';

    angular.module('easyorder.services.voucherservice', [
        'ngResource',
        'easyorder.services.config'
    ]).factory('VoucherService', function ($resource, $q,
                                           ConfigService) {

        const service = {};

        const serviceURL = () => {
            const linxwebservices = ConfigService.config.linxwebservices;

            return (linxwebservices.secure ? 'https://' : 'http://')
                + linxwebservices.host + ':' + linxwebservices.port + '/linx-webservices/v1/voucher';
        };

        service.create = (nsu, voucherNumber, companyNumber, storeNumber, terminalNumber, value, type) => {
            return $resource(serviceURL() + '/create', {}, {
                create: {
                    method: 'POST'
                }
            }).create({
                nsu: nsu,
                voucherNumber: voucherNumber,
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                value: value,
                type: type,
            }).$promise;
        };

        service.search = (voucherNumber, storeNumber, type, companyNumber, terminalNumber, nsu, password) => {
            return $resource(serviceURL() + '/search', {}, {
                search: {
                    method: 'POST'
                }
            }).search({
                voucherNumber: voucherNumber,
                storeNumber: storeNumber,
                type: type,
                companyNumber: companyNumber,
                terminal: terminalNumber,
                nsu: nsu,
                password: password
            }).$promise;
        };

        service.balance = (nsu, voucherNumber, storeNumber, type, companyNumber, terminalNumber) => {
            return $resource(serviceURL() + '/search-balance', {}, {
                balance: {
                    method: 'POST'
                }
            }).balance({
                nsu: nsu,
                voucherNumber: voucherNumber,
                storeNumber: storeNumber,
                type: type,
                companyNumber: companyNumber,
                terminal: terminalNumber
            }).$promise;
        };

        service.searchByCustomer = function (customer, storeNumber,companyNumber, terminalNumber, nsu, type, success, error) {
            var params = {
                customerDocument: {
                    value: customer.document,
                    type: customer.documentType
                },
                type: type,
                storeNumber: storeNumber,
                companyNumber: companyNumber,
                terminal: terminalNumber,
                nsu: nsu
            };
            return $resource(serviceURL() + '/search', {}, {
                'process': {
                    method: 'POST'
                }
            }).process({}, params, success, error).$promise;
        };

        service.searchByDocument = (companyNumber, storeNumber, terminalNumber, type, document, documentType) => {
            return $resource(serviceURL() + '/search', {}, {
                'search': {
                    method: 'POST'
                }
            }).search({
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminal: terminalNumber,
                type: type,
                customerDocument: {
                    value: document,
                    type: documentType
                }
            }).$promise;
        };

        service.authorize = (
            nsu,
            voucherNumber,
            storeNumber,
            type,
            value,
            companyNumber,
            terminalNumber,
            contextUid,
            password,
            operator,
            authorizer,
            document,
            documentType,
            mac,
            date, // no caso do voucher storex, será a data da transação que integramos no banco
            undoNsu
        ) => {
            return $resource(serviceURL() + '/authorize', {}, {
                authorize: {
                    method: 'POST'
                }
            }).authorize({
                nsu: nsu,
                voucherNumber: voucherNumber,
                storeNumber: storeNumber,
                type: type,
                companyNumber: companyNumber,
                terminalNumber: terminalNumber,
                value: value,
                contextUid: contextUid,
                password: password,
                operator: operator,
                authorizer: authorizer,
                document: document,
                documentType: documentType,
                mac: mac,
                date: date,
                undoNsu: undoNsu
            }).$promise;
        };


        service.authorizeBySerialNumber = (
            companyNumber,
            storeNumber,
            terminalNumber,
            type,
            cardISO,
            serialNumber,
            value,
            nsu,
            document,
            documentType,
            contextUid,
            date
        ) => {
            return $resource(serviceURL() + '/authorize/serial-number', {}, {
                authorize: {
                    method: 'POST'
                }
            }).authorize({
                companyNumber: companyNumber,
                storeNumber: storeNumber,
                terminalNumber: terminalNumber,
                type: type,
                cardISO: cardISO,
                serialNumber: serialNumber,
                value: value,
                nsu: nsu,
                customerDocument: {
                    value: document,
                    type: documentType
                },
                contextUid: contextUid,
                date: date
            }).$promise;
        };

        service.confirm = (
            nsu,
            voucherNumber,
            storeNumber,
            type,
            contextUid,
            value,
            terminalNumber,
            companyNumber,
            date // no caso do voucher storex, será a data da transação que integramos no banco
        ) => {
            return $resource(serviceURL() + '/confirm', {}, {
                confirm: {
                    method: 'POST'
                }
            }).confirm({
                nsu: nsu,
                voucherNumber: voucherNumber,
                storeNumber: storeNumber,
                type: type,
                contextUid: contextUid,
                value: value,
                terminalNumber: terminalNumber,
                companyNumber: companyNumber,
                date: date
            }).$promise;
        };

        service.cancel = (
            nsu,
            voucherNumber,
            storeNumber,
            type,
            companyNumber,
            terminalNumber,
            authorizationCode,
            value,
            contextUid,
            date
        ) => {
            return $resource(serviceURL() + '/cancel', {}, {
                cancel: {
                    method: 'POST'
                }
            }).cancel({
                nsu: nsu,
                voucherNumber: voucherNumber,
                storeNumber: storeNumber,
                type: type,
                companyNumber: companyNumber,
                terminalNumber: terminalNumber,
                authorizationCode: authorizationCode,
                value: value,
                contextUid: contextUid,
                date // no caso do voucher storex, será a data da transação que integramos no banco
            }).$promise;
        };

        service.checkVoucherUsed = (type, voucherNumber, currentSalePayments) => {
            var defer = $q.defer();
            if (currentSalePayments && currentSalePayments.length) {
                const hasVoucher = currentSalePayments.filter(p => p.paymentCodeMethod === type && p.voucherNumber === voucherNumber);
                if (hasVoucher && hasVoucher.length) {
                    defer.reject(new VoucherAlreadyUsedError('Voucher ' + voucherNumber + ' já utilizado!'));
                } else {
                    defer.resolve(voucherNumber);
                }
            } else {
                defer.resolve(voucherNumber);
            }

            return defer.promise;
        };

        service.isSameVoucher = (cardISO, serialNumber, voucherNumber) => {
            // o ISO são os 8 primeiros e o serial os 9 últimos (desconsiderando o último, digito verificador)
            return cardISO && serialNumber && voucherNumber && voucherNumber.length > 20 && voucherNumber.substring(0,8) === cardISO && voucherNumber.substring(voucherNumber.length-10,voucherNumber.length-1) === serialNumber;
        };

        return service;
    });
})();


class VoucherAlreadyUsedError extends Error {
    constructor(message) {
        super(message);
        this.name = 'VoucherAlreadyUsedError';
    }
}

class VoucherNotFoundError extends Error {
    constructor(message) {
        super(message);
        this.name = 'VoucherNotFoundError';
    }
}

class VoucherOperationAbortedError extends Error {
    constructor(message) {
        super(message);
        this.name = 'VoucherOperationAbortedError';
    }
}
