/**
 * Adiciona à tipagem numérica uma função de trunc
 * @param {Quantidade de dígitos a serem truncados} scale 
 * @returns Valor truncado
 */
 Number.prototype.toTruncFixed = function(scale) {
    const re = new RegExp("(\\d+\\.\\d{" + scale + "})(\\d)"),
        m = this.toString().match(re);
    return m ? parseFloat(m[1]) : this.valueOf();
};

class MathUtils {
    static subtract(val1, val2, scale) {
        val1 = parseFloat(val1 || 0);
        val2 = parseFloat(val2 || 0);
        scale = parseInt(scale || 2);
        return this.round(val1 - val2, scale);
    }

    static add(val1, val2, scale) {
        val1 = parseFloat(val1 || 0);
        val2 = parseFloat(val2 || 0);
        scale = parseInt(scale || 2);
        return this.round(val1 + val2, scale);
    }

    static multiply(val1, val2, scale) {
        val1 = parseFloat(val1 || 0);
        val2 = parseFloat(val2 || 0);
        scale = parseInt(scale || 2);
        return this.round(val1 * val2, scale);
    }

    static divide(val1, val2, scale) {
        val1 = parseFloat(val1 || 0);
        val2 = parseFloat(val2 || 0);
        scale = parseInt(scale || 2);
        return this.round(val1 / val2, scale);
    }

    /**
     * Arredonda valores com a quantidade de casas decimais informadas na escala (Padrão 2 casas)
     * @param {Valor a ser arredondado} val 
     * @param {Escala de arredondamento} scale 
     * @returns Valor arredondado
     */
    static round(val, scale) {
        val = parseFloat(val || 0);
        scale = parseFloat(scale || 2);
        return parseFloat(val.toFixed(scale));
    }

    /**
     * Realiza um corte de casas decimais baseado na quantidade de dígitos informados por parâmetro (Padrão 2 casas)
     * @param {Valor a ser truncado} val 
     * @param {Quantidade de casas decimais aceitas} scale 
     * @returns Valor truncado
     */
    static trunc(val, scale) {
        val = parseFloat(val || 0);
        scale = parseFloat(scale || 2);
        return val.toTruncFixed(scale);
    }
}