class PartnerProductCode {	
	constructor(str) {
		this.SEPARATOR = "$";
		this.PARTNER_1 = "1";
		this.PARTNER_2 = "2";
		this.barCode = str;

        if (str == null) {
            return;
        }

        if (str.indexOf(this.SEPARATOR) < 0) {
            return;
        }

        var arr = str.split(this.SEPARATOR);
        this.productCode = arr[0];
        this.partnerIndex = arr[1][0];
        this.checkDigits = arr[1][1] + arr[1][2];
	}
	
	calculateCheckDigits(productCode, partnerIndex) {
        var checkDigits = "";
        var item = productCode + partnerIndex;
        var x = item.length - 1;
        var y = 25;
        var ret = 0;
        while (x >= 0) {
            if (x == item.length - 1) {
                ret = ret + (Number(item.substring(x)) * y);
            } else {
                ret = ret + (Number(item.substring(x, x + 1)) * y);
            }

            y = y - 1;

            if (y == 1) {
                y = 25;
            }

            x = x - 1;
        }

        var digit1 = ret % 11;

        if (digit1 == 0 || digit1 >= 10) {
            digit1 = 0;
        }

        x = item.length - 1;
        y = 2;
        var max_y = 25;
        ret = 0;
        while (x >= 0) {
            if (x == item.length - 1) {
                ret = ret + (Number(item.substring(x)) * y);
            } else {
                ret = ret + (Number(item.substring(x, x + 1)) * y);
            }

            y = y + 1;

            if (y == max_y) {
                y = 2;
            }

            x = x - 1;
        }

        var digit2 = ret % 11;

        if (digit2 == 0 || digit2 >= 10) {
            digit2 = 0;
        }

        return digit1.toString() + digit2.toString();
    }
	
	generateParnterCodes(barCode) {
        return ret = [
            barCode + "$1" + this.calculateCheckDigits(barCode, "1"),
            barCode + "$2" + this.calculateCheckDigits(barCode, "2")
        ];
    }

    toString() {
        return this.productCode + "$" + this.partnerIndex + this.checkDigits; 
    }


    getReversePartnerIndex() {
		if (this.partnerIndex == this.PARTNER_2) {
			return this.PARTNER_1;
		} else {
			return this.PARTNER_2;
		}
	}

    getReversePartnerCode() {
        return this.productCode
                + this.SEPARATOR
                + this.getReversePartnerIndex()
                + this.calculateCheckDigits(this.productCode, this.getReversePartnerIndex());
    }

    getPartnerIndex() {
        var partnerProductCode = this.barCode;
        var separatorIndex = partnerProductCode.indexOf(this.SEPARATOR);
        return partnerProductCode.substring(separatorIndex + 1, separatorIndex + 2);
    }

    isValid() {
        var partnerProductCode = this.barCode;
        if (partnerProductCode == null) {
			return false;
		}

        var separatorIndex = partnerProductCode.indexOf(this.SEPARATOR);
        if (separatorIndex < 0) {
            return false;
        }

        var productCode = partnerProductCode.substring(0, separatorIndex);
        if (productCode.length < 1) {
            return false;
        }

        if (partnerProductCode.substring(separatorIndex + 1).length != 3) {
            return false;
        }

        var partnerIndex = partnerProductCode.substring(separatorIndex + 1, separatorIndex + 2);
        if (partnerIndex != this.PARTNER_1 && partnerIndex != this.PARTNER_2) {
            return false;
        }

        var checkDigits = partnerProductCode.substring(separatorIndex + 2);
        if (checkDigits != this.calculateCheckDigits(productCode, partnerIndex)) {
            return false;
        }

        return true;
    }
}