class ValidateStateRegistration {

    // Validador de Inscrição Estadual
    static isValid(code, state) {

        if (code === null) return false;

        // Remove caracteres numéricos
        const alpha = code.replace(/[0-9]/g, '');
        if (alpha.length > 0) {
            return false;
        }

        // Remove caracteres não numéricos
        code = code.replace(/[^0-9]/g, '');

        if (code === '' || parseInt(code) === 0) {
            return false;
        }

        switch (state) {
            case 'AC':
                return this.isValidAC(code);
            case 'AL':
                return this.isValidAL(code);
            case 'AM':
                return this.isValidAM(code);
            case 'AP':
                return this.isValidAP(code);
            case 'BA':
                return this.isValidBA(code);
            case 'CE':
                return this.isValidCE(code);
            case 'DF':
                return this.isValidDF(code);
            case 'ES':
                return this.isValidES(code);
            case 'GO':
                return this.isValidGO(code);
            case 'MA':
                return this.isValidMA(code);
            case 'MG':
                return this.isValidMG(code);
            case 'MS':
                return this.isValidMS(code);
            case 'MT':
                return this.isValidMT(code);
            case 'PA':
                return this.isValidPA(code);
            case 'PB':
                return this.isValidPB(code);
            case 'PE':
                return this.isValidPE(code);
            case 'PI':
                return this.isValidPI(code);
            case 'PR':
                return this.isValidPR(code);
            case 'RJ':
                return this.isValidRJ(code);
            case 'RN':
                return this.isValidRN(code);
            case 'RO':
                return this.isValidRO(code);
            case 'RR':
                return this.isValidRR(code);
            case 'RS':
                return this.isValidRS(code);
            case 'SC':
                return this.isValidSC(code);
            case 'SE':
                return this.isValidSE(code);
            case 'SP':
                return this.isValidSP(code);
            case 'TO':
                return this.isValidTO(code);
            default:
                return false;
        }

    }

    static weightedSum(base, startW, endW) {
        let auxW = startW;

        let sum = 0;
        for (let i = base.length - 1; i >= 0; i--) {
            sum += (base.charAt(i) - '0') * auxW;

            if (auxW < endW) {
                auxW++;
            } else {
                auxW = startW;
            }
        }

        return sum;
    }

    // Calcula a Inscrição Estadual do estado: Acre
    static isValidAC(code) {
        if (code.length > 13) {
            return false;
        }

        const completeCode = code.padStart(13, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-2));
        const base = completeCode.substring(0, completeCode.length-2);

        // 1st digit
        let sum = 0;
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        let rest = sum % 11;

        let digit1 = 11 - rest;
        if (digit1 === 10 || digit1 === 11) {
            digit1 = 0;
        }

        // 2nd digit
        sum = ValidateStateRegistration.weightedSum(base + digit1,2,9);
        rest = sum % 11;

        let digit2 = 11 - rest;
        if (digit2 === 10 || digit2 === 11) {
            digit2 = 0;
        }

        let calculatedDigit = digit1.toString() + digit2.toString();

        if (parseInt(calculatedDigit) !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Alagoas
    static isValidAL(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        if (completeCode.substring(0, 2) !== '24') {
            return false;
        }

        // 1st digit
        let sum = 0;
        sum = ValidateStateRegistration.weightedSum(base,2,9) * 10;
        let rest = sum % 11;

        let calculatedDigit = rest;
        if (calculatedDigit === 10) {
            calculatedDigit = 0;
        }


        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Amazonas
    static isValidAM(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        // 1st digit
        let sum = 0;
        let calculatedDigit = 0;

        sum = ValidateStateRegistration.weightedSum(base,2,9);
        let rest = sum % 11;

        if ( sum < 11 ) {
            calculatedDigit = 11 - rest;
        } else if ( rest <= 1 ) {
            calculatedDigit = 0;
        } else {
            calculatedDigit = 11 - rest;
        }


        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Amapá
    static isValidAP(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        if (completeCode.substring(0, 2) !== '03') {
            return false;
        }

        let number = parseInt(base);

        let p = 0;
        let d = 0;

        if (number > 3000000 && number < 3017001) {
            p = 5;
            d = 0;
        } else if (number > 3017000 && number < 3012023) {
            p = 9;
            d = 1;
        } else if (number > 3017002) {
            p = 0;
            d = 0;
        }

        // 1st digit
        let sum = 0;
        sum = ValidateStateRegistration.weightedSum(base,2,9) + p;
        let rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (calculatedDigit === 10) {
            calculatedDigit = 0;
        } else if (calculatedDigit === 11) {
            calculatedDigit = d;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Bahia
    static isValidBA(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-2));
        const base = completeCode.substring(0, completeCode.length-2);

        let d2NumSrc = parseInt(base.substring(1, 2));
        let sum = 0;
        let digit1 = 0;
        let digit2 = 0;
        let rest = 0;

        if (d2NumSrc < 6 || d2NumSrc === 8) {
            sum = ValidateStateRegistration.weightedSum(base,2,8);
            rest = sum % 10;

            // 2nd digit
            digit2 = 10 - rest;
            if (digit2 === 10) {
                digit2 = 0;
            }

            // 1st digit
            sum = ValidateStateRegistration.weightedSum(base + digit2,2,9);
            rest = sum % 10;

            digit1 = 10 - rest;
            if (digit1 === 10) {
                digit1 = 0;
            }
        } else {
            // 2nd digit
            sum = ValidateStateRegistration.weightedSum(base,2,8);
            rest = sum % 11;

            digit2 = 11 - rest;
            if (digit2 >= 10) {
                digit2 = 0;
            }

            // 1st digit
            sum = ValidateStateRegistration.weightedSum(base + digit2,2,9);
            rest = sum % 11;

            digit1 = 11 - rest;
            if (digit1 >= 10) {
                digit1 = 0;
            }
        }

        let calculatedDigit = digit1.toString() + digit2.toString();

        if (parseInt(calculatedDigit) !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Ceará
    static isValidCE(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        // 1st digit
        let sum = 0;
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        let rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if ( rest === 0 ) {
            calculatedDigit = 0;
        } else if ( rest === 1 ) {
            if ( parseInt(base) > 10103104 && parseInt(base) < 10119998 ) {
                calculatedDigit = 1;
            } else {
                calculatedDigit = 0;
            }
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Distrito Federal
    static isValidDF(code) {
        if (code.length > 13) {
            return false;
        }

        const completeCode = code.padStart(13, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-2));
        const base = completeCode.substring(0, completeCode.length-2);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let digit1 = 11 - rest;
        if (digit1 === 10 || digit1 === 11) {
            digit1 = 0;
        }

        // 2nd digit
        sum = ValidateStateRegistration.weightedSum(base + digit1,2,9);
        rest = sum % 11;

        let digit2 = 11 - rest;
        if (digit2 === 10 || digit2 === 11) {
            digit2 = 0;
        }

        let calculatedDigit = digit1.toString() + digit2.toString();

        if (parseInt(calculatedDigit) !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Espírito Santo
    static isValidES(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Goiás
    static isValidGO(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0) {
            calculatedDigit = 0;
        } else if (rest === 1) {
            if ( parseInt(base) > 10103104 && parseInt(base) < 10119998 ) {
                calculatedDigit = 1;
            } else {
                calculatedDigit = 0;
            }
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Maranhão
    static isValidMA(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Mato Grosso
    static isValidMT(code) {
        if (code.length > 11) {
            return false;
        }

        const completeCode = code.padStart(11, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Mato Grosso do Sul
    static isValidMS(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        if (completeCode.substring(0, 2) !== '28') {
            return false;
        }

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Minas Gerais
    static isValidMG(code) {
        if (code.length > 13) {
            return false;
        }

        const completeCode = code.padStart(13, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-2));
        const base = completeCode.substring(0, completeCode.length-2);

        // 1st digit
        const baseD1 = base.substring(0, 3).concat('0').concat(base.substring(3, base.length));

        let seqD1 = '';
        for (let i = 1; i <= baseD1.length; i++) {
            if (i % 2 === 1) {
                seqD1 = seqD1.concat(baseD1.substring(i-1,i));
            } else {
                seqD1 = seqD1.concat((parseInt(baseD1.substring(i,i-1))*2).toString());
            }
        }

        let sumD1 = 0;
        for (let i = 0; i < seqD1.length; i++) {
            sumD1 = sumD1 + parseInt(seqD1.substring(i,i+1));
        }

        const str = sumD1.toString().padStart(2, '0');

        let decimal = 0;
        if (sumD1 % 10 !== 0) {
            decimal = parseInt((parseInt(str.substring(0,1))+1).toString().concat('0'));
        } else {
            decimal = sumD1;
        }

        let d1 = decimal - sumD1;

        // 2nd digit
        const baseD2 = base + d1.toString();

        let aux = 2;
        let sumD2 = 0;
        for (let i = baseD2.length-1; i >= 0; i--) {
            sumD2 = sumD2 + (parseInt(baseD2.substring(i,i+1))*aux);
            if ( aux === 11 ) {
                aux = 2;
            } else {
                aux++;
            }
        }

        sumD2 = sumD2 % 11;
        let d2 = 0;

        if (sumD2 === 0 || sumD2 === 1) {
            d2 = 0;
        } else {
            d2 = 11 - sumD2;
        }

        const calculatedDigit = d1.toString().concat(d2.toString());

        if (parseInt(calculatedDigit) !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Pará
    static isValidPA(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        if (completeCode.substring(0, 2) !== '15') {
            return false;
        }

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Paraíba
    static isValidPB(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Paraná
    static isValidPR(code) {
        if (code.length > 10) {
            return false;
        }

        const completeCode = code.padStart(10, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-2));
        const base = completeCode.substring(0, completeCode.length-2);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,7);
        rest = sum % 11;

        let digit1 = 11 - rest;
        if (digit1 === 10 || digit1 === 11) {
            digit1 = 0;
        }

        // 2nd digit
        sum = ValidateStateRegistration.weightedSum(base + digit1,2,7);
        rest = sum % 11;

        let digit2 = 11 - rest;
        if (digit2 === 10 || digit2 === 11) {
            digit2 = 0;
        }

        let calculatedDigit = digit1.toString() + digit2.toString();

        if (parseInt(calculatedDigit) !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Pernambuco
    static isValidPE(code) {
        if (code.length > 14) {
            return false;
        }

        let completeCode = '';
        if ( code.length >= 10 ) {
            completeCode = code.padStart(14, '0');
        } else {
            completeCode = code;
        }

        let digit = '';
        let base = '';

        if ( completeCode.length === 14 ) {
            digit = parseInt(completeCode.substring(completeCode.length-1));
            base = completeCode.substring(0, completeCode.length-1);

            let sum = 0;
            let rest = 0;
            let auxW = 2;

            for (let i = base.length - 1; i >= 0; i--) {
                sum += (base.charAt(i) - '0') * auxW;
                if ( auxW < 9 ) {
                    auxW++;
                } else {
                    auxW = 1;
                }
            }

            rest = sum % 11;
            let calculatedDigit = 11 - rest;

            if (calculatedDigit > 9) {
                calculatedDigit -= 10;
            }

            if (calculatedDigit !== digit) {
                return false;
            }

        } else if ( completeCode.length >= 2 && completeCode.length <= 9 ) {
            digit = parseInt(completeCode.substring(completeCode.length-2));
            base = completeCode.substring(0, completeCode.length-2);

            base = base.padStart(7, '0');

            let sum = 0;
            let rest = 0;

            let digit1 = 0;
            let digit2 = 0;
            let auxW = 8;

            // 1st Digit
            for (let i = 0; i < base.length; i++) {
                sum += (base.charAt(i) + '') * auxW--;
            }

            rest = sum % 11;

            if (rest === 0 || rest === 1) {
                digit1 = 0;
            } else {
                digit1 = 11 - rest;
            }

            // 2nd Digit
            auxW = 9;
            let numAux = base + digit1;

            sum = 0;
            for (let i = 0; i < numAux.length; i++) {
                sum += (numAux.charAt(i) + '') * auxW--;
            }

            rest = sum % 11;

            if (rest === 0 || rest === 1) {
                digit2 = 0;
            } else {
                digit2 = 11 - rest;
            }

            let calculatedDigit = digit1.toString() + digit2.toString();

            if (parseInt(calculatedDigit) !== digit) {
                return false;
            }

        } else {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Piauí
    static isValidPI(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Rio Grande do Norte
    static isValidRN(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9) * 10;
        rest = sum % 11;

        let calculatedDigit = rest;
        if (rest === 10) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Rio Grande do Sul
    static isValidRS(code) {
        if (code.length > 10) {
            return false;
        }

        const completeCode = code.padStart(10, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Rio de Janeiro
    static isValidRJ(code) {
        if (code.length > 8) {
            return false;
        }

        const completeCode = code.padStart(8, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,7);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Rondônia
    static isValidRO(code) {
        if (code.length > 14) {
            return false;
        }

        const completeCode = code.padStart(14, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Roraima
    static isValidRR(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        for (let i = 0; i < base.length; i++) {
            let auxW = i + 1;
            sum += (base.charAt(i) + '0') * auxW;
        }

        rest = sum % 9;

        let calculatedDigit = rest;
        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Santa Catarina
    static isValidSC(code) {
        if (code.length > 9) {
            return false;
        }

        const completeCode = code.padStart(9, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(0, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: São Paulo
    static isValidSP(code) {
        if (code.length > 12) {
            return false;
        }

        const completeCode = code.padStart(12, '0');
        const d1 = parseInt(completeCode.substring(8, 9));
        const d2 = parseInt(completeCode.substring(completeCode.length - 1));
        const baseD1 = completeCode.substring(0, 9);
        const base = completeCode.substring(0, completeCode.length - 1);

        // 1st digit
        let d1W = [1, 3, 4, 5, 6, 7, 8, 10];

        let sum = 0;
        for (let i = 0; i < 8; i++) {
            sum += (baseD1.charAt(i) - '0') * d1W[i];
        }

        let rest = sum % 11;
        let digit1 = rest;

        if (rest === 10) {
            digit1 = 0;
        }

        if (digit1 !== d1) {
            return false;
        }

        // 2nd digit

        sum = 0;
        sum = ValidateStateRegistration.weightedSum(base,2,10);

        rest = sum % 11;

        let digit2 = rest;
        if (rest === 10) {
            digit2 = 0;
        }

        if (digit2 !== d2) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Sergipe
    static isValidSE(code) {
        if (code.length > 10) {
            return false;
        }

        const completeCode = code.padStart(10, '0');
        const digit = parseInt(completeCode.substring(completeCode.length-1));
        const base = completeCode.substring(1, completeCode.length-1);

        let sum = 0;
        let rest = 0;

        // 1st digit
        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }

    // Calcula a Inscrição Estadual do estado: Tocantins
    static isValidTO(code) {
        if (code.length > 11) {
            return false;
        }

        let completeCode = '';
        if ( code.length <= 9 ) {
            completeCode = code.padStart(9, '0');
        } else if ( code.length <= 11 ) {
            completeCode = code.padStart(11, '0');
        }

        const digit = parseInt(completeCode.substring(completeCode.length-1));

        let baseSrInic = '';
        let base = '';
        if ( completeCode.length > 9 ) {
            baseSrInic = completeCode.substring(0, 2);
            base = completeCode.substring(4, completeCode.length-1);
        } else {
            base = completeCode.substring(0, completeCode.length-1);
        }

        base = baseSrInic + base;

        let sum = 0;
        let rest = 0;

        sum = ValidateStateRegistration.weightedSum(base,2,9);
        rest = sum % 11;

        let calculatedDigit = 11 - rest;
        if (rest === 0 || rest === 1) {
            calculatedDigit = 0;
        }

        if (calculatedDigit !== digit) {
            return false;
        }

        return true;
    }
}
